# -*- coding: utf-8 -*-
# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

require 'ec_rider/erb_util_ext'

class ApplicationController < ActionController::Base
  include EcRider::PluginModule::Includable
  
  # class_desc, descをコントローラで使えるようにするためにincludeしています。
  include Prestruct

  prepend_before_filter :goldberg_security_up
  after_filter :reset_user

  # since rails-2.3.2
  protect_from_forgery # See ActionController::RequestForgeryProtection for details

  include SslRequirement

  # エラーメッセージなどのi18nサポート
  include EcRider::I18nSupport

  # 障害発生時のメール通知
  include ExceptionNotifiable

  # アクセスチェック
  before_filter :allowed?

  protected
  #
  # Goldberg::Userの情報取得
  #
  def current_user
    if session[:current_user]
      return session[:current_user]
    end
    if session[:user]
      @current_user = Goldberg::User.find_by_id(session[:user])
      session[:current_user] = @current_user
    end
  end

  def logged_in?
    !!current_user
  end

  # override SslRequirement module's method
  def ssl_required?(logging = true)
    # ec_frontなら to_be_ssl?(params) メソッドで確認するが、 ec_adminは必ずtrueです。
    if RAILS_ENV == "production"
      if EcRider.config.ssl_ignored
        return false
      end
      return true
    else
      return false
    end
  end

  # override ActionController module's method
  def default_url_options(options = {})
    if ssl_required? && !ssl_allowed?
      {:protocol => 'https', :port => EcRider.config.ec_admin_https_port.to_s}
    end
  end

  private
  # override SslRequirement module's method
  def ensure_proper_protocol
    # ec_adminはSSLには8443ポートを使用します。
    return true if ssl_allowed?

    if ssl_required? && !request.ssl?
      redirect_to "#{EcRider.config.ec_admin_https_url}#{request.request_uri}"
      return false
    elsif request.ssl? && !ssl_required?
      redirect_to "http://" + request.host + request.request_uri
      return false
    end
  end

  # 渡されたパラメータの値をTrimしてHashで返す
  def trim_values(org_hash, new_hash={})
    if org_hash
      org_hash.each_pair do |key, val|
        new_hash[key] = val.strip
      end
    end
    return new_hash
  end

  # 渡されたパラメータの値をTrimしてHashWithIndifferentAccessで返す
  def trim_values_indifferent(org_hash)
    trim_values(org_hash, HashWithIndifferentAccess.new())
  end

  # 特定のアクションで使用される検索条件などのオブジェクトを準備するためのメソッドです。
  # 検索条件の場合には、このメソッドではなく、prepare_session_conditionメソッドを使用してください。
  def prepare_session_object(name, options = nil)
    options = {
      :controller => self.controller_name,
      :action => self.action_name,
      :attrs_name => name
    }.update(options || {})
    controller = options[:controller]
    action = options[:action]
    key_name = "#{controller}_#{action}_#{name}"
    # clearが付いてたら一旦セッション上のオブジェクトを消します。
    cleared_now = (params[:clear] == 'true')
    session[key_name] = nil if cleared_now
    result = session[key_name]
    # オブジェクトがあってもなくてもyieldします。
    # ポイントはnilかもしれないsession上のオブジェクトを渡して、yield側で判断してもらうことです。
    result = nil if params[options[:attrs_name] || :conditions]
    result = yield(result) if block_given?
    result.cleared = cleared_now if result.respond_to?(:cleared=)
    session[key_name] = result
    return result
  end

  # 検索条件オブジェクトを使用する際にはこのメソッドを必ず使用してください。
  # 例)
  # def index
  #   @condition = prepare_session_condition do |condition|
  #     condition || Product::SearchCondition.new(params[:condition])
  #   end
  # end
  def prepare_session_condition(name = :condition, options = nil, &block)
    prepare_session_object(name, options, &block)
  end

  # 権限チェック
  def allowed?
    return if request.path_parameters[:controller] == "goldberg/auth"
    unless session[:user]
      redirect_to :controller => "goldberg/auth", :action => 'login'
      return
    end
  end

  # 404エラー画面を表示する
  def render_404
    respond_to do |type|
      type.html { render :file => "#{RAILS_ROOT}/public/ec_rider/404.html", :status => "404 Not Found" }
      type.all  { render :nothing => true, :status => "404 Not Found" }
    end
  end

  # 500エラー画面を表示する
  def render_500
    respond_to do |type|
      type.html { render :file => "#{RAILS_ROOT}/public/ec_rider/500.html", :status => "500 Error" }
      type.all  { render :nothing => true, :status => "500 Error" }
    end
  end

  # 
  # (元goldbergの)フィルタ用メソッド。
  # メソッド内はほぼ原型を留めておりません。
  # 元々は、lib/goldberg/filters.rb で定義されていたGoldberg::Filtersモジュールのメソッド
  # でしたが、分けているメリットもほとんどないので、メソッドを移動してincludeしないようにしました。
  # 
  def goldberg_security_up
    logger.debug "Setting user..."
    set_user or return false
    
    # Check whether the user is authorised for this page or action.
    logger.debug "Checking permissions..."
    check_permissions or return false
    
    session[:last_time] = Time.now
    return true
  end

  # 元々は、lib/goldberg/filters.rb で定義されていたGoldberg::Filtersモジュールのメソッド
  # でしたが、分けているメリットもほとんどないので、メソッドを移動してincludeしないようにしました。
  def set_user
    user = session[:user]         # ここではIDが帰ってくるはず。
    Goldberg::User.model_stamper  # Goldberg::UserがIDを覚えられるようにする。
    Goldberg::User.stamper = user # IDを覚えておく。stamper=にはオブジェクトを渡してもOK。
    Goldberg.set_user(user)       # IDを指定してログイン中のユーザーを記録。
    if Goldberg.user              # ここではGoldberg::Userのインスタンスが返されます。
      session[:user] = Goldberg.user.id
    end
    return true
  end

  def reset_user
    Goldberg::User.model_stamper  # Goldberg::Userが覚えているIDを忘れることができるようにする。
    Goldberg::User.reset_stamper  # Goldberg::Userが覚えているIDを実際に忘れる。
    Goldberg.clear!
  end

  
  # 元々は、lib/goldberg/filters.rb で定義されていたGoldberg::Filtersモジュールのメソッド
  # でしたが、分けているメリットもほとんどないので、メソッドを移動してincludeしないようにしました。
  def check_permissions
    return true if action_authorised?
    if Goldberg.user
      # render_404
      redirect_to :controller => "/top", :action => "access_denied"
    else
      redirect_to :controller => 'goldberg/auth', :action => 'login'
    end
    return false
  end

  # 元々は、lib/goldberg/filters.rb で定義されていたGoldberg::Filtersモジュールのメソッド
  # でしたが、分けているメリットもほとんどないので、メソッドを移動してincludeしないようにしました。
  #
  # このメソッドは必要に応じてオーバーライドされることを期待しています。
  def action_authorised?
    self.class.prepare_action_descriptions # クラス定義時に呼び出されるはずなんですが、テスト時には呼び出されていないので、呼び出します。
    controller_desc = self.class.prestruct
    raise "INVALID Controller!!! #{controller_path}/#{action_name}" unless controller_desc
    action_desc = controller_desc.method_descriptions[action_name.to_sym]
    raise "INVALID Action!!! in #{controller_path}/#{action_name}" unless action_desc

    accessible_action?(controller_desc, action_desc)
  end

  def accessible_action?(controller_desc, action_desc)
    return true if controller_desc[:auth] == :user_login
    return true if action_desc[:auth] == :user_login
    auth_controller = 
      !controller_desc[:auth_as].blank? ? controller_desc[:auth_as].to_s :
      !controller_desc[:controller_path].blank? ? controller_desc[:controller_path] : controller_path
    auth_action = !action_desc[:auth_as].blank? ? action_desc[:auth_as].to_s : action_desc.name
    result = Goldberg.action_authorised?(auth_controller, auth_action)
    # アクセス不可の判断をした場合のデバッグのためにログに出力します。
    if !result && (RAILS_ENV =~ /development|test/)
      ActiveRecord::Base.logger.warn(
        "#{controller_path.inspect}/#{action_name.inspect} => " << 
          "#{auth_controller.inspect}/#{auth_action.inspect} IS NOT AUTHORISED!!!!!\n" << action_desc.inspect)
    end
    result
  end



  ACTION_TO_CATEGORY_NAME = {
    'index'          => 'index',
    'init_index'     => 'index',
    'list'           => 'index',
    'show'           => 'index' ,
    'new'            => 'create',
    'create'         => 'create',
    'create_confirm' => 'create',
    'confirm_create' => 'create',
    'edit'           => 'update',
    'update'         => 'update',
    'update_confirm' => 'update',
    'confirm_update' => 'update',
    'destroy'        => 'destroy',
  }

  BASE_ACTION_DISPLAY_NAME_SUFFIXES = {
    'index'          => '検索',
    'init_index'     => '検索',
    'list'           => '検索',
    'show'           => '詳細',
    'new'            => '新規登録',
    'create_confirm' => '新規登録確認',
    'confirm_create' => '新規登録確認',
    'create'         => '新規登録',
    'edit'           => '編集',
    'update_confirm' => '更新確認',
    'confirm_update' => '更新確認',
    'update'         => '更新',
    'destroy'        => '削除',
  }

  # 
  # classの定義が完了した後にdescが書かれていないアクションに対して
  # 自動でdescを補完するために設定を追加しています。
  # descが書かれていないアクションのうち、標準的なものに関しては自動でdescを補完しています。
  # 
  def self.prepare_action_descriptions
    return if @prepared_action_descriptions

    controller_desc = Prestruct.find_or_new_module(self)
    controller_desc.update(:controller_path => self.controller_path)
    if controller_desc[:display_name].blank?
      controller_desc.update(:display_name => self.controller_path)
    end
    controller_display_base = controller_desc[:display_name].sub(/管理$/, '')
    actions = self.action_methods
    actions.each do |action|
      action_desc = controller_desc.find_or_new_method(action.to_s)
      category_name = action_desc[:category_name]
      if category_name.blank?
        if category_name = ACTION_TO_CATEGORY_NAME[action.to_s]
          action_desc.update(:category_name => category_name)
        end
      end
      display_name = action_desc[:display_name]
      if display_name.blank?
        if display_name_suffix = BASE_ACTION_DISPLAY_NAME_SUFFIXES[action.to_s]
          action_desc.update(:display_name => "#{controller_display_base}#{display_name_suffix.to_s}")
        end
      end
      auth_as = action_desc[:auth_as]
      if auth_as.blank?
        if auth_as = ACTION_TO_CATEGORY_NAME[action.to_s]
          if self.action_methods.include?(auth_as)
            unless auth_as == action.to_s
              action_desc.update(:auth_as => auth_as.to_s)
            end
          end
        end
      end
    end
    @prepared_action_descriptions = true
  end

end

# 各コントローラが定義され終わった直後にdescの補完を実行できるように、
unless ActiveSupport::Dependencies.respond_to?(:load_missing_constant_with_ecr_omission)

  ActiveSupport::Dependencies.instance_eval do
    def load_missing_constant_with_ecr_omission(from_mod, const_name, &block)
      result = load_missing_constant_without_ecr_omission(from_mod, const_name, &block)
      if result.is_a?(Class) && (result < ApplicationController)
        result.prepare_action_descriptions
      end
      result
    end

    self.instance_eval do
      alias :load_missing_constant_without_ecr_omission :load_missing_constant
      alias :load_missing_constant :load_missing_constant_with_ecr_omission
    end
  end

end


# 
unless ActionView::Base.instance_methods.include?('link_to_with_ecr_omission')
  
  class ActionView::Base
    
    def link_to_with_ecr_omission(*args, &block)
      url_options = block ? args.first : args[1]
      if new_url = change_url_if_need(url_options)
        if block
          args.shift # url_options
          link_to_without_url_for(new_url, *args, &block)
        else
          name = args.shift
          args.shift # url_options
          link_to_without_url_for(name, new_url, *args)
        end
      else
        link_to_without_ecr_omission(*args, &block)
      end
    end

    UNACCESSIBLE_HREF = "javascript:alert('この機能を利用する権限がありません。システム管理者にお問い合わせください。')"

    THROUGH_PASS_URL_PATTERN = /^javascript\:|\.(css|js|pdf|jpg|jpeg|gif|png|htm|html)$/i

    # リンクでのアクセス可否判断
    def change_url_if_need(url_options)
      if url_options.is_a?(String)
        return nil if THROUGH_PASS_URL_PATTERN =~ url_options
        src_url = url_options.sub(/\?.*?$/, '')
        url_options = ::ActionController::Routing::Routes.recognize_path(src_url)
      end
      controller = url_options[:controller] || params[:controller]
      action = url_options[:action] || 'index'

      # return nil if controller == "spec/rails/example/helper_example_group"

      controller_class = find_controller(controller)
      raise "Unknown controller '#{controller}'" unless controller_class
      unless controller_class.respond_to?(:prestruct)
        raise "Invalid controller #{controller_class.name}"
      end
      controller_class.prepare_action_descriptions
      controller_desc = controller_class.prestruct
      return nil if controller_desc[:auth] == :user_login
      
      action = 'show' if /^\d+?/ =~ action.to_s # product_images/1123 を product_images/show/1123として理解
      unless controller_class.action_methods.include?(action.to_s)
        raise "Unknown action #{controller.to_s}/#{action.to_s}"
      end


      action_desc = controller_class.prestruct.method_descriptions[action.to_sym]
      raise "desc not found for #{controller.to_s}/#{action.to_s}" unless action_desc
      return nil if action_desc[:auth] == :user_login
      
      if @controller.__send__(:accessible_action?, controller_desc, action_desc)
        return nil # アクセス可能な場合はURLを変更しない
      else
        return ActionView::Base::UNACCESSIBLE_HREF # アクセス不可ならメッセージを出力するJS
      end
    end


    def find_controller(controller_path)
      # see ActionController::Routing::RouteSet#recognize
      #     actionpack/lib/action_controller/routing/route_set.rb
      "#{controller_path.to_s.camelize}Controller".constantize
    rescue Exception
      nil
    end

    # url_forに"javascript:xxxx" という文字列を渡すと recognize_pathでNo route matches というエラーが出てしまうので、
    # url_forを使用しない link_to を用意しました。
    # 
    # see ActionView::Helpers::UrlHelper#link_to
    #     in actionpack/lib/action_view/helpers/url_helper.rb
    def link_to_without_url_for(*args, &block)
      if block_given?
        options      = args.first || {}
        html_options = args.second
        concat(link_to(capture(&block), options, html_options))
      else
        name         = args.first
        options      = args.second || "javascript:void(0)"
        html_options = args.third

        url = options

        if html_options
          html_options = html_options.stringify_keys
          href = html_options['href']
          convert_options_to_javascript!(html_options, url)
          tag_options = tag_options(html_options)
        else
          tag_options = nil
        end

        href_attr = "href=\"#{url}\"" unless href
        "<a #{href_attr}#{tag_options}>#{name || url}</a>"
      end
    end

    alias_method_chain :link_to, :ecr_omission
  end
end


# ERB::Utilの拡張
unless ERB::Util.respond_to?(:html_escape_with_ec_rider) 
	ERB::Util.extend(EcRider::ErbUtilExt) 
end 
 
# ERB::Utilの拡張
unless ActionView::Base.ancestors.include?(EcRider::ErbUtilExt) 
	ActionView::Base.module_eval do	 
		include(EcRider::ErbUtilExt) 
	end 
end 

# ActiveRecord::Baseの拡張
unless ActiveRecord::Base.ancestors.include?(SafeEval::ArModel)
  ActiveRecord::Base.module_eval do
    include SafeEval::ArModel
  end
end
