# -*- coding: utf-8 -*-
require 'tmpdir'
require 'pathname'
require 'fileutils'
require 'zip/zipfilesystem'


class CmsFilesController < ApplicationController
  class_desc :display_name => 'プロジェクト管理->ファイル管理'

  include EcRider::CloudFront
  before_filter :find_cms_project
  before_filter :setup_base_path

  private
  def find_cms_project
    session[:project_id] ||= params[:project_id]
    if params[:project_id] && session[:project_id] != params[:project_id]
      session[:project_id] = params[:project_id]
    end
    @cms_project = CmsProject.find(session[:project_id])
  end

  public
  def index
    @condition = prepare_session_condition do |condition|
      condition || CmsFile::SearchCondition.new(params[:condition])
    end
    @condition.cms_project_id = @cms_project.id
    cond, @delete_ids = @condition.create_condition
    @cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path')
    @cms_files = filetree_sort(@cms_files)
  end

  def show
    cms_file = CmsFile.find(params[:id])
    send_file(cms_file.pj_file_path)
  end

  def create
    if params[:source_id]
      # PJ公開時に削除する場合。コントローラ名とアクション名をコピーして公開時に削除フラグをtrueにセット
      src_file = CmsFile.find(params[:source_id])
      cms_file = CmsFile.new(:cms_project_id => @cms_project.id, :delete_at_publish => true, :public_file_path => src_file.public_file_path)
      cms_file.save!
    else
      unless params[:upload_file]
        public_file_path = File.join(params[:parent_dir], *params[:child_dir].split('/'))
        cms_file = CmsFile.new(:public_file_path => "#{public_file_path}/")
        unless cms_file.valid?
          flash[:error] = cms_file.errors.full_messages
          return redirect_to(:action => 'index')
        end

        # ディレクトリを作成
        FileUtils.mkdir_p(File.join(@base_path, @cms_project.id.to_s, public_file_path))
        # DBに登録
        unless params[:child_dir].blank?
          cms_file = CmsFile.new(:public_file_path => "#{public_file_path}/")
          cms_file.save!
        end

       return redirect_to(:action => 'index')
      end

      uploading = Uploading.new(
        :base_path => @base_path,
        :project_id => @cms_project.id.to_s,
        :upload_file_io => params[:upload_file],
        :parent_dir => params[:parent_dir],
        :overwrite => params[:overwrite]
        )
      if uploading.valid?
        begin
          # ファイルをアップロード
          public_file_paths = uploading.execute

          # DBに登録
          # 新規アップロード場合は新規レコードを登録します。
          # 上書きアップロードの場合、
          # ・プロジェクト編集対象のときは、既存データを日付のみ更新します。
          public_file_paths.each do |public_file_path|
            if public_file_path =~ /.*\/$/
              if CmsFile.count(:conditions => ["public_file_path = ? ", public_file_path]) == 0
                cms_file = CmsFile.create!(:public_file_path => public_file_path)
              end
            else
              present_version_file = CmsFile.find(:first, :conditions => ["public_file_path = ? and delete_at_publish is null", public_file_path])
              cms_file = CmsFile.new(:cms_project_id => @cms_project.id, :public_file_path => public_file_path)
              CmsFile.transaction do
                if present_version_file
                  cms_file.save! unless present_version_file.current_project?(@cms_project.id)
                  present_version_file.save!
                else
                  cms_file.save!
                end
              end
            end
          end

        rescue Zip::ZipEntryNameError
          flash[:error] = "ZIPファイル内のディレクトリ、ファイル名に不正な文字が入っています。"
        end
      else
        flash[:error] = uploading.errors.full_messages
      end
    end
    redirect_to :action => 'index'
  end

  def destroy
    if params[:source_id]
      src_file = CmsFile.find(params[:source_id])
      cms_file = CmsFile.find(:first,
        :conditions => ["cms_project_id = ? and public_file_path = ? and delete_at_publish = ?", @cms_project.id, src_file.public_file_path, true])
      cms_file.destroy
    else
      cms_files = CmsFile.find(:all, :conditions => ["public_file_path like ?", "#{params[:path]}%"])
      pj_dirs = Dir.glob(File.join(@base_path, "*"))

      cms_files.each do |cms_file|
        # DBから削除
        cms_file.destroy
        # ファイルを削除
        pj_dirs.each do |pj_dir|
          FileUtils.rm_rf(File.join(pj_dir, cms_file.public_file_path))
          delete_from_s3(File.join(EcRider.config.ec_front_root, cms_file.public_file_path))
        end
        # プレビュー用ファイルを削除
        FileUtils.rm_rf(File.join(EcRider.config.ec_admin_root, cms_file.public_file_path))
      end
    end
    redirect_to :action => 'index'
  end

  private

  def setup_base_path
    # @base_path は、ビューでも使われているインスタンス変数です
    @base_path = File.expand_path(File.join(EcRider.config.ec_admin_root, 'public', 'projects'))
  end

  #
  # ファイルパスの配列をディレクトリ－ファイルの親子関係を意識してソートします
  #
  def filetree_sort(cms_pages)
    # ディレクトリ-ファイルの親子関係を整理する
    files = {}
    cms_pages.each do |cms_page|
      if cms_page.directory?
        files[cms_page.public_file_path] ||= [cms_page]
      else
        dirname = cms_page.public_dir_path
        if files[dirname].nil?
          files[dirname] = [cms_page]
        else
          files[dirname] << cms_page
        end
      end
    end
    # ディレクトリ名でソートし、各ディレクトリ内をファイル名でソートする
    result = []
    files.keys.sort.each do |dirname|
      sorted_files = files[dirname].sort {|x, y| x[:public_file_path] <=> y[:public_file_path]}
      result.concat(sorted_files)
    end
    return result
  end

  # このクラスはビューに直接マップされる訳ではありませんが、
  # ビジネスロジックをまとめるためにActiveFormを継承しています。
  # 処理をHTTP層に依存しないように記述してあります
  #
  class Uploading < ActiveForm

    include EcRider::CloudFront
    attr_accessor :upload_file_io
    attr_accessor :base_path, :project_id
    attr_accessor :parent_dir, :overwrite

    def validate
      if upload_file_io.blank?
        errors.add_to_base("アップロードパスを指定して下さい。")
        return
      end
      @upload_file_path = upload_file_io.original_path # アップロードされたPC上でのパス取得
      @zip_flg = (@upload_file_path =~ /\.zip$/i)
      @base_name = File.basename(upload_file_io.original_filename)
      # File.basename(xxx, '.*')で拡張子を除いています
      @path_from_public = File.join(parent_dir, @zip_flg ? File.basename(@base_name, '.*') : @base_name)
      # ディレクトリが無ければ作成します
      FileUtils.makedirs(File.dirname(File.join(@base_path, project_id, @path_from_public)))
      @dest_path = File.join(@base_path, project_id, @path_from_public)

      #
      # アップロードされたPC上でのパス取得
      if /[一-龠]+|[ぁ-ん]+|[ァ-ヴー]+/ =~ upload_file_io.original_path
        errors.add_to_base("アップロードパスに全角文字が入っています。")
      end
      unless @base_name =~ /^[A-Za-z0-9_\/\.]*$/
        errors.add_to_base "ファイル名に半角英数字と_.以外の文字が含まれています。"
      end
      if !overwrite && CmsFile.count(:conditions => ["public_file_path = ? ", public_file_path]) > 0
        errors.add_to_base("同一の#{@zip_flg ? 'フォルダ' : 'ファイル'}が存在している為、#{@path_from_public}は上書きませんでした。")
      end
    end

    def execute
      if @zip_flg
        dest_tmp_dir = RAILS_ROOT, "tmp"
        tmp_zip_path = File.join(dest_tmp_dir, @base_name)
        # IOオブジェクトの内容をファイルに保存して、zip解凍できるようにします
        open(tmp_zip_path, 'w+') do |f|
          f.write(upload_file_io.read)
        end
        extract(tmp_zip_path, File.basename(@dest_path))
        return @file_entries.uniq
      else
        open(@dest_path, 'w') do |f|
          f.write(upload_file_io.read)
        end
        return [@path_from_public]
      end
    end

    #
    # = zipファイルを展開します =
    # ec_admin/tmpフォルダに一度解凍し、アップロードパスに移動します
    # zip_file_path: tempディレクトリの中のzipファイルのパス（tempディレクトリ/xxx.zip）
    # unzipped_name: zipファイル名（xxx.zip）
    #
    def extract(zip_file_path, unzipped_name)
      dest_tmp_dir = File.join(RAILS_ROOT, "tmp", "cms_real_files")
      FileUtils.rm_rf(File.join(dest_tmp_dir, unzipped_name)) if File.exist?(File.join(dest_tmp_dir, unzipped_name))

      unzip(zip_file_path, dest_tmp_dir)

      unzipped_name = '.' unless @dir_include
      FileUtils.mkdir_p(File.join(@base_path, project_id, parent_dir))
      FileUtils.cp_r(File.join(dest_tmp_dir, unzipped_name), File.join(@base_path, project_id, parent_dir))
      FileUtils.rm_rf(File.join(dest_tmp_dir, unzipped_name))
    end

    #
    # = zipファイルを解凍します =
    # zip_file_path: tempディレクトリの中のzipファイルのパス（tempディレクトリ/xxx.zip）
    # dest_dir: 一度解凍するディレクトリ（tempディレクトリ/cms_real_files）
    #
    def unzip(zip_file_path, dest_dir)
      @file_entries = []
      Zip::ZipInputStream.open(zip_file_path) do |stream|
        # 解凍ファイル内容取得
        while (entry = stream.get_next_entry)
          # ファイル生成先作成
          file_path = File.join(dest_dir, entry.name)
          unless entry.name =~ /^[ -~｡-ﾟ]*$/
            FileUtils.rm_rf(dest_dir)
            raise Zip::ZipEntryNameError
          end
          # 解凍した結果、ディレクトリが返ってきた場合
          if file_path =~ /\/$/
            FileUtils.mkdir_p(File.dirname(file_path))
            @dir_include = true
          # 解凍した結果、ファイルが返ってきた場合
          else
            FileUtils.mkdir_p(File.dirname(file_path))
            # 空ファイルにデータ書き込み
            File.open(file_path, 'w+b') do |f|
              f.puts(stream.read)
            end
          end
          @file_entries << File.dirname(parent_dir + entry.name) + '/' unless @file_entries.include?(File.dirname(parent_dir + entry.name))
          @file_entries << parent_dir + entry.name
        end
      end
    end
  end

end
