# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module OrderTag
      extend MethodDescription

      desc <<-EOS
        [概要：決済エラー（クレジットカードの認証エラー等）が発生した場合、その中身を実行します。]
        [引数：なし]
        [戻り値：なし]
        [使用例：{{{
          <ec:if_settlement_auth_failed>
            <p>申し訳ございません。お手数ですが、以下のご入力内容をお確かめ下さい。</p>
            <ec:order_settlement_error>
          </ec:if_settlement_auth_failed>
        }}}]
      EOS
      #メソッド名「if_settlement_failed」はすでに存在するため、「if_settlement_auth_failed」としました。
      def if_settlement_auth_failed(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        if session[:settlement_error]
          yield
        end
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :if_cedit_auth_failed, :if_settlement_auth_failed

      desc <<-EOS
        [概要：決済エラーを表示します]
        [引数：なし]
        [戻り値：なし]
        [使用例：ec:if_settlement_auth_failedを参照してください。]
      EOS
      def order_settlement_error
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        return "" unless  auth_error = session[:settlement_error]
        error_msg = ""
        for error in auth_error
          error_msg << "#{error}<br/>"
        end
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :order_credit_auth_error, :order_settlement_error

      desc <<-EOS
        [概要:配送商品一覧を出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_product_list>
             <ec:order_product_name />
             <ec:order_product_price />
             <ec:order_product_qty />
             <ec:order_product_price_x_qty />
             <ec:order_product_discount_price />
             <ec:order_product_sub_total_price />
           </ec:order_product_list>
           }}}
        ]
      EOS
      def order_product_list(&block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list_order_by_present.nil?
        session[:cart].item_list_order_by_present.each do |order_product|
          @order_product = order_product
          begin
            @template.concat(@template.capture(&block))
          ensure
            @order_product = nil
          end
        end
      end
      
      desc <<-EOS
        [概要:配送商品の種類数を出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_product_count>
           }}}
        ]
      EOS
      def order_product_count
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].product_list.nil?
        session[:cart].product_list.length
      end

      desc <<-EOS
        [概要:配送商品の画像を表示します。配送商品一覧内で使用してください。]
        [引数 type(任意):商品画像のサイズを指定します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_image(*args, &block)
        raise UsageError, "order_product_nameはec:order_product_listの中で使用してください" unless @order_product
        type, options = extract(args, :type => nil)
        image = @order_product.product_sku.product.main_image
        options["alt"] ||= @order_product.name
        @template.image_tag(image.public_filename(type), options) unless image.nil?
      end

      desc <<-EOS
        [概要:配送商品の名前を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_name(*args, &block)
        raise UsageError, "order_product_nameはec:order_product_listの中で使用してください" unless @order_product
        @order_product.name if @order_product.name
      end

      desc <<-EOS
        [概要:{{{
          配送商品の獲得ポイントを表示します。配送商品一覧内で使用してください。
          このタグで出力されるポイントは、注文全体に対する割引やポイントアップキャンペーンが適用される前の値です。
          注文全体に対するキャンペーンが適用された場合、実際に付与されるポイントと差異がありますのでご注意ください。
        }}}]
        [引数 prefix(任意):単位を指定します。数値の前に表示されます。]
        [引数 postfix(任意):単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_given_point(*args, &block)
        raise UsageError, "order_product_given_pointはec:order_product_listの中で使用してください" unless @order_product
        postfix, prefix, options = extract(args, :postfix => "", :prefix => "")
        display_number_with_unit(@order_product.given_point, prefix, postfix)
      end

      desc <<-EOS
        [概要:配送商品の会員ランク毎の単価を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_price(*args, &block)
        raise UsageError, "order_product_priceはec:order_product_listの中で使用してください" unless @order_product
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@order_product.unit_price, pre_currency, post_currency)
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :order_product_normal_price, :order_product_price


      desc <<-EOS
        [概要:配送商品の数量を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_qty(*args, &block)
        raise UsageError, "order_product_qtyはec:order_product_listの中で使用してください" unless @order_product
        @template.number_with_delimiter(@order_product.qty) if @order_product.qty
      end

      desc <<-EOS
        [概要:配送商品の割引金額を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_discount_price(*args, &block)
        raise UsageError, "order_product_discount_priceはec:order_product_listの中で使用してください" unless @order_product
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@order_product.discount_amount, pre_currency, post_currency, true)
      end

      desc <<-EOS
        [概要:配送商品の小計(割引金額適用前)を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_price_x_qty(*args, &block)
        raise UsageError, "order_product_price_x_qtyはec:order_product_listの中で使用してください" unless @order_product
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@order_product.subtotal_amount_for_discount, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:配送商品の小計(割引金額適用後)を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_sub_total_price(*args, &block)
        raise UsageError, "order_product_sub_total_priceはec:order_product_listの中で使用してください" unless @order_product
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@order_product.total_amount, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:注文全体の割引適用後の合計金額を表示します。ec:order_confirmation_form内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：{{{<ec:order_total_price />}}}]
      EOS
      def order_total_price(*args, &block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(session[:cart].total_amount, pre_currency, post_currency)
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :order_product_total_price, :order_total_price

      desc <<-EOS
        [概要:注文全体に対する割引金額を表示します。(商品単品の割引金額は加算されません。)ec:order_confirmation_form内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例:{{{<ec:order_total_discount_price />}}}]
      EOS
      def order_total_discount_price(*args, &block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(session[:cart].discount_amount, pre_currency, post_currency)
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :order_product_total_discount_price, :order_total_discount_price

      desc <<-EOS
        [概要:注文の合計獲得ポイントを表示します。ec:order_confirmation_form内で使用してください。]
        [引数 prefix(任意):単位を指定します。数値の前に表示されます。]
        [引数 postfix(任意):単位を指定します。数値の後ろに表示されます。]
        [使用例：{{{<ec:order_total_given_point/>}}}]
      EOS
      def order_total_given_point(*args, &block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        postfix, prefix, options = extract(args, :postfix => "", :prefix => "")
        display_number_with_unit(session[:cart].total_given_point, prefix, postfix)
      end

      desc <<-EOS
        [概要:注文時に利用するポイントを表示します。ec:order_confirmation_form内で使用してください。]
        [引数 prefix(任意):単位を指定します。数値の前に表示されます。]
        [引数 postfix(任意):単位を指定します。数値の後ろに表示されます。]
        [使用例：{{{<ec:order_total_given_point/>}}}]
      EOS
      def order_use_point(*args)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        display_number_with_unit(session[:order_form].use_point)
      end

      desc <<-EOS
        [概要:注文後利用可能ポイントを表示します。ec:order_confirmation_form内で使用してください。]
        [使用例：{{{<ec:order_total_point/>}}}]
      EOS
      def order_total_point(*args, &block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        raise UsageError, "session[:order_form]がありません" if session[:cart].nil? || session[:order_form].nil?
        display_number_with_unit(session[:order_form].after_total_point)
      end

      desc <<-EOS
        [概要:配送料の合計を表示します。ec:order_confirmation_form内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：{{{<ec:order_total_shipping_cost/>}}}]
      EOS
      def order_total_shipping_cost(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        # 2009/11/25 現在、orderとorder_shippingは1対1なので、
        # order_shippingsメソッドで返却される配列の一番目のオブジェクト取得しています。
        order_shipping = @order.object.order_shippings.first
        shipping_cost = order_shipping.shipping_cost
        # shipping_costは代引き手数料込みの金額なので、代引き手数料を引いた金額を取得しています。
        shipping_cost = shipping_cost - order_shipping.cod_charge if order_shipping.cod_charge
        display_number_with_unit(shipping_cost, pre_currency, post_currency)
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :order_product_shipping_cost, :order_total_shipping_cost
      
      desc <<-EOS
        [概要:注文全体の合計金額（送料、手数料を含む）を表示します。ec:order_confirmation_form内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：{{{<ec:order_total_price_include_shipping_cost/>}}}]
      EOS
      def order_total_price_include_shipping_cost(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        order_shipping = @order.object.order_shippings.first
        display_number_with_unit((order_shipping.total_amount + order_shipping.shipping_cost), pre_currency, post_currency)
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :order_product_total_price_include_shipping_cost, :order_total_price_include_shipping_cost

      desc <<-EOS
        [概要:代引き手数料を表示します。ec:order_confirmation_form内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：
          {{{
            代金引換手数料：<ec:order_total_cod_charge post_currency="円"/>
            →代金引換手数料：1,000円
          }}}
        ]
      EOS
      def order_total_cod_charge(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        order_shipping = @order.object.order_shippings.first
        display_number_with_unit(order_shipping.cod_charge, pre_currency, post_currency)
      end
      #既存のページがエラーにならないように古いタグの名前を残しておきます
      alias_method :order_product_cod_charge, :order_total_cod_charge

      desc <<-EOS
        [概要:適用されているキャンペーン一覧を返します]
        [引数 display_qty(任意): 最大表示数を設定します。設定しない場合は適用されるキャンペーン分すべてを繰り返し実行します。]
        [使用例:
          {{{
          <ec:each_order_applied_campaigns>
            <ec:order_applied_campaign_link>
              <ec:order_applied_campaign_name />
            </ec:order_applied_campaign_link>
          </ec:each_order_applied_campaigns>
             →適用されているキャンペーン名称リンクを繰り返し表示します。
          }}}
        ]
        [同様のタグ： ec:each_cart_applied_campaigns ショッピングカート画面で使用します]
      EOS
      def each_order_applied_campaigns(*args, &block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        display_qty, options = extract(args, :display_qty => nil)
        @campaigns = session[:cart].applied_campaigns_for_cart_and_items
        begin
          _each_with_index(:@campaigns, :@campaign, :@index) do
            if display_qty.nil? || display_qty.to_i > @index.to_i
              @template.concat(@template.capture(&block))
            else
              break
            end
          end
        ensure
          @campaigns = nil
        end
      end
      alias_method :each_cart_applied_campaigns, :each_order_applied_campaigns

      # TODO 多言語対応
      # 2009/10現在ではcampaignに言語ごとの名称を保持していないため
      # campaign.nameを表示しています。
      desc <<-EOS
        [概要:適用されているキャンペーン名称を表示します。ec:each_order_applied_camapginsの中で使用してください。]
        [引数 なし:]
        [戻り値:キャンペーン名称]
        [使用例: ec:each_order_applied_camapginsを参照してください。]
        [同様のタグ： ec:cart_applied_campaign_name ショッピングカート画面で使用します]
      EOS
      def order_applied_campaign_name
        raise UsageError, "order_applied_campaign_nameはec:each_order_applied_camapginsの中で使用してください" unless @campaign
        return @campaign.name
      end
      alias_method :cart_applied_campaign_name, :order_applied_campaign_name

      desc <<-EOS
        [概要:適用されているキャンペーンページへのリンクを表示します。ec:each_order_applied_camapginsの中で使用してください。]
        [引数 なし:]
        [戻り値:キャンペーンページリンク]
        [使用例: ec:each_order_applied_camapginsを参照してください。]
        [同様のタグ： ec:cart_applied_campaign_link ショッピングカート画面で使用します]
      EOS
      def order_applied_campaign_link(*args, &block)
        raise UsageError, "order_applied_campaign_linkはec:each_order_applied_camapginsの中で使用してください" unless @campaign
        text, options = extract(args, :text => nil)
        text = @template.capture(&block) if block_given?
        href = @campaign.main_page_url
        if href.blank?
         return @template.concat(text) if text
        else
         return @template.concat(@template.link_to(text, {:href => href, :escape => false}, options)) if text
        end
      end
      alias_method :cart_applied_campaign_link, :order_applied_campaign_link
      
      desc <<-EOS
        [概要:支払い方法が代引き手数料の場合中身を表示します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例：
          {{{
          <ec:if_order_total_cod_charge>
            代金引換手数料：<ec:order_total_cod_charge />円
          </ec:if_order_total_cod_charge>
          → 支払方法が代金引き当ての場合、代金引換手数料を表示します。
          }}}
        ]
      EOS
      def if_order_total_cod_charge(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        order_shipping = @order.object.order_shippings.first
        if order_shipping.payment_type_div == OrderShipping.payment_type_div_id_by_key(:cod)
          yield if block_given?
        end
      end
      alias_method :if_order_product_cod_charge, :if_order_total_cod_charge

      desc <<-EOS
        [概要:お届け先指定・お支払方法用の確認画面へ進むボタンを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:button_to_confirmation_order>
             次へ進む→
           </ec:button_to_confirmation_order>
           }}}
        ]
      EOS
      def button_to_confirmation_order(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        button_name = @template.capture(&block) if block_given?
        result = @template.submit_tag(button_name, {:id => "send"})
        @template.concat(result) if result && block_given?
      end
      
      private
      def display_number_with_unit(value, prefix="", postfix="", not_render_zero=false)
        return "" unless value
        str_value = @template.number_with_delimiter(value)
        return "" if str_value.blank?
        return "" if str_value == "0" && not_render_zero
        "#{prefix}#{str_value}#{postfix}"
      end
    end
  end
end
