# -*- coding: utf-8 -*-

module EcRider
  module EcHelper
    module ProductTag
      extend MethodDescription

      PRODUCT_TAG_BIND_OPTION_METHODS = [
        :product_count, :product_name_1, :product_name_2,
        :product_standard_price,
        :product_outline, :product_explanation, :product_recommendation, :product_recommendation_2, :product_keyword, :product_stock_count
      ]

      desc <<-EOS
        [概要:商品の総数を表示します。]
        [引数:なし]
        [戻り値:商品の総数。3桁カンマ区切りで表示されます。]
        [使用例:
          {{{
          全部で<ec:product_count/>商品
          ⇒　全部で1,000商品
          }}}
        ]
      EOS
      def product_count(*args, &block)
        get_all_product
        @template.instance_variable_get("@products").size
      end

      desc <<-EOS
        [概要:{{{商品検索フォームからの検索結果を出力する場合に使用します。このタグ内に<ec:each_products>を記述すると検索結果の一覧を表示できます。}}}]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
          <ec:searched_products>
              <ec:product_crumbs_display/>
              <ec:each_products>
                  <ec:product_name_1/>
              </ec:each_products>
          </ec:searched_products>
          　　⇒ 検索条件にヒットした商品の商品名を表示します。
          }}}
        ]
      EOS
      def searched_products(*args, &block)
        # 商品検索パラメータが有る場合、検索商品をインスタンスに格納
        params = @template.send(:params)
        if !params[:condition].nil? || !params[:category_code].nil?
          condition_hash = {}
          if params[:category_code]
            condition_hash[:category_code] = params[:category_code]
            condition_hash[:per_page] = !params[:per_page].nil? ? params[:per_page] : 20
          else
            condition_hash = params[:condition]
          end

          condition_hash[:user_rank_id] = get_user_rank_id()
          condition_hash[:order] = params[:order] if params[:order]
          @condition = Product::CategorySearchForm.new(condition_hash)
          @per_page = !@condition.per_page.blank? ? @condition.per_page.to_i : 20
          @condition_page = params[:page]
          search_products = Product.paginate(@condition.to_find_options(:page => @condition_page, :per_page => @per_page))
        end
        @template.instance_variable_set("@condition", @condition)
        @template.instance_variable_set("@search_products", search_products)
        @template.instance_variable_set("@products", search_products)
        begin
          yield
        ensure
          @products = nil
          @template.instance_variable_set("@products", nil)
          @template.instance_variable_set("@condition", nil)
          @template.instance_variable_set("@search_products", nil)
        end
      end

      desc <<-EOS
        [概要:{{{カテゴリコードを引数に指定して商品を検索します。このタグ内に<ec:each_products>を記述すると検索結果の一覧を表示できます。}}}]
        [引数 category_codes(任意):カテゴリコードをカンマ区切りで指定します。]
        [引数 product_separate_value(任意):検索結果の１ページに表示する件数を指定します。指定しない場合は10件表示します。]
        [戻り値:なし]
        [使用例:
          {{{
          <ec:condition_products category_codes="code1, code2, code3">
              <ec:each_products>
                  <ec:product_name_1/>
              </ec:each_products>
          </ec:condition_products>
          　　⇒ カテゴリコードcode1, code2, code3のカテゴリを持つ商品の商品名を表示します。
          }}}
        ]
      EOS
      def condition_products(*args, &block)
        condition_hash = {:user_rank_id => get_user_rank_id()}
        condition_hash[:category_codes], condition_hash[:per_page] = extract(args, :category_codes => nil, :product_separate_value => 10)
        # カテゴリコードが指定されていない場合は検索しない
        unless condition_hash[:category_codes].nil?
          @condition = Product::CategorySearchForm.new(condition_hash)
          @per_page = !@condition.per_page.blank? ? @condition.per_page.to_i : 20
          params = @template.send(:params)
          @condition_page = params[:page] || 1
          search_products = Product.paginate(@condition.to_find_options(:page => @condition_page, :per_page => @per_page))
          @template.instance_variable_set("@products", search_products)
          @template.instance_variable_set("@condition", @condition)
          @template.instance_variable_set("@search_products", search_products)
          begin
            yield
          ensure
            @products = nil
            @template.instance_variable_set("@products", nil)
            @template.instance_variable_set("@condition", nil)
            @template.instance_variable_set("@search_products", nil)
          end
        end
      end
      
      desc <<-EOS
        [概要:繰り返し表示している商品のインデックスを表示します。]
        [引数:なし]
        [戻り値:インデックス]
        [使用例:
          {{{
            <ec:product_index /> ⇒ 1
          }}}
        ]
      EOS
      def product_index
        @product_index + 1
      end

      desc <<-EOS
        [概要:取得している商品の分だけ繰り返しを行います。]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
            <ec:each_products>
              <ec:product_name_1/>
            </ec:each_products>
            ⇒ 全ての商品の商品名を表示
          }}}
        ]
      EOS
      def each_products(*args, &block)
        get_all_product
        @products = @template.instance_variable_get("@products")
        _each_with_index(:@products) do
          @template.instance_variable_set("@each_product", @product)
          yield
        end
        @template.instance_variable_set("@each_product", nil)
      end

      desc <<-EOS
        [概要:取得している商品のSKU分だけ繰り返しを行います。]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
            <ec:each_product_skus>
              <div>
                <ec:product_variation_item_name_1/>
                <ec:product_standard_price post_currency="円" />
              </div>
              <hr/>
            </ec:each_product_skus>
            ⇒ SKUごとのバリエーションと通常価格を表示
          }}}
        ]
      EOS
      def each_product_skus(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_skus = product_info(product_code, "skus")
        begin
          product_skus.each do |product_sku|
            @template.instance_variable_set(:@each_product_sku, product_sku)
            yield
          end
        ensure
          @each_product_sku = nil
        end
      end

      def instance_variable_get_with_code(name, code)
        @template.instance_variable_get("@#{name}#{code}".gsub(/-/, '_'))
      end

      def instance_variable_set_with_code(name, code, value)
        @template.instance_variable_set("@#{name}#{code}".gsub(/-/, '_'), value)
      end

      #product_codeが商品コードかSKUコードかを判断する
      def is_sku_code?(product_code)
        return (product_code =~ /^\w{1,13}-\w{2}-\w{2}-\w{2}$/)
      end
      #product_code（商品コード、または、商品SKUコード）から商品コードを取得する
      def get_product_code(product_code)
        return product_code[/^(\w{1,13})-\w{2}-\w{2}-\w{2}$/, 1] if is_sku_code?(product_code)
        product_code
      end
      #product_code（商品コード、または、商品SKUコード）からSKUコードを取得する
      def get_sku_code(product_code)
        return product_code if is_sku_code?(product_code)
        product = Product.find_by_product_code(product_code)
        return product.primary_sku.sku_code if product && product.primary_sku
        product_code
      end

      # TODO 未実装
#      desc <<-EOS
#        [概要:
#           {{{
#          商品をすべて取得します。何度もec:product_infoやec:product_image_linkなどを使用する場合、
#          その前にこのタグを置くと速くなるかもしれません。
#          引数にしていできる条件の種類には文字列と数値があります。
#          文字列の条件の指定の例：
#          ・完全一致：<ec:fetch_products category_name_1="=帽子">
#            ⇒ 「帽子」のみ
#          ・前方一致：<ec:fetch_products category_name_1="帽子*"/>
#            ⇒ 「帽子」「帽子掛け」など
#          ・後方一致：<ec:fetch_products category_name_1="*帽子"/>
#            ⇒ 「帽子」「麦わら帽子」など
#          ・部分一致(明示)：<ec:fetch_products category_name_1="*帽子*"/>
#          ・部分一致(非明示)：<ec:fetch_products category_name_1="帽子"/>
#            ⇒ 「帽子」「帽子掛け」「麦わら帽子」など
#          数値の条件の指定の例：
#          ・完全一致：<ec:fetch_products price="=1000"/>
#            ⇒ 1000のみ
#          ・以上：<ec:fetch_products price=">=1000"/>
#            ⇒ 1000以上。1000を含む
#          ・より大きい：<ec:fetch_products price=">=1000"/>
#            ⇒ 1000より大きいもの。1000を含まない
#          ・以下：<ec:fetch_products price=">=1000"/>
#            ⇒ 1000以下。1000を含む
#          ・より小さい：<ec:fetch_products price=">=1000"/>
#            ⇒ 1000より小さい。1000を含まない<br/>
#          ・範囲：<ec:fetch_products price=">=1000<=2000"/>
#            ⇒ 1000以上かつ2000以下。
#
#          「または」の指定：
#          条件の指定にカンマを使用することで「AまたはB」という条件も指定可能です。
#          ・<ec:fetch_products category_name_1="=帽子,*キャップ,*ハット"/>
#            ⇒ 「帽子」「キャップ」「ベースボールキャップ」「ハット」「シャンプーハット」
#          ・<ec:fetch_products price="=1000,=2000"/>
#            ⇒ 1000あるいは2000のもの
#          ・<ec:fetch_products price="<=1000,>=2000"/>
#            ⇒ 1000以下あるいは2000以上のもの
#           }}}
#        ]
#        [引数 category_name_1(任意): カテゴリ名1(文字列)]
#        [引数 category_name_2(任意): カテゴリ名2(文字列)]
#        [引数 category_name_3(任意): カテゴリ名3(文字列)]
#        [引数 product_name_1(任意): 商品名1(文字列)]
#        [引数 product_name_2(任意): 商品名2(文字列)]
#        [引数 product_code(任意): 商品コード(文字列)]
#        [引数 price(任意): 価格(数値)]
#        [引数 stock_count(任意): 在庫数(数値)]
#        [戻り値:なし]
#        [使用例:{{{<ec:fetch_products/>}}}]
#      EOS
      # TODO 未実装
      def fetch_products(*args, &block)
#        options = args.extract_options!
#        products = Product.find(:all, Product::FinderForm.new(options).to_find_options)
#        products_by_id = products.inject({}) do |dest, product|
#          dest[product.id] = product
#          dest
#        end
#        products_hash = products.inject({}) do |dest, product|
#          dest[product.product_code] = product
#          dest
#        end
#        @fetched_products ||= {}
#        @fetched_products.update(products_hash)
#        product_displays = ProductDisplay.find(:all,
#          :conditions => ["locale = ? AND product_id in (?)", (I18n.locale || 'ja').to_s, products_by_id.keys])
#        product_displays.each do |display|
#          product = products_by_id[display.product_id]
#          product.current_display = display if product
#        end
        return ""
      end

       desc <<-EOS
        [概要:商品詳細へのリンクを表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 text(任意):リンクの文字列を指定します。指定しない場合は商品名が使用されます。]
        [戻り値:詳細URLに登録したURLへのリンク。]
        [使用例:
          {{{
          <ec:product_detail_link product_code="001"/>
          }}}
        ]
      EOS
      def product_detail_link(*args, &block)
        product_code, text, options = extract(args, :product_code => "", :text => "")
        text ||= product_info(product_code, 'product_name_1')
        detail_url = product_info(product_code, 'detail_url')
        link_to(text, {:href => detail_url, :escape => false})
      end
      
      desc <<-EOS
        [概要:商品詳細へのリンクを表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:詳細URLに登録したURLへのリンク。]
        [使用例:
          {{{
          <ec:product_link product_code="001">
            <ec:product_name_2/>
          </ec:product_link>
          }}}
        ]
      EOS
      def product_link(*args, &block)
        product_code, options = extract(args, :product_code => "")
        text = @template.capture(&block) if block_given?
        detail_url = product_info(product_code, 'detail_url')
        result = link_to(text, {:href => detail_url, :escape => false}, options) if text && detail_url
        # ブロックが渡されたメソッド(空でない中身を持つecタグ)は <%= %> ではなく <% %>で出力されるので、
        # 結果をoutput_bufferに出力するようにconcatします
        @template.concat(result) if result && block_given?
        result
      end

      desc <<-EOS
        [概要:商品検索フォームを出力します。]
        [引数 render_name(必須):商品検索結果を表示する画面のURLを指定します。]
        [引数 product_separate_value(任意):商品検索結果の１ページの表示件数を指定します。何も指定しない場合は10が適用されます。]
        [使用例:
          {{{
            <ec:product_search_form render_name="list">
              <ec:submit>検索</ec:submit>
            </ec:product_search_form>
          　「検索ボタンを押すと、listのページに遷移し、検索結果を表示する」検索フォーム。
          }}}
        ]
      EOS
      def product_search_form(*args, &block)
        per_page, render_name, options = extract(args, :product_separate_value => 10, :render_name => nil)
        raise UsageError, "ec:product_search_formを使用する時は遷移先のrender_nameオプションを設定してください" unless render_name
        @template.form_for(:condition, :url => {:controller =>'cms', :action => render_name},:html => {:method => :get}) do |f|
          @product_search_form = f
          @template.concat(@template.hidden_field_tag('condition[per_page]', per_page))
          begin
            @template.concat(@template.capture(&block))
          ensure
            @product_search_form = nil
          end
        end
      end

      desc <<-EOS
        [概要:すべての商品カテゴリをセレクトボックスで表示します。商品検索フォーム内でのみ使用できます。]
        [引数 include_name(任意):カテゴリセレクトのoptions上部に文字列を設定します。]
        [使用例:
          {{{
            <ec:product_search_form_category_select/>

            <ec:product_search_form_category_select include_name="すべてのカテゴリ"/>

            <ec:product_search_form_category_select include_name=""/>

          }}}
        ]
      EOS
      def product_search_form_category_select(*args)
        raise UsageError, "ec:product_search_form_category_selectはec:product_search_formの中で使用してください" unless @product_search_form
        categories = ProductCategory.find_all_categories(I18n.locale.to_s, nil, '　 ')
        include_name, options = extract(args, :include_name => nil)
        product_category_list = []
        unless include_name.nil?
          product_category_list << [include_name, ""]
        end
        categories.each do |category|
          product_category_list << [category.name, category.code]
        end
        options = {}
        if condition_params = @template.send(:params)[:condition]
          options[:selected] = condition_params[:category_code]
        end
        @product_search_form.select(:category_code, product_category_list, options)
      end

      desc <<-EOS
        [概要:商品検索キーワードを入力するinputタグを出力します。商品検索フォーム内でのみ使用できます。]
      EOS
      def product_search_form_keyword(options = {})
        raise UsageError, "ec:product_search_form_keywordはec:product_search_formの中で使用してください" unless @product_search_form
        options = {:maxLength => 20,:size => 20,:class=>:default}.update(options || {})
        if condition_params = @template.send(:params)[:condition]
          options[:value] = condition_params[:keyword]
        end
        @product_search_form.text_field_valid(:keyword, options)
      end

      desc <<-EOS
        [概要:検索条件をセレクトボックスで表示します。商品検索フォーム内でのみ使用できます。]
        [引数 and_text(必須):複数の検索条件をどちらも一致させたい場合の文言を入力します。]
        [引数 or_text(必須):複数の検索条件のいずれかに一致させたい場合の文言を入力します。]
      EOS
      def product_search_form_condition_select(*args)
        raise UsageError, "ec:product_search_form_condition_selectはec:product_search_formの中で使用してください" unless @product_search_form
        and_text, or_text, options = extract(args, :and_text => nil, :or_text => nil)
        raise UsageError, "ec:product_search_form_condition_selectは引数｢and_text｣を必ず設定して利用してください" if and_text.nil?
        raise UsageError, "ec:product_search_form_condition_selectは引数｢or_text｣を必ず設定して利用してください" if or_text.nil?
        if condition_params = @template.send(:params)[:condition]
          selected = condition_params[:condition_div].to_i
        end
        @product_search_form.select(:condition_div, [[and_text, 0],[or_text, 1]], {:selected => selected}, options)
      end

      desc <<-EOS
        [概要:商品検索結果を表示する時に該当する商品がなかった場合にこのタグ内の内容を表示します。使用する場合はec:searched_productsの中で記述します。]
        [使用例:カテゴリに子階層がある場合
          {{{
          <ec:searched_product>
            <ec:searched_product_empty>
              該当する商品はありません。
            </ec:searched_product_empty>
            <ec:searched_product_not_empty>
              <ec:each_products>
                <ec:product_name_1/>
              </ec:each_products>
            </ec:searched_product_not_empty>
          </ec:searched_product>
            →
            << 検索結果商品が０件の場合>>
              該当する商品はありません。
            << 検索結果商品が０件ではない場合>>
            検索結果商品の商品名を繰り返し表示
          }}}
        ]
      EOS
      def searched_product_empty(&block)
        search_products = @template.instance_variable_get(:@search_products)
        raise UsageError, "ec:search_product_emptyはec:searched_productsの中で使用してください" unless search_products
        # 商品一覧と商品検索結果を兼用で表示させたい場合もあるため@search_productsがnilの場合は表示しない
        if search_products.empty?
          @template.concat(@template.capture(&block))
        end
      end

      desc <<-EOS
        [概要:商品検索結果を表示する時に該当する商品がある場合にこのタグ内の内容を表示します。使用する場合はec:searched_productsの中で記述します。]
      EOS
      def searched_product_not_empty(&block)
        search_products = @template.instance_variable_get(:@search_products)
        # 商品一覧と商品検索結果を兼用で表示させたい場合もあるため@search_productsがnilの場合は表示しない
        raise UsageError, "ec:search_product_not_emptyはec:searched_productsの中で使用してください" unless search_products
        unless search_products.empty?
          @template.concat(@template.capture(&block))
        end
      end

      desc <<-EOS
        [概要:検索ワードがある場合は中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_search_condition_exits>検索結果</ec:if_search_condition_exits>
           }}}
        ]
      EOS
      def if_search_condition_exits(&block)
        condition =  @template.instance_variable_get(:@condition)
        unless condition.nil? || condition.keyword.nil?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:検索ワードがない場合は中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:unless_search_condition_exits>検索結果</ec:unless_search_condition_exits>
           }}}
        ]
      EOS
      def unless_search_condition_exits(&block)
        condition = @template.instance_variable_get(:@condition)
        if condition.nil? || condition.keyword.nil?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:検索結果一覧ページで、検索キーワードを表示します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_search_condition_exits>検索結果:</ec:if_search_condition_exits>
           }}}
        ]
      EOS
      def search_condition(&block)
        condition = @template.instance_variable_get(:@condition)
        unless condition.nil? || condition.keyword.nil?
          condition.keyword
        end
      end

      desc <<-EOS
        [概要:商品検索結果を表示する時に商品の表示開始数を表示します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:product_pagination_total_count/>件中
            <ec:product_pagination_start_index/>～<ec:product_pagination_last_index/>件を表示しています。
          }}}
        ]
      EOS
      def product_pagination_total_count(options = {})
        search_products = @template.instance_variable_get(:@search_products)
        raise UsageError, "ec:product_pagination_total_countはec:searched_productsの中で使用してください" unless search_products
        search_products.total_entries
      end

      desc <<-EOS
        [概要:商品検索結果を表示する時に商品の表示開始数を表示します。使用する場合はec:searched_productsの中で記述します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:product_pagination_total_count/>件中
            <ec:product_pagination_start_index/>～<ec:product_pagination_last_index/>件を表示しています。
          }}}
        ]
      EOS
      def product_pagination_start_index(options = {})
        search_products = @template.instance_variable_get(:@search_products)
        raise UsageError, "ec:product_pagination_start_indexはec:searched_productsの中で使用してください" unless search_products
        search_products.offset + 1
      end

      desc <<-EOS
        [概要:商品検索結果を表示する時に商品の表示開始数を表示します。使用する場合はec:searched_productsの中で記述します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:product_pagination_total_count/>件中
            <ec:product_pagination_start_index/>～<ec:product_pagination_last_index/>件を表示しています。
          }}}
        ]
      EOS
      def product_pagination_last_index(options = {})
        search_products = @template.instance_variable_get(:@search_products)
        raise UsageError, "ec:product_pagination_last_indexはec:searched_productsの中で使用してください" unless search_products
        search_products.offset + search_products.length
      end

      desc <<-EOS
        [概要:商品検索結果が複数ページある場合に、ページ遷移リンクを表示します。ec:searched_productsの中で使用します。]
        [引数:previous_label(任意)前のページのリンクボタンの名前を指定できます。デフォルトは｢前へ｣になります。]
        [引数:next_label(任意)前のページのリンクボタンの名前を指定できます。デフォルトは｢次へ｣になります。]
        [使用例:
          {{{
            <ec:product_pagination_link previous_label="前のページ" next_label="次のページ"/>
            →　<a herf="">前のページ</a><a herf="">1</a><a herf="">2</a><a herf="">3</a><a herf="">次のページ</a>
          }}}
        ]
      EOS
      def product_pagination_link(*args)
        next_label, previous_label, option = extract(args, :next_label =>'次へ' , :previous_label => '前へ')
        search_products = @template.instance_variable_get(:@search_products)
        raise UsageError, "ec:product_pagination_linkはec:searched_productsの中で使用してください" unless search_products
        # lang[]のパラメータが入ってしまいCMSコントローラでエラーになってしまうのでパラメータを書き換え
        # 言語対応に支障が出るようであれば別方法を検討する必要があります。
        @template.will_paginate(search_products, :previous_label => previous_label, :next_label => next_label, :params=>{:lang=>nil})
      end

      desc <<-EOS
        [概要:商品検索結果の表示件数を切り替えるセレクトボックスを表示します。]
        [引数:render_name(必須)商品検索結果表示後のページ名を設定します。]
        [引数:select_count(任意)セレクトボックスで選ぶ表示件数の件数を｢○,○,○｣形式で設定します。何も指定しなかった場合、10,20,30を選択肢にします。]
        [使用例:
          {{{
          <ec:product_pagination_count_per_page/>
          <ec:product_pagination_count_per_page select_count="20,50,100"/>
          }}}
        ]
      EOS
      def product_pagination_count_per_page(*args)
        render_name, select_count, options = extract(args, :render_name => nil, :select_count => "10, 20, 30")
        raise UsageError, "ec:product_pagination_count_per_pageを使用する時は遷移先のrender_nameオプションを設定してください" unless render_name
        @template.form_for(:condition, :url => {:controller =>'cms', :action => render_name},:html => {:method => :get, :id => 'count_per_page_form'}) do |f|
          @template.concat(f.hidden_field(:category_code))
          @template.concat(f.hidden_field(:keyword))
          @template.concat(f.hidden_field(:condition_div))
          condition_page = @template.instance_variable_get(:@condition_page)
          @template.concat(@template.hidden_field_tag(:page, condition_page)) unless condition_page.nil?
          @template.concat(f.select(:per_page,select_count.split(/\s*,\s*/),{}, {:onChange => "this.form.submit();"}))
        end
        ""
      end

      desc <<-EOS
        [概要:商品検索結果用の並び替えリンクを表示します。]
        [引数 order(任意):検索結果の表示順序を設定します。何も指定しなかった場合、商品情報の表示順に従った並びとなります。(1→価格の安い順、2→価格の高い順、3→新着順）]
        [引数 (オプション):HTMLオプションを指定します。]
        [使用例:
          {{{
          <ec:product_sort_link order="2" class="sort_link">価格の高い順</ec:product_sort_link>
          }}}
        ]
      EOS
      def product_sort_link(*args, &block)
        order, options = extract(args, :order => '0')
        @category_item = @template.instance_variable_get(:@category_item)
        return '' unless block_given?

        controller = @template.send(:params)[:controller]
        action = @template.send(:params)[:action]
        per_page = @template.send(:params)[:per_page]
        category_code = @template.send(:params)[:category_code]
        condition = @template.send(:params)[:condition]
        link_to_url = {:controller => controller, :action => action, :per_page => per_page, :order => order, :category_code => category_code, :condition => condition}

        result = link_to(@template.capture(&block), link_to_url, options)
        @template.concat(result)
      end

      desc <<-EOS
        [概要:商品名1を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:商品名1に登録した文字列。]
        [使用例:
          {{{
          <ec:product_name_1 product_code="001"/>
          }}}
        ]
      EOS
      def product_name_1(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_info(product_code, 'product_name_1')
      end
      alias_method :product_group_name, :product_name_1
      alias_method :product_group_product_name, :product_name_1

      desc <<-EOS
        [概要:商品名2を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:商品名2に登録した文字列。]
        [使用例:
          {{{
          <ec:product_name_2 product_code="001"/>
          }}}
        ]
      EOS
      def product_name_2(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_info(product_code, 'product_name_2')
      end

      desc <<-EOS
        [概要:在庫数を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:在庫数]
        [使用例:
          {{{
          <ec:product_orderable_count product_code="001"/>
          }}}
        ]
      EOS
      def product_orderable_count(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_sku_info(product_code, "orderable_count")
      end

      desc <<-EOS
        [概要:商品のキャンペーン適用後の商品価格を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。デフォルトは店舗情報で設定した通貨単位です。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。デフォルトは店舗情報で設定した通貨単位です。]
        [戻り値:キャンペーン適用価格]
        [使用例:
          {{{
          <ec:product_campaign_price product_code="001"/>
          }}}
        ]
      EOS
      def product_campaign_price(*args, &block)
        product_code, options = extract(args, :product_code => "")
        pre_currency, post_currency = setting_pre_currency_and_post_currency(options)
        product_sku_code = product_sku_info(product_code, 'sku_code')
        simulation_cart = simulation_cart_for(product_sku_code)
        campaign_price = @template.number_with_delimiter(simulation_cart.total_amount)
        return "#{pre_currency}#{campaign_price}#{post_currency}"
      end

      desc <<-EOS
        [概要:商品に適用できるキャンペーンごとに繰り返しをします。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 display_qty(任意): 最大表示数を設定します。設定しない場合は適用されるキャンペーン分すべてを繰り返し実行します。]
        [戻り値:]
        [使用例:
          {{{
          <ec:product_available_campaign_each product_code="001">
            <ec:product_available_campaign_link>
              <ec:product_available_campaign_name />
            </ec:product_available_campaign_link>
          </ec:product_available_campaign_each>
             →適用されるキャンペーン名称リンクを繰り返し表示します。
          }}}
        ]
      EOS
      def product_available_campaign_each(*args, &block)
        display_qty, product_code, options = extract(args, :display_qty => nil, :product_code => "")
        product_sku_code = product_sku_info(product_code, 'sku_code')
        raise "No sku code given." if product_sku_code.blank?
        simulation_cart = simulation_cart_for(product_sku_code)
        @campaigns = simulation_cart.applied_campaigns_for_cart_and_items
        begin
          _each_with_index(:@campaigns, :@campaign, :@index) do
            if display_qty.nil? || display_qty.to_i > @index.to_i
              @template.concat(@template.capture(&block))
            else
              break
            end
          end
        ensure
          @campaigns = nil
        end
      end

      # TODO 多言語対応
      # 2009/10現在ではcampaignに言語ごとの名称を保持していないため
      # campaign.nameを表示しています。
      desc <<-EOS
        [概要:商品に適用できるキャンペーン名称を表示します。ec:product_available_campaign_eachの中で使用してください。]
        [引数:なし]
        [戻り値:キャンペーン名称]
        [使用例: ec:product_available_campaign_eachを参照ください]
      EOS
      def product_available_campaign_name
        raise UsageError, "product_available_campaign_nameはec:product_available_campaign_eachの中で使用してください" unless @campaign
        return @campaign.name
      end

      desc <<-EOS
        [概要:商品に適用できるキャンペーンページへのリンクを表示します。ec:product_available_campaign_eachの中で使用してください。]
        [引数:なし]
        [戻り値:キャンペーンページリンク]
        [使用例: ec:product_available_campaign_eachを参照ください]
      EOS
      def product_campaign_page_link(*args, &block)
        raise UsageError, "product_available_campaign_nameはec:product_available_campaign_eachの中で使用してください" unless @campaign
        options = extract(args).first #extract(args)は配列を返すので、戻り値は[options]になります。optionsだけ取り出すために.firstしています。
        text = @template.capture(&block) if block_given?
        href = @campaign.main_page_url
        if href.blank?
         return @template.concat(text) if text
        else
          return @template.concat(@template.link_to(text, {:href => href, :escape => false}, options)) if text
        end
      end

      desc <<-EOS
        [概要:SKUコードを表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:在庫数]
        [使用例:
          {{{
          <ec:product_sku_code product_code="001"/>
          }}}
        ]
      EOS
      def product_sku_code(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_sku_info(product_code, "sku_code")
      end

      desc <<-EOS
        [概要:商品重量（グラム単位）を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 suffix(任意):重さ単位を指定します。数値の後に表示されます。]
        [戻り値:在庫数]
        [使用例:
          {{{
          <ec:product_weight suffix="g"/>
            ⇒ 100g
          }}}
        ]
      EOS
      def product_weight(*args, &block)
        product_code, suffix = extract(args, :product_code => "", :suffix => "")
        weight = product_sku_info(product_code, "weight")
        "#{weight}#{suffix}"
      end

      # 商品カテゴリ
      (1..3).each do |index|
        desc <<-EOS
        [旧名称: category_name_#{index}（非推奨）]
        [概要:カテゴリー名#{index}を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:カテゴリー名#{index}に登録した文字列。]
        [使用例:
          {{{
          <ec:product_category_name_#{index} product_code="001"/>
          }}}
        ]
        EOS
        module_eval(<<-"EOS", __FILE__, __LINE__)
          def product_category_name_#{index}(*args, &block)
            product_code, options = extract(args, :product_code => "")
            product_info(product_code, 'category_code_#{index}_name')
          end
          # 互換性のために古い名前も残しておきます。
          alias_method :category_name_#{index}, :product_category_name_#{index}
        EOS
      end

      # 商品バリエーション
      (1..3).each do |index|
        desc <<-EOS
          [概要:商品バリエーション名#{index}を表示します。]
          [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
          [戻り値:在庫数]
          [使用例:
            {{{
            <ec:product_variation_name_#{index} product_code="001"/>
            }}}
          ]
        EOS
        module_eval(<<-"EOS", __FILE__, __LINE__)
          def product_variation_name_#{index}(*args, &block)
            product_code, options = extract(args, :product_code => "")
            product_info(product_code, "variation_#{index}_name")
          end
        EOS
      end

      desc <<-EOS
        [旧名称: product_price（非推奨）]
        [概要:商品価格を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [戻り値:商品価格に登録した数値。3桁カンマ区切りで表示されます。]
        [使用例:
          {{{
          <ec:product_standard_price product_code="001" post_currency="円"/>
            ⇒ 2,800円
          }}}
        ]
      EOS
      def product_standard_price(*args, &block)
        product_code, options = extract(args, :product_code => "")
        pre_currency, post_currency = setting_pre_currency_and_post_currency(options)
        standard_price = @template.number_with_delimiter(product_price_info(product_code, 'standard_price'))
        "#{pre_currency}#{standard_price}#{post_currency}"
      end
      # 互換性のために古い名前も残しておきます。
      alias_method :product_price, :product_standard_price
      alias_method :product_group_price, :product_standard_price


      desc <<-EOS
        [概要:定期購入価格を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [戻り値:定期購入価格に登録した数値。3桁カンマ区切りで表示されます。]
        [使用例:
          {{{
          <ec:product_regular_purchase_price product_code="001" post_currency="円"/>
            ⇒ 3,000円
          }}}
        ]
      EOS
      def product_regular_purchase_price(*args, &block)
        product_code, options = extract(args, :product_code => "")
        pre_currency, post_currency = setting_pre_currency_and_post_currency(options)
        regular_purchase_price = @template.number_with_delimiter(product_price_info(product_code, 'regular_purchase_price'))
        "#{pre_currency}#{regular_purchase_price}#{post_currency}"
      end

      desc <<-EOS
        [概要:出荷可能日数を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 suffix(任意): 単位を指定します。数値の後ろに表示されます。]
        [戻り値:出荷可能日数に登録した数値が表示されます。]
        [使用例:
          {{{
          <ec:product_lead_time suffix="日"/>
            ⇒ 3日
          }}}
        ]
      EOS
      def product_lead_time(*args, &block)
        product_code, suffix = extract(args, :product_code => "", :suffix => "")
        lead_time = product_sku_info(product_code, 'lead_time')
        "#{lead_time}#{suffix}"
      end

      desc <<-EOS
        [概要:メーカー希望小売価格を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [戻り値:メーカー希望小売価格に登録した数値。3桁カンマ区切りで表示されます。]
        [使用例:
          {{{
          <ec:product_maker_price product_code="001" post_currency="円"/>
            ⇒ 3,500円
          }}}
        ]
      EOS
      def product_maker_price(*args, &block)
        product_code, options = extract(args, :product_code => "")
        pre_currency, post_currency = setting_pre_currency_and_post_currency(options)
        maker_price = product_sku_info(product_code, 'maker_price')
        if maker_price
          "#{pre_currency}#{@template.number_with_delimiter(maker_price)}#{post_currency}"
        end
      end
      alias_method :product_group_maker_price, :product_maker_price

      desc <<-EOS
        [概要:JANコードを表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:JANコード]
        [使用例:
          {{{
          <ec:product_jan_code product_code="001"/>
            ⇒ 3,500円
          }}}
        ]
      EOS
      def product_jan_code(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_sku_info(product_code, 'jan_code')
      end

      desc <<-EOS
        [概要:取得している商品構成の分だけ繰り返しを行います。]
        [引数 なし]
        [戻り値:なし]
        [使用例:
          {{{
            <ec:each_product_components>
              <ec:product_component_name/>　×　<ec:product_component_count/>個<br/>
            </ec:each_product_components>
          　　⇒ 商品構成の商品名と構成数を表示
          }}}
        ]
      EOS
      def each_product_components(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_components =  product_sku_info(product_code, "components")

        return @template.concat("<!-- セット内容はありません -->") if product_components.empty?

        product_components.each do |component|
          @template.instance_variable_set(:@product_component, component)
          yield
        end
      end

      desc <<-EOS
        [概要:商品構成の商品名を表示します。]
        [引数 なし]
        [戻り値:商品構成の商品名が表示されます。]
        [使用例:{{{<ec:each_product_components>を参照してください。}}}]
      EOS
      def product_component_name(*args, &block)
        product_component =  @template.instance_variable_get(:@product_component)
        raise UsageError, "product_component_nameはec:each_product_componentsの中で使用してください" unless product_component
        component_sku_id = product_component.component_sku_id
        sku = ProductSku.find(component_sku_id)
        sku.product_name_1 if sku
      end

      desc <<-EOS
        [概要:商品構成の構成数を表示します。]
        [戻り値:商品構成の構成数が表示されます。]
        [使用例:{{{<ec:each_product_components>を参照してください。}}}]
      EOS
      def product_component_count(*args, &block)
        product_component =  @template.instance_variable_get(:@product_component)
        raise UsageError, "product_component_countはec:each_product_componentsの中で使用してください" unless product_component
        product_component.count
      end

      desc <<-EOS
        [概要:取得している商品画像の分だけ繰り返しを行います。]
        [引数 display(任意):商品画像の表示数を指定します。指定した数だけ繰り返します。]
        [戻り値:なし]
        [使用例:
          {{{
          <ul>
            <ec:each_product_images display="3">
              <li>
                <ec:product_image type="small" />
              <li>
            </ec:each_product_images>
          <ul>
          　　⇒ 表示数で指定した分の商品画像を表示
          }}}
        ]
      EOS
      def each_product_images(*args, &block)
        display, product_code, options = extract(args, :display => 20, :product_code => "")
        images =  product_info(product_code, "images")
        return @template.concat("<!-- 商品画像は未登録です -->") if images.empty?
        display.to_i.times do |index|
          if images[index]
            @template.instance_variable_set(:@product_image, images[index])
            yield
          end
        end
      end

      desc <<-EOS
        [概要:原産国を表示します。]
        [引数 product_code(準必須):{{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:原産国に登録した国名称。]
        [使用例:{{{<ec:product_origin product_code="001"/>}}}]
      EOS
      def product_origin(*args, &block)
        product_code, options = extract(args, :product_code => "")
        country_code = product_info(product_code, 'origin')
        if country_code && country_code.length == 2
          if country = Country.find(:first, :conditions => ['code = ?', country_code])
            country.name
          end
        else
          country_code
        end
      end

      desc <<-EOS
        [概要:商品画像を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 index(任意):{{{何番目の商品画像を表示するかを指定します。指定しない場合はメイン画像が表示されます。<ec:each_product_images>の中では使用できません。}}}]
        [引数 type(任意):商品画像のサイズを指定します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:商品画像に登録した画像。]
        [使用例:
          {{{
          <ec:product_image product_code="001"/>
          }}}
        ]
      EOS
      def product_image(*args, &block)
        index, product_code, type, options = extract(args, :index => nil, :product_code => "", :type => nil)
        image = @template.instance_variable_get(:@product_image)
        if index.nil? && image.nil?
          image =  product_info(product_code, 'main_image')
        end
        if index
          product_id = product_info(product_code, 'id')
          image = ProductImage.find(:first, :conditions => ["product_id = ? and display_order = ?", product_id, index])
        end
        if image
          options["alt"] ||= product_info(product_code, 'product_name_1') if product_info(product_code, 'product_name_1')
          src = options["src"]
          src ||= image.blank? ? nil : image.public_filename(type)
          return @template.image_tag(src, options) if src
        else
          return ''
        end
      end

      desc <<-EOS
        [概要: 商品の画像を、詳細画面へのリンクつきで表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 index(任意):{{{何番目の商品画像を表示するかを指定します。指定しない場合はメイン画像が表示されます。}}}]
        [引数 type(任意):商品画像のサイズを指定します。指定しない場合はアップロードしたままのサイズで表示します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数 (オプション): HTMLオプションを指定します。]
        [戻り値: 詳細画面へのリンクつきの商品画像。]
        [使用例:{{{<ec:product_image_link product_code="001"/>}}}]
      EOS
      def product_image_link(*args, &block)
        index, product_code, type, options = extract(args, :index => nil, :product_code => "", :type => nil)
        image =  product_info(product_code, 'main_image')
        if index
          product_id = product_info(product_code, 'id')
          image = ProductImage.find(:first, :conditions => ["product_id = ? and display_order = ?", product_id, index])
        end
        if image
          src = options["src"] ? options["src"] : image.public_filename(type)
          options["alt"] ||= product_info(product_code, 'product_name_1') if product_info(product_code, 'product_name_1')
          detail_url = product_info(product_code, 'detail_url')
          link_to(@template.image_tag(src, options), {:href => detail_url, :escape => false})
        else
          return ''
        end
      end
      alias_method :product_group_image_link, :product_image_link

      desc <<-EOS
        [概要:商品概要を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:商品概要に登録した文字列。]
        [使用例:
          {{{
          <ec:product_outline product_code="001"/>
          }}}
        ]
      EOS
      def product_outline(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_info(product_code, 'product_outline')
      end

      desc <<-EOS
        [概要:商品説明を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:商品説明に登録した文字列。]
        [使用例:
          {{{
          <ec:product_explanation product_code="001"/>
          }}}
        ]
      EOS
      def product_explanation(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_info(product_code, 'product_explanation')
      end

      desc <<-EOS
        [概要:検索キーワードを表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:検索キーワードに登録した文字列。]
        [使用例:
          {{{
            <ec:head>
              <ec:product_keyword product_code="001"/>
            </ec:head>
          }}}
        ]
      EOS
      def product_keyword(*args, &block)
        lang, product_code = extract(args, :lang => nil, :product_code => "" )
        prokey =  product_info(product_code, 'product_keyword')
        @template.content_for :meta do
          meta(:name => 'keywords', :content => prokey, :lang => lang)
        end
      end
      desc <<-EOS
        [概要:商品価格に換算レートを掛けた値を表示します。]
        [引数 product_code(準必須)：{{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 pre_currency(任意)：通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意)：通貨単位を指定します。数値の後ろに表示されます。]
        [戻り値:商品価格に換算レートを掛けた数値。3桁カンマ区切りで小数第二位まで表示されます。]
        [使用例:
          {{{
          <ec:exchange_price product_code="001" post_currency="元"/>
          　　⇒ 3,000.00元
          }}}
        ]
      EOS
      def exchange_price(options = {}, &block)
        product_code = options[:product_code] || ""
        post_currency = options[:post_currency] || ""
        pre_currency = options[:pre_currency] || ""
        price_jpn = product_info(product_code, 'price')

        if price_jpn == I18n.t("activerecord.errors.messages.EM990034", :value=> '')
          return nil
        end
        exchange_price = ShopSetting.exchange(price_jpn)
        return "" if exchange_price.nil?
        "#{pre_currency}#{@template.number_with_delimiter exchange_price}#{post_currency}"
      end

      # 未使用。（商品価格＋送料に換算レートを掛けた値を表示するタグ。）
      def exchange_price_include_shipping_cost(options = {}, &block)
        return ""
      end

      desc <<-EOS
        [旧名称: order_limit（非推奨）]
        [概要:購入上限数を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:購入上限数に登録した数値。3桁カンマ区切りで表示されます。]
        [使用例:
          {{{
          お一人様<ec:product_order_limit product_code="001"/>個まで
          　　⇒ お一人様5個まで
          }}}
        ]
      EOS
      def product_order_limit(*args, &block)
        product_code, options = extract(args, :product_code => "")
        @template.number_with_delimiter product_sku_info(product_code, 'order_limit')
      end
      # 互換性のために古い名前も残しておきます。
      alias_method :order_limit, :product_order_limit

      desc <<-EOS
        [旧名称: recommendation（非推奨）]
        [概要:推奨文言を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:推奨文言に登録した文字列。]
        [使用例:
          {{{
          <ec:product_recommendation product_code="001"/>
          }}}
        ]
      EOS
      def product_recommendation(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_info(product_code, 'recommendation')
      end
      # 互換性のために古い名前も残しておきます。
      alias_method :recommendation, :product_recommendation

      desc <<-EOS
        [旧名称: recommendation_2（非推奨）]
        [概要:推奨文言2を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:推奨文言2に登録した文字列。]
        [使用例:
          {{{
          <ec:product_recommendation_2 product_code="001"/>
          }}}
        ]
      EOS
      def product_recommendation_2(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_info(product_code, 'recommendation_2')
      end
      # 互換性のために古い名前も残しておきます。
      alias_method :recommendation_2, :product_recommendation_2

      desc <<-EOS
        [旧名称: stock_count（非推奨）]
        [概要:在庫数を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値:在庫数に登録した文字列。3桁カンマ区切りで表示されます。]
        [使用例:
          {{{
          あと<ec:product_stock_count product_code="001"/>個
          　　⇒ あと3個
          }}}
        ]
      EOS
      def product_stock_count(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_info(product_code, 'stock_count')
      end
      # 互換性のために古い名前も残しておきます。
      alias_method :stock_count, :product_stock_count

      desc <<-EOS
        [概要:取得している関連商品の分だけ繰り返しを行います。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 display(任意):関連商品の表示数を指定します。指定した数だけ繰り返します。]
        [引数 group_name(任意):関連商品グループ名を指定します。指定しなければ該当商品を含むすべての関連商品グループが対象となります。]
        [引数 all(任意):"all"を指定すると、該当商品を含むかどうかに関わらず"group_name"で指定した関連商品グループが対象となります。]
        [戻り値:なし]
        [使用例:
          {{{
          <ul>
            <ec:each_product_groups display="5" all="all">
              <li>
                <ec:product_image_link type="small"/>
                <ec:product_name_1 />
                価格：<ec:product_standard_price post_currency="円"/>
                定価：<ec:product_maker_price post_currency="円"/>
              <li>
            </ec:each_product_groups>
          <ul>
          　　⇒ 表示数で指定した分の関連商品の商品イメージと商品名を表示
          }}}
        ]
      EOS
      def each_product_groups(*args, &block)
        all, display, group_name, product_code, options = extract(args, :all => nil, :display => "5", :group_name => nil, :product_code => "")
        sku_id =  product_sku_info(product_code, "id")

        product_group_skus = ProductGroupSku.find_by_include_sku_id(sku_id, all, group_name, display, get_user_rank_id)
        display.to_i.times do |index|
          if product_group_skus[index] && product_group_skus[index].product_sku
            @template.instance_variable_set(:@product_group_sku, product_group_skus[index].product_sku)
            yield
            @template.instance_variable_set(:@product_group_sku, nil)
          end
        end
      end

      desc <<-EOS
        [概要: パンくずリストを表示します。<ec:searched_products>タグの中にこのタグを使用した場合、検索条件のカテゴリをパンくずリストにして表示します。検索結果のない場合はパンくずリストを表示しません。]
        [引数 category_code(準必須): カテゴリコード]
        [引数 separator(任意): パンくずリストを表示する際の区切り文字を設定します。]
        [戻り値: 詳細URLに登録したURLへのリンク。]
        [使用例: {{{<ec:product_crumbs_display />}}}]
      EOS
      def product_crumbs_display(*args, &block)
        category_code, separator, options = extract(args, :category_code => nil, :separator => ' > ')

        # 検索条件で使用されている場合、インスタンスのcategory_idを使用
        # カテゴリ検索ではないまたは検索結果がない場合かつ店舗側の設定もない場合はパンくずリストを表示しない
        product_condition = @template.instance_variable_get(:@condition)
        unless product_condition.nil? || product_condition.category_code.blank?
          category_code = product_condition.category_code
        end

        pankuzu_list = ProductCategory.get_pankuzu_list(category_code) unless category_code.nil?

        unless pankuzu_list.nil?
          links = []
          pankuzu_list.each do |pankuzu|
            if pankuzu.product_category.url.blank?
              links << pankuzu.name
            else
              url = pankuzu.product_category.url + '?category_code=' + pankuzu.product_category.code
              links << link_to(pankuzu.name, {:href => url, :escape => false})
            end
          end
          links.join(separator)
        end
      end

      desc <<-EOS
        [概要: 商品用パンくずリストを表示します。指定した商品に登録されているカテゴリを表示します。]
        [引数 display_number(必須): 商品管理で登録したカテゴリの何番目を表示させるかを指定します。]
        [引数 product_code(準必須): 商品コード]
        [引数 separator(任意): パンくずリストを表示する際の区切り文字を設定します。]
        [戻り値: カテゴリディスプレイに登録したURLへのリンク。]
        [使用例:
          {{{
          <ec:product_crumbs display_number=2/>
            →<a herf="/list?category_code="code13">衣料品</a> > <a herf="/list?category_code="code15">Tシャツ</a>
          }}}
        ]
      EOS
      def product_crumbs(*args, &block)
        display_number, product_code, separator, option = extract(args, :display_number=>nil, :separator => ' > ', :product_code=>nil )
        raise UsageError, "ec:product_crumbsは必ずdisplay_numberを指定してください" unless display_number
        category_id = product_info(product_code, "category_#{display_number}_id")
        return if category_id.blank?
        # idで指定されたカテゴリが見つからなくてもエラーにしない
        product_category = ProductCategory.find(:first, :conditions => ["id=?", category_id])
        product_crumbs_display(:category_code => product_category.code, :separator => separator) if product_category
      end

      desc <<-EOS
        [概要: 商品に関するお問い合わせへのリンクを表示します。]
        [引数 text(必須): リンクの文字列または画像。]
        [引数 product_code(準必須):  {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [戻り値: 詳細URLに登録したURLへのリンク。]
        [使用例:
           {{{
           <ec:button_to_product_inquiry> この商品について問い合わせる</ec:button_to_product_inquiry>
           }}}
        ]
      EOS
      def button_to_product_inquiry(*args, &block)
        product_code, options = extract(args, :product_code => "")
        option = args.shift || {}
        options.update(option)
        if product_code == ""
          product_code = product_info(product_code, 'product_code')
        end
        button = @template.button_to_url(@template.capture(&block), {:controller => :inquiry, :action => :index, :product_code => product_code}, options)
        @template.concat(button) if button && block_given?
      end

      desc <<-EOS
        [概要:カテゴリツリーリンクの繰り返し処理を出力します。]
        [引数 base_category_id(任意):指定したカテゴリの階層のみ表示したい場合に使用します。]
        [使用例:
          {{{
            <ec:product_category_tree>
              <ul>
              <ec:category_item>
                <li>
                  <ec:descendant_link/>
                </li>
              </ec:category_item>
              </ul>
            </ec:product_category_tree>
          　　⇒
            <ul>
              <li><a herf="xxx">カテゴリ１</a></li>
              <li><a herf="xxx">カテゴリ２</a></li>
            </ul>
          }}}
        ]
      EOS
      def product_category_tree(*args,&block)

        base_category_code, options = extract(args, :base_category_code => nil)
        @product_categories = ProductCategory.find(:all, :include =>'displays',
          :conditions => ['parent_id is null AND product_category_displays.locale = ? ', I18n.locale.to_s ], :order => 'display_order, product_categories.id')
        block_backup = @product_category_tree_block
        @product_category_tree_block = block
        begin
          _each_with_index(:@product_categories, :@product_category) do
            @template.concat(@template.capture(&block))
          end
        ensure
          @product_category_tree_block = block_backup
        end
      end

      desc <<-EOS
        [概要:カテゴリリンクの繰り返し処理を出力します。さらに子階層がある場合、子階層がなくなるまでcategory_tree_blockを実行します。]
        [使用例:
          {{{
            <ec:product_category_tree>
              <ul>
              <ec:category_item>
                <li>
                  <ec:descendant_link/>
                </li>
              </ec:category_item>
            </ec:product_category_tree>
          　　⇒
            <ul>
              <li><a herf="xxx">カテゴリ１</a></li>
              <li><a herf="xxx">カテゴリ２</a></li>
            </ul>

          }}}
        ]
        [使用例:カテゴリに子階層がある場合
          {{{
            <ec:product_category_tree>
              <ul>
              <ec:category_item>
                <li>
                  <ec:descendant_link>
                </li>
              </ec:category_item>
            </ec:product_category_tree>
          　　⇒
            <ul>
              <li><a herf="xxx"><div class="category1">カテゴリ１</div></a></li>
              <ul>
                <li><a herf="xxx"><div class="subcategory1">子カテゴリ１</div></a></li>
                <li><a herf="xxx"><div class="subcategory2">子カテゴリ２>/div></a></li>
                <li><a herf="xxx"><div class="subcategory3">子カテゴリ３</div></a></li>
              </ul>
              <li><a herf="xxx"><div class="category2">カテゴリ２</div></a></li>
            </ul>

          }}}
        ]
      EOS
      def category_item(*args, &block)
        @template.logger.debug("cateogory_item_block\n@product_category: " << @product_category.inspect)
        @category_item = @product_category
        raise UsageError, "category_item_blockはec:category_treeの中で使用してください" unless @category_item
        @template.instance_variable_set(:@category_item, @category_item)
        @template.concat(@template.capture(&block))
        # 子を取得、子がある場合に実行
        category_children = @category_item.children
        unless category_children.empty?
          category_children.each do |category_child|
            # 自分の言語のカテゴリが有る場合のみ表示させる
            category_child.displays.each do |display|
              if display.locale == I18n.locale.to_s
                @product_category = category_child
                @template.concat(@template.capture(&@product_category_tree_block))
              end
            end
          end
        end
        @category_items = nil
      end

      desc <<-EOS
        [概要:カテゴリリンクを表示します。リンクを押すとそのカテゴリで検索した商品検索ページに遷移します。]
        [引数:render_name(任意)商品検索結果表示後のページ名を設定します。設定がない場合はカテゴリ管理で登録した対象商品一覧ページに遷移するリンクを生成します。対象一覧ページを未登録の場合はカテゴリ名をテキストで表示します。]
        [引数:product_separate_value(任意)商品検索結果表示後の１ページ目に表示する商品数を指定します。何も指定しない場合は検索商品を10件表示します。]
        [引数:カテゴリリンクのhtmlオプションを設定できます。]
      EOS
      def descendant_link(*args)
        @category_item = @template.instance_variable_get(:@category_item)
        raise UsageError, "ec:descendant_linkはcategory_itemの中で使用してください" unless @category_item
        per_page, render_name, options = extract(args, :product_separate_value => 10, :render_name => @category_item.url)
        # 自分の言語のカテゴリを表示
        name = ""
        @category_item.displays.each do |display|
          name = display.name if (display.locale == I18n.locale.to_s)
        end

        return "" if name.blank?
        return "<div class=\"#{@category_item.code}\">#{name}</div>" if render_name.blank?

        return link_to("<div class=\"#{@category_item.code}\">#{name}</div>", @template.url_for({:controller => :cms, :action => render_name.gsub('/', ''), :category_code => @category_item.code, :per_page => per_page}), options)
      end

      desc <<-EOS
        [概要:{{{商品バリエーション1のセレクトボックスを表示します。選択したバリエーションのSKUが<ec:include_show_cart_in />に反映されます。}}}]
        [引数 product_code(準必須):  {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [使用例:{{{<ec:include_show_cart_in/>の使用例を参照}}}]
      EOS
      def product_variation_item_select_1(*args, &block)
        product_variation_item_select(1, args)
      end

      desc <<-EOS
        [概要:{{{商品バリエーション2のセレクトボックスを表示します。選択したバリエーションのSKUが<ec:include_show_cart_in />に反映されます。}}}]
        [引数 product_code(準必須):  {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [使用例:{{{<ec:include_show_cart_in/>の使用例を参照}}}]
      EOS
      def product_variation_item_select_2(*args, &block)
        product_variation_item_select(2, args)
      end

      desc <<-EOS
        [概要:{{{商品バリエーション3のセレクトボックスを表示します。選択したバリエーションのSKUが<ec:include_show_cart_in />に反映されます。}}}]
        [引数 product_code(準必須):  {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [使用例:{{{<ec:include_show_cart_in/>の使用例を参照}}}]
      EOS
      def product_variation_item_select_3(*args, &block)
        product_variation_item_select(3, args)
      end

      private
      # 商品バリエーションのセレクトボックスを表示します。
      def product_variation_item_select(index, args)
        product_code, options = extract(args, :product_code => "")
        variation = product_info(product_code, "variation_#{index}")
        product_code = product_info(product_code, "product_code")
        skus = product_info(product_code, "skus")
        variation_item = product_sku_info(product_code, "variation_item_#{index}")
        if variation && variation.is_a?(ProductVariation)
          variation_options = ProductVariationItem.options_by_ids(skus.map{|sku| sku.send("variation_item_#{index}_id") }.compact)
          html = @template.select_tag("variation_#{index}_item", @template.options_for_select(variation_options, variation_item.code), options)
          html << @template.observe_field("variation_#{index}_item", {
              :update => :ec_rider_show_cart_in,
              :on => 'change',
              :with => "'sku_code=#{product_code}-' + $('variation_1_item').value + '-' + $('variation_2_item').value + '-' + $('variation_3_item').value",
              :loading => 'showLoading()',
              :complete =>'hideLoading()',
              :url => { :controller => :common, :action => :show_cart_in }})
          return html
        else
          @template.hidden_field_tag("variation_#{index}_item", "00")
        end
      end

      public
      desc <<-EOS
        [概要:カートに入れる入力フォームを挿入します。「_show_cart_in」ページを挿入します。]
        [引数:なし]
        [戻り値:カートに入れるフォーム]
        [使用例:
           {{{
            <ec:product_variation_item_select_1 label="選択してください"/>
            <ec:product_variation_item_select_2 label="選択してください"/>
            <ec:product_variation_item_select_3 label="選択してください"/>
            <ec:include_show_cart_in />
           }}}
        ]
      EOS
      def include_show_cart_in
        '<div id="ec_rider_show_cart_in">'<<
        @template.render(:partial => 'show_cart_in')<<
        '</div>'
      end

      desc <<-EOS
        [概要:そのSKUがある場合、このタグで囲まれた領域を表示します。「_show_cart_in」ページで使用します。]
        [使用例:
           {{{
            <ec:if_sku_exist>
              <ec:cart_in_button/>
            </ec:if_sku_exist>
            →SKUがあればカートに入れるボタンを表示します。
           }}}
        ]
      EOS
      def if_sku_exist(*args, &block)
        sku_id = product_sku_info('', 'id')
        yield if !sku_id.blank? && block_given?
      end

      desc <<-EOS
        [概要:そのSKUが無い場合、このタグで囲まれた領域を表示します。「_show_cart_in」ページで使用します。]
        [使用例:
           {{{
            <ec:unless_sku_exist>
              この組み合わせの商品はありません。
            </ec:unless_sku_exist>
            →SKUが無い場合に「この組み合わせの商品はありません」を表示します。
           }}}
        ]
      EOS
      def unless_sku_exist(*args, &block)
        sku_id = product_sku_info('', 'id')
        yield if sku_id.blank? && block_given?
      end

      desc <<-EOS
        [概要:その商品の商品Q&Aがある場合、このタグで囲まれた領域を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [使用例:
          {{{
          <ec:if_product_faq_exists product_code="00000001">
            商品Q&A
          </ec:if_product_faq_exists>
          }}} → 商品コード｢00000001｣の商品Q&Aがある場合、｢商品Q&A｣を表示します。商品コード｢00000001｣の商品Q&Aがない場合、｢商品Q&A｣は表示されません。
        ]
      EOS
      def if_product_faq_exists(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_id = product_info(product_code, 'id')
        raise UsageError, product_code + "の商品コードの商品はありません。" unless product_id
        faq_count = ProductFaq.count(
          :joins => ["left outer join product_faq_relation_products on product_faq_relation_products.product_faq_id = product_faqs.id",
            "Left outer join product_faq_displays on product_faq_displays.product_faq_id = product_faqs.id"],
          :conditions => ["product_faq_displays.locale = ? AND product_faqs.invisible_flg = 0 AND product_faq_relation_products.product_id = ?", (I18n.locale || 'ja').to_s, product_id]
        )
        if faq_count > 0
          begin
            @template.concat(@template.capture(&block))
          ensure
          end
        end
      end

      desc <<-EOS
        [概要:その商品の商品Q&Aがある場合、このタグで囲まれた領域を繰り返し表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 display_qty(任意): {{{表示する最大件数を指定します。何も指定しない場合、商品Q&Aを全件数分繰り返し表示します。]
        [使用例:
          {{{
          <ec:each_product_faqs product_code="00000001" display_qty="4">
            質問者：<ec:product_faq_inquiry_user />様
          </ec:each_product_faqs>
          }}} → 商品コード｢00000001｣の商品Q&Aの質問者名を表示順に従い４つまで表示します。
        ]
      EOS
      def each_product_faqs(*args, &block)
        display_qty, product_code, options = extract(args, :display_qty => nil, :product_code => "")
        product_id = product_info(product_code, 'id')
        raise UsageError, product_code + "の商品コードの商品はありません。" unless product_id
        @product_faqs = ProductFaq.find(:all,
          :select => "product_faq_displays.inquiry_user, product_faq_displays.inquiry as faq_inquiry, product_faq_displays.answer_user, product_faq_displays.answer as faq_answer, product_faqs.answer_date",
          :joins => ["left outer join product_faq_relation_products on product_faq_relation_products.product_faq_id = product_faqs.id",
            "Left outer join product_faq_displays on product_faq_displays.product_faq_id = product_faqs.id"],
          :conditions => ["product_faq_displays.locale = ? AND product_faqs.invisible_flg = 0 AND product_faq_relation_products.product_id = ?", (I18n.locale || 'ja').to_s, product_id],
          :order => "display_order"
        )
        begin
          _each_with_index(:@product_faqs, :@product_faq, :@index) do
            if display_qty.nil? || display_qty.to_i > @index.to_i
              @template.concat(@template.capture(&block))
            else
              break
            end
          end
        ensure
          @product_faqs = nil
        end
      end

      desc <<-EOS
        [概要:{{{商品Q&Aの質問者を表示します。<ec:each_product_faqs>内で使用してください。}}}]
        [使用例:
          {{{
          <ec:each_product_faqs product_code="00000001">
            質問者：<ec:product_faq_inquiry_user />様
          </ec:each_product_faqs>
          }}} → 商品コード｢00000001｣の商品Q&Aの質問者名を繰り返し表示します。
        ]
      EOS
      def product_faq_inquiry_user
        raise UsageError, "ec:product_faq_inquiry_userはec:each_product_faqsの中で使用してください" unless @product_faq
        @product_faq.inquiry_user
      end

      desc <<-EOS
        [概要:{{{商品Q&Aのお問合せ内容を表示します。<ec:each_product_faqs>内で使用してください。}}}]
        [使用例:
          {{{
          <ec:each_product_faqs product_code="00000001">
            お問合せ内容：<ec:product_faq_inquiry />
          </ec:each_product_faqs>
          }}} → 商品コード｢00000001｣の商品Q&Aのお問合せ内容を繰り返し表示します。
        ]
      EOS
      def product_faq_inquiry
        raise UsageError, "ec:product_faq_inquiryはec:each_product_faqsの中で使用してください" unless @product_faq
        @template.add_br_tag(@product_faq.faq_inquiry)
      end

      desc <<-EOS
        [概要:{{{商品Q&Aの回答者名を表示します。<ec:each_product_faqs>内で使用してください。}}}]
        [使用例:
          {{{
          <ec:each_product_faqs product_code="00000001">
            商品問合せ担当者：<ec:product_faq_answer_user />
          </ec:each_product_faqs>
          }}} → 商品コード｢00000001｣の商品Q&Aの回答者名を繰り返し表示します。
        ]
      EOS
      def product_faq_answer_user
        raise UsageError, "ec:product_faq_answer_userはec:each_product_faqsの中で使用してください" unless @product_faq
        @product_faq.answer_user
      end

      desc <<-EOS
        [概要:{{{商品Q&Aの回答内容を表示します。<ec:each_product_faqs>内で使用してください。}}}]
        [使用例:
          {{{
          <ec:each_product_faqs product_code="00000001">
            回答内容：<ec:product_faq_answer />
          </ec:each_product_faqs>
          }}} → 商品コード｢00000001｣の商品Q&Aの回答内容を繰り返し表示します。
        ]
      EOS
      def product_faq_answer
        raise UsageError, "ec:product_faq_answerはec:each_product_faqsの中で使用してください" unless @product_faq
        @template.add_br_tag(@product_faq.faq_answer)
      end

      desc <<-EOS
        [概要:{{{商品Q&Aの回答日を表示します。<ec:each_product_faqs>内で使用してください。}}}]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y"、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [使用例:
          {{{
          <ec:each_product_faqs product_code="00000001">
            回答日：<ec:product_faq_answer_date format="%y/%m/%d %H:%M" />
          </ec:each_product_faqs>
          }}} → 商品コード｢00000001｣の商品Q&Aの回答日を繰り返し表示します。(例：2009/03/01 15:30)
        ]
        [使用例:
           {{{
           <ec:product_faq_answer_date format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def product_faq_answer_date(*args)
        raise UsageError, "ec:product_faq_answer_dateはec:each_product_faqsの中で使用してください" unless @product_faq
        format = extract(args, :format => '%y/%m/%d')
        @product_faq.answer_date.strftime( "#{format}" )
      end

      desc <<-EOS
        [概要:{{{パラメータで渡された商品コードを元にclassとして指定したDIVタグを表示します。}}}]
        [引数:HTMLオプションが使用できます。classを指定すると、パラメータに値がある場合でもスペースを区切り文字として追加します。]
        [使用例:
          {{{
          <ec:div_with_product_code class='styleA'>
            商品A
          </ec:div_with_product_code>
          }}} → <div class="00000500 styleA"> test </div>
        ]
      EOS
      def div_with_product_code(*args, &block)
        product_code, options = extract(args, :product_code => "")
        product_code = product_info(product_code, "product_code")
        classes = []
        classes << product_code if product_code
        classes << options["class"] if options["class"]
        @template.concat("<div class='#{classes.join(' ').strip}'>#{@template.capture(&block)}</div>")
      end

      desc <<-EOS
        [概要:{{{パラメータで渡されたカテゴリコードを元にclassとして指定したDIVタグを表示します。}}}]
        [引数:HTMLオプションが使用できます。classを指定すると、パラメータに値がある場合でもスペースを区切り文字として追加します。]
        [使用例:
          {{{
          <ec:div_with_category_code class='styleB'>
            商品A
          </ec:div_with_category_code>
          }}} → <div class="200 styleB"> test </div>
        ]
      EOS
      def div_with_category_code(*args, &block)
        class_options, options = extract(args, :class => "")
        classes = []
        classes << @template.send(:params)[:category_code] if @template.send(:params)[:category_code]
        classes << class_options if class_options
        @template.concat("<div class='#{classes.join(' ').strip}'>#{@template.capture(&block)}</div>")
      end

      desc <<-EOS
        [概要:{{{パラメータで渡された商品skuコードを元にclassとして指定したDIVタグを表示します。}}}]
        [引数:HTMLオプションが使用できます。classを指定すると、パラメータに値がある場合でもスペースを区切り文字として追加します。]
        [使用例:
          {{{
          <ec:div_with_sku_code class='styleC'>
            商品A
          </ec:div_wit_sku_code>
          }}} → <div class="00500 styleC"> test </div>
        ]
      EOS
      def div_with_sku_code(*args, &block)
        product_code, options = extract(args, product_code=>"")
        sku_code = product_sku_info(product_code, "sku_code")
        classes = []
        classes << sku_code if sku_code
        classes << options["class"] if options["class"]
        @template.concat("<div class='#{classes.join(' ').strip}'>#{@template.capture(&block)}</div>")
      end

      desc <<-EOS
        [概要:{{{商品の販売開始日を表示します。<ec:each_products>内で使用してください。}}}]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y"、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [使用例:
          {{{
          <ec:each_product product_code="00000001">
            販売開始日：<ec:product_sale_started_at format="%y/%m/%d %H:%M" />
          </ec:each_products>
          }}} → 商品コード｢00000001｣の商品販売開始日を繰り返し表示します。(例：2009/03/01 15:30)
        ]
        [使用例:
           {{{
           <ec:product_sale_started_at format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def product_sale_started_at(*args)
        raise UsageError, "ec:product_sale_started_atはec:each_productsの中で使用してください" unless @product
        format = extract(args, :format => '%y/%m/%d')
        @product.sale_started_at.strftime( "#{format}" )
      end

      desc <<-EOS
        [概要:{{{商品の販売終了日を表示します。<ec:each_products>内で使用してください。}}}]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y"、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [使用例:
          {{{
          <ec:each_product product_code="00000001">
            販売終了日：<ec:product_sale_finished_at format="%y/%m/%d %H:%M" />
          </ec:each_products>
          }}} → 商品コード｢00000001｣の商品販売終了日を繰り返し表示します。(例：2009/03/01 15:30)
        ]
        [使用例:
           {{{
           <ec:product_sale_finished_at format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def product_sale_finished_at(*args)
        raise UsageError, "ec:product_sale_finished_atはec:each_productsの中で使用してください" unless @product
        format = extract(args, :format => '%y/%m/%d')
        @product.sale_finished_at.strftime( "#{format}" )
      end

      #
      # =登録されている下記の条件に一致する商品データを取得します。=
      # SKUと商品表示情報が登録されているもの、
      # 非表示フラグの立っていないもの、
      # 公開期間内のもの、
      # プレゼント品でないもの
      #
      private
      def get_all_product
        products = @template.instance_variable_get("@products")
        if products.nil?
          condition_hash = {}
          condition_hash[:user_rank_id] = get_user_rank_id()
          condition_hash[:order] = @template.send(:params)[:order]
          condition = Product::CategorySearchForm.new(condition_hash)
          products = Product.find(:all, condition.to_find_options)
          @template.instance_variable_set("@products", products)
        end
      end

      #
      # =商品の指定した情報を返します。=
      #
      private
      def product_info(product_code, key)
        # 商品コードを取得する(product_code)
        # 優先順位
        #   1. each_product
        #   2. URLパラメータで渡されたproduct_code
        #   3. 引数で渡されたproduct_code
        product_code = @template.send(:params)[:product_code] if @template.send(:params)[:product_code]
        product_code = @template.instance_variable_get("@each_product").product_code if @template.instance_variable_get("@each_product")

        # 商品情報を取得する(product)
        # 優先順位
        #   1. product_group
        #   2. each_product
        #   3. キャッシュ
        #   4. fetched_products[product_code]
        #   5. DB
        product = @template.instance_variable_get("@product_group_sku").product if @template.instance_variable_get("@product_group_sku")
        if product.nil? && @template.instance_variable_get("@each_product")
          product ||= @template.instance_variable_get("@each_product") if @template.instance_variable_get("@each_product")
          product.current_display = product.product_displays.first
        end
        product ||= instance_variable_get_with_code('product', product_code)
        product ||= @fetched_products[product_code] if @fetched_products
        unless product
          if EcRider.config.ec_admin
            product = Product.find(:first, :conditions => ["product_code = ?", get_product_code(product_code)])
          else
            conditions = ['product_code = ?']
            conditions << '(publish_started_at is null or publish_started_at <= ?)'
            conditions << '(publish_finished_at is null or publish_finished_at >= ?)'
            conditions << '(invisible_flg is null or invisible_flg = 0)' unless EcRider.config.ec_test
            now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
            product = Product.find(:first, :conditions => [conditions.join(' AND '), get_product_code(product_code), now, now])
          end
          instance_variable_set_with_code('product', product_code, product)
        end
        product.nil? ? '' : product.send(key)
      end

      #
      # =商品SKUの指定した情報を返します。=
      # product_codeには、商品コード、または、SKUコードが渡されます。
      #
      private
      def product_sku_info(product_code, key)
        # 商品コード(SKUコード)を取得する(product_code)
        # 優先順位
        #   1. each_product
        #   2. URLパラメータで渡されたproduct_code
        #   3. 引数で渡されたproduct_code
        #   4. product_infoから取得
        product_code = @template.send(:params)[:product_code] if @template.send(:params)[:product_code]
        product_code = @template.instance_variable_get("@each_product").product_code if @template.instance_variable_get("@each_product")
        product_code = product_info(product_code, 'product_code') if product_code.blank?

        # 商品SKU情報を取得する(product_sku)
        # 優先順位
        #   1. each_product_sku
        #   2. product_group
        #   3. each_product
        #   4. キャッシュ(SKU)
        #   5. DB
        product_sku = @template.instance_variable_get("@each_product_sku") if @template.instance_variable_get("@each_product_sku")
        product_sku ||= @template.instance_variable_get("@product_group_sku") if @template.instance_variable_get("@product_group_sku")
        product_sku ||= @template.instance_variable_get("@each_product").primary_sku if @template.instance_variable_get("@each_product")
        product_sku ||= instance_variable_get_with_code('product_sku', product_code)
        unless product_sku
          product_sku = ProductSku.find(:first, :conditions => ["sku_code = ?", get_sku_code(product_code)])
          instance_variable_set_with_code('product_sku', product_code, product_sku)
        end
        product_sku.nil? ? '' : product_sku.send(key)
      end

      private
      def product_price_info(product_code, key)
        # 商品コードを取得する(product_code)
        # 優先順位
        #   1. each_product
        #   2. URLパラメータで渡されたproduct_code
        #   3. 引数で渡されたproduct_code
        #   4. product_infoから取得
        product_code = @template.send(:params)[:product_code] if @template.send(:params)[:product_code]
        product_code = @template.instance_variable_get("@each_product").product_code if @template.instance_variable_get("@each_product")
        product_code = product_info(product_code, 'product_code') if product_code.blank?

        # 商品SKU情報を取得する(product_sku)
        # 優先順位
        #   1. each_product_skus
        #   2. product_group
        #   3. each_products
        #   4. DB
        product_sku_id = @template.instance_variable_get("@each_product_sku").id if @template.instance_variable_get("@each_product_sku")
        product_price = nil
        unless product_sku_id
          # @product_group_sku、@each_productはeagerLoadingにより該当の会員ランクの価格だけが取得されているのでprices.firstで取得すればOK
          prices = @template.instance_variable_get("@product_group_sku").prices if @template.instance_variable_get("@product_group_sku")
          prices ||= @template.instance_variable_get("@each_product").primary_sku.prices if @template.instance_variable_get("@each_product")
          product_price = prices.first if prices
        end
        unless product_price
          product_sku_id ||= product_sku_info(product_code, 'id')
          user_rank_id = get_user_rank_id()
          product_price = ProductPrice.find(:first, :conditions => ["product_sku_id = ? AND user_rank_id = ?", product_sku_id, user_rank_id])
        end
        product_price.nil? ? 0 : product_price.send(key)
      end

      private
      def setting_pre_currency_and_post_currency(options)
        pre_currency = options["pre_currency"] ? options["pre_currency"] : ""
        post_currency = options["post_currency"] ? options["post_currency"] : ""

        return pre_currency, post_currency
      end

      private

      # 商品毎のキャンペーン適用を判断するのに使用します。
      # キャンペーンの計算処理はカートモデル内で処理されるため、
      # cartモデルに該当の商品を入れた状態とみなしてECタグ内で適用できるキャンペーンを取得しています。
      # キャンペーン適用後のカートオブジェクトを返却します。
      # ※実際のカートに商品は入りません
      def simulation_cart_for(product_sku_code)
        cart = Cart.new
        cart.user = @template.current_user if @template.logged_in?
        cart.add(product_sku_code, :nomal, 1)
        cart.calculate # これをしないと適用キャンペーンが取得できないため削除しないでください。
        cart
      end

      # ユーザーランクIDを取得します
      def get_user_rank_id
        user_rank_id = @template.instance_variable_get("@user_rank_id")
        unless user_rank_id
          user_rank_id = @template.current_user.user_rank_id if @template.logged_in?
          user_rank_id ||= UserRank.find(:first, :conditions => ["no_membership_flg = 1"]).id
          @template.instance_variable_set("@user_rank_id", user_rank_id)
        end
        return user_rank_id
      end
    end
  end
end
