# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module UserPointTag
      extend MethodDescription

      # TODO 現在１注文ごとにポイント付与・利用をしていますが、
      # １商品ごとにポイント利用、付与となるとこの処理を１商品ごとに表示するように変更する必要があります。

      desc <<-EOS
        [概要:ログインユーザーの保有ポイントを表示します。]
        [引数:なし]
        [戻り値:ログインユーザーの保有ポイント]
        [使用例:<ec:full_name/>]
      EOS
      def user_point_total_point
        UserPoint.total_point_for_user(@template.current_user.id) if @template.current_user
      end

      desc <<-EOS
        [概要:ログインユーザーのポイント有効期限を表示します。]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [戻り値:ログインユーザーのポイント有効期限]
        [使用例:
           {{{
           <ec:user_point_term_date format="%y/%m/%d %H:%M"/>
           ⇒ 2009/03/01 15:30
           }}}
        ]
        [使用例:
           {{{
           <ec:user_point_term_date format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def user_point_term_date(*args)
        format = extract(args, :format => '%y/%m/%d')
        term_date = UserPoint.term_date_for_user(@template.current_user.id) if @template.current_user
        unless term_date.nil?
          term_date.strftime("#{format}")
        end
      end

      desc <<-EOS
        [概要:ログインユーザーのポイント有効期限までの有効ポイントを表示します。]
        [引数:なし]
        [戻り値:ログインユーザーのポイント有効期限までの有効ポイント]
        [使用例:<ec:full_name/>]
      EOS
      def user_point_term_point
        if @template.current_user
          term_date = UserPoint.term_date_for_user(@template.current_user.id)
          point = UserPoint.point_to_term_date(@template.current_user.id, term_date)
          return point
        end
      end

      desc <<-EOS
        [概要:ログインユーザーのポイント履歴分繰り返しをします。]
        [使用例:
           {{{
           <ec:user_point_point_list>
             <ec:user_point_order_code />
             <ec:user_point_order_datetime />
             <ec:user_point_given_point />
             <ec:user_point_used_point />
           </ec:user_point_point_list>
           }}} ⇒　ポイントの注文コード・注文日・加算ポイント・利用ポイントを繰り返し表示します。
        ]
      EOS
      def user_point_point_list(*args, &block)
        @points = @template.instance_variable_get(:@points)
        if @points.nil?
          options = {
            :select => "*, sum(given_point)as order_given_point, sum(used_point) as order_used_point",
            :group => "order_id",
            :conditions => ["user_id =? ", @template.current_user.id],
            :order => "created_at desc"
            }
          @points = UserPoint.find(:all, options)
        end
        #結果が0行の場合空の行を表示する
        if @points.size == 0
          @point = 0
          @template.concat(@template.capture(&block))
        end
        _each_with_index(:@points, :@point) do
          @template.concat(@template.capture(&block))
        end
        ""
      end

      desc <<-EOS
        [概要：ポイントの注文Noを表示します。user_point_list内で使用できます。 ]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
      EOS
      def user_point_order_code(*args)
        raise UsageError, "point_order_codeはec:user_point_listの中で使用してください" unless @point
        empty_format = extract(args, :empty_format => '')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @point.order.ec_order_code
        end
      end

      desc <<-EOS
        [概要:注文日を出力します。]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [使用例:
           {{{
           <ec:point_order_datetime format="%y/%m/%d %H:%M"/>
           ⇒ 2009/03/01 15:30
           }}}
        ]
        [使用例:
           {{{
           <ec:point_order_datetime format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def user_point_order_datetime(*args)
        raise UsageError, "point_order_codeはec:user_point_listの中で使用してください" unless @point
        empty_format, format = extract(args, :empty_format => "", :format => '%y/%m/%d')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @point.order.ordered_at.strftime("#{format}")
        end
      end

      desc <<-EOS
        [概要：ポイントの注文金額を表示します。user_point_list内で使用できます。 ]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
        [使用例:
           {{{
           <ec:user_point_order_price empty_format="-"/>
           ⇒ -
           }}}
        ]
        [使用例:
           {{{
           <ec:user_point_order_price format="円"/>
           ⇒ 1,000円
           }}}
        ]
      EOS
      def user_point_order_price(*args)
        raise UsageError, "point_order_codeはec:user_point_listの中で使用してください" unless @point
        empty_format, format = extract(args, :empty_format => '', :format => '')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @template.fix_num_format(@point.order.total_info[:total]) + format
        end
      end

      desc <<-EOS
        [概要:注文商品の数だけ繰り返します。ec:user_point_list内で使用してください。]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
        [使用例:
           {{{
           <ec:user_point_list>
             <ec:user_point_order_product_list>
                AAA
             </ec:user_point_order_product_list>
           </ec:user_point_list>
           }}}
        ]
      EOS
      def user_point_order_product_list(*args,&block)
        raise UsageError, "ec:user_point_order_product_listはec:user_point_listの中で使用してください" unless @point
        empty_format = extract(args, :empty_format => '')
        if @point == 0
          @order_item = 0
          @template.concat(@template.capture(&block))
        elsif @point.order.nil?
          empty_format
        else
          order_shippings = @point.order.order_shippings
          shipping_order_items = []
          order_shippings.each do |order_shipping|
            shipping_order_items << order_shipping.order_items
          end
          @order_items = shipping_order_items.flatten
          _each_with_index(:@order_items, :@order_item, :@point_product_index) do
            @template.concat(@template.capture(&block))
          end
          ""
        end
      end

      desc <<-EOS
        [概要:注文商品の名前を表示します。ec:user_point_order_product_list内で使用してください。]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
        [戻り値:注文商品の名前。]
        [使用例:
           {{{
           <ec:order_shipping_product_name empty_format="-"/>
           }}}
        ]
      EOS
      def user_point_order_product_name(*args)
        raise UsageError, "ec:point_order_product_nameはec:user_point_order_product_listの中で使用してください" unless @order_item
        empty_format = extract(args, :empty_format => '')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @order_item.product_name
        end
      end

      desc <<-EOS
        [概要:注文時の加算ポイントを表示します。ec:user_point_list内で使用してください。]
        [引数:なし]
        [戻り値:注文時の加算ポイント。]
        [使用例:
           {{{
           <ec:user_point_given_point/>ポイント
           }}}
        ]
      EOS
      def user_point_given_point
        raise UsageError, "ec:user_point_given_pointはec:user_point_listの中で使用してください" unless @point
        @point.order_given_point unless @point == 0
      end

      desc <<-EOS
        [概要:注文時の利用ポイントを表示します。ec:user_point_list内で使用してください。]
        [引数:なし]
        [戻り値:注文時の利用ポイント。]
        [使用例:
           {{{
           <ec:user_point_used_point/>ポイント
           }}}
        ]
      EOS
      def user_point_used_point
        raise UsageError, "ec:user_point_given_pointはec:user_point_listの中で使用してください" unless @point
        @point.order_used_point unless @point == 0
      end

      desc <<-EOS
        [概要:ポイント一覧を複数ページで切り替える場合に、user_point_listの外側に設定します。]
        [引数 product_separate_value(任意):商品検索結果の１ページの表示件数を指定します。何も指定しない場合は10が適用されます。]
        [使用例:
          {{{
            <ec:user_point_list_area product_separate_value=20>
              <ec:user_point_pagination_link previous_label="前のページ" next_label="次のページ"/>
              <ec:user_point_point_list>
                <ec:user_point_order_code />
                <ec:user_point_order_datetime />
                <ec:user_point_given_point />
                <ec:user_point_used_point />
              </ec:user_point_point_list>
            </ec:user_point_list_area>
          }}}   →　１ページ２０件分のポイント一覧とページ遷移リンクを表示します。
        ]
      EOS
      def user_point_list_area(*args, &block)
        per_page, options = extract(args, :product_separate_value => 10)
          per_page = @template.send(:params)[:condition][:per_page] if @template.send(:params)[:condition]
          page = @template.send(:params)[:page]
        options = {
          :select => "*, sum(given_point)as order_given_point, sum(used_point) as order_used_point",
          :group => "order_id",
          :conditions => ["user_id =? ", @template.current_user.id],
          :order => "created_at desc",
          :page => page,
          :per_page => per_page
          }
        @template.instance_variable_set("@points", UserPoint.paginate(:all, options))
        @template.instance_variable_set("@condition_page", page)
        begin
          @template.concat(@template.capture(&block))
        ensure
          @template.instance_variable_set("@points", nil)
        end
      end

      desc <<-EOS
        [概要:ポイント一覧が複数ページある場合に、ページ遷移リンクを表示します。user_point_pagination_linkを使用する場合はuser_point_list_areaを使用してください。]
        [引数:previous_label(任意)前のページのリンクボタンの名前を指定できます。デフォルトは｢前へ｣になります。]
        [引数:next_label(任意)前のページのリンクボタンの名前を指定できます。デフォルトは｢次へ｣になります。]
        [使用例:
          {{{
            <ec:user_point_list_area>
              <ec:user_point_pagination_link previous_label="前のページ" next_label="次のページ"/>
            </ec:user_point_list_area>
          }}}   →　<a herf="">前のページ</a><a herf="">1</a><a herf="">2</a><a herf="">3</a><a herf="">次のページ</a>

        ]
      EOS
      def user_point_pagination_link(*args)
        points = @template.instance_variable_get(:@points)
        raise UsageError, "ec:user_point_pagination_linkはec:user_point_list_areaの中で使用してください" unless points
        next_label, previous_label, option = extract(args, :next_label =>'次へ' , :previous_label => '前へ')
        # lang[]のパラメータが入ってしまいCMSコントローラでエラーになってしまうのでパラメータを書き換え
        # 言語対応に支障が出るようであれば別方法を検討する必要があります。
        @template.will_paginate(points, :previous_label => previous_label, :next_label => next_label, :params=>{:lang=>nil})
      end

      desc <<-EOS
        [概要:ポイント一覧の表示件数を切り替えるセレクトボックスを表示します。使用する場合はec:user_point_list_areaの中で記述します。]
        [引数:render_name(必須)商品検索結果表示後のページ名を設定します。]
        [引数:select_count(任意)セレクトボックスで選ぶ表示件数の件数を｢○,○,○｣形式で設定します。何も指定しなかった場合、10,20,30を選択肢にします。]
        [使用例:
          {{{
            <ec:user_point_list_area>
              <ec:product_pagination_count_per_page/>
              <ec:product_pagination_count_per_page select_count="20,50,100"/>
            </ec:user_point_list_area>
          }}}
        ]
      EOS
      def user_point_pagination_count_per_page(*args)
        raise UsageError, "ec:user_point_pagination_count_per_pageはec:user_point_list_areaの中で使用してください" unless @template.instance_variable_get(:@points)
        select_count, options = extract(args, :select_count => "10, 20, 30")
        @template.send(:params)[:condition][:per_page] = @template.send(:params)[:condition][:per_page].to_i if @template.send(:params)[:condition]
        
        @template.form_for(:condition, :url => {:controller =>'member', :action => 'point_history'},:html => {:method => :get, :id => 'count_per_page_form'}) do |f|
          @template.concat(f.select(:per_page,select_count.split(/\s*,\s*/),{}, {:onChange => "this.form.submit();"}))
        end
        ""
      end

      desc <<-EOS
        [概要:ポイント一覧を表示する時にポイントの全件数を表示します。使用する場合はec:user_point_list_areaの中で記述します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:user_point_list_area>
              <ec:user_point_pagination_total_count/>件中
              <ec:user_point_pagination_start_index/>～<ec:user_point_pagination_last_index/>件を表示しています。
            </ec:user_point_list_area>
          }}}
        ]
      EOS
      def user_point_pagination_total_count
        points = @template.instance_variable_get(:@points)
        raise UsageError, "ec:user_point_pagination_total_countはec:user_point_list_areaの中で使用してください" unless points
        points.total_entries
      end

      desc <<-EOS
        [概要:ポイント一覧を表示する時に商品の表示開始数を表示します。使用する場合はec:user_point_list_areaの中で記述します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:user_point_list_area>
              <ec:user_point_pagination_total_count/>件中
              <ec:user_point_pagination_start_index/>～<ec:user_point_pagination_last_index/>件を表示しています。
            </ec:user_point_list_area>
          }}}
        ]
      EOS
      def user_point_pagination_start_index
        points = @template.instance_variable_get(:@points)
        raise UsageError, "ec:user_point_pagination_start_indexはec:user_point_list_areaの中で使用してください" unless points
        points.offset + 1
      end

      desc <<-EOS
        [概要:ポイント一覧を表示する時にポイントの表示終了数を表示します。使用する場合はec:user_point_list_areaの中で記述します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:user_point_list_area>
              <ec:user_point_pagination_total_count/>件中
              <ec:user_point_pagination_start_index/>～<ec:user_point_pagination_last_index/>件を表示しています。
            </ec:user_point_list_area>
          }}}
        ]
      EOS
      def user_point_pagination_last_index
        points = @template.instance_variable_get(:@points)
        raise UsageError, "ec:user_point_pagination_last_indexはec:user_point_list_areaの中で使用してください" unless points
        points.offset + points.length
      end
    end
  end
end
