# -*- coding: utf-8 -*-
class Banner < ActiveRecord::Base
  has_many :displays, :class_name => "BannerDisplay", :foreign_key => 'banner_id', :dependent => :destroy
  accepts_nested_attributes_for :displays, :allow_destroy => true

  has_many :conditions, :class_name => "BannerCondition", :foreign_key => 'banner_id', :dependent => :destroy
  accepts_nested_attributes_for :conditions, :allow_destroy => true

  before_validation :update_or_delete_displays

  selectable_attr :type_cd do
    entry '1', :image, '画像'
    entry '2', :text, 'テキスト'
  end

  validates_presence_of :code
  validates_presence_of :type_cd
  validates_numericality_of :display_order, :allow_nil => true

  def setup_displays_for(locales = I18n.supported_locales)
    return if locales.nil? || locales.empty?
    supported_locales = I18n.supported_locales.map(&:to_s)
    displays_locales = displays.map(&:locale)
    (supported_locales - displays_locales).each do |locale|
      displays.build(:locale => locale.to_s)
    end
  end

  def setup_condition_at_least_1
    return unless self.conditions.empty?
    conditions.build
  end

  def display_for(locale)
    displays.detect{|display| display.locale == locale.to_s}
  end

  def update_or_delete_displays
    deleting_displays = []
    self.displays.each do |display|
      display.type_key = type_key
      deleting_displays << display if display.useless?
    end
    self.displays.delete(*deleting_displays)
  end

  class << self
    def all_codes
      connection.select_values("select code from banners group by code order by code")
    end
  end


  class SearchForm < EcRider::ConditionForm
    include SelectableAttr::Base

    attr_accessor :code # バナーコード
    multi_selectable_attr :type_cds, :enum => Banner.type_enum # バナータイプ: 画像、テキスト（チェックボックス）両方チェック
    attr_accessor :publish_started_at, :publish_finished_at # 公開期間
    attr_accessor :link_url # リンク先URL

    def to_find_options(options = nil)
      @where, @params = [], []
      build_code
      build_publish_started_at
      build_publish_finished_at
      build_link_url
      result = {
        :order => "code asc, type_cd asc, created_at asc"
      }
      @params.unshift(@where.join(' AND ')) unless @where.empty?
      result[:conditions] = @params unless @params.empty?
      result
    end

    private

    def build_code
      return if code.blank?
      @where << "code like ?"
      @params << "%#{code}%"
    end

    def build_publish_started_at
      return if publish_started_at.blank?
      @where << "publish_started_at >= ?"
      @params << date_to_time(publish_started_at, 0, 0, 0)
    end

    def build_publish_finished_at
      return if publish_finished_at.blank?
      @where << "publish_finished_at <= ?"
      @params << date_to_time(publish_finished_at, 23, 59, 59)
    end

    def build_link_url
      return if link_url.blank?
      @where << "link_url like ?"
      @params << "%#{link_url}%"
    end

    def date_to_time(value, offset_h, offset_m, offset_s)
      case value
      when Time then
        value
      when Date then
        Time.new(value.year, value.month, value.day, offset_h, offset_m, offset_s)
      else
        Time.parse('%s %02d:%02d:%02d' % [value.to_s, offset_h, offset_m, offset_s])
      end
    end

  end

  attr_writer :weight_point # 計算保持用 重みポイント
  def weight_point
    @weight_point || 0
  end
  
  # バナーコードからバナーグループを取得し、並び順優先順位に従い並び替え、返却します。
  # 並び順優先順位
  # 条件重みポイント合計の多い順 > 表示優先度の数字大きい順 > 公開開始日時降順 > ID降順 > ランダム
  def self.banner_group(bunner_code, user, display_qty)
    # バナー取得
    # テスト環境ではテスト対象日で表示するバナーを検索します。
    # 管理側プレビュー画面では日付を見ません。
    today = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at.strftime('%Y-%m-%d %H:%M:%S') : Time.now.strftime('%Y-%m-%d %H:%M:%S')
    conditions = ["code = ?", bunner_code]
    unless EcRider.config.ec_admin
      conditions[0] << " AND publish_started_at <= ? AND publish_finished_at >= ? "
      conditions << today
      conditions << today
    end
    
    finded_banners = Banner.find(:all, :conditions => conditions)
    banners = []
    random_banners = []
    # 重みポイントを計算
    finded_banners.each do |banner|
      if banner.conditions_in_random?
        random_banners << banner
      else
        banner.weight_point = banner.calc_total_weight_point(user)
        banners << banner
      end
    end

    # ランダム以外をソート
    banners.sort! do |banner, banner_next|
      if banner.weight_point == banner_next.weight_point
        if banner.display_order == banner_next.display_order
          if banner.publish_started_at == banner_next.publish_started_at
            banner_next.id <=> banner.id
          else
            banner.publish_started_at <=> banner_next.publish_started_at
          end
        else
          banner.display_order = banner.display_order || 0
          banner_next.display_order = banner_next.display_order || 0
          banner_next.display_order <=> banner.display_order
        end
      else
        banner_next.weight_point <=> banner.weight_point
      end
    end
    #
    # 表示個数制御
    if banners.size > display_qty.to_i
      # 表示個数まで削除
      delete_qty = banners.size - display_qty.to_i
      delete_qty.times{banners.pop}
    else
      # 表示個数までランダムバナー追加
      unless random_banners.empty?
        add_qty = display_qty.to_i - banners.size
        add_qty.times do
          random_banner = random_banners.delete_at(rand(random_banners.length))
          banners << random_banner unless random_banner.nil?
        end
      end
    end
    return banners
  end

  # そのバナーの重みポイント合計を返却する
  def calc_total_weight_point(user)
    total_weight_point = 0
    self.conditions.each do |condition|
      total_weight_point += condition.calc_weight_point(user)
    end
    return total_weight_point
  end

  # 条件にランダムが含まれる場合trueを返却する。
  def conditions_in_random?
    cond_in_random = self.conditions.select{|cond| cond.type_key == :random}
    return !cond_in_random.empty?
  end
end
