# -*- coding: utf-8 -*-
class CampaignCondition < ActiveRecord::Base
  versioned
  
  include SafeEval

  belongs_to :campaign

  # 削除を指示するためのフラグ
  attr_accessor :deleting

  validates_presence_of :no
  validates_presence_of :type_cd
  validates_presence_of :operator_cd
  # validates_presence_of :value # valueはnull可

  # 演算子
  # 比較演算子については、以下を参照してください
  #   http://datamapper.rubyforge.org/dm-core/
  selectable_attr :operator_cd do
    entry '01', :lt  , 'より小さい', :ruby => "<" #未使用
    entry '02', :gt  , 'より大きい', :ruby => ">" #未使用
    entry '03', :lte , '以下'     , :ruby => "<="
    entry '04', :gte , '以上'     , :ruby => ">="
    entry '05', :eql , '等しい'   , :ruby => "=="
    entry '06', :not , '以外'     , :ruby => "!=" #未使用
    entry '07', :like, 'を含む'   , :ruby => "=~"
    entry '08', :not_like, 'を含まない'   , :ruby => "!~" #未使用
  end

  # 項目種
  selectable_attr :type_cd do
    entry '01', :user_gender, '会員性別', :accessor => 'user.sex', :func => "'%s'", :operator => :eql, :input => :gender
    entry '02', :user_country, '会員住所（国）', :accessor => 'user.address_country_code', :func => "'%s'", :operator => :eql, :input => :country
    entry '03', :user_address, '会員住所', :accessor => "user.address_for_campaign_condition", :func => "Regexp.new(Regexp.escape('%s'))", :operator => :like, :input => :text
    entry '04', :user_address_isolated, '会員住所（離島）', :accessor => "user.address_isolated?", :func => "('%s'.to_i==(1))", :operator => :eql, :input => :boolean
    entry '05', :user_age, '会員年齢', :accessor => 'user.age(campaign_attached_at)', :func => "CampaignCondition.parse_age('%s')", :operator => :compare, :input => :text

    with_options(:operator => :compare) do |s|
      s.entry '07', :user_birthday        , '会員誕生日(年月日)', :accessor => 'user.birthday'         , :func => "CampaignCondition.parse_date('%s')", :input => :text,
        :today_parse => '%Y/%m/%d', :suffix => "例： 2010/01/10 または today"
      s.entry '08', :user_birthday_of_year, '会員誕生日(月日)'  , :accessor => 'user.birthday_of_year' , :func => "CampaignCondition.parse_date_of_year('%s')", :input => :text,
        :today_parse => '%m/%d', :suffix => "例： 03/15 または today"
      s.entry '09', :user_birth_year      , '会員誕生日(年)'    , :accessor => 'user.birth_year.to_i'  , :func => "CampaignCondition.parse_year('%s')", :input => :year,
        :today_parse => '%Y'
      s.entry '10', :user_birth_month     , '会員誕生日(月)'    , :accessor => 'user.birth_month.to_i' , :func => "CampaignCondition.parse_month('%s')", :input => :month,
        :today_parse => '%m'
      s.entry '11', :user_birth_day       , '会員誕生日(日)'    , :accessor => 'user.birth_day.to_i'   , :func => "CampaignCondition.parse_day_of_month('%s')", :input => :day,
        :today_parse => '%d'
    end
    
    entry '22', :member_category_div, '会員種別', :accessor => 'user.member_category_div', :func => "'%s'", :operator => :eql, :input => :member_category_div

    entry '19', :user_rank, '会員ランク', :accessor => 'user.user_rank_id', :func => "'%s'.to_i", :operator => :eql, :input => :user_rank
    
    with_options(:operator => :compare) do |s|
      s.entry '21', :cart_product_subtotal_price, '小計額', :accessor => 'cart_or_item.subtotal_amount.to_i', :func => "'%s'.to_i", :input => :text
      s.entry '23', :cart_product_total_price,    '合計額', :accessor => 'cart_or_item.total_amount.to_i', :func => "'%s'.to_i", :input => :text
    end

  end

  # Campaignで結合してしようするための接続演算子付きのスクリプトを生成します。
  def build
    result = (no > 0) ? '&& ' : ''
    result << build_script
    result
  end


  # 評価用のRubyスクリプトを生成します
  def build_script
    errors.clear
    unless validate_to_build_script
      raise ActiveRecord::RecordInvalid.new(self)
    end
    func = type_entry[:func]
    value_script = func.respond_to?(:call) ? func.call(type_entry, self, value_replaced_today) : func.to_s % value_replaced_today
    "%s %s %s" % [
      type_entry[:accessor], operator_entry[:ruby], value_script
    ]
  end

  # 日付の値が'today'の場合に値をキャンペーン適用日に置き換えて返します
  def value_replaced_today
    parse_format = type_entry[:today_parse]
    if parse_format.nil?
      return value
    elsif value !~ /today/i
      return value
    else
      return campaign.attached_at.strftime(parse_format)
    end
  end

  # build_scriptを実行可能かどうかを判断し、できない場合はerrorsにメッセージを追加します。
  def validate_to_build_script
    errors.add_on_blank(:type_cd)
    errors.add_on_blank(:operator_cd)
    errors.empty?
  end

  # 与えられたコンテキストでこの条件を評価します
  # 余計なアクセスなどを行わないように、スレッドの$SAFE(セーフレベル)を指定して、安全に実行できるようにしています。
  # スレッドは平行処理するためではないので、このメソッドはスレッドを実行して終了を待って戻り値を返します。
  def match?(context = {})
    !!safe_instance_eval(build_script, context, 3)
  end

  def self.parse_date(str)
    Date.parse(str)
  end

  def self.parse_date_of_year(str)
    # 2000年は閏年
    d = Date.new(2000, *str.split('/')[-2..-1].map(&:to_i))
    d.strftime('%m/%d')
  end

  def self.parse_year(str)
    str.to_i
  end

  def self.parse_month(str)
    str.to_i
  end

  def self.parse_day_of_month(str)
    str.to_i
  end

  week_days = {
    1 => %w(月 月曜日 Mon Monday),
    2 => %w(火 火曜日 Tue Tuesday),
    3 => %w(水 水曜日 Wed Wednesday),
    4 => %w(木 木曜日 Thu Thursday),
    5 => %w(金 金曜日 Fri Friday),
    6 => %w(土 土曜日 Sat Saturday),
    7 => %w(日 日曜日 Sun Sunday)
  }
  WEEK_DAYS = {}
  week_days.each do |num, keys|
    keys.each{|key| WEEK_DAYS[key] = num}
  end
  WEEK_DAYS.freeze

  def self.parse_day_of_week(str)
    WEEK_DAYS[str]
  end

  def self.parse_time_of_day(str)
    str = str.split(/\s/).last
    raise ArgumentError, "Invalid time format #{str}" unless str =~ /\d{1,2}:\d{1,2}/
    d = Time.local(2000, 1, 1, *str.split(':')[0..1].map(&:to_i))
    d.strftime('%H:%M')
  end

  def self.parse_age(str)
    num_str = (str || '').gsub(/\D/, '')
    raise ArgumentError, "Invalid age #{str}" if num_str.blank?
    num_str.to_i
  end

  # 項目種別から表示する区分値名を表示する
  # 例) 項目が｢会員種別｣の場合項目名称区分から｢会員種別名称｣を取得して表示
  def value_for_type
    case self.type_entry[:input]
    when :gender
      User.sex_name_by_id(value)
    when :country
      country = Country.find(:first, :select => "name", :conditions => ["code = ?", value])
      country ? country.name : ""
    when :boolean
      value == "1" ? "はい" : "いいえ"
    when :member_category_div
      User.member_category_div_name_by_id(value)
    when :user_rank
      user_rank = UserRank.find(:first, :select => "name", :conditions => ["id = ?", value])
      user_rank ? user_rank.name : ""
    else
      value
    end
  end
end
