# -*- coding: utf-8 -*-
#
# キャンペーンの効果
class CampaignEffect < ActiveRecord::Base
  versioned

  belongs_to :campaign

  belongs_to :present_product_sku, :class_name => 'ProductSku', :foreign_key => 'present_product_sku_id' 

  validates_presence_of :type_cd
  validates_presence_of :operator_cd
  
  validates_presence_of :present_product_sku_id, :if => Proc.new{|r| r.type_key == :present}, 
    :message => 'を選択してください。'

  # 演算子
  selectable_attr :operator_cd do
    entry('01', :assign    , '代入'){ def calc(val1, val2); val2; end }
    entry('02', :add       , '加算'){ def calc(val1, val2); val1 + val2; end }
    entry('03', :substract , '減算'){ def calc(val1, val2); val1 - val2; end } #未使用
    entry('04', :multiply  , '乗算'){ def calc(val1, val2); val1 * val2; end }
    entry('05', :devide    , '除算'){ def calc(val1, val2); val1 / val2; end } #未使用
  end

  # 項目種
  selectable_attr :type_cd do 
    entry '01', :discount_percentage, '割引率', :group => :discount, :operator => :assign, :suffix => "% 割引" do
      def apply(cart_or_item, effect)
        value = cart_or_item.subtotal_amount_for_discount * effect.value.to_i / 100
        result = effect.operator_entry.calc(cart_or_item.discount_amount, value)
        assign_if_prior_or_valuable(effect, cart_or_item, :discount_amount, result)
      end
    end
    entry '02', :discount_price     , '割引金額', :group => :discount, :operator => :assign, :suffix => "円 割引" do
      def apply(cart_or_item, effect)
        # 割引対象金額が0(==明細すべてが割引済みの場合は総合計の割引は行わない)
        return if cart_or_item.subtotal_amount_for_discount.to_i == 0
        value = effect.value.to_i
        result = effect.operator_entry.calc(cart_or_item.discount_amount, value)
        assign_if_prior_or_valuable(effect, cart_or_item, :discount_amount, result)
      end
    end

    entry '03', :point              , 'ポイント', :operator => :add_or_multiply do
      def apply(cart_or_item, effect)
        value  =  effect.value.to_i
        result = effect.operator_entry.calc(cart_or_item.point, value)
        assign_if_prior_or_valuable(effect, cart_or_item, :given_point, result)
      end
    end

    entry '04', :shipping_cost      , '送料'    , :operator => :assign, :suffix => "送料無料" do
      def apply(cart_or_item, effect)
        cart_or_item.shipping_cost_effect_id = effect.id
        remember_effect(effect, cart_or_item)
      end
    end

    entry '05', :present            , 'プレゼント', :operator => :assign do
      def apply(cart_or_item, effect)
        cart = cart_or_item.is_a?(Cart) ? cart_or_item : cart_or_item.cart
        return unless cart
        product_sku = ProductSku.find(effect.present_product_sku_id)
        cart.add(product_sku, :present)
        remember_effect(effect, cart_or_item)
      end
    end
  end

  # type_cd の 各entryオブジェクトに追加されるメソッドを定義するモジュールです。
  module TypeEntryMetods
    # 割引率、割引金額を一つの明細／カートなどに重複させないように、これらを一つのグループに
    # 属するものとして、entryの引数:groupでグルーピングします。:groupが定義されていない場合
    # entry#keyが使用されます。
    #
    # これにより適用されるキャンペーン効果は、各グループ毎にカート全体／明細につき一つとなります。
    def group
      self[:group] || self.key
    end

    def remember_effect(effect, cart_or_item)
      cart_or_item.remember_effect(effect)
    end

    def remove_last_effect(cart_or_item, effect)
      if idx = cart_or_item.applied_effects.rindex(effect)
        cart_or_item.applied_effects.delete_at(idx)
      end
    end

    def assign_and_remember_effect(effect, cart_or_item, attr, result)
      cart_or_item.send("#{attr}=", result)
      remember_effect(effect, cart_or_item)
    end

    # cart_or_itemについて、このentryと同じgroupに属する最後に適用されたeffectを返します。
    def last_effect_of_group(cart_or_item)
      effects = cart_or_item.applied_effects
      idx = effects.rindex{|effect| effect.type_entry.group == self.group}
      idx ? effects[idx] : nil
    end

    # キャンペーン効果をセットします
    def assign_if_prior_or_valuable(effect, cart_or_item, attr, result)
      # プレゼント付与キャンペーンは複数付与できる
      unless self.group == :present
        if last_effect = last_effect_of_group(cart_or_item)
          # 同じ効果で優先度の高いものが適用済だった場合は何もしない
          return if effect.campaign.priority <= last_effect.campaign.priority
        end
      end
      # キャンペーン効果を適用する
      assign_and_remember_effect(effect, cart_or_item, attr, result)
    end
  end

  # 項目種の全てのエントリにTypeEntryMetodsをextendさせています。
  type_entries.each{|entry| entry.extend(TypeEntryMetods)}

  TYPE_KEYS_EXCEPT_POINT = self.type_keys - [:point]

  def validate
    if [:discount_percentage, :shipping_cost, :point, :discount_price].include?(type_key)
      if value.blank?
        errors.add(:value, "は、#{type_name}を選択した場合、必ず入力してください。")
      elsif /^\d+$/ !~ value
        errors.add(:value, "は、#{type_name}を選択した場合、数値を入力してください。")
      end
    end
  end

  def targets_cart?
    campaign ? campaign.targets_cart? : true
  end

  def targets_items?
    !targets_cart?
  end

  # プレゼントなのに在庫切れ？
  def present_but_no_stock?(cart = nil)
    return false unless type_key == :present
    sku = self.present_product_sku
    # 引き当て済み在庫数を取得する
    secured_stock_count = cart ? cart.secured_stock_count_by_sku_id(sku.id) : 0
    (sku.orderable_count + secured_stock_count) < 1 # 在庫切れ
  end

  # 渡されたカートに対してキャンペーン効果を適用します
  def apply(cart)
    if targets_cart?
      type_entry.apply(cart, self)
    else
      apply_to_items(cart)
    end
  end

  # 渡されたカートの各明細に対してキャンペーン効果を適用します
  def apply_to_items(cart)
    return unless campaign
    target_product_sku_ids = campaign.target_sku_ids
    cart.each do |cart_item|
      if target_product_sku_ids.include?(cart_item.product_sku_id)
        if self.campaign.match_to_conditions?(cart_item.to_campaign_context)
          type_entry.apply(cart_item, self)
        end
      end
    end
  end

  # CampaignEffectの適用対象となるものに共通のメソッドを集めたモジュールです。
  # 具体的には、Cart および Cart::Item にincludeされています。
  module Targetable
    include ModelSerializable

    # 実際に適用されたキャンペーン効果
    # ただし、計算中は優先順位などの計算により追加されたものが削除されることもあります。
    attr_deserializable_array(:applied_effects, CampaignEffect, :applied_effect_ids)
    
    # 実際に適用されたキャンペーン
    # applied_effectsから求めています。
    def applied_campaigns
      return [] if applied_effects.nil? 
      applied_effects.map{|effect| effect.campaign}
    end

    # 適用されたキャンペーン効果として記録します
    def remember_effect(effect)
      applied_effects << effect
    end
    
    # 適用されたキャンペーン効果の記録をクリアします
    def clear_effects
      applied_effects.clear
    end

    # キャンペーンが適用されたかどうかを判断します。
    def effect_applied?(&block)
      block ? 
        applied_effects.any?(&block) :
        !applied_effects.empty?
    end
    
  end

end
