# -*- coding: utf-8 -*-
#
# ショッピングカート
#
#                               |   Cart                       |  Cart::Item      
# ------------------------------|------------------------------|----------------------------
# unit_price                    |  N/A                         |  単価 (商品SKUの定価とは異なる場合もあり)
# qty                           |  N/A                         |  数量
# unit_price_x_qty              |  N/A                         |  単価 x 数量
# subtotal_amount               |  小計額                      |  == 単価 x 数量
#                               |  明細のtotal_amountの合計    |  == unit_price_x_qty
# subtotal_amount_for_discount  |  割引されていない明細の      |  == subtotal_amount
#                               |        total_amountの合計    | 
# discount_amount               |  カート全体に対する割引額    |  明細ごとに設定される割引額
# total_amount                  |  カートの総合計              |  明細ごとの合計
# given_point                   |  獲得ポイントの合計          |  明細ごとに設定される獲得ポイント
# 
# N/A は "Not Available"の略で「使えません」という意味です。
#
class Cart
  include CampaignEffect::Targetable

  def self.deprecate_alias_method(deprecated, source)
    alias_method deprecated, source
    deprecate deprecated => source
  end

  attr_reader :items
  deprecate_alias_method :products, :items

  def user
    @user ||= User.find(@user_id) if @user_id
    @user ||= User.new
    @user  
  end  
  attr_writer :user

  # subtotal_amount, discount_amount, total_amount, given_point はCart::Itemと名前を合わせてあります。
  # これらの名前はCampaignEffectで使用されていますので、注意が必要です。
  # これらの値は書き込み不可です。calculateメソッドで算出します。
  attr_reader :subtotal_amount # 小計(明細の合計)
  attr_reader :discount_amount # 割引
  attr_writer :discount_amount # 割引。このメソッドを使用するのはCampaignEffectあるいはテストのみです。
  attr_writer :given_point     # 獲得ポイント。このメソッドを使用するのはCampaignEffectあるいはテストのみです。
  # attr_reader :total_amount    # 総合計

  attr_reader :subtotal_amount_for_discount # 割引適用対象の小計(割引が設定されていない明細の合計)

  # 計算時にいつの時点のキャンペーンを適用するのかを指定可能です。
  # 指定されてなかった場合は現時刻が使用されます。
  attr_writer :campaign_attached_at
  def campaign_attached_at
    time_now = (EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now)
    @campaign_attached_at || time_now
  end

  # 引き当て済みの在庫情報
  # 注文編集時に、引き当て済み在庫があればセットします。
  attr_accessor :secured_stocks

  # 送料に関するキャンペーン効果のid
  attr_accessor :shipping_cost_effect_id
  
  # コンストラクタ
  def initialize
    clear
  end

  def clear
    @item_list = [] #カートに入っている商品をカートに表示する順番にソートしたもの
    @items = {}     #カートに入っている商品のハッシュ
    clear_amounts
  end
  
  def user_rank
    @user_rank ||= (user ? user.rank : nil)
    @user_rank
  end
  attr_writer :user_rank

  #
  # = カートにある商品をカートに追加した時刻の降順で並べます。 =
  #
  def item_list(reversing = true)
    # item.cartはキャンペーンの計算で使用するため、カートにitemをセットする時点で入れているが、
    # セッションから取り出したカートの場合、item.cartがnilになっていることがあるためここでもセットする
    @item_list.each do |item|
      item.cart ||= self
    end
    reversing ? @item_list.reverse : @item_list
  end

  #
  # = カート内の商品の並び順をプレゼントはまとめて最後列に並び替えした配列を返すメソッド =
  # ※@item_list内の並び順は変更されません。
  def item_list_order_by_present
    # プレゼント品を後ろに追加するため並べ替え
    result = []
    present_list = []
    each do |item|
      if item.product_sku.product.present_flg
        present_list << item
      else
        result << item
      end
    end
    result.concat(present_list)
    return result
  end
  deprecate_alias_method :product_list, :item_list
  deprecate_alias_method :normal_products, :item_list

  def each(&block)
    item_list.each(&block)
  end

  # カート明細が空の場合はtrue
  def empty?
    @items.empty?
  end

  #
  # = カートの総合計 =
  # 総合計 = 小計(明細のtotal_amountの合計) - 割引
  def total_amount
    subtotal_amount.to_i - discount_amount.to_i
  end
  deprecate_alias_method :total_price, :total_amount
  
  #
  # = 通常送付の総額 =
  # 送料は含みません
  def normal_deli_total_price
    item_list.sum(&:subtotal_amount)
  end

  #
  # = 総数量、通常送付の数量を返します。=
  def total_qty
    item_list.sum(&:qty)
  end
  deprecate_alias_method :normal_deli_qty, :total_qty

  #
  # カートに対するキャンペーン適用前のポイントを返します
  # 明細に対してポイントキャンペーンが適用されていれば明細ごとのポイントの総計を、
  # 明細に対してポイントキャンペーンが適用されていなければカート合計金額から算出
  #
  def point
    point = 0
    item_campaign_flg = false
    item_list.each do |item|
      point += item.given_point
      item_campaign_flg = (item_campaign_flg || item.assign_point_campaign?)
    end
    if !item_campaign_flg && total_amount > 0 && point_setting_available?
      point = total_amount / point_setting_issue_price
    end   
    point
  end

  # カートのポイントを返します（キャンペーン反映済み）
  def given_point
    @given_point ? @given_point : point
  end
  alias_method :total_given_point, :given_point

  # 総送料ポイントを返します
  def total_shipping_cost_point
    item_list.sum do |item|
      item.product_sku.shipping_cost_point.to_i * item.qty.to_i
    end
  end

  # 送料に関するキャンペーン効果をクリアします
  def clear_shipping_cost_effect
    self.shipping_cost_effect_id = nil
  end

  # 送料に関するキャンペーン効果を持っているか？
  # (calculate後に有効な値を返すようになります)
  def has_shipping_cost_effect?
    !!self.shipping_cost_effect_id
  end
  
  # 送料無料になるか？
  alias_method :free_shipping_cost?, :has_shipping_cost_effect?

  #
  # = 該当するキャンペーンが含まれる場合trueを返却します。=
  def campaign_type_include?(type_code)
    type_id = CampaignEffect.type_id_by_key(type_code)
    self.campaigns_for_cart.each do |cam|
      cam.effects.each do |effect|
        if effect.type_cd == type_id
          return true
        end
      end
    end
    return false
  end

  #
  # 販売期間外の商品を返却します。
  def unsalable_period_items(time = Time.now())
    unsaleable_period_items =[]
    item_list.each do |item|
      unless (item.product_sku.product.sale_started_at.nil? || item.product_sku.product.sale_started_at <= time) &&
          (item.product_sku.product.sale_finished_at.nil? || item.product_sku.product.sale_finished_at >= time)
          unsaleable_period_items << item
      end
    end
    return unsaleable_period_items
  end


  #
  # 同一の商品がすでにカートに存在する場合　その商品をかえします。
  # 存在しない場合は nilを返します。
  # 商品の判別は、商品コードとコース区分で判別します。
  def item_for(sku_or_sku_code, course_div_key = :normal)
    return nil unless @items
    sku_code = sku_or_sku_code.is_a?(ProductSku) ? sku_or_sku_code.sku_code : sku_or_sku_code.to_s
    item_key = Cart::Item.key_for(sku_code, course_div_key)
    @items[item_key]
  end
  deprecate_alias_method :same_product, :item_for

  #
  # 同一キーの商品がカート内に存在する場合trueを返します。
  # 
  def has_item_key?(item_key)
    @items.has_key?(item_key)
  end
  deprecate_alias_method :has_same_key_product, :has_item_key?
  
  # = カートに商品をSKU追加します。 =
  #
  def add(sku_or_sku_code, course_div_key = :normal, qty = 1)
    sku = sku_or_sku_code.is_a?(ProductSku) ? sku_or_sku_code : nil
    sku_code = sku_or_sku_code.is_a?(ProductSku) ? sku_or_sku_code.sku_code : sku_or_sku_code.to_s
    item_key = Cart::Item.key_for(sku_code, course_div_key)
    if @items.has_key?(item_key)
      # すでにカートに同一の商品が存在する場合
      item = @items[item_key]
      sku ||= ProductSku.find_by_sku_code(sku_code)
      item.add_qty(qty, self.secured_stock_count_by_sku_id(sku.id))
    else
      sku ||= ProductSku.find_by_sku_code(sku_code)
      item = Cart::Item.new(
        :cart => self,
        :key => item_key, 
        :qty => qty,
        :product_sku => sku, 
        :course_div_key => course_div_key, 
        :user_rank_id => user_rank ? user_rank.id : nil)
      @items[item_key] = item
      @item_list << item
    end
    item
  end
  deprecate_alias_method :add_sku, :add
  deprecate_alias_method :add_product, :add_sku

  #
  # カートに複数の商品を追加します
  #
  def add_items(order_item_hash = {})
    # 商品SKUをまとめて取得
    product_sku_ids = order_item_hash.values.map{|item| item[:product_sku_id].to_i}
    product_skus = ProductSku.find(:all, :conditions => ["id in (?)", product_sku_ids], :include => [:prices, {:product => :product_displays}])
    product_sku_hash = product_skus.inject({}){|result, sku| result[sku.id] = sku; result}

    # カートに商品を追加（プレゼント品は再計算時に追加されるのでここではカートに入れない）
    order_item_hash.keys.map{|index|index.to_i}.sort.each do |idx|
      sku = product_sku_hash[order_item_hash[idx.to_s][:product_sku_id].to_i]
      self.add(sku, :normal, order_item_hash[idx.to_s][:qty].to_i) unless sku.product.present_flg
    end
  end

  # = カート内商品SKU削除 =
  # 指定のキーの商品SKUをカートから全て取り除きます
  #
  def remove_by_item_key(item_key)
    if @items[item_key]
      item = @items.delete(item_key)
      @item_list.delete(item) if item
    end
    self
  end
  deprecate_alias_method :remove_product, :remove_by_item_key
  
  # = カート内商品SKU削除 =
  # SKUコードとコース種別を指定して、商品SKUをカートから全て取り除きます
  #
  def remove(sku_or_sku_code, course_div_key)
    sku_code = sku_or_sku_code.is_a?(ProductSku) ? sku_or_sku_code.sku_code : sku_or_sku_code.to_s
    item_key = Cart::Item.key_for(sku_code, course_div_key)
    remove_by_item_key(item_key)
  end


  # 商品SKUをリプレースする。
  # 指定の商品SKUコードに合致する明細を指定の条件の明細に変更します。
  # resourse_course_keyは、変更前のコースを指定してください。
  # 
  def replace(product_code, resourse_course_key, course_div_key = :normal, qty = 1)
    item_key = Cart::Item.key_for(product_code, resourse_course_key)
    remove_by_item_key(item_key)
    self.add(product_code, course_div_key, qty)
    self
  end
  deprecate_alias_method :replace_product, :replace

  #  数量を変更します。 
  def change_qty(item_key, qty)
    @items[item_key].change_qty(qty)
    self
  end

  # 引き当て済み数を返します
  def secured_stock_count_by_sku_id(sku_id)
    count = 0
    self.secured_stocks ||= []
    self.secured_stocks.each do |secured_stock|
      count += secured_stock.count if secured_stock.product_sku_id == sku_id
    end
    return count
  end
  
  def delete_nonperiodic_sale_present
    @items.each do |key, item|
      if item.course_div_key == :present
        sku = item.product_sku
        # 販売期間外のプレゼント商品を削除します
        remove(sku, :present) unless sku.product.check_sales_period?
      end
    end
  end
  
  # 在庫切れのプレゼント品を取り除きます
  def delete_out_of_stock_present
    @items.each do |key, item|
      if item.course_div_key == :present
        sku = item.product_sku
        new_qty = [item.qty, sku.orderable_count].min
        if new_qty > 0
          change_qty(key, new_qty)
        else
          remove(sku, :present)
        end
      end
    end
  end
  
  # キャンペーンなどを考慮して金額を計算します。
  def calculate
    clear_amounts
    clear_given_points
    clear_presents
    clear_remembered_effects
    clear_shipping_cost_effect
    clear_point_setting
    prepare_point_setting
    detach_campaigns
    attach_campaigns

    # ～([:discount_price, :point])ではなく、～(:discount_price, :point)という形で引数を受け取るため*を付けています
    apply_campaigns_for_items(*CampaignEffect.type_keys)
    calculate_subtotal_amounts
    apply_campaigns_for_cart(*CampaignEffect.type_keys)
  end

  # 適用されているキャンペーンの一覧を返す
  def applied_campaigns_for_cart_and_items
    item_list.map(&:applied_campaigns).flatten.uniq.sort_by{|v| v.priority}.reverse + applied_campaigns
  end

  # ポイント設定から取得したポイント制度使用の可否を返します。
  def point_setting_available?
    prepare_point_setting if @point_setting_available.nil?
    @point_setting_available
  end

  # ポイント設定から取得したポイント発行の金額（1ポイント／?円）の可否を返します。
  def point_setting_issue_price
    prepare_point_setting if @point_setting_issue_price.nil?
    @point_setting_issue_price
  end

  private

  # ポイント制度使用の可否をポイント設定から取得します。
  def clear_point_setting
    @point_setting_issue_price = nil
    @point_setting_available = nil
  end

  # ポイント制度使用の可否をポイント設定から取得します。
  def prepare_point_setting
    point_setting = PointSetting.find(:first) # point_settingはセッションに載らないようにインスタンス変数には保持しない
    @point_setting_issue_price = (point_setting ? point_setting.issue_price : nil) || 0
    @point_setting_available = (point_setting && @point_setting_issue_price > 0) ?
      !!point_setting.available_flg_true? : false
  end

  def clear_amounts
    @subtotal_amount = 0
    @discount_amount = 0
    @total_amount    = 0
  end

  def clear_given_points
    @given_point = nil
    self.each{|item|item.clear_given_points}
  end

  def clear_presents
    removing_item_keys = items.map{|(key, item)| item && item.present? ? key : nil}.compact
    removing_item_keys.each do |key|
      remove_by_item_key(key)
    end
  end

  def clear_remembered_effects
    self.clear_effects
    self.items.each{|key, item| item.clear_effects}
  end

  include ModelSerializable

  # このカートの明細に適用されるキャンペーンを保持します。
  attr_deserializable_array(:campaigns_for_items, Campaign, :campaign_ids_for_items)

  # このカートに適用されるキャンペーンを保持します。
  attr_deserializable_array(:campaigns_for_cart, Campaign, :campaign_ids_for_cart)

  def detach_campaigns
    @campaigns_for_items = nil
    @campaigns_for_cart = nil
    @campaign_ids_for_items = nil
    @campaign_ids_for_cart = nil
  end

  # カートの明細、カートに適用され得るキャンペーンをそれぞれ優先度の高い順に並べてセットします
  def attach_campaigns
    campaigns = Campaign.availables(campaign_attached_at).find(:all,
      :include => [:conditions, :effects, {:targets => :product_sku}])
    campaigns.each do |campaign|
      # プレゼントのキャンペーンでプレゼント品が在庫切れならキャンペーンの適用対象外とします。
      next if campaign.present_but_no_stock?(self)
      # キャンペーンに対象日時をセットする
      campaign.attached_at = campaign_attached_at
      # この時点ではCart#subtotal_amountなどは有効な数字が入っていません。
      # より優先度の高いものが適用された後でないと自身が適用されるかどうかは判断できないので、
      # ここではマッチするかどうかは考慮せずに、campaigns_for_cartに入れておきます。
      # 実際にマッチするかどうかの判断は、キャンペーン適用時に行います。
      if campaign.targets_cart?
        campaigns_for_cart << campaign
      elsif campaign.match_to_target?(self)
        campaigns_for_items << campaign 
      end
    end
  end

  # 並べられた順に、カートの明細にキャンペーンを適用します
  def apply_campaigns_for_items(*available_type_keys)
    campaigns = campaigns_for_items.select{|campaign| available_type_keys.include?(campaign.effect_type_key) }
    campaigns.each do |campaign|
      if campaign.targets_items?
        campaign.apply(self)
      end
    end
  end

  def calculate_subtotal_amounts
    @subtotal_amount = @item_list.sum(&:total_amount)
    @subtotal_amount_for_discount = @item_list.select{|item| item.discount_amount.to_i == 0}.sum(&:total_amount)
  end

  # 並べられた順に、カートにキャンペーンを適用します
  def apply_campaigns_for_cart(*available_type_keys)
    campaigns = campaigns_for_cart.select{|campaign| available_type_keys.include?(campaign.effect_type_key) }
    campaigns.each do |campaign|
      if campaign.match?(self)
        campaign.apply(self)
        calculate_subtotal_amounts
      end
    end
  end

  public

  # Campaign#match?の引数として渡すコンテキストのHashを生成します
  # Cart::Itemsにも同じ名前のメソッドが用意してあります
  def to_campaign_context(options = nil)
    result = {:cart_or_item => self, :user => self.user, :campaign_attached_at => self.campaign_attached_at}
    result.update(options) if options
    result
  end

  MANIFACTURED_MARSHALING_ATTRS = %w(@applied_effects @campaigns_for_cart @campaigns_for_items) +
    %w(@items @user @user_rank)

  # シリアライズするためのオブジェクトを返します。
  # marshal_loadも参照してください。
  # http://www.ruby-lang.org/ja/man/html/Marshal.html
  def marshal_dump
    attrs = self.instance_variables - MANIFACTURED_MARSHALING_ATTRS
    result = attrs.inject({}){|dest, attr| dest[attr] = instance_variable_get(attr); dest}
    result['@applied_effect_ids'] = to_id_array(@applied_effects)
    result['@campaign_ids_for_cart'] = to_id_array(@campaigns_for_cart)
    result['@campaign_ids_for_items'] = to_id_array(@campaigns_for_items)
    result['@items'] = nil
    result['@user_id'] = user.id if user
    result
  end

  # デシリアライズします。
  # marshal_dumpも参照してください。
  # http://www.ruby-lang.org/ja/man/html/Marshal.html
  def marshal_load(obj)
    obj.each do |attr, value|
      self.instance_variable_set(attr, value)
    end
    @items = @item_list.inject({}) do |dest, item|
      dest[item.key] = item
      dest
    end
  end

  private
  def to_id_array(objects)
    (objects || []).map(&:id)
  end

  #============================================================================
  # カート内の明細 (Cart::Item)
  #============================================================================
  class Item
    include SelectableAttr::Base
    include CampaignEffect::Targetable

    selectable_attr :course_div, :enum => OrderItem.course_div_enum

    class << self
      def deprecate_alias_method(deprecated, source)
        alias_method deprecated, source
        deprecate deprecated => source
      end

      #
      # カートの格納キーを生成します。
      #
      def key_for(sku_code, course_div_key)
        "#{sku_code}_#{self.course_div_id_by_key(course_div_key)}"
      end
    end

    attr_accessor :cart
    
    attr_accessor(
      :key,                  # カート項目キー "#{SKUコード}_#{course_div}"
      :name,                 # 商品名
      :course_div,           # コース区分( <'1', :normal, '単品' # 都度購入>, <'2', :periodic, '定期' # 定期購入> )
      :qty,                  # 商品SKU毎の数量
      :unit_price,           # 商品SKUの単価
      :user_rank_id          # ユーザーランクID。Cart#user_rank.idと同じ
    )

    alias_attribute :price, :unit_price
    deprecate :price => :unit_price

    # 値をセットする
    def initialize(attributes = nil)
      if attributes
        attributes.each do |attr, value|
          send("#{attr}=", value)
        end
      end
      self.course_div_key ||= :normal
      if @product_sku
        @key ||= self.class.key_for(@product_sku.sku_code, self.course_div_key)
        @name ||= @product_sku.product_name_1
        unless self.course_div_key == :present
          @unit_price ||= (@user_rank_id ? @product_sku.price_for(@user_rank_id).to_i : nil) || @product_sku.price.to_i
        end
      end
    end

    # 割引金額
    def discount_amount
      @discount_amount.to_i
    end
    attr_writer :discount_amount
    deprecate_alias_method :total_discount_price, :discount_amount

    # 付与ポイント
    # @given_pointがあればそれを、無ければは明細合計金額から計算して返します。
    def given_point 
      (@given_point || point).to_i
    end
    attr_writer :given_point

    # 明細合計金額を元に付与ポイントを計算して返す
    def point
      if total_amount > 0 && cart.point_setting_available?
        total_amount / cart.point_setting_issue_price
      else
        0
      end
    end

    # ポイントアップキャンペーンが適用されていればtrueを返す
    def assign_point_campaign?
      !(@given_point.nil?)
    end

    # キャンペーン適用後ポイントをクリアする
    def clear_given_points
      @given_point = nil
    end

    # skuを返す
    def product_sku
      if @product_sku_id
        @product_sku = ProductSku.find(@product_sku_id) 
        @product_sku_id = nil
      end
      @product_sku
    end
    attr_writer :product_sku

    # sku_idを返す
    def product_sku_id
      product_sku ? product_sku.id : nil
    end

    # 小計金額（割引適用前）
    def unit_price_x_qty
      unit_price.to_i * qty.to_i
    end
    alias_method :subtotal_amount, :unit_price_x_qty
    alias_method :subtotal_amount_for_discount, :subtotal_amount # 割引額計算用の額

    # 合計金額（割引適用後）
    def total_amount
      unit_price_x_qty - discount_amount
    end
    deprecate_alias_method :total_price, :total_amount

    # 数量変更
    def change_qty(new_qty)
      self.qty = new_qty unless self.qty == new_qty
    end

    # 数量追加
    def add_qty(qty, secure_stock_count = 0)
      _qty = [(self.product_sku.orderable_count + secure_stock_count), (self.qty + qty)].min
      change_qty(_qty)
    end

    # 数量削減
    def reduce_qty(qty=1)
      @qty -= qty
    end
    deprecate_alias_method :remove, :reduce_qty
    
    # Campaign#match?の引数として渡すコンテキストのHashを生成します
    # Cartにも同じ名前のメソッドが用意してあります
    def to_campaign_context(options = nil)
      result = {:cart_or_item => self, :user => self.cart.user, :campaign_attached_at => self.cart.campaign_attached_at}
      result.update(options) if options
      result
    end

    #
    # orderable_count
    # 注文可能数を返します
    # 購入数上限と在庫数のうち、小さい方の値。
    # ただし、購入可能数上限が未入力の場合は、99と在庫数のうち、小さい方の値。
    #
    def orderable_count
      self.product_sku.orderable_count
    end

    # 送料キャンペーン効果
    # (2009/12/02時点では、明細に対する送料に関するキャンペーン効果はカート全体に適用される)
    def shipping_cost_effect_id
      cart.shipping_cost_effect_id
    end
    def shipping_cost_effect_id=(value)
      cart.shipping_cost_effect_id = value
    end


    # プレゼントにする
    def be_present
      self.unit_price = 0
      self.discount_amount = 0
    end

    # プレゼントかどうか
    def present?
      # findするとincludeなどで取得済のproductとは関係なく再度findしてしまい効率が悪いです。
      # 修正したいところですが、テストがパスしないので後回しにします。
      product = Product.find(self.product_sku.product_id) if self.product_sku
      # product = self.product_sku.product # Product.find(self.product_sku.product_id)
      product && product.present_flg
    end

    MANIFACTURED_MARSHALING_ATTRS = %w(@cart @product_sku @applied_effects)
    
    # シリアライズするためのオブジェクトを返します。
    # marshal_loadも参照してください。
    # http://www.ruby-lang.org/ja/man/html/Marshal.html
    def marshal_dump
      attrs = self.instance_variables - MANIFACTURED_MARSHALING_ATTRS
      result = attrs.inject({}){|dest, attr| dest[attr] = instance_variable_get(attr); dest}
      result['@cart'] = nil # cartは保持しません。
      result['@product_sku_id'] = @product_sku.id if @product_sku
      result['@applied_effect_ids'] = (@applied_effects || []).map(&:id)
      result
    end

    # デシリアライズします。
    # marshal_dumpも参照してください。
    # http://www.ruby-lang.org/ja/man/html/Marshal.html
    def marshal_load(obj)
      obj.each do |attr, value|
        self.instance_variable_set(attr, value)
      end
    end

  end
end
