# -*- coding: utf-8 -*-
require 'ec_rider/config'
require 'uri'

class CmsDeadLinkCheck < ActiveRecord::Base
  
  belongs_to :cms_publish, :foreign_key=> "cms_publish_id"
  has_many :resources, :class_name => 'CmsDeadLinkResource', :foreign_key => 'cms_dead_link_check_id', 
    :order => "cms_dead_link_resources.resource"
  
  selectable_attr :status_cd do
    entry '0', :initialized, '初期化済み'
    entry '1', :waiting    , 'チェック待ち'
    entry '2', :processing , 'チェック処理中'
    entry '3', :checked    , 'チェック済み'
    entry '4', :failed     , 'チェック失敗'
    entry '5', :canceled   , 'キャンセル'
  end
  
  def status_name_for_test
    case status_key
      when :waiting, :processing
        'チェック中'
      when :checked
        'チェック完了'
      when :failed
        'チェック失敗'
      when :canceled
        'キャンセル'
    end
  end
  
  # 一覧用にリソースをソートします。
  def sorted_resources
    classified_resouces = {}
    resources.each do |r|
      classified_resouces[r.resource] ||= []
      classified_resouces[r.resource] << r
    end
    result = []
    classified_resouces.keys.sort.each do |key|
      case CmsDeadLinkResource.resource_key_by_id(key)
        when :cms_bind
          result += classified_resouces[key].sort_by{|r| r.cms_bind.cms_page_id.to_s + r.locale}
        when :cms_template
          result += classified_resouces[key].sort_by{|r| r.cms_template.id.to_s + r.locale}
        when :cms_file
          result += classified_resouces[key].sort_by{|r| r.cms_file.public_file_path}
        when :product
          result += classified_resouces[key].sort_by{|r| r.product.product_code unless r.product.nil?}
        when :product_image
          result += classified_resouces[key].sort_by{|r| r.product_image.product.product_code unless r.product_image.nil?}
        when :product_display
          result += classified_resouces[key].sort_by{|r| (r.product_display.product.product_code + r.product_display.locale) unless r.product_display.nil?}
        when :banner
          result += classified_resouces[key].sort_by{|r| r.banner.code unless r.banner.nil?}
        when :banner_display
          result += classified_resouces[key].sort_by{|r| r.banner_display.banner.code unless r.banner_display.nil?}
        when :shop_setting
          result += classified_resouces[key].sort_by{|r| r.shop_setting.setting_div unless r.shop_setting.nil?}
        else
          result += classified_resouces[key]
        end
    end
    result
  end
  
  # リンク切れチェックを行います。
  def execute
    begin
      check_waiting_publish
      # 対象のリンクをセット
      setup_link_matrix
      # チェック実行
      check_link
      self.resources << exception_resources unless exception_resources.empty?
      # うまくいったらopenedに更新
      self.status_key = :checked
    rescue PublishInterruptedError
      # 中止されたらcancelsdに更新
      self.status_key = :canceled
    rescue Exception => err
      raise unless RAILS_ENV == "production"
      logger.error("#{self.class}[#{self.id}].execute failure.....\n" << 
        "  because of #{err}\n  " << err.backtrace.join("\n"))
      # 失敗したらfailedに更新
      self.status_key = :failed
    end
    self.save!
  end
  
  # execute内で使用される属性
  # keyのpath(url)にはURLのパラメータ('?'以下)は含みません。
  attr_accessor :http_resource_path_matrix, :public_resource_path_matrix, :action_resource_path_matrix,
    :exception_resources
  # チェック対象となるマトリックスを作成します。
  def setup_link_matrix
    @http_resource_path_matrix, @public_resource_path_matrix, @action_resource_path_matrix, @exception_resources = LinkMatrixSetup.new(self).process
  end
  

  private

  class LinkMatrixSetup
    
    attr_reader :cms_publish

    def initialize(cms_dead_link_check = nil)
      @owner = cms_dead_link_check || CmsDeadLinkCheck.new
      @cms_publish = @owner.cms_publish
    end

    def process
      @http_resource_path_matrix = {}
      @public_resource_path_matrix = {}
      @action_resource_path_matrix = {}
      @exception_resources = []
      time_now = cms_publish.designated_at
      visible_condition = ["((publish_started_at is null or publish_started_at <= ?) and "<<
        "(publish_finished_at is null or publish_finished_at >= ?))", time_now, time_now]
      process_products(visible_condition)
      process_banners(visible_condition)
      process_shop_setting_images
      process_shop_setting_actions
      @owner.check_waiting_publish
      process_cms_bindings
      process_cms_default_templates
      @owner.check_waiting_publish
      process_cms_files
      return [@http_resource_path_matrix, @public_resource_path_matrix, @action_resource_path_matrix, @exception_resources]
    end

    private

    def add(resource_path)
      url_path = resource_path.to_url_path
      if url_path =~ /^http/
        @http_resource_path_matrix[url_path] ||= []
        @http_resource_path_matrix[url_path] << resource_path
      elsif url_path =~ /^public\//
        @public_resource_path_matrix[url_path] ||= []
        @public_resource_path_matrix[url_path] << resource_path
      else
        @action_resource_path_matrix[url_path] ||= []
        @action_resource_path_matrix[url_path] << resource_path
      end
    end

    def process_products(visible_condition)
      products = Product.find(:all, :order => "products.product_code desc", :include => [:images, :product_displays],
        :conditions => visible_condition)
      products.each do |product|
        # 商品画像
        product.images.each do |image|
          add(ResourcePath.new(:product_image, image.id, image.filename, File.join('public', 'product_images', image.filename)))
          add(ResourcePath.new(:product_image, image.id, image.public_filename(:thumb), File.join('public', image.public_filename(:thumb))))
        end
        # 商品表示
        product.product_displays.each do |display|
          extract_target_path_in_xml([display.product_outline, display.product_explanation, display.normal_explanation, 
            display.recommendation, display.recommendation_2], :product_display, display.id).each do |path|
            add(ResourcePath.new(:product_display, display.id, path.original_path, path.check_path))
          end
        end
        # 商品詳細URL
        add(ResourcePath.new(:product, product.id, product.detail_url, product.detail_url))
      end
    end

    def process_banners(visible_condition)
      banners = Banner.find(:all, :order => "banners.id", :include => [:displays], :conditions => visible_condition)
      banners.each do |banner|
        # バナーリンク先
        add(ResourcePath.new(:banner, banner.id, banner.link_url, banner.link_url))
        # バナー画像
        banner.displays.each do |display|
          if banner.type_key == :image
            add(ResourcePath.new(:banner_display, display.id, display.image_public_path,
                File.join('public', display.image_public_path)))
          else
            extract_target_path_in_xml([display.text_title, display.text_description], :banner_display, display.id).each do |path|
              add(ResourcePath.new(:banner_display, display.id, path.original_path, path.check_path))
            end
          end
        end
      end
    end


    def process_shop_setting_images
      # 店舗設定(画像)
      setting_divs = [:shop_logo].map{|key| ShopSetting.setting_div_id_by_key(key)}
      shop_settings = ShopSetting.find(:all, :conditions => ["setting_div in (?)", setting_divs])
      shop_settings.each do |setting|
        add(ResourcePath.new(:shop_setting, setting.id, setting.value, File.join('public', setting.value)))
      end
    end

    def process_shop_setting_actions
      # 店舗設定(アクション)
      setting_divs = [:top_action, :continue_shopping_action].map{|key| ShopSetting.setting_div_id_by_key(key)}
      shop_settings = ShopSetting.find(:all, :conditions => ["setting_div in (?)", setting_divs])
      shop_settings.each do |setting|
        add(ResourcePath.new(:shop_setting, setting.id, setting.value, setting.value))
      end
    end

    def process_cms_bindings
      # ページ
      bind_map = CmsBind.filename_map(
        {:project_ids => cms_publish.cms_published_projects.map{|p| p.cms_project_id}, 
        :find_options => {:include => [{:cms_template => :contents}, {:items => :contents}]}}
      )
      files = Dir.glob(File.join(EcRider.config.ec_test_root, 'app', 'views', '**', "*.html.erb"))
      file_paths = files.map{|path| path.sub(File.join(EcRider.config.ec_test_root, 'app', 'views') << '/', '')}
      checked_bind = []
      file_paths.each do |file_path|
        next unless bind_map[file_path]
        bind = bind_map[file_path].first
        base_url = bind.cms_page.real_local_path
        next if checked_bind.include?(bind.cms_template_id)
        bind.cms_template.contents.each do |content|
          extract_target_path_in_xml([content.descriptions], :cms_bind, bind.id, base_url).each do |path|
            add(ResourcePath.new(
              :cms_bind, bind.id, path.original_path, path.check_path, content.locale))
          end
        end
        if bind.items
          bind.items.each do |item|
            item.contents.each do |content|
              extract_target_path_in_xml([content.descriptions], :cms_bind, bind.id, base_url).each do |path|
                add(ResourcePath.new(
                  :cms_bind, bind.id, path.original_path, path.check_path, content.locale))
              end
            end
          end
        end
        checked_bind << bind.cms_template_id
      end
    end

    def process_cms_default_templates
      # デフォルトテンプレート
      template_map = CmsTemplate.default_template_filename_map(
        :projects => cms_publish.cms_published_projects.map{|published| published.cms_project}, 
        :find_options => {:include => :contents}
      )
      template_map.values.each do |template|
        next unless template.last == 'ja'
        default_template = template.first
        default_template.contents.each do |content|
          extract_target_path_in_xml([content.descriptions], :cms_template, default_template.id).each do |path|
            add(ResourcePath.new(
              :cms_template, default_template.id, path.original_path, path.check_path, content.locale))
          end
        end
      end
    end

    def process_cms_files
      files = Dir.glob(File.join(EcRider.config.ec_test_root, 'public', '**', "*.css"))
      files.delete_if {|path| /^\S*\-\d{14}\.\S*$/ =~ path}
      # css
      files.each do |file|
        cms_file = CmsFile.find(:first, :conditions => ['public_file_path = ?', file.sub(EcRider.config.ec_test_root + '/', '')])
        if cms_file
          extract_target_path_in_css(file).each do |path|
            add(ResourcePath.new(:cms_file, cms_file.id, path.original_path, path.check_path))
          end
        end
      end
    end

    # cssファイルを開いてチェック対象のファイルやアクションのパスを取り出します。
    def extract_target_path_in_css(path)
      @target_paths = []
      reg_ex = Regexp.escape(EcRider.config.ec_test_root)
      base_dir = File.dirname(path).sub(/#{reg_ex}\S*?public/, '')
      open(path, 'r') do |f|
        extract_path_in_css(f.read, base_dir)
      end
      return @target_paths
    end
    
    def extract_path_in_css(source, base_dir = nil)
      return unless source
      # コメント文を取り除く
      source = source.gsub(/\/\*.+?\*\//m, '').gsub(/\/\/.+?$/m, '')
      # url()、@import""、@import''を取り出す
      reg_ex = Regexp.union(
        /\@import\s+?\"(.*?)\"/,
        /\@import\s+?\'(.*?)\'/,
        /url\((.*?)\)/i)
      paths = source.scan(reg_ex).flatten.compact
      paths.map!{|path| path.gsub(/"|'|\s/m, '')}.uniq!
      paths.each do |path|
        relative_path = (path =~ /^http|^\// || base_dir.nil?) ? path : File.join(base_dir, path)
        if File.extname(path) =~ /.css/i
          @target_paths << ResourcePath.new(nil, nil, path, format_src_path(relative_path, 'stylesheet'))
        else
          @target_paths << ResourcePath.new(nil, nil, path, format_src_path(relative_path, nil))
        end
      end
    end

    # xmlからチェック対象のファイルやアクションのパスを取り出します。
    def extract_target_path_in_xml(sources, resource_key, resource_id, base_url = nil)
      @target_paths = []
      sources.each do |source|
        next if source.blank?
        unless source.include?("<ec xmlns:ec=\"http://dummy\">#{source}</ec>")
          source = "<ec xmlns:ec=\"http://dummy\">#{source}</ec>"
        end
        begin
          doc = REXML::Document.new(source)
          extract_path_in_xml(doc.elements.first, base_url)
        rescue REXML::ParseException
          ActiveRecord::Base.logger.error("REXML::ParseException...resource:#{resource_key},id:#{resource_id}")
          # XML不正もリンクチェック不正に｢XML不正｣として追加する
          resource = CmsDeadLinkResource.new(:resource_key => resource_key,
            "#{resource_key}_id".intern => resource_id)
          resource.links << CmsDeadLink.new(:url => "XML記述が不正です。", :response_status => "XML記述が不正です。")
          @exception_resources << resource
        end
      end
      return @target_paths
    end

    def extract_path_in_xml(parent, base_url)
      href_option_tags = %w(stylesheet javascript script) + 
        %w(button image_link shop_logo_link img link a area) + 
        %w(link_to_member_registration link_to_member_registration link_to_logout)
      src_options_tags = %w(stylesheet javascript image image_link img link script) + 
        %w(cart_link product_image link_to_member_registration link_to_login link_to_logout)

      parent.children.each do |child|
        if child.class == REXML::Element
          # href属性(アクションへのリンク)を持つHTMLタグ、ECタグ
          if href_option_tags.include?(child.name)
            child.name = 'a' if child.prefix == 'ec' && child.name == 'link'
            original_path = child.attributes["href"]
            unless original_path.blank? || original_path =~ /^\#$|javascript:void\(0\)/
              unless base_url.nil? || %w(stylesheet javascript script link).include?(child.name) || original_path =~ /^\//
                original_path = base_url.slice(0..base_url.rindex('/')) + original_path
              end
              @target_paths << ResourcePath.new(nil, nil, original_path, format_href_path(original_path, child.name))
            end
          end
          # src属性を持つHTMLタグ、ECタグ
          if src_options_tags.include?(child.name)
            original_path = child.attributes['src']
            unless original_path.blank?
              @target_paths << ResourcePath.new(nil, nil, original_path, format_src_path(original_path, child.name))
            end
          end
          # styleタグ
          if child.name == 'style'
            extract_path_in_css(child.text, nil)
          end
          # style属性をもつタグ
          if attr = child.attributes['style']
            extract_path_in_css(attr, nil)
          end
          # 子要素
          unless child.children.empty?
            extract_path_in_xml(child, base_url)
          end
        end
      end
    end

    # モデルでActionViewに含まれるクラスをincludeすることは禁止ですが
    # リンクのhref,src指定文字列をフォーマットするのに
    # stylesheet_path、javascript_path、image_pathを使用したいので例外的に認めます。
    include ActionView::Helpers::AssetTagHelper

    def format_href_path(target_path, child_name)
      case child_name
      when 'stylesheet', 'link'   then target_path = stylesheet_path(target_path)
      when 'javascript', 'script' then target_path = javascript_path(target_path)
      else return target_path
      end
      target_path =~ /^http/ ? target_path : "public#{target_path}"
    end

    def format_src_path(target_path, child_name)
      case child_name
      when 'stylesheet', 'link'   then target_path = stylesheet_path(target_path)
      when 'javascript', 'script' then target_path = javascript_path(target_path)
      else target_path = image_path(target_path)
      end
      target_path =~ /^http/ ? target_path : "public#{target_path}"
    end
  end

  public

  require 'net/http'
  
  def check_link
    @ng_resources = {}
    check_http_link
    check_public_link
    check_action_link
    self.resources = @ng_resources.values
  end
  
  def add_dead_resources(resource_paths, response_status)
    resource_paths.each do |resource_path|
      key =  resource_path.cms_dead_link_resource.attributes.reject!{|key, value| key =~ /_at/ || value.nil?}.to_a.sort.to_s
      ng_resource = @ng_resources[key] || resource_path.cms_dead_link_resource
      ng_resource.links << CmsDeadLink.new(:url => resource_path.original_path, :response_status => response_status.to_s)
      @ng_resources[key] = ng_resource
    end
  end
  
  # httpリンク切れチェックを行います
  def check_http_link
    cache = {}
    http_resource_path_matrix.each do |url, resource_paths|
      check_waiting_publish
      # チェック済みの場合
      if cache.has_key?(url)
        response_status = cache[url]
      else
        response_status = fetch(resource_paths.first.check_path)
        cache[url] = response_status
      end
      unless response_status == Net::HTTPSuccess || response_status.is_a?(Net::HTTPSuccess)
        add_dead_resources(resource_paths, response_status.class)
      end
    end
  end
  
  def fetch(url, limit = 10)
    return 'http redirect too deep' if limit == 0
    Net::HTTP.version_1_2    # おまじない
    if external_site_ignore_flg == true
      response = Net::HTTPSuccess
    else
      response = Net::HTTP.get_response(URI.parse(url))
    end
    # リダイレクトの場合
    if response == Net::HTTPRedirection
      fetch(response['location'], limit - 1)
    else
      response
    end
  end
  
  # ファイルのリンク切れチェックを行います
  def check_public_link
    cache = {}
    public_resource_path_matrix.each do |url, resource_paths|
      check_waiting_publish
      # チェック済みの場合
      if cache.has_key?(url)
        response_status = cache[url]
      else
        response_status = File.exists?(File.join(EcRider.config.ec_test_root, url)) ? Net::HTTPSuccess : Net::HTTPNotFound
        cache[url] = response_status
      end
      if response_status == Net::HTTPNotFound
        add_dead_resources(resource_paths, response_status)
      end
    end
  end
  
  # アクションのリンク切れチェックを行います
  def check_action_link
    target_urls = action_resource_path_matrix.keys
    unless target_urls.empty?
      Net::HTTP.version_1_2   # おまじない
      test_uri = URI.parse(EcRider.config.ec_test_url)
      test_uri.path = "/runtime/validate_url"
      params_string = (target_urls).map{|url| "url[]=" + url}.join("&")
      request = Net::HTTP::Get.new("/runtime/validate_url?" << params_string)
      request.basic_auth(EcRider.config.ec_test_basic_auth_account, EcRider.config.ec_test_basic_auth_password)
      Net::HTTP.start(test_uri.host, test_uri.port) {|http|
        response = http.request(request)
        if response.is_a?(Net::HTTPSuccess)
          response_body = ActiveSupport::JSON.decode(response.body)
          response_body["invalid_urls"].each do |invalid_url|
            add_dead_resources(action_resource_path_matrix[invalid_url], Net::HTTPNotFound)
          end
        else
          logger.info("URL Validation is failed.\n" << response.body)
        end
      }
    end
  end
  
  class ResourcePath
    attr_reader :cms_dead_link_resource, :original_path, :check_path
    
    def initialize(resource_key, resource_id, original_path, check_path, locale = nil)
      @resource_key = resource_key
      if resource_key
        @cms_dead_link_resource = CmsDeadLinkResource.new(
          :resource_key => resource_key, "#{resource_key}_id".intern => resource_id, :locale => locale
        )
      end
      @original_path = original_path  # パス、URLの登録値
      @check_path = check_path        # チェック用にフォーマットしたパス、URL
    end

    def to_url_path
      case @resource_key
      when :product_image   then check_path
      when :product_display then check_path.sub(/\?.*?$/, '')
      when :product         then check_path.sub(/\?.*?$/, '')
      when :banner          then check_path.sub(/\?.*?$/, '')
      when :banner_display  then check_path
      when :shop_setting    then check_path
      when :cms_bind        then check_path.sub(/\?.*?$/, '')
      when :cms_template    then check_path.sub(/\?.*?$/, '')
      when :cms_file        then check_path.sub(/\?.*?$/, '')
      end
    end

  end
  
  class << self
    # テスト環境リンク切れチェックタスク
    def execute_if_waiting
      CmsDeadLinkCheck.exclusive_processing do |check|
        check.checked_at = Time.now
        check.execute
      end
    end
    
    # リンクチェック対象となるもの(check_candidate)がある場合、それを:processingの状態に変更して、他のCmsDeadLinkCheckとの
    # 排他制御に引っかからないようにします。
    # その後ブロックを実行して、正常に動けば:openedに、エラーがあれば:failedにします。
    def exclusive_processing
      check = nil
      transaction do
        # リンクチェック対象となるものを検索します。
        # 他のプロジェクトのステータスが何も:processingになっていない状態で、
        # :waitingのレコードがあった場合にそのレコードを返します。
        #
        # :lock => trueで FOR UPDATE を付きのSELECTになります。超重要！！！
        check = CmsDeadLinkCheck.find(:first, :lock => true, :include => {:cms_publish => :cms_published_projects},
          :conditions => [
            "status_cd = ? AND not exists (select * from cms_dead_link_checks where status_cd = ?)",
            status_id_by_key(:waiting),
            status_id_by_key(:processing)
          ])
        return unless check
        check.status_key = :processing
        check.save!
      end
      yield( check )
    end
  end
  
  class PublishInterruptedError < StandardError; end
    
  def check_waiting_publish
    raise PublishInterruptedError if CmsPublish.exists?(["status_cd = ?", CmsPublish.status_id_by_key(:waiting)])
  end
end
