# -*- coding: utf-8 -*-
class CmsProject < ActiveRecord::Base

  has_many :cms_project_assigns, :order => 'role'
  has_many :assigns, :class_name => 'CmsProjectAssign', :foreign_key => 'cms_project_id'
  has_many :assign_editors, :class_name => 'CmsProjectAssign', :foreign_key => 'cms_project_id', :conditions => ['role = ?', CmsProjectAssign.role_id_by_key(:editor)]
  has_many :assign_checkers, :class_name => 'CmsProjectAssign', :foreign_key => 'cms_project_id', :conditions => ['role = ?', CmsProjectAssign.role_id_by_key(:checker)]
  has_many :assign_approvers, :class_name => 'CmsProjectAssign', :foreign_key => 'cms_project_id', :conditions => ['role = ?', CmsProjectAssign.role_id_by_key(:approver)]

  has_many :templates, :class_name => 'CmsTemplate', :foreign_key => 'cms_project_id'

  has_many :pages, :class_name => 'CmsPage', :foreign_key => 'cms_project_id'
  has_many :files, :class_name => 'CmsFile', :foreign_key => 'cms_project_id'
  # Rails2.3.2のnested_form に移行しやすくなるように名前をつけています。
  attr_accessor :assigns_attributes
  attr_accessor :check_flg

  selectable_attr :status_cd do
    entry '0', :editing, '編集中'
    entry '1', :requesting_check, '確認依頼中'
    entry '2', :requesting_approval, '承認依頼中'
    entry '3', :approved, '公開承認'
    entry '4', :opened, '公開中'
    entry '5', :closed, '公開終了'
  end

  selectable_attr :acceptance_type do
    entry '0', :no_approval, '承認なし'
    entry '1', :only_approval, '承認のみ'
    entry '2', :ed_ck_ap, '作成者→確認者→承認者'
    entry '3', :ed_ap, '作成者→承認者'
  end

  BASE_PROJECT_ID = 1

  validates_presence_of :project_code, :project_name, :started_at, :acceptance_type
  validates_uniqueness_of :project_code
  validates_numericality_of :priority, :only_integer =>true, :message => :EM990003, :if => Proc.new {|cms_project| !cms_project.priority.blank?}

  # validateメソッド内で記述しているエラーメッセージは、ec_adminのみで使用するメッセージです
  def validate
    if self.check_flg
      # 公開開始日＞公開終了日の時はエラーとする
      unless self.started_at.nil? || self.finished_at.nil?
        errors.add :finished_at, :EM300002 if self.started_at > self.finished_at
      end
  
      # メンバーのチェック
      supervisors= []
      roles = []
      if self.acceptance_type_key != :no_approval && self.assigns_attributes.nil?
        return errors.add_to_base "プロジェクトメンバーを入力してください。"
      end
      if self.assigns_attributes
        self.assigns_attributes.each_value do |assigns_attribute|
          supervisors << assigns_attribute[:goldberg_user_id]
          roles << assigns_attribute[:role]
        end
  
        # 必須チェック
        errors.add_to_base "プロジェクトメンバーの役割を入力してください。" if roles.index("")
        errors.add_to_base "プロジェクトメンバーのユーザーを入力してください。" if supervisors.index("")
  
        # 承認経路に必要なロールチェック
        if [:ed_ck_ap, :ed_ap].include?(self.acceptance_type_key) && !roles.include?(CmsProjectAssign.role_id_by_key(:editor))
          errors.add_to_base "プロジェクトメンバーに作成者を設定してください。"
        end
        if [:ed_ck_ap, :ck_ap].include?(self.acceptance_type_key) && !roles.include?(CmsProjectAssign.role_id_by_key(:checker))
          errors.add_to_base "プロジェクトメンバーに確認者を設定してください。"
        end
        if [:only_approval, :ed_ck_ap, :ck_ap, :ed_ap].include?(self.acceptance_type_key) && !roles.include?(CmsProjectAssign.role_id_by_key(:approver))
          errors.add_to_base "プロジェクトメンバーに承認者を設定してください。"
        end
      end
    end
  end

  # 指定したシステム利用者の承認を待っているプロジェクトの件数を返す
  def self.count_pending_approval(approver_id, type=:approval)
    where = 'cms_project_assigns.goldberg_user_id = :user_id AND cms_project_assigns.role = :role AND cms_projects.status_cd = :status_cd'
    param = (type == :check) ? 
      {:user_id => approver_id, :role => CmsProjectAssign.role_id_by_key(:checker), :status_cd => CmsProject.status_id_by_key(:requesting_check)} :
      {:user_id => approver_id, :role => CmsProjectAssign.role_id_by_key(:approver), :status_cd => CmsProject.status_id_by_key(:requesting_approval)}
    
    return count(:include => :assigns, :conditions => [where, param])
  end
  
  # 指定したシステム利用者の確認を待っているプロジェクトの件数を返す
  def self.count_pending_check(checker_id)
    count_pending_approval(checker_id, :check)
  end
  
  #
  # =一覧の検索条件=
  #
  class SearchCondition < EcRider::ConditionForm
    attr_accessor :project_code, :project_name, :status_cd, :started_at, :finished_at

    def to_find_options(options = nil)
      joins = []
      where = Condition.block do |c|
        c.and "project_code", "like", "%#{project_code}%" unless project_code.blank?
        c.and "project_name", "like", "%#{project_name}%" unless project_name.blank?
        c.and "status_cd", status_cd unless status_cd.blank?
        c.and "started_at", ">=", Time.parse(started_at + " 00:00:00") unless started_at.blank?
        c.and "finished_at", "<=", Time.parse(finished_at + " 23:59:59") unless finished_at.blank?
      end
      result = {:conditions => where}
      result[:joins] = joins.uniq.join(" ")
      result.update(options) if options
      return result
    end
  end

  def edittable?
    status_key == :editing || (acceptance_type_key == :no_approval && status_key == :approved) ||
    (acceptance_type_key == :only_approval && status_key == :requesting_approval)
  end

  def reusable?
    status_key == :opened || status_key == :closed
  end

  # 引数の時刻の時点で、公開期間中で承認済みまたは公開中のプロジェクトと公開期間外で公開中のプロジェクトを返します。
  def self.find_all_to_be_deployed_or_deleted(time = Time.now)
    options = {
      :include => [{:pages => [{:binds => [{:items => :contents}, :cms_template]}, :titles]}, :files, {:templates => :contents}],
      :conditions => [
        "((status_cd IN (?, ?)) AND (started_at <= ?) AND (finished_at >= ?)) OR" <<
        "((status_cd = ?) AND (finished_at < ?))",
        CmsProject.status_id_by_key(:approved), CmsProject.status_id_by_key(:opened), time, time,
        CmsProject.status_id_by_key(:opened), time
      ]
    }
    find(:all, options)
  end

  def to_be_opened?(time = Time.now)
    (started_at.to_time <= time) and (finished_at.to_time >= time)
  end

  def to_be_closed?(time = Time.now)
    finished_at.to_time <= time
  end

  def workflow_holder?(user_id)
    assigns.each do |assign|
      return assign if assign.goldberg_user_id == user_id && assign.role == status_cd
    end
    return nil
  end

  def initialize_status
    case acceptance_type_key
      when :no_approval
        self.status_key = :approved
      when :only_approval
        self.status_key = :requesting_approval
      else
        self.status_key = :editing
    end
  end

  def up_status(time = Time.now)
    case status_key
      when :editing
        self.status_key = :requesting_check if acceptance_type_key == :ed_ck_ap
        self.status_key = :requesting_approval if acceptance_type_key == :ed_ap
      when :requesting_check
        self.status_key = :requesting_approval
      when :requesting_approval
        self.status_key = :approved
      when :approved
        self.status_key = :opened if to_be_opened?
      when :opened
        self.status_key = :closed if to_be_closed?
    end
  end
  
  def approval_mail_to
    case status_key
      when :editing
        assign_editors.map{|assign| assign.goldberg_user.email}
      when :requesting_check
        assign_checkers.map{|assign| assign.goldberg_user.email}
      when :requesting_approval
        assign_approvers.map{|assign| assign.goldberg_user.email}
    end
  end
end
