# -*- coding: utf-8 -*-
class CmsTemplate < ActiveRecord::Base
  include EcRider::EcHelper
  include EcRider::I18nSupport

  belongs_to :cms_project, :foreign_key=> "cms_project_id"
  has_many :contents, :class_name => 'CmsTemplateContent', :foreign_key => 'cms_template_id', :order => 'locale asc'
  has_many :binds, :class_name => 'CmsBind', :foreign_key => 'cms_template_id'

  I18n.supported_locales.each do |locale|
    has_one "content_#{locale.to_s}".intern, :class_name => 'CmsTemplateContent', :foreign_key => 'cms_template_id',
      :conditions => ['locale = ?', locale.to_s]
  end

  CmsTemplateContent.carrier_ids.each do |locale|
    has_one "content_#{locale.to_s}".intern, :class_name => 'CmsTemplateContent', :foreign_key => 'cms_template_id',
      :conditions => ['locale = ?', locale.to_s]
  end

  #デフォルト言語のcontentを返す
  def content_default_locale
    ((self.terminal_key == :mobile) ? content_docomo : content_ja)
  end

  #有効なcontent.descriptionsを返す
  def get_descriptions(content)
    (content.descriptions.blank? ? content_default_locale.descriptions : content.descriptions)
  end

  # Rails2.3.2のnested_form に移行しやすくなるように名前をつけています。
  attr_accessor :contents_attributes

  attr_accessor :present_version_id

  attr_accessor :default_template_position #バリデーションチェック用
    
  selectable_attr :display_position do
    entry '0', :top, '上'
    entry '1', :bottom, '下'
    entry '2', :left, '左'
    entry '3', :right, '右'
    entry '4', :center, '中央'
    entry '5', :left_right, '左または右'
  end

  selectable_attr :display_position_mobile do
    entry '0', :top, '上'
    entry '1', :bottom, '下'
    entry '4', :center, '中央'
  end

  selectable_attr :default_template_position do
    entry '0', :top, '上'
    entry '1', :bottom, '下'
    entry '2', :left, '左'
    entry '3', :right, '右'
  end

  selectable_attr :default_template_position_mobile do
    entry '0', :top, '上'
    entry '1', :bottom, '下'
  end

  selectable_attr :terminal do
    entry '0', :pc, 'PC'
    entry '1', :mobile, '携帯'
  end

  validates_presence_of :name
  validates_uniqueness_of :name, :scope => [:cms_project_id]

  after_create :copy_cms_pages_from_present_version_id

  # validateメソッド内で記述しているエラーメッセージは、ec_adminのみで使用するメッセージです
  def validate
    # 表示可能位置とデフォルトテンプレート表示位置の整合性チェック
    unless self.display_position_key.to_s.include?(self.default_template_position_key.to_s)
      errors.add_to_base "表示可能位置とデフォルトテンプレート表示位置が合っていません。"
    end

    if self.terminal_key == :pc && self.contents_attributes && self.contents_attributes[:ja][:descriptions].blank?
      errors.add_to_base "日本語の内容は必須です。"
    end
    if self.terminal_key == :mobile && self.contents_attributes && self.contents_attributes[:docomo][:descriptions].blank?
      errors.add_to_base "NTTドコモの内容は必須です。"
    end

    # 内容
    self.contents_attributes.each_value do |contents_attribute|
      next unless contents_attribute[:descriptions]

      # 文字数チェック
      if  contents_attribute[:descriptions].size > 65535
        errors.add_to_base("#{locale_for_display(contents_attribute[:locale])} 内容 65535バイト以内で入力してください。")
      end

      # XMLの文法チェック
      unless CmsXml.validate_tags(contents_attribute[:descriptions])
        errors.add_to_base("#{locale_for_display(contents_attribute[:locale])} 内容 XMLの記述が不正です。")
      end

      # ecタグとしての存在チェック
      # XML的にはOKなので、ここでは閉じタグはチェックしません。
      ec_tags = contents_attribute[:descriptions].scan(/\<ec:(.*?)[\/]{0,1}\>/m)
      ec_tags.flatten.compact.each do |ec_tag|
        ec_tag_parts = ec_tag.split(/\s/, 2)
        ec_tag_name = ec_tag_parts.first
        errors.add_to_base("#{locale_for_display(contents_attribute[:locale])} 内容 ec:#{ec_tag} は存在しないタグです。") unless ec(self.terminal_key).respond_to?(ec_tag_name)
      end
    end if contents_attributes
  end

  #   
  # CmsTemplateをpresent_version_idで参照されるCmsTemplateを元に作成した場合、
  # 元のCmsTemplateに関連するCmsPageに対応するCmsPage(がなければ生成)に対して、
  # このCmsTemplateのdelete_at_publishが偽(nil, false)の場合、
  #   元のCmsTemplateへ参照しているCmsBindをこのCmsTemplateへの参照に書き換えます。
  # このCmsTemplateのdelete_at_publishが真(trueなど)の場合、
  #   positionがcenterのCmsBindでCmsPageとCmsTemplateが繋がっている場合、
  #     CmsPageのdelete_at_publishをtrueに、
  #   そうでない場合、
  #     元のCmsTemplateへ参照しているCmsBindをこのCmsTemplateへの参照に書き換えます。
  #   
  def copy_cms_pages_from_present_version_id
    return unless present_version_id
    current_project = self.cms_project
    source_template = CmsTemplate.find(present_version_id, :include => {
        :binds => [{:cms_page => :titles}, {:items => :contents}]})
    source_template.binds.each do |source_bind|
      source_page = source_bind.cms_page
      current_page = current_project.pages.find(:first, :conditions => [
          "controller = ? AND action = ?", source_page.controller, source_page.action
        ])
      current_page_found = !current_page.nil?
      current_page ||= source_page.clone(:include => [:titles, {:binds => {:items => :contents} }])

      current_page.binds.each do |current_bind|
        if current_bind.cms_template_id == source_template.id
          if self.delete_at_publish
            if current_bind.position_key == :center
              current_page.delete_at_publish = true
              current_page.binds.clear
            else
              current_bind.cms_template = nil
              current_bind.cms_template_id = nil
            end
          else
            current_bind.cms_template_id = self.id
          end
          # current_page以下が既に存在していた場合は、変更したCmsBind毎に保存
          current_bind.save! if current_page_found
        end
      end
      # current_page以下をコピーして作成した場合、CmsPage以下を一気に保存
      unless current_page_found
        current_page.cms_project_id = self.cms_project_id
        current_page.save!
      end
      
    end
  end


  #
  # =一覧の検索条件=
  # プロジェクト編集対象のテンプレートと、プロジェクト編集対象外かつ公開中のファイルを表示します。
  # ファイルパス名が同じテンプレートは、プロジェクトの優先度が高いものを表示します。
  #
  class SearchCondition < EcRider::ConditionForm
    attr_accessor :template_name, :display_position, :status_cd, :edit_target, :cms_project_id, :terminal, :disp_count

    # 一覧に表示するテンプレートのIDのリストと、対象プロジェクト分の公開時削除フラグのリストを取得します。
    # テンプレート取得優先度は以下の通り
    #
    # ターゲットのプロジェクトで編集済みのテンプレート ＞ 公開中プロジェクトの優先順位の大きい順 > 公開開始日の大きい順 > プロジェクトIDの大きい順
    #
    def template_list_ids(target_project_id)
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :order => 'name',
        :conditions => ["cms_project_id = ? or (cms_project_id <> ? and cms_projects.status_cd in (?, ?))",
          target_project_id, target_project_id, CmsProject.status_id_by_key(:opened), CmsProject.status_id_by_key(:closed)])

      # TODO ロジック改善 if文のネストが多く、見難くなっています。
      # 最優先テンプレートをsqlで一括取得できないか等検討する必要があります。
      code_id = {}
      deleted_ids = []
      delete_template_codes = []
      cms_templates.each do |template|
        replace_status = :keep # テンプレートを置き換えるか、何もしないかを保持します :changeが入ると置き換えます。
        template_code = template.template_code
        if code_id[template_code].nil?
          # 同じテンプレートコードを持つテンプレートが他に無ければそれを採用する
          code_id[template_code] = template
        else
          comparing_project = template.cms_project # 比較中のテンプレートのプロジェクト
          stored_project = code_id[template_code].cms_project # code_idに格納されているテンプレートのプロジェクト

          # 同じテンプレートコードを持つテンプレートが複数あった場合
          # ターゲットのプロジェクトで作られたテンプレートがある場合はそのテンプレートを格納する(最優先)
          # 削除取り消しとして一覧に表示するためcode_idからは削除しません
          if comparing_project.id == target_project_id
            if template.delete_at_publish
              deleted_ids << code_id[template_code].id
            else
              code_id[template_code] = template
            end
          else
            # すでにターゲットのプロジェクトで作られたテンプレートがcode_idに保管されている場合
            if stored_project.id == target_project_id
              # 何もしない。
              replace_status = :keep
              next
            else
            # ターゲットのプロジェクトで未作成のテンプレートの場合
              # 公開中のプロジェクトの優先度が高い（数値が大きい）ものを優先する
              if comparing_project.status_key == :opened
                if comparing_project.priority == stored_project.priority
                  # プロジェクトの優先度が同じ場合はプロジェクトの公開日時の遅いものを優先する
                  # 公開日時も同じ場合は、プロジェクトのidの大きいものを優先する
                  if comparing_project.started_at == stored_project.started_at
                    if comparing_project.id > stored_project.id
                      replace_status = :change
                    end
                  elsif comparing_project.started_at > stored_project.started_at
                    replace_status = :change
                  end
                elsif comparing_project.priority > stored_project.priority
                  replace_status = :change
                else
                  # template.cms_project.priority < code_id[template_code].cms_project.priorityの場合は
                  # なにもしない
                  replace_status = :keep
                  next
                end
              else
                if stored_project.status_key != :opened
                  # 比較しているテンプレートのプロジェクトが公開終了 かつ
                  # 格納されているテンプレートのプロジェクトも公開終了の場合
                  # 優先順位の高いプロジェクトのテンプレートを参照する(優先度付けは公開中のものと同じ)
                  if comparing_project.priority == stored_project.priority
                    # プロジェクトの優先度が同じ場合はプロジェクトの公開日時の遅いものを優先する
                    # 公開日時も同じ場合は、プロジェクトのidの大きいものを優先する
                    if comparing_project.started_at == stored_project.started_at
                      if comparing_project.id > stored_project.id
                        replace_status = :change
                      end
                    elsif comparing_project.started_at > stored_project.started_at
                      replace_status = :change
                    end
                  elsif comparing_project.priority > stored_project.priority
                    replace_status = :change
                  else
                    # template.cms_project.priority < code_id[template_code].cms_project.priorityの場合は
                    # なにもしない
                    replace_status = :keep
                    next
                  end
                end
              end
            end
          end
        end
        # replace_statusが:changeの場合現在格納されているテンプレートを
        # 優先度の高いテンプレートに置き換える
        # 削除日時が含まれている場合は削除テンプレートに追加する
        if replace_status == :change
          if template.delete_at_publish
            deleted_ids << code_id[template_code].id
            delete_template_codes << template_code
          else
            code_id[template_code] = template
          end
        end
      end
      # 公開中の公開削除中のテンプレートは一覧に表示しないためcode_idから削除します
      delete_template_codes.each do |delete_template_code|
        code_id.delete(delete_template_code)
      end
      [code_id.values.map{|cms_template| cms_template.id }, deleted_ids]
    end

    def create_condition(options = nil)
      delete_ids = []
      where = Condition.block do |c|
        template_ids, delete_ids = template_list_ids(cms_project_id)
        template_ids << '0' if template_ids.empty?
        c.and "cms_templates.id", "in", template_ids
        c.and "name", "like", "%#{template_name}%" unless template_name.blank?
        c.and "display_position", "like", "%#{display_position}%" unless display_position.blank?
        c.and "cms_projects.status_cd", status_cd unless status_cd.blank?
        c.and "cms_project_id", cms_project_id if edit_target == "1"
        c.and "terminal", "=", "#{terminal}" unless terminal.blank?
      end
      [where, delete_ids]
    end

    def create_condition_for_page(options = nil)
      delete_ids = []
      where = Condition.block do |c|
        template_ids, delete_ids = template_list_ids(cms_project_id)
        template_ids << '0' if template_ids.empty?
        c.and "cms_templates.id", "in", template_ids
        if [CmsTemplate.display_position_id_by_key(:top), CmsTemplate.display_position_id_by_key(:bottom), CmsTemplate.display_position_id_by_key(:center)].include?(display_position)
          c.and "display_position", display_position
        else
          c.and "display_position", "in", [display_position, CmsTemplate.display_position_id_by_key(:left_right)]
        end
        c.and "terminal", "=", "#{terminal}" unless terminal.blank?
      end
      where
    end
  end

  def current_project?(project_id)
    cms_project_id.blank? ? true : cms_project_id.to_s == project_id.to_s
  end

  # テンプレートと内容をコピーしたCmsTemplateオブジェクトを返します。
  def new_copy
    cms_template = CmsTemplate.new(self.attributes)
    contents.each do |content|
      cms_template.contents << CmsTemplateContent.new(content.attributes)
    end
    cms_template.present_version_id = self.id
    return cms_template
  end

  # パラメータから登録データをセットします。
  def set_values_for_create
    self.contents_attributes.each_value do |contents_attribute|
      self.contents << CmsTemplateContent.new(contents_attribute)
    end

    # 全く新規のテンプレートを作成する時はテンプレートコードを採番します。
    if self.template_code.blank?
      self.template_code = Time.now.strftime("%Y%m%d%H%M%S") + object_id.abs.to_s
    end
  end

  # バインドタグを取り出します。
  # 全言語分のec:bindタグを検索し、重複する番号を取り除いて返します。
  def bind_items
    items = []
    numbers = []
    contents.each do |content|
      if content.descriptions
        bind_tags = content.descriptions.scan(/\<ec:bind([0-9]{2})\sname.+?"(.+?)".*?\/\>/m)
        bind_tags.each do |tag|
          items << {:no => tag[0], :name => tag[1]} unless numbers.include?(tag[0])
          numbers << tag[0]
        end
      end
    end
    items
  end

  # 上、下、左、右、中央テンプレートセレクトボックスの選択肢を返します。
  def self.part_template_options(position, project_id, terminal = nil)
    position_cond = CmsTemplate::SearchCondition.new(:cms_project_id => project_id, :display_position => position, :terminal => terminal)
    find(:all, :include => :cms_project, :conditions => position_cond.create_condition_for_page, :order => 'name').map{|template| [template.name, template.id]}
  end

  # デフォルトテンプレート以外でページ依存のない場合のテンプレートのファイル名を返します。
  #
  # デフォルトテンプレート以外のビューのファイル名の管理はCmsBindを中心に行いますので、
  # ページ依存がある場合(CmsBindのitemsがemptyじゃない場合)は
  # CmsBind#published_filenameを使用してください。
  def published_filename(locale = :ja)
    if (CmsTemplateContent.carrier_ids.include?(locale.to_s))
      locale_str = (locale.to_s == 'docomo') ? '_mobile' : '_mobile_' << locale.to_s
    else
      locale_str = (locale.to_s == 'ja') ? nil : '_' << locale.to_s.gsub(/\-/, '_')
    end
    "cms/_#{display_position_key}_#{self.template_code}#{locale_str}.html.erb"
  end

  # デフォルトテンプレートとしてのファイル名を返します。
  def published_default_template_filename(position_key, locale = :ja)
    if (CmsTemplateContent.carrier_ids.include?(locale.to_s))
      locale_str = (locale.to_s == 'docomo') ? '_mobile' : '_mobile_' << locale.to_s
    else
      locale_str = (locale.to_s == 'ja') ? nil : '_' << locale.to_s.gsub(/\-/, '_')
    end
    "cms/_default_#{position_key}#{locale_str}.html.erb"
  end

  # デフォルトテンプレートのファイルを生成します。
  def overwrite(position_key, target_key)
    case target_key
      when :production
        root_dir = EcRider.config.ec_front_root
      when :test
        root_dir = EcRider.config.ec_test_root
      else
        root_dir = EcRider.config.ec_admin_root
    end
    
    if self.terminal_key == :mobile
      locales = CmsTemplateContent.carrier_ids
    else
      locales = I18n.supported_locales
    end
    locales.each do |locale|
      filename = published_default_template_filename(position_key, locale.to_s)
      path = File.join(root_dir, 'app/views', filename)
      dir = File.dirname(path)
      FileUtils.mkdir_p(dir) unless File.exist?(dir)
      template_content = send("content_#{locale.to_s}")
      if template_content
        open(path, 'w') do |f|
          descriptions = get_descriptions(template_content).dup
          f.write CmsXml.generate_erb(descriptions)
        end
      end

    end
  end
  
  class << self
    # TODO モバイル未対応。CmsDeadLinkCheckで使用している。
    def default_template_filename_map(options = {})
      templates = primary_default_templates(options)[:pc]
      result = {}
      templates.each do |position, template|
        I18n.supported_locales.each do |locale|
          result[template.published_default_template_filename(position, locale.to_s)] = [template, locale.to_s]
        end
      end
      result
    end

    def primary_default_templates(options = {})
      preferred_project_id = options.delete(:preferred_project_id)
      options = {
        :find_options => {}
       }.update(options)
      options[:projects] ||= CmsProject.find_all_to_be_deployed_or_deleted
      cms_project_ids = options[:projects].map(&:id) + [preferred_project_id]

      default_template_codes = CmsDefaultTemplate.primary_default_template_codes(cms_project_ids, preferred_project_id)

      query = <<-"EOS"
        SELECT cms_templates.*
        FROM cms_templates
        JOIN cms_projects ON cms_projects.id = cms_templates.cms_project_id AND cms_projects.id IN (?)
        WHERE NOT EXISTS (
          SELECT *
          FROM cms_templates st
          JOIN cms_projects sp ON sp.id = st.cms_project_id AND sp.id IN (?)
          WHERE (
            sp.priority > cms_projects.priority
            OR (sp.priority = cms_projects.priority AND sp.started_at > cms_projects.started_at)
            OR (sp.priority = cms_projects.priority AND sp.started_at = cms_projects.started_at AND sp.id > cms_projects.id)
          ) AND st.template_code = cms_templates.template_code
        ) AND cms_templates.template_code IN (?)
      EOS
      templates = CmsTemplate.find_by_sql([query, cms_project_ids, cms_project_ids, default_template_codes.values])
      result = {:pc => {}, :mobile => {}}
      templates.each do |template|
        result[template.terminal_key][default_template_codes.position_key(template.template_code, template.terminal_key)] = template
      end
      # 優先されるプロジェクト（preferred_project_id）で編集したテンプレートがあればそれを使う
      if preferred_project_id
        preferred_templates = CmsTemplate.find(:all, :conditions => ["cms_project_id = ? AND template_code IN (?)", preferred_project_id, default_template_codes.values])
        preferred_templates.each do |template|
          result[template.terminal_key][default_template_codes.position_key(template.template_code, template.terminal_key)] = template
        end
      end
      result
    end
    
  end
end
