# -*- coding: utf-8 -*-
#
# Order
#  amount（メソッド）  | 注文の請求金額(order_shipping.total_amount + order_shipping.shipping_cost)
#  -                   | 割引前の合計金額(sum(order_item.subtotal_amount)) = 輸出用invoiceに記載する総額
# 
# OrderShipping
#  total_amount        | 売上金額 = 送料、代引き手数料を含まない注文金額(subtotal_amount - discount_amount)
#  subtotal_amount     | 注文明細の合計（sum(order_item.total_amount))
#  discount_amount     | 注文に対する割引金額
#  shipping_cost       | 送料＋代引き手数料
#  cod_charge          | 代引き手数料
#  -                   | 送料 = order_shipping.shipping_cost - order_shipping.cod_charge
#
# OrderItem
#  unit_price          | 商品単価
#  qty                 | 数量
#  shipping_cost       | 送料（STEP2では使わない）
#  subtotal_amount     | 割引前の合計金額(order_item.unit_price * order_item.qty) = 輸出用invoiceに記載する商品明細の合計金額
#  discount_amount     | 割引金額
#  total_amount        | 割引適用後の合計金額（order_item.subtotal_amount - order_item.discount_amount)
#
require 'digest/sha1'

class Order < ActiveRecord::Base
  unloadable
  include EcRider::PluginModule::Extendable

  versioned

  include EcRider::I18nSupport

  acts_as_paranoid

  has_many   :order_shippings, :class_name => 'OrderShipping', :foreign_key => 'order_id', :dependent => :destroy

  belongs_to :user

  belongs_to :include_order, :class_name => 'Order', :foreign_key => 'include_order_id'
  has_many   :included_orders, :class_name => 'Order', :foreign_key => 'include_order_id'

  has_many :mail_deliveries, :class_name => 'MailDelivery', :foreign_key => 'order_id'
  has_many :user_points, :class_name => 'UserPoint', :foreign_key => 'order_id'
  has_many :accountings, :class_name => 'Accounting', :foreign_key => 'order_id'
  has_many :order_campaigns

  # 注文コードの採番
  after_create :generate_ec_order_code

  before_validation :set_test_flg

  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  # クレジット実行区分
  selectable_attr :on_line_payment_exe_div do
    entry 1, :not_need, 'オンライン決済不要'
    entry 2, :not_exe, '未実施'
    entry 3, :error, 'オーソリエラー'
    entry 4, :success, 'オーソリ完了'
  end

  # 購入サイト区分
  selectable_attr :site_type_div do
    entry '1', :pc, 'PC'
    entry '2', :mobile, 'モバイル'
  end

  # キャンペーン対象日時を取得する
  def campaign_attached_at_or_ordered_at
    campaign_attached_at || ordered_at
  end

  # 引数に渡された検索条件で検索します
  def self.find_any_condition(condition, limit = nil, offset = nil)
    if condition.is_a?(Hash)
      condition = condition.reject{|key, val| val.empty?}
      condition = SearchInfo.new(condition)
    end
    find(:all, :include => SearchInfo.create_include, :conditions => condition.create_condition,
      :limit => limit, :offset => offset, :order => 'orders.id DESC')
  end

  # 渡された検索条件にHITする注文の合計金額を返す
  def self.total_amount_and_shipping_cost(condition)
    condition = condition.reject{|key, val| val.empty?}
    condition = SearchInfo.new(condition) if condition.is_a?(Hash)
    
    condition_and_params = condition.create_condition
    query = <<-EOS
      SELECT sum(total_amount) AS total_amount, sum(shipping_cost) AS shipping_cost FROM(
        SELECT DISTINCT order_shippings.id, order_shippings.total_amount, order_shippings.shipping_cost
        FROM orders
        #{SearchInfo.create_join(true)}
        WHERE #{condition_and_params.shift}
      ) subquery
    EOS
    result = Order.find_by_sql([query]+condition_and_params).first
    {:total_amount => result[:total_amount].to_i, :shipping_cost => result[:shipping_cost].to_i}
  end

  # 適用されているキャンペーンの一覧を返します
  def applied_campaigns
    result = []
    order_shippings.each do |order_shipping|
      result << order_shipping.order_items.map{|item| item.order_item_campaigns }
    end
    result.flatten.uniq.sort_by{|v| v.id} + order_campaigns.sort_by{|v| v.id}
  end

  #
  # 支払い方法がクレジットの場合に、OrderShippingを配列に入れて返す
  #
  def credit_order_shippings
    order_shipping = self.order_shippings.first #1注文に配送先は一つ
    order_shipping.payment_type_div_key == :credit ? [order_shipping] : []
  end

  #
  # = クレジット決済処理実行 =
  #
  #　事後条件：クレジット支払の履歴オブジェクトを返す。
  #  クレジット決済プラグインで拡張されています。なにも拡張していない場合は例外を発生させています。
  def credit_card_payment(order_form)
    raise ArgumentError, "指定された決済システムサービスは、ご利用できません。"
  end

  #
  # = コンビニ決済処理実行 =
  #
  def cvs_payment(credit_card, user)
    # 処理はプラグイン側に記述されています。
  end

  #
  # = PayPal決済処理実行 =
  #
  def paypal_payment(token, payer_id, amt)
    # 処理はプラグイン側に記述されています。
  end

  # 注文の請求金額(order_shipping.total_amount + order_shipping.shipping_cost)
  def amount
    total_amount = 0
    shipping_cost = 0
    self.order_shippings.each do |shipping|
      total_amount  += shipping.total_amount
      shipping_cost += shipping.shipping_cost
    end
    total_amount + shipping_cost
  end

  # 決済処理例外
  # 決済システムの処理結果がNGの際に発生する
  class SettlementError < StandardError
    attr_accessor :error_payment # 例外を起こした決済オブジェクト。create_payment_error_historyメソッドで例外の記録をDBにinsertします
  end


  # 定期購入商品が含まれる場合はtrueを返す
  def has_regurar_order?
    self.order_shippings.each do |order_shipping|
      order_shipping.order_items.each do |order_item|
        return true if order_item.course_div_key == :periodic
      end
    end
    return false
  end

  #
  # = キャンセル可能かどうかを判断する =
  #
  # :possible（キャンセル可能）
  # :impossible（キャンセル不可）
  # :cancel_request（キャンセル依頼中）
  # :canceled（キャンセル完了）
  #
  def cancel_status
    canceled = false
    cancel_request = false
    impossible = false
    # 複数お届け先のうち、すべてが未発送、または保留ならばキャンセル可能
    self.order_shippings.each do |order_shipping|
      unless order_shipping.status_div_entry[:cancel_requestable]
        impossible = true
      end
    end
    return :canceled if canceled
    return :cancel_request if cancel_request
    return :impossible if impossible
    return :possible
  end

  #
  # 値をセットしたOrderインスタンスを返す
  #
  def self.create_new_order(user, user_address, order_form, cart, site_type_div = :pc)
    order = Order.new(
      :ordered_at => Time.now,
      :site_type_div_key => site_type_div,
      :free_request => order_form.free_request,
      :locale => I18n.locale.to_s,
      :gift_flg => order_form.gift_flg,
      :send_invoice_separately_flg => order_form.send_invoice_separately_flg,
      :write_sender_flg => order_form.write_sender_flg,
      :include_order_id => order_form.include_order_id,
      :campaign_attached_at => cart.campaign_attached_at || Time.now
    )

    unless cart.empty?
      # OrderCampaignインスタンスを生成する
      cart.applied_campaigns.each do |campaign|
        order.order_campaigns << OrderCampaign.new_by_campaign(campaign)
      end
      # OrderShippingインスタンスを生成する
      order.order_shippings << create_new_order_shipping(cart, user, user_address, order_form, site_type_div)
    end
    return order
  end

  #
  # 値をセットしたOrderShippingのインスタンスを返す
  #
  def self.create_new_order_shipping(cart, user, user_address, order_form, site_type_div = :pc)
    shipping_cost = ShippingCost.calc_shipping_cost(
      user_address,
      cart.total_shipping_cost_point,
      cart.free_shipping_cost?)
    cod_charge = 0
    if order_form.normal_payment_type == OrderShipping.payment_type_div_id_by_key(:cod)
      #支払い方法が代引きの場合、手数料を加算する
      cod_charge = (ShopSetting.get_setting(:cod_charge, I18n.locale) || 0).to_i
      shipping_cost = shipping_cost + cod_charge
    end
    order_shipping = OrderShipping.new({
        :total_amount    => cart.total_amount, # 割引適用後の合計金額
        :subtotal_amount => cart.subtotal_amount, # 割引適用前の合計金額(明細の総合計)
        :discount_amount => cart.discount_amount, # 割引額
        :shipping_cost => shipping_cost,
        :payment_type_div => order_form.normal_payment_type,
        :status_div_key => :before_settlement,
        :csv_output_div_key => :not_yet,
        :cod_charge => cod_charge,
        :arrival_date => order_form.delivery_date,
        :arrival_hour => order_form.delivery_time
      })
    order_shipping.merge_only_need(user_address.attributes)
    if site_type_div == :pc
      order_shipping.merge_only_need(user.user_email.attributes)
    else
      order_shipping.merge_only_need(user.user_email_mobile.attributes)
    end

    # OrderItemインスタンスを生成する
    cart.item_list.map do |cart_item|
      order_item = OrderItem.new(
        :product_name => cart_item.name,
        :order_product_name_1 => cart_item.product_sku.product_name_for_admin_ja,
        :order_product_name_2 => cart_item.product_sku.product.order_product_name_2,
        :course_div => cart_item.course_div,
        :unit_price => cart_item.unit_price,
        :qty => cart_item.qty,
        :subtotal_amount => cart_item.subtotal_amount,
        :discount_amount => cart_item.discount_amount,
        :total_amount    => cart_item.total_amount,
        :shipping_cost => 0,
        :product_sku_id => cart_item.product_sku.id
      )

      # OrderItemComponentインスタンスを生成する
      product_sku = ProductSku.find(order_item.product_sku_id)
      product_sku.components.each do |component|
        oic = OrderItemComponent.new({
            :order_item_id => order_item.id,
            :component_sku_id => component.component_sku_id, # 子商品SKUのID
            :qty => component.count * order_item.qty, # 子商品SKUの総数量
        })
        # 商品名はOrderItemと同等のものを設定
        oic.product_name = oic.product_sku.product.product_name_1
        oic.order_product_name_1 = oic.product_sku.product_name_for_admin_ja
        oic.order_product_name_2 = oic.product_sku.product.order_product_name_2
        order_item.order_item_components << oic
      end

      # OrderItemCampaignインスタンスを生成する
      cart_item.applied_campaigns.each do |campaign|
        order_item.order_item_campaigns << OrderItemCampaign.new_by_campaign(campaign)
      end

      order_shipping.order_items << order_item
    end

    return order_shipping
  end

  # 紐付くキャンペーン情報を更新します
  def update_campaigns(cart)
    # 注文に紐付くキャンペーンを更新する
    OrderCampaign.destroy_all(:order_id => self.id)
    cart.applied_campaigns.each do |campaign|
      order_campaign = OrderCampaign.new_by_campaign(campaign)
      order_campaign.order_id = self.id
      order_campaign.save!
    end

    # 商品に紐付くキャンペーンを商品SKUをキーにしたハッシュにする
    cart_item_hash = {}
    cart.item_list.each do |cart_item|
      cart_item_hash[cart_item.product_sku.id] ||= []
      cart_item.applied_campaigns.each do |campaign|
        cart_item_hash[cart_item.product_sku.id] << campaign
      end
    end

    # 商品に紐付くキャンペーンを更新する
    self.order_shippings.each do |order_shipping|
      order_shipping.order_items.each do |order_item|
        OrderItemCampaign.destroy_all(:order_item_id => order_item.id)
        cart_item_hash[order_item.product_sku_id].each do |campaign|
          order_item_campaign = OrderItemCampaign.new_by_campaign(campaign)
          order_item_campaign.order_item_id = order_item.id
          order_item_campaign.save!
        end
      end
    end
  end

  #
  # 注文コードを生成
  #
  def generate_ec_order_code
    self.ec_order_code = self.id.to_s.rjust(11, '0')
    self.save!
  end

  #
  # =注文情報詳細をテキストでフォーマットして返します=
  #
  def print_order_detail(type = :text, options = {})
    result = []
    total_info = self.total_info

    case self.locale
    when 'ja'
      print_locale = 'ja'
    when 'zh_CN'
      print_locale = 'zh_CN'
    else
      print_locale = 'en'
    end
    # 店舗宛出荷完了メールは日本語固定のためロケールを日本語に上書きします。
    unless options[:shipment_complete_mail].nil?
      print_locale = 'ja'
    end

    i18n_tmp_block(print_locale) do
      # http://api.rubyonrails.org/classes/I18n.html#M002268
      #
      # Scope can be either a single key, a dot-separated key or an array of keys or dot-separated keys.
      # Keys and scopes can be combined freely. So these examples will all look up the same short date format:
      # I18n.t 'date.formats.short'
      # I18n.t 'formats.short', :scope => 'date'
      # I18n.t 'short', :scope => 'date.formats'
      # I18n.t 'short', :scope => %w(date formats)
      #
      # TODO ここどっちだろう？ orderitems.print_order_detail かも
      I18n.with_options(:scope => 'orders.print_order_detail') do |i18n|
        yen = i18n.t('yen', :default => '')
        pre_name = i18n.t('pre_name', :default => '')
        pre_name = pre_name.blank? ? '' : (pre_name + ' ')
        post_name = i18n.t('post_name', :default => '')
        post_name = post_name.blank? ? '' : (' ' + post_name)
        order_shipping_subtotal_amount = 0
        order_shipping_discount_amount = 0
          
        result << "[#{i18n.t('ec_order_code')}] #{self.ec_order_code}"
        result << "================================================================"
        self.order_shippings.each do |order_shipping|
          order_shipping.order_items.each do |order_item|
            case print_locale
            when 'ja'
              print_product_name = order_item.order_product_name_1
            when 'zh_CN'
              product_display = Product.find(:first, :conditions => ["product_code = ? and locale = ?", order_item.product_sku.product.product_code, print_locale],
                :joins =>"inner join product_displays on products.id = product_displays.product_id")
              print_product_name = (product_display.nil?) ? nil : product_display.product_name_1
              print_product_name ||= order_item.order_product_name_2
            else
              print_product_name = order_item.order_product_name_2
            end
            result << "[#{order_item.product_sku.sku_code}] #{print_product_name}"
            order_item_discount = (order_item.discount_amount && order_item.discount_amount > 0) ? order_item.discount_amount : nil
            result << "      #{order_item.unit_price ? order_item.unit_price : 0}#{yen} x #{order_item.qty}#{ ' - %d' % order_item_discount + yen if order_item_discount } = #{order_item.total_amount}#{yen}"
          end
          result << ''
          result << "[#{i18n.t('order_code')}] No.#{order_shipping.order_code}"
          result << "[#{i18n.t('address')}] #{pre_name}#{order_shipping.fullname}#{post_name}"
          result << "  #{i18n.t('zip_code')}#{order_shipping.zip_code}"
          if order_shipping.is_japanese_format?
            result << "  #{order_shipping.address_1}#{order_shipping.address_2}#{order_shipping.address_3}"
          elsif order_shipping.is_chinese_zh_cn_format?
            result << "  #{order_shipping.address_1}#{order_shipping.fulladdress}, #{order_shipping.country.name}"
          else
            result << "  #{order_shipping.fulladdress}, #{order_shipping.address_1}, #{order_shipping.country.name}"
          end
          result << "  (#{i18n.t('phone_number')})#{order_shipping.tel_num}"
          result << "[#{i18n.t('payment_type')}] #{order_shipping.payment_type_div_name}"
          result << "[#{i18n.t('arrival_date')}] #{order_shipping.arrival_date.strftime('%Y/%m/%d') if order_shipping.arrival_date} #{order_shipping.arrival_time_display(print_locale)}"
          result << "================================================================"
          order_shipping_subtotal_amount += order_shipping.subtotal_amount if order_shipping.subtotal_amount
          order_shipping_discount_amount += order_shipping.discount_amount if order_shipping.discount_amount
        end
        result << "#{i18n.t('subtotal')} #{sprintf('%26s',order_shipping_subtotal_amount)}#{yen}"
        result << "#{i18n.t('discount')} #{sprintf('%26s',order_shipping_discount_amount)}#{yen}" if order_shipping_discount_amount > 0
        result << "#{i18n.t('carriage')} #{sprintf('%19s',total_info[:carriage])}#{yen}"
        result << "----------------------------------------------------------------"
        result << "#{i18n.t('total')} #{sprintf('%26s',total_info[:total])}#{yen}"
        result << "----------------------------------------------------------------"
      end
    end
    result << ''
    result.join("\n")
  end

  #
  # = EC注文番号に紐付く商品の小計・送料・合計・個数・送付先数・請求金額を返す =
  #
  # 入力：EC注文見出しID
  # 出力：:subtotal => 小計、:carriage => 送料&手数料、:total => 合計、:count => 個数、:address_count => 送付先数, :due => 請求金額
  #
  def total_info
    result = {:total => 0, :subtotal => 0, :carriage => 0, :count => 0, :address_count => 0, :due => 0}

    self.order_shippings.each do |order_shipping|
      order_shipping.order_items.each do |order_item|
        result[:count] += order_item.qty.to_i
      end
      result[:subtotal] += order_shipping.total_amount
      result[:carriage] += order_shipping.shipping_cost
      unless order_shipping.payment_type_div_entry[:pay_in_advance]
        result[:due] += order_shipping.total_amount + order_shipping.shipping_cost
      end
    end
    result[:total] = result[:subtotal] + result[:carriage]
    result[:address_count] = self.order_shippings.size

    result
  end

  # =販売期間をチェックする=
  # 販売期間を超える注文が含まれる場合は、OrderItem::NunPeriodicSalesErrorが発生します
  # オフライン販売(管理画面での注文)時には、チェックしません。
  def check_sales_period
    return true if EcRider.config.ec_admin
    self.order_shippings.each do |order_shipping|
      current_order_items = order_shipping.order_items.dup
      current_order_items.each do |order_item|
        order_item.check_sales_period
      end
    end
  end

  #
  # =在庫をチェックする=
  # 在庫数を超える注文が含まれる場合はOrder::OutOfStockErrorが発生します
  #
  def check_stocks
    self.order_shippings.each do |order_shipping|
      current_order_items = order_shipping.order_items.dup
      current_order_items.each do |order_item|
        order_item.check_stocks
      end
    end
  end

  #
  # =在庫を仮引き当て、引き当て済みにする=
  # 引数(mode)  :paidを指定すると引き当て済みにします。指定しないと仮引き当てにします。
  # 在庫数を超える注文が含まれる場合はOrder::OutOfStockErrorが発生します
  #
  def secure_stocks(mode = :paid)
    raise OrderItem::SecureStockError, 'cannot secure_stocks for new_record' if new_record?
    self.order_shippings.each do |order_shipping|
      current_order_items = order_shipping.order_items.dup
      current_order_items.each do |order_item|
        order_item.secure_stocks(mode)
      end
    end
  end

  #
  # =在庫を開放する=
  #
  def unsecure_stocks
    self.order_shippings.each do |order_shipping|
      current_order_items = order_shipping.order_items.dup
      current_order_items.each do |order_item|
        order_item.unsecure_stocks
      end
    end
  end

  #
  # =ポイントを作成する=
  # 
  # 発行ポイント及び利用ポイントを作成します。
  # 
  def create_user_point(given_point = 0, used_point = 0)
    point_setting = PointSetting.find(:first)
    if point_setting.available_flg_true?
      # 発行ポイントがある場合付与ポイントを発行
      unless given_point == 0
        user_point = UserPoint.new({:user_id => user_id, :order_id => id,
            :term_date => point_setting.create_term_date(Date.today), 
            :given_point => given_point, :used_point => used_point, :inactive_flg => true
          })
        user_point.save!
        # 有効期限がポイント発行最終付与時の場合、その会員の保有ポイントの有効期限をすべて付け替える
        UserPoint.term_date_update_for_last_point(user_id, user_point.term_date)
      end
    end
  end

  #
  #  # =仮ポイント付与、ポイント付与にする=
  #  TODO ポイント機能リリース時に利用します。
  #  # 引数(mode)  :paidを指定すると発行ポイントにします。指定しないと仮ポイントにします。
  #  def secure_point(mode = :paid)
  #     user_point = self.user_points.clone
  #     user_point.paid_flg = mode
  #     user_point.updated_at = Date.new
  #     user_point.save!
  #  end

  #
  # =ポイントを削除する=
  #
  # この注文に関連するポイントを全て削除します
  #  TODO ポイント機能リリース時に変更が必要です。
  # #仮ポイント削除時にも利用するよう変更します。
  def delete_point
    self.user_points.each {|point| point.delete}
  end
  
  # 注文に紐付くポイントの合計（獲得ポイント - 利用ポイント）を返す
  def total_point
    given_point - used_point
  end
  
  # 注文に紐付く獲得ポイントを返す
  def given_point
    if self.user_points.nil? || self.user_points.any? {|point| point.given_point < 0}
      # 有効期限切れのポイントを含む場合は0を返す
      return 0
    else
      self.user_points.inject(0) {|result, point| result + point.given_point}
    end
  end
  
  # 注文に紐付く利用ポイントを返す
  def used_point
    if self.user_points.nil? || self.user_points.any? {|point| point.given_point < 0}
      # 有効期限切れのポイントを含む場合は0を返す
      return 0
    else
      self.user_points.inject(0) {|result, point| result + point.used_point}
    end
  end

  #
  # テストデータかどうかを返す
  #
  def test?
    test_flg ? test_flg : false
  end

  #
  # =同送であり、親であるか判定する=
  #
  #親でない場合はnilを返す
  #
  def includable_shipping_parent?
    if self.include_order_id.nil? # 親か同送でない
      include_order = Order.find_by_include_order_id(self.id)
      unless include_order.nil?
        return true
      end
    end
    return nil
  end

  #
  # =ステータス更新=
  #
  def update_status(new_status_div_key = nil)
    self.order_shippings.each do |order_shipping|
      order_shipping.update_status(new_status_div_key)
    end
  end
  
  # 指定した年月に受け付けた注文のうち、配送済み以降のものの件数と注文金額合計を返す
  def self.count_at_term(start_date, end_date)
    statuses = [OrderShipping.status_div_id_by_key(:delivered), OrderShipping.status_div_id_by_key(:deliver_advised)]
    where = <<-"EOS"
      (test_flg IS NULL OR test_flg = 0) 
      AND orders.ordered_at BETWEEN :start_date AND :end_date
      AND order_shippings.status_div IN (:statuses)
    EOS
    conditions = [where, {:start_date => start_date, :end_date => end_date, :statuses => statuses}]
    join_order_shippings = "JOIN order_shippings ON orders.id = order_shippings.order_id AND order_shippings.deleted_at IS NULL"
    select = 'SUM(order_shippings.total_amount) AS amount, COUNT(orders.id) AS count'
    result = Order.find(:first, :select => select, :joins => join_order_shippings, :conditions => conditions)
    return {:count => result[:count].to_i, :amount => result[:amount].to_i}
  end
  
  # 決済オーダーIDを返す 決済プラグインで拡張しています
  def payment_order_id
  end
end
