# -*- coding: utf-8 -*-
class PointSetting < ActiveRecord::Base

  # ポイント制度利用区分
  selectable_attr :available_flg do
    entry false, :false, '使用しない'
    entry true,  :true, '使用する'
  end

  # 有効期限有無区分
  selectable_attr :terminable_flg do
    entry false, :false, 'なし'
    entry true,  :true, 'あり'
  end

  # 有効期限基準日区分
  selectable_attr :term_div do
    entry '0',  :last_point, '最終ポイント発行日'
    entry '99', :manual, '日付指定'
  end

  # 有効期間区分
  selectable_attr :point_period_div do
    entry '1',  1, '1ヶ月'
    entry '6',  6, '6ヶ月'
    entry '12', 12, '12ヶ月'
    entry '24', 24, '24ヶ月'
  end

  # ポイント有効期限を返却する
  def create_term_date(today)
    # ポイント制度利用区分 なしなら何もしない
    # 有効期限 なし なら何もしない
    return nil unless self.available_flg
    return nil unless self.terminable_flg
    case self.term_div_key
    when :last_point
      # 最終ポイント発行日の場合 一番新しく発行されたポイントの発行日から有効期間後を有効期限とする
      today = today >> self.point_period_div_key
      return today.end_of_day
    when :manual
      # 日付指定の場合
      term_day = Date.new(today.year, self.term_month, self.term_day)
      return term_day.end_of_day if term_day > today
      # 本日より有効期限が小さい場合は
      # 来年を設定
      term_day = term_day >> 12
      return term_day.end_of_day
    end
  end


  #未入力バリデーション
  validates_presence_of :term_div, :if => :available_flg_true?
  validates_presence_of :point_period_div, :if => :available_flg_true?
  validates_presence_of :term_month, :if => :term_div_checked_manual?
  validates_presence_of :term_day, :if => :term_div_checked_manual?

  # 数値バリデーション
  validates_numericality_of :issue_price, :exchange_rate, :only_integer =>true, :message => :EM990003, :if => :available_flg_true?
  validates_numericality_of :exchangeable_price, :only_integer =>true, :message => :EM990003, :if => :exchangeable_price_checked?
  validates_numericality_of :exchangeable_point, :only_integer =>true, :message => :EM990003, :if => :exchangeable_point_checked?

  #文字形式バリデーション
  validates_inclusion_of   :term_month,  :in => 1..12, :message =>:EM990007, :if => :term_div_checked_manual?
  validates_inclusion_of   :term_day,    :in => 1..31, :message =>:EM990007, :if => :term_div_checked_manual?

  # ポイント更新メソッド
  # 有効期限を設定している場合、全会員の有効期限切れでないポイントを新しい有効期限に付け替えます。
  def update_and_all_user_point_term_date
    return unless self.changed?
    new_term_date = create_term_date(Date.today)
    if new_term_date.nil?
      # ここが実行されるときは有効期限を使用しない設定となっているため、有効期限を削除します。
      user_points = UserPoint.find(:all, :conditions =>["expired_flg = ?", false])
      user_points.each do |user_point|
        user_point.term_date = nil
        user_point.save!
      end
    else
      if term_div_checked_manual?
        # 日付指定の場合
        # 有効期限切れのポイントを一律新有効期限に更新する
        user_points = UserPoint.find(:all, :conditions =>["expired_flg = ?", false])
        user_points.each do |user_point|
          user_point.term_date = new_term_date
          user_point.save!
        end
      else
        # 最終ポイント付与時の場合
        # 会員ごとにポイントを取得し、その会員の最終発行日から有効期限を設定する
        # すでに有効期限切れのポイントは有効期限を変更しない。
        last_points_every_users = UserPoint.find_by_sql("SELECT user_id, MAX(user_points.created_at) as last_created_at FROM user_points where expired_flg = 0 GROUP BY user_id")
        last_points_every_users.each do |last_points_every_user|
          user_id = last_points_every_user.user_id
          last_created_at = last_points_every_user.last_created_at
          user_points = UserPoint.find(:all, :conditions => ["user_id =? AND (expired_flg is null OR expired_flg = ?)", user_id, false])
          user_points.each do |user_point|
            date = Date.strptime(last_created_at, "%Y-%m-%d %H:%M:%S")
            user_point.term_date = create_term_date(date)
            user_point.save!
          end
        end
        # 有効期限切れポイントになった場合、有効期限切れポイントを発行
        UserPoint.update_expired
      end
    end
    self.save!
  end

  # ポイント制度が「使用する」の場合trueを返す
  def available_flg_true?
    return false if self.nil?
    self.available_flg_key == :true
  end

  # 有効期限が「あり」の場合trueを返す
  def terminable_flg_true?
    return false if self.nil?
    self.terminable_flg_key == :true
  end

  # 有効期限基準日が「日付指定」かどうかを返します。
  def term_div_checked_manual?
    available_flg_true? && terminable_flg_true? && self.term_div_key == :manual
  end

  # チェックがついている場合にtrueを返す
  def exchangeable_price_checked?
    available_flg_true? && self.exchangeable_price_flg
  end

  # チェックがついている場合にtrueを返す
  def exchangeable_point_checked?
    available_flg_true? && self.exchangeable_price_flg
  end

  # 注文金額からポイント利用が可能かどうかを返却します。
  # ポイント利用ができる場合trueを返却します。
  # exchangeable_price_flg,exchangeable_point_flgがnilの場合はポイント利用可能の仕様にしています。
  def self.exchangeable(user_id, amount)
    point_setting = self.find(:first)
    if point_setting.exchangeable_price_flg && amount.to_i < point_setting.exchangeable_price
      return false
    end
    if point_setting.exchangeable_point_flg
      total_point = UserPoint.total_point_for_user(user_id)
      return false if total_point < point_setting.exchangeable_point
    end
    return true
  end
end
