# -*- coding: utf-8 -*-
require 'date'

class Product < ActiveRecord::Base
  include EcRider::CloudFront

  acts_as_paranoid
  versioned

  belongs_to :country, :foreign_key => 'origin'
  has_many :product_displays, :order => 'locale asc', :dependent => :destroy
  has_many :skus, :class_name => "ProductSku", :dependent => :destroy, :foreign_key => "product_id"
  has_many :images, :class_name => "ProductImage", :foreign_key => "product_id", :order => "display_order"
  has_one :main_image, :class_name => "ProductImage", :foreign_key => "product_id", :conditions => "main_flg = 1"

  has_many :inquiry_product, :class_name => 'InquiryProduct', :foreign_key => 'product_id'
  has_many :product_faq_relation_product, :class_name => 'ProductFaqRelationProduct', :foreign_key => 'product_id'
  has_one :primary_sku, :class_name => 'ProductSku', :conditions => ["primary_sku_flg = 1"]

  named_scope :with_displays do |*args|
    locale = (args.shift || I18n.locale || 'ja').to_s
    { :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
      :conditions => ['cond_product_displays.locale = ?', locale]}
  end

  # 表示する各言語用の商品名などを取得するためにProductDisplayを保持する属性
  attr_writer :current_display
  def current_display
    @current_display ||= self.product_displays.find(:first,
      :conditions => ["locale = ?", (I18n.locale || 'ja').to_s])
  end

  display_attrs = ["locale",
    "product_name_1", "product_name_2", "product_outline",
    "product_explanation", "normal_explanation",
    "regular_explanation", "recommendation", "recommendation_2", "product_keyword"]

  display_attrs.each do |attr|
    self.module_eval <<-EOS
      def #{attr}; current_display ? current_display.#{attr} : nil; end
    EOS
  end

  validates_presence_of :product_code, :course_div, :order_product_name_1,  :order_product_name_2, :origin,  :message => :EM990001

  validates_length_of   :kana_product_name, :maximum=>120, :allow_nil => true
  validates_length_of   :detail_url, :maximum=>255, :allow_nil => true
  validates_length_of   :product_code, :maximum=>13
  
  validates_format_of   :product_code, :with => /^[0-9a-zA-Z]*$/ , :message => :EM990025

  validates_uniqueness_of :product_code,:message => :EM990027

  attr_accessor :shipping_cost_price, :discount_price, :normal_price, :unit_price, :regular_price, :product_discount_id

  before_destroy :error_if_unable_to_delete_product

  # コース区分
  # 使用されてないはずですが、constの見直しに当たって一応宣言するようにしました。
  selectable_attr :course_div, :enum => OrderItem.course_div_enum

  # 商品SKUが1つでも登録されていれば、trueを返却します
  def has_sku?
    self.skus.size > 0
  end

  #
  # = CloudFrontへアップロードする商品画像のパスを返却します。 =
  #
  def image_sources
    sources = []
    self.images.each do |image|
      sources << image.public_filename()
      sources << image.public_filename(:thumb)
      sources << image.public_filename(:large)
      sources << image.public_filename(:medium)
      sources << image.public_filename(:small)
    end
    return sources
  end

  #
  # = 商品SKU一覧に表示する情報を取得します =
  #
  def product_skus
    # 商品バリエーション
    variation_1_items = self.variation_1 ? self.variation_1.items : []
    variation_2_items = self.variation_2 ? self.variation_2.items : []
    variation_3_items = self.variation_3 ? self.variation_3.items : []

    # 商品SKUの一覧に使用するデータを作成
    @tmp_product_skus = []
    variation_1_items.each do |v1|
      if variation_2_items.empty?
        @tmp_product_skus << ProductSku.new(:variation_item_1_id => v1.id)
        next
      end
      variation_2_items.each do |v2|
        if variation_3_items.empty?
          @tmp_product_skus << ProductSku.new(:variation_item_1_id => v1.id,
            :variation_item_2_id => v2.id)
          next
        end
        variation_3_items.each do |v3|
          @tmp_product_skus << ProductSku.new(:variation_item_1_id => v1.id,
            :variation_item_2_id => v2.id,
            :variation_item_3_id => v3.id)
        end
      end
    end

    if @tmp_product_skus.size == 0
      @tmp_product_skus = [ProductSku.new(:variation_item_1_id => nil, :variation_item_2_id => nil, :variation_item_3_id => nil)]
    end

    # DBに登録済みの場合は、登録された商品SKUで置き換える
    db_data = {}
    self.skus.each do |product_sku|
      db_data[product_sku.sku_code] = product_sku
    end

    @product_skus = []
    @tmp_product_skus.each do |sku|
      sku_code = "#{self.product_code}-#{sku.branch_code}"
      if db_data.include?(sku_code)
        @product_skus << db_data[sku_code]
      else
        @product_skus << sku
      end
    end

    return @product_skus
  end

  # バリエーション1～3
  (1..3).each do |index|
    base_name = "variation_#{index}"
    belongs_to(base_name.to_sym, :class_name => 'ProductVariation', :foreign_key => "#{base_name}_id")

    class_eval(<<-"EOS", __FILE__, __LINE__)
      def #{base_name}_name_for_admin
        obj = #{base_name}
        obj ? obj.name_for_admin : nil
      end

      def #{base_name}_name(locale = I18n.locale)
        obj = #{base_name}
        obj ? obj.display_name(locale) : nil
      end
    EOS
  end

  # カテゴリ1～5
  (1..5).each do |index|
    base_name = "category_#{index}"
    belongs_to base_name.to_sym, :class_name => 'ProductCategory', :foreign_key => "#{base_name}_id"

    class_eval(<<-"EOS", __FILE__, __LINE__)
      def #{base_name}_name(locale = I18n.locale)
        obj = #{base_name}
        obj ? obj.display_name(locale) : nil
      end
    EOS
  end

  # 国名を返す
  def origin_name
    country ? country.name : nil
  end

  #
  # = 商品価格 =
  #
  # 代表SKUの会員ランクが一番低い会員の標準売価を返却する
  def price
    _price = 0
    self.skus.each do |sku|
      _price = sku.price if sku.primary_sku_flg
    end
    return _price
  end

  #
  # ロケールに対応するProductDisplayを返します。
  # 該当するものがない場合は引数forceが真ならば新しいインスタンスを、偽ならばnilを返します。デフォルトはfalseです。
  def display_for(locale, force = false)
    product_displays.detect{|d| d.locale == locale.to_s} || (force ? ProductDisplay.new(:product => self, :locale => locale) : nil)
  end

  #
  # = 商品一覧用リストを作成 =
  #
  def self.product_list(time_now = Time.now())
    Product.find(:all, :order => 'display_order, id')
  end

  #
  # = 商品画像に選択可能な画像のリストを返します =
  #
  def self.image_list
    ShopSetting.image_list()
  end

  #
  # = 商品名リストと商品コードのリストを取得します。 =
  #
  # 商品リストを{:code=>商品コード:name=>商品名,:locale=>言語}の配列で返します。
  #
  def self.find_product_list_name_and_code
    result = Product.connection.select_all(<<-EOS)
      select
        products.order_product_name_1,
        products.product_code,
        product_displays.locale
      from products
        inner join product_displays on product_displays.product_id = products.id
      order by
        products.display_order, products.product_code
    EOS
  end

  def detail_url_outside_flg_name
    if self.detail_url_outside_flg
      name = "外部ページを利用する"
    else
      name = "内部ページを利用する"
    end
    return name
  end

  # detail_urlの商品コードパラメータを除いた部分を返します
  def detail_url_action
    return self.detail_url[/\/[^\?]*/] if self.detail_url
  end
  attr_writer :detail_url_action
  
  def set_attributes_and_detail_url(hash)
    self.attributes = hash
    return unless hash[:detail_url_action]
    unless self.detail_url_outside_flg
      self.detail_url = "#{hash[:detail_url_action]}?product_code=#{self.product_code}"
    end
  end
  
  # 商品が削除可能かどうかを判定します
  def deletable_product?
    self.skus.each do |sku|
      # 代表SKUではない商品SKUの場合、商品構成が登録されていないかつ他の商品SKUの構成商品でなければ削除できる
      return false if sku.has_parent_sku?
    end
    return true
  end
  
  # 配送前の注文に含まれていたら削除できないようにバリデートします
  # destroy実行時に呼び出されます
  def error_if_unable_to_delete_product
    result = true
    # 発送前のものがなければfalseをセット
    if included_in_order_before_delivery?
      result = false
      errors.add_to_base("配送前の注文に含まれているため商品を削除できません。")
    end

    # 商品構成の子になっていなければfalseをセット
    if include_product_component?
      result = false
      errors.add_to_base("商品構成の子になっているため商品を削除できません。")
    end
    result
  end

  # 配送前の注文に含まれていたらtrueを返します
  def included_in_order_before_delivery?
    before_delivery_count = OrderItem.count(
      :joins => "inner join order_shippings on order_shippings.id = order_items.order_shipping_id",
      :conditions => [
        "order_items.product_sku_id in (?) AND order_shippings.status_div in (?)",
        self.skus.map{|sku| sku.id}, OrderShipping::STATUS_DIVS_BEFORE_DELIVERY
      ])
    before_delivery_count > 0
  end

  # 商品構成の子になっていたらtrueを返します
  def include_product_component?
    product_component_count = ProductComponent.count(
      :joins => "inner join product_skus ps on ps.id = product_components.product_sku_id and ps.deleted_at is null
                 inner join products p on ps.product_id = p.id and p.deleted_at is null",
      :conditions => ["product_components.component_sku_id in (?)", self.skus.map{|sku| sku.id}]
    )
    product_component_count > 0
  end

  # 指定した日時が販売期間内かどうかを判定します。
  def check_sales_period?(time = Time.now())
    (self.sale_started_at.nil? || self.sale_started_at.blank? || time >= self.sale_started_at) && 
      (self.sale_finished_at.nil? || self.sale_finished_at.blank? || time <= self.sale_finished_at)
  end
end
