# -*- coding: utf-8 -*-
class Product < ActiveRecord::Base

  #
  # ec_front 商品選択、商品カテゴリ検索で使用するフォーム
  # 商品取得の条件は
  # 指定したカテゴリコードより自カテゴリ含む子供のカテゴリを含む商品
  # かつ、公開期間中の商品
  # かつ、非表示フラグの立っていない商品
  # かつ、プレゼント商品ではない商品
  #
  class CategorySearchForm < EcRider::ConditionForm
    include SelectableAttr::Base
    attr_accessor :keyword, :condition_div, :render_name, :per_page, :category_code, :order, :user_rank_id, :category_codes, :image_display_div

    selectable_attr :condition_div do
      entry '0', :all, "すべてを含む"
      entry '1', :which, "いずれかを含む"
    end
    
    selectable_attr :order do
      entry '0', :default, "標準"
      entry '1', :low_to_high, "価格の安い順"
      entry '2', :high_to_low, "価格の高い順"
      entry '3', :new_to_old, "新着順"
    end

    selectable_attr :image_display_div do
      entry '0', :visible, "画像表示"
      entry '1', :invisible, "画像非表示"
    end

    def to_find_options(options = nil)
      # 検索条件作成
      query = []
      param = []

      # 検索条件に設定するカテゴリIDを取得
      category_ids = []
      # カテゴリ(単数)
      unless category_code.blank?
        # 選択されたカテゴリの子カテゴリも取得する
        product_category = ProductCategory.find(:first, :conditions => ["code = ?", category_code])
        category_ids = product_category.descendant_ids(true)
      end
      
      # カテゴリ(複数)
      unless category_codes.blank?
        # 選択されたカテゴリの子カテゴリも取得する
        category_codes.split(',').each do |code|
          product_category = ProductCategory.find(:first, :conditions => ["code = ?", code.strip])
          category_ids.concat(product_category.descendant_ids(true)) if product_category
        end
      end

      # カテゴリ
      unless category_ids.empty?
        sub_query = []
        (1..5).each do |index|
          sub_query << "products.category_#{index}_id IN (?)"
          param << category_ids
        end
        query << "(#{sub_query.join(' OR ')})"
      end

      # キーワード
      unless keyword.blank?
        search_target_columns = ['product_name_1', 'product_name_2', 'product_outline', 'product_explanation', 'recommendation', 'recommendation_2', 'product_keyword']
        sub_query = []
        keyword.split(' ').each do |key|
          sub_query_per_key = []
          search_target_columns.each do |column|
            sub_query_per_key << "product_displays.#{column} like ?"
            param << "%#{key.strip}%"
          end
          sub_query << "(#{sub_query_per_key.join(' OR ')})"
        end
        separator = (condition_div_key == :all) ? ' AND ' : ' OR '
        query << "(#{sub_query.join(separator)})"
      end

      # ユーザランクを取得
      if user_rank_id.blank?
        user_rank = UserRank.find(:first, :conditions=>['no_membership_flg = 1'])
        self.user_rank_id = user_rank.id unless user_rank.nil?
      end

      # 基本的な表示条件
      now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
      query << 'product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL'
      query << 'products.deleted_at IS NULL AND product_skus.deleted_at IS NULL'
      query << 'product_displays.locale = ?'
      param << "#{I18n.locale || 'ja'}"
      query << '(present_flg IS NULL OR present_flg = 0)'
      query << 'product_prices.user_rank_id = ?'
      param << user_rank_id.to_i
      unless EcRider.config.ec_admin
        query << "(publish_started_at IS NULL OR publish_started_at <= ?)"
        query << "(publish_finished_at IS NULL OR publish_finished_at >= ?)"
        param << now
        param << now
      end
      unless (EcRider.config.ec_test ||  EcRider.config.ec_admin)
        query << '(invisible_flg IS NULL OR invisible_flg = 0)'
      end

      #ソート条件
      #標準
      order_by = 'products.display_order is null, products.display_order, products.product_code'
      #追加条件
      case order_key
      when :low_to_high; order_by = "product_prices.standard_price, #{order_by}"
      when :high_to_low; order_by = "product_prices.standard_price desc, #{order_by}"
      when :new_to_old;  order_by = "products.publish_started_at desc, #{order_by}"
      end

      result = {:conditions => [query.join(' AND ')]+param, :order => order_by, :include => [:product_displays, :main_image, {:primary_sku => :prices}]}
      result.update(options) if options
      return result
    end
  end

end
