# -*- coding: utf-8 -*-
class ProductFaq < ActiveRecord::Base
  versioned

  has_many :displays, :class_name => 'ProductFaqDisplay',          :dependent => :destroy
  has_many :relation_products, :class_name => 'ProductFaqRelationProduct', :dependent => :destroy

  accepts_nested_attributes_for :displays
  accepts_nested_attributes_for :relation_products, :allow_destroy => true

  # answer_dateがdate型で帰ってくるため、バリデーション用に追加しています。
  attr_accessor :answer_date_text
  # 関連商品全削除有無フラグ
  attr_accessor :targets_modified

  validates_numericality_of :display_order, :only_integer =>true, :message => :EM990003, :if => :display_order_not_nil
  validates_format_of       :answer_date_text, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :answer_date_text_not_nil


  def display_order_not_nil
    !(self.display_order.nil? || self.display_order == "")
  end

  def answer_date_text_not_nil
    !(self.answer_date_text.nil? || self.answer_date_text == "")
  end

  # 商品Q&A管理用バリデーション
  def validate_for_product_faq
    valid_flg = true
    unless self.valid?
      valid_flg = false
    end
    display_errors = {}
    display_input = []
    self.displays.each do |d|
      display_errors[d.locale] = d
      display_input << d.input_display?
    end

    unless display_input.index(true)
      errors.add(:general, "言語を一つ指定して入力してください。")
      valid_flg = false
    end

    relation_product_delete_list = []
    relation_product_list = self.relation_products
      # 商品がすべて削除対象の場合はエラーを追加する
      relation_product_list.each do |r_product|
        relation_product_delete_list << !r_product.marked_for_destruction?.nil?
      end
      if (relation_product_delete_list.index(false).nil?)
        errors.add(:general, "商品を１つ以上選択してください。")
        valid_flg = false
      end

    # accepts_nested_attributes_forしているので配下のdisplayのエラーまで出力されてしまうため
    # いったんdisplayのつくエラー以外を抜き出し、errorsを削除してからもう一度addで入れてます。
    faq_errors = {}
    errors.each do |error_key,msg|
      unless error_key.to_s.index('display')
        faq_errors[error_key.to_s] = msg.to_s
      end
    end
    self.errors.clear
    faq_errors.each_pair do |key, value|
      errors.add(key, value)
    end
    return valid_flg, display_errors
  end

  #
  # = 非公開フラグがtrueの場合｢非公開｣の文字列を返却します。=
  #
  def invisible_flg_name
    if self.invisible_flg
      return "非公開"
    else
      return "公開"
    end
  end

  # 削除フラグ付与処理
  def old_product_mark_for_destruction(modified = targets_modified)
    if ActiveRecord::ConnectionAdapters::Column.value_to_boolean(modified)
      self.relation_products.each do |target|
        target.mark_for_destruction
      end
    end
  end

  #
  # = 商品Q＆Aに紐づく商品コードを配列で返却します。=
  #
  def product_codes
    product_codes = []
    product_faq_relation_products = self.relation_products
    unless product_faq_relation_products.nil?
      product_faq_relation_products.each do |pfrp|
        product_codes << pfrp.product.product_code unless pfrp.product.nil?
      end
    end
    return product_codes
  end

  #
  # = 商品Q＆Aに紐づく商品名を配列で返却します。=
  #
  def product_names
    product_names = []
    product_faq_relation_product = self.relation_products
    unless product_faq_relation_product.nil?
      product_faq_relation_product.each do |pfrp|
        product_names << pfrp.product.order_product_name_1 unless pfrp.product.nil?
      end
    end
    return product_names
  end

  #
  # = 商品Q＆Aに紐づく問合せ内容を返却します。=
  #
  def inquiry
    product_faq_display = self.displays
    unless product_faq_display.nil?
      product_faq_display.each do |pfd|
        if pfd.locale == 'ja'
          return inquiriy = pfd.inquiry unless (pfd.inquiry.nil? || pfd.inquiry.empty?)
        end
      end
      product_faq_display.each do |pfd|
        return inquiriy = pfd.inquiry unless (pfd.inquiry.nil? || pfd.inquiry.empty?)
      end
    end
  end

  #
  # = 商品Q＆Aに紐づく回答内容を配列で返却します。=
  #
  def answer
    product_faq_display = self.displays
    unless product_faq_display.nil?
      product_faq_display.each do |pfd|
        return answer = pfd.answer unless (pfd.inquiry.nil? || pfd.inquiry.empty?)
      end
    end
  end

  #
  # = 商品Q＆Aに紐づく言語を配列で返却します。=
  #
  def locales
    locales = []
    product_faq_display = self.displays
    unless product_faq_display.nil?
      product_faq_display.each do |pfd|
        locales << pfd.locale
      end
    end
    return locales
  end

  #
  # = 商品Q&A検索入力フォーム(管理画面用) =
  #
  class SearchInfo < EcRider::ConditionForm
    attr_accessor :product_code, :product_name, :inquiry, :answer

    #
    # = 検索条件を作成します。 =
    #
    def to_find_options(options = nil)
      where = Condition.block do |c|
        c.filter(:ignore_blank)
        #商品コード条件
        c.and("products.product_code", "like", "%#{product_code}%" )

        #商品名条件
        c.and("products.order_product_name_1", "like", "%#{product_name}%" )

        #問い合わせ内容条件
        c.and("product_faq_displays.inquiry", "like", "%#{inquiry}%" )

        #回答内容条件
        c.and("product_faq_displays.answer", "like", "%#{answer}%" )
      end
      result = {:conditions => where}
      result[:joins] = ["LEFT OUTER JOIN product_faq_displays on product_faqs.id = product_faq_displays.product_faq_id",
                        "LEFT OUTER JOIN product_faq_relation_products on product_faqs.id = product_faq_relation_products.product_faq_id",
                        "LEFT OUTER JOIN products on product_faq_relation_products.product_id = products.id"]
      result[:group] = "product_faqs.id"
      result.update(options) if options
      return result
    end
  end
end
