# -*- coding: utf-8 -*-
class ProductSkuInventory < ActiveRecord::Base
  versioned

  belongs_to :product_sku

  #その時点での在庫数（子SKUの在庫を考慮しない）を取得する
  def get_stock_count
    ProductSkuInventory.sum('count', :conditions => ['product_sku_id = ? AND id <= ?', self.product_sku_id, self.id])
  end
  
  #sku_idとorderable_countを返すSQL（一覧画面などのJOIN用：レスポンス注意）
  def self.select_stock_count_from_product_skus
    ProductSkuInventory::SearchForm.select_stock_count_from_product_skus
  end

  #履歴登録フォーム
  class CreateForm < ActiveForm
    include SelectableAttr::Base
    include ActiveRecord::Validations
    
    attr_accessor :product_sku_id
    attr_accessor :in_out_div
    attr_accessor :count
    attr_accessor :comment
    
    validates_presence_of :count
    validates_numericality_of :count, :only_integer =>true, :greater_than => 0
    validates_length_of :comment, :maximum=>255, :allow_nil => true
    
    def validate
      #在庫を減らす場合、新しい在庫が現在の引き当て数よりも少なくなってはいけない
      return true if self.in_out_div == 'in' || self.product_sku_id.nil?
      product_sku = ProductSku.find(self.product_sku_id)
      if (product_sku.secure_stock_count > (product_sku.parent_stock_count - self.count.to_i))
        errors.add(:count, :EM200003)
        return false
      end
      return true
    end
    
    def create_inventory_history
      ProductSkuInventory.transaction do
        # 在庫履歴を作成する
        inventory = ProductSkuInventory.new()
        inventory.product_sku_id = self.product_sku_id
        inventory.comment = self.comment
        inventory.count = (self.in_out_div == 'in') ?  self.count.to_i : (-1 * self.count.to_i)
        inventory.save

        # SKUも更新する
        sku = inventory.product_sku
        sku.parent_stock_count += inventory.count
        composable_count = sku.stock_count(:all)
        sku.composable_stock_count = composable_count[:stock_count]
        sku.orderable_count = composable_count[:orderable_count]
        sku.save!

        # 親商品も再計算する
        sku.parent_skus.each do |parent_component|
          parent_sku = parent_component.product_sku
          composable_count = parent_sku.stock_count(:all)
          parent_sku.composable_stock_count = composable_count[:stock_count]
          parent_sku.orderable_count = composable_count[:orderable_count]
          parent_sku.save!
        end
      end
    end
  end
  
  #在庫管理検索フォーム（ProductSkuごとの在庫数一覧を検索するフォーム）
  class SearchForm < EcRider::ConditionForm
    include SelectableAttr::Base

    attr_accessor :product_code # 商品コード
    attr_accessor :product_name # 商品名
    attr_accessor :stock_count_min  # 在庫数（最小）
    attr_accessor :stock_count_max  # 在庫数（最大）
    attr_accessor :orderable_count_min # 販売可能数（最小）
    attr_accessor :orderable_count_max # 販売可能数（最大）
    attr_accessor :out_of_stock_flg # 在庫切れのみ表示する
    attr_accessor :out_of_safety_stock_flg # 安全在庫切れのみ表示する

    def self.create_selects(sku_table_name = 'ps')
      select = <<-EOS
--        pc.id AS component_parent_sku_id,
        #{sku_table_name}.id,
        #{sku_table_name}.product_id,
        #{sku_table_name}.sku_code,
        #{sku_table_name}.safety_stock_count,
        #{sku_table_name}.parent_stockable_flg,
        #{sku_table_name}.parent_stock_count,
        #{sku_table_name}.secure_stock_count,
        #{sku_table_name}.composable_stock_count,
        #{sku_table_name}.orderable_count,
        p.product_code,
        p.order_product_name_1,
        /*--- バリエーション名(「LL - 赤」など)(sku_variation_name) ---*/
        CONCAT_WS(' - ', pvi1.name, pvi2.name, pvi3.name) AS sku_variation_name
      EOS
      return select
    end

    def self.create_joins(sku_table_name = 'ps')
      joins = <<-EOS
        INNER JOIN products p ON #{sku_table_name}.product_id = p.id AND p.deleted_at IS NULL
     --   LEFT JOIN product_components pc ON pc.product_sku_id = #{sku_table_name}.id
        LEFT JOIN product_variation_item_displays pvi1 ON pvi1.product_variation_item_id = #{sku_table_name}.variation_item_1_id AND pvi1.locale = 'ja'
        LEFT JOIN product_variation_item_displays pvi2 ON pvi2.product_variation_item_id = #{sku_table_name}.variation_item_2_id AND pvi2.locale = 'ja'
        LEFT JOIN product_variation_item_displays pvi3 ON pvi3.product_variation_item_id = #{sku_table_name}.variation_item_3_id AND pvi3.locale = 'ja'
      EOS
      return joins
    end

    # 商品SKUごとに各種在庫状況を取得するSQLのSELECT文を生成して返す
    def self.select_stock_count_from_product_skus(sku_table_name = 'ps')
      query = <<-"EOS"
        SELECT
          #{create_selects}
        FROM product_skus #{sku_table_name}
        #{create_joins}
      EOS
      return query
    end

    # 在庫一覧表示用のSQL文を生成して返す
    def to_paginate_condition
      where = []
      param = []

#      where << '(pc.id IS NULL OR NOT EXISTS (SELECT * FROM product_components epc WHERE pc.id > epc.id AND pc.product_sku_id = epc.product_sku_id))'
      
      # 商品コード/SKUコード
      unless product_code.blank?
        where << "sku_code LIKE ?"
        param << '%'+product_code+'%'
      end
      # 商品名/バリエーション名
      unless product_name.blank?
        where << "(order_product_name_1 LIKE ? OR CONCAT_WS(' - ', pvi1.name, pvi2.name, pvi3.name) LIKE ?)"
        param << '%'+product_name+'%'
        param << '%'+product_name+'%'
      end
      # 在庫数（最小）
      unless stock_count_min.blank?
        where << "composable_stock_count >= ?"
        param << stock_count_min
      end
      # 在庫数（最大）
      unless stock_count_max.blank?
        where << "composable_stock_count <= ?"
        param << stock_count_max
      end
      # 販売可能数（最小）
      unless orderable_count_min.blank?
        where << "orderable_count >= ?"
        param << orderable_count_min
      end
      # 販売可能数（最大）
      unless orderable_count_max.blank?
        where << "orderable_count <= ?"
        param << orderable_count_max
      end
      # 在庫切れのみ表示する
      if out_of_stock_flg == '1'
        where << "composable_stock_count = 0"
      end
      # 安全在庫切れのみ表示する
      if out_of_safety_stock_flg == '1'
        where << "composable_stock_count < safety_stock_count"
      end

      return [where.join(' AND ')] + param
    end
    
    def get_per_page
      per_page = self.disp_count.to_i
      per_page = (per_page.to_i == 0) ? 20 : per_page
    end
  end
  
end
