# -*- coding: utf-8 -*-
class UserAddress < ActiveRecord::Base
  versioned

  include EcRider::ModelUtility
  include EcRider::PluginModule::Extendable

  belongs_to :user
  belongs_to :country, :foreign_key => 'country_code'

  attr_accessor :index

  #
  # アドレス種類区分
  #
  selectable_attr :address_div do
    entry '0', :user, '会員の住所'
    entry '1', :ship_to, '配送先の住所'
    entry '2', :user_en, '会員の住所（海外配送用）' do #会員の住所が日本語の場合のみ
      # メッセージの項目名を英語で表示したいため、独自のバリデーションとして実装する
      def validate(user_address)
        length_msg = 'activerecord.errors.messages.too_long'
        if user_address.name_last.blank?
          user_address.errors.add('family name', :blank)
        else
          user_address.errors.add('family name', I18n.t(length_msg, :count => 30)) if user_address.name_last.split(//).size > 30
          user_address.errors.add('family name', :EM990028) unless user_address.name_last =~ /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/
        end
        if user_address.name_first.blank?
          user_address.errors.add('first name', :blank)
        else
          user_address.errors.add('first name', I18n.t(length_msg, :count => 30)) if user_address.name_first.split(//).size > 30
          user_address.errors.add('first name', :EM990028) unless user_address.name_first =~ /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/
        end
        if user_address.address_1.blank?
          user_address.errors.add('prefecture', :blank)
        else
          user_address.errors.add('prefecture', I18n.t(length_msg, :count => 30)) if user_address.address_1.split(//).size > 30
          user_address.errors.add('prefecture', :EM990028) unless user_address.address_1 =~ /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/
        end
        if user_address.address_2.blank?
          user_address.errors.add('address1', :blank)
        else
          user_address.errors.add('address1', I18n.t(length_msg, :count => 60)) if user_address.address_2.split(//).size > 60
          user_address.errors.add('address1', :EM990028) unless user_address.address_2 =~ /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/
        end
        unless user_address.address_3.blank?
          user_address.errors.add('address2', I18n.t(length_msg, :count => 60)) if user_address.address_3.split(//).size > 60
          user_address.errors.add('address2', :EM990028) unless user_address.address_3 =~ /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/
        end
      end
    end
  end

  #
  # バリデーション
  #
  validates_presence_of :name_last, :unless => :is_sub_address?
  validates_presence_of :name_first, :unless => :is_sub_address?
  validates_presence_of :tel_num, :unless => :is_sub_address?
  validates_presence_of :address_1, :unless => :is_sub_address?
  validates_presence_of :address_2, :unless => :is_sub_address?
  validates_presence_of :country_code
  validates_presence_of :locale

  # 日本語住所で必須
  validates_presence_of :kana_name_last, :if => :is_japanese_format?
  validates_presence_of :kana_name_first, :if => :is_japanese_format?
  validates_presence_of :zip_code, :if => :is_japanese_format?

  validates_length_of :name_last, :in => 0..30, :unless => :is_sub_address?
  validates_length_of :name_first, :in => 0..30, :unless => :is_sub_address?
  validates_length_of :address_1, :in => 0..30, :unless => :is_sub_address?
  validates_length_of :address_2, :in => 0..60, :unless => :is_sub_address?
  validates_length_of :address_3, :in => 0..60, :allow_nil => true, :unless => :is_sub_address?

  # 氏名（半角英数＋半角記号＋半角スペース）
  validates_format_of :name_last , :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/, :message => :EM990028, :if => :is_english_format?
  validates_format_of :name_first, :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/, :message => :EM990028, :if => :is_english_format?
  # 氏名（半角記号「^ ` { | } ~ & < > " '」以外（コンビニ決済での制限））
  validates_format_of :name_last , :with => /^[^\^\`\{\|\}\~\&\<\>\"\']+$/, :message => :EM990007, :if => :is_japanese_format?
  validates_format_of :name_first, :with => /^[^\^\`\{\|\}\~\&\<\>\"\']+$/, :message => :EM990007, :if => :is_japanese_format?

  # 住所（半角英数＋半角記号＋半角スペース）
  validates_format_of :address_1, :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :is_english_format?
  validates_format_of :address_2, :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :is_english_format?
  validates_format_of :address_3, :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :is_english_format?

  # カナ
  validates_format_of :kana_name_last,  :with => /^[ァ-ヶ・ーヽヾ]+$/, :message => :EM990006, :if => :is_japanese_format?
  validates_format_of :kana_name_first, :with => /^[ァ-ヶ・ーヽヾ]+$/, :message => :EM990006, :if => :is_japanese_format?

  # 電話番号の桁数
  validates_length_of :tel_num, :in => 12..13, :if => :is_japanese_format?
  validates_format_of :tel_num, :with => /^\d{2,5}-\d{1,4}-\d{4}$/, :message => :EM990007, :if => :is_japanese_format?

  validates_format_of :tel_num, :with => /^[\-\+0-9]*$/, :message => :EM990007

  # FAX番号の桁数
  validates_length_of :fax_num, :in => 12..13, :if => :fax_num_check?
  validates_format_of :fax_num, :with => /^\d{2,5}-\d{1,4}-\d{4}$/, :message => :EM990007, :if => :fax_num_check?

  validates_format_of :fax_num, :with => /^[\-\+0-9]*$/, :message => :EM990007, :if => :fax_num_check?

  # 郵便番号の桁数
  validates_format_of :zip_code, :with => /\d{7}/, :message => :EM990020, :if => :is_japanese_format?

  # address_divごとのvalidateを実行
  def validate
    super
    if errors.on(:value).nil? or errors.on(:value).empty?
      if entry = address_div_entry
        entry.validate(self) if entry.respond_to?(:validate)
      end
    end
    # 簡体字バリデーション
    if is_chinese_format?
      errors.add(:name_last, :EM990043) unless UserAddress.is_chinese?(self.name_last)
      errors.add(:name_first, :EM990043) unless UserAddress.is_chinese?(self.name_first)
      errors.add(:address_1, :EM990043) unless UserAddress.is_chinese?(self.address_1)
      errors.add(:address_2, :EM990043) unless UserAddress.is_chinese?(self.address_2)
      errors.add(:address_3, :EM990043) unless UserAddress.is_chinese?(self.address_3)
    end
    if is_japanese_format?
      # Shift_JISで40Byte以内（コンビニ決済での制限）
      if Iconv.iconv('Shift_JIS', 'UTF-8', self.fullname).to_s.length > 40
        errors.add_to_base("お名前（姓＋名）は長すぎます。")
      end
      if Iconv.iconv('Shift_JIS', 'UTF-8', self.fullname_kana).to_s.length > 40
        errors.add_to_base("フリガナ（姓＋名）は長すぎます。")
      end
    end
  end

  # 渡された文字列が簡体字かどうかをチェックする
  def self.is_chinese?(str)
    return true if str.nil?
    begin
      Iconv.conv('gb2312', 'utf-8', str)
    rescue Iconv::Failure
      return false
    end
    return true
  end

  # 住所が日本語表記の場合はtrueを返す
  def is_japanese_format?
    (locale == 'ja') && (country_code == 'JP')
  end

  # 住所が会員の住所（海外配送用）の場合はtrueを返す
  def is_sub_address?
    address_div_key == :user_en
  end

  # 住所が英語表記の場合はtrueを返す
  def is_english_format?
    !is_japanese_format? && !is_sub_address? && !((locale == 'zh_CN') && (country_code == 'JP'))
  end

  # 住所が中国語表記の場合はtrueを返す
  # システム設定値で中国語表記を許さない設定をした場合はfalseを返却します。
  def is_chinese_format?
    (locale == 'zh_CN' || locale == 'ja') && (country_code == 'CN') && EcRider.config.chinese_format_using
  end

  # 住所が英語表記の場合はtrueを返す
  def is_english_format?
    (!is_japanese_format? || is_sub_address?) && !is_chinese_format? && !((locale == 'zh_CN') && (country_code == 'JP'))
  end

  # FAX番号をチェックする場合は、trueを返す()
  def fax_num_check?
    return !fax_num.blank? && :is_japanese_format?
  end

  #
  # = 自身（会員住所（海外配送用））に会員住所の値を補完する =
  #
  def set_sub_address_from_address(address)
    self.tel_num = address.tel_num
    self.fax_num = address.fax_num
    self.zip_code = address.zip_code
    self.country_code = address.country_code
    self.locale = 'en'
    self.address_div_key = :user_en
  end

  #
  # =ハッシュから会員住所（海外配送用）を生成して返す=
  #
  def self.hash_to_sub_address(hash)
    #ハッシュが空ならnilを返す
    return nil if hash.nil? || hash.values.join('').length < 1
    #ハッシュに値があればUserAddressインスタンスを返す
    return UserAddress.new(hash)
  end

  #
  # =フルネームを連結して返す=
  #
  def fullname
    if is_japanese_format?
      return "#{self.name_last} #{self.name_first}"
    else
      return "#{self.name_first} #{self.name_last}"
    end
  end

  #
  # =フルネームのヨミガナを連結して返す=
  #
  def fullname_kana
    if is_japanese_format?
      return "#{self.kana_name_last} #{self.kana_name_first}"
    else
      return ""
    end
  end

  #
  # =住所（市町村以下）を連結して返す=
  # 
  # TODO この住所はaddress_1を含んでいないので "fulladdress"という名前は相応しくないと思います。
  def fulladdress(include_address_1 = false, options = nil)
    options = { 
      :delimeter => is_japanese_format? ? ' ' : ', '
    }.update(options || {})
    results = [address_3, address_2]
    results << address_1 if include_address_1
    results.reverse! if is_japanese_format?
    results.reject!{|address| address.blank?}
    results.join(options[:delimeter])
  end
  
  # =住所が日本の場合は000-0000にして返す=
  def zip_code_jp
    is_japanese_format? ? "#{zip_code[0, 3].to_s}-#{zip_code[3, 6].to_s}" : zip_code
  end

  # 郵便番号から住所を取得する
  #   入力 : 郵便番号文字列
  #   出力 : [都道府県, 市区町村]
  def self.zip_to_address(zip_code)
    address = JpAddress.find_by_zipcode(zip_code)
    address ? [address.prefecture, address.city + address.address] : ["", ""]
  end

end

