# -*- coding: utf-8 -*-
class UserRank < ActiveRecord::Base
  versioned
  has_many :displays, :class_name => "UserRankDisplay", :foreign_key => 'user_rank_id', :dependent => :destroy
  accepts_nested_attributes_for :displays, :allow_destroy => true
  has_many :prices, :class_name => "ProductPrice", :foreign_key => 'user_rank_id',
    :dependent => :destroy
  has_many :conditions, :class_name => "UserRankCondition", :foreign_key => 'user_rank_id', :dependent => :destroy
  accepts_nested_attributes_for :conditions, :allow_destroy => true

  selectable_attr :available_div do
    entry '0', :unused, '未使用'
    entry '1', :used, '使用'
  end

  selectable_attr :grading_period_div do
    entry '0',  0,  '期間判定をしない'
    entry '1',  1,  '1ヶ月'
    entry '6',  6,  '6ヶ月'
    entry '12', 12, '12ヶ月'
    entry '24', 24, '24ヶ月'
  end
  
  selectable_attr :connector_cd do
    entry '0', :and, "AND検索"
    entry '1', :or,  "OR検索"
  end

  validates_presence_of :name, :code, :available_div, :connector_cd
  
  validates_length_of   :name, :maximum => 120, :allow_nil => true
  validates_length_of   :code, :maximum => 120, :allow_nil => true
  validates_format_of   :code, :with => /^[0-9a-zA-Z]*$/ , :message => :EM990025, :allow_nil => true
  validates_uniqueness_of :code, :message => :EM990027, :allow_nil => true
  validates_inclusion_of :priority, :in => 1..999
  validates_numericality_of :priority, :greater_than => 0, :allow_nil => true

  def validate
    condition_all_blank = true
    self.conditions.each do |condition|
      if condition.not_all_inputed?
        condition_all_blank = false
      end
    end
    errors.add_to_base("検索条件は１件以上入力してください。") if condition_all_blank
  end
  # ユーザランク新規登録時、対応言語分のuser_rank_displaysを作成します。
  def setup_displays_for(locales = I18n.supported_locales)
    return if locales.nil? || locales.empty?
    supported_locales = I18n.supported_locales.map(&:to_s)
    displays_locales = self.displays.map(&:locale)
    (supported_locales - displays_locales).each do |locale|
      self.displays.build(:locale => locale.to_s)
    end
  end

  # ユーザランク新規登録時、user_rank_conditionsを作成します。
  def setup_conditions
    return unless self.conditions.empty?
    5.times do |count|
      self.conditions.build(:no => count+1)
    end
  end

  # 登録rank_listされている会員ランクの一覧を返却します。
  def self.user_rank_list()
    list = []
    find(:all, :order => 'id').each do |rank|
      list << [rank.name, rank.id]
    end
    return list
  end
  
  # CSVのヘッダの凡例に使用します
  def self.rank_remark
    find(:all, :order => 'code').map{|rank| "#{rank.code}:#{rank.name}"}.join('、')
  end
  
  # 会員ランク一括更新バッチ
  # 対象会員：WEB会員、オフライン会員
  # 会員ごとに会員ランク更新期間内の会員ランクアップ有無を判断し、会員ランクを更新します。
  def self.update_all_user_rank
    update_users = {} # ユーザランクIDをキーにそのランクに更新する会員のIDを入れます
    active_user_ranks = UserRank.find(:all, :conditions => ["available_div = ?", self.available_div_id_by_key(:used)], :include => [:conditions], :order => "priority desc")
    users = User.find(:all, :conditions => ["member_div in (?)", [User.member_div_id_by_key(:membership), User.member_div_id_by_key(:offline_membership)]])
    active_user_ranks.each do |active_user_rank|
      update_users[active_user_rank.id.to_s] = []
      users.each do |user|
        upgradable, user_rank_id = active_user_rank.upgrade?(user)
        next unless upgradable
        update_users[active_user_rank.id.to_s] << user.id
      end
      User.update_all("user_rank_id = #{active_user_rank.id}", ["user_rank_id != #{active_user_rank.id} and id in (?)", update_users[active_user_rank.id.to_s]])
    end
#    active_user_ranks.each_key do |user_rank_id|
#      User.update_all("user_rank_id = #{user_rank_id}", "id in (#{update_users[user_rank_id.to_s]})")
#    end
  end

  # ランクアップ判定メソッド
  def upgrade?(user)
    result = false
    # 判定基準月の設定がある かつ conditionに注文金額、購入回数がある場合のみ判定終了日を取得する
    grading_period_conds = self.conditions.select{|con| con.type_key == :order_amount || con.type_key == :order_count}
    if !(self.grading_period_div_key == 0) && grading_period_conds.empty?
      # ここからここまでの期間を作成
      user.user_rank_check_end_date = self.grading_period_date(user.rank_up_date)
    end
    case self.connector_key
    when :and
      # AND条件の場合 一つの条件でもfalseだとランクアップしない
      self.conditions.each do |cond|
        next unless cond.not_all_inputed?
        unless cond.judge(user)
          return false, self.id
        end
      end
      result = true
    when :or
      # OR条件の場合 一つの条件でもtrueならランクアップ
      self.conditions.each do |cond|
        next unless cond.not_all_inputed?
        if cond.judge(user)
          result = true
          break
        end
      end
    end
    return result, self.id
  end

  # 会員のランクアップした日付から判定期間終了日を返却します
  def grading_period_date(user_rankup_date = nil)
    return if user_rankup_date.nil?
    return user_rankup_date >> self.grading_period_div_key
  end
  
  class SearchForm < EcRider::ConditionForm
    include SelectableAttr::Base

    attr_accessor :code    # 会員ランクコード
    attr_accessor :name    # 会員ランク名
    attr_accessor :type_cd # 条件項目

    def to_find_options(options = nil)
      @where, @params = [], []
      build_code
      build_name
      build_type_cd
      result = {
        :order => "code asc, created_at asc"
      }
      @params.unshift(@where.join(' AND ')) unless @where.empty?
      result[:conditions] = @params unless @params.empty?
      unless type_cd.blank?
        result[:joins] = ["inner join user_rank_conditions on user_rank_conditions.user_rank_id = user_ranks.id"]
        result[:group] = "user_ranks.id"
      end
      result
    end
    
    private
    def build_code
      return if code.blank?
      @where << "code like ?"
      @params << "%#{code}%"
    end
    
    def build_name
      return if name.blank?
      @where << "name like ?"
      @params << "%#{name}%"
    end

    def build_type_cd
      return if type_cd.blank?
      @where << "user_rank_conditions.type_cd = ?"
      @params << "#{type_cd}"
    end
  end
end
