# -*- coding: utf-8 -*-
module Goldberg
  class SiteAction < ActiveRecord::Base
    include Goldberg::Model
    
    RESTFUL_NAMES = %w(index show new create edit update destroy)
    RESTFUL_NAME_PATTERN = /^(:?#{RESTFUL_NAMES.join('|')})$/

    # belongs_to :site_action_group, :class_name => 'Goldberg::SiteActionGroup', :foreign_key => 'site_action_group_id'
    belongs_to :controller, :class_name => 'Goldberg::SiteActionGroup', :foreign_key => 'site_action_group_id'

    has_many :role_actions, :class_name => 'Goldberg::RoleAction', :foreign_key => 'site_action_id'
    # has_many :roles, :through => :role_actions
    
    validates_presence_of :name, :category_name, :site_action_group_id
    validates_uniqueness_of :name, :scope => 'site_action_group_id'
    
    selectable_attr :category_name do
      entry 'index'  , :index  , "検索・詳細"
      entry 'create' , :create , "新規登録"
      entry 'update' , :update , "編集・更新"
      entry 'destroy', :destroy, "削除"
      entry 'other'  , :other  , "その他"
    end
    
    before_validation :set_category_name

    def set_category_name
      unless self.class.category_name_ids.include?(self.category_name.to_s)
        self.category_name_key = :other
      end
    end


    def short_name
      return name if display_name.blank?
      group = self.controller 
      return display_name unless group
      return display_name if group.base_name.blank?
      display_name.sub(/^#{group.base_name}/, '')
    end


    def fullname
      "#{self.controller.name}: #{self.name}"
    end

    def url
      @url ||= "/#{self.controller.name}/#{self.name}"
    end

    def restful?
      RESTFUL_NAME_PATTERN =~ self.name
    end
    

    def allowed_for?(role_or_role_id)
      role_id = role_or_role_id.respond_to?(:new_record?) ? role_or_role_id.id : role_or_role_id.to_i
      @allowed_role_ids ||= self.role_actions.map{|ra| ra.role_id}
      @allowed_role_ids.include?(role_id)
    end

    
    # クラスメソッドは別に分けておきます。
    class << self
      def find_by_path(*args)
        args = args.flatten
        if args.length == 2
          controller, action = *args
        elsif args.length == 1
          case arg = args.first
          when Hash
            controller, action = arg[:controller], arg[:action]
          else
            parts = arg.split('/')
            action = parts.pop
            controller = parts.join("/")
          end
        else
          raise ArgumentError
        end
        SiteAction.find(:first,
          :joins => "inner join goldberg_site_action_groups on goldberg_site_action_groups.id = goldberg_site_actions.site_action_group_id",
          :conditions => [
            "goldberg_site_action_groups.name = ? AND goldberg_site_actions.name = ? ", controller, action])
      end

      # 指定したコントローラの情報を登録します。
      # 現在の登録データからの変更分を登録、削除します。
      def append_actions_from_desc_by_controller(controller)
        action_ids, action_group_ids = append_actions_from_desc_by_class(controller)
        Goldberg::SiteAction.destroy_all(["id not in (?) and site_action_group_id = ?", action_ids, action_group_ids.first])
      end

      # descの記述からSiteActionのデータを登録します。
      # 現在の登録データからの変更分を登録、削除します。
      def append_actions_from_desc
        Dir.glob(File.join(RAILS_ROOT, "app/controllers/**/*_controller.rb")) do |path|
          require path
        end
        # プラグインで新設されたコントローラも含めます。
        Dir.glob(File.join(RAILS_ROOT, "vendor/plugins/ecr_*/app/controllers/**/*_controller.rb")) do |path|
          require path
        end

        yield if block_given?
        after_action_group_ids = []
        after_action_ids = []
        self.transaction do
          Prestruct.modules.keys.each do |class_name|
            action_ids, action_group_ids = append_actions_from_desc_by_class(class_name.to_s.constantize)
            after_action_ids << action_ids if action_ids
            after_action_group_ids << action_group_ids if action_group_ids
          end
        end
        Goldberg::SiteAction.destroy_all(["id not in (?)", after_action_ids.flatten.compact])
        Goldberg::SiteActionGroup.destroy_all(["id not in (?)", after_action_group_ids.flatten.compact])
      end

      private
      def append_actions_from_desc_by_class(klass)
        action_group_ids = []
        action_ids = []

        return unless klass < ApplicationController
        klass.prepare_action_descriptions # descのデータを補完を行います。
        klass_prestruct = klass.prestruct
        return if klass_prestruct[:invisible]
        controller_name = klass.controller_path
        action_group = Goldberg::SiteActionGroup.find_by_name(controller_name) ||
          Goldberg::SiteActionGroup.new(:name => controller_name)
        action_group.display_name = klass_prestruct[:display_name]
        action_group.save!
        klass_prestruct.method_descriptions.each do |action_name, description|
          next if description[:invisible]

          # プラグインで拡張した際の別名メソッドには権限を振らない
          next if action_name.to_s =~ /_with_ecr_/
          next if action_name.to_s =~ /_without_ecr_/

          if auth_as = description[:auth_as]
            # :auth_as が指定されていたら、参照されているアクションが存在するかチェック
            if auth_as.is_a?(Hash)
            else
              unless klass.action_methods.include?(auth_as.to_s)
                raise "'#{auth_as.to_s}' action not found in #{klass.name} #{klass.action_methods.to_a.sort.inspect}"
              end
            end
            next
          end
          display_name = description[:display_name]
          display_name = nil if display_name.blank?
          action = action_group.actions.find_by_name(action_name.to_s) || SiteAction.new(:name => action_name.to_s)
          action.attributes = {
            :site_action_group_id => action_group.id,
            :category_name => description[:category_name].to_s,
            :display_name => display_name
            }
          action.set_category_name
          action.save!
          action_ids << action.id
        end
        action_group_ids << action_group.id
        
        return action_ids, action_group_ids
      end
    end

  end
end
