# -*- coding: utf-8 -*-
require 'digest/sha1'

module Goldberg
  class User < ActiveRecord::Base
    versioned

    include Goldberg::Model
    include ::SelectableAttr::Base

    acts_as_paranoid
    
    belongs_to :role, :class_name => 'Goldberg::Role'

    has_many :csv_order_output_items, :foreign_key => "supervisor_id", :order=> "id"
    has_many :cms_project_assigns, :foreign_key => "goldberg_user_id", :class_name => "CmsProjectAssign"

    attr_accessor :password_confirmation
    attr_accessor :confirm_password

    before_save :encrypt_password
    before_validation :set_name_if_blank

    normal_user_allowed_url = [
      "top", "user", "gruff", "inquiry", "login", "order", "product",
      "shortest_contact_modify"]

    # 権限ID
    selectable_attr :role_id, :base_name => 'role' do
      entry 1, :sys_administrator, 'システム管理者', :allowed_url => :all
      entry 2, :net_administrator, '情報システム部責任者', :allowed_url => normal_user_allowed_url + ["name_master", "payments" ]
      entry 3, :normal_user, '一般ユーザー', :allowed_url => normal_user_allowed_url
    end

    attr_accessor :password
    attr_accessor :password_confirmation

    # 必須チェック
    validates_presence_of :login,    :message => :EM990001
    validates_presence_of :email,    :message => :EM990001
    
    validates_presence_of :password, :message => :EM990001, :if => :check_password_confirm?
    validates_presence_of :password_confirmation, :message => :EM990001, :if => :check_password_confirm?
    # パスワード確認チェック
    validates_confirmation_of :password, :message => :EM990008, :if => :check_password_confirm?

    # フォーマットチェック
    validates_format_of   :login,    :message => :EM990007, :with => /^([a-zA-Z0-9\._-])+$/, :if => :validate_login?
    validates_format_of   :email,    :message => :EM990007, :with => /^([a-zA-Z0-9])+([a-zA-Z0-9\._-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/, :if => :validate_email?

    #文字数バリデーション
    validates_length_of      :password, :minimum => 4, :if => :check_password_confirm?

    def validate
      # ユニークチェック
      errors.add(:login, :EM990035) unless is_unique_goldberg_user?
      errors.add(:email, :EM990023) unless is_unique_email?
    end

    # パスワードの比較チェックをする場合はtrueを返す
    def check_password_confirm?
      return (new_record? || self.password || self.password_confirmation)
    end

    # アクセス可否
    # このGoldberg::Userが指定された名前のコントローラにアクセス可能かどうかを判断します。
    def accessable?(controller_name)
      allowed_url = self.role_entry[:allowed_url]
      (allowed_url == :all) ? true :
        allowed_url.respond_to?(:include?) ? allowed_url.include?(controller_name) :
        false
    end

    def check_password(clear_password)
      self.crypted_password == Digest::SHA1.hexdigest("--#{self.salt.to_s}--#{clear_password}--")
    end

    def self.find_for_permission(permission_id)
      rp = Goldberg::RolesPermission.find_by_permission_id(permission_id)
      users = nil
      unless rp.nil?
        role_id = rp.role_id
        users = self.find(:all , :conditions => ["role_id = ?", role_id])
      end
      return users
    end

    # =SearchCondition=
    # 検索条件を保持するためのクラス
    class SearchCondition < EcRider::ConditionForm
      attr_accessor :goldberg_user_id 
      attr_accessor :login
      attr_accessor :disp_count
      attr_accessor :deleted_after

      def to_find_options(options = nil)
 	      where = Condition.block do |c|
          c.filter(:ignore_blank)
          # 管理者IDの条件
          c.and "goldberg_users.id", "like", "%#{goldberg_user_id}%"
          # 管理者名の条件
          c.and "goldberg_users.login", "like", "%#{delete_space(login)}%"
 	      end
 	      result = {:conditions => where}
        result[:order] = "goldberg_users.id"
        #result[:joins] = joins.uniq.join(" ")
 	      result.update(options) if options
 	      return result
 	    end
      
      private
      # 指定の文字列のスペースを削除します。
      def delete_space(str)
        return nil unless str
        # 半角スペースを削除
        str.gsub(' ', '').gsub('　', '')
      end
    end

    def get_start_path
      if self.start_path and self.start_path.length > 0
        self.start_path
      else
        self.role.get_start_path
      end
    end

    # Encrypts some data with the salt.
    def self.encrypt(password, salt)
      Digest::SHA1.hexdigest("--#{salt}--#{password}--")
    end

    # Encrypts the password with the user salt
    def encrypt(password)
      self.class.encrypt(password, salt)
    end

    def authenticated?(password)
      self.crypted_password == encrypt(password)
    end

    protected

    # ~~before filter~~ モデルではフィルターではなく、コールバックと言います
    def encrypt_password
      return if password.blank?
      self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{login}--")
      self.crypted_password = encrypt(password)
      # # パスワードの暗号化とは関係ないけど、ひとまずここでログイン後に表示するページパスを設定する
      # self.start_path = "/top" if self.start_path.nil?
    end

    def set_name_if_blank
      self.name = self.login if name.blank?
    end

    private

    # = 管理者IDがユニークな場合はtrueを返します。 =
    def is_unique_goldberg_user?
      goldberg_user_id = self.id ? self.id : 0
      return true if self.login.blank?
      Goldberg::User.count(
        :conditions => [
          "goldberg_users.login = ? and goldberg_users.id <> ?", self.login, goldberg_user_id]) <= 0
    end

    # = メールアドレスがユニークな場合はtrueを返します。 =
    def is_unique_email?
      return true if self.email.blank?
      goldberg_user_id = self.id ? self.id : 0
      Goldberg::User.count(
        :conditions => [
          "goldberg_users.email = ? and goldberg_users.id <> ?", self.email, goldberg_user_id]) <= 0
    end

    # = 管理者名のバリデーション =
    def validate_login?
      !self.login.blank?
    end

    # = メールアドレスのバリデーション =
    def validate_email?
      !self.email.blank?
    end

    public

    # =認証=
    def self.authenticate(name, password)
      s = self.find(:first, :conditions => ['name = ?', name])
      return nil unless s
      s && s.authenticated?(password) ? s : nil
    end

  end
end
