# -*- coding: utf-8 -*-
require 'uri'
require 'optparse'
require 'rubygems'
require 'httpclient'
require 'nokogiri'

class ZabbixFrontendClient
  
  SUPPORTED_SERVER_VERSIONS = %w(1.4 1.6)

  class ServerVersionError < StandardError
  end

  def initialize(base_url, username, password, options)
    @base_url = base_url
    @username = username
    @password = password
    @options = options
    uri = URI.parse(base_url)
    @server_host = uri.host
    @server_version = (options[:server_version] || '1.6').to_s
    @logger = options[:logger] || Logger.new(STDOUT)
    unless SUPPORTED_SERVER_VERSIONS.include?(@server_version)
      raise ServerVersionError, 
        "Unsupported Zabbix server version #{@server_version.inspect}, supports only #{SUPPORTED_SERVER_VERSIONS.inspect}"
    end
    @http_client = nil
  end
  
  def login
    @http_client = HTTPClient.new
    # @http_client.proxy = "http://localhost:8080"
    @http_client.debug_dev = STDOUT if @options[:debug]
    boundary = "123456"
    res = post("index.php", {
        "form" => "1", 
        "form_refresh" => "1", 
        "name" => @username, 
        "password" => @password, 
        "enter" => "Enter"
      },
      "content-type" => "multipart/form-data, boundary=#{boundary}")
    repair_sessionid(res)
    zbx_sessionid_cookie = @http_client.cookie_manager.cookies.detect{|cookie| cookie.name == "zbx_sessionid"}
    zbx_sessionid_cookie.path = "/zabbix/"
    debug_cookies
  end

  def index
    res_content = get_content("index.php")
    @sid = res_content.scan(/<input type="hidden" name="sid" id="sid" value="(\w+)" \/>/).flatten.first
    @sid ||= res_content.scan(/\?sid=(\w+)"/).flatten.first
    debug_cookies
  end

  def hosts
    uri = "hosts.php?groupid=0&config=0"
    uri << "&sid=#{@sid}" if @sid
    res = get(uri)
    debug("cookies: " << @http_client.cookie_manager.cookies.inspect) if @options[:debug]
    @parsed_hosts = parse_hosts_body(res.content)
  end

  def templates
    uri = "hosts.php?groupid=0&config=3"
    uri << "&sid=#{@sid}" if @sid
    res = get(uri)
    debug("cookies: " << @http_client.cookie_manager.cookies.inspect) if @options[:debug]
    @parsed_templates = parse_templates_body(res.content)
  end

  def groups
    uri = "hosts.php?groupid=0&config=1"
    uri << "&sid=#{@sid}" if @sid
    res = get(uri)
    debug("cookies: " << @http_client.cookie_manager.cookies.inspect) if @options[:debug]
    @parsed_groups = parse_groups_body(res.content)
  end

  class AddHostError < StandardError
  end

  def add_host(registration_or_hostname, ip_address = nil)
    registration = registration_or_hostname.is_a?(Hash) ? 
      registration_or_hostname : 
      {'dns' => registration_or_hostname, 'ip' => ip_address}
    registration['host'] ||= registration['dns']
    
    form = default_params_add_host = YAML.load_file(
      File.join(File.dirname(__FILE__), "resources/add_host-#{@server_version}.yml"))
    case @server_version
    when '1.4' then
      (registration.delete('templates') || {}).each do |template_id, name|
        form["templates[#{template_id}]"] = name
      end
      form["groups[]"] = (registration.delete('groups') || []).join(",")
      form.update(registration)
      # error messages
      # * Host 'test01' already exists 
      # * Incorrect characters used for DNS 
    when '1.6' then
      (registration.delete('templates') || {}).each do |template_id, name|
        form["templates[#{template_id}]"] = name
      end
      (registration.delete('groups') || {}).each do |group_id|
        form["groups[#{group_id}]"] = group_id.to_s
      end
      form.update(registration)
      form['sid'] = @sid
    end
    # res = post('hosts.php', form)
    # res_content = res.content
    res_content = post_content('hosts.php', form)
    if res_content =~ /Added new host \[(.*?)\]/m
      return $1
    else
      errors = res_content.scan(/ERROR: ([^<>]*)<\/strong>/m).flatten
      errors.concat(res_content.scan(/<li class="error">(.*)<\/li>/m).flatten.map{|s| s.strip})
      raise AddHostError, errors.join
    end
  end

  class RemoveHostError < StandardError
  end

  def remove_host(registration_or_hostname, ip_address = nil)
    registration = registration_or_hostname.is_a?(Hash) ? 
      registration_or_hostname : 
      {'dns' => registration_or_hostname, 'ip' => ip_address}
    host_id = host_id_for(registration['dns'], registration['ip'])
    # raise RemoveHostError, "host not found for: #{local_hostname}, #{local_ipv4}" unless host_id
    form = {
      'config' => '0',
      'hosts[]' => host_id,
      'delete' => 'Delete selected'
    }
    case @server_version
    when '1.4' then
      # 
    when '1.6' then
      form['sid'] = @sid
    end
    res = post('hosts.php', form)
    debug(res.content)
    if res.content =~ /Host deleted/m
      return $1
    else
      errors = res.content.scan(/\[ERROR: ([^\[\]]*)\]/m).flatten
      errors.concat(res.content.scan(/<li class="error">(.*)<\/li>/m).flatten.map{|s| s.strip})
      raise RemoveHostError, errors.join
    end
  end

  private

  def debug_cookies
    debug("cookies: " << @http_client.cookie_manager.cookies.inspect) if @options[:debug]
  end

  def parse_hosts_body(content)
    debug("parse_hosts_body#0")
    hosts = []
    html = Nokogiri.HTML(content)
    rows = html.css("table.tableinfo tr")
    debug("parse_hosts_body#1 rows.length => #{rows.length}")
    rows.each do |row|
      debug("parse_hosts_body#1.0: row['class'] =>" << row['class'])
      next if %w(header footer).include?(row['class'])
      debug("parse_hosts_body#1.1")
      host_id = nil
      inputs = row.css("input[type=checkbox]")
      check = inputs.first
      next unless check
      debug("parse_hosts_body#1.2")
      host_id = check["value"]
      next unless host_id
      debug("parse_hosts_body#1.3")
      result = {:id => host_id.strip}
      tds = row.css("td")
      debug("parse_hosts_body#1.4")
      if tds.length > 2
        debug("parse_hosts_body#1.4.0")
        result.update({
            :name => tds[0].inner_text.strip,
            :hostname => tds[1].inner_text.strip,
            :ipaddress => tds[2].inner_text.strip
          })
        debug("parse_hosts_body#1.4.1 #{result.inspect}")
        hosts << result
      else
        debug("parse_hosts_body#1.4.2 #{tds.inspect}")
      end
    end
    debug("hosts: #{hosts.inspect}")
    hosts
  end

  def parse_templates_body(content)
    debug("parse_templates_body#0")
    result = {}
    html = Nokogiri.HTML(content)
    rows = html.css("table.tableinfo tr")
    debug("parse_templates_body#1 rows.length => #{rows.length}")
    rows.each do |row|
      debug("parse_templates_body#1.0: row['class'] =>" << row['class'])
      next if %w(header footer).include?(row['class'])
      debug("parse_templates_body#1.1")
      template_id = nil
      inputs = row.css("input[type=checkbox]")
      check = inputs.first
      next unless check
      debug("parse_templates_body#1.2")
      template_id = check["value"]
      next unless template_id
      debug("parse_templates_body#1.3")
      links = row.css("a")
      debug("parse_templates_body#1.4")
      if links.first
        debug("parse_templates_body#1.4.0")
        value = links.first.inner_text.strip
        debug("parse_templates_body#1.4.1 #{template_id.inspect} => #{value.inspect}")
        result[template_id] = value
      else
        debug("parse_templates_body#1.4.2 #{links.inspect}")
      end
    end
    debug("result: #{result.inspect}")
    result
  end

  def parse_groups_body(content)
    debug("parse_groups_body#0")
    result = {}
    html = Nokogiri.HTML(content)
    rows = html.css("table.tableinfo tr")
    debug("parse_groups_body#1 rows.length => #{rows.length}")
    rows.each do |row|
      debug("parse_groups_body#1.0: row['class'] =>" << row['class'])
      next if %w(header footer).include?(row['class'])
      debug("parse_groups_body#1.1")
      template_id = nil
      inputs = row.css("input[type=checkbox]")
      check = inputs.first
      next unless check
      debug("parse_groups_body#1.2")
      template_id = check["value"]
      next unless template_id
      debug("parse_groups_body#1.3")
      links = row.css("a")
      debug("parse_groups_body#1.4")
      if links.first
        debug("parse_groups_body#1.4.0")
        value = links.first.inner_text.strip
        debug("parse_groups_body#1.4.1 #{template_id.inspect} => #{value.inspect}")
        result[template_id] = value
      else
        debug("parse_groups_body#1.4.2 #{links.inspect}")
      end
    end
    debug("result: #{result.inspect}")
    result
  end

  def host_id_for(hostname, ipaddress)
    raise ArgumentError, "Unsupported operation. #hosts before #remove_host" unless @parsed_hosts
    @parsed_hosts.each do |parsed_host|
      if parsed_host[:hostname] == hostname && parsed_host[:ipaddress] == ipaddress
        return parsed_host[:id]
      end
    end
    nil
  end

  def uri_for(path)
    "#{@base_url}/#{path}"
  end

  def get(path, query = nil, extheader = {}, &block); request(:get, path, query, nil, extheader, &block); end
  def post(path, body = '', extheader = {}, &block); request(:post, path, nil, body, extheader, &block); end
  def request(method, path, query = nil, body = nil, extheader = {}, &block)
    uri = uri_for(path)
    debug("#{method.to_s.upcase} #{uri}")
    res = @http_client.request(method, uri, query, body, extheader, &block)
    return res
  end

  def get_content(path, query = nil, extheader = {}, &block)
    uri = uri_for(path)
    debug("GET #{uri}")
    res_content = @http_client.get_content(uri, query, extheader, &block)
    debug("")
    return res_content
  end
  def post_content(path, body = nil, extheader = {}, &block)
    uri = uri_for(path)
    debug("POST #{uri}")
    res_content = @http_client.post_content(uri, body, extheader, &block)
    debug("")
    return res_content
  end

  def debug(msg)
    log(:debug,  msg) if @options[:debug]
  end
  
  %w(info warn error fatal).each do |log_level|
    module_eval(<<-"EOS")
      def #{log_level}(msg); log(:#{log_level}, msg); end
    EOS
  end

  def log(level, msg)
    @logger.send(level, msg)
  end

  def repair_sessionid(res)
    return unless @server_version == '1.6'
    return unless @http_client.cookie_manager.cookies.empty?
    zabbix_ession_id = nil
    res.header.all.each do |(key, value)|
      next unless key == 'Set-Cookie'
      next unless value =~ /^zbx_sessionid=/
      s = value.sub(/^zbx_sessionid=/, '')
      next if s =~ /^delete/
      zabbix_ession_id = s
    end
    if zabbix_ession_id
      c = WebAgent::Cookie.new
      c.use = true
      c.name = "zbx_sessionid"
      c.value = zabbix_ession_id
      c.path = "/zabbix/"
      c.domain = @server_host
      c.url = URI.parse(uri_for("index.php"))
      @http_client.cookie_manager.cookies << c
    end
  end

end
