# -*- coding: utf-8 -*-
module EcRider
  module CloudFront
    def self.included(mod)
      if mod < ActionController::Base
        mod.__send__(:private, *self.public_instance_methods(false))
      end
    end


    # require 'aws/s3'

    #
    # CloudFront上のURLを生成します
    #
    def url_for_cloud_front(file_path)
      # 1回のリクエスト中に何回も同じ処理を行う必要はないので、インスタンス変数にキャッシュする
      @public_dir_for_cloud_front ||= File.expand_path(File.join(EcRider.config.ec_front_root, 'public'))
      local_dir = @public_dir_for_cloud_front
      find_str = format_file_name(:find, file_path)
      if local_file = Dir.glob(File.join(local_dir, find_str)).first
        local_file.gsub(local_dir, EcRider.config.cloud_front_url)
      else
        file_path
      end
    end

    # CloudFrontへアップロードするためのタイムスタンプ付きのファイルを生成します。
    def create_timestamped_files(sources, options = {})
      # 1回のリクエスト中に何回も同じ処理を行う必要はないので、インスタンス変数にキャッシュする
      @public_dir_for_cloud_front ||= File.expand_path(File.join(EcRider.config.ec_front_root, 'public'))
      options ||= {}
      options[:timestamp] ||= Time.now
      if (d = options[:timestamp]).respond_to?(:strftime)
        options[:timestamp] = d.strftime(CLOUD_FRONT_VERSION_FORMAT)
      end
      timestamp = options[:timestamp]

      context = CreateTimestampContext.new(self, timestamp, @public_dir_for_cloud_front)

      copiers = context.new_copiers(sources, @public_dir_for_cloud_front)

      all_copiers = copiers.map{|copier| [copier] + copier.all_related_copiers}.flatten

      all_copiers.map do |copier|
        copier.execute
        copier.remove_old_timestamp_files
        copier.dest.gsub(/^#{Regexp.escape(@public_dir_for_cloud_front)}\//, '')
      end
    end

    class CreateTimestampContext
      attr_reader :public_root_path
      def initialize(service, timestamp, public_root_path)
        @service, @timestamp = service, timestamp
        @public_root_path = public_root_path
      end

      def format_dest(src)
        @service.format_file_name(:add, src, :timestamp => @timestamp)
      end

      def new_copiers(sources, base_dir)
        results = sources.map do |src|
          if ABSOLUTE_PATH_EXP =~ src
            actual_src = File.exist?(src) ? src : File.join(@public_root_path, src)
          else
            actual_src = File.join(base_dir, src)
          end
          ext = File.extname(src).downcase
          klass = COPIERS_BY_EXT[ext] || CopyWithTimestamp
          copier = klass.new(self, actual_src)
          copier
        end
        results.each{|result| result.setup}
        results
      end

      ABSOLUTE_PATH_EXP = /^\//
      
      # pathが'/'で始まる絶対パスなら@public_root_pathと、相対パスなら引数base_dirと、pathを合わせたパスを返します。
      def join_path(base_dir, path)
        File.join(ABSOLUTE_PATH_EXP =~ path ? @public_root_path : base_dir, path)
      end
    end

    class CopyWithTimestamp
      EMPTY_ARRAY = [].freeze

      attr_reader :src, :dest

      def initialize(context, src)
        @context = context
        @src = src.freeze
        @dest = @context.format_dest(@src)
        @old_timestamp_files = Dir.glob("#{@src[0..@src.rindex(".")-1]}-#{CLOUD_FRONT_VERSION_QUERY}#{@src[@src.rindex(".")..-1]}")
      end
      def setup; end
      def execute; FileUtils.cp(@src, @dest, :verbose => false); end
      def remove_old_timestamp_files; FileUtils.rm_rf(@old_timestamp_files) if @old_timestamp_files; end
      def all_related_copiers; EMPTY_ARRAY; end
    end

    class RelativeCopy < CopyWithTimestamp
      attr_reader :related_copiers
      def setup
        related_filenames = []
        base_dir = File.dirname(@src)
        open(src, 'r') do |f|
          f.each_with_index do |line, index|
            do_if_available(base_dir, line, index) do |line, index, existing_externals|
              if existing_externals
                 related_filenames << existing_externals.keys.uniq
              end
            end
          end
        end
        filenames = related_filenames.flatten.compact
        @related_copiers = @context.new_copiers(filenames, base_dir)
        @related_copiers
      end

      def execute
        base_dir = File.dirname(@src)
        open(@src, 'r') do |src_file|
          open(@dest, 'w') do |dest_file|
            src_file.each_with_index do |line, index|
              do_if_available(base_dir, line, index) do |line, index, existing_externals|
                if existing_externals
                  new_line = line.dup
                  existing_externals.keys.uniq.sort.reverse.each do |external|
                    new_line.gsub!(external, @context.format_dest(external))
                  end
                  dest_file.puts(new_line)
                else
                  dest_file.puts(line)
                end
              end
            end
          end
        end
      end

      def do_if_available(base_dir, line, line_no)
        externals = find_externals(line).flatten.compact.map do |filepath|
          filepath.gsub(/^\"|\"/, '').gsub(/^\'|\''/, '')
        end
        if externals && !externals.empty?
          existing_externals = {}
          externals.each_with_index do |external, index|
            next unless external
            next if File.extname(external).blank? # 拡張子のないファイルは無視
            external.sub!(/\?.*$/, '')
            external_src = @context.join_path(base_dir, external)
            if File.exist?(external_src)
              existing_externals[external] = external_src
            else
              @file_not_founds ||=  []
              @file_not_founds << "In #{@src}[#{line_no + 1}] #{external} no file uploaded."
            end
          end
          yield(line, line_no, existing_externals)
        else
          yield(line, line_no, nil)
        end
      end

      def all_related_copiers
        @related_copiers + @related_copiers.map{|copier| copier.all_related_copiers}.flatten.compact
      end

      private
      def find_externals(line)
        raise NoMethodError, 'not implemented'
      end
    end

    class CopyHtml < RelativeCopy
      patterns =
        ['\'', '"'].map{|quote|
        q = Regexp.escape(quote)
        %w(href link src).map{|attr|
          /#{attr}\=#{q}(.*?)#{q}/i} }.flatten
      REGEXP = Regexp.union(*patterns)

      def find_externals(line)
        line.scan(REGEXP)
      end
    end

    class CopyCss < RelativeCopy
      REGEXP = Regexp.union(
        /\@import\s+?\"(.*?)\"/,
        /\@import\s+?\'(.*?)\'/,
        /url\((.*?)\)/i)
      def find_externals(line)
        line.scan(REGEXP)
      end
    end



    COPIERS_BY_EXT = {
      '.html' => CopyHtml,
      '.htm' => CopyHtml,
      '.css' => CopyCss
    }

    # S3にアップロードするものを作成して、実際にアップロードを行います。
    def create_and_upload_to_s3(untimestamped_files, options = {})
      if EcRider.config.s3_access_key_id && EcRider.config.s3_secret_access_key
        AWS::S3::Base.establish_connection!(
          :access_key_id     => EcRider.config.s3_access_key_id,
          :secret_access_key => EcRider.config.s3_secret_access_key)
        timestamped_files = create_timestamped_files(untimestamped_files, options)
        timestamped_files.each do |timestamped_file|
          delete_from_s3(format_file_name(:cut, timestamped_file))
          AWS::S3::S3Object.store(
            timestamped_file,
            open(File.join(EcRider.config.ec_front_root, 'public', timestamped_file), "r+b"),
            EcRider.config.s3_bucket_name, :access => :public_read)
        end
        timestamped_files
      end
    end


    # DEPRECATED
    # S3にアップロードします
    #
    def upload_to_s3(public_no_version_file, options = {})
      delete_from_s3(public_no_version_file)

      public_version_file = format_file_name(:add, public_no_version_file, :timestamp => options[:timestamp])
      FileUtils.cp(public_no_version_file, public_version_file)

      if EcRider.config.s3_access_key_id && EcRider.config.s3_secret_access_key
        AWS::S3::Base.establish_connection!(
          :access_key_id     => EcRider.config.s3_access_key_id,
          :secret_access_key => EcRider.config.s3_secret_access_key)
        s3_file = public_version_file[public_version_file.index("\/public\/")+8..-1]
        AWS::S3::S3Object.store(s3_file, open(public_no_version_file, "r+b"), EcRider.config.s3_bucket_name, :access => :public_read)
      end
    end

    #
    # S3から削除します
    #
    def delete_from_s3(public_no_version_file)
      Dir.glob(format_file_name(:find, public_no_version_file)).each do |public_version_file|

        if EcRider.config.s3_access_key_id && EcRider.config.s3_secret_access_key
          AWS::S3::Base.establish_connection!(
            :access_key_id     => EcRider.config.s3_access_key_id,
            :secret_access_key => EcRider.config.s3_secret_access_key)
          s3_file = public_version_file[public_version_file.index("\/public\/")+8..-1]
          AWS::S3::S3Object.delete(s3_file, EcRider.config.s3_bucket_name)
        end
        FileUtils.rm_rf(public_version_file)
      end
    end

    CLOUD_FRONT_VERSION_FORMAT = '%Y%m%d%H%M%S'.freeze
    CLOUD_FRONT_VERSION_QUERY = '??????????????'.freeze

    #
    # ファイル名をフォーマットします。
    # :add -> バージョンを付加した文字列を返します。
    # :cut -> バージョンを削除した文字列を返します。
    # :find -> 検索用の文字列を返します。
    #
    def format_file_name(operation, file_path, options = nil)
      return nil unless file_path
      if dot_index = file_path.rindex('.')
        # basename に File.basenameを使うとディレクトリ名も取り除かれてしまうので、rindexで。
        basename = file_path[0..dot_index -1]
        ext = file_path[dot_index..-1] # -1で最後までを表します
      else
        basename = file_path
        ext = nil
      end
      case operation
      when :add
        options ||= {}
        options[:timestamp] ||= Time.now
        if (d = options[:timestamp]).respond_to?(:strftime)
          options[:timestamp] = d.strftime(CLOUD_FRONT_VERSION_FORMAT)
        end
        '%s-%s%s' % [basename, options[:timestamp], ext]
      when :cut
        hyphen_pos = file_path.rindex('-')
        hyphen_pos ? '%s%s' % [file_path[0..hyphen_pos - 1], ext] : file_path
      when :find
        '%s-%s%s' % [basename, CLOUD_FRONT_VERSION_QUERY, ext]
      end
    end

    #
    # ec_front/publish以下のCloudFrontへアップロードするファイルを返します。
    #
    def self.files_to_upload(all = false)
      ignore_exts = %w(.rb .cgi .fcgi .rhtml .erb)
      pattern = "*-#{EcRider::CloudFront::CLOUD_FRONT_VERSION_QUERY}.*"
      uploadeds = Dir.glob(File.join(EcRider.config.ec_front_root, 'public', '**', pattern))
      files = Dir.glob(File.join(EcRider.config.ec_front_root, 'public', '**', '*.*'))
      files = files - uploadeds

      if all
        files = files.reject do |filename|
          filename.include?('.svn') ||
          filename =~ /\~$/ ||
          ignore_exts.include?(File.extname(filename).downcase)
        end
      else
        files = files.reject do |filename|
          filename.include?('.svn') ||
          filename.include?('public/ec_rider/product_images') ||
          filename.include?('public/banner_images') ||
          filename =~ /\~$/ ||
          ignore_exts.include?(File.extname(filename).downcase)
        end
      end
      files
    end
  end

end
