# -*- coding: utf-8 -*-
module EcRider
  module CmsSetup
    include EcRider::PluginModule::Extendable

    #
    # ec_rider提供ページとしてCMSで編集可能なビューが置いてあるディレクトリ
    # プラグインで追加する場合はself.ecr_controllersを拡張します
    #
    # ※注意※
    # ・コントローラはあっても、その名前のディレクトリにビューが無い場合は登録する必要はありません
    # ・コントローラは無くても、その名前のディレクトリにビューがある場合は登録してください
    #
    ECR_CONTROLLERS = ['account', 'cart', 'inquiry', 'member', 'order', 'reminder', 'resigned_reason', 'thanks']
    def self.ecr_controllers
      ECR_CONTROLLERS
    end

    #
    # DBにプロジェクト情報を読み込みます（プラグイン拡張用）
    # プラグインでデフォルトプロジェクト用のXMLファイルを追加した場合はここを拡張します。
    #
    def db_data_load_for_plugins
    end

    #
    # ベースプロジェクト以外のプロジェクトのデータを削除します。
    # ベースプロジェクトを初期化します。
    #
    def cleanup_project
      target_tables = [
        'cms_publishes', 
        'cms_published_projects',
        'cms_projects', 
        'cms_project_approvals',
        'cms_project_assigns',
        'cms_templates',
        'cms_template_contents',
        'cms_pages',
        'cms_page_titles',
        'cms_binds', 
        'cms_bind_items',
        'cms_bind_contents'
      ]
      target_tables.each do |table|
        CmsProject.connection.execute("truncate #{table};")
      end
    end

    #
    # DBにプロジェクト情報を読み込みます。
    # initila_data/db_data以下のYAMLファイルを登録します。
    #
    def db_data_load
      require 'active_record/fixtures'
      ActiveRecord::Base.establish_connection(RAILS_ENV.to_sym)
      Dir.glob(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'db_data', '*.{yml,csv}')).each do |fixture_file|
        Fixtures.create_fixtures(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'db_data'), File.basename(fixture_file, '.*'))
      end
    end

    #
    # ベースプロジェクトのec-rider提供ページをデータベースに登録します。
    # initial_data/cms/ecrider_pages以下のファイルを登録します。
    #
    def create_ecrider_cms_pages
      EcRider::CmsSetup.ecr_controllers.each do |controller|
        files = Dir.glob(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'ecrider_pages', controller, "*.xml"))

        # プラグイン内のinitial_dataも取り込みます。
        ecr_plugin_for_admins.each do |ecr_plugin_root|
          EcRider::CmsSetup.ecr_controllers.each do |controller|
            files << Dir.glob(File.join(ecr_plugin_root, 'initial_data', 'cms', 'ecrider_pages', controller, '*.xml'))
          end
        end
        files.flatten!

        update_template_content_from_fixture(files)
      end
    end

    #
    # ベースプロジェクトの店舗独自ページをデータベースに登録します。
    # initial_data/cms/cms_pages以下のファイルをすべて登録します。
    #
    def create_shop_base_cms_pages
      files = Dir.glob(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_pages', '*'))

      # プラグイン内のinitial_dataも取り込みます。
      ecr_plugin_for_admins.each do |ecr_plugin_root|
        files << Dir.glob(File.join(ecr_plugin_root, 'initial_data', 'cms', 'cms_pages', '*'))
      end
      files.flatten!

      files.reject!{|f| f =~ /\/\.{1,2}$/}
      update_template_content_from_fixture(files)
    end

    #
    # ベースプロジェクトの中央以外のテンプレートをデータベースに登録します。
    # initial_data/cms/layouts以下のファイルを登録します。
    #
    def create_shop_base_cms_templates
      CmsTemplate.display_position_keys.each do |position|
        files = Dir.glob(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_layouts', "#{position}*.xml"))

        # プラグイン内のinitial_dataも取り込みます。
        ecr_plugin_for_admins.each do |ecr_plugin_root|
          files << Dir.glob(File.join(ecr_plugin_root, 'initial_data', 'cms', 'cms_layouts', "#{position}*.xml"))
        end
        files.flatten!

        update_template_content_from_fixture(files)
      end
      # topとbottomをデフォルトテンプレートに設定します
      template = CmsTemplate.find(:first, :conditions => ['development_code = ?', 'cms/top'])
      CmsDefaultTemplate.create_or_update_default_template(1, template.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template.template_code)

      template = CmsTemplate.find(:first, :conditions => ['development_code = ?', 'cms/bottom'])
      CmsDefaultTemplate.create_or_update_default_template(1, template.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:bottom), template.template_code)

    end
    
    #
    # test/fixtures/public以下のファイルをベースプロジェクトのファイルとして登録します。
    #
    def create_base_cms_file
      dest_root_dir = File.join(EcRider.config.ec_admin_root, 'public', 'projects', CmsProject::BASE_PROJECT_ID.to_s)

      src_root_dir = File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms')
      src_files = Dir.glob(File.join(src_root_dir, 'public', "**", '*'))
      src_files_hash = {src_root_dir => src_files}

      # プラグイン内のinitial_dataも取り込みます。
      ecr_plugin_for_admins.each do |ecr_plugin_root|
        src_files_hash[ecr_plugin_root] = Dir.glob(File.join(ecr_plugin_root, 'initial_data', 'cms', 'public', "**", '*'))
      end

      src_files_hash.each do |root_dir, files|
        files.each do |src_file|
          next if File.basename(src_file) =~ /^\./
          if File.directory?(src_file)
            # ディレクトリの場合は末尾に/をつける
            public_file_path = src_file.sub(root_dir + '/', '') + '/'
            dest_file = File.join(dest_root_dir, public_file_path)
            FileUtils.mkdir_p(dest_file) unless File.exist?(dest_file)
            # ディレクトリの場合はcms_project_id=nil
            cms_project_id = nil
          else
            public_file_path = src_file.sub(root_dir + '/', '')
            dest_file = File.join(dest_root_dir, public_file_path)
            FileUtils.mkdir_p(File.dirname(dest_file)) unless File.exist?(File.dirname(dest_file))
            FileUtils.cp(src_file, dest_file)
            # ファイルの場合はcms_project_id=ベースプロジェクトのID
            cms_project_id = CmsProject::BASE_PROJECT_ID
          end
          unless CmsFile.exists?(:cms_project_id => cms_project_id, :public_file_path => public_file_path)
            CmsFile.create(:cms_project_id => cms_project_id, :public_file_path => public_file_path)
          end
        end
      end
    end

    private
    # EC-Riderが想定している（initial_dataのテンプレートに含まれている）ロケール群から、
    # 日本語を除いたロケールのXMLのファイル名にマッチする正規表現
    ECR_KNOWN_FOREIGN_LOCALES_REGEXP = Regexp.union(
      %w(en ko th zh_CN zh_TW mobile).map{|locale| /_#{locale}\.xml$/}
    )

    # 実際に店舗担当者が扱うロケール群
    SUPPORED_LOCALES = I18n.supported_locales.
      reject{|locale| locale.to_s == 'ja'}.map{|locale| locale.to_s.gsub('-', '_')}

    #
    # ファイルの内容でテンプレートを更新します。
    #
    def update_template_content_from_fixture(files)
      files_ja = files.reject{|file| file =~ ECR_KNOWN_FOREIGN_LOCALES_REGEXP}
      files_ja.each do |file_ja|
        controller = (File.dirname(file_ja) =~ /\/cms_layouts$|\/cms_pages$/) ? 'cms' : "#{File.dirname(file_ja).split('/').last}"
        if template = CmsTemplate.find(:first, :include => :contents,
            :conditions => ["cms_project_id = ? and development_code = ?", CmsProject::BASE_PROJECT_ID, "#{controller}/#{File.basename(file_ja, '.xml')}"])

          template.content_ja.descriptions = IO.read(file_ja)
          template.content_ja.save!
          SUPPORED_LOCALES.each do |foreign_locale|
            if File.exists?(file_ja.sub(".xml", "_#{foreign_locale}.xml"))
              if template.send("content_#{foreign_locale}")
                template.send("content_#{foreign_locale}").descriptions = IO.read(file_ja.sub(".xml", "_#{foreign_locale}.xml"))
                template.send("content_#{foreign_locale}").save!
              else
                template.contents << CmsTemplateContent.new(:locale => foreign_locale.to_s, :descriptions => IO.read(file_ja.sub(".xml", "_#{foreign_locale}.xml")))
              end
            end
          end
          template.save!
        else
          create_cms_template_from_fixture(file_ja)
        end
      end
    end

    #
    # ファイルの内容でテンプレートとページのデータを作成します。
    #
    def create_cms_template_from_fixture(file_ja)
      # cms_templates,cms_template_contents
      controller = (File.dirname(file_ja) =~ /\/cms_layouts$|\/cms_pages$/) ? 'cms' : "#{File.dirname(file_ja).split('/').last}"
      cms_template = CmsTemplate.new(
        :name => "#{controller}/#{File.basename(file_ja, '.xml')}",
        :display_position => CmsTemplate.display_position_id_by_key(:center),
        :cms_project_id => CmsProject::BASE_PROJECT_ID,
        :development_code => "#{controller}/#{File.basename(file_ja, '.xml')}")

      cms_template.contents << CmsTemplateContent.new(:locale => 'ja', :descriptions => IO.read(file_ja))
      SUPPORED_LOCALES.each do |foreign_locale|
        if File.exists?(file_ja.sub(".xml", "_#{foreign_locale}.xml"))
          cms_template.contents << CmsTemplateContent.new(:locale => foreign_locale.to_s, :descriptions => IO.read(file_ja.sub(".xml", "_#{foreign_locale}.xml")))
        end
      end
      cms_template.template_code = Time.now.strftime("%Y%m%d%H%M%S") + cms_template.object_id.abs.to_s
      begin
        cms_template.save!
      rescue
        ActiveRecord::Base.logger.error("#{$!.to_s}\n#{cms_template.inspect}\n  " << $!.backtrace.join("\n  ") )
        raise
      end

      if File.dirname(file_ja) !~ /\/layouts$/
        # cms_pages,cms_page_titles,cms_binds
        cms_page = CmsPage.new(:cms_project_id => CmsProject::BASE_PROJECT_ID, :controller => controller, :action => File.basename(file_ja, ".xml"))
        cms_page.binds << CmsBind.new(:cms_template_id => cms_template.id, :position_key => :center)
        CmsBind.position_keys.each{|key| cms_page.binds << CmsBind.new(:position_key => key) unless key == :center}
        I18n.supported_locales.each do |locale|
          cms_page.titles << CmsPageTitle.new(:locale => locale.to_s, :name => File.basename(file_ja, ".xml"))
        end

        begin
          cms_page.save!
        rescue
          ActiveRecord::Base.logger.error("#{$!.to_s}\n#{cms_page.inspect}\n  " << $!.backtrace.join("\n  ") )
          raise
        end
      end
    end

    def ecr_plugin_for_admins
      return @ecr_plugin_for_admins if @ecr_plugin_for_admins
      ecr_plugins_config_path = File.expand_path(File.join(RAILS_ROOT, 'ecr_plugins.yml'))
      return @ecr_plugin_for_admins = [] unless File.exist?(ecr_plugins_config_path)

      ecr_plugins = YAML.load_file(ecr_plugins_config_path) || {}
      @ecr_plugin_for_admins = ecr_plugins.values.flatten.map{|plugin_path| "#{EcRider.config.ec_admin_root}/#{plugin_path}"}
    end

    public
    
    # プラグインで初期データを投入するためのモジュール
    class InitialData
      attr_accessor :cms_template, :cms_template_content, :cms_page, :cms_page_title, :cms_binds
      
      # jaとdocomoのテンプレートのみ対応しています。その他のロケールに対応するためには修正が必要です。
      # positionが中央以外の場合はページ情報は登録しません。
      def self.setup(terminal, controller, action, name, code, xml_path=nil, locale='ja', position_key=:center)
        initialdata = EcRider::CmsSetup::InitialData.new()
        initialdata.cms_template = CmsTemplate.new(
          :cms_project_id => 1,
          :display_position_key => position_key,
          :name => name,
          :terminal_key => terminal,
          :template_code => code,
          :development_code => "#{controller}/#{action}"
        )
        if position_key == :center
          initialdata.cms_page = CmsPage.new(:cms_project_id => 1, :controller => controller, :action => action, :terminal_key => terminal)
          initialdata.setup_binds
        end
        initialdata.setup_locale(name, xml_path, locale)
        return initialdata.save
      end

      # 既存のテンプレート、ページに対して、ロケールを追加します
      def self.add_locale(terminal, controller, action, name, code, xml_path=nil, locale='ja')
        initialdata = EcRider::CmsSetup::InitialData.new()
        terminal_cd = CmsTemplate.terminal_id_by_key(terminal)
        initialdata.cms_template = CmsTemplate.find(:first, :conditions => {:cms_project_id => 1, :template_code => code, :terminal => terminal_cd})
        initialdata.cms_page = CmsPage.find(:first, :conditions => {:cms_project_id => 1, :controller => controller, :action => action, :terminal => terminal_cd})
        initialdata.setup_locale(name, xml_path, locale)
        return initialdata.save
      end

      def setup_binds
        @cms_binds = []
        [:center, :top, :bottom, :left, :right].each do |position|
          if cms_template.terminal_key == :pc || position == :center || position == :top || position == :bottom
            cms_bind = CmsBind.new(:position_key => position)
            cms_bind.cms_page = cms_page
            cms_bind.cms_template = cms_template if position == :center
            @cms_binds << cms_bind
          end
        end
      end
      
      def setup_locale(name, xml_path=nil, locale='ja')
        @cms_template_content = CmsTemplateContent.new(:locale => locale)
        @cms_template_content.cms_template = cms_template
        @cms_template_content.descriptions = (xml_path && File.exist?(xml_path)) ? IO.read(xml_path) : ''
        if cms_page
          @cms_page_title = CmsPageTitle.new(:name => name, :locale => locale)
          @cms_page_title.cms_page = cms_page
        end
      end

      def save
        result = false
        CmsTemplate.transaction do
          cms_template.save!
          cms_template_content.save!
          cms_page.save! if cms_page
          cms_page_title.save! if cms_page_title
          cms_binds.each {|bind| bind.save!} if cms_binds
          result = true
        end
        result
      end
    end
  end
end
