# -*- coding: utf-8 -*-
# Condition Builder for ActiveRecord
#
# (c) 2005-2006 Ezra Zygmuntovic, Jens-Christian Fischer and Duane Johnson
# distributed under the same license as Ruby On Rails
#
# Version 1.1
#
# Jan 4, 2007: Added "AND" and "OR" using 'and' and 'or'.  Also added ability to use sub-conditions
#   e.g.
#   Condition.block { |c|
#     c.and "user_id", user_id
#     c.and { |d|
#       d.or "admin", 1
#       d.or "joined_at", "<", 10.days.ago
#     } unless @dont_override_user_id
#   }
#

module EcRider
  class Condition
    attr_reader :args

    FILTERS = { 
      :ignore_nil => Proc.new{|column, *vals| !val.last.nil?},
      :ignore_blank => Proc.new{|column, *vals| !vals.last.blank?},
      :ignore_like_with_double_percent => Proc.new do |args, operator, value| 
        if operator and value
          !((operator =~ /like/i) and (value == '%%'))
        else
          true
        end
      end
    }

    private
    def symbol_to_filter(filter)
      (filter.is_a?(Symbol) ? FILTERS[filter] : filter) or raise ArgumentError, "Unknown filter named #{filter.inspect}"
    end
    def symbol_to_filters(filters)
      filters.map{|f| symbol_to_filter(f)}.flatten
     end

     public
     def self.block(*filters, &block)
       new(*filters, &block).where
     end

     def initialize(*filters, &block)
       @args = []
       @filters = [symbol_to_filter(:ignore_like_with_double_percent)] + symbol_to_filters(filters)
      # リファクタが進んだ暁にはこうしたい！
      # instance_eval(&block) if block_given?
      yield(self) if block_given?
    end

    # args are ...
    #   1. column_name, value
    #   2. column_name, operator, value
    #   3. column_name, 'BETWEEN', value1, value2
    def and(*args, &block)
      # raise ArgumentError, "No value specified for condition" if args.empty? && !block_given?
      @logic = " AND "
      if block_given?
        block(&block)
      else
        @args << args if filters.all?{|filter| filter.call(args)}
      end
      self
    end
    alias :add :and

    # args are ...
    #   1. column_name, value
    #   2. column_name, operator, value
    #   3. column_name, 'BETWEEN', value1, value2
    def or(*args, &block)
      # raise ArgumentError, "No value specified for condition" if args.empty? && !block_given?
      @logic = " OR "
      if block_given?
        block(&block)
      else
        @args << args if filters.all?{|filter| filter.call(args)}
      end
      self
    end

    def add_like_partial(*args, &block)
      percent_args_for_like('%', '%', *args) do |filtered_args|
        add(*filtered_args, &block)
      end
    end

    def add_like_forward(*args, &block)
      percent_args_for_like(nil, '%', *args) do |filtered_args|
        add(*filtered_args, &block)
      end
    end

    def add_like_backward(*args, &block)
      percent_args_for_like('%', nil, *args) do |filtered_args|
        add(*filtered_args, &block)
      end
    end
    alias_method :add_like, :add_like_partial

    private
    def percent_args_for_like(forward, backward, *args)
      value = args.pop
      return self unless value
      args << "#{forward}#{value}#{backward}"
      yield(args) if block_given?
      return self
    end

    public
    def block(*filters, &block)
      @args << self.class.new(*filters, &block)
    end

    def filters
      @filters ||= []
    end

    def filter(*callables)
      symbol_to_filters(callables).each{|f| filters << f}
      self
    end


    def where(conditions = @args)
      return nil if conditions.empty?
      # Build the condition string with ?s using the 'left' array, and
      # build the value string using the 'right' array:
      left, right = [], []
      conditions.each do |column, *values|
        if (sub_condition = column).is_a?(Condition)
          # Integrate the sub-condition
          sub_sql, *sub_values = sub_condition.where
          if sub_sql
            left << "(#{sub_sql})"
            right += sub_values
          end
          next
        elsif column.to_s.downcase == "sql"
          # Treat the first 'value' as pure SQL
          left << values.shift
          next
        end
        case values.size
        when 0
          raise ArgumentError, "No value specified for Condition"
        when 1
          if values.last.is_a?(Array)
            left << "#{column} IN (?)"
          else
            left << "#{column} = ?"
          end
          right << values.last
        when 2
          operator = values.shift
          if values.last.is_a?(Array)
            left << "#{column} IN (?)"
          else
            left << "#{column} #{operator} ?"
          end
          right << values.last
        else
          operator = values.first
          if operator.upcase == "BETWEEN"
            left << "#{column} #{operator} ? AND ?"
            right << values[-2] << values[-1]
          else
            raise ArgumentError, "Unknown operator for multiple values in Condition: #{operator}"
          end
        end
      end
      return [left.compact.join(@logic)].concat(right.compact)
    end
  end
end
