# -*- coding: utf-8 -*-

# テストを実行する際はschema_commentsのschema_comments.ymlへの出力を抑制します。
Rake.application.send(:eval, "@tasks.delete('db:test:prepare')")
namespace :db do
  namespace :test do
    desc 'Check for pending migrations and load the test schema'
    task :prepare => 'db:abort_if_pending_migrations' do
      SchemaComments.quiet = true
      puts "SchemaComments.quiet => #{SchemaComments.quiet.inspect}"
      if defined?(ActiveRecord) && !ActiveRecord::Base.configurations.blank?
        Rake::Task[{ :sql  => "db:test:clone_structure", :ruby => "db:test:load" }[ActiveRecord::Base.schema_format]].invoke
      end
    end
  end

  desc 'Migrate the database include ECR plugins and Load schema_comments'
  task :migrate_all => :environment do
    EcRider::SchemaCommentsExt.dump_schema_comments
    Rake::Task["db:migrate"].invoke
    Rake::Task["ecr:plugins:migrate_all"].invoke
    EcRider::SchemaCommentsExt.load_schema_comments
  end

  namespace :schema_comments do
    desc 'Load schema_comments from yaml to database'
    task :load => :environment do
      EcRider::SchemaCommentsExt.load_schema_comments
    end

    desc 'Dump schema_comments from database to yaml'
    task :dump => :environment do
      EcRider::SchemaCommentsExt.dump_schema_comments
    end
  end
end

require 'fileutils'
namespace :ecr do

  def symlink_or_mkdir_p(src, dest)
    if RUBY_PLATFORM =~ /mswin|mingw/
      FileUtils.mkdir_p(dest, :verbose => true)
    else
      FileUtils.mkdir_p(src, :verbose => true) unless File.exist?(src)
      FileUtils.rm_rf(dest, :verbose => true) if File.exist?(dest)
      FileUtils.ln_s(File.expand_path(src), File.expand_path(dest), :verbose => true)
    end
  end

  desc "setup cms_pages, cms_layouts and cms_files"
  task :cms => [
    :"ecr:cms:create_git_dir",
    :"ecr:cms:symlinks",
    :"ecr:cms:deploy",
    :"ecr:cms:deploy_all_projects",
    :"ecr:cms:copy_fixtures"]

  namespace :cms do
    desc "create git dir"
    task :create_git_dir => :environment do
      git_dir = ENV['ECR_CMS_GIT_DIR'] || File.join(RAILS_ROOT, '..', 'cms_under_git')
      unless File.exist?(git_dir)
        puts "cd #{git_dir} && git init && git add . %% git commit -a -m 'initial commit'"
      end
    end

    desc "create_symlink_for_ec_test"
    task :create_symlink_for_ec_test => :environment do
      log_error_or_success("rake ecr:cms:create_symlink_for_ec_test") do
        CmsPublish.create_symlink_for_ec_test
      end
    end

    desc "symlinks to cms_git_dir"
    task :symlinks => [:"ecr:cms:symlinks:ec_admin", :"ecr:cms:symlinks:ec_front"]

    namespace :symlinks do
      desc "symlinks to cms_git_dir for ec_admin"
      task :ec_admin => :environment do
        # プレビューを行う際に必要なファイルは本来ec_frontと同じものである必要がありますが、
        # 本番時はcapistranoでのデプロイ時にsymlinkが張られるので、以下の処理は必要ありません。
        git_dir = ENV['ECR_CMS_GIT_DIR'] || File.join(RAILS_ROOT, '..', 'cms_under_git')
        symlink_or_mkdir_p(
          File.join(git_dir, 'public_cms'),
          File.join(EcRider.config.ec_admin_root, 'public', 'cms'))
      end

      desc "symlinks to cms_git_dir for ec_front"
      task :ec_front => :environment do
        # ec_admin側のapp/views/cmsはCMSのバージョン管理下にはなく、
        # subversionで管理されます。
        # # ただし、プレビュー実行時に常に最新を取得して更新してから使われるので、
        # # 内容は全てsvn:externalsの対象になります。
        # よって、セットアップ時にはディレクトリを作成する必要はありません。
        git_dir = ENV['ECR_CMS_GIT_DIR'] || File.join(RAILS_ROOT, '..', 'cms_under_git')
        symlink_or_mkdir_p(
          File.join(git_dir, 'app_views', 'cms'),
          File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms'))
        symlink_or_mkdir_p(
          File.join(git_dir, 'public_cms'),
          File.join(EcRider.config.ec_front_root, 'public', 'cms'))
      end
    end

    desc "copy cms_files in test/fixtures"
    task :copy_fixtures => :environment do
      dest_dir = File.join(EcRider.config.ec_front_root, *"public/cms".split('/'))
      Dir[File.join(dest_dir, '*')].each do |file|
        FileUtils.rm_rf(file, :verbose => true)
      end
      src_dir = File.join(RAILS_ROOT, *"test/fixtures/cms_files".split('/'))
      Dir[File.join(src_dir, '*')].each do |file|
        FileUtils.cp_r(file, File.join(EcRider.config.ec_front_root, *"public/cms".split('/')), :verbose => true)
      end
    end

    desc "backup cms_git_dir"
    task :backup => :environment do
      git_dir = ENV['ECR_CMS_GIT_DIR'] || File.join(RAILS_ROOT, '..', 'cms_under_git')
      message = "バックアップ at #{Time.now.strftime('%Y-%m-%d %H:%M:%S')}".to_s
      git = Git.open(git_dir)
      # 追加、編集するファイルを登録する
      git.add('.')
      # 削除するファイルを登録する
      deleted_files = git.status.deleted.map{|item| item.first.gsub(/\\(\d{3})/){ $1.to_i(8).chr } }
      git.remove(deleted_files) unless deleted_files.empty?
      # コミットする
      git.commit(message)
    end

    def log_error_or_success(name)
      ActiveRecord::Base.logger.info("#{name} begin"); puts("#{name} begin")
      begin
        yield
        ActiveRecord::Base.logger.info("#{name} success!"); puts("#{name} success!")
      rescue Exception => error
        msg = "#{name} failed!\n" << error.to_s << "\n  " << error.backtrace.join("\n  ")
        ActiveRecord::Base.logger.info(msg); puts(msg)
        raise error
      end
    end

    # ecr:cms:initialize 
    # DBを、CMSプロジェクトが１つだけ存在するように初期化します。
    desc "initialize cms data on DB"
    task :initialize => :environment do
      log_error_or_success("rake ecr:cms:initialize") do
        ecr_page = Object.new
        ecr_page.extend(::EcRider::CmsSetup)
        ecr_page.cleanup_project
        # YAMLからロード
        ecr_page.db_data_load
        # DBからロード（プラグイン拡張分）
        ecr_page.db_data_load_for_plugins
        puts "creating ecrider cms page data"
        ecr_page.create_ecrider_cms_pages
        puts "creating base project shop cms page data"
        ecr_page.create_shop_base_cms_pages
        puts "creating base project shop cms template data"
        ecr_page.create_shop_base_cms_templates
        puts "creating base project cms file data"
        ecr_page.create_base_cms_file
      end
    end
    
    # test/fixtures/public以下のファイルをベースプロジェクトのファイルとして登録します。
    desc "add base project cms file from fixture"
    task :add_base_cms_file => :environment do
      ecr_setup = Object.new
      ecr_setup.extend(::EcRider::CmsSetup)
      ecr_setup.create_base_cms_file
    end
    
    # ecr:cms:publish_production
    # DB上のCMSプロジェクトの状態に従って、CmsPublishを生成して、それに基づいてファイルを公開して、CloudFrontにアップロードします
    desc "publish cms_pages and cms_files. and upload to CloudFront"
    task :publish_production => :environment do
      log_error_or_success("rake ecr:cms:publish_production") do
        CmsPublish.create_and_execute(:target_key => :production, 
          :cloud_front_upload => true)
      end
    end
    
    # ecr:cms:publish
    # DB上のCMSプロジェクトの状態に従って、CmsPublishを生成して、それに基づいてファイルを公開ます
    desc "publish cms_pages and cms_files. and NOT upload to CloudFront"
    task :publish => :environment do
      log_error_or_success("rake ecr:cms:publish") do
        CmsPublish.create_and_execute(:target_key => :production, 
          :cloud_front_upload => false)
      end
    end

    # ecr:cms:publish_test
    # 既に登録されている(はずの)DB上のCmsPublishを元に、テスト環境にファイルを公開します。
    desc "publish cms_pages and cms_files to test environment"
    task :publish_test => :environment do
      log_error_or_success("rake ecr:cms:publish_test") do
        CmsPublish.execute_if_waiting(:target_key => :test)
      end
    end

    # ecr:cms:send_reload_request_production
    # production(shop)のビューをリロードさせるリクエストを送ります。
    desc "send http request to reload views of production"
    task :send_reload_request_production => :environment do
      log_error_or_success("rake ecr:cms:send_reload_request_production") do
        CmsPublish.send_reload_request(:production)
      end
    end

    # ecr:cms:send_reload_request_test
    # test(shop)のビューをリロードさせるリクエストを送ります。
    desc "send http request to reload views of test"
    task :send_reload_request_test => :environment do
      log_error_or_success("rake ecr:cms:send_reload_request_test") do
        CmsPublish.send_reload_request(:test)
      end
    end



    # ecr:cms:link_check
    # テスト環境に公開されているコンテンツのリンクチェックを行います。
    desc "execute link check of test environment"
    task :link_check => :environment do
      CmsDeadLinkCheck.execute_if_waiting
    end

  end


  namespace :plugins do
    # ecr_plugins.ymlの例)
    # ---
    # - ecr_plugins/ecr_standard_plugin1
    # - ecr_plugins/ecr_standard_plugin1_admin

    app_root_path = File.expand_path(File.join(File.dirname(__FILE__), '../..'))
    this_app = File.basename(app_root_path)

    svn_repo = "svn://lions/opt/svnroot/xec"
    master_plugins_path = "/master/#{this_app}/vendor/plugins"
    ecr_plugins_config_path = File.expand_path(File.join(app_root_path, 'ecr_plugins.yml'))

    desc "svn propset svn:externals on vendor/plugins by using config/ecr_plugins.yml and #{master_plugins_path} to "
    task :svn_externals do
      master_plugins = `svn ls #{svn_repo}#{master_plugins_path}`.split(/$/).
        map{|s| s.strip}.select{|s| s =~ /\/$/}
      prop_value = master_plugins.
        map{|d| name = d.sub(/\/$/, '').strip; "^#{master_plugins_path}/#{name} #{name}" }.
        join("\n")
      if File.exist?(ecr_plugins_config_path)
        ecr_plugins = YAML.load_file(ecr_plugins_config_path) || {}
        prop_value << "\n"
        prop_value << ecr_plugins.values.flatten.map{|plugin_path| "../../#{plugin_path} #{File.basename(plugin_path)}"}
      end
      cmd = "svn propset svn:externals \"#{prop_value}\" vendor/plugins"
      # puts cmd
      system(cmd)
    end
    
    desc "delete symlinks and set symlinks by using ecr_plugins.yml"
    task :symlink do
      plugin_paths = Dir['vendor/plugins/ecr_*']
      symlink_paths = plugin_paths.select{|path| File.symlink?(path)}
      # symlinkのはずだけど、symlinkじゃなかったら警告
      undeleted_paths = plugin_paths - symlink_paths
      unless undeleted_paths.empty?
        puts "WARNING: these plugins aren't symlink: #{undeleted_paths}"
      end
      # 一旦すべてのECRプラグインのsymlinkを削除
      symlink_paths.each do |plugin_path|
        FileUtils.rm(plugin_path, :verbose => true)
      end
      # で、symlinkを貼り直す
      ecr_plugins = YAML.load_file(ecr_plugins_config_path) || {}
      ecr_plugins.values.flatten.each do |plugin_path|
        FileUtils.ln_sf(
          File.join(app_root_path, plugin_path),
          File.join(app_root_path, 'vendor/plugins', File.basename(plugin_path)),
          :verbose => true
          )
      end
    end

    desc "remove all ECR plugins and copy available ECR plugins"
    task :reset => :reset_soft

    # 不要なECRプラグインを削除して、必要でかつまだコピーされていないECRプラグインをコピー
    desc "remove unavailable ECR plugins and copy available uncopied ECR plugins"
    task :reset_soft => [:delete_unavailable, :copy]
    
    # ECRプラグインを一度すべて消して、必要なものをコピー
    desc "remove all ECR plugins and copy available ECR plugins"
    task :reset_hard => [:delete_all, :copy]

    # すべてのECRプラグインをvendor/pluginsから削除
    desc "delete all ECR plugins"
    task :delete_all do
      current_plugin_paths = Dir[File.join(app_root_path, 'vendor/plugins/ecr_*')]
      current_plugin_paths.each do |plugin_path|
        FileUtils.rm_rf(plugin_path, :verbose => true)
      end
    end

    # 不要なECRプラグインを削除
    desc "delete unavailable ECR plugins"
    task :delete_unavailable do
      current_plugin_paths = Dir[File.join(app_root_path, 'vendor/plugins/ecr_*')]
      current_plugin_names = current_plugin_paths.map{|path| File.basename(path)}
      available_plugin_paths = YAML.load_file(ecr_plugins_config_path) || {}
      available_plugin_paths = available_plugin_paths.values.flatten
      available_plugin_names = available_plugin_paths.map{|path| File.basename(path)}
      (current_plugin_names - available_plugin_names).each do |plugin_name|
        FileUtils.rm_rf(File.join(app_root_path, 'vendor/plugins', plugin_name), :verbose => true)
      end
    end

    # 有効でかつコピーされていないECRプラグインをコピー
    desc "copy available ECR plugins unless exists"
    task :copy do
      available_plugin_paths = YAML.load_file(ecr_plugins_config_path) || {}
      available_plugin_paths = available_plugin_paths.values.flatten
      available_plugin_paths.each do |plugin_path|
        plugin_name = File.basename(plugin_path)
        unless File.exist?(File.join(app_root_path, 'vendor/plugins', plugin_name))
          src_path = File.join(app_root_path, plugin_path)
          dest_path = File.join(app_root_path, 'vendor/plugins', File.basename(plugin_path))
          FileUtils.cp_r(src_path, dest_path, :verbose => true)
          Dir[File.join(dest_path, '**/.svn')].each do |svn_path|
            FileUtils.rm_rf(svn_path, :verbose => true)
          end
        end
      end
    end
    
    desc "Create tables for current ECR plugins"
    task :migrate_all do
      current_plugin_paths = Dir[File.join(app_root_path, 'vendor/plugins/ecr_*')]
      current_plugin_names = current_plugin_paths.map{|path| File.basename(path)}
      current_plugin_names.each do |plugin_name|
        next unless Rake::Task.task_defined?("#{plugin_name}:db:migrate")
        Rake::Task["#{plugin_name}:db:migrate"].invoke
      end
    end
    
  end


  namespace :development do

    desc "make symlink from ec_front to ec_admin"
    task :symlink do
      ec_admin_root = File.join(File.dirname(__FILE__), '../..')
      ec_admin_path = File.expand_path(ec_front_root)
      ec_front_path = File.expand_path(File.join(ec_front_path, '../ec_front'))
      # 
      cms_under_git_path = File.expand_path(File.join(ec_front_path, '../cms_under_git'))
      raise "ec_front doesn't exist!" unless File.exist?(ec_front_path)
      raise "ec_admin doesn't exist!" unless File.exist?(ec_admin_path)
      raise "cms_under_git doesn't exist!" unless File.exist?(cms_under_git_path)
      symlinks = [
        'app/helpers/ec_rider',
        'app/models',
        # 'app/views/error_mailer',
        'config/deploy',
        # 'config/initializers', # ec_adminとec_frontで別の設定をすることがあるので分けました。
        'config/locales',
        'db',
        'ecr_plugins',
        'lib/ec_rider',
        # 'public/cms',
        # 'public/themes',
        'spec/helpers/ec_rider',
        'spec/models',
        'test/fixtures',
        # 'test/functional/ec_rider',
        # 'test/unit',
        # プラグインは数が多すぎてデプロイに時間がかかりすぎるので、svn:externalsを使わないようにしたので、
        # symlinkも必要なくなりました。
        # 'vendor/plugins/actionmailserver',
        # 'vendor/plugins/active_form',
        # 'vendor/plugins/acts_as_authenticated',
        # 'vendor/plugins/acts_as_paranoid',
        # 'vendor/plugins/ar_fixtures',
        # 'vendor/plugins/attachment_fu',
        # 'vendor/plugins/deep_cloning',
        # 'vendor/plugins/dir_fixtures',
        # 'vendor/plugins/dynamic_session_exp',
        # 'vendor/plugins/exception_notification',
        'vendor/plugins/i18n_erb_backend', # これはecrider用にカスタマイズしているので、他のプラグインとは別扱いです。
        # 'vendor/plugins/jp_address',
        # 'vendor/plugins/rename_model',
        # 'vendor/plugins/rspec',
        # 'vendor/plugins/rspec-rails',
        # 'vendor/plugins/rspec_on_rails',
        # 'vendor/plugins/schema_comments',
        # 'vendor/plugins/selectable_attr',
        # 'vendor/plugins/selenium-on-rails',
        # 'vendor/plugins/ssl_requirement'
      ]

      symlinks.each do |symlink|
        path_in_front = File.join(ec_front_path, symlink)
        if FileTest.symlink?(path_in_front)
          FileUtils.rm_f(path_in_front, :verbose => true)
        end
        FileUtils.ln_sf(
          File.join(ec_admin_path, symlink),
          path_in_front, :verbose => true)
      end
    end

    desc "export action YAML"
    task "export_action_yaml" => :environment do
      Dir.glob(File.join(RAILS_ROOT, "app/controllers/**/*_controller.rb")) do |path|
        require path
      end
      ObjectSpace.each_object(Class) do |klass|
        if klass < ApplicationController
          # 通常はActiveSupport::Dependencies.load_missing_constant をフックしている箇所で自動で
          # prepare_action_descriptionsが実行されますが、ここではrequire しているので、明示的に呼び出しています。
          klass.prepare_action_descriptions
        end
      end
      hash = Prestruct.export(ApplicationController)
      hash.instance_variable_set(:@controller_names, hash.keys.sort)
      hash.extend(ControllerNamedHash)
      open(File.join(RAILS_ROOT, "config/actions.yml"), 'w') do |f|
        YAML.dump(hash, f)
      end
    end


    desc "rebuild goldberg_site_actions & goldberg_site_action_groups. Don't use in production enveironment!!"
    task "rebuild_site_actions" => :environment do
      Goldberg::SiteAction.transaction do
        Goldberg::RoleAction.delete_all
        Goldberg::SiteAction.delete_all
        Goldberg::SiteActionGroup.delete_all
        Goldberg::SiteAction.append_actions_from_desc do
          ObjectSpace.each_object(Class) do |klass|
            if klass < ApplicationController
              # 通常はActiveSupport::Dependencies.load_missing_constant をフックしている箇所で自動で
              # prepare_action_descriptionsが実行されますが、ここではrequire しているので、明示的に呼び出しています。
              klass.prepare_action_descriptions
            end
          end
        end

        #権限の初期データを作成します。
        role = Goldberg::Role.find_by_name('システム管理者')
        role.action_ids = Goldberg::SiteAction.connection.select_values("select id from goldberg_site_actions order by id")
        role.save!

        role = Goldberg::Role.find_by_name('情報システム部責任者')
        action_paths = %w(inquiry/index inquiry/update payments/index payments/update name_master/index name_master/create name_master/update) +
          %w(order/index order/update) +
          %w(product/index product/create product/update product/destroy user/index user/update)
        role.action_ids = action_paths.map{|path| Goldberg::SiteAction.find_by_path(path)}.compact.map{|action| action.id}
        role.save!

        role = Goldberg::Role.find_by_name('一般ユーザー')
        action_paths = %w(inquiry/index product/index user/index)
        role.action_ids = action_paths.map{|path| Goldberg::SiteAction.find_by_path(path)}.compact.map{|action| action.id}
        role.save!
      end
    end
  end

  module ControllerNamedHash
    def each
      @controller_names.each do |controller_name|
        yield(controller_name, self[controller_name])
      end
    end
  end

  namespace :permission do
    fixtures_dir = "#{RAILS_ROOT}/tmp/fixtures/goldberg/"
    desc "Extract permission data to the tmp/fixtures/ directory."
    task :extract => :environment do
      sql = "SELECT * FROM %s order by id" # svn diffなどで比較できるようにidでソート
      skip_tables = ["schema_migrations"]
      ActiveRecord::Base.establish_connection
      FileUtils.mkdir_p(fixtures_dir)

      if ENV['FIXTURES']
        table_names = ENV['FIXTURES'].split(/,/)
      else
        table_names = (ActiveRecord::Base.connection.tables - skip_tables)
      end

      extract_fixtures = ["goldberg_site_controllers", "goldberg_controller_actions"]

      table_names.each do |table_name|
        if extract_fixtures.include?(table_name)
          File.open("#{fixtures_dir}#{table_name}.yml", "w") do |file|
            objects  = ActiveRecord::Base.connection.select_all(sql % table_name)
            objects.each do |obj|
              file.write fixture_entry(table_name, obj) + "\n\n"
            end
          end
        end
      end
    end
    
    desc "update goldberg_site_actions & goldberg_site_action_groups."
    task :update=> :environment do
      Goldberg::RoleAction
      Goldberg::SiteAction
      Goldberg::SiteActionGroup
      Goldberg::SiteAction.transaction do
        Goldberg::SiteAction.append_actions_from_desc do
          ObjectSpace.each_object(Class) do |klass|
            if klass < ApplicationController
              # 通常はActiveSupport::Dependencies.load_missing_constant をフックしている箇所で自動で
              # prepare_action_descriptionsが実行されますが、ここではrequire しているので、明示的に呼び出しています。
              klass.prepare_action_descriptions
            end
          end
        end
      end
    end

    desc "authorization for system administrator."
    task :auth_for_system_admin do
      Goldberg::RoleAction.authorize_for_system_admin
    end
  end

  namespace :point do
    desc "expiration Expired point to new create"
    task :expiration => :environment do
      UserPoint.update_expired
    end
  end

  namespace :accounting do
    # ポイント残高を集計して経理テーブルに登録します
    desc "make up point summary"
    task :point_summary => :environment do
      Accounting.create_for_point_summary
    end
  end


end
