# -*- coding: utf-8 -*-
require 'find'

class ForeignKey

  attr_accessor :klass, :src_table, :src_col, :dest_table, :dest_col, :index
  def initialize(klass, src_table, src_col, dest_table, dest_col)
    @@index ||= 0
    @@index += 1
    @index = @@index
    @klass, @src_table, @src_col, @dest_table, @dest_col = 
      klass, src_table, src_col, dest_table, dest_col
  end

  # EQL_ATTRS = [:src_table, :src_col, :dest_table, :dest_col]
  EQL_ATTRS = [:src_table, :src_col]

  # uniqで必要
  def eql?(other)
    EQL_ATTRS.all?{|name| self.send(name) == other.send(name)}
  end

  # uniqで必要
  def hash
    EQL_ATTRS.map{|name| self.send(name)}.hash
  end

  def generate(migration_command)
    case migration_command
    when :add_index, :remove_index then
      index_name = "index_#{src_table}_on_#{src_col}"
      if index_name.length > 64
        index_name = "index_#{src_table}_on_%02d" % self.class.generation_index(migration_command, src_table)
        case migration_command
        when :add_index then
          "#{migration_command.to_s} '#{src_table}', '#{src_col}', :name => '#{index_name}'"
        when :remove_index then
          "#{migration_command.to_s} '#{src_table}', :name => '#{index_name}'"
        end
      else
        "#{migration_command.to_s} '#{src_table}', '#{src_col}'"
      end
    end
  end


  def inspect
    "#{klass.name} #{src_table}.#{src_col} -> #{dest_table}.#{dest_col}"
  end

  class << self
    def generation_index(migration_command, table_name)
      @generation_indexes ||= {}
      key = "#{migration_command}/#{table_name}"
      @generation_indexes[key] ||= 0
      @generation_indexes[key] += 1
    end

    def each(&block)
      models = []
      ObjectSpace.each_object(Class) do |klass|
        models << klass if klass < ActiveRecord::Base
      end
      fks = []
      conn = ActiveRecord::Base.connection
      if conn.adapter_name =~ /mysql/i
        config = ActiveRecord::Base.configurations[RAILS_ENV]
        view_names = conn.select_values("select TABLE_NAME from information_schema.TABLES where TABLE_TYPE = 'VIEW' AND TABLE_SCHEMA = '%s'" % config["database"])
      end
      models.each do |model|
        next if view_names && view_names.include?(model.table_name)
        model.reflections.values.each do |reflection|
          fks << new_foreign_key(model, reflection)
        end
      end
      fks.compact!
      if view_names
        fks.reject!{|fk| view_names.include?(fk.src_table) || view_names.include?(fk.dest_table)}
      end
      fks.uniq!
      fks = fks.sort_by{|fk| "#{fk.src_table}-#{fk.index}"}
      fks.each(&block)
    end

    IGNORED_OPTION_KEYS = [:through, :source_type, :as]

    def new_foreign_key(model, reflection)
      ref_options = (reflection.options || {}).merge(
        reflection.instance_variable_get(:@optinos) || {})
      return nil if IGNORED_OPTION_KEYS.any?{|key| ref_options.key?(key)}
      case reflection.macro
      when :belongs_to then
        table_name = (ref_options[:class_name] ?
          ref_options[:class_name].constantize.table_name : nil) ||
          reflection.name.to_s.pluralize
        new(model,
          model.table_name, ref_options[:foreign_key] || "#{reflection.name}_id",
          table_name.to_s, 'id')
      else
        klass = (ref_options[:class_name] || reflection.name.to_s.classify).constantize
        default_fk_name = '%s_id' % model.name.underscore.sub(/^.*\//, '')
        new(model,
          klass.table_name.to_s, ref_options[:foreign_key] || default_fk_name,
          model.table_name, 'id')
      end
    end

  end
end


def load_models
  model_file_names = []
  Dir.glob(File.join(RAILS_ROOT, "app/models*/**/*.rb")) do |f|
    model_file_names << f unless /\/.svn/ =~ f
  end
  model_file_names.each{|f| require f}
end

def show_lines(migration_command)
  conn = ActiveRecord::Base.connection
  lines = []
  load_models
  ForeignKey.each do |fk|
    begin
      idxs = conn.indexes(fk.src_table)
    rescue Exception
      puts fk.inspect
      raise
    end
    commented = idxs.any?{|idx| idx.columns == [fk.src_col]}
    cmd = "    #{'# ' if commented}#{fk.generate(migration_command)}"
    lines << cmd
  end
  puts lines.join("\n")
end

namespace :gen_migrate do
  desc "generate add_index lines from model's reflections"
  task :add_index => :environment do
    show_lines(:add_index)
  end
  
  desc "generate remove_index lines from model's reflections"
  task :remove_index => :environment do
    show_lines(:remove_index)
  end

  desc "index migration file content from associations"
  task :indexes => :environment do
    puts "  def self.up"
    show_lines(:add_index)
    puts "  end"
    puts "  "
    puts "  def self.down"
    show_lines(:remove_index)
    puts "  end"
    
  end

end
