# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CampaignEffect do
  before(:each) do
    @valid_attributes = {
      :campaign_id => 1,
      :type_cd => "9",
      :operator_cd => "9",
      :value => "value for value"
    }
  end

  it "should create a new instance given valid attributes" do
    CampaignEffect.create!(@valid_attributes)
  end

  describe "validation" do
    {
      :discount_percentage => '割引率',
      :discount_price => '割引金額',
      :shipping_cost => '送料'
    }.each do |type_key, type_name|

      describe "if #{type_key}" do
        it "with valid attributes" do
          effect = CampaignEffect.new(:type_key => type_key,
            :operator_key => :assign, :value => '12')
          effect.type_cd.should_not be_blank
          effect.valid?
          effect.errors.full_messages.should == []
        end

        it "with blank value" do
          effect = CampaignEffect.new(:type_key => type_key,
            :operator_key => :assign, :value => '')
          effect.valid?
          effect.errors.full_messages.should == ["値 は、#{type_name}を選択した場合、必ず入力してください。"]
        end

        it "with not numeric value" do
          effect = CampaignEffect.new(:type_key => type_key,
            :operator_key => :assign, :value => 'ABC')
          effect.valid?
          effect.errors.full_messages.should == ["値 は、#{type_name}を選択した場合、数値を入力してください。"]
        end
      end
    end

    describe "if point" do
      it "with valid attributes" do
        effect = CampaignEffect.new(:type_key => :point,
          :operator_key => :add, :value => '12',
          :point_started_at => Time.parse('2009/10/1 00:00'),
          :point_finished_at => Time.parse('2009/12/31 23:59'))
        effect.valid?
        effect.errors.full_messages.should == []
      end

      it "with blank value" do
        effect = CampaignEffect.new(:type_key => :point,
          :operator_key => :add, :value => '',
          :point_started_at => Time.parse('2009/10/1 00:00'),
          :point_finished_at => Time.parse('2009/12/31 23:59'))
        effect.valid?
        effect.errors.full_messages.should == ["値 は、ポイントを選択した場合、必ず入力してください。"]
      end

      it "with not numeric value" do
        effect = CampaignEffect.new(:type_key => :point,
          :operator_key => :add, :value => 'ABC',
          :point_started_at => Time.parse('2009/10/1 00:00'),
          :point_finished_at => Time.parse('2009/12/31 23:59'))
        effect.valid?
        effect.errors.full_messages.should == ["値 は、ポイントを選択した場合、数値を入力してください。"]
      end
    end


    describe "if present" do
      it "with valid attributes" do
        effect = CampaignEffect.new(:type_key => :present,
          :operator_key => :assign, :present_product_sku_id => '1')
        effect.valid?
        effect.errors.full_messages.should == []
      end

      it "with blank value" do
        effect = CampaignEffect.new(:type_key => :present,
          :operator_key => :assign)
        effect.valid?
        effect.errors.full_messages.should == ["プレゼント商品SKU を選択してください。"]
      end

    end
  end

  describe "apply" do
    fixtures :users, :user_ranks, :user_rank_displays, :user_rank_conditions,
      :user_points, :point_settings,
      :products, :product_skus, :product_prices,
      :campaigns, :campaign_conditions, :campaign_effects, :campaign_target_product_skus

    before(:each) do
      @user_rank = UserRank.new(:name => 'testadmin_name', :code => 'test001', :available_div => "01", :connector_cd => "01")
      @user_rank.displays << UserRankDisplay.new(:locale => "ja", :name => "testname")
      @user_rank.conditions << UserRankCondition.new(:type_cd => "01", :operator_cd => "01", :value => 1)
      @user_rank.save!
      @user = users(:users_00010) # 会員二十三
      @user.user_rank_id = @user_rank.id
      @user.save!
      @user.reload
      @product_skus = {}
      @product_skus[:A] = products(:products_00003).primary_sku # 商品A インテリア家具
      @product_skus[:B] = products(:products_00005).primary_sku # 商品B タンブラー
      @product_skus[:C] = products(:products_00006).primary_sku # 商品C ガム10箱
      prices = {:A => 100000, :B => 10000, :C => 1000}
      prices.each do |name, price|
        ProductPrice.create(
          :product_sku_id => @product_skus[name].id,
          :user_rank_id => @user_rank.id,
          :standard_price => price,
          :regular_purchase_price => price - 200  # 定期購入売価を標準売価より安くしておいて、テスト対象としてく区別する
        )
      end
      @cart = Cart.new
      @cart.user_rank = @user_rank
    end

    describe "割引率" do
      it "代入の場合" do
        effect = CampaignEffect.new(:type_key => :discount_percentage,
          :operator_key => :assign, :value => '20')
        effect.should_receive(:targets_cart?).once.and_return(true)
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])
        @cart.calculate
        lambda{
          effect.apply(@cart)
        }.should_not change(@cart, :subtotal_amount)
        @cart.subtotal_amount.should == 11000
        @cart.discount_amount.should ==  2200
      end
    end

    describe "割引金額" do
      it "代入の場合" do
        effect = CampaignEffect.new(:type_key => :discount_price,
          :operator_key => :assign, :value => '200')
        effect.should_receive(:targets_cart?).once.and_return(true)
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])
        @cart.calculate
        lambda{
          effect.apply(@cart)
        }.should_not change(@cart, :subtotal_amount)
        @cart.subtotal_amount.should == 11000
        @cart.discount_amount.should ==   200
        @cart.total_amount.should    == 10800
      end
    end

    describe "プレゼント" do
      it "代入の場合" do
        effect = CampaignEffect.new(:type_key => :present,
          :operator_key => :assign, :present_product_sku_id => 281)
        effect.should_receive(:targets_cart?).once.and_return(true)
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])
        @cart.calculate
        lambda{
          lambda{
            effect.apply(@cart)
          }.should_not change(@cart, :subtotal_amount)
        }.should change(@cart.items, :length).by(1)
        @cart.subtotal_amount.should == 11000
        @cart.discount_amount.should ==     0
        @cart.total_amount.should    == 11000
      end
    end

    describe "ポイント" do
      it "代入の場合" do
        effect = CampaignEffect.new(:type_key => :point,
          :operator_key => :add, :value => 500, :point_started_at => '2009-09-10 12:05:00',
          :point_finished_at => '2029-09-10 12:05:00')
        effect.should_receive(:targets_cart?).once.and_return(true)
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])
        @cart.calculate
        lambda{
          effect.apply(@cart)
        }.should_not change(@cart, :subtotal_amount)
        @cart.subtotal_amount.should   == 11000
        @cart.discount_amount.should   ==     0
        @cart.total_amount.should      == 11000
        @cart.item_for(@product_skus[:B]).given_point.should == 10
        @cart.total_given_point.should ==  511
      end
    end
  end

  describe "在庫チェック" do
    fixtures :product_skus, :product_sku_inventories, :product_sku_secure_inventories
    it "在庫切れ" do
      effect = CampaignEffect.new(:type_key => :present, :present_product_sku_id => 283)
      effect.present_but_no_stock?.should be_true
    end

    it "在庫あり" do
      effect = CampaignEffect.new(:type_key => :present, :present_product_sku_id => 265)
      effect.present_but_no_stock?.should be_false
    end

    it "引き当て済み" do
      secure_inventory = ProductSkuSecureInventory.new(:product_sku_id => 283, :count => 1)
      cart = Cart.new()
      cart.secured_stocks = [secure_inventory]
      effect = CampaignEffect.new(:type_key => :present, :present_product_sku_id => 283)
      effect.present_but_no_stock?(cart).should be_false
    end
  end

end
