require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CmsProject do
  fixtures :cms_projects
  
  before(:each) do
    pending unless EcRider.config.ec_admin
  end
  
  before(:each) do
    @cms_project = CmsProject.new
    @cms_project.priority = "1"
    @cms_project.project_code = "ABC"
    @cms_project.project_name = "projectABC"
    @cms_project.started_at = DateTime.now
    @cms_project.finished_at = DateTime.now + 1
    @cms_project.status_cd = "1"
    @cms_project.acceptance_type = '0'
    @cms_project.check_flg = true
  end

  describe "validation" do
    it "should be valid" do
      @cms_project.should be_valid
      @cms_project.finished_at = nil
      @cms_project.should be_valid
    end

    it "should be invalid" do
      @cms_project.project_code = nil
      @cms_project.project_name = nil
      @cms_project.started_at = nil
      @cms_project.should_not be_valid
      @cms_project.should have(1).errors_on(:project_code)
      @cms_project.should have(1).errors_on(:project_name)
      @cms_project.should have(1).errors_on(:started_at)
    end

    it "should be invalid when started_at > finished_at." do
      @cms_project.finished_at = DateTime.new(2009, 1, 1, 0, 0, 0)
      @cms_project.should_not be_valid
      @cms_project.should have(1).errors_on(:finished_at)
    end

    it "should be invalid when member mismatch" do
      @cms_project.acceptance_type = '0'
      @cms_project.assigns_attributes = {:"0" => {:role => :editor, :goldberg_user_id => ""}}
      @cms_project.should_not be_valid
      @cms_project.errors.should_not be_nil
      
      @cms_project.acceptance_type = '2'
      @cms_project.assigns_attributes = {:"0" => {:role => :editor, :goldberg_user_id => "1001"}}
      @cms_project.should_not be_valid
      @cms_project.errors.should_not be_nil

      @cms_project.acceptance_type = '3'
      @cms_project.assigns_attributes = {:"0" => {:role => :checker, :goldberg_user_id => "1001"}}
      @cms_project.should_not be_valid
      @cms_project.errors.should_not be_nil
    end
  end

  describe "create search condition" do
    it "should create search condition" do
      condition =CmsProject::SearchCondition.new
      cms_projects = CmsProject.find(:all, condition.to_find_options)
      cms_projects.size.should == 4

      condition =CmsProject::SearchCondition.new(:project_code => '00001')
      cms_projects = CmsProject.find(:all, condition.to_find_options)
      cms_projects.size.should == 1

      condition =CmsProject::SearchCondition.new(:project_name => 'project1')
      cms_projects = CmsProject.find(:all, condition.to_find_options)
      cms_projects.size.should == 1

      condition =CmsProject::SearchCondition.new(:status_cd => '0')
      cms_projects = CmsProject.find(:all, condition.to_find_options)
      cms_projects.size.should == 1

      condition =CmsProject::SearchCondition.new(:finished_at => '2020/01/31')
      cms_projects = CmsProject.find(:all, condition.to_find_options)
      cms_projects.size.should == 3

      condition =CmsProject::SearchCondition.new(:started_at => '2009/02/02')
      cms_projects = CmsProject.find(:all, condition.to_find_options)
      cms_projects.size.should == 2

      condition =CmsProject::SearchCondition.new(:started_at => '2009/01/01', :finished_at => '2020/01/31')
      cms_projects = CmsProject.find(:all, condition.to_find_options)
      cms_projects.size.should == 3
    end
  end

  describe "reusable?" do
    it "should return false" do
      @cms_project.status_key = :editing
      @cms_project.reusable?.should be_false
      @cms_project.status_key = :requesting_check
      @cms_project.reusable?.should be_false
      @cms_project.status_key = :requesting_approval
      @cms_project.reusable?.should be_false
      @cms_project.status_key = :approved
      @cms_project.reusable?.should be_false
    end

    it "should return true" do
      @cms_project.status_key = :opened
      @cms_project.reusable?.should be_true
      @cms_project.status_key = :closed
      @cms_project.reusable?.should be_true
    end
  end
  
  describe 'approval_mail_to' do
    fixtures :cms_projects, :cms_project_assigns
    before(:each) do
      CmsProjectAssign.create!(:role_key => :checker, :goldberg_user_id => 1, :cms_project_id => cms_projects(:cms_projects_00002).id)
      @cms_project = CmsProject.find(cms_projects(:cms_projects_00002).id, :include => :assigns)
    end
    
    it '確認依頼' do
      @cms_project.status_key = :requesting_check
      @cms_project.approval_mail_to.sort.should == ['test_admin@ec-one.com', 'test_net_admin@ec-one.com']
    end
    
    it '承認依頼' do
      @cms_project.status_key = :requesting_approval
      @cms_project.approval_mail_to.should == ['test_admin@ec-one.com']
    end
    
    it '差し戻し' do
      @cms_project.status_key = :editing
      @cms_project.approval_mail_to.should == ['public@ec-one.com']
    end
  end
  
  describe 'count pending approval or check' do
    fixtures :cms_projects, :cms_project_assigns, :cms_project_approvals
    it "確認依頼中" do
      cms_project = cms_projects(:cms_projects_00001)
      cms_project.status_key = :requesting_check
      cms_project.save
      cms_project = cms_projects(:cms_projects_00002)
      cms_project.status_key = :requesting_check
      cms_project.save
      
      CmsProject.count_pending_approval(2).should == 0
      CmsProject.count_pending_check(2).should == 1
      
      CmsProject.count_pending_approval(1).should == 0
      CmsProject.count_pending_check(1).should == 0
    end
    
    it "承認依頼中" do
      cms_project = cms_projects(:cms_projects_00001)
      cms_project.status_key = :requesting_approval
      cms_project.save
      cms_project = cms_projects(:cms_projects_00002)
      cms_project.status_key = :requesting_approval
      cms_project.save
      
      CmsProject.count_pending_approval(2).should == 0
      CmsProject.count_pending_check(2).should == 0
      
      CmsProject.count_pending_approval(1).should == 2
      CmsProject.count_pending_check(1).should == 0
    end
    
    it "承認済み" do
      cms_project = cms_projects(:cms_projects_00001)
      cms_project.status_key = :approved
      cms_project.save
      cms_project = cms_projects(:cms_projects_00002)
      cms_project.status_key = :approved
      cms_project.save
      
      CmsProject.count_pending_approval(2).should == 0
      CmsProject.count_pending_check(2).should == 0
      
      CmsProject.count_pending_approval(1).should == 0
      CmsProject.count_pending_check(1).should == 0
    end
  end

end
