# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CmsTemplate do
  fixtures :cms_templates, :cms_projects, :cms_template_contents, :cms_default_templates

  before(:each) do
    @cms_template = CmsTemplate.new
    @cms_template.cms_project_id = '1'
    @cms_template.name = 'template_name'
    @cms_template.display_position = '0'
    @cms_template.terminal = '0'
    @cms_template.contents_attributes = {
      :ja      => {:locale => 'ja'    , :descriptions => "日本語"},
      :en      => {:locale => 'en'    , :descriptions => "英語"  },
      :"zh-CN" => {:locale => 'zh_CN' , :descriptions => "簡体字"},
      :"zh-TW" => {:locale => 'zh_TW' , :descriptions => "繁体字"},
      :ko      => {:locale => 'ko'    , :descriptions => "韓国語"},
      :th      => {:locale => 'th'    , :descriptions => "タイ語"}}
    @cms_template.stub!(:request).and_return(mock_model(Object, {:mobile? => false}))

    @cms_template_mb = CmsTemplate.new
    @cms_template_mb.cms_project_id = '1'
    @cms_template_mb.name = 'template_name'
    @cms_template_mb.display_position = '0'
    @cms_template_mb.terminal = '1'
    @cms_template_mb.contents_attributes = {
      :docomo   => {:locale => 'docomo'   , :descriptions => "DOCOMO"},
      :au       => {:locale => 'au'       , :descriptions => "AU"  },
      :softbank => {:locale => 'softbank' , :descriptions => "SOFTBANK"}}
    @cms_template_mb.stub!(:request).and_return(mock_model(Object, {:mobile? => true}))
  end

  describe "content_default_locale" do
    it "call content_ja" do
      template = CmsTemplate.new(:terminal_key => :pc)
      template.should_receive(:content_ja)
      template.content_default_locale
    end
    it "call content_docomo" do
      template = CmsTemplate.new(:terminal_key => :mobile)
      template.should_receive(:content_docomo)
      template.content_default_locale
    end
  end

  describe "get_descriptions" do
    before do
      @cms_template = CmsTemplate.find(1)
      @cms_content_en = CmsTemplateContent.find(:first, :conditions => ['cms_template_id = ? AND locale = ?', 1, 'en'])
      @cms_content_en.descriptions = 'content_en'
      @cms_content_en.save
      @cms_content_ja = CmsTemplateContent.find(:first, :conditions => ['cms_template_id = ? AND locale = ?', 1, 'ja'])
      @cms_content_ja.descriptions = 'content_ja'
      @cms_content_ja.save
    end
    it "get descriptions" do
      @cms_template.get_descriptions(@cms_content_en).should == @cms_content_en.descriptions
    end
    it "get default_locale descriptions if blank" do
      @cms_content_en.descriptions = ''
      @cms_template.get_descriptions(@cms_content_en).should == @cms_content_ja.descriptions
    end
    it "get default_locale descriptions if nil" do
      @cms_content_en.descriptions = nil
      @cms_template.get_descriptions(@cms_content_en).should == @cms_content_ja.descriptions
    end
  end

  describe "validation" do
    it "should valid." do
      @cms_template.should be_valid
    end

    it "should invalid when template_position mismatch." do
      @cms_template.default_template_position = '3'
      @cms_template.should_not be_valid
      @cms_template.errors.on('base').should == "表示可能位置とデフォルトテンプレート表示位置が合っていません。"
    end

    it "should invalid with blank description of japanese." do
      @cms_template.contents_attributes[:ja][:descriptions] = ""
      @cms_template.should_not be_valid
      @cms_template.errors.on('base').should == "日本語の内容は必須です。"

      @cms_template.errors.clear
      @cms_template.contents_attributes[:ja][:descriptions] = nil
      @cms_template.should_not be_valid
      @cms_template.errors.on('base').should == "日本語の内容は必須です。"
    end

    it "should invalid with blank description of docomo." do
      @cms_template_mb.terminal = '1'
      @cms_template_mb.contents_attributes[:docomo][:descriptions] = ""
      @cms_template_mb.should_not be_valid
      @cms_template_mb.errors.on('base').should == "NTTドコモの内容は必須です。"

      @cms_template_mb.errors.clear
      @cms_template_mb.contents_attributes[:docomo][:descriptions] = nil
      @cms_template_mb.should_not be_valid
      @cms_template_mb.errors.on('base').should == "NTTドコモの内容は必須です。"
    end

    it "should invalid with too long descriptions." do
      @cms_template.contents_attributes[:ja][:descriptions] = 'あ'*30000
      @cms_template.should_not be_valid
      @cms_template.errors.on('base').should == "日本語[ja] 内容 65535バイト以内で入力してください。"

      @cms_template.errors.clear
      @cms_template.contents_attributes[:ja][:descriptions] = 'a'*65536
      @cms_template.should_not be_valid
      @cms_template.errors.on('base').should == "日本語[ja] 内容 65535バイト以内で入力してください。"
    end

    it "should invalid with wrong xml." do
      @cms_template.contents_attributes[:ja][:descriptions] = '<ec:today>'
      @cms_template.should_not be_valid
      @cms_template.errors.on('base').should == "日本語[ja] 内容 XMLの記述が不正です。"
    end

    it "should invalid with wrong ec tag." do
      @cms_template.contents_attributes[:ja][:descriptions] = '<ec:wrongtag/>'
      @cms_template.should_not be_valid
      @cms_template.errors.on('base').should == "日本語[ja] 内容 ec:wrongtag は存在しないタグです。"
    end
  end

  describe "create search condition" do
    it "should create search condition" do
      condition =CmsTemplate::SearchCondition.new(:cms_project_id => 6)
      cond, delete_ids = condition.create_condition
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :conditions => cond)
      cms_templates.size.should == 47

      condition =CmsTemplate::SearchCondition.new(:cms_project_id => 6, :template_name => '上')
      cond, delete_ids = condition.create_condition
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :conditions => cond)
      cms_templates.size.should == 1

      condition =CmsTemplate::SearchCondition.new(:cms_project_id => 6,:display_position => '1')
      cond, delete_ids = condition.create_condition
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :conditions => cond)
      cms_templates.size.should == 1

      condition =CmsTemplate::SearchCondition.new(:cms_project_id => 6, :status_cd => '4')
      cond, delete_ids = condition.create_condition
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :conditions => cond)
      cms_templates.size.should == 47

      condition =CmsTemplate::SearchCondition.new(:cms_project_id => 2, :edit_target => '1')
      cond, delete_ids = condition.create_condition
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :conditions => cond)
      cms_templates.size.should == 3

      condition =CmsTemplate::SearchCondition.new(:cms_project_id => 6, :terminal => '0')
      cond, delete_ids = condition.create_condition
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :conditions => cond)
      cms_templates.size.should == 46

      cms_template = CmsTemplate.find(cms_templates(:cms_templates_001).id)
      cms_template.terminal = '1'
      cms_template.save!
      condition =CmsTemplate::SearchCondition.new(:cms_project_id => 6, :terminal => '1')
      cond, delete_ids = condition.create_condition
      cms_templates = CmsTemplate.find(:all, :include => :cms_project, :conditions => cond)
      cms_templates.size.should == 2
    end
  end

  describe "current_project?" do
    it "should be ture" do
      @cms_template.cms_project_id.should == 1
      current_project_id = "1"
      @cms_template.current_project?(current_project_id).should be_true
    end

    it "should be false" do
      current_project_id = ""
      @cms_template.current_project?(current_project_id).should be_false
      current_project_id = nil
      @cms_template.current_project?(current_project_id).should be_false
      current_project_id = "100"
      @cms_template.current_project?(current_project_id).should be_false
    end
  end

  describe "new_copy" do
    it "should copy" do
      org_template = CmsTemplate.find(1, :include => :contents)
      new_template = org_template.new_copy

      new_template.present_version_id.should == 1
      new_contents = new_template.contents
      new_contents.class.should == Array
      new_contents.length.should == 6
      new_contents[0].class.should == CmsTemplateContent
    end
  end

  describe "new_create" do
    it "テンプレートコードを採番する" do
      cms_template = CmsTemplate.new(:name => 'new_template', :display_position => "0",
        :contents_attributes => {
          :ja => {:locale => 'ja' , :descriptions => "日本語"},
          :en => {:locale => 'en' , :descriptions => "英語"},
          :"zh_CN" => {:locale => 'zh_CN' , :descriptions => "簡体字"},
          :"zh_TW" => {:locale => 'zh_TW' , :descriptions => "繁体字"},
          :ko => {:locale => 'ko' , :descriptions => "韓国語"},
          :th => {:locale => 'th' , :descriptions => "タイ語"}})
      cms_template.set_values_for_create

      cms_template.template_code.should_not be_nil
      cms_template.contents.class.should == Array
      cms_template.contents.length.should == I18n.supported_locales.length
      cms_template.contents[0].class.should == CmsTemplateContent
    end

    it "テンプレートコードを採番しない" do
      cms_template = CmsTemplate.new(:name => 'new_template', :display_position => "0", :template_code => '1234567890',
        :contents_attributes => {
          :ja => {:locale => 'ja' , :descriptions => "日本語"},
          :en => {:locale => 'en' , :descriptions => "英語"},
          :"zh_CN" => {:locale => 'zh_CN' , :descriptions => "簡体字"},
          :"zh_TW" => {:locale => 'zh_TW' , :descriptions => "繁体字"},
          :ko => {:locale => 'ko' , :descriptions => "韓国語"},
          :th => {:locale => 'th' , :descriptions => "タイ語"}})
      cms_template.set_values_for_create

      cms_template.template_code.should == '1234567890'
      cms_template.contents.class.should == Array
      cms_template.contents.length.should == I18n.supported_locales.length
      cms_template.contents[0].class.should == CmsTemplateContent
    end
  end

  describe "bind_items" do
    it "should copy" do
      cms_template = CmsTemplate.find(16, :include => :contents)
      cms_template.bind_items.should == [{:name=>"おすすめコメント", :no=>"01"}, {:name=>"店舗住所", :no=>"02"}, {:name=>"店舗画像", :no=>"03"}]
    end
  end

  describe "get template options" do
    it "上テンプレート" do
      CmsTemplate.part_template_options('0', 3).should == [["上部バー", 1]]
      CmsTemplate.part_template_options('0', 3, '').should == [["上部バー", 1]]
      CmsTemplate.part_template_options('0', 3, '0').should == [["上部バー", 1]]
      CmsTemplate.part_template_options('0', 3, '1').should == []
    end

    it "上テンプレート（携帯）" do
      cms_template = CmsTemplate.find(:first, :conditions => "display_position = '0'")
      cms_template.terminal = '1'
      cms_template.save!
      CmsTemplate.part_template_options('0', 3).should == [["上部バー", 1]]
      CmsTemplate.part_template_options('0', 3, '').should == [["上部バー", 1]]
      CmsTemplate.part_template_options('0', 3, '0').should == []
      CmsTemplate.part_template_options('0', 3, '1').should == [["上部バー", 1]]
    end

    it "下テンプレート" do
      CmsTemplate.part_template_options('1', 3).should == [["下部バー", 2]]
      CmsTemplate.part_template_options('1', 3, '').should == [["下部バー", 2]]
      CmsTemplate.part_template_options('1', 3, '0').should == [["下部バー", 2]]
      CmsTemplate.part_template_options('1', 3, '1').should == []
    end

    it "下テンプレート（携帯）" do
      cms_template = CmsTemplate.find(:first, :conditions => "display_position = '1'")
      cms_template.terminal = '1'
      cms_template.save!
      CmsTemplate.part_template_options('1', 3).should == [["下部バー", 2]]
      CmsTemplate.part_template_options('1', 3, '').should == [["下部バー", 2]]
      CmsTemplate.part_template_options('1', 3, '0').should == []
      CmsTemplate.part_template_options('1', 3, '1').should == [["下部バー", 2]]
    end

    it "左テンプレート" do
      CmsTemplate.part_template_options('2', 3).should == [["左サイドバー", 3],["左サイドバー_商品検索", 328]]
      CmsTemplate.part_template_options('2', 3, '').should == [["左サイドバー", 3],["左サイドバー_商品検索", 328]]
      CmsTemplate.part_template_options('2', 3, '0').should == [["左サイドバー", 3],["左サイドバー_商品検索", 328]]
      CmsTemplate.part_template_options('2', 3, '1').should == []
    end

    it "左テンプレート（携帯）" do
      cms_templates = CmsTemplate.find(:all, :conditions => "display_position = '2'")
      cms_templates.each do |cms_template|
        cms_template.terminal = '1'
        cms_template.save!
      end
      CmsTemplate.part_template_options('2', 3).should == [["左サイドバー", 3],["左サイドバー_商品検索", 328]]
      CmsTemplate.part_template_options('2', 3, '').should == [["左サイドバー", 3],["左サイドバー_商品検索", 328]]
      CmsTemplate.part_template_options('2', 3, '0').should == []
      CmsTemplate.part_template_options('2', 3, '1').should == [["左サイドバー", 3],["左サイドバー_商品検索", 328]]
    end

    it "右テンプレート" do
      CmsTemplate.part_template_options('3', 3).should == [["右サイドバー", 4]]
      CmsTemplate.part_template_options('3', 3, '').should == [["右サイドバー", 4]]
      CmsTemplate.part_template_options('3', 3, '0').should == [["右サイドバー", 4]]
      CmsTemplate.part_template_options('3', 3, '1').should == []
    end

    it "右テンプレート（携帯）" do
      cms_template = CmsTemplate.find(:first, :conditions => "display_position = '3'")
      cms_template.terminal = '1'
      cms_template.save!
      CmsTemplate.part_template_options('3', 3).should == [["右サイドバー", 4]]
      CmsTemplate.part_template_options('3', 3, '').should == [["右サイドバー", 4]]
      CmsTemplate.part_template_options('3', 3, '0').should == []
      CmsTemplate.part_template_options('3', 3, '1').should == [["右サイドバー", 4]]
    end

    it "中央テンプレート" do
      CmsTemplate.part_template_options('4', 2).length.should > 3
      CmsTemplate.part_template_options('4', 2, '').length.should > 3
      CmsTemplate.part_template_options('4', 2, '0').length.should > 3
      # ↓携帯
      CmsTemplate.part_template_options('4', 2, '1').length.should == 1
    end

    it "中央テンプレート（携帯）" do
      cms_template = CmsTemplate.find(:first, :conditions => "display_position = '4'")
      cms_template.terminal = '1'
      cms_template.save!
      CmsTemplate.part_template_options('4', 2, '1').length.should == 2
    end
  end

  describe "published_default_template_filename" do
    describe "デフォルトテンプレートとして" do
      it "上部バー" do
        template = CmsTemplate.find(1)
        template.should_not be_nil
        template.display_position_key.should == :top
        template.published_default_template_filename(:top, :ja).should     == "cms/_default_top.html.erb"
        template.published_default_template_filename(:top, :en).should     == "cms/_default_top_en.html.erb"
        template.published_default_template_filename(:top, 'zh-CN').should == "cms/_default_top_zh_CN.html.erb"
        template.published_default_template_filename(:top, :docomo).should   == "cms/_default_top_mobile.html.erb"
        template.published_default_template_filename(:top, :au).should       == "cms/_default_top_mobile_au.html.erb"
        template.published_default_template_filename(:top, :softbank).should == "cms/_default_top_mobile_softbank.html.erb"

        template.published_default_template_filename(:bottom, :ja).should     == "cms/_default_bottom.html.erb"
        template.published_default_template_filename(:bottom, :en).should     == "cms/_default_bottom_en.html.erb"
        template.published_default_template_filename(:bottom, 'zh-CN').should == "cms/_default_bottom_zh_CN.html.erb"
        template.published_default_template_filename(:bottom, :docomo).should   == "cms/_default_bottom_mobile.html.erb"
        template.published_default_template_filename(:bottom, :au).should       == "cms/_default_bottom_mobile_au.html.erb"
        template.published_default_template_filename(:bottom, :softbank).should == "cms/_default_bottom_mobile_softbank.html.erb"
      end

      it "下部バー" do
        template = CmsTemplate.find(2)
        template.should_not be_nil
        template.display_position_key.should == :bottom
        template.published_default_template_filename(:bottom, :ja).should     == "cms/_default_bottom.html.erb"
        template.published_default_template_filename(:bottom, :en).should     == "cms/_default_bottom_en.html.erb"
        template.published_default_template_filename(:bottom, 'zh-CN').should == "cms/_default_bottom_zh_CN.html.erb"
        template.published_default_template_filename(:bottom, :docomo).should   == "cms/_default_bottom_mobile.html.erb"
        template.published_default_template_filename(:bottom, :au).should       == "cms/_default_bottom_mobile_au.html.erb"
        template.published_default_template_filename(:bottom, :softbank).should == "cms/_default_bottom_mobile_softbank.html.erb"
      end
    end
  end

  describe "published_filename" do
    describe "普通のテンプレートとして" do
      it "上部バー" do
        template = CmsTemplate.find(1)
        template.should_not be_nil
        template.published_filename(:ja).should     == "cms/_top_2009082119101338424990.html.erb"
        template.published_filename(:en).should     == "cms/_top_2009082119101338424990_en.html.erb"
        template.published_filename('zh-CN').should == "cms/_top_2009082119101338424990_zh_CN.html.erb"
        template.published_filename(:docomo).should     == "cms/_top_2009082119101338424990_mobile.html.erb"
        template.published_filename(:au).should     == "cms/_top_2009082119101338424990_mobile_au.html.erb"
        template.published_filename(:softbank).should == "cms/_top_2009082119101338424990_mobile_softbank.html.erb"
      end

      it "下部バー" do
        template = CmsTemplate.find(2)
        template.should_not be_nil
        template.published_filename(:ja).should     == "cms/_bottom_2009082119101338424900.html.erb"
        template.published_filename(:en).should     == "cms/_bottom_2009082119101338424900_en.html.erb"
        template.published_filename('zh-CN').should == "cms/_bottom_2009082119101338424900_zh_CN.html.erb"
        template.published_filename(:docomo).should     == "cms/_bottom_2009082119101338424900_mobile.html.erb"
        template.published_filename(:au).should     == "cms/_bottom_2009082119101338424900_mobile_au.html.erb"
        template.published_filename(:softbank).should == "cms/_bottom_2009082119101338424900_mobile_softbank.html.erb"
      end
    end
  end

  describe "default_template_filename_map" do
    it "デフォルトテンプレート"do
      if EcRider.config.ec_admin
        map = CmsTemplate.default_template_filename_map(:projects => CmsProject.all)

        template = CmsTemplate.find(1) # 上部バー
        map["cms/_default_top.html.erb"      ].first.id.should == template.id
        map["cms/_default_top_en.html.erb"   ].first.id.should == template.id
        map["cms/_default_top_zh_CN.html.erb"].first.id.should == template.id
        map["cms/_default_top.html.erb"      ].last.should == 'ja'
        map["cms/_default_top_en.html.erb"   ].last.should == 'en'
        map["cms/_default_top_zh_CN.html.erb"].last.should == 'zh_CN'

        template = CmsTemplate.find(2) # 下部バー
        map["cms/_default_bottom.html.erb"      ].first.id.should == template.id
        map["cms/_default_bottom_en.html.erb"   ].first.id.should == template.id
        map["cms/_default_bottom_zh_CN.html.erb"].first.id.should == template.id
        map["cms/_default_bottom.html.erb"      ].last.should == 'ja'
        map["cms/_default_bottom_en.html.erb"   ].last.should == 'en'
        map["cms/_default_bottom_zh_CN.html.erb"].last.should == 'zh_CN'
      end
    end
    
  end

  describe "primary_default_templates" do
    fixtures :cms_templates, :cms_projects, :cms_template_contents, :cms_default_templates
    # TODO モバイルのテストが未実施
    it "上下左右のそれぞれについて、最も優先度の高い(priorityの小さい)プロジェクトに属するCmsTemplateを返します。" do
      if EcRider.config.ec_admin
        # Proj:id: 1, priority: 100, acceptance_type_key: only_approval, project_code: 00001, project_name: project1, status_key: opened
        #   DefaultTemplate: id: 1, name: 上部バー, display_position_key: top, default_template_position_key: top, delete_at_publish: , template_code: cms/top, terminal:
        #         Content id: 2, locale: en
        #           上1テンプレート英語
        #         Content id: 1, locale: ja
        #           上1テンプレート日本語
        #         Content id: 5, locale: ko
        #           上1テンプレート韓国語
        #         Content id: 6, locale: th
        #           上1テンプレートタイ語
        #         Content id: 3, locale: zh_CN
        #           上1テンプレート簡体字
        #         Content id: 4, locale: zh_TW
        #           上1テンプレート繁体字
        #   DefaultTemplate: id: 2, name: 下部バー, display_position_key: bottom, default_template_position_key: bottom, delete_at_publish: , template_code: cms/bottom, terminal:
        #         Content id: 8, locale: en
        #           <h1>下1テンプレート英語</h1>
        #         Content id: 7, locale: ja
        #           下1テンプレート日本語
        #         Content id: 11, locale: ko
        #           下1テンプレート韓国語
        #         Content id: 12, locale: th
        #           下1テンプレートタイ語
        #         Content id: 9, locale: zh_CN
        #           下1テンプレート簡体字
        #         Content id: 10, locale: zh_TW
        #           下1テンプレート繁体字
        #   ......
        #
        # Proj:id: 2, priority: 20, acceptance_type_key: ed_ck_ap, project_code: 00002, project_name: project2, status_key: editing
        #   DefaultTemplate: id: 11, name: トップ, display_position_key: center, default_template_position_key: top, delete_at_publish: true, template_code: cms/index, terminal:
        #   ......
        #
        template = CmsTemplate.find(11)
        template.update_attributes(:delete_at_publish => false)
        CmsDefaultTemplate.create_or_update_default_template(template.cms_project_id, template.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template.template_code)
        #
        CmsDefaultTemplate.count(:all).should == 3
        templates = CmsTemplate.primary_default_templates(:projects => CmsProject.all)[:pc]
        templates.keys.length.should == 2
        # from CmsProject[2]
        templates[:top].class.should == CmsTemplate
        templates[:top].id.should == 11
        # from CmsProject[1]
        templates[:bottom].class.should == CmsTemplate
        templates[:bottom].id.should == 2
        #
        templates[:left].should == nil
        templates[:right].should == nil
      end
    end
    it "優先順位が同じプロジェクトがある場合 公開開始日の遅いプロジェクトで編集されたテンプレートが取得できること" do
      if EcRider.config.ec_admin
        cms_projects_00001 = CmsProject.find(cms_projects(:cms_projects_00001).id)
        cms_projects_00001.update_attributes(:priority => 100, :started_at => "2010-02-14 00:00:00")
        cms_projects_00002 = CmsProject.find(2)
        cms_projects_00002.update_attributes(:priority => 100, :started_at => "2010-02-13 00:00:00")

        # project1で編集したデフォルトテンプレート
        template1 = CmsTemplate.find(1)
        template1.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template1.cms_project_id, template1.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template1.template_code)

        # project2で編集したデフォルトテンプレート
        template15 = CmsTemplate.find(15)
        template15.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template15.cms_project_id, template15.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template15.template_code)

        #
        templates = CmsTemplate.primary_default_templates(:projects => CmsProject.all)[:pc]
        templates.keys.length.should == 2
        # from CmsProject[2]
        templates[:top].class.should == CmsTemplate
        templates[:top].id.should == 1
        # from CmsProject[1]
        templates[:bottom].class.should == CmsTemplate
        templates[:bottom].id.should == 2
        #
        templates[:left].should == nil
        templates[:right].should == nil
      end
    end
    it "優先順位が同じ かつ 公開開始日が同じプロジェクトがある場合 プロジェクトIDの大きい方で編集されたテンプレートが取得できること" do
      if EcRider.config.ec_admin
        cms_projects_00001 = CmsProject.find(cms_projects(:cms_projects_00001).id)
        cms_projects_00001.update_attributes(:priority => 100, :started_at => "2010-02-14 00:00:00")
        cms_projects_00002 = CmsProject.find(cms_projects(:cms_projects_00002).id)
        cms_projects_00002.update_attributes(:priority => 100, :started_at => "2010-02-14 00:00:00")

        # project1で編集したデフォルトテンプレート
        template1 = CmsTemplate.find(1)
        template1.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template1.cms_project_id, template1.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template1.template_code)

        # project2で編集したデフォルトテンプレート
        template15 = CmsTemplate.find(15)
        template15.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template15.cms_project_id, template15.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template15.template_code)

        #
        templates = CmsTemplate.primary_default_templates(:projects => CmsProject.all)[:pc]
        templates.keys.length.should == 2
        # from CmsProject[2]
        templates[:top].class.should == CmsTemplate
        templates[:top].id.should == 15
        # from CmsProject[1]
        templates[:bottom].class.should == CmsTemplate
        templates[:bottom].id.should == 2
        #
        templates[:left].should == nil
        templates[:right].should == nil
      end
    end
  end

  describe "CmsTemplate.create後" do
    before{ pending } unless EcRider.config.ec_admin

    # fixtures :cms_templates, :cms_projects, :cms_template_contents
    fixtures :cms_pages, :cms_page_titles, :cms_binds, :cms_bind_items, :cms_bind_contents

    before(:each) do
      @current_project = CmsProject.create!(
        :project_code => '00005', :project_name => 'project5',
        :priority => 10, 
        :acceptance_type => CmsProject.acceptance_type_id_by_key(:no_approval), 
        :started_at => 1.week.from_now,
        :status_cd => CmsProject.status_id_by_key(:approved)
        )
    end


    describe "削除フラグがfalse" do
      describe "元となったCmsTemplateが関連するCmsPageに対応するCmsPageがCmsProjectにあった場合、そのCmsPageの関連先を自身に変更します" do
        %w(top bottom center).each do |position|
          it "positionが#{position}の場合" do
            # 前提条件の設定
            old_page = CmsPage.find(:first, :conditions => [
                "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', 6])
            old_template = old_page.send("bind_#{position}").cms_template
            new_page = old_page.clone(:include => [
                {:binds => {:items => :contents}},
                :titles
              ])
            new_page.cms_project_id = @current_project.id
            new_page.save!
            # 前提条件の設定完了

            new_template = old_template.new_copy
            new_template.present_version_id.should == old_template.id
            new_template.cms_project_id = @current_project.id
            new_template.save!

            # 終了条件の確認
            new_template.reload
            new_page.reload
            new_page.delete_at_publish.should == nil
            new_template.binds.detect{|b| b.cms_page_id == new_page.id}.should_not be_nil
            new_page.binds.detect{|b| b.cms_template_id == new_template.id}.should_not be_nil
            new_page.send("bind_#{position}").cms_template_id.should == new_template.id
          end
        end
      end

      describe "元となったCmsTemplateが関連するCmsPageに対応するCmsPageがCmsProjectになかったら、対応するCmsPage以下を作成します" do
        %w(top bottom center).each do |position|
          it "positionが#{position}の場合" do
            # 前提条件の設定
            old_page = CmsPage.find(:first, :conditions => [
                "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', 6])
            old_template = old_page.send("bind_#{position}").cms_template
            # 前提条件の設定完了

            new_template = old_template.new_copy
            new_template.present_version_id.should == old_template.id
            new_template.cms_project_id = @current_project.id
            new_template.save!

            # 終了条件の確認
            new_template.reload
            new_page = CmsPage.find(:first, :conditions => [
                "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', @current_project.id])
            new_page.should_not be_nil
            new_page.delete_at_publish.should == nil
            new_template.binds.detect{|b| b.cms_page_id == new_page.id}.should_not be_nil
            new_page.binds.detect{|b| b.cms_template_id == new_template.id}.should_not be_nil
            new_page.send("bind_#{position}").cms_template_id.should == new_template.id
          end
        end
      end
    end

    describe "削除フラグがtrue" do
      describe "元となったCmsTemplateが関連するCmsPageに対応するCmsPageがCmsProjectにあった場合、そのCmsPageの関連先を自身に変更します" do
        %w(top bottom).each do |position|
          it "positionが#{position}の場合" do
            # 前提条件の設定
            old_page = CmsPage.find(:first, :conditions => [
                "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', 6])
            old_template = old_page.send("bind_#{position}").cms_template
            new_page = old_page.clone(:include => [
                {:binds => {:items => :contents}},
                :titles
              ])
            new_page.cms_project_id = @current_project.id
            new_page.save!
            # 前提条件の設定完了

            new_template = CmsTemplate.new(:cms_project_id => @current_project.id, :delete_at_publish => true,
              :name => old_template.name, :template_code => old_template.template_code)
            new_template.present_version_id = old_template.id

            new_template.save!

            # 終了条件の確認
            new_template.reload
            new_page.reload
            new_page.delete_at_publish.should == nil
            new_page.send("bind_#{position}").cms_template_id.should be_nil
          end
        end

        it "positionがcenterの場合" do
          # 前提条件の設定
          old_page = CmsPage.find(:first, :conditions => [
              "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', 6])
          old_template = old_page.bind_center.cms_template
          new_page = old_page.clone(:include => [
              {:binds => {:items => :contents}},
              :titles
            ])
          new_page.cms_project_id = @current_project.id
          new_page.save!
          # 前提条件の設定完了

          new_template = CmsTemplate.new(:cms_project_id => @current_project.id, :delete_at_publish => true,
            :name => old_template.name, :template_code => old_template.template_code)
          new_template.present_version_id = old_template.id
          new_template.save!

          # 終了条件の確認
          new_template.reload
          new_page.reload
          new_page.delete_at_publish.should_not == true
        end
      end

      describe "元となったCmsTemplateが関連するCmsPageに対応するCmsPageがCmsProjectになかったら、対応するCmsPage以下を作成します" do
        %w(top bottom).each do |position|
          it "positionが#{position}の場合" do
            # 前提条件の設定
            old_page = CmsPage.find(:first, :conditions => [
                "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', 6])
            old_template = old_page.send("bind_#{position}").cms_template
            # 前提条件の設定完了

            new_template = CmsTemplate.new(:cms_project_id => @current_project.id, :delete_at_publish => true,
              :name => old_template.name, :template_code => old_template.template_code)
            new_template.present_version_id = old_template.id
            new_template.save!

            # 終了条件の確認
            new_template.reload

            new_page = CmsPage.find(:first, :conditions => [
                "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', @current_project.id])
            new_page.should_not be_nil
            new_page.delete_at_publish.should == nil
            new_template.binds.detect{|b| b.cms_page_id == new_page.id}.should be_nil
            new_page.binds.detect{|b| b.cms_template_id == new_template.id}.should be_nil
            new_page.send("bind_#{position}").cms_template_id.should be_nil
          end
        end

        it "positionがcenterの場合" do
          # 前提条件の設定
          old_page = CmsPage.find(:first, :conditions => [
              "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', 6])
          old_template = old_page.bind_center.cms_template
          new_page = old_page.clone(:include => [
              {:binds => {:items => :contents}},
              :titles
            ])
          # 前提条件の設定完了

          new_template = CmsTemplate.new(:cms_project_id => @current_project.id, :delete_at_publish => true,
            :name => old_template.name, :template_code => old_template.template_code)
          new_template.present_version_id = old_template.id
          new_template.save!

          # 終了条件の確認
          new_template.reload
          new_page = CmsPage.find(:first, :conditions => [
              "controller = ? AND action = ? AND cms_project_id = ?", 'cms', 'index', @current_project.id])
          new_page.should_not be_nil
          new_page.delete_at_publish.should == true
        end
      end
    end
  end
  
  describe "template_list_ids" do
    before do
      CmsProject.delete_all
      CmsTemplate.delete_all
      @cms_projects = {}
      @cms_projects["100"] = CmsProject.new(:started_at => '2008-10-10 00:00:00', :priority => 100, :status_cd => 4, :acceptance_type => '0', :project_name => "test100", :project_code => "test100")
      @cms_projects["101"] = CmsProject.new(:started_at => '2009-10-10 00:00:00', :priority => 101, :status_cd => 4, :acceptance_type => '0', :project_name => "test101", :project_code => "test101")
      @cms_projects["102"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 102, :status_cd => 4, :acceptance_type => '0', :project_name => "test102", :project_code => "test102")
      @cms_projects["103"] = CmsProject.new(:started_at => '2008-10-10 00:00:00', :priority => 103, :status_cd => 4, :acceptance_type => '0', :project_name => "test103", :project_code => "test103")
      @cms_projects["104"] = CmsProject.new(:started_at => '2009-10-10 00:00:00', :priority => 103, :status_cd => 4, :acceptance_type => '0', :project_name => "test104", :project_code => "test104")
      @cms_projects["105"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 103, :status_cd => 4, :acceptance_type => '0', :project_name => "test105", :project_code => "test105")
      @cms_projects["106"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 104, :status_cd => 4, :acceptance_type => '0', :project_name => "test106", :project_code => "test106")
      @cms_projects["107"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 104, :status_cd => 4, :acceptance_type => '0', :project_name => "test107", :project_code => "test107")
      @cms_projects["108"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 104, :status_cd => 4, :acceptance_type => '0', :project_name => "test108", :project_code => "test108")

      @cms_templates = {}
      @cms_templates["1000"] = CmsTemplate.new(:template_code => "1000", :name => "100")
      @cms_templates["1001"] = CmsTemplate.new(:template_code => "1000", :name => "101")
      @cms_templates["1002"] = CmsTemplate.new(:template_code => "1000", :name => "102")
      @cms_templates["1003"] = CmsTemplate.new(:template_code => "1000", :name => "103")
      @cms_templates["1004"] = CmsTemplate.new(:template_code => "1000", :name => "104")
      @cms_templates["1005"] = CmsTemplate.new(:template_code => "1000", :name => "105")
      @cms_templates["1006"] = CmsTemplate.new(:template_code => "1000", :name => "106")
      @cms_templates["1007"] = CmsTemplate.new(:template_code => "1000", :name => "107")
      @cms_templates["1008"] = CmsTemplate.new(:template_code => "1000", :name => "108")

      @cms_templates["1009"] = CmsTemplate.new(:template_code => "1001", :name => "109")
      @cms_templates["1010"] = CmsTemplate.new(:template_code => "1001", :name => "110")
      @cms_templates["1011"] = CmsTemplate.new(:template_code => "1001", :name => "111")
      @cms_templates["1012"] = CmsTemplate.new(:template_code => "1001", :name => "112")
      @cms_templates["1013"] = CmsTemplate.new(:template_code => "1001", :name => "113")
      @cms_templates["1014"] = CmsTemplate.new(:template_code => "1001", :name => "114")
      @cms_templates["1015"] = CmsTemplate.new(:template_code => "1001", :name => "115")
      @cms_templates["1016"] = CmsTemplate.new(:template_code => "1001", :name => "116")
      @cms_templates["1017"] = CmsTemplate.new(:template_code => "1001", :name => "117")

      @cms_templates["1000"].cms_project = @cms_projects["100"]
      @cms_templates["1001"].cms_project = @cms_projects["101"]
      @cms_templates["1002"].cms_project = @cms_projects["102"]
      @cms_templates["1003"].cms_project = @cms_projects["103"]
      @cms_templates["1004"].cms_project = @cms_projects["104"]
      @cms_templates["1005"].cms_project = @cms_projects["105"]
      @cms_templates["1006"].cms_project = @cms_projects["106"]
      @cms_templates["1007"].cms_project = @cms_projects["107"]
      @cms_templates["1008"].cms_project = @cms_projects["108"]

      @cms_templates["1009"].cms_project = @cms_projects["100"]
      @cms_templates["1010"].cms_project = @cms_projects["101"]
      @cms_templates["1011"].cms_project = @cms_projects["102"]
      @cms_templates["1012"].cms_project = @cms_projects["103"]
      @cms_templates["1013"].cms_project = @cms_projects["104"]
      @cms_templates["1014"].cms_project = @cms_projects["105"]
      @cms_templates["1015"].cms_project = @cms_projects["106"]
      @cms_templates["1016"].cms_project = @cms_projects["107"]
      @cms_templates["1017"].cms_project = @cms_projects["108"]
    end
    describe "ターゲットのプロジェクトで編集されたテンプレートの場合" do
      it "ターゲットのプロジェクト 公開中でない場合　ターゲットプロジェクトで編集されたテンプレートが取得できること" do
        @cms_templates.keys.each do |key|
          @cms_templates[key].save!
        end

        condition =CmsTemplate::SearchCondition.new(:cms_project_id => @cms_projects["101"].id)
        template_ids, delete_ids = condition.template_list_ids(@cms_projects["101"].id)
        template_ids.size.should == 2
        CmsTemplate.find_by_id(template_ids[0]).name.should == "101"
        CmsTemplate.find_by_id(template_ids[1]).name.should == "110"
      end
      it "ターゲットのプロジェクト 公開中の場合　ターゲットプロジェクトで編集されたテンプレートが取得できること" do
        @cms_templates.keys.each do |key|
          @cms_templates[key].save!
        end

        condition =CmsTemplate::SearchCondition.new(:cms_project_id => @cms_projects["101"].id)
        template_ids, delete_ids = condition.template_list_ids(@cms_projects["101"].id)
        template_ids.size.should == 2
        CmsTemplate.find_by_id(template_ids[0]).name.should == "101"
        CmsTemplate.find_by_id(template_ids[1]).name.should == "110"
      end
    end
    describe "ターゲットのプロジェクトで未編集のテンプレートの場合" do
      it "優先順位が同順、かつ公開開始日も同順の場合、プロジェクトIDの大きいプロジェクトのテンプレートが取得できること" do
        @cms_templates["1006"].save!
        @cms_templates["1007"].save!
        @cms_templates["1008"].save!
        condition =CmsTemplate::SearchCondition.new(:cms_project_id => 9999)
        template_ids, delete_ids = condition.template_list_ids("9999")
        CmsTemplate.find_by_id(template_ids[0]).name.should == "108"
      end
      it "優先順位が同順、の場合、公開開始日の大きいプロジェクトのテンプレートが取得できること" do
        @cms_templates["1003"].save!
        @cms_templates["1004"].save!
        @cms_templates["1005"].save!
        condition =CmsTemplate::SearchCondition.new(:cms_project_id => 9999)
        template_ids, delete_ids = condition.template_list_ids("9999")
        CmsTemplate.find_by_id(template_ids[0]).name.should == "105"
      end
      it "優先順位の高いプロジェクトのテンプレートが取得できること" do
        @cms_templates["1000"].save!
        @cms_templates["1001"].save!
        @cms_templates["1002"].save!
        condition =CmsTemplate::SearchCondition.new(:cms_project_id => 9999)
        template_ids, delete_ids = condition.template_list_ids("9999")
        CmsTemplate.find_by_id(template_ids[0]).name.should == "102"
      end
      describe "公開終了プロジェクトを含むテスト" do
        it "優先度の高い公開終了プロジェクトがあった場合、公開中の優先順位の高いプロジェクトのテンプレートが取得されること" do
          @cms_templates["1002"].cms_project.status_cd = 5
          @cms_templates["1000"].save!
          @cms_templates["1001"].save!
          @cms_templates["1002"].save!
          condition =CmsTemplate::SearchCondition.new(:cms_project_id => 9999)
          template_ids, delete_ids = condition.template_list_ids("9999")
          CmsTemplate.find_by_id(template_ids[0]).name.should == "101"
        end
      end
      describe "公開終了プロジェクトのみの場合" do
        it "優先順位が同順、かつ公開開始日も同順の場合、プロジェクトIDの大きいプロジェクトのテンプレートが取得できること" do
          @cms_templates["1006"].cms_project.status_cd = 5
          @cms_templates["1007"].cms_project.status_cd = 5
          @cms_templates["1008"].cms_project.status_cd = 5
          @cms_templates["1006"].save!
          @cms_templates["1007"].save!
          @cms_templates["1008"].save!
          condition =CmsTemplate::SearchCondition.new(:cms_project_id => 9999)
          template_ids, delete_ids = condition.template_list_ids("9999")
          CmsTemplate.find_by_id(template_ids[0]).name.should == "108"
        end
        it "優先順位が同順、の場合、公開開始日の大きいプロジェクトのテンプレートが取得できること" do
          @cms_templates["1003"].cms_project.status_cd = 5
          @cms_templates["1004"].cms_project.status_cd = 5
          @cms_templates["1005"].cms_project.status_cd = 5
          @cms_templates["1003"].save!
          @cms_templates["1004"].save!
          @cms_templates["1005"].save!
          condition =CmsTemplate::SearchCondition.new(:cms_project_id => 9999)
          template_ids, delete_ids = condition.template_list_ids("9999")
          CmsTemplate.find_by_id(template_ids[0]).name.should == "105"
        end
        it "優先順位の高いプロジェクトのテンプレートが取得できること" do
          @cms_templates["1000"].cms_project.status_cd = 5
          @cms_templates["1001"].cms_project.status_cd = 5
          @cms_templates["1002"].cms_project.status_cd = 5
          @cms_templates["1000"].save!
          @cms_templates["1001"].save!
          @cms_templates["1002"].save!
          condition =CmsTemplate::SearchCondition.new(:cms_project_id => 9999)
          template_ids, delete_ids = condition.template_list_ids("9999")
          CmsTemplate.find_by_id(template_ids[0]).name.should == "102"
        end

      end
    end
  end
end
