require File.join(File.dirname(__FILE__), '..', 'spec_helper')
 
describe Condition do
  
  it "should add multiple values" do
    where = Condition.block do |c|
      c.add("user_name", "like", "%ABE%")
    end
    where.should == ['user_name like ?', '%ABE%']

    where = Condition.block do |c|
      c.add("user_name", "like", "%ABE%")
      c.add("birthday", ">=", Date.new(1973, 1, 1))
    end
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
  end
  
  it "should support simple WHERE building" do
    where = Condition.block do |c|
      c.add("user_name", "like", "%ABE%").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
  end
  
  it "should add value with filter" do
    blank_filter = Proc.new{|cond, *vals| !vals.last.blank?}
    
    where = Condition.block do |c|
      c.filter(blank_filter)
      c.and("user_name", "like", "%ABE%")
      c.add("birthday", ">=", Date.new(1973, 1, 1))
    end
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
    
    where = Condition.block do |c|
      c.filter(blank_filter).
        add("user_name", "like", nil).
        add("birthday", ">=", Date.new(1973, 1, 1)) 
    end
    where.should == ['birthday >= ?', Date.new(1973, 1, 1)]
    
    where = Condition.block do |c|
      c.filter(blank_filter).
        add("user_name", "like", nil).
        add("birthday", ">=", nil)
    end
    where.should == nil
  end
  
  it "should work with default_filter" do
    where = Condition.block do |c|
      c.add("user_name", "like", "%ABE%").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
    
    where = Condition.block do |c|
      c.add("user_name", "like", "%%").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['birthday >= ?', Date.new(1973, 1, 1)]
    
  end
  
  it "should work with filters" do
    blank_filter = Proc.new{|cond, val| !val.blank?}
    
    where = Condition.block(blank_filter) do |c|
      c.add("user_name", "like", "%ABE%").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
    
    where = Condition.block(:ignore_blank) do |c|
      c.add("user_name", "like", nil).
        add("birthday", ">=", Date.new(1973, 1, 1))
    end
    where.should == ['birthday >= ?', Date.new(1973, 1, 1)]
    
    where = Condition.block(:ignore_blank) do |c|
      c.add("user_name", "like", nil).
        add("birthday", ">=", nil)
    end
    where.should == nil
  end

  it "should add_like" do
    blank_filter = Proc.new{|cond, val| !val.blank?}
    
    where = Condition.block(blank_filter) do |c|
      c.add_like("user_name", "like", "ABE").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
    
    
    where = Condition.block(blank_filter) do |c|
      c.add_like_partial("user_name", "like", "ABE").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
    
    where = Condition.block(blank_filter) do |c|
      c.add_like_forward("user_name", "like", "ABE").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['user_name like ? AND birthday >= ?', 'ABE%', Date.new(1973, 1, 1)]
    
    where = Condition.block(blank_filter) do |c|
      c.add_like_backward("user_name", "like", "ABE").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['user_name like ? AND birthday >= ?', '%ABE', Date.new(1973, 1, 1)]
    
    where = Condition.block(blank_filter) do |c|
      c.add_like_partial("user_name", "like", "ABE").
        add("birthday", ">=", Date.new(1973, 1, 1))
    end 
    where.should == ['user_name like ? AND birthday >= ?', '%ABE%', Date.new(1973, 1, 1)]
    
    where = Condition.block(:ignore_blank) do |c|
      c.add_like("user_name", "like", nil).
        add("birthday", ">=", Date.new(1973, 1, 1))
    end
    where.should == ['birthday >= ?', Date.new(1973, 1, 1)]
    
    where = Condition.block(:ignore_blank) do |c|
      c.add_like("user_name", "like", nil).
        add("birthday", ">=", nil)
    end
    where.should == nil
  end

  
  
  
  
  it "should work with block" do
    where = Condition.block(:ignore_blank) do |c|
      c.or("user_name", "like", '%AEG%')
      c.or do |c1|
        c1.add("birthday", ">=", Date.new(2001, 1, 1))
        c1.add("birthday", "<=", Date.new(2004,12,31))
      end
    end
    where.should == 
      ['user_name like ? OR (birthday >= ? AND birthday <= ?)', '%AEG%', Date.new(2001, 1, 1), Date.new(2004,12,31)]
  end
  
  it "should work with block without args" do
    pending
    where = Condition.block(:ignore_blank) do
      add("user_name", "like", '%AEG%')
      with('AND') do
        add("birthday", ">=", Date.new(2001, 1, 1))
        add("birthday", "<= ", Date.new(2004,12,31))
      end
    end
    where.should == 
      ['user_name like ? OR (birthday >= ? AND birthday <= ?)', '%AEG%', Date.new(2001, 1, 1), Date.new(2004,12,31)]
  end
  
  it "should raise ArgumentError when added empty arguments" do
    lambda{
      Condition.block(:ignore_blank) do |c|
        c.filters.clear
        c.add
      end
    }.should raise_error(ArgumentError)
  end

  it "should raise ArgumentError when added column name only" do
    lambda{
      Condition.block(:ignore_blank) do |c|
        c.filters.clear
        c.add("one_column_name_only")
      end
    }.should raise_error(ArgumentError)
  end

  it "should be able to deal IN operator with Array" do
    where = Condition.block(:ignore_blank) do |c|
      c.add("status_cd", [1,2,3])
    end
    where.should == ["status_cd IN (?)", [1,2,3]]
  end

  it "should ignore operator with Array value" do
    where = Condition.block(:ignore_blank) do |c|
      c.add("status_cd", 'UNEXIST_OPERATOR', [1,2,3])
    end
    where.should == ["status_cd IN (?)", [1,2,3]]
  end

  it "should deal BETWEEN 2 values" do
    where = Condition.block(:ignore_blank) do |c|
      c.add("status_cd", 'BETWEEN', 1, 2)
    end
    where.should == ["status_cd BETWEEN ? AND ?", 1,2]
  end

  it "should raise ArgumentError with multiple arguments but without BETWEEN" do
    lambda{ 
      Condition.block(:ignore_blank) do |c|
        c.add("status_cd", 'IN', 1, 2)
      end
    }.should raise_error(ArgumentError)
  end
  
end
 
