# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe OrderItem do
  fixtures :order_items, :orders, :order_shippings, :kbn_codes, 
    :products, :product_skus, :product_sku_inventories, :product_sku_secure_inventories,
    :product_components

  it "course_div means periodic purchase or 1 time purchase." do
    OrderItem.course_div_options.should == [["単品", '1'], ["定期", '2'], ['プレゼント', '3']]
    OrderItem.course_div_name_by_id('1').should == "単品"
    OrderItem.course_div_name_by_id('2').should == "定期"
    OrderItem.course_div_name_by_id('3').should == 'プレゼント'

    p1 = OrderItem.new
    p1.course_div.should be_nil
    p1.course_div_key.should be_nil
    p1.course_div_name.should be_nil
    p1.course_div = '1'
    p1.course_div.should == '1'
    p1.course_div_key.should == :normal
    p1.course_div_name.should == '単品'
    p1.course_div = '2'
    p1.course_div.should == '2'
    p1.course_div_key.should == :periodic
    p1.course_div_name.should == '定期'
    p1.course_div = '3'
    p1.course_div.should == '3'
    p1.course_div_key.should == :present
    p1.course_div_name.should == 'プレゼント'
  end

  describe "check_stocks" do
    it "test check_stocks" do
      order = OrderItem.find(1)

      product_sku = ProductSku.find(265)
      order.qty.should == 1
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 19

      order.check_stocks.should == 19
      product_sku.orderable_count.should == 19

      order.qty = 11
      order.check_stocks.should == 9
      product_sku.orderable_count.should == 19

      attributes = order.attributes.dup
      attributes.delete(:id)
      new_order = OrderItem.new(attributes)
      new_order.qty = 7
      new_order.save_with_stock_control(:undelivery)

      product_sku = ProductSku.find(265)
      order.check_stocks.should == 2
      product_sku.orderable_count.should == 12

      new_order.qty = 19
      new_order.check_stocks.should == 0

      new_order.qty = 20
      lambda { new_order.check_stocks }.should raise_error(OrderItem::OutOfStockError)
    end
  end

  it "test_save_with_stock_control" do
    order = OrderItem.find(1)
    
    product_sku = ProductSku.find(265)
    order.qty.should == 1
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 19

    order.qty = 11
    order.save_with_stock_control(:undelivery)
    
    product_sku = ProductSku.find(265)
    product_sku.parent_stock_count.should == 20
    product_sku.secure_stock_count.should == 11
    product_sku.stock_count.should == 20
    product_sku.composable_stock_count.should == 20
    product_sku.orderable_count.should == 9

    attributes = order.attributes.dup
    attributes.delete(:id)
    new_order = OrderItem.new(attributes)
    new_order.qty = 7
    new_order.save_with_stock_control(:undelivery)
    
    product_sku = ProductSku.find(265)
    product_sku.parent_stock_count.should == 20
    product_sku.secure_stock_count.should == 18
    product_sku.stock_count.should == 20
    product_sku.composable_stock_count.should == 20
    product_sku.orderable_count.should == 2

    new_order.qty = 15
    assert_raise(OrderItem::OutOfStockError){
      new_order.save_with_stock_control(:undelivery)
    }
  end

  it "test_save_with_stock_control target product deleted" do
    order = OrderItem.find(1)
    product = Product.find(1)
    product.destroy

    order.qty.should == 1
    order.qty = 11
    order.save_with_stock_control(:undelivery)
    OrderItem.find(1).qty.should == 11
  end

  it "test_destroy_with_stock_control" do
    order = OrderItem.find(1)
    
    product_sku = ProductSku.find(265)
    product_sku.parent_stock_count.should == 20
    product_sku.secure_stock_count.should == 1
    product_sku.stock_count.should == 20
    product_sku.composable_stock_count.should == 20
    product_sku.orderable_count.should == 19

    order.destroy_with_stock_control
    
    product_sku = ProductSku.find(265)
    product_sku.parent_stock_count.should == 20
    product_sku.secure_stock_count.should == 0
    product_sku.stock_count.should == 20
    product_sku.composable_stock_count.should == 20
    product_sku.orderable_count.should == 20
  end

  it "test_destroy_with_stock_control target product deleted" do
    order = OrderItem.find(1)
    product = Product.find(1)
    product.destroy

    order.qty.should == 1
    order.destroy_with_stock_control
    OrderItem.find(:all, :conditions => {:id => 1}).size.should == 0
  end
  
  describe "reduce_stocks" do
    it "引き当てされている場合" do
      product_sku = ProductSku.find(266)
      org_stock_count = product_sku.stock_count
      org_orderable_count = product_sku.orderable_count
      org_secure_stock_count = product_sku.secure_stock_count
      
      child_product_sku = ProductSku.find(289)
      org_child_stock_count = child_product_sku.stock_count
      org_child_orderable_count = child_product_sku.orderable_count
      org_child_secure_stock_count = child_product_sku.secure_stock_count
      
      org_inventory_history_count = ProductSkuInventory.count()

      order_item = OrderItem.find(14)
      order_item.reduce_stocks
      
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == org_stock_count - 10 #在庫減
      product_sku.orderable_count.should == org_orderable_count #購入可能数はそのまま
      product_sku.secure_stock_count == org_secure_stock_count - 10 #引き当て数減
      
      child_product_sku = ProductSku.find(289)
      child_product_sku.stock_count.should == org_child_stock_count - 1 #在庫減
      child_product_sku.orderable_count.should == org_child_orderable_count #購入可能数はそのまま
      child_product_sku.secure_stock_count == org_child_secure_stock_count - 1 #引き当て数減
      
      ProductSkuInventory.count().should == org_inventory_history_count + 2
    end
    
    it "引き当てされていない場合" do
      order_item = OrderItem.find(14)
      order_item.product_sku_secure_inventories.each do |secure_inventory|
        order_item.product_sku.secure_stock_count -= secure_inventory.count
        order_item.product_sku.orderable_count += secure_inventory.count
        order_item.product_sku.save!
        secure_inventory.destroy
      end
      
      product_sku = ProductSku.find(266)
      org_stock_count = product_sku.stock_count
      org_orderable_count = product_sku.orderable_count
      org_secure_stock_count = product_sku.secure_stock_count
      
      child_product_sku = ProductSku.find(289)
      org_child_stock_count = child_product_sku.stock_count
      org_child_orderable_count = child_product_sku.orderable_count
      org_child_secure_stock_count = child_product_sku.secure_stock_count
      
      org_inventory_history_count = ProductSkuInventory.count()

      order_item = OrderItem.find(14)
      order_item.reduce_stocks
      
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == org_stock_count
      product_sku.orderable_count.should == org_orderable_count
      product_sku.secure_stock_count == org_secure_stock_count
      
      child_product_sku = ProductSku.find(289)
      child_product_sku.stock_count.should == org_child_stock_count
      child_product_sku.orderable_count.should == org_child_orderable_count
      child_product_sku.secure_stock_count == org_child_secure_stock_count
      
      ProductSkuInventory.count().should == org_inventory_history_count
    end
  end
  
  describe "restore_stocks" do
    it "減らした在庫を戻す" do
      order_item = OrderItem.find(14)
      order_item.product_sku_id = 282
      order_item.qty = 2
      order_item.save!
      
      lambda {
        order_item.restore_stocks
      }.should change(ProductSkuInventory, :count).by(3)
      
      order_item = OrderItem.find(14)
      inventory = ProductSkuInventory.find(:first, :conditions => ["product_sku_id = ?", 282], :order => "id desc")
      inventory.count.should == 2
      inventory.comment.should_not be_nil
      inventory = ProductSkuInventory.find(:first, :conditions => ["product_sku_id = ?", 288], :order => "id desc")
      inventory.count.should == 2
      inventory.comment.should_not be_nil
      inventory = ProductSkuInventory.find(:first, :conditions => ["product_sku_id = ?", 289], :order => "id desc")
      inventory.count.should == 4
      inventory.comment.should_not be_nil

      secure_inventory = ProductSkuSecureInventory.find(:first, :conditions => ["product_sku_id = ? AND order_item_id = ?", 282, 14], :order => "id desc")
      secure_inventory.count.should == 2
      secure_inventory = ProductSkuSecureInventory.find(:first, :conditions => ["product_sku_id = ? AND order_item_id = ?", 288, 14], :order => "id desc")
      secure_inventory.count.should == 2
      secure_inventory = ProductSkuSecureInventory.find(:first, :conditions => ["product_sku_id = ? AND order_item_id = ?", 289, 14], :order => "id desc")
      secure_inventory.count.should == 4

      sku282 = ProductSku.find(282)
      sku282.parent_stock_count.should == 101
      sku282.secure_stock_count.should == 2
      sku282.composable_stock_count.should == 27
      sku282.orderable_count.should == 24

      sku288 = ProductSku.find(288)
      sku288.parent_stock_count.should == 53
      sku288.secure_stock_count.should == 2
      sku288.composable_stock_count.should == 53
      sku288.orderable_count.should == 51

      sku289 = ProductSku.find(289)
      sku289.parent_stock_count.should == 54
      sku289.secure_stock_count.should == 5
      sku289.composable_stock_count.should == 54
      sku289.orderable_count.should == 49
    end
  end

  describe "#1182 64bitOSの本番サーバで注文するとエラー" do
    it "qtyは(10進表記で)５桁まで入力可" do
      item = OrderItem.new(:qty => 99999, :subtotal_amount => 10)
      item.valid?
      item.errors.full_messages.should == []
    end

    it "qtyは(10進表記で)6桁以上ならエラー" do
      item = OrderItem.new(:qty => 100000, :subtotal_amount => 10)
      item.valid?
      item.errors.full_messages.should == ["数量 は5文字以内で入力してください。"]
    end

    it "subtotal_amountは(10進表記で)10桁まで入力可" do
      item = OrderItem.new(:qty => 1, :subtotal_amount => 9999999999)
      item.valid?
      item.errors.full_messages.should == []
    end

    it "subtotal_amountは(10進表記で)11桁以上ならエラー" do
      item = OrderItem.new(:qty => 1, :subtotal_amount => 10000000000)
      item.valid?
      item.errors.full_messages.should == ["小計 は10文字以内で入力してください。"]
    end
  end

  describe "check_sales_period" do
    it "check_sales_period = true" do
      order_item = OrderItem.find(1)
      order_item.product_sku.product.sale_started_at.should be_nil
      order_item.product_sku.product.sale_finished_at.should be_nil
      order_item.check_sales_period.should == true
    end
    it "check_sales_period now < sale_started_at false" do
      order_item = OrderItem.find(14)
      order = order_item.order_shipping.order
      order.ordered_at = Time.parse("2009/02/01 00:00:00")
      order.save!
      product= order_item.product_sku.product
      product.sale_started_at = Time.parse('2010-06-02 11:16:00')
      product.save!
      lambda {order_item.check_sales_period}.should raise_error(OrderItem::NonPeriodicSalesError)
    end
    it "check_sales_period now > sale_finished_at false" do
      order_item = OrderItem.find(14)
      order = order_item.order_shipping.order
      order.ordered_at = Time.parse("2009/12/01 00:00:00")
      product = order_item.product_sku.product
      product.sale_finished_at = Time.parse('2009-06-02 11:16:00')
      product.save!
      lambda {order_item.check_sales_period}.should raise_error(OrderItem::NonPeriodicSalesError)
    end
  end

end
