# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe Order do

  fixtures(:users, :user_emails, :user_addresses, :countries, :shipping_costs, :shipping_cost_boundaries, :prefectures, :prefecture_displays,
    :products, :product_displays, :product_skus, :product_prices, :product_components,
    :product_sku_inventories, :product_sku_secure_inventories,
    :orders, :order_shippings, :order_items,
    :point_settings, :user_points, :user_ranks,
    :distributors, :distributor_delivery_times, :distributor_delivery_time_displays,
    :kbn_codes,:kbn_code_names, :shop_settings, :campaigns)

  before(:all) do
    @target = Order.new
  end

  #
  # =Orderの検索のテスト=
  #
  it "test_find" do
    # 論理削除されていない状態で検索
    include = [{:order_shippings => :order_items}, {:user => :user_addresses}]
    orders = Order.find(:all, :include => include, :conditions => 'order_items.deleted_at is null')
    assert_equal(orders.length, 12)

    order_shippings = []
    orders.each do |order|
      order_shippings += order.order_shippings
    end
    assert_equal(order_shippings.length, 11)

    order_items = []
    order_shippings.each do |order_shipping|
      order_items += order_shipping.order_items
    end
    assert_equal(order_items.length, 16)

    # 論理削除された状態で検索
    Order.destroy(6)
    OrderShipping.destroy(11)
    OrderItem.destroy(1)

    orders = Order.find(:all, :include => include, :conditions => 'order_items.deleted_at is null')
    orders.length.should == 10

    order_ids = []
    order_shippings = []
    orders.each do |order|
      order_ids << order.id
      order_shippings += order.order_shippings
    end
    assert_equal(order_ids.sort, [2, 3, 4, 5, 7, 8, 9, 10, 11, 12])
    assert_equal(order_shippings.length, 8)

    order_shipping_ids = []
    order_items = []
    order_shippings.each do |order_shipping|
      order_shipping_ids << order_shipping.id
      order_items += order_shipping.order_items
    end
    assert_equal(order_shipping_ids.sort, [2, 3, 4, 5, 6, 9, 10, 11])
    assert_equal(order_items.length, 11)

    order_ids = []
    order_items.each do |order|
      order_ids << order.id
    end
    assert_equal(order_ids.sort, [2, 3, 4, 5, 6, 7, 12, 13, 14, 20, 21])
  end

  it "test_find_any_condition" do
    orders = Order.find_any_condition({}, nil, nil)
    assert_equal(orders.length, 8)

    orders = Order.find_any_condition({}, 5, 3)
    assert_equal(orders.length, 5)

    orders = Order.find_any_condition({:name_first => '山下'}, nil, nil)
    assert_equal(orders.length, 1)
    assert_equal(orders[0].id, 5)
    assert_equal(orders[0].order_shippings.length, 2)
    order_shipping_ids = [orders[0].order_shippings[0].id, orders[0].order_shippings[1].id]
    assert_equal(order_shipping_ids.sort, [5, 6])

    orders = Order.find_any_condition({:delivery_kanji_name => '桃子'}, nil, nil)
    assert_equal(orders.length, 1)
    assert_equal(orders[0].id, 5)
    assert_equal(orders[0].order_shippings.length, 1)
    assert_equal(orders[0].order_shippings[0].id, 6)
  end

  describe "test_total_amount" do
    it "pattern1" do
      total_amount = Order.total_amount_and_shipping_cost({})[:total_amount]
      total_amount.should == 331940
    end
    it "pattern2" do
      total_amount = Order.total_amount_and_shipping_cost({:name_first => '山下'})[:total_amount]
      total_amount.should == 45360
    end
    it "pattern3" do
      total_amount = Order.total_amount_and_shipping_cost({:delivery_kanji_name => '桃子'})[:total_amount]
      total_amount.should == 11340
    end
    it "pattern4" do
      OrderShipping.destroy(3)
      total_amount = Order.total_amount_and_shipping_cost({})[:total_amount]
      total_amount.should == 328580
    end
  end

  describe "test_total_shipping_cost" do
    it "pattern1" do
      total_shipping_cost = Order.total_amount_and_shipping_cost({})[:shipping_cost]
      assert_equal(total_shipping_cost, 310)
    end
    it "pattern2" do
      total_shipping_cost = Order.total_amount_and_shipping_cost({:name_first => '山下'})[:shipping_cost]
      assert_equal(total_shipping_cost, 0)
    end
    it "pattern3" do
      total_shipping_cost = Order.total_amount_and_shipping_cost({:delivery_kanji_name => '桃子'})[:shipping_cost]
      assert_equal(total_shipping_cost, 0)
    end
    it "pattern4" do
      OrderShipping.destroy(3)
      total_shipping_cost = Order.total_amount_and_shipping_cost({})[:shipping_cost]
      assert_equal(total_shipping_cost, 100)
    end
  end

  it "cancel_requestable 全ステータスで確認" do
    OrderShipping.status_div_keys.each do |key|
      order_shipping = OrderShipping.new(:status_div_key => key)
      order = Order.new()
      order.stub(:order_shippings).and_return([order_shipping])

      case key
      when :before_settlement;     order.cancel_status.should == :impossible
      when :undelivery_settlement; order.cancel_status.should == :possible
      when :undelivery_unpaid;     order.cancel_status.should == :possible
      when :undelivery;            order.cancel_status.should == :possible
      when :deliver_request;       order.cancel_status.should == :impossible
      when :delivered;             order.cancel_status.should == :impossible
      when :deliver_advised;       order.cancel_status.should == :impossible
      when :cancel_request;        order.cancel_status.should == :impossible
      when :canceled;              order.cancel_status.should == :impossible
      when :settlement_failed;     order.cancel_status.should == :impossible
      when :stock_error;           order.cancel_status.should == :impossible
      when :internal_error;        order.cancel_status.should == :impossible
      when :external_error;        order.cancel_status.should == :impossible
      when :returned;              order.cancel_status.should == :impossible
      when :sales_period_error;    order.cancel_status.should == :impossible
      else violated '想定外のステータスです'
      end
    end
  end

  it "generate_ec_order_code" do
    order = Order.create
    order.ec_order_code.should == order.id.to_s.rjust(11, '0')
  end

  it "normal print_order_detail" do
    order_shipping1 = orders(:orders_00007)
    order_shipping1.print_order_detail.should == <<EOS
[オンライン受注番号] 00000000007
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱
      9702円 x 10 = 97020円

[伝票番号] No.00000000010
[送付先] 田村 拓哉 様
  〒1000001
  東京都千代田区東田３００－１００　千代田生命ビル１１１号
  (TEL)000-111-2226
[支払方法] 代金引換
[配達希望日時] 2007/12/20 日本語3309
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱
      9702円 x 10 = 97020円

[伝票番号] No.00000000011
[送付先] 加藤 慎吾 様
  〒1000003
  東京都千代田区一ツ橋１－２－３　一ツ橋目の前にあるマンション４０４号
  (TEL)000-111-2227
[支払方法] クレジットカード
[配達希望日時] 2008/01/01 日本語2567
================================================================
小計                          0円
送料+手数料                   0円
----------------------------------------------------------------
合計                     194040円
----------------------------------------------------------------
EOS
  end

  it "normal print_order_detail for oversea" do
    order_shipping1 = Order.find(7, :include => {:order_shippings => :order_items})
    order_shipping1.order_shippings.first.country_code = 'CN'
    order_shipping1.print_order_detail.should == <<EOS
[オンライン受注番号] 00000000007
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱
      9702円 x 10 = 97020円

[伝票番号] No.00000000010
[送付先] 拓哉 田村 様
  〒1000001
  東田３００－１００　千代田生命ビル１１１号, 千代田区, 東京都, China
  (TEL)000-111-2226
[支払方法] 代金引換
[配達希望日時] 2007/12/20 
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱
      9702円 x 10 = 97020円

[伝票番号] No.00000000011
[送付先] 加藤 慎吾 様
  〒1000003
  東京都千代田区一ツ橋１－２－３　一ツ橋目の前にあるマンション４０４号
  (TEL)000-111-2227
[支払方法] クレジットカード
[配達希望日時] 2008/01/01 日本語2567
================================================================
小計                          0円
送料+手数料                   0円
----------------------------------------------------------------
合計                     194040円
----------------------------------------------------------------
EOS
  end

  it "normal print_order_detail from not Japanese and not simplified Chinese locale" do
    order_shipping1 = Order.find(7, :include => {:order_shippings => :order_items})
    order_shipping1.order_shippings.first.country_code = 'CN'
    order_shipping1.locale = 'th'
    order_shipping1.print_order_detail.should == <<EOS
[Online order number] 00000000007
================================================================
[0000000009050-00-00-00] Energy Drink(10case)
      9702yen x 10 = 97020yen

[Slip Number] No.00000000010
[shipping adress] 拓哉 田村
  ZIP1000001
  東田３００－１００　千代田生命ビル１１１号, 千代田区, 東京都, China
  (Phone Number)000-111-2226
[Payment type] Price substitution
[Delivery hope time] 2007/12/20 
================================================================
[0000000009050-00-00-00] Energy Drink(10case)
      9702yen x 10 = 97020yen

[Slip Number] No.00000000011
[shipping adress] 加藤 慎吾
  ZIP1000003
  東京都千代田区一ツ橋１－２－３　一ツ橋目の前にあるマンション４０４号
  (Phone Number)000-111-2227
[Payment type] credit card
[Delivery hope time] 2008/01/01 英語2567
================================================================
subtotal                          0yen
shipping                   0yen
----------------------------------------------------------------
total                     194040yen
----------------------------------------------------------------
EOS
  end

  it "normal print_order_detail from simplified Chinese locale (product_name found)" do
    product_display = ProductDisplay.new(:product_id => 2, :locale => 'zh_CN', :product_name_1 => '栄養ドリンク10箱(簡体字)')
    product_display.save!
    order1 = Order.find(7, :include => {:order_shippings => :order_items})
    order1.order_shippings[0].country_code = 'CN'
    order1.order_shippings[1].country_code = 'CN'
    order1.locale = 'zh_CN'
    order1.save!
    order1.print_order_detail.should == <<EOS
[在线购入编码] 00000000007
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱(簡体字)
      9702日元 x 10 = 97020日元

[购入编码] No.00000000010
[邮寄地址] 田村 拓哉
  邮政编码1000001
  東京都千代田区 東田３００－１００　千代田生命ビル１１１号, China
  (电话号码)000-111-2226
[付款方式] 货到支付现金
[希望邮寄日期时间] 2007/12/20 
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱(簡体字)
      9702日元 x 10 = 97020日元

[购入编码] No.00000000011
[邮寄地址] 加藤 慎吾
  邮政编码1000003
  東京都千代田区 一ツ橋１－２－３　一ツ橋目の前にあるマンション４０４号, China
  (电话号码)000-111-2227
[付款方式] 信用卡
[希望邮寄日期时间] 2008/01/01 
================================================================
小计                          0日元
邮费                   0日元
----------------------------------------------------------------
合计                     194040日元
----------------------------------------------------------------
EOS
  end

  it "normal print_order_detail from simplified Chinese locale (product_name not found)" do
    order1 = Order.find(7, :include => {:order_shippings => :order_items})
    order1.order_shippings[0].country_code = 'CN'
    order1.order_shippings[1].country_code = 'CN'
    order1.locale = 'zh_CN'
    order1.save!
    order1.print_order_detail.should == <<EOS
[在线购入编码] 00000000007
================================================================
[0000000009050-00-00-00] Energy Drink(10case)
      9702日元 x 10 = 97020日元

[购入编码] No.00000000010
[邮寄地址] 田村 拓哉
  邮政编码1000001
  東京都千代田区 東田３００－１００　千代田生命ビル１１１号, China
  (电话号码)000-111-2226
[付款方式] 货到支付现金
[希望邮寄日期时间] 2007/12/20 
================================================================
[0000000009050-00-00-00] Energy Drink(10case)
      9702日元 x 10 = 97020日元

[购入编码] No.00000000011
[邮寄地址] 加藤 慎吾
  邮政编码1000003
  東京都千代田区 一ツ橋１－２－３　一ツ橋目の前にあるマンション４０４号, China
  (电话号码)000-111-2227
[付款方式] 信用卡
[希望邮寄日期时间] 2008/01/01 
================================================================
小计                          0日元
邮费                   0日元
----------------------------------------------------------------
合计                     194040日元
----------------------------------------------------------------
EOS
  end

  it "normal print_order_detail_for_shop" do
    order_shipping1 = orders(:orders_00007)
    order_shipping1.print_order_detail(:text, {:shipment_complete_mail =>true}).should == <<EOS
[オンライン受注番号] 00000000007
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱
      9702円 x 10 = 97020円

[伝票番号] No.00000000010
[送付先] 田村 拓哉 様
  〒1000001
  東京都千代田区東田３００－１００　千代田生命ビル１１１号
  (TEL)000-111-2226
[支払方法] 代金引換
[配達希望日時] 2007/12/20 日本語3309
================================================================
[0000000009050-00-00-00] 栄養ドリンク10箱
      9702円 x 10 = 97020円

[伝票番号] No.00000000011
[送付先] 加藤 慎吾 様
  〒1000003
  東京都千代田区一ツ橋１－２－３　一ツ橋目の前にあるマンション４０４号
  (TEL)000-111-2227
[支払方法] クレジットカード
[配達希望日時] 2008/01/01 日本語2567
================================================================
小計                          0円
送料+手数料                   0円
----------------------------------------------------------------
合計                     194040円
----------------------------------------------------------------
EOS
  end

  it "test_check_sales_period admin" do
    EcRider.config.stub!(:ec_admin).and_return(true)
    order = Order.find(2)
    order.ordered_at.should == Time.parse('2007-12-10 11:06:54')
    order_items = order.order_shippings[0].order_items
    order_items.length.should == 2
    product1 = order_items[0].product_sku.product
    product2 = order_items[1].product_sku.product
    product1.sale_started_at.should be_nil
    product1.sale_finished_at.should be_nil
    product2.sale_started_at.should be_nil
    product2.sale_finished_at.should be_nil
    
    lambda { order.check_sales_period }.should_not raise_error()

    product1.sale_started_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should_not raise_error()

    product1.sale_started_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should_not raise_error()

    product1.sale_started_at = nil
    product1.sale_finished_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should_not raise_error()
    
    product1.sale_finished_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should_not raise_error()
  end

  it "test_check_sales_period no admin" do
    EcRider.config.stub!(:ec_admin).and_return(false)
    order = Order.find(2)
    order.ordered_at.should == Time.parse('2007-12-10 11:06:54')
    order_items = order.order_shippings[0].order_items
    order_items.length.should == 2
    product1 = order_items[0].product_sku.product
    product2 = order_items[1].product_sku.product
    product1.sale_started_at.should be_nil
    product1.sale_finished_at.should be_nil
    product2.sale_started_at.should be_nil
    product2.sale_finished_at.should be_nil
    
    lambda { order.check_sales_period }.should_not raise_error()

    product1.sale_started_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should raise_error(OrderItem::NonPeriodicSalesError)

    product1.sale_started_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should_not raise_error()

    product1.sale_started_at = nil
    product1.sale_finished_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should_not raise_error()
    
    product1.sale_finished_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order.check_sales_period }.should raise_error(OrderItem::NonPeriodicSalesError)
  end

  it "test_check_stocks" do
    order = Order.find(7)

    product_sku = ProductSku.find(266)
    order.order_shippings[0].order_items[0].qty.should == 10
    product_sku.stock_count.should == 50
    product_sku.orderable_count.should == 23
    
    lambda { order.check_stocks }.should_not raise_error()
    product_sku.orderable_count.should == 23

    #secure_stocksを2回呼び出しても重複して引き当てしない
    order.secure_stocks(:before_pay)

    #数量変更、注文商品追加
    order.order_shippings[0].order_items[0].qty = 20
    order.order_shippings[1].order_items[0].qty = 12
    lambda { order.check_stocks }.should_not raise_error()
    product_sku.orderable_count.should == 23

    order.order_shippings[0].order_items[0].qty = 100
    lambda { order.check_stocks }.should raise_error(OrderItem::OutOfStockError)
    product_sku.orderable_count.should == 23
  end
  
  describe "secure_stocks" do
    it "仮引き当て　複数回呼び出し" do
      order = Order.find(7)
      order_item1 = order.order_shippings[0].order_items[0]
      order_item1.qty.should == 10
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => order_item1.id})
      secures.length.should == 1
  
      product_sku = ProductSku.find(266)
      order.order_shippings[0].order_items[0].qty.should == 10
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 23
  
      #secure_stocksを2回呼び出しても重複して引き当てしない
      order.secure_stocks(:before_pay)
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => order_item1.id})
      secures.length.should == 1
      secures[0].count.should == 10
      secures[0].paid_flg.should be_false
  
      product_sku = ProductSku.find(266)
      order.order_shippings[0].order_items[0].qty.should == 10
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 23
  
      #数量変更、注文商品追加
      order.order_shippings[0].order_items[0].qty = 20
      order.order_shippings[1].order_items[0].qty = 12
      order.secure_stocks(:before_pay)
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => order_item1.id})
      secures.length.should == 1
      secures[0].count.should == 20
      secures[0].paid_flg.should be_false
  
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 11
  
      order.order_shippings[0].order_items[0].save!
      order.order_shippings[1].order_items[0].save!
  
      order.order_shippings[0].order_items[0].qty = 100
      assert_raise(OrderItem::OutOfStockError){
        order.secure_stocks(:before_pay)
      }
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => order_item1.id})
      secures.length.should == 1
      secures[0].count.should == 20
      secures[0].paid_flg.should be_false
    end
  
    it "ec_testでは在庫を更新しない" do
      EcRider.config.stub!(:ec_test).and_return(true)
      order = Order.find(7)
  
      product_sku = ProductSku.find(266)
      order.order_shippings[0].order_items[0].qty.should == 10
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 23
  
      order.secure_stocks(:before_pay)
  
      product_sku = ProductSku.find(266)
      order.order_shippings[0].order_items[0].qty.should == 10
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 23
  
      #数量変更、注文商品追加
      order.order_shippings[0].order_items[0].qty = 20
      order.order_shippings[1].order_items[0].qty = 12
      order.secure_stocks(:before_pay)
  
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 23
  
      order.order_shippings[0].order_items[0].save!
      order.order_shippings[1].order_items[0].save!
  
      order.order_shippings[0].order_items[0].qty = 100
      assert_raise(OrderItem::OutOfStockError){
        order.secure_stocks(:before_pay)
      }
    end
    
    it "子SKUなし" do
      order = Order.find(4, :include => {:order_shippings => :order_items})
      new_order = order.clone(:include => {:order_shippings => :order_items})
      new_order.save!
      new_order_item = new_order.order_shippings[0].order_items[0]
      new_order_item.qty.should == 1
  
      product_sku = ProductSku.find(new_order_item.product_sku_id)
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 23
  
      # 仮引き当て
      new_order.secure_stocks(:before_pay)
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => new_order_item.id})
      secures.length.should == 1
      secures[0].count.should ==1
      secures[0].paid_flg.should be_false
      product_sku = ProductSku.find(new_order_item.product_sku_id)
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 22
      # 引き当て
      new_order.secure_stocks
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => new_order_item.id})
      secures.length.should == 1
      secures[0].count.should ==1
      secures[0].paid_flg.should be_true
      product_sku = ProductSku.find(new_order_item.product_sku_id)
      product_sku.stock_count.should == 50
      product_sku.orderable_count.should == 22
      # 開放
      new_order.unsecure_stocks
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => new_order_item.id})
      secures.length.should == 0
    end
    
    it "子SKUあり" do
      order = Order.find(4, :include => {:order_shippings => :order_items})
      new_order = order.clone(:include => {:order_shippings => :order_items})
      new_order.save!
      new_order_item = new_order.order_shippings.first.order_items.first
      new_order_item.product_sku_id = 282
      new_order_item.qty = 2
      new_order_item.save!
  
      product_sku = ProductSku.find(new_order_item.product_sku_id)
      product_sku.stock_count.should == 25
      product_sku.orderable_count.should == 24
  
      # 仮引き当て
      new_order.secure_stocks(:before_pay)
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => new_order_item.id}, :order => 'product_sku_id')
      secures.length.should == 3
      secures[0].count.should == 2
      secures[0].paid_flg.should be_false
      secures[1].count.should == 2
      secures[1].paid_flg.should be_false
      secures[2].count.should == 4
      secures[2].paid_flg.should be_false
      product_sku = ProductSku.find(new_order_item.product_sku_id)
      product_sku.stock_count.should == 25
      product_sku.orderable_count.should == 22
      # 引き当て
      new_order.secure_stocks
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => new_order_item.id}, :order => 'product_sku_id')
      secures.length.should == 3
      secures[0].count.should == 2
      secures[0].paid_flg.should be_true
      secures[1].count.should == 2
      secures[1].paid_flg.should be_true
      secures[2].count.should == 4
      secures[2].paid_flg.should be_true
      product_sku = ProductSku.find(new_order_item.product_sku_id)
      product_sku.stock_count.should == 25
      product_sku.orderable_count.should == 22
      # 開放
      new_order.unsecure_stocks
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => new_order_item.id})
      secures.length.should == 0
    end
  end

  #
  # = unsecure_stocksのテスト =
  #
  it "test_destroy_with_stock_control" do
    order = Order.find(7)

    product_sku = ProductSku.find(266)
    product_sku.stock_count.should == 50
    product_sku.orderable_count.should == 23

    order.unsecure_stocks

    product_sku = ProductSku.find(266)
    order.order_shippings[0].order_items[0].qty.should == 10
    product_sku.stock_count.should == 50
    product_sku.orderable_count.should == 43
  end

  #
  # =total_infoのテスト=
  #
  it "test total_info" do
    order = Order.find(7)
    total_info = order.total_info

    total_info[:total].should == 194040
    total_info[:subtotal].should == 194040
    total_info[:carriage].should == 0
    total_info[:count].should == 20
    total_info[:address_count].should == 2
    total_info[:due].should == 0
  end

  it "test total_info" do
    order = Order.find(7)
    order.order_shippings.each do |order_shipping|
      order_shipping.payment_type_div_key = :convenience
    end
    total_info = order.total_info

    total_info[:total].should == 194040
    total_info[:subtotal].should == 194040
    total_info[:carriage].should == 0
    total_info[:count].should == 20
    total_info[:address_count].should == 2
    total_info[:due].should == 194040
  end

  describe "create_new_order" do
    before do
      @product_sku = ProductSku.find(265)
      # course_div = 0
      course_div_key = :normal
      key = Cart::Item.key_for(@product_sku.sku_code, course_div_key)
      @cart = Cart.new()
      @cart.add(@product_sku, course_div_key, 3)

      @user = User.find(1002)
      @user_address = User.find(1003).user_address
      @order_form = OrderForm.new(
        :normal_payment_type => OrderShipping.payment_type_div_id_by_key(:cod),
        :gift_flg => true,
        :send_invoice_separately_flg => true,
        :write_sender_flg => false,
        :free_request => 'test free request'
      )
    end
    
    it "キャンペーンが適用されている場合" do
      @cart.stub!(:applied_campaigns).and_return([Campaign.find(2)])
      cart_item = @cart.item_list.first.dup
      cart_item.stub!(:applied_campaigns).and_return([Campaign.find(3)])
      @cart.stub!(:item_list).and_return([cart_item])

      order = Order.create_new_order(@user, @user_address, @order_form, @cart)
      
      order.gift_flg.should == true
      order.send_invoice_separately_flg.should == true
      order.write_sender_flg.should == false
      order.ordered_at.should_not be_nil
      order.site_type_div_key.should == :pc
      order.free_request.should == 'test free request'
      order.locale.should == 'ja'
      order.order_shippings.length.should == 1

      order_shipping = order.order_shippings.first

      order_shipping.name_last.should == @user_address.name_last
      order_shipping.name_first.should == @user_address.name_first
      order_shipping.kana_name_last.should == @user_address.kana_name_last
      order_shipping.kana_name_first.should == @user_address.kana_name_first
      order_shipping.tel_num.should == @user_address.tel_num
      order_shipping.zip_code.should == @user_address.zip_code
      order_shipping.country_code.should == @user_address.country_code
      order_shipping.address_1.should == @user_address.address_1
      order_shipping.address_2.should == @user_address.address_2
      order_shipping.address_3.should == @user_address.address_3
      order_shipping.email.should == @user.user_email.email
      order_shipping.status_div_key.should == :before_settlement

      order.order_campaigns.length.should == 1
      order.order_campaigns.first.campaign_code.should == 'T0001A'

      order_item = order_shipping.order_items.first
      order_item.order_item_campaigns.length.should == 1
      order_item.order_item_campaigns.first.campaign_code.should == 'T0002A'
    end

    it "order_shippingsとorder_itemsをセットしたorderインスタンスを作成" do
      @cart.calculate
      order = Order.create_new_order(@user, @user_address, @order_form, @cart)

      order.gift_flg.should == true
      order.send_invoice_separately_flg.should == true
      order.write_sender_flg.should == false
      order.ordered_at.should_not be_nil
      order.site_type_div_key.should == :pc
      order.free_request.should == 'test free request'
      order.locale.should == 'ja'
      order.order_shippings.length.should == 1

      order_shipping = order.order_shippings.first
      order_shipping.total_amount.should == 24570
      order_shipping.shipping_cost.should == 400
      order_shipping.name_last.should == @user_address.name_last
      order_shipping.name_first.should == @user_address.name_first
      order_shipping.kana_name_last.should == @user_address.kana_name_last
      order_shipping.kana_name_first.should == @user_address.kana_name_first
      order_shipping.tel_num.should == @user_address.tel_num
      order_shipping.zip_code.should == @user_address.zip_code
      order_shipping.country_code.should == @user_address.country_code
      order_shipping.address_1.should == @user_address.address_1
      order_shipping.address_2.should == @user_address.address_2
      order_shipping.address_3.should == @user_address.address_3
      order_shipping.email.should == @user.user_email.email
      order_shipping.status_div_key.should == :before_settlement
      order_shipping.cod_charge.should == 300
    end

    it "order_shippingsとorder_itemsをセットしたorderインスタンスを作成（site_type_divがモバイルの場合）" do
      # PC用メールアドレスをモバイル用メールアドレスとして保存する。
      @user = User.find(1002)
      @user.user_email.pc_mobile_div_key = :mobile
      @user.user_email.save!
      # 取得しなおす。
      @user = User.find(1002)

      @cart.calculate
      order = Order.create_new_order(@user, @user_address, @order_form, @cart, :mobile)

      order.gift_flg.should == true
      order.send_invoice_separately_flg.should == true
      order.write_sender_flg.should == false
      order.ordered_at.should_not be_nil
      order.site_type_div_key.should == :mobile
      order.free_request.should == 'test free request'
      order.locale.should == 'ja'
      order.order_shippings.length.should == 1

      order_shipping = order.order_shippings.first
      order_shipping.total_amount.should == 24570
      order_shipping.shipping_cost.should == 400
      order_shipping.name_last.should == @user_address.name_last
      order_shipping.name_first.should == @user_address.name_first
      order_shipping.kana_name_last.should == @user_address.kana_name_last
      order_shipping.kana_name_first.should == @user_address.kana_name_first
      order_shipping.tel_num.should == @user_address.tel_num
      order_shipping.zip_code.should == @user_address.zip_code
      order_shipping.country_code.should == @user_address.country_code
      order_shipping.address_1.should == @user_address.address_1
      order_shipping.address_2.should == @user_address.address_2
      order_shipping.address_3.should == @user_address.address_3
      order_shipping.email.should == @user.user_email_mobile.email
      order_shipping.status_div_key.should == :before_settlement
      order_shipping.cod_charge.should == 300
    end

    it "最小限の引数" do
      product_sku = ProductSku.find(:first)
      key = Cart::Item.key_for(product_sku.sku_code, product_sku.product.course_div_key)
  
      user = User.find(:first)
      user_address = user.user_addresses.first
      payment_type = OrderShipping.payment_type_div_id_by_key(:cod)
  
      new_order_shipping = Order.create_new_order_shipping(@cart, user, user_address, OrderForm.new(:normal_payment_type => "1"))
  
      new_order_shipping.name_last.should == user_address.name_last
      new_order_shipping.name_first.should == user_address.name_first
      new_order_shipping.order_items.first.product_sku.sku_code.should == product_sku.sku_code
      new_order_shipping.order_items.first.qty.should == 3
      new_order_shipping.status_div_key.should == :before_settlement
      new_order_shipping.order_items.first.order_item_components.length.should == 0
    end

    it "最小限の引数（site_type_divがモバイルの場合）" do
      product_sku = ProductSku.find(:first)
      key = Cart::Item.key_for(product_sku.sku_code, product_sku.product.course_div_key)

      user = @user
      user.user_email.pc_mobile_div_key = :mobile
      user.user_email.save!
      user_address = user.user_addresses.first
      payment_type = OrderShipping.payment_type_div_id_by_key(:cod)

      new_order_shipping = Order.create_new_order_shipping(@cart, user, user_address, OrderForm.new(:normal_payment_type => "1"), :mobile)

      new_order_shipping.name_last.should == user_address.name_last
      new_order_shipping.name_first.should == user_address.name_first
      new_order_shipping.order_items.first.product_sku.sku_code.should == product_sku.sku_code
      new_order_shipping.order_items.first.qty.should == 3
      new_order_shipping.status_div_key.should == :before_settlement
      new_order_shipping.order_items.first.order_item_components.length.should == 0
    end

    it "最小限の引数（セット商品の場合）" do
      product_sku = ProductSku.find(282)
      course_div_key = :normal
      cart = Cart.new()
      cart.add(product_sku, course_div_key, 3)

      user = User.find(:first)
      user_address = user.user_addresses.first

      new_order_shipping = Order.create_new_order_shipping(cart, user, user_address, OrderForm.new(:normal_payment_type => "1"))

      new_order_shipping.name_last.should == user_address.name_last
      new_order_shipping.name_first.should == user_address.name_first
      new_order_shipping.order_items.length.should == 1
      new_order_shipping.order_items.first.product_sku.sku_code.should == product_sku.sku_code
      new_order_shipping.order_items.first.qty.should == 3
      new_order_shipping.order_items.first.product_name.should == "商品1の商品名1"
      new_order_shipping.order_items.first.order_product_name_1.should == "商品1の注文商品名1 (バリエーションなし)"
      new_order_shipping.order_items.first.order_product_name_2.should == "Product1"
      new_order_shipping.status_div_key.should == :before_settlement

      order_item_components = new_order_shipping.order_items.first.order_item_components
      order_item_components.length.should == 2
      order_item_components[0].component_sku_id.should == 288
      order_item_components[0].qty.should == 3
      order_item_components[0].product_name.should == "商品7の商品名1"
      order_item_components[0].order_product_name_1.should == "商品7の商品名1 (バリエーションなし)"
      order_item_components[0].order_product_name_2.should == "Product7"
      order_item_components[1].component_sku_id.should == 289
      order_item_components[1].qty.should == 6
      order_item_components[1].product_name.should == "商品8の商品名1"
      order_item_components[1].order_product_name_1.should == "商品8の商品名1 (バリエーションなし)"
      order_item_components[1].order_product_name_2.should == "Product8"
    end
  end

  describe "test?" do
    it "テスト注文" do
      Order.new(:test_flg => true).test?.should be_true
    end

    it "本番注文" do
      Order.new().test?.should be_false
      Order.new(:test_flg => "").test?.should be_false
      Order.new(:test_flg => nil).test?.should be_false
      Order.new(:test_flg => false).test?.should be_false
    end
  end

  describe 'set_test_flg' do
    it "ec_adminの場合　本番データ" do
      order = Order.new
      order.set_test_flg
      order.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      order = Order.new(:test_flg => true)
      order.set_test_flg
      order.test_flg.should be_true
    end

    it "ec_frontの場合" do
      order = Order.new
      order.set_test_flg
      order.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      order = Order.new
      order.set_test_flg
      order.test_flg.should be_true
    end

    it "ec_testの場合　更新" do
      EcRider.config.stub!(:ec_test).and_return(true)
      order = Order.new(:test_flg => true)
      order.set_test_flg
      order.test_flg.should be_true
    end
  end


  describe "include_shpping_parent" do
    it "iinclude_shpping_parent true" do
      order = Order.find(2)
      order.include_order_id = 9
      order.save!
      Order.find(9).includable_shipping_parent?.should be_true
    end

    it "iinclude_shpping_parent true" do
      order = Order.find(9)
      order.should_not == be_nil
      order.includable_shipping_parent?.should_not be_true
    end
  end

  describe "create user point" do
    it "create user point true" do
      point_setting = PointSetting.find(:first, :order => 'id desc')
      point_setting.available_flg_true?.should be_true
      order = Order.find(2)
      order.create_user_point(100)
      order.user_points.should be_true
      order.user_points.last.given_point.should == 100
    end

    it "create user point false" do
      point_setting = PointSetting.find(:first, :order => 'id desc')
      point_setting.available_flg_true?.should be_true
      point_setting.available_flg = false
      point_setting.save!
      point_setting.available_flg_true?.should be_false
      order = Order.find(3)
      order.create_user_point(100)
      order.user_points.should be_blank
    end
  end

  describe "count_at_term" do
    it "一定期間の注文を集計する(ヒット0件）" do
      assert = Order.count_at_term(Time.parse('2007-12-10 11:00:00'), Time.parse('2007-12-10 11:30:00'))
      assert[:count].should == 0
      assert[:amount].should == 0
    end
    
    it "一定期間の注文を集計する" do
      OrderShipping.update_all(:status_div => OrderShipping.status_div_id_by_key(:delivered))
      assert = Order.count_at_term(Time.parse('2007-12-10 11:00:00'), Time.parse('2007-12-10 11:30:00'))
      assert[:count].should == 5
      assert[:amount].should == 75810
    end
  end
  
  describe "payment_order_id" do
    it "どちらでもない場合" do
      order = Order.find(8)
      order.payment_order_id.should be_nil
    end
  end

  describe "applied_campaigns" do
    it "get campaigns" do
      OrderCampaign.destroy_all
      OrderItemCampaign.destroy_all

      OrderCampaign.new(:order_id => 1, :campaign_code => 'c001').save
      OrderCampaign.new(:order_id => 2, :campaign_code => 'c002').save
      OrderCampaign.new(:order_id => 2, :campaign_code => 'c003').save

      OrderItemCampaign.new(:order_item_id => 1, :campaign_code => 'c004').save
      OrderItemCampaign.new(:order_item_id => 2, :campaign_code => 'c005').save
      OrderItemCampaign.new(:order_item_id => 3, :campaign_code => 'c006').save
      OrderItemCampaign.new(:order_item_id => 3, :campaign_code => 'c007').save

      order = Order.find(2)
      campaigns = order.applied_campaigns
      campaigns.length.should == 5

      campaign_ids = campaigns.map{|campaign| campaign.campaign_code }
      campaign_ids[0].should == 'c005'
      campaign_ids[1].should == 'c006'
      campaign_ids[2].should == 'c007'
      campaign_ids[3].should == 'c002'
      campaign_ids[4].should == 'c003'
    end
  end

  describe "campaign_attached_at_or_ordered_at" do
    it "get ordered_at" do
      order = Order.new(:ordered_at => '2010-01-01 10:00:00')
      order.campaign_attached_at_or_ordered_at.strftime('%Y-%m-%d %H:%M:%S').should == '2010-01-01 10:00:00'
    end

    it "get campaign_attached_at" do
      order = Order.new(:ordered_at => '2010-01-01 10:00:00', :campaign_attached_at => '2010-02-02 12:00:00')
      order.campaign_attached_at_or_ordered_at.strftime('%Y-%m-%d %H:%M:%S').should == '2010-02-02 12:00:00'
    end
  end

  describe "update_campaigns" do
    it "注文に紐付くキャンペーンを更新する" do
      product_sku = ProductSku.find(265)
      cart = Cart.new()
      cart.add(product_sku, :normal, 3)

      cart.stub!(:applied_campaigns).and_return([Campaign.find(2)])
      cart_item = cart.item_list.first.dup
      cart_item.stub!(:applied_campaigns).and_return([Campaign.find(3)])
      cart.stub!(:item_list).and_return([cart_item])

      order = Order.find(1)
      order_shipping = order.order_shippings.first
      order_item = order_shipping.order_items.first

      order.order_campaigns.size.should == 0
      order_item.order_item_campaigns.size.should == 0

      order.update_campaigns(cart)

      order = Order.find(1)
      order_shipping = order.order_shippings.first
      order_item = order_shipping.order_items.first

      order.order_campaigns.size.should == 1
      order.order_campaigns.first.campaign_id.should == 2
      order_item.order_item_campaigns.size.should == 1
      order_item.order_item_campaigns.first.campaign_id.should == 3
    end
  end
end
