require 'yaml'
require 'rails_generator/rename_commands'
class RenameModelGenerator < Rails::Generator::NamedBase
  
  def initialize(runtime_args, runtime_options = {})
    if runtime_args.any?{|arg|/^(\-g|\-\-generate\-config)/ =~ arg}
      runtime_args.unshift(DUMMY_MODEL_NAME)
    end
    if runtime_args == ['-g']
      runtime_args.unshift(DUMMY_MODEL_NAME)
    end
    if runtime_args.any?{|arg|/^(\-m|\-\-mappings\=)/ =~ arg}
      runtime_args.unshift(DUMMY_MODEL_NAME)
    end
    super
  end

  DUMMY_MODEL_NAME = 'DummyModelName'

  def add_options!(opt)
    opt.on('-S', '--show-each-diff') {|value| options[:diff] = true}
    opt.on('-r', '--show-replacements') {|value| options[:show_replacements] = true}
    opt.on('-d', '--using-diff') {|value| options[:using_diff] = true}
    opt.on('-e', '--encoding=utf8') {|value| options[:encoding] = value}
    opt.on('-g', '--generate-config') {|value| options[:generate_config] = true}
    opt.on('-C', "--config-path=#{DEFAULT_CONFIG_NAME}") {|value| options[:config_path] = value}
    opt.on('-m', '--mappings=#{CSV_FILEPATH}', 
      "CSV's line must be (old_table).(old_column),(new_table).(new_column)"){|value| options[:mappings_csv] = value}
  end

  def manifest
    record do |m|
      config_path = File.join(RAILS_ROOT, options[:config_path] || DEFAULT_CONFIG_NAME)
      if options[:generate_config]
        m.directory 'config'
        m.template 'config_rename.rb', 'config/rename.rb', :assigns => {:klass => RenameModelGenerator}
      else
        load_config(config_path)
        options[:encoding] = nil if options[:encoding] == 'none'
        diff_enabled = (`diff`; true) rescue false
        if options[:using_diff].blank? and diff_enabled
          puts "you can use --using-diff option"
        end
        if options[:using_diff] and !diff_enabled
          raise "you can't use --using-diff option"
        end
        if RUBY_PLATFORM =~ /(:?mswin|mingw)/
          if options[:encoding].nil?
            options[:encoding] = 'sjis' 
            puts "(only for windows) automatically set --encoding=sjis. If you don't like that, set --encoding=none or something"
          end
        end
      
        @duplications = {}
        model_names_hash_array = []
        unless class_name == DUMMY_MODEL_NAME
          old_names = assigned_model_names
          assign_names!(args[0])
          new_names = assigned_model_names
          puts "old_names:\n" << old_names.inspect
          puts "new_names:\n" << new_names.inspect
          model_names_hash_array << {:old => old_names, :new => new_names}
        end
        
        table_mappings, column_mappings = load_replacements_from_mappings
        table_mappings.each do |old_table, new_table|
          assign_names!(old_table.classify)
          old_names = assigned_model_names
          assign_names!(new_table.classify)
          new_names = assigned_model_names
          puts "old_names:\n" << old_names.inspect
          puts "new_names:\n" << new_names.inspect
          model_names_hash_array << {:old => old_names, :new => new_names}
        end
        
        replacements = {}
        model_names_hash_array.each do |names_hash|
          old_names = names_hash[:old]
          new_names = names_hash[:new]
          options[:replace_keywords].each do |keyword|
            store_with_check_duplication(replacements, old_names[keyword], new_names[keyword])
          end
        end
        
        # column_names_hash_array = []
        column_mappings.each do |old_column, new_column|
          old_column_names = assigned_column_names(old_column)
          new_column_names = assigned_column_names(new_column)
          # column_names_hash_array << {:old => old_column_names, :new => new_column_names}
          old_column_names.keys.each do |keyword|
            store_with_check_duplication(replacements, old_column_names[keyword], new_column_names[keyword])
          end
        end
        
        
        
        if options[:show_replacements]
          replacements.extend(SortedKeyHash)
          replacements.each do |k, v|
            puts("#{k} => #{v}")
          end
        end
        
        @duplications.each do |duplication, value|
          puts "WARNING: DUPLICATION IGNORED: #{duplication.inspect} => #{value.inspect}"
        end
        
        
        if replacements.empty?
          raise ArgumentError, "no replacement"
        end
        
        target_files = Dir.glob(options[:file_patterns])
        target_files.each do |target_file|
          m.replace(target_file, replacements, options)
        end
  
        options[:mv_path_keywords].each do |mv_path_key|
          model_names_hash_array.each do |names_hash|
            old_names = names_hash[:old]
            new_names = names_hash[:new]
            old_file_path = old_names[mv_path_key]
            new_file_path = new_names[mv_path_key]
            unless File.exist?(old_file_path)
              puts "FILE: #{old_file_path} doesn't exist"
              next 
            end
            m.mv(old_file_path, new_file_path,
              options.merge(:source_required => PATHS_REQUIRED.include?(mv_path_key)))
          end
        end
      end
    end
  rescue
    $!
    $!.backtrace.join("\n  ")
    raise
  end

  def load_replacements_from_mappings
    path = options[:mappings_csv]
    return {}, {} if path.blank?
    path = File.join(RAILS_ROOT, path)
    raise ArgumentError, "No file exists: #{path}" unless File.exist?(path)
    table_mappings, column_mappings = {}, {}
    File.open(path) do |f|
      lineno = 0
      f.each do |line|
        lineno += 1
        old_entry, new_entry = *line.split(',', 2)
        old_table, old_column = *old_entry.split('.', 2)
        new_table, new_column = *new_entry.strip.split('.', 2)
        store_with_check_duplication(table_mappings, old_table, new_table){|msg|"\##{lineno} TABLE: #{msg}"}
        store_with_check_duplication(column_mappings, old_column, new_column){|msg|"\##{lineno} COLUMN: #{msg}"}
      end
    end
    return table_mappings, column_mappings
  end

  DEFAULT_IGNORANCES = %w(id name key def class session request response module do begin end try rescue ensure)

  def store_with_check_duplication(hash, search, replace, &block)
    return if search.blank?
    return if replace.blank?
    return if search == replace
    old_value = hash[search]
    if DEFAULT_IGNORANCES.include?(search) # or DEFAULT_IGNORANCES.include?(replace)
      msg = "INFO: ignored string: #{replace.inspect} for #{search.inspect}"
      msg = yield(msg) if block_given?
      puts msg
      return 
    end
    if !old_value.blank? and old_value != replace
      msg = "WARNING!!: duplicate value associated. #{old_value.inspect} and #{replace.inspect} for #{search.inspect} were ignored."
      msg = yield(msg) if block_given?
      puts msg
      hash.delete(search)
      @duplications[search] = @duplications[search] || {}
      @duplications[search] = [old_value, replace]
    else
      hash[search] = replace
    end
  end
  
  module SortedKeyHash
    def each(&block)
      sorted_keys = self.keys.sort rescue self.keys
      sorted_keys.each do |k|
        yield(k, self[k])
      end
    end
  end

  ASSIGNED_NAME_ATTRS = [:class_path, :class_name, :class_name_without_nesting,
    :class_nesting, :class_nesting_depth,
    :file_path, :singular_name, :plural_name, :table_name]
  
  def assigned_model_names
    result = ASSIGNED_NAME_ATTRS.inject({}) do |dest, attr|
      dest[attr] = instance_variable_get("@#{attr.to_s}")
      dest
    end
    result.update(
      :model_path => "app/models/#{result[:file_path]}.rb",
      :unit_test_path => "test/unit/#{result[:file_path]}_test.rb",
      :spec_model_path => "spec/models/#{result[:file_path]}_spec.rb",
      :fixture_yml_path => "test/fixtures/#{result[:table_name]}.yml",
      :fixture_yaml_path => "test/fixtures/#{result[:table_name]}.yaml",
      :fixture_csv_path => "test/fixtures/#{result[:table_name]}.csv",
      :foreign_key_name => "#{result[:singular_name]}_id",
      :unit_test_name => "#{result[:class_name]}Test"
    )
    if RenameModelGenerator.respond_to?(:update_assigned_names)
      RenameModelGenerator.update_assigned_names(result)
    end
    result
  end
  
  def assigned_column_names(column_name)
    result = {:column_name => column_name}
    if RenameModelGenerator.respond_to?(:update_column_names)
      RenameModelGenerator.update_column_names(result)
    end
    result
  end
  
  
  def self.config
    yield(@options_for_config)
  end
  
  DEFAULT_CONFIG_NAME = 'config/rename.rb'
  CSV_FILEPATH = "path/to/csv_file"
  
  def load_config(config_path)
    if File.exist?(config_path)
      RenameModelGenerator.instance_variable_set("@options_for_config", options)
      begin
        load(config_path)
      ensure
        RenameModelGenerator.instance_variable_set("@options_for_config", nil)
      end
    end
    options[:file_patterns] ||= DEFAULT_FILE_PATTERNS.dup
    options[:mv_path_keywords] ||= MV_PATH_KEYWORDS.dup
    options[:replace_keywords] ||= DEFAULT_REPLACE_KEYWORDS.dup
  end
  
  DEFAULT_REPLACE_KEYWORDS = [
        :class_name,
        :foreign_key_name, 
        :unit_test_name,
        :table_name,
        # :file_path, 
      ]
  
  DEFAULT_FILE_PATTERNS = [
    'app/**/*.{rb,rhtml,erb}', 
    'config/**/*.{rb,rhtml,erb}', 
    'lib/**/*.{rb,rhtml,erb}', 
    'spec/**/*.rb', 
    'spec/fixtures/**/*.{yml,yaml}', 
    'test/**/*.rb', 
    'test/fixtures/**/*.{yml,yaml}']
  
  PATHS_REQUIRED = [:model_path]
  MV_PATH_KEYWORDS = PATHS_REQUIRED + [
    :unit_test_path, 
    :spec_model_path,
    :fixture_yml_path, 
    :fixture_yaml_path, 
    :fixture_csv_path]


  protected
    # Override with your own usage banner.
    def banner
      result = []
      result << "Usage: #{$0} model OldModelName NewModelName [options]"
      result << "  rename file"
      result << ("    " << MV_PATH_KEYWORDS.map{|key|key.to_s}.join("\n    "))
      result << ""
      result << "  replace files"
      result << ("    " << DEFAULT_FILE_PATTERNS.map{|pattern|pattern.to_s}.join("\n    "))
      result << ""
      result << ""
      result.join("\n")
    end
end
