# -*- coding: utf-8 -*-

require 'ec_rider/erb_util_ext'

# require 'gettext/rails' # Ruby-GetText-Packageを使う宣言です。
class ApplicationController < ActionController::Base
  include EcRider::PluginModule::Includable

  # until rails-2.2.2
  # See ActionController::RequestForgeryProtection for details
  # Uncomment the :secret if you're not using the cookie session store
  # protect_from_forgery :secret => '198374ca2e959b18fd137bcb32742490fedc6bb81062e522846cdf8811e70be27b57293d671352bea98fca78671341a60d5704088ad13d079a6dfdf69ea5cfe3'

  # since rails-2.3.2
  protect_from_forgery # See ActionController::RequestForgeryProtection for details

  # Pick a unique cookie name to distinguish our session data from others'

  # verify_form_posts_have_security_token

  include SslRequirement

  # エラーメッセージなどのi18nサポート
  include EcRider::I18nSupport

  # 障害発生時のメール通知
  include ExceptionNotifiable

  # ユーザー情報の有無を確認
  before_filter :clear_session_user_info

  # リファラーの情報を管理する
  before_filter :check_session_forward_url

  # ビューキャッシュのクリアが必要かどうかをチェック
  before_filter :published_check
  
  # Be sure to include AuthenticationSystem in Application Controller instead
  include AuthenticatedSystem

  after_filter :set_content_language

  protected

  def set_content_language
    response.headers['Content-Language'] = I18n.locale.to_s
  end

  # ビューキャッシュのクリアが必要かどうかチェックし、
  # 必要な場合はキャッシュをクリアします。
  def published_check
    if RAILS_ENV == 'production' && EcRider.config.enable_passenger
      EcRider::ViewCacheCtrl.published_check
    end
  end

  # # If you want "remember me" functionality, add this before_filter to Application Controller
  # before_filter :login_from_cookie, :create_security_token
  # def create_security_token
  #   @session_id_validation = security_token
  # end


  # override SslRequirement module's method
  def ssl_required?(logging = true)
    result = ssl_required_controller?(params[:controller])
    if behave_as_production?
      if EcRider.config.ssl_ignored
        logger.info("ssl_required? return #{result} but ignored cause of EcRider.config.ssl_ignored == true") if logging
        return false
      end
      return result
    else
      logger.debug("ssl_required? return #{result} but ignored cause of *#{RAILS_ENV}* mode") if logging
      return false
    end
  end

  # override SslRequirement module's method
  def ssl_allowed?(logging = true)
    result = ssl_acceptable_controller?(params[:controller])
  end

  # テストで一時的に(stub!などで)productionモードとして扱えるように別メソッドにします。
  def behave_as_production?
    RAILS_ENV == "production"
  end

  private
  def ssl_required_controller?(controller)
    logged_in? ? true : EcRider::SslConfig.ssl_required_controllers.include?(controller)
  end

  def ssl_acceptable_controller?(controller)
    EcRider::SslConfig.ssl_acceptable_controllers.include?(controller)
  end

  # SSLに関する仕様
	# A ログイン時
	#	 1. 常にSSLを要求。httpでのアクセスはhttpsへリダイレクト
	# B 未ログイン時
	#	 1. SSLが要求されるコントローラへのアクセスをhttpで行った場合、httpsへリダイレクト。
	#	 2. SSLが要求されるコントローラへのアクセスをhttpsで行った場合、そのまま処理
	#	 3. SSLが要求されないコントローラへのアクセスをhttpで行った場合、そのまま処理を行う
	#	 4. SSLが要求されないコントローラへのアクセスをhttpsで行った場合
	#		a. コントローラがSSLを受け付ける場合は そのまま処理を行う
	#		b. コントローラがSSLを受け付けない場合は httpへリダイレクト
  def ensure_proper_protocol
    if ssl_required? && !request.ssl?
      reset_session_with_keep_content
      redirect_to "https://" + request.host + request.request_uri
      return false
    elsif !logged_in? && request.ssl? && !ssl_required?
      if ssl_allowed?
        return true
      else
        cart = session[:cart] # カートだけはhttpsからhttpへの遷移でも保持します。
        reset_session # HTTPSからHTTPに戻る際には、(脆弱性になる恐れがあるので)セッションの中身は引き継ぎません。
        session[:cart] = cart
        redirect_to "http://" + request.host + request.request_uri
        return false
      end
    end
  end

  def reset_session_with_keep_content
    # セッションの保持しているデータを一つ一つHashに取っておいてから、reset_sessionを行い、
    # その後に新しく作られたsessionに、取っておいた値を設定します。
    kept_values = {}
    session.to_hash.each do |key, value|
      kept_values[key] = value
    end
		reset_session # セッションを作り直し
		kept_values.each do |key, value|
			session[key] = value
		end
  end


  private

  # public/cms以下に店舗担当者の登録した404.htmlがあったらそれを、なかったらpublic/404.htmlを使います。
  def render_404
    respond_to do |type|
      type.html { render :file => error_file_path("404.html"), :status => "404 Not Found" }
      type.all  { render :nothing => true, :status => "404 Not Found" }
    end
  end

  # public/cms以下に店舗担当者の登録した500.htmlがあったらそれを、なかったらpublic/500.htmlを使います。
  def render_500
    respond_to do |type|
      type.html { render :file => error_file_path("500.html"), :status => "500 Error" }
      type.all  { render :nothing => true, :status => "500 Error" }
    end
  end

  def error_file_path(filename)
    cms_path = "#{RAILS_ROOT}/public/cms/#{filename}"
    File.exist?(cms_path) ? cms_path : "#{RAILS_ROOT}/public/#{filename}"
  end

  #
  # バリデーションエラーメッセージ
  #
  def render_error_messages_for(*objects)
    count = objects.inject(0){|sum, object| sum + object.errors.count}
    unless count.zero?
      error_messages = objects.map {|object| object.errors.full_messages.map {|msg| ApplicationController.helpers.content_tag(:li, ERB::Util.html_escape(msg)) }}
      render(:text => ApplicationController.helpers.content_tag(:div, ApplicationController.helpers.content_tag(:ul, error_messages), :class=> 'alert_msg'))
    end
  end

  protected

  # http://shop.ecrider.com/ja でアクセスしている場合は、config/routes.rbの設定に従って
  # params[:lang]には'ja'が指定されるはずなので、その画面から作成されるURLには基本的に
  # :lang => 'ja' を付加したいです。
  #
  def default_url_options(options)
    if params && (lang = params[:lang])
      lang = lang.first if lang.is_a?(Array)
      params[:lang].nil? ? nil : {:lang => lang}
    end
  end

  def authorize
    redirect_to_root unless logged_in?
  end

  def back_url(default_url, &ignore)
    ignore.call(request.referer) || default_url if ignore
  end

  #
  # カート事前処理
  # カートにデータが存在しない場合は、その旨のメッセージを表示する画面へリダイレクトする。
  #
  # TODO: 名前変更 redirect_to_no_input_if_empty_cart
  # xxx? メソッドは、if/unlessで処理されることを
  # 期待する結果(false/true, nil/非nil)を返さなければならない！！！！！！！！！！
  #
  def cart_in?
    if session[:cart] == nil
      redirect_to :controller=>'cart', :action => 'cart_no_input' unless performed?
      return
    end
    if session[:cart].empty?
      redirect_to :controller=>'cart', :action => 'cart_no_input' unless performed?
      return
    end
  end

  #
  # 決済に必要なユーザー情報の削除
  # 戻るボタンなどによる、弊害を事前に防ぐために、決済系の画面以外へのアクセス時は、決済のユーザー情報をセッションから削除する
  #
  # TODO: 特定のコントローラに関わる処理を app/controllers/application.rbから追い出す。
  #
  def clear_session_user_info
    controller_name = request.path_parameters[:controller] || self.controller_name
    if controller_name
      unless controller_name.index('order') || controller_name.index('sps_payment') || 
        controller_name.index('zip') || controller_name.index('common')
        session[:cart_tmp_user] = nil
      end
    end
  end

  #
  # = 会員登録処理・ログイン・リマインダーの遷移管理 =
  #
  # 画面遷移を正常に保つために、遷移先のコントローラがaccount又はreminderの以外の場合、
  # 会員登録など処理後の転送先URLを保存してあるセッションを削除する。
  # 上記の条件外で、遷移元のコントローラがaccount又はreminderの場合、
  # 会員登録などへ遷移する際のリファラーをセッションへ保存する。
  #
  def check_session_forward_url
    controller_name = request.path_parameters[:controller] || self.controller_name
    if controller_name &&( controller_name.index('account') || controller_name.index('reminder'))
      forward_url = request.referer
      unless forward_url.nil?
        unless forward_url.index('account') || forward_url.index('reminder')
          session[:forward_url] = forward_url
        end
      end
    else
      session[:forward_url] = nil
    end
  end

  protected

  # Cookieの値からlangを設定
  def before_init_i18n
    lang = cookies["lang"]
    if lang.blank?
      lang = (request.headers['HTTP_ACCEPT_LANGUAGE'] || '').split(/,/).first
      lang = 'ja' if lang.blank?
    end
    params["lang"] ||= lang
  end
  before_init_locale :before_init_i18n

  #
  # Ajaxリクエストではない場合、白紙のレスポンスを返します。
  # 
  def is_ajax_request
    return unless request.xml_http_request?
  end

  def redirect_to_root
    redirect_to(:controller => 'cms', :action => ShopSetting.get_setting(:top_action, I18n.locale))
  end

  def redirect_to_404
    redirect_to(:controller => 'cms', :action => 'error_404')
  end

  def redirect_to_404_unless_performed
    redirect_to_404 unless performed?
    nil
  end

  #
  # = MyPage領収書事前処理 =
  #
  # ec_admin及びec_frontで画面を共用しているため、
  # ec_admin側からの遷移かを確認する。
  # admin側からの遷移の場合、認証確認はしない。
  #
  def check_go_to_front
    return if request.referer =~ /#{EcRider.config.ec_admin_url}/
    return if params[:from_admin]
    return authorize
  end

end


# ERB::Utilの拡張
unless ERB::Util.respond_to?(:html_escape_with_ec_rider) 
	ERB::Util.extend(EcRider::ErbUtilExt) 
end 
 
# ERB::Utilの拡張
unless ActionView::Base.ancestors.include?(EcRider::ErbUtilExt) 
	ActionView::Base.module_eval do	 
		include(EcRider::ErbUtilExt) 
	end 
end 

# ActiveRecord::Baseの拡張
unless ActiveRecord::Base.ancestors.include?(SafeEval::ArModel)
  ActiveRecord::Base.module_eval do
    include SafeEval::ArModel
  end
end
