# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module BaseTag
      extend MethodDescription

      private
      #
      # ecタグの各メソッドに渡される引数から、その処理で使用する値の配列として返します。
      #   extract(args, 必須1, 必須２, オプション1 => "デフォルト値1", オプション2 => "デフォルト値2")
      # という呼び出しを行うと、
      #   argsの最初のオブジェクトをHashとして見なし、必須1, 必須２, オプション1, オプション2
      # という順番の配列を返します。
      #
      #   tag_args: 処理対象となるecタグの各メソッドに渡された引数
      #   names: 必須のパラメータ名の配列
      #   namesの最後のHash: デフォルト値を指定したオプショナルなパラメータ
      #
      # 例）
      #   text, product_code = *extract(args, :text, :product_code)
      #   text, product_code = *extract(args, :text, :product_code => "")
      #   href, product_code = *extract(args, [:text, :src], :product_code => "")
      #   product_code, text = *extract(args, :text => "", :product_code => "")
      #   product_code, text = *extract(args, :product_code => "", :text => "")
      #
      # 注意）オプショナルなパラメータ名はソートされるので、戻り値を代入する際は気をつけてください。
      #
      #     text, name, product_code, disabled = *extract(args, :text, :name, :product_code => "00001", :disabled => "false")
      #
      #   argsに product_code と disabled が含まれていなかった場合は
      #     product_code =>  "false"
      #     disabled => "00001"
      #   となってしまいます。オプショナルなパラメータについてはソートされることを意識して、以下のように記述してください。
      #
      #     text, name, disabled, product_code = *extract(args, :text, :name, :product_code => "00001", :disabled => "false")
      #
      #
      #
      def extract(actual_args, *names)
        actual_options = actual_args.shift || {} # actual_argsは 空の配列か、Hashを一つだけ持つ配列
        actual_options.stringify_keys!
        optional_defs = names.extract_options!
        optional_defs.stringify_keys!
        required_values = names.map do |required|
          (!required.is_a?(Array) ? actual_options[required.to_s] :
            # namesで指定されているキーに該当するactual_optionsの値は全てdeleteする
            (required.map{|nm| actual_options.delete(nm.to_s)}.compact.first)) ||
            raise(ArgumentError, "必須の属性#{required.is_a?(Array) ? required.first : required}が指定されていません。")
        end
        optional_values = optional_defs.keys.sort.map{|name| actual_options.delete(name) || optional_defs[name]}
        (required_values + optional_values) << actual_options
      end

      # ecタグによる繰り返しは必ず、このメソッドを使用してください。
      #
      # @orders = @template.instance_variable_get(:order_list)
      # でOderオブジェクトの配列を取得できる場合、以下のようか書き方が可能です。
      #
      # 例1
      # _each_with_index(:@orders) do
      #   要素        => @order
      #   インデックス => @order_index
      # end
      #
      # 例2
      # _each_with_index(:@orders, :@user_order) do
      #   要素        => @user_order
      #   インデックス => @user_order_index
      # end
      #
      # 例3
      # _each_with_index(:@orders, :@user_order, :@order_list_index) do
      #   要素        => @user_order
      #   インデックス => @order_list_index
      # end
      #
      #
      def _each_with_index(enumerable_name, item_name = nil, index_name = nil)
        enumerable = instance_variable_get(enumerable_name)
        raise ArgumentError, "#{enumerable_name.inspect} は繰り返しできません。" unless enumerable.respond_to?(:each_with_index)
        item_name ||= enumerable_name.to_s.singularize
        index_name ||= "#{item_name}_index"
        backup_item = instance_variable_get(item_name)
        backup_index = instance_variable_get(index_name)
        begin
          enumerable.each_with_index do |item, index|
            instance_variable_set(item_name, item)
            instance_variable_set(index_name, index)
            yield
          end
        ensure
          instance_variable_set(item_name, backup_item)
          instance_variable_set(index_name, backup_index)
        end
      end

      public

      desc <<-EOS
        [概要:現在日付を表示します。YYYY/MM/SS形式で表示します。]
        [引数:なし]
        [戻り値:YYYY/MM/SS形式の現在日付。]
        [使用例:
           {{{
           <ec:today/>
           }}}
           ⇒ 2009/03/01
        ]
      EOS
      def today(*args, &block)
        Time.now.strftime('%y/%m/%d')
      end

      desc <<-EOS
        [概要:現在日付を表示します。YYYY年MM月SS日形式で表示します。]
        [引数:なし]
        [戻り値:YYYY年MM月SS日形式の現在日付。]
        [使用例:
           {{{
           <ec:today_jpn/>
           }}}
           ⇒ 2009年03月01日
        ]
      EOS
      def today_jpn(*args, &block)
        Time.now.strftime('%y年%m月%d日')
      end

      desc <<-EOS
        [概要:現在日付の曜日を表示します。]
        [引数(format):曜日のフォーマットを変更します。format="ja"を指定すると月～日を、それ以外の場合はMonday～Sundayを表示します。]
        [戻り値:本日の曜日]
        [使用例:
           {{{
           <ec:weekday format="ja"/>曜日
           }}}
           ⇒ 月曜日
           {{{
           <ec:weekday/>
           }}}
           ⇒ Monday
        ]
      EOS
      def weekday(*args)
        format, options = extract(args, :format => nil)
        today = Time.now
        wdays = []
        if format == "ja"
          wdays = ["日", "月", "火", "水", "木", "金", "土"]
          return wdays[today.wday]
        else
          today.strftime("%A")
        end
      end

      desc <<-EOS
        [概要:指定されたvaluesの要素の分だけ繰り返しを行います。]
        [引数 values(必須):要素をカンマ(あるいは引数delimeter)で区切った文字列。]
        [引数 key(必須):繰り返しの対象となる要素。]
        [引数 delimiter(オプション):引数valuesの要素を区切る文字列。指定しなければカンマ , が使用されます。]
        [戻り値:なし]
        [使用例:
           {{{
           <ec:each values="1,2,3" key="stock_count">
             No.<ec:print key="name"/>
           </ec:each>
           }}}
           ⇒ No.1 No.2 No,3
        ]
      EOS
      def each(*args, &block)
        values, key, delimeter = *extract(args, :values, :key, :delimeter => ',' )
        values.split(delimeter).each do |v|
          @template.instance_variable_set("@#{key}", v)
          yield
        end
        nil
      end

      desc <<-EOS
        [概要:keyで指されるデータとvalueと等しければその中身を実行します。]
        [引数 key(必須):比較の対象となる要素。ProductTagのタグ名を指定できます。]
        [引数 value(必須):比較の基準となる数値または文字列。]
        [引数 product_code(任意):プロダクトコード。]
        [戻り値:なし]
        [使用例:
           {{{
           <ec:if_equal key="stock_count" value="0" product_code="001">
             売り切れ
           </ec:if_equal>
           }}}
           ⇒ プロダクトコード001の在庫数が0の場合に「売り切れ」と表示
        ]
      EOS
      def if_equal(*args, &block)
        key, value, product_code = *extract(args, :key, :value, :product_code =>"")
        target = @template.instance_variable_get("@#{key}").to_s
        target = product_info(product_code, key) if target.blank?
        if value.to_s == target.to_s
          yield
        end
      end

      desc <<-EOS
        [概要:keyで指されるデータがvalue以上ならばその中身を実行します。]
        [引数 key(必須):比較の対象となる要素。ProductTagのタグ名を指定できます。]
        [引数 value(必須):比較の基準となる数値または文字列。]
        [引数 product_code(任意):プロダクトコード。]
        [戻り値:なし]
        [使用例:
           {{{
           <ec:if_greater_equal key="stock_count" value="10" product_code="001">
             在庫あり
           </ec:if_equal><br/>
           }}}
           ⇒ プロダクトコード001の在庫数が10以上の場合に「在庫あり」と表示
        ]
      EOS
      def if_greater_equal(*args, &block)
        key, value, product_code = *extract(args, :key, :value, :product_code =>"")
        target = @template.instance_variable_get("@#{key}").to_s
        target = product_info(product_code, key) if target.blank?
        if value.to_i <= target.to_i
          yield
        end
      end

      desc <<-EOS
        [概要:keyで指されるデータがvalueより大きければその中身を実行します。]
        [引数 key(必須):比較の対象となる要素。ProductTagのタグ名を指定できます。]
        [引数 value(必須):比較の基準となる数値または文字列。]
        [引数 product_code(任意):プロダクトコード。]
        [戻り値:なし]
        [使用例:
           {{{
           <ec:if_greater_than key="stock_count" value="10" product_code="001">
             在庫あり
           </ec:if_equal>
           }}}
           ⇒ プロダクトコード001の在庫数が10より大きい場合に「在庫あり」と表示
        ]
      EOS
      def if_greater_than(*args, &block)
        key, value, product_code = *extract(args, :key, :value, :product_code =>"")
        target = @template.instance_variable_get("@#{key}").to_s
        target = product_info(product_code, key) if target.blank?
        if value.to_i < target.to_i
          yield
        end
      end

      desc <<-EOS
        [概要:keyで指されるデータがvalue以下ならばその中身を実行します。]
        [引数 key(必須):比較の対象となる要素。ProductTagのタグ名を指定できます。]
        [引数 value(必須):比較の基準となる数値または文字列。]
        [引数 product_code(任意):プロダクトコード。]
        [戻り値:なし]
        [使用例:
           {{{
           <ec:if_less_equal key="stock_count" value="5" product_code="001">
             残りわずか
           </ec:if_equal>
           }}}
           ⇒ プロダクトコード001の在庫数が5以下の場合に「残りわずか」と表示
        ]
      EOS
      def if_less_equal(*args, &block)
        key, value, product_code = *extract(args, :key, :value, :product_code =>"")
        target = @template.instance_variable_get("@#{key}").to_s
        target = product_info(product_code, key) if target.blank?
        if value.to_i >= target.to_i
          yield
        end
      end

      desc <<-EOS
        [概要:keyで指されるデータがvalueより小さければその中身を実行します。]
        [引数 key(必須):比較の対象となる要素。ProductTagのタグ名を指定できます。]
        [引数 value(必須):比較の基準となる数値または文字列。]
        [引数 product_code(任意):プロダクトコード。]
        [戻り値:なし]
        [使用例:
           {{{
           <ec:if_less_than key="stock_count" value="5" product_code="001">
             残りわずか
           </ec:if_equal><br/>
           }}}
           ⇒ プロダクトコード001の在庫数が5未満の場合に「残りわずか」と表示
        ]
      EOS
      def if_less_than(*args, &block)
        key, value, product_code = *extract(args, :key, :value, :product_code =>"")
        target = @template.instance_variable_get("@#{key}").to_s
        target = product_info(product_code, key) if target.blank?
        if value.to_i > target.to_i
          yield
        end
      end

      desc <<-EOS
        [概要:汎用的に動的なタグ出力を行います。通常ec:attrと組み合わせて使用します。]
        [引数 tag_name(準必須):出力するタグ名を指定します。]
        [引数 (オプション):HTMLオプションを指定します。]
        [使用例:
           * 商品の残り数量によってdivのクラス名を変えたい場合
           {{{
           <ec:any tag_name="div">
             <ec:attr name="class">product_count_<ec:stock_count/></ec:attr>
             商品A
           </ec:any>
           }}}
           ==>
             商品の在庫数が3ならば、<div class="product_count_3">商品A</div>
             商品の在庫数が4ならば、<div class="product_count_4">商品A</div>
        ]
      EOS
      def any(options = {}, &block)
        # ec:anyがネストした場合にec:attrが直近のec:anyの属性を操作できるようにスタックに積んでおく
        @attrs_for_any_stack ||= []
        @attrs_for_any_stack << options
        content = nil
        begin
          content = @template.capture(&block) if block_given?
        ensure
          @attrs_for_any_stack.pop
        end
        tag_name = options.delete(:tag_name)
        raise ArgumentError, "tag_name is required" unless tag_name
        if content.blank?
          @template.concat(@template.tag(tag_name, options))
        else
          @template.concat(@template.tag(tag_name, options, true))
          @template.concat(content)
          @template.concat("</#{tag_name}>")
        end
      end

      desc <<-EOS
        [概要:ec:anyタグ内で、そのec:anyタグで出力されるタグの属性を設定します。]
        [引数 name(必須):ec:anyで出力するタグの属性名を指定します。]
        [使用例:
           * 商品の残り数量によってdivのクラス名を変えたい場合
           {{{
           <ec:any tag_name="div">
             <ec:attr name="class">product_count_<ec:stock_count/></ec:attr>
             商品A
           </ec:any>
           }}}
           ==>
             商品の在庫数が3ならば、<div class="product_count_3">商品A</div>
             商品の在庫数が4ならば、<div class="product_count_4">商品A</div>

          * 注意！ 以下のように記述することはできません！
           {{{
           <div>
             <ec:attr name="class">product_count_<ec:stock_count/></ec:attr>
             商品A
           </div>
           }}}
           このような場合、ec:divタグを使用してください
           {{{
           <ec:div>
             <ec:attr name="class">product_count_<ec:stock_count/></ec:attr>
             商品A
           </ec:div>
           }}}
        ]
      EOS
      def attr(*args, &block)
        name, options = extract(args, :name)
        value = block_given? ? @template.capture(&block) : options[:value]
        add_attr_to_last_any(name, value)
      end

      desc <<-EOS
        [概要:ec:anyタグ内で、そのec:anyタグで出力されるタグの属性の値にseparatorで分割された値を設定します。]
        [引数 separator(任意):子要素の値を分割するための文字列。デフォルトはカンマ(,)]
        [使用例:
           {{{
           <ec:tr>
             <ec:attr_cycle name="class" separator=",">even,odd</ec:cycle>
             <td>AAA</td>
           </ec:tr>
           }}}
           ==>
           これを4回繰り返される場所で使用すると、以下のようになります。
           {{{
           <tr class="even">
             <td>AAA</td>
           </tr>
           <tr class="odd">
             <td>AAA</td>
           </tr>
           <tr class="even">
             <td>AAA</td>
           </tr>
           <tr class="odd">
             <td>AAA</td>
           </tr>
           }}}
        ]
      EOS
      def attr_cycle(*args, &block)
        name, separator, value, options = extract(args, :name, :separator => ',', :value => nil)
        value = block_given? ? @template.capture(&block) : options[:value]
        add_attr_to_last_any(name, @template.cycle(*value.split(separator)))
      end

      desc <<-EOS
        [概要:子要素をseparatorで分割した値を繰り返しのたびに順番に出力します]
        [引数 separator(任意):子要素の値を分割するための文字列。デフォルトはカンマ(,)]
        [使用例:
           {{{
           <p><ec:cycle separator=",">even,odd</ec:cycle></p>
           }}}
           ==>
           これを7回繰り返される場所で使用すると、以下のようになります。
           {{{
           <p>even</p>
           <p>odd</p>
           <p>even</p>
           <p>odd</p>
           <p>even</p>
           <p>odd</p>
           <p>even</p>
           }}}
        ]
      EOS
      def cycle(*args, &block)
        separator, value, options = extract(args, :separator => ',', :value => nil)
        value = block_given? ? @template.capture(&block) : options[:value]
        @template.concat(@template.cycle(*value.split(separator)))
      end

      desc <<-EOS
        [概要:条件が成り立つ場合に子要素を出力します。成り立たなければ子要素を出力しません]
        [引数 test(必須):条件として指定するスクリプトです。ただしRubyのセキュリティレベル4で実行されます]
        [使用例:
           {{{
           <ec:if test="Date.today >= Date.new(2009, 5, 5)">
             <p>Open!</p>
           </ec:if>
           }}}
           ==>
           2009/5/5以降ならば
           {{{
           Open!
           }}}
           という文字が出力されます。それより前は何も出力されません。
        ]
      EOS
      def if(*args, &block)
        test, options = extract(args, :test)
        # $SAFE = 4で動かすために、testで参照される文字列の汚染マークを取り除くことも考えましたがやめました。
        # http://www.ruby-lang.org/ja/man/html/Object.html#untaint
        # test.untaint
        result = nil
        Thread.start {
          # SAFE_LEVEL 4が最も制限の厳しい実行レベルです。
          # http://www.ruby-lang.org/ja/man/html/_A5BBA5ADA5E5A5EAA5C6A5A3A5E2A5C7A5EB.html
          $SAFE = 3 #
          result = self.instance_eval(test)
        }.join
        yield if !!result
      end

      desc <<-EOS
        [概要:bodyタグにオプションを付加します]
        [引数 (オプション):HTMLオプションを指定します。idオプションは指定できません。]
        [使用例:
           {{{
             <ec:body_attr link="#000000"/>
             ==>
             bodyタグにlinkオプションが付加されます。
           }}}
        ]
      EOS
      def body_attr(*args)
        id, options = extract(args, :id=>nil)
        body_options=""
        options.each do |key, value|
          body_options << "#{key}=\"#{value}\" "
        end
        @template.content_for(:body_options, body_options)
        return ''
      end

      private

      # attr, attr_cycleで使用されています
      def add_attr_to_last_any(name, value)
        return unless value
        attrs_for_any = @attrs_for_any_stack.nil? ? nil : @attrs_for_any_stack.last
        raise ArgumentError, "ec:attr tag must be used in ec:any tag" unless attrs_for_any
        attr_sym = name.to_sym
        case attr_sym
        when :class then
          if attrs_for_any[attr_sym].blank?
            attrs_for_any[attr_sym] = value
          else
            attrs_for_any[attr_sym] << ' ' << value
          end
        when :style then
          if attrs_for_any[attr_sym].blank?
            attrs_for_any[attr_sym] = value
          else
            attrs_for_any[attr_sym] << ';' if attrs_for_any[attr_sym] =~ /[^;]$/
            attrs_for_any[attr_sym] << ' ' << value
          end
        else
          attrs_for_any[attr_sym] = value
        end
      end

      public
      1.upto(30).each do |number|
        if number == 1
          desc <<-EOS
            [概要:テンプレート中の、バインドにする範囲をdivタグで囲んで出力します。bind01～bind30まで指定できます。]
            [引数 name(必須):バインドの説明を指定します。ページ管理で表示されます。]
            [使用例:
               {{{
               <ec:bind01 name="商品名"/>
               }}}
            ]
          EOS
        end
        method_number = number<10 ? "0#{number}" : "#{number}"
        define_method("bind#{method_number}") do
        end
      end

    end
  end
end
