# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module CartTag
      extend MethodDescription

      # == STEP1から ==
      # cart_in_button                        => cart_add_button
      # cart_link                             => cart_link
      #
      # == STEP2から ==
      # カート画面制御関係
      # cart_link_to_remider                  => cart_link_to_remider
      # continue_shopping_button              => cart_continue_shopping_button
      #
      # カートログイン関係
      # cart_login_form                       => cart_login_form
      # cart_login_error_messages             => cart_login_error_messages
      #
      # カートの表示時のエラー
      # cart_error_messages                   => cart_error_messages
      #
      # カート明細関係
      # if_empty_products                     => if_empty_cart
      # if_not_empty_products                 => if_not_empty_cart
      # cart_item_list_include                => cart_item_list_include
      # cart_item_list                        => cart_item_list
      # cart_delete_product_button            => cart_item_delete_button
      # cart_qty_select                       => cart_item_qty_select
      # --- order_tagから分離したもの
      # order_product_image                   => cart_item_image
      # order_product_name                    => cart_item_product_name
      # order_product_normal_price            => cart_item_product_sku_unit_price
      # order_product_price                   => cart_item_unit_price
      # order_product_qty                     => cart_item_qty_select
      # (該当なし)                            => cart_item_subtotal_amount
      # order_product_total_discount_price    => cart_item_discount_amount
      # order_product_sub_total_price         => cart_item_total_amount
      #
      # カート合計関係
      # cart_total_amount_include             => cart_sum_total_amount_include
      # cart_normal_products_quantity         => cart_sum_total_qty
      # cart_normal_products_subtotal_amount  => cart_sum_subtotal_amount
      # cart_normal_products_discount_amount  => cart_sum_discount_amount
      # cart_normal_products_total_price      => cart_sum_total_amount

      desc <<-EOS
        [概要:「買い物かごを見る」リンクを表示します。]
        [引数 src(任意):画像ファイルのパスまたはURLを指定します。
          ファイルマネージャでアップロードしたファイルの場合、imageフォルダ(/image)は省略できます。]
        [引数 (オプション):画像のHTMLオプションを指定します。]
        [戻り値:ショッピングカート画面へのリンク]
        [使用例:&lt;ec:cart_link src="cart.gif" width="190" height="31" alt="買い物かごを見る"/&gt;（画像リンク）<br/>
          　　⇒ &lt;a href="/cart"&gt;&lt;img height="31" width="190" src="/images/cart.gif" alt="買い物かごを見る"/&gt;&lt;/a&gt;]
      EOS
      def cart_link(*args, &block)
        options = args.shift || {}
        src = options.delete(:src)
        text = options.delete(:text) || I18n.t(:"activerecord.errors.messages.NM990013")
        if src
          link_to(@template.image_tag(src, options), :controller => 'cart', :action => 'index')
        else
          link_to(text, {:controller => 'cart', :action => 'index'}, options)
        end
      end

      desc <<-EOS
        [概要:数値選択セレクトボックスとカートに入れるボタンを表示します。]
        [引数 product_code(準必須):商品コードを指定します。&lt;ec:each_products&gt&lt;/ec:each_products&gtの中では不要です。]
        [引数 values(オプション):購入数選択セレクトボックスの選択肢をカンマ区切りで指定します。
                       指定しないなければ1～購入上限数設定値を設定します。
                       購入上限数が設定されていなければ1～99を設定します。
                       在庫数が設定されていれば在庫数以下の値を設定します。]
        [引数 class(オプション):ボタンのクラスを指定します。
                       指定しなければbutton_cart1が適用されます。]
        [引数 label(オプション):セレクトボックスの前に表示する文字列を指定します。]
        [引数 button_name(オプション):ボタンの名称を指定します。]
        [戻り値:数値選択セレクトボックスとカートに入れるボタン。]
        [使用例:
          {{{
          <ec:cart_in_button product_code="001" values="1,2,3"/> （購入数の選択肢を指定）
          }}}
        ]
      EOS
      def cart_in_button(*args, &block)
        button_name, class_name, label, product_code, values = *extract(args, :button_name => I18n.t(:"activerecord.errors.messages.NM990008"), :class => "button_cart1",  :label => "", :product_code => "", :values => "")
        amount_options = []
        orderable_count = product_sku_info(product_code, 'orderable_count').to_i
        
        # 販売期間内でない場合
        sale_started_at = product_info(product_code, 'sale_started_at')
        sale_finished_at = product_info(product_code, 'sale_finished_at')
        time_now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
        unless (sale_started_at.nil? || sale_started_at.blank? || sale_started_at <= time_now) &&
            (sale_finished_at.nil? || sale_finished_at.blank? || sale_finished_at >= time_now)
          return "<div>#{product_info(product_code, 'product_name_1')}#{I18n.t(:"activerecord.errors.messages.EM990044")}</div>"
        end
        if orderable_count > 0
          order_limit = product_sku_info(product_code, 'order_limit').to_i
          order_limit = 99 if order_limit == 0 || order_limit > 99
          limit = [order_limit, orderable_count].min
          value_array = []
          if values.blank?
            value_array = (1..limit).to_a
          else
            values.split(',').each{|i| value_array << i.to_i if i.to_i <= limit}
          end
          value_array.each do |v|
            amount_options << ((v == value_array.first) ? %(<option value="#{v}" selected="selected">#{v}</option>\n) : %(<option value="#{v}">#{v}</option>\n))
          end

          product_code = product_info(product_code, 'product_code') if product_code.blank?
          sku_code = (product_sku_info(product_code, 'sku_code') || get_sku_code(product_code))
          text = %(<form method="get" action="" name="form_#{sku_code.gsub(/-/, '_')}" class="cart_button_form">\n)
          unless @favorite
            text << '<div class="number_select_label">'
            text << label
            text << '</div>'
            text << '<div class="number_select_div">'
            text << @template.select_tag("amount_#{sku_code.gsub(/-/, '_')}", amount_options, {})
            text << '</div>'
            text << '<div class="cart_button_div">'
            text << @template.button_to_function(button_name, "cart_on_click('#{sku_code}', '#{sku_code.gsub(/-/, '_')}')", :class => class_name)
            text << '</div>'
            text << '</form>'
            unless @exist_cart_on_click_function
              @exist_cart_on_click_function = true
              text << <<-EOS
                <script type='text/javascript'>
                  function cart_on_click(sku_code, sku_code_for_name) {
                    location.href = '/cart/index?sku_code=' + sku_code + '&qty=' + document.getElementById('amount_' + sku_code_for_name).value;
                  }
                </script>
              EOS
            end
          else
            # お気に入り商品一覧画面のカートボタンは数量のセレクトボックスは出さず、数量は「1」固定。
            # カートに入れた後、お気に入り商品のカート追加日時を更新するため、お気に入り商品IDをリクエストパラメータに付与。
            text << '<div class="cart_button_div">'
            text << @template.button_to_function(button_name, "cart_on_click('#{sku_code}', '#{@favorite.id}')", :class => class_name)
            text << '</div>'
            text << '</form>'
            unless @exist_cart_on_click_function
              @exist_cart_on_click_function = true
              text << <<-EOS
                <script type='text/javascript'>
                  function cart_on_click(sku_code, favorite_id) {
                    location.href = '/cart/index?sku_code=' + sku_code + '&qty=1&favorite_to_cart=' + favorite_id;
                  }
                </script>
              EOS
            end
          end
        else
          text = product_sku_info(product_code, 'stockout_display_name')
        end
        return text
      end


      desc <<-EOS
        [概要: ｢お買い物を続ける｣ボタンを表示します。戻り先は商品一覧に設定してあるページになります。カート用のページ内で使用してください。]
        [引数 (オプション): class ボタンのクラス名を指定できます。]
        [使用例:
          {{{
          <ec:cart_continue_shopping_button class="aaa"/>
          }}}
          　　==> <input class="aaa button_gray" type="button" value="←お買い物を続ける" onclick="location.href='設定した商品一覧';"/>
        ]
      EOS
      def cart_continue_shopping_button(options = {})
        options[:class] = "#{options[:class]} button_gray"
        shop_setting = ShopSetting.get_setting(:continue_shopping_action, I18n.locale)
        shop_setting.sub!(/\//, '') if shop_setting
        @template.button_to_url(I18n.t(:"activerecord.errors.messages.NM990012"), {:controller => 'cms', :action => (shop_setting || 'index') }, options)
      end

      desc <<-EOS
        [概要:カート表示時のエラーメッセージがあればそれを表示します。カート用のページ内で使用してください。]
        [使用例:
          {{{
          <ec:cart_error_messages/>
          　　==> <div class="alert_msg"><ul><li>商品Aは売り切れました</li></ul></div>
          }}}
        ]
      EOS
      def cart_error_messages
        result = ''
        if @template.instance_variable_get(:@stock_error)
          result << @template.content_tag(:li, @template.instance_variable_get(:@stock_error))
        end
        if @template.instance_variable_get(:@sales_period_error)
          result << @template.content_tag(:li, @template.instance_variable_get(:@sales_period_error))
        end
        @template.content_tag(:div, @template.content_tag(:ul, result), :class=> 'alert_msg') unless result.blank?
      end

      desc <<-EOS
        [概要:カートに商品が入っていない場合、このタグで囲んだ部分を表示します。]
        [引数 (オプション):なし]
        [戻り値:]
        [使用例:{{{<ec:if_empty_cart>カートに商品が入っていません</ec:if_empty_cart>}}}]
      EOS
      def if_empty_cart(&block)
        cart = session[:cart]
        cart_items = cart.item_list
        raise UsageError, "ここでは使用できません。" unless cart_items
        if cart_items.empty?
          begin
            @template.concat(@template.capture(&block))
          ensure
            ""
          end
        end
      end

      desc <<-EOS
        [概要:カート内に商品が一つでも入っている場合、このタグで囲んだ部分を表示します。]
        [引数 (オプション):なし]
        [戻り値:]
        [使用例:{{{<ec:if_not_empty_cart>カートに入っている商品一覧</ec:if_not_empty_cart>}}}]
      EOS
      def if_not_empty_cart(&block)
        cart = session[:cart]
        cart_items = cart.item_list
        raise UsageError, "ここでは使用できません。" unless cart_items
        unless cart_items.empty?
          begin
            @template.concat(@template.capture(&block))
          ensure
            ""
          end
        end
      end

      desc <<-EOS
        [概要:カートの明細の一覧を表示します。（「共通部品_カート内容」ページが挿入されます）]
        [引数 (オプション):なし]
        [戻り値:カートの明細の一覧]
      EOS
      def cart_list_include(&block)
        cart = session[:cart]
        raise UsageError, "ここでは使用できません。" unless cart
        @template.render(:partial => "cart/cart_list")
      end


      desc <<-EOS
        [概要:カートに入っている商品を繰り返し表示します。商品情報はec:order_product_list内で使用できるタグが使用できます。]
        [引数 (オプション):なし]
        [戻り値:カートに入っている商品一覧]
        [使用例:
           {{{
           <ec:cart_item_list>
             <ec:cart_item_image/>
             <ec:cart_item_product_name/>
             <ec:cart_item_product_sku_unit_price/>
           </ec:cart_item_list>
           }}}
        ]
      EOS
      def cart_item_list(&block)
        cart = session[:cart]
        cart_items = cart.item_list_order_by_present
        raise UsageError, "ここでは使用できません。" unless cart_items
        cart_items.each do |cart_item|
          @cart_item = cart_item
          begin
            @template.concat(@template.capture(&block))
          ensure
            @cart_item = nil
          end
        end
      end


      desc <<-EOS
        [概要:配送商品の画像を表示します。配送商品一覧内で使用してください。]
        [引数 type(任意):商品画像のサイズを指定します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数:なし]
        [使用例：ec:cart_item_listを参照してください。]
      EOS
      def cart_item_image(*args, &block)
        raise UsageError, "cart_item_imageはec:cart_item_listの中で使用してください" unless @cart_item
        type, options = extract(args,:type => nil)
        unless @cart_item.product_sku.product.main_image.nil?
          url = @cart_item.product_sku.product.main_image.public_filename(type)
          options["alt"] ||= @cart_item.name
          return @template.image_tag(url, options)
        end
      end

      desc <<-EOS
        [概要:配送商品の名前を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:cart_item_listを参照してください。]
      EOS
      def cart_item_product_name(*args, &block)
        raise UsageError, "cart_item_product_nameはec:cart_item_listの中で使用してください" unless @cart_item
        @cart_item.name
      end

      desc <<-EOS
        [概要:配送商品SKUの非会員用の定価を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:cart_item_listを参照してください。]
      EOS
      def cart_item_product_sku_unit_price(*args)
        raise UsageError, "cart_item_product_sku_unit_priceはec:cart_item_listの中で使用してください" unless @cart_item
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@cart_item.product_sku.price, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:配送商品の単価(割引前)を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:cart_item_listを参照してください。]
      EOS
      def cart_item_unit_price(*args)
        raise UsageError, "cart_item_unit_priceはec:cart_item_listの中で使用してください" unless @cart_item
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@cart_item.unit_price, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:ショッピングカート内の数量を変更するセレクトボックスを表示します。cart_item_list内で使用できます。]
        [引数 format(任意):個数の単位を指定します。個数、または個数セレクトボックスの後ろに表示されます]
        [戻り値:数量を変更するセレクトボックス]
      EOS
      def cart_item_qty_select(options = {}, &block)
        raise UsageError, "cart_item_qty_selectはec:cart_item_listの中で使用してください" unless @cart_item
        # TODO プレゼント品ならば、selectタグではなく文字列を出力する

        course_div_param = "course_div=#{@cart_item.course_div}"
        key = @cart_item.key
        order_limit = @cart_item.product_sku.order_limit
        order_limit = 99 if order_limit.nil? || order_limit > 99
        limit = [@cart_item.product_sku.orderable_count, order_limit].min
        order_count_list = (1..limit).to_a

        # 現在カートに入っている個数をセレクトボックスに追加する
        # 商品をカートに入れてから実在庫が変更された場合に、実際と表示が食い違ってしまうことを防ぐため
        order_count_list = (order_count_list << @cart_item.qty).sort.uniq  

        qty_format = options[:format] || ""
        result = ''
        # プレゼント品ならば、selectタグではなく文字列を出力する
        if @cart_item.present?
          result << "<span id='#{"product_row_#{key}_qty"}'>#{@cart_item.qty.to_s}</span>"
          result << qty_format
        else
          result << @template.select("", "", order_count_list, {:selected => @cart_item.qty}, {:id => "product_row_#{key}_qty"})
          result << qty_format
          result << @template.observe_field("product_row_#{key}_qty",
            :update => 'normal_cart_list_xx',
            :url =>{:controller => 'cart', :action => 'recalculate', :id => @cart_item.key},
            :loading => "$('normal_indicator').show();",
            :success => "$('normal_qty_error_msg').hide();",
            :failure =>"$('normal_qty_error_msg').show();new Effect.Fade('normal_qty_error_msg', {duration: 3});",
            :complete => "request.responseText.evalScripts(); $('normal_indicator').hide();",
            :with => "#{course_div_param} + \'&qty='+escape($F('product_row_#{key}_qty')) + '&session_id_validation=#{@session_id_validation}'"
          )
        end
        result
      end


      # TODO このメソッドは cart_item_delete_link ！！
      desc <<-EOS
        [概要:ショッピングカート内の商品を削除するボタンを表示します。cart_item_list内で使用できます。]
        [引数 (オプション):del ete_lie 削除ボタンの名前を設定できます。何も設定しない場合はdeleteを表示します。]
        [使用例:{{{<ec:cart_item_delete_link delete_link_name="削除" />}}}]
      EOS
      def cart_item_delete_button(options = {}, &block)
        raise UsageError, "cart_item_delete_buttonはec:cart_item_listの中で使用してください" unless @cart_item
        # プレゼント品ならば、「削除」ボタンを表示しない
        return if @cart_item.present?
        session_id_validation = @template.instance_variable_get(:@session_id_validation)
        delete_button_name = options[:delete_button_name] || "delete"
        @template.link_to_remote(delete_button_name,
          :update => 'normal_cart_list_xx',
          :url => {:controller => 'cart', :action=> 'remove', :id => @cart_item.key},
          :complete => "request.responseText.evalScripts(); Element.disable('nomal_deli_recalculate')",
          :loading => "$('submit').disable()",
          :with => "'session_id_validation=#{session_id_validation}'")
      end

      desc <<-EOS
        [概要:配送商品の小計を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:cart_item_listを参照してください。]
      EOS
      def cart_item_subtotal_amount(*args)
        raise UsageError, "cart_item_subtotal_amountはec:cart_item_listの中で使用してください" unless @cart_item
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@cart_item.subtotal_amount, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:配送商品の割引金額を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:cart_item_listを参照してください。]
      EOS
      def cart_item_discount_amount(*args)
        raise UsageError, "cart_item_discount_amountはec:cart_item_listの中で使用してください" unless @cart_item
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@cart_item.discount_amount, pre_currency, post_currency, true)
      end

      desc <<-EOS
        [概要:配送商品の小計(割引後)を表示します。配送商品一覧内で使用してください。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [使用例：ec:cart_item_listを参照してください。]
      EOS
      def cart_item_total_amount(*args)
        raise UsageError, "cart_item_total_amountはec:cart_item_listの中で使用してください" unless @cart_item
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        display_number_with_unit(@cart_item.total_amount, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:カートの合計欄を表示します。（_cart_total_pricesページが挿入されます。）]
        [引数 (オプション):なし]
        [戻り値:カート合計欄]
      EOS
      def cart_sum_total_amount_include(&block)
        cart = session[:cart]
        raise UsageError, "ここでは使用できません。" unless cart
        @template.render(:partial => "cart/cart_total_prices")
      end

      desc <<-EOS
        [概要:カート内の通常商品の合計数量を表示します。cart_item_list内で使用できます。]
        [引数 (オプション):なし]
        [戻り値:カート内の通常商品の合計数量]
        [使用例:{{{<ec:cart_sum_total_qty />点}}} ⇒ {{{2点}}}]
      EOS
      def cart_sum_total_qty(&block)
        cart = session[:cart]
        normal_deli_qty = cart.total_qty # @template.instance_variable_get(:@normal_deli_qty)
        raise UsageError, "ここでは使用できません。" unless normal_deli_qty
        "<span id=\"normal_deli_qty\">#{normal_deli_qty}</span>"
      end

      desc <<-EOS
        [概要:カート内の通常商品の小計金額を表示します。cart_item_list内で使用できます。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [戻り値:カート内の通常商品の小計金額]
        [使用例:{{{<ec:cart_sum_subtotal_amount />円}}} ⇒ {{{1,000円}}}]
      EOS
      def cart_sum_subtotal_amount(*args)
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        span_for_cart_number_attribute(:subtotal_amount, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:カート内の通常商品の小計に対する割引金額を表示します。cart_item_list内で使用できます。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [戻り値:カート内の通常商品の小計に対する割引金額金額]
        [使用例:{{{<ec:cart_sum_discount_amount />円}}} ⇒ {{{1,000円}}}]
      EOS
      def cart_sum_discount_amount(*args)
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        span_for_cart_number_attribute(:discount_amount, pre_currency, post_currency)
      end

      desc <<-EOS
        [概要:カート内の通常商品の合計金額を表示します。cart_item_list内で使用できます。]
        [引数 pre_currency(任意):通貨単位を指定します。数値の前に表示されます。]
        [引数 post_currency(任意):通貨単位を指定します。数値の後ろに表示されます。]
        [戻り値:カート内の通常商品の合計金額]
        [使用例:{{{<ec:cart_sum_total_amount />円}}} ⇒ {{{1,000円}}}]
      EOS
      def cart_sum_total_amount(*args)
        post_currency, pre_currency, options = extract(args, :post_currency => "", :pre_currency => "")
        span_for_cart_number_attribute(:total_amount, pre_currency, post_currency, :id => 'total_amount')
      end

      private

      def span_for_cart_number_attribute(attr, prefix, postfix, options = {})
        cart = session[:cart]
        value = cart.send(attr)
        options = {
          :id => "cart_#{attr.to_s}"
        }.update(options || {})
        raise UsageError, "ここでは使用できません。" unless value
        span_tag = @template.content_tag(:span, @template.number_with_delimiter(value), options)
        "#{prefix}#{span_tag}#{postfix}"
      end

      # 数値をカンマ区切りにして接頭辞と接尾辞を付ける。数値が空なら空文字を返す。
      def display_number_with_unit(value, prefix, postfix, not_render_zero=false)
        return "" unless value
        str_value = @template.number_with_delimiter(value)
        return "" if str_value.blank?
        return "" if str_value == "0" && not_render_zero
        "#{prefix}#{str_value}#{postfix}"
      end

      public

      desc <<-EOS
        [概要:カート用のログインフォームタグを表示します。ショッピングカート画面内で使用できます。入力項目はuser_login_form_tagを使用できます。]
        [引数 (オプション):なし]
        [戻り値:ログインフォームタグ]
        [使用例:{{{<ec:cart_login_form> ～ </ec:cart_login_form>}}} ⇒ {{{<form method="post" action="/en/account/authenticate~"> ～ </form>}}}]
      EOS
      def cart_login_form(options = {}, &block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user_login_form)
        options = {:completion_url => {:controller => 'cart', :action => 'index'}}
        url = {:controller => 'account', :action => :authenticate}.update(options)
        @template.form_for(:user_login_form, :url => url) do |f|
          @user_login_form = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @user_login_form = nil
          end
        end
      end

      desc <<-EOS
        [概要:カート用のログイン時のエラーを表示します。ショッピングカート画面内で使用できます。]
        [引数 (オプション):なし]
        [戻り値:ログイン時エラー文]
        [使用例:{{{<ec:cart_login_error_messages />}}}]
      EOS
      def cart_login_error_messages
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user_login_form)
        @template.instance_variable_get(:@login_error_notice)
      end


      desc <<-EOS
        [概要: カート用パスワード再設定画面へのリンクを表示します。パスワード再設定後、カート画面｢cart/index｣に遷移します。]
        [引数 (オプション): リンクのHTMLオプションを指定します。]
        [戻り値: パスワード再設定画面へのリンク]
        [使用例:
          {{{
          <ec:cart_link_to_reminder text="パスワードを忘れた時は？"/>
          <ec:cart_link_to_reminder> パスワードを忘れた時は？ </ec:cart_link_to_reminder>
          }}}
        ]
      EOS
      def cart_link_to_reminder(*args, &block)
        text, html_options = extract(args, :text => nil)
        params = @template.send(:params)
        completion_url = {:controller => :cart, :action => :index}
        back_to_url = {:controller => params[:controller], :action => params[:action]}
        options = {:controller => :reminder, :action => :index, :params => {:completion_url => completion_url, :back_to_url => back_to_url}}
        text = @template.capture(&block) if block_given?
        result = link_to(text, options, html_options) if text
        # ブロックが渡されたメソッド(空でない中身を持つecタグ)は <%= %> ではなく <% %>で出力されるので、
        # 結果をoutput_bufferに出力するようにconcatします
        @template.concat(result) if result
      end
      alias_method :cart_link_to_remider, :cart_link_to_reminder

      def cart_clear_button(*args, &block)
        cart = session[:cart]
        raise UsageError, "ここでは使用できません。" unless cart
        button_name, options = *extract(args, :button_name => "ｶｰﾄを空にする")
        @template.button_to(button_name, :controller=>'cart', :action=>'remove_mobile', :all=>"true")
      end
    end
  end
end
