# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module HtmlTag
      extend MethodDescription

      unless defined?(SUPPORTED_HTML_TAGS)
        # meta, p はメソッドがあるので要注意
        # link は別途メソッドがあるので外します。
        SUPPORTED_HTML_TAGS = %w(html head title script body div span br hr h1 h2 h3 tr td th table tbody thead tfoot).freeze
        SUPPORTED_HTML_TAGS.each do |html_tag|
          module_eval(<<-"EOS")
            def #{html_tag}(options = {}, &block)
              any({:tag_name => '#{html_tag}'}.update(options), &block)
            end
          EOS
        end
      end

      desc <<-EOS
        [概要:
          {{{中身をHTMLの<head>の中のcssをロードするタグの前に追加します。
          そのページでのみ有効になるtitle, metaタグなどはこちらに記述してください}}}]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
          <ec:head_in_page><meta name="description" content="商品の一覧"></ec:head_in_page>
          }}}
        ]
      EOS
      def head_in_page(*args, &block)
        @template.content_for(:head, &block)
      end
      alias_method :html_head, :head_in_page

      desc <<-EOS
        [概要:
          {{{中身をHTMLの<head>の中のcssをロードする箇所に追加します。
          そのページでのみ有効になるcssをロードするec:stylesheetタグなどはこちらに記述してください}}}]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
          <ec:css_in_page><ec:stylesheet href="xecprint.css"/></ec:css_in_page>
          }}}
        ]
      EOS
      def css_in_page(*args, &block)
        @template.content_for(:css, &block)
      end

      desc <<-EOS
        [概要:
          {{{中身をHTMLの<head>の中のJavaScriptをロードする箇所に追加します。
          そのページでのみ有効になるjsファイルをロードするec:javascriptタグなどはこちらに記述してください。}}}]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
          <ec:javascript_in_page><ec:javascript href="xec.js"/></ec:javascript_in_page>
          }}}
        ]
      EOS
      def javascript_in_page(*args, &block)
        @template.content_for(:javascript, &block)
      end

      desc <<-EOS
        [概要:スタイルシートを読み込むタグを出力します。]
        [引数 hrefあるいはsrc(必須):参照する.cssファイルのパスまたはURLを指定します。]
        [引数 media(オプション):出力メディアを指定します。screen、print、allなどを指定できます。指定しない場合はscreenが設定されます。]
        [戻り値:スタイルシートタグ]
        [使用例:
          {{{
          <ec:stylesheet href="xecprint.css" media="print"/>
          （hrefに、ファイルアップロードしたcssファイルのパスを指定、mediaに"print"を指定）
          　⇒ <link href="/stylesheets/ecrprint.css" media="print" rel="stylesheet" type="text/css" />
          }}}
        ]
      EOS
      def stylesheet(*args, &block)
        href, media, options = *extract(args, [:href, :src], :media => 'screen')
        @template.stylesheet_link_tag(href, :media => media)
      end

      desc <<-EOS
        [概要:javascriptを読み込みます。]
        [引数 srcあるいはhref(必須):参照する.jsファイルのパスまたはURLを指定します。]
        [戻り値:スクリプトタグ]
        [使用例:{{{<ec:javascript href="xec.js"/> （hrefに、ファイルアップロードしたjsファイルのパスを指定）}}}]
      EOS
      def javascript(*args, &block)
        href, options = *extract(args, [:src, :href])
        @template.javascript_include_tag(href)
      end

      desc <<-EOS
        [概要:METAタグを作成できます。]
        [引数 name(必須):プロパティを指定します。]
        [引数 content(必須):内容を指定します。]
        [引数 lang(任意):内容の言語を指定します。]
        [戻り値:METAタグ]
        [使用例:
          {{{
          <ec:head>
            <ec:meta name="description" content="ホームページの紹介文"/>
          </ec:head>
          }}}
        ]
      EOS
      def meta(*args, &block)
        name, content, lang = *extract(args, :name, :content, :lang => nil)
        @template.tag('meta', :name => name, :content => content, :lang => lang)
      end

      desc <<-EOS
        [概要:ページに関連するキーワードを指定できます。]
        [引数 content(必須):キーワードを指定します。]
        [引数 lang(必須):キーワードの言語を指定します。]
        [戻り値:METAキーワードタグ]
        [使用例:
          {{{
          <ec:head>
            <ec:keyword content="オンラインショッピング,ファッション" lang="ja"/>
          </ec:head>
          }}}
        ]
      EOS
      def keyword(*args, &block)
        content, lang, options = *extract(args, :content, :lang => nil)
        meta(:name => 'keywords', :content => content, :lang => lang)
      end

      desc <<-EOS
        [概要:画像を表示します。]
        [引数 src(必須):{{{画像ファイルのまたはURLを指定します。
          ファイルマネージャでアップロードしたファイルの場合、imageフォルダ(/image)は省略できます。}}}]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:画像。]
        [使用例:
          {{{
          <ec:image src="logo.gif" alt="ロゴ" width="50%" height="50%"/> （HTMLオプションalt,width,heightを指定）
          　　⇒ <img height="50%" width="50%" src="/images/logo.gif" alt="ロゴ"/>
          }}}
        ]
      EOS
      def image(*args, &block)
        options = args.shift || {}
        src = options.delete(:src)
        @template.image_tag(src, options) if src
      end

      desc <<-EOS
        [概要:テキストリンクを表示します。タグの入れ子も可能です。]
        [引数 text(必須):リンクの文字列を指定します。]
        [引数 href(必須):リンク先URLを指定します。]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:テキストリンク。]
        [使用例1:
          {{{
          <ec:link text="商品一覧" href="/list" class="linkClass"/>　（HTMLオプションclassを指定）
          　　⇒ <a class="linkClass" href="/list">商品一覧</a>
          }}}
        ]
        [使用例2:
          {{{
          <ec:link href="/list" class="linkClass">　（HTMLオプションclassを指定）<br/>
          &nbsp;&nbsp;  <ec:image src="logo.gif" alt="ロゴ" width="50%" height="50%"/> <br/>
          &nbsp;&nbsp;  普通のテキスト
          </ec:link>
          　　⇒ <a class="linkClass" href="/list"> <br/>
              &nbsp;&nbsp;  <img height="50%" width="50%" src="/cms/logo.gif" alt="ロゴ"/> <br/>
              &nbsp;&nbsp;  普通のテキスト
              </a>
           }}}
         ]
      EOS
      def link(*args, &block)
        href, text, options = extract(args, [:href, :src], :text => nil)
        text = @template.capture(&block) if block_given?
        result = link_to(text, {:href => href, :escape => false}, options) if text && href
        # ブロックが渡されたメソッド(空でない中身を持つecタグ)は <%= %> ではなく <% %>で出力されるので、
        # 結果をoutput_bufferに出力するようにconcatします
        @template.concat(result) if result && block_given?
        result
      end

      private
      def link_to(*args, &block)
        @template.link_to_without_ecr_omission(*args, &block)
      end

      public

      desc <<-EOS
        [概要:店舗情報で設定したトップページに遷移するリンクを表示します。設定していない場合は｢index｣に遷移します。]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:テキストリンク。]
        [使用例:
           {{{
            <ec:top_link class="linkClass">トップにもどる</ec:top_link>　（HTMLオプションclassを指定）<br/>
          　　⇒ <a class="linkClass" href="/index">トップにもどる</a>
           }}}]
             EOS
      def top_link(*args, &block)
        text, options = extract(args, :text => nil)
        text = @template.capture(&block) if block_given?
        href = {:controller => 'cms', :action => ShopSetting.get_setting(:top_action, I18n.locale)}
        result = @template.link_to(text, href, options) if text
        @template.concat(result) if result && block_given?
        return result unless block_given?
      end

      desc <<-EOS
        [概要:店舗情報で設定したサイト入り口に遷移するリンクを表示します。設定していない場合は｢index｣に遷移します。]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:テキストリンク。]
        [使用例:
           {{{
            <ec:entrance_link class="linkClass">サイト入り口にもどる</ec:entrance_link>　（HTMLオプションclassを指定）<br/>
          　　⇒ <a class="linkClass" href="/index">サイト入り口にもどる</a>
           }}}]
             EOS
      def entrance_link(*args, &block)
        text, options = extract(args, :text => nil)
        text = @template.capture(&block) if block_given?
        href = {:controller => 'cms', :action => :index}
        result = @template.link_to(text, href, options) if text
        @template.concat(result) if result && block_given?
        return result unless block_given?
      end

      desc <<-EOS
        [概要:店舗情報で設定したトップページに遷移するボタンリンクを表示します。設定していない場合は｢index｣に遷移します。]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:フォームボタンリンク。]
        [使用例:
           {{{
            <ec:top_button class="linkClass">トップにもどる</ec:top_button>　（HTMLオプションclassを指定）<br/>
          　　⇒ <input onclick=\"location.href='/';" type="button" value="トップにもどる" />
           }}}]
             EOS
      def top_button(*args, &block)
        text, options = extract(args, :text => nil)
        text = @template.capture(&block) if block_given?
        href = {:controller => 'cms', :action => ShopSetting.get_setting(:top_action, I18n.locale)}
        result = @template.button_to_url(text, href, options) if text
        @template.concat(result) if result && block_given?
        return result unless block_given?
      end

      desc <<-EOS
        [概要:店舗情報で設定したサイト入り口に遷移するボタンリンクを表示します。設定していない場合は｢index｣に遷移します。]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:フォームボタンリンク。]
        [使用例:
           {{{
              <ec:entrance_button class="linkClass">サイト入り口にもどる</ec:entrance_button>　（HTMLオプションclassを指定）<br/>
          　　⇒ <a class="linkClass" href="/index">サイト入り口にもどる</a>
           }}}]
             EOS
      def entrance_button(*args, &block)
        text, options = extract(args, :text => nil)
        text = @template.capture(&block) if block_given?
        href = {:controller => 'cms', :action => :index}
        result = @template.button_to_url(text, href, options) if text
        @template.concat(result) if result && block_given?
        return result unless block_given?
      end

      # お買物方法へのリンク
      def link_to_howto_shopping(*args, &block)
        return ""
      end

      desc <<-EOS
        [概要:画面を遷移するボタンを表示します。]
        [引数 href(任意):ボタンを押したときの遷移先URLを指定します。指定しない場合はトップページへ遷移します。]
        [戻り値:画面遷移リンク。]
        [使用例:{{{<ec:language_links/>}}}]
      EOS
      def button(*args, &block)
        options = args.shift || {}
        href = options.delete(:href)
        text = options.delete(:text)
        text = @template.capture(&block) if block_given?
        result = @template.button_to_url(text, href, options) if text
        # ブロックが渡されたメソッド(空でない中身を持つecタグ)は <%= %> ではなく <% %>で出力されるので、
        # 結果をoutput_bufferに出力するようにconcatします
        @template.concat(result) if result && block_given?
        result
      end

      desc <<-EOS
        [概要:直前に表示していた画面に戻るボタンを表示します。]
        [戻り値:画面遷移リンク。]
        [使用例:{{{<ec:button_to_back/>}}}]
      EOS
      def button_to_back(*args, &block)
        options = args.shift || {}
        href = @template.send(:request).referer || '/'
        result = @template.button_to_url(@template.capture(&block), href, options) if block_given?
        @template.concat(result) if result && block_given?
      end

      desc <<-EOS
        [概要:画像リンクを表示します。]
        [引数 href(必須):画像を押したときの遷移先URLを指定します。]
        [引数 src(必須):画像ファイルのパスまたはURLを指定します。
          ファイルマネージャでアップロードしたファイルの場合、imageフォルダ(/image)は省略できます。]
        [引数 (オプション):画像のHTMLオプションを指定します。]
        [戻り値:画像リンク。]
        [使用例:
          {{{
          <ec:image_link src="logo.gif" href="/" border="1"/>　（HTMLオプションborderを指定）
          　　⇒ <a href="/"><img border="1" src="/images/logo.gif" alt="Logo"/></a>
          }}}
        ]
      EOS
      def image_link(*args, &block)
        options = args.shift || {}
        src = options.delete(:src)
        href = options.delete(:href)
        link_to(@template.image_tag(src, options), href) if src && href
      end

      desc <<-EOS
        [概要:言語を切り替えるリンクを表示します。]
        [引数:なし]
        [戻り値:言語切り替えリンク。]
        [使用例:{{{<ec:language_links/>}}}]
      EOS
      def language_links(*args, &block)
        links = []
        I18n.supported_locales.each_with_index do |locale, index|
          locale_str = locale.respond_to?(:to_simple) ? locale.to_simple.to_s : locale.to_s
          links << link_to(
            I18n.t(:native_lang_word, :locale => locale_str),
            {:controller => 'locale', :action => :update, :id => locale_str}, :method => :put)
        end
        '| %s |' % links.join(' | ')
      end

      desc <<-EOS
        [概要:指定した文字数だけ値を表示します。]
        [引数 limit(必須):表示する文字数]
        [引数 leader(任意):省略を表す文字列を指定します。何も指定しない場合は"..."になります。]
        [戻り値:character_limitで指定した文字数までの文字列にleaderで指定した文字列を結合して返します。]
        [使用例:&lt;ec:display_with_character_limit limit="50" leader="..."&gt&lt;ec:recommendation/&gt&lt;/ec:display_with_character_limit&gt]
      EOS
      def display_with_character_limit(*args, &block)
        limit, leader = *extract(args, :limit, :leader => "...")
        limit = limit.to_i
        original = @template.capture(&block)

        parts = []
        first = 0
        html_tags = original.to_s.scan(/<.+?>/m)
        html_tags.each do |tag|
          last = first + original[first..-1].index(tag)
          parts.concat(original[first..last-1].split(//))
          parts << tag
          first = last + tag.length
        end
        parts.concat(original.to_s[first..-1].split(//))

        # HTMLタグは文字数としてカウントしません
        parts.each_with_index do |part, index|
          limit += 1 if index <= limit && part =~ /^<.*>$/
        end

        if original.length > limit.to_i
          @template.output_buffer << parts[0..limit-1].join << leader
        else
          @template.output_buffer << parts.join
        end
      end
      
      desc <<-EOS
        [概要:条件にマッチするバナーを繰り返します。]
        [引数 banner_code(必須):管理画面で設定したバナーのコードを入力します。]
        [引数 display_qty(必須):表示するバナーの最大個数を設定します。]
        [戻り値:設定したバナーコードのバナー]
        [使用例:
           {{{
              <ec:each_banners banner_code="0000001" display_qty="6">
                <ec:banner_link/>
              </ec:each_banners>
           }}}
           ⇒ バナーコード｢00001｣のバナー条件からログイン会員にマッチするバナーを表示します。
        ]
      EOS
      def each_banners(*args, &block)
        banner_code, display_qty, options = extract(args, :banner_code => "", :display_qty => nil)
        raise UsageError, "banner_codeを設定してください。" unless banner_code
        raise UsageError, "display_qtyを設定してください。" unless display_qty
        if @template.logged_in? && @template.current_user
          user = @template.current_user
        else
          user = User.new()
        end
        @banners = Banner.banner_group(banner_code, user, display_qty)
        begin
          _each_with_index(:@banners, :@banner, :@index) do
            @template.concat(@template.capture(&block))
          end
        ensure
          @banners = nil
        end
      end
      alias_method :banners_for, :each_banners

      desc <<-EOS
        [概要:バナーのタイトルまたは画像を表示します。banners_for内で使用してください。urlの指定がある場合はリンクになります。]
        [引数:no_link(任意) trueを指定すると、urlが指定されていてもリンクになりません。]
        [引数:no_image(任意) trueを指定すると、画像バナーの場合は何も表示しません。]
        [戻り値:設定したバナーコードのバナー]
        [使用例:
           {{{
              <ec:banners_for banner_code="0000001" display_qty="6">
                <ec:banner_title_link class="class_A"/>
              </ec:banners_for>
　　　　　　　　　　　⇒　urlが設定されている場合、そのurlにリンクするリンクバナー(タイトル)を表示します。
              <ec:banners_for banner_code="0000001" display_qty="6">
                <ec:banner_title_link no_link="true" class="class_A"/>
              </ec:banners_for>
　　　　　　　　　　　⇒　テキストバナータイトル(必ずテキスト表示されます。)
           }}}
           ⇒ バナーコード｢00001｣のバナー条件からログイン会員にマッチするバナーリンクを表示します。
        ]
      EOS
      def banner_title_link(*args)
        raise UsageError, "bannerはec:banners_forの中で使用してください" unless @banner
        no_image, no_link, options = extract(args, :no_image => '', :no_link => nil)
        display = @banner.display_for(I18n.locale.to_s)
        return "" if display.nil?
        link_url = @banner.link_url
        if @banner.type_key == :image
          return '' if no_image == 'true'
          path = '/' + display.image_public_path
          image_or_text = @template.image_tag(path, :alt => display.image_alt)
        else
          image_or_text = display.text_title
        end
        return (!no_link.nil? || link_url.blank?) ? image_or_text : link_to(image_or_text, {:href => link_url, :escape => false}, options)
      end
      
      desc <<-EOS
        [概要:バナーの詳細または画像を表示します。banners_for内で使用してください。urlの指定がある場合のみリンクになります。]
        [引数:no_link(任意) trueを指定すると、urlが指定されていてもリンクになりません。]
        [引数:no_image(任意) trueを指定すると、画像バナーの場合は何も表示しません。]
        [戻り値:設定したバナーコードのバナー]
        [使用例:
           {{{
              <ec:banners_for banner_code="0000001" display_qty="6">
                <ec:banner_link class="class_A"/>
              </ec:banners_for>
　　　　　　　　　　　⇒　urlが設定されている場合、そのurlにリンクするリンクバナー(詳細)を表示します。
              <ec:banners_for banner_code="0000001" display_qty="6">
                <ec:banner_link no_link="true" class="class_A"/>
              </ec:banners_for>
　　　　　　　　　　　⇒　テキストバナー詳細(必ずテキスト表示されます。)
           }}}
           ⇒ バナーコード｢00001｣のバナー条件からログイン会員にマッチするバナーリンクを表示します。
        ]
      EOS
      def banner_link(*args)
        raise UsageError, "bannerはec:banners_forの中で使用してください" unless @banner
        no_image, no_link, options = extract(args, :no_image => '',  :no_link => nil)
        display = @banner.display_for(I18n.locale.to_s)
        return "" if display.nil?
        link_url = @banner.link_url
        if @banner.type_key == :image
          return '' if no_image == 'true'
          path = '/' + display.image_public_path
          image_or_text = @template.image_tag(path, :alt => display.image_alt)
        else
          image_or_text = display.text_description
        end
        return (no_link == 'true') ? image_or_text : link_to(image_or_text, {:href => link_url, :escape => false}, options)
      end

      desc <<-EOS
        [概要:メールリンクを表示します]
        [引数 subject(任意):メールに初期表示するメールタイトルを設定します]
        [引数 mail_to(任意):メールに初期設定するあて先を設定します]
        [戻り値:メールリンク]
        [使用例:
           {{{
           <ec:mail_to mail_to="inquiry@ec-rider.com" subject="お問合せ">
            お問合せはこちらから
           </ec:mail_to>
           }}}
        ]
      EOS
      def mail_to_tag(*args, &block)
        email, subject, options = extract(args, :mail_to => nil,  :subject => "")
        return "" if email.nil?
        name = @template.capture(&block)
        name.strip!
        name.chomp!
        mail_body = ""
        begin
          @template.capture(&block)
          mail_body = @template.instance_variable_get("@mail_body") if @template.instance_variable_get("@mail_body")
          mail_body.strip!
          mail_body.chomp!
        ensure
          @template.instance_variable_set("@mail_body", nil)
        end
        result = conversion_mail_to(email, name, subject, mail_body, options)
        @template.concat(result)
        result
      end

      desc <<-EOS
        [概要:メール本文を表示します]
        [引数:なし]
        [戻り値:メール本文]
        [使用例:
           {{{
           <ec:mail_to mail_to="inquiry@ec-rider.com" subject="お問合せ">
             <ec:mail_to_body>
               メール本文にはなにも記入せず、このまま送信してください。
             </ec:mail_to_body>
             お問合せはこちらから
           </ec:mail_to>
           }}}
        ]
      EOS
      def mail_to_body(&block)
        @template.instance_variable_set("@mail_body", @template.capture(&block)) if block_given?
        return ""
      end

      # モバイルプラグインで文字コード変換を拡張しています。
      def conversion_mail_to(email, name, subject, mail_body, options = {})
        @template.mail_to(email, name, options.update({:subject => subject.tosjis, :body => mail_body.tosjis}))
      end

      desc <<-EOS
        [概要:携帯の絵文字を表示します]
        [引数 code(必須):絵文字コードを指定します]
        [戻り値:絵文字]
        [使用例:
           {{{
           <ec:emoji code="50"/>
           }}}
        ]
      EOS
      def emoji(*args, &block)
        code, options = *extract(args, :code)
        if EcRider.config.ec_admin
          path = '/ec_rider/images/mobile/emoji/' << code << '.gif'
          @template.image_tag(path, :alt => '[emoji' << code << ']')
        else
          # jpmobileのmobile_filterをオフにしてELIXIRで絵文字変換した場合、
          # 絵文字のバイナリコード入力ではau携帯とsoftbank携帯で絵文字が表示されず、
          # 絵文字のUnicodeのテキスト入力ではdocomo携帯で絵文字が表示されません。
          # そのため、キャリアによって絵文字の指定方法を変えています。
          if @template.send(:request).mobile.class.name == "Jpmobile::Mobile::Docomo"
            emoji_code = EcRider::Emoji::EMOJI_DOCOMO_BINARY[code]
            emoji_code.nil? ? '' : [emoji_code].pack('U')
          else
            emoji_code = EcRider::Emoji::EMOJI_DOCOMO_UNICODE[code]
            emoji_code.nil? ? '' : emoji_code
          end
        end
      end
    end
  end
end
