# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module OrderFormTag
      extend MethodDescription

      desc <<-EOS
          [概要：ポイント設定で注文金額・保有ポイントがポイント利用条件に一致している場合に中身を表示します。]
          [使用例：
            {{{
              <ec:if_exchangeable_point>
                ポイント利用できます。
              </ec:if_exchangeable_point>
            }}}
            ⇒ (ポイント利用条件に一致している場合)ポイント利用できます。
          ]
      EOS
      def if_exchangeable_point(&block)
        raise UsageError, "ec:order_form_use_point_radioはec:order_formの中で使用してください" unless @order_form
        if PointSetting.exchangeable(@template.current_user.id, session[:cart].total_amount)
          @template.concat(@template.capture(&block))
        end
      end

      desc <<-EOS
          [概要：ポイント設定で注文金額・保有ポイントがポイント利用条件に一致していない場合に中身を表示します。]
          [使用例：
            {{{
              <ec:if_not_exchangeable_point>
                ポイント利用できません。
              </ec:if_not_exchangeable_point>
            }}}
            ⇒ (ポイント利用条件に一致している場合)ポイント利用できません。
          ]
      EOS
      def if_not_exchangeable_point(&block)
        raise UsageError, "ec:order_form_use_point_radioはec:order_formの中で使用してください" unless @order_form
        unless PointSetting.exchangeable(@template.current_user.id, session[:cart].total_amount)
          @template.concat(@template.capture(&block))
        end
      end

      desc <<-EOS
          [概要：ポイントを利用するのラジオボタンを表示します。ポイント設定で注文金額がポイント利用条件に一致していない場合は表示しません。]
          [使用例：
            {{{
              <ec:order_form_use_point_radio>使用する
            }}}
          ]
      EOS
      def order_form_use_point_radio(options = {})
        raise UsageError, "ec:order_form_use_point_radioはec:order_formの中で使用してください" unless @order_form
        if PointSetting.exchangeable(@template.current_user.id, session[:cart].total_amount)
          @order_form.radio_button(:use_point_flg, true, options)
        end
      end

      desc <<-EOS
          [概要：何ポイント利用するかのテキストボックスを表示します。ポイント設定で注文金額がポイント利用条件に一致していない場合は表示しません。]
          [使用例：ec:order_formを参照してください。]
            {{{
              <ec:order_form_use_point_text>使用する
            }}}
          ]
      EOS
      def order_form_use_point_text(options = {})
        raise UsageError, "ec:order_form_use_point_radioはec:order_formの中で使用してください" unless @order_form
        options[:style] = "ime-mode: disabled"
        if PointSetting.exchangeable(@template.current_user.id, session[:cart].total_amount)
          @order_form.text_field(:use_point, options)
        end
      end

      desc <<-EOS
          [概要：｢ポイントを利用しない｣のラジオボタンを表示します。]
          [引数 label(任意)：ポイントを利用するラジオボタンのラベル名を設定します。ポイント設定で注文金額がポイント利用条件に一致していない場合は表示しません。]
          [使用例：ec:order_formを参照してください。]
            {{{
              <ec:order_form_not_use_point_radio>使用する
            }}}
          ]
      EOS
      def order_form_not_use_point_radio(options = {})
        raise UsageError, "ec:order_form_free_requestはec:order_formの中で使用してください" unless @order_form
        if PointSetting.exchangeable(@template.current_user.id, session[:cart].total_amount)
          @order_form.radio_button(:use_point_flg, false, options)
        end
      end

      desc <<-EOS
        [概要:お届け先指定・お支払方法入力用のフォームを出力します。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form>
            <ec:order_form_error_messages/>
            <ec:order_form_expire_month :name_for_error => "有効期限（月）" />
            ／年（YEAR）
            <ec:order_form_expire_year :name_for_error => "有効期限（年）" />
            <ec:order_form_card_name />
            <ec:order_form_free_request />
          </ec:order_form>
          }}}
        ]
      EOS
      def order_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        @template.form_for(:order_form, @template.instance_variable_get(:@order_form), :url => {:controller => :order, :action => :confirmation_order}, :html => {:id => "order_form" }) do |f|
          @template.fields_for(:user_address, @template.instance_variable_get(:@user_address)) do |f1|
            @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f2|
              @order_form = f
              @user_address = f1
              @user_profile = f2
              begin
                @template.concat(@template.capture(&block))
              ensure
                @order_form = nil
                @user_address = nil
                @user_profile = nil
              end
            end
          end
        end
      end

      desc <<-EOS
        [概要:お届け先指定・お支払方法入力のエラーメッセージがあればそれを表示します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_error_messages/>
          }}}
        ]
      EOS
      def order_form_error_messages
        @template.template_error_messages_for(:order_form)
      end

      desc <<-EOS
        [概要:お届け先を選択するためのラジオボタンを出力します。ec:shipping_addresses、あるいは、ec:shipping_listの中で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_addresses>
             <ec:order_form_shipping_address />
           </ec:shipping_addresses>
           }}}

           {{{
           <ec:shipping_list>
             <ec:order_form_shipping_address />
           </ec:shipping_list>
           }}}
        ]
      EOS
      def order_form_shipping_address
        raise UsageError, "ec:order_form_shipping_addressはec:shipping_addresses、あるいは、ec:shipping_listの中で使用してください" unless @shipping_address
        options = {:class =>'send_address_radio_buttons'}.update(options || {})
        js_func = @template.javascript_tag(<<-"EOS")
          (function(){
            Event.observe("order_form_send_address_index_#{@shipping_address_index}", 'click', function(event){
              $('order_form_include_order_id').value = ""
              new Effect.Appear('shipping_option_area');
            });
          })();
        EOS
        if @order_form
        @order_form.radio_button(:send_address_index, @shipping_address_index.to_s, options) << js_func
      else
        shipping_addresses = session[:shipping_addresses] || []
        raise UsageError, "想定していない画面遷移をしている可能性があります。" if shipping_addresses.empty?
        # radioボタンのvalueに指定されるindexは、session[:shipping_addresses]上での@shipping_addressの位置です。
        # 必ずすべてsession[:shipping_addresses]に含まれるはずですが、万が一の場合は0を設定しています。
        # 本来は例外を出すべき？
        index = shipping_addresses.index(@shipping_address) || 0
        @template.radio_button_tag("order_form[send_address_index]", index, false, options) << js_func
      end
    end

    desc <<-EOS
        [概要:「ギフト」チェックボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数 チェックボックスのHTML属性(任意):]
        [戻り値:「ギフト」チェックボックス]
        [使用例:
           {{{
           <ec:write_sender_check/> ギフト
           }}}
        ]
    EOS
    def gift_check(options = {})
      raise UsageError, "ec:gift_checkはec:order_formの中で使用してください" unless @order_form
      @order_form.check_box(:gift_flg, options)
    end

    desc <<-EOS
        [概要:「領収書・納品書を別送する」チェックボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数 チェックボックスのHTML属性(任意):]
        [戻り値:「領収書・納品書を別送する」チェックボックス]
        [使用例:
           {{{
           <ec:send_invoice_separately_check/> 商品に領収書・納品書を同梱しません
           }}}
        ]
    EOS
    def send_invoice_separately_check(options = {})
      raise UsageError, "ec:send_invoice_separately_checkはec:order_formの中で使用してください" unless @order_form
      @order_form.check_box(:send_invoice_separately_flg, options)
    end

    desc <<-EOS
        [概要:「送り状に購入者名を表示する」チェックボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数 チェックボックスのHTML属性(任意):]
        [戻り値:「送り状に購入者名を表示する」チェックボックス]
        [使用例:
           {{{
           <ec:write_sender_check/> 送り状に購入者名を表示します
           }}}
        ]
    EOS
    def write_sender_check(options = {})
      raise UsageError, "ec:write_sender_checkはec:order_formの中で使用してください" unless @order_form
      @order_form.check_box(:write_sender_flg, options)
    end

    desc <<-EOS
        [概要:お支払い方法のセレクトボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数 セレクトボックスのHTML属性(任意):]
        [戻り値:お支払い方法のセレクトボックス]
        [使用例:
          {{{
          <ec:order_form_payment_type />
          }}}
        ]
    EOS
    def order_form_payment_type(*args, &block)
      raise UsageError, "ec:order_form_shipping_address_radio_buttonはec:order_formの中で使用してください" unless @order_form
      on_changes = []
      show_or_hide_forms = []
      if EcRider.config.ecr_payment_types.include?('credit')
        on_changes << "show_or_hide_form('credit_form', '#{OrderShipping.payment_type_div_id_by_key(:credit)}');"
        show_or_hide_forms << "show_or_hide_form('credit_form', '#{OrderShipping.payment_type_div_id_by_key(:credit)}', true);"
      end
      if EcRider.config.ecr_payment_types.include?('cvs')
        on_changes << "show_or_hide_form('cvs_form', '#{OrderShipping.payment_type_div_id_by_key(:cvs)}');"
        show_or_hide_forms << "show_or_hide_form('cvs_form', '" + OrderShipping.payment_type_div_id_by_key(:cvs) + "', true);"
      end
      options = {:class=>'payment_type', :onChange => on_changes.to_s}
      payment_type_list = OrderShipping.select_form_list
      text = @order_form.select(:normal_payment_type, payment_type_list, {}, options)
      text << <<-EOS
        <script>
            window.onload = hide_form;
            function hide_form(){
                #{show_or_hide_forms.to_s}
            }
            // フォームの表示を切り替える
            function show_or_hide_form(form_class_name, payment_type_value, visible_flag){
                var inputs = $$('.' + form_class_name + ' input');
                var selects = $$('.' + form_class_name + ' select');
                for (var i = 0; i < selects.length; i++) {
                    inputs.push(selects[i]);
                }
                var disp = isDisp2(payment_type_value);
                /* TODO 有効にするにはバグ修正が必要（inputsにはいろんな要素が入ることの考慮漏れ）
                for (var i = 0; i < inputs.length; i++) {
                    if (!disp){
                        inputs[i].value = '';
                        inputs[i].disable();
                    } else {
                        inputs[i].enable();
                    }
                }
                */
                if (disp){
                    new Effect.Appear($$('.' + form_class_name)[0]);
                } else {
                    if (visible_flag) {
                        $$('.' + form_class_name)[0].hide();
                    } else {
                        new Effect.Fade($$('.' + form_class_name)[0]);
                    }
                }
            }
            // フォームの表示要否を判断する
            function isDisp2(payment_type_value){
                var payment_types = $$('.payment_type')
                for (var i = 0; i < payment_types.length; i++){
                    if (payment_types[i].value == payment_type_value){ return true; }
                }
                return false;
            }
        </script>
      EOS
    end

    desc <<-EOS
        [概要:配送希望日を入力するセレクトボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_delivery_date />
          }}}
        ]
    EOS
    def order_form_delivery_date(*args, &block)
      raise UsageError, "ec:order_form_delivery_dateはec:order_formの中で使用してください" unless @order_form
      # TODO 希望配送日は「日本」で固定表示しています。本来は国ごとに変わります。
      selects = @order_form.object.available_delivery_date
      @order_form.select(:delivery_date, selects, {}, {})
    end

    desc <<-EOS
        [概要:配送希望時間を入力するセレクトボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_hope_delivery_time />
          }}}
        ]
    EOS
    def order_form_delivery_time_select(*args, &block)
      raise UsageError, "ec:order_form_delivery_dateはec:order_formの中で使用してください" unless @order_form
      # TODO 希望配送時間は「日本」で固定表示しています。本来は国ごとに変わります。
      selects = @order_form.object.available_delivery_time
      display_name = []
      displays = []
      selects.each do |dt|
        unless dt.code.blank?
          @order_form_delivery_time = dt
          display = @order_form_delivery_time.displays.select{|t| t.locale == I18n.locale.to_s}
          display.first ? display.first.name : ""
          displays << [display.first.name, dt.code]
        end
      end
      @order_form.select(:delivery_time, displays, {}, {})
    end

    desc <<-EOS
        [概要:配送希望時間を出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_delivery_time_radio_blank />指定しない
          <ec:order_form_delivery_time>
            <ec:order_form_delivery_time_radio />
            <ec:order_form_delivery_time_label />
          </ec:order_form_delivery_time>
          }}}
        ]
    EOS
    def order_form_delivery_time(&block)
      raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
      raise UsageError, "ec:order_form_delivery_timeはec:order_formの中で使用してください" unless @order_form
      # TODO 希望配送時間は「日本」で固定表示しています。本来は国ごとに変わります。
      delivery_times = @order_form.object.available_delivery_time

      delivery_times.each do |dt|
        unless dt.code.blank?
          @order_form_delivery_time = dt
          begin
            @template.concat(@template.capture(&block))
          ensure
            @order_form_delivery_time = nil
          end
        end
      end
    end

    desc <<-EOS
        [概要:配送希望時間に「指定しない」を入力するラジオボタンを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例：ec:order_form_delivery_timeを参照してください。]
    EOS
    def order_form_delivery_time_radio_blank(options = {})
      raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
      raise UsageError, "ec:order_form_delivery_timeはec:order_formの中で使用してください" unless @order_form
      options = {}.update(options || {})
      options[:class] ||= 'delivery_time_radios'
      @order_form.radio_button('delivery_time', '', options)
    end
    
    desc <<-EOS
        [概要:配送希望時間を入力するラジオボタンを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例：ec:order_form_delivery_timeを参照してください。]
    EOS
    def order_form_delivery_time_radio(options = {})
      raise UsageError, "ec:order_form_delivery_time_radioはec:order_form_delivery_timeの中で使用してください" unless @order_form_delivery_time
      options = {}.update(options || {})
      options[:class] ||= 'delivery_time_radios'
      @order_form.radio_button('delivery_time', @order_form_delivery_time.code, options)
    end

    desc <<-EOS
        [概要:配送希望時間を入力するラジオボタンのラベル出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例：ec:order_form_delivery_timeを参照してください。]
    EOS
    def order_form_delivery_time_label
      raise UsageError, "ec:order_form_delivery_time_labelはec:order_form_delivery_timeの中で使用してください" unless @order_form_delivery_time
      display = @order_form_delivery_time.displays.select{|t| t.locale == I18n.locale.to_s}
      display.first ? display.first.name : ""
    end

    desc <<-EOS
        [概要:お届け先指定・お支払方法用のコンビニ一覧セレクトボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_cvs_kind />
          }}}
        ]
    EOS
    def order_form_cvs_kind(&block)
      raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
      @order_form.select(:cvs_kind, OrderForm.cvs_kind_options)
    end

    desc <<-EOS
        [概要:お届け先指定・お支払方法用のカード種類を出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_card_kind>
            <ec:order_form_card_kind_radio />
            <ec:order_form_card_kind_label />
          </ec:order_form_card_kind>
          }}}
        ]
    EOS
    def order_form_card_kind(&block)
      raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
      raise UsageError, "ec:order_form_card_kindはec:order_formの中で使用してください" unless @order_form
      checked_card_kind = @order_form.object.card_kind
      @order_form.radio_button_group(:card_kind) do |b|
        b.each do
          @order_form_card_kind = b
          begin
            @template.concat(@template.capture(&block))
          ensure
            @order_form_card_kind = nil
          end
        end
      end
      radio_class_mapping = {}
      OrderForm.card_kind_ids.each{|card_kind| radio_class_mapping[card_kind] = 'normal'}
      radio_class_mapping['amex'] = 'amex'
      radio_class_mapping['diners'] = 'amex'
      radio_class_names = radio_class_mapping.values.uniq
      @template.concat(@template.javascript_tag(<<-"EOS"))
          (function(){
            var class_name_mapping = #{radio_class_mapping.to_json};

            // 各カード種類のラジオボタンにカード番号入力欄切り替えイベントを追加する
            $$(".card_kind_radios").each(function(card_kind_radio){
              Event.observe(card_kind_radio, 'click', function(event){
                var radio = Event.element(event);
                var visible_class = class_name_mapping[radio.value];
                change_card_no_fields(visible_class);
              });
            });

            // 画面表示時にカード番号入力欄切り替えイベントを追加する
            Event.observe(window, 'load', function(event){
              var visible_class = "#{radio_class_mapping[checked_card_kind] || 'normal'}";
              change_card_no_fields(visible_class);
            });
          })();

          function change_card_no_fields(visible_class) {
            var radio_class_names = #{radio_class_names.to_json};

            radio_class_names.each(function(radio_class_name){
              var panes = $A($$(".card_no_" + radio_class_name));
              var inputs = $A($$(".card_type_" + radio_class_name + "_input"));
              if (visible_class == radio_class_name) {
                panes.each(function(pane){pane.show()});
                inputs.each(function(input){input.enable()});
              } else {
                panes.each(function(pane){pane.hide()});
                inputs.each(function(input){input.disable(); input.value="";});
              }
            });
          }
      EOS
    end

    desc <<-EOS
        [概要:お届け先指定・お支払方法用のカード種類を入力するラジオボタンを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例：ec:order_form_card_kindを参照してください。]
    EOS
    def order_form_card_kind_radio(options = {})
      raise UsageError, "ec:order_form_card_kind_radioはec:order_form_card_kindの中で使用してください" unless @order_form_card_kind
      options = {}.update(options || {})
      options[:class] ||= ''
      options[:class] << 'card_kind_radios'
      @order_form_card_kind.radio_button(options)
    end

    desc <<-EOS
        [概要:お届け先指定・お支払方法用のカード種類を入力するラジオボタンのラベル出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例：ec:order_form_card_kindを参照してください。]
    EOS
    def order_form_card_kind_label(options = {})
      raise UsageError, "ec:order_form_card_kind_radio_labelはec:order_form_card_kindの中で使用してください" unless @order_form_card_kind
      @order_form_card_kind.label
    end

    class CardNoFieldAndExample
      def initialize(template, order_form, card_type, name_for_error)
        @template = template
        @order_form = order_form
        @card_type_key = card_type
        @name_for_error = name_for_error
      end

      def fields(options = {})
        case @card_type_key
        when "amex" then
          text_options = {:style=>"ime-mode: disabled"}.update(options || {})
          setup_class(text_options, 'card_type_amex_input') # このクラス名で表示／非表示の制御を行います。
          js_options = {:numeric => true, :realtime => true, :update => 'card_no_err_msg', :error => @name_for_error}
          @order_form.text_field_valid(:no_1, {:size => 4, :maxlength => 4}.update(text_options), {:min => 4}.update(js_options)) <<
            '-' <<
            @order_form.text_field_valid(:no_2, {:size => 6, :maxlength => 6}.update(text_options), {:min => 6}.update(js_options)) <<
            '-' <<
            @order_form.text_field_valid(:no_3, {:size => 5, :maxlength => 5}.update(text_options), {:min => 4}.update(js_options)) # 14桁or15桁が入力可
        else
          text_options = {:size => 4, :maxlength => 4, :style => "ime-mode: disabled"}.update(options || {})
          setup_class(text_options, 'card_type_normal_input') # このクラス名で表示／非表示の制御を行います。
          js_options = {:numeric => true, :min => 4, :realtime => true, :update => 'card_no_err_msg', :error => @name_for_error}
          @order_form.text_field_valid(:no_1, text_options, js_options) <<
            '-' <<
            @order_form.text_field_valid(:no_2, text_options, js_options) <<
            '-' <<
            @order_form.text_field_valid(:no_3, text_options, js_options) <<
            '-' <<
            @order_form.text_field_valid(:no_4, text_options, js_options)
        end
      end

      def example(options = {}, &block)
        options = {:tag_name => :p}.update(options || {})
        setup_class(options, 'sample')
        tag_name = options.delete(:tag_name)
        @template.concat(@template.tag(tag_name, options, true))
        @template.concat(@template.capture(&block))
        @template.concat("</#{tag_name}>")
      end

      def setup_class(options, additional_value)
        options[:class] ||= ''
        options[:class] << ' ' << additional_value
      end

    end

    desc <<-EOS
        [概要：お届け先指定・お支払方法用のカード番号を入力する入力フィールドとその入力例を出力します。注文用のフォーム内でのみ使用できます。]
        [引数 card_kind(必須)：対象となるカードの種類を指定します。指定できるのは、特定のカードを指定しないnormalとvisa,master,amex,diners,jcb]
        [引数 tag_name(任意)：カードの入力フィールドとその入力例の領域のHTMLタグの名称をしていします。デフォルトはdivです。]
        [引数 (オプション)：カードの入力フィールドとその入力例の領域のHTMLオプションを指定します。]
        [使用例：
          <ec:order_form_card_no card_kind="normal" name_for_error="カード番号">
            <ec:order_form_card_no_field/>
            <ec:order_form_card_no_example>
              例：1234-5678-9000-0000 ※16桁
            </ec:order_form_card_example>
          </ec:order_form_card_no>

          <ec:order_form_card_no card_kind="amex" name_for_error="カード番号">
            <ec:order_form_card_no_field/>
            <ec:order_form_card_no_example>
              例：1234-567890-0000 ※14桁、1234-567890-00000 ※15桁
            </ec:order_form_card_no_example>
          </ec:order_form_card_no>

          <ec:order_form_card_no_error_message/>
    EOS
    def order_form_card_no(*args, &block)
      raise UsageError, "ec:order_form_card_noはec:order_formの中で使用してください" unless @order_form
      card_kind, tag_name, options = extract(args, :card_kind, :tag_name => :div)


      if (card_kind != 'normal') and !OrderForm.card_kind_ids.include?(card_kind)
        raise ArgumentError, "#{card_kind}は不正なcard_kindです。card_kindには以下のいづれかを指定してください。normal, #{OrderForm.card_kind_ids.join(', ')}"
      end
      options[:class] ||= ''
      options[:class] << ' ' << "card_no_#{card_kind}"

      if card_kind != "normal"
        options[:style] ||= ''
        options[:style] << ';display:none;' # 画面を開いたときの、表示／非表示の制御
      end

      @order_form_card_no = CardNoFieldAndExample.new(@template, @order_form, card_kind, options['name_for_error'] || "card no")
      begin
        @template.concat(@template.tag(tag_name, options, true))
        @template.concat(@template.capture(&block))
        @template.concat("</#{tag_name}>")
      ensure
        @order_form_card_no = nil
      end
    end
      

    desc <<-"EOS"
        [概要：お届け先指定・お支払方法用のカード番号を入力するフィールドカードのを出力します。出力するフィールドの数はカードの種類によります。お届け先指定・お支払方法用のフォーム内でのみ使用できます。]
        [引数 (オプション)：各入力フィールドのHTMLオプションを指定します。]
        [使用例：ec:order_form_card_noを参照してください。]
    EOS
    def order_form_card_no_field(*args, &block)
      raise UsageError, "ec:order_form_card_no_fieldsはec:order_form_card_noの中で使用してください" unless @order_form_card_no
      @order_form_card_no.fields(*args, &block)
    end

    desc <<-"EOS"
        [概要：お届け先指定・お支払方法用のカード番号の入力例を出力する領域を表示します。お届け先指定・お支払方法用のフォーム内でのみ使用できます。]
        [引数 (オプション)：入力例の領域のHTMLオプションを指定します。]
        [使用例：ec:order_form_card_noを参照してください。]
    EOS
    def order_form_card_no_example(*args, &block)
      raise UsageError, "ec:order_form_card_no_fieldsはec:order_form_card_noの中で使用してください" unless @order_form_card_no
      @order_form_card_no.example(*args, &block)
    end

    desc <<-"EOS"
        [概要：お届け先指定・お支払方法用のカード番号の入力エラーを出力する領域を表示します。お届け先指定・お支払方法用のフォーム内でのみ使用します。]
        [引数 tag_name(任意)：入力エラーを出力する領域のHTMLタグの名称をしていします。デフォルトはspanです。]
        [引数 (オプション)：入力エラーを出力する領域のHTMLオプションを指定します。]
        [使用例：ec:order_form_card_noを参照してください。]
    EOS
    def order_form_card_no_error_message(*args, &block)
      tag_name, options = extract(args, :tag_name => :span)
      options[:id] = "card_no_err_msg"
      @template.concat(@template.tag(tag_name, options, true))
      @template.concat(@template.capture(&block))
      @template.concat("</#{tag_name}>")
    end


    desc <<-EOS
        [概要：お届け先指定・お支払方法用の有効期限（年）を入力する入力フィールドを出力します。お届け先指定・お支払方法用のフォーム内でのみ使用できます。]
        [引数: :name_for_error(任意)：：入力エラーに表示する項目名]
        [使用例：ec:order_formを参照してください。]
        ]
    EOS
    def order_form_expire_year(options = {})
      raise UsageError, "ec:order_form_expire_yearはec:order_formの中で使用してください" unless @order_form
      options = {:maxLength => 2, :size => 2, :style=>"ime-mode: disabled"}.update(options || {})
      validation_option = {:numeric => true, :min => 2, :realtime => true, :update=>'expire_err_msg', :error => I18n.t(:"activerecord.attributes.order_form.expire_year") || 'expire expire_year'}
      @order_form.text_field_valid(:expire_year, options, validation_option)
    end

    desc <<-EOS
        [概要：お届け先指定・お支払方法用の有効期限（月）を入力する入力フィールドを出力します。お届け先指定・お支払方法用のフォーム内でのみ使用できます。]
        [引数： name_for_error(任意)：入力エラーに表示する項目名]
        [使用例：ec:order_formを参照してください。]
    EOS
    def order_form_expire_month(options = {})
      raise UsageError, "ec:order_form_expire_monthはec:order_formの中で使用してください" unless @order_form
      options = {:maxLength => 2, :size => 2, :style=>"ime-mode: disabled"}.update(options || {})
      validation_option = {:numeric => true, :min => 2, :realtime => true, :update=>'expire_err_msg', :error => I18n.t(:"activerecord.attributes.order_form.expire_month") || 'expire expire_month'}
      @order_form.text_field_valid(:expire_month, options, validation_option)
    end


    desc <<-EOS
        [概要：お届け先指定・お支払方法用のカード名義の入力フィールドを出力します。お届け先指定・お支払方法用のフォーム内でのみ使用できます。]
        [引数:なし]
        [使用例：ec:order_formを参照してください。]
        ]
    EOS
    def order_form_card_name(options = {})
      raise UsageError, "ec:order_form_card_nameはec:order_formの中で使用してください" unless @order_form

      options = {:maxlength=>20, :size => 20, :style => "ime-mode: inactive"}.update(options || {})
      validation_option = {:empty => true, :alpha => true, :update => 'card_name_err_msg', :error => options[:name_for_error] || 'card name' }
      @order_form.text_field_valid(:card_name, options, validation_option)
    end

    desc <<-EOS
        [概要：店舗設定でセキュリティコードの利用設定がされている場合のみ中身を表示します。]
        [引数:なし]
        [使用例：
          {{{
            <ec:if_using_security_code>
              ※クレジットカード裏面に印字されている末尾3桁または4桁の数字を記入してください。
              セキュリティコード：<ec:order_form_card_security_code/>
            </ec:if_using_security_code>
          }}}
        ]
    EOS
    def if_using_security_code(&block)
      return "" unless ShopSetting.get_setting(:security_code, I18n.locale) == "true"
      @template.concat(@template.capture(&block))
    end

    desc <<-EOS
        [概要：お届け先指定・お支払方法用のカードセキュリティコードの入力フィールドを出力します。
お届け先指定・お支払方法用のフォーム内でのみ使用できます。
店舗情報設定でセキュリティコードを利用しない設定にした場合は、このタグを使用していても中身を表示しません。]
        [引数:なし]
        [使用例：ec:if_using_security_codeを参照してください。]
        ]
    EOS
    def order_form_card_security_code(options = {})
      raise UsageError, "ec:order_form_card_security_codeはec:order_formの中で使用してください" unless @order_form
      return "" unless ShopSetting.get_setting(:security_code, I18n.locale) == "true"
      options = {:maxlength=>4, :size => 4, :style => "ime-mode: inactive"}.update(options || {})
      validation_option = {:empty => true, :alpha => true, :update => 'card_security_code_err_msg', :error => options[:name_for_error] || 'card security_code' }
      @order_form.text_field_valid(:card_security_code, options, validation_option)
    end


    desc <<-EOS
        [概要：お届け先指定・お支払方法用の自由記入欄の入力エリアを出力します。お届け先指定・お支払方法用のフォーム内でのみ使用できます。]
        [引数:なし]
        [使用例：ec:order_formを参照してください。]
        ]
    EOS
    def order_form_free_request(options = {})
      raise UsageError, "ec:order_form_free_requestはec:order_formの中で使用してください" unless @order_form
      options = {:cols => "88", :rows => "10", :class => "remark"}.update(options || {})
      @order_form.text_area(:free_request, options)
    end

    desc <<-EOS
        [概要:お届け先指定・お支払方法入力確認用のフォームを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>

           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form(&block)
      raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order)
      @template.form_for(:user, @template.instance_variable_get(:@user), :url => {:controller => :order, :action => :create}, :html => {:id => "order_confirmation_form" }) do |f|
        @template.fields_for(:user_address, @template.instance_variable_get(:@user_address)) do |f1|
          @template.fields_for(:user_email, @template.instance_variable_get(:@user_email)) do |f2|
            @template.fields_for(:order, @template.instance_variable_get(:@order)) do |f3|
              @user = f
              @user_address = f1
              @user_email = f2
              @order = f3
              @shipping_address = f3.object.order_shippings.first # 一注文につき、配送先は一つ
              begin
                @template.concat(@template.capture(&block))
              ensure
                @user = nil
                @user_address = nil
                @user_email = nil
                @order = nil
                @shipping_address = nil
              end
            end
          end
        end
      end
    end
    
    desc <<-EOS
        [概要:{{{在庫不足エラーの場合に<ec:if_stock_error>～</ec:if_stock_error>の中が実行されます。お届け先指定・お支払方法入力確認画面内で使用してください。}}}]
        [引数:なし]
        [使用例:
           {{{
             <ec:if_stock_error>
               <ec:stock_error_messages/> は終了しました。注文処理を完了させるには「注文する」ボタンを押してください。
             </ec:if_stock_error>
           }}}
        ]
    EOS
    def if_stock_error
      if @template.instance_variable_get(:@stock_error) 
        yield if block_given?
      end  
    end
    
    desc <<-EOS
        [概要:注文時に在庫不足エラーとなったプレゼント商品があれば商品名を表示します。お届け先指定・お支払方法入力確認画面内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
             <ec:stock_error_messages/>
          　　==> トートバッグ
           }}}
        ]
    EOS
    def stock_error_messages
      @template.instance_variable_get(:@stock_error)    
    end

    desc <<-EOS
        [概要:{{{販売期間エラーの場合に<ec:if_sales_period_error>～</ec:if_sales_period_error>の中が実行されます。お届け先指定・お支払方法入力確認画面内で使用してください。}}}]
        [引数:なし]
        [使用例:
           {{{
             <ec:if_sales_period_error>
               <ec:stock_error_messages/> は終了しました。注文処理を完了させるには「注文する」ボタンを押してください。
             </ec:if_sales_period_error>
           }}}
        ]
    EOS
    def if_sales_period_error
      if @template.instance_variable_get(:@sales_period_error) 
        yield if block_given?
      end  
    end
    
    desc <<-EOS
        [概要:注文時に在庫不足エラーとなったプレゼント商品があれば商品名を表示します。お届け先指定・お支払方法入力確認画面内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
             <ec:sales_period_error_messages/>
          　　==> トートバッグ
           }}}
        ]
    EOS
    def sales_period_error_messages
      @template.instance_variable_get(:@sales_period_error)
    end

    desc <<-EOS
        [概要:お客様住所の都道府県以下の入力内容を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_full_address/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_full_address
      raise UsageError, "ec:order_confirmation_form_full_addressはec:order_confirmation_formの中で使用してください" unless @user_address
      @template.print_address(@user_address.object, :include_zipcode => true)
    end

    desc <<-EOS
        [概要:お客様のメールアドレスを出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_email/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_email
      raise UsageError, "ec:order_confirmation_form_emailはec:order_confirmation_formの中で使用してください" unless @user_email
      ERB::Util.h(@user_email.object.email)
    end

    desc <<-EOS
        [概要:お客様の性別を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_gender_name/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_gender_name
      raise UsageError, "ec:order_confirmation_form_gender_nameはec:order_confirmation_formの中で使用してください" unless @user
      @user.object.sex_name
    end

    desc <<-EOS
        [概要:お客様の生年月日を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_birth_day/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_birth_day
      raise UsageError, "ec:order_confirmation_form_birth_dayはec:order_confirmation_formの中で使用してください" unless @user
      "#{@user.object.birth_year}/#{@user.object.birth_month}/#{@user.object.birth_day}" if @user.object.birth_day
    end

    desc <<-EOS
        [概要:お客様の職業を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_job_div_name/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_job_div_name
      raise UsageError, "ec:order_confirmation_form_job_div_nameはec:order_confirmation_formの中で使用してください" unless @user
      @user.object.job_div_name
    end

    desc <<-EOS
        [概要:支払い方法の入力内容を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_payment_type/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_payment_type
      raise UsageError, "ec:order_confirmation_form_payment_typeはec:order_confirmation_formの中で使用してください" unless @shipping_address
      @shipping_address.payment_type_div_name
    end

    desc <<-EOS
        [概要:「ギフト」チェックボックスがチェックされていたら、中身を実行します。ec:order_confirmation_form内で使用してください。]
        [引数: なし]
        [使用例:
           {{{
           <ec:if_gift>
             ギフト
           </ec:if_gift>
           }}}
        ]
    EOS
    def if_gift
      raise UsageError, "ec:if_giftはec:order_confirmation_formの中で使用してください" unless @order
      if @order.object.gift_flg
        yield if block_given?
      end
    end

    desc <<-EOS
        [概要:「ギフト」チェックボックスがチェックされていなければ、中身を実行します。ec:order_confirmation_form内で使用してください。]
        [引数: なし]
        [使用例:
           {{{
           <ec:unless_gift>
             送り状に購入者名を表示しません。
           </ec:unless_gift>
           }}}
        ]
    EOS
    def unless_gift
      raise UsageError, "ec:unless_giftはec:order_confirmation_formの中で使用してください" unless @order
      unless @order.object.gift_flg
        yield if block_given?
      end
    end

    desc <<-EOS
        [概要:「領収書・納品書を別送する」チェックボックスがチェックされていたら、中身を実行します。ec:order_confirmation_form内で使用してください。]
        [引数: なし]
        [使用例:
           {{{
           <ec:if_send_invoice_separately>
             商品に領収書・納品をを同梱しません。
           </ec:if_send_invoice_separately>
           }}}
        ]
    EOS
    def if_send_invoice_separately
      raise UsageError, "ec:if_send_invoice_separatelyはec:order_confirmation_formの中で使用してください" unless @order
      if @order.object.send_invoice_separately_flg
        yield if block_given?
      end
    end

    desc <<-EOS
        [概要:「領収書・納品書を別送する」チェックボックスがチェックされていなければ、中身を実行します。ec:order_confirmation_form内で使用してください。]
        [引数: なし]
        [使用例:
           {{{
           <ec:unless_send_invoice_separately>
             商品に領収書・納品をを同梱します。
           </ec:unless_send_invoice_separately>
           }}}
        ]
    EOS
    def unless_send_invoice_separately
      raise UsageError, "ec:unless_send_invoice_separatelyはec:order_confirmation_formの中で使用してください" unless @order
      unless @order.object.send_invoice_separately_flg
        yield if block_given?
      end
    end

    desc <<-EOS
        [概要:「送り状に購入者名を表示する」チェックボックスがチェックされていたら、中身を実行します。ec:order_confirmation_form内で使用してください。]
        [引数: なし]
        [使用例:
           {{{
           <ec:if_write_sender>
             送り状に購入者名を表示します。
           </ec:if_write_sender>
           }}}
        ]
    EOS
    def if_write_sender
      raise UsageError, "ec:if_write_senderはec:order_confirmation_formの中で使用してください" unless @order
      if @order.object.write_sender_flg
        yield if block_given?
      end
    end

    desc <<-EOS
        [概要:「送り状に購入者名を表示する」チェックボックスがチェックされていなければ、中身を実行します。ec:order_confirmation_form内で使用してください。]
        [引数: なし]
        [使用例:
           {{{
           <ec:unless_write_sender>
             送り状に購入者名を表示しません。
           </ec:unless_write_sender>
           }}}
        ]
    EOS
    def unless_write_sender
      raise UsageError, "ec:unless_write_senderはec:order_confirmation_formの中で使用してください" unless @order
      unless @order.object.write_sender_flg
        yield if block_given?
      end
    end

    desc <<-EOS
        [概要:配送希望日を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_delivery_date format="%y年%m月%d日"/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_delivery_date(*args)
      raise UsageError, "ec:order_confirmation_form_delivery_dateはec:order_confirmation_formの中で使用してください" unless @order
      format= extract(args, :format => '%y/%m/%d')
      @shipping_address.arrival_date.strftime("#{format}") if @shipping_address.arrival_date
    end
    
    desc <<-EOS
        [概要:配送希望時間を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_delivery_time/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_delivery_time
      raise UsageError, "ec:order_confirmation_form_delivery_timeはec:order_confirmation_formの中で使用してください" unless @order
      @shipping_address.arrival_time_display(I18n.locale.to_s)
    end
    
    desc <<-EOS
        [概要:自由記入・備考欄の入力内容を出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_free_request/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_free_request
      raise UsageError, "ec:order_confirmation_form_free_requestはec:order_confirmation_formの中で使用してください" unless @order
      @template.add_br_tag(@order.object.free_request)
    end

    desc <<-EOS
        [概要:配送先住所の都道府県以下の入力内容をフルネームで出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_shipping_address/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_shipping_address
      raise UsageError, "ec:order_confirmation_form_shipping_addressはec:order_confirmation_formの中で使用してください" unless @shipping_address
      @template.print_address(@shipping_address, :include_zipcode => true)
    end

    desc <<-EOS
        [概要:配送先住所の姓名の入力内容をフルネームで出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_shipping_full_name/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_shipping_full_name
      raise UsageError, "ec:order_confirmation_form_shipping_full_nameはec:order_confirmation_formの中で使用してください" unless @shipping_address
      ERB::Util.h(@shipping_address.fullname)
    end

    desc <<-EOS
        [概要:配送先住所が日本フォーマットによる入力ならば、中身を実行します。。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:if_shipping_address_japanese_format>
               <ec:order_confirmation_form_shipping_kana_last_name/>
               <ec:order_confirmation_form_shipping_kana_first_name/>
             </ec:if_shipping_address_japanese_format>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def if_shipping_address_japanese_format
      raise UsageError, "ec:if_shipping_address_japanese_formatはec:order_confirmation_formの中で使用してください" unless @shipping_address
      if @shipping_address.is_japanese_format?
        yield if block_given?
      end
    end

    desc <<-EOS
        [概要:配送先住所のカナ姓の入力内容をフルネームで出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_shipping_kana_last_name/>
             <ec:order_confirmation_form_shipping_kana_first_name/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_shipping_kana_last_name
      raise UsageError, "ec:order_confirmation_form_shipping_kana_last_nameはec:order_confirmation_formの中で使用してください" unless @shipping_address
      ERB::Util.h(@shipping_address.kana_name_last)
    end

    desc <<-EOS
        [概要:配送先住所のカナ名の入力内容をフルネームで出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_shipping_kana_last_name/>
             <ec:order_confirmation_form_shipping_kana_first_name/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_shipping_kana_first_name
      raise UsageError, "ec:order_confirmation_form_shipping_kana_first_nameはec:order_confirmation_formの中で使用してください" unless @shipping_address
      ERB::Util.h(@shipping_address.kana_name_first)
    end

    desc <<-EOS
        [概要:配送先住所の国の入力内容をフルネームで出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_shipping_country/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_shipping_country
      raise UsageError, "ec:order_confirmation_form_shipping_countryはec:order_confirmation_formの中で使用してください" unless @shipping_address
      ERB::Util.h(@shipping_address.country.name)
    end

    desc <<-EOS
        [概要:配送先住所の電話番号の入力内容をフルネームで出力します。ec:order_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_confirmation_form>
             <ec:order_confirmation_form_shipping_tel_num/>
           </ec:order_confirmation_form>
           }}}
        ]
    EOS
    def order_confirmation_form_shipping_tel_num
      raise UsageError, "ec:order_confirmation_form_shipping_tel_numはec:order_confirmation_formの中で使用してください" unless @shipping_address
      ERB::Util.h(@shipping_address.tel_num)
    end

  end
end
end
