# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module ShippingTag
      extend MethodDescription

      desc <<-EOS
        [概要:存在しない配送先にアクセスがあった場合、中身を表示します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_shipping_address_error>
             <font color="red">ご指定の配送先住所は存在しません。</font>
           </ec:if_shipping_address_error>
           }}}
        ]
      EOS
      def if_shipping_address_error(&block)
        @template.concat(@template.capture(&block)) unless @template.instance_variable_get(:@error_flg).nil?
      end

      desc <<-EOS
        [概要:配送先一覧を出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_list>
             <ec:order_form_shipping_radio_button />
             <ec:shipping_fullname />
             <ec:shipping_countryname />
             <ec:shipping_zipcode />
             <ec:shipping_address />
             <ec:shipping_telnum />
             <ec:shipping_cost />
           </ec:shipping_list>
           }}}
        ]
      EOS
      def shipping_list(&block)
        @shipping_addresses = @template.instance_variable_get(:@shipping_addresses)
        raise UsageError, "配送先が登録されていません" unless @shipping_addresses

        _each_with_index(:@shipping_addresses, :@shipping_address, :@shipping_address_index) do
          @template.concat(@template.capture(&block))
        end
      end

      desc <<-EOS
        [概要:配送先の名前を表示します。配送先一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:shipping_listを参照してください。]
      EOS
      def shipping_fullname
        raise UsageError, "ec:shipping_fullnameはec:shipping_listの中で使用してください" unless @shipping_address
        @shipping_address.fullname
        if @shipping_address.is_japanese_format? && @shipping_address.kana_name_last && @shipping_address.kana_name_first
          "#{ERB::Util.h(@shipping_address.fullname)}<br/>（#{ERB::Util.h(@shipping_address.kana_name_last)}　#{ERB::Util.h(@shipping_address.kana_name_first)}）"
        else
          ERB::Util.h(@shipping_address.fullname)
        end
      end

      desc <<-EOS
        [概要:配送先の国を表示します。配送先一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:shipping_listを参照してください。]
      EOS
      def shipping_countryname
        raise UsageError, "ec:shipping_countrynameはec:shipping_listの中で使用してください" unless @shipping_address
        @shipping_address.country.name
      end

      desc <<-EOS
        [概要:配送先の郵便番号を表示します。配送先一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:shipping_listを参照してください。]
      EOS
      def shipping_zipcode
        raise UsageError, "ec:shipping_zipcodeはec:shipping_listの中で使用してください" unless @shipping_address
        zip_code_mark = @shipping_address.country.name.downcase.eql?("japan")? "〒" : ""
        ERB::Util.h("#{zip_code_mark}#{@shipping_address.zip_code}")
      end

      desc <<-EOS
        [概要:配送先の住所を表示します。配送先一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:shipping_listを参照してください。]
      EOS
      def shipping_address
        raise UsageError, "ec:shipping_addressはec:shipping_listの中で使用してください" unless @shipping_address
        @template.print_address(@shipping_address, :inline => true)
      end

      desc <<-EOS
        [概要:配送先の電話番号を表示します。配送先一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:shipping_listを参照してください。]
      EOS
      def shipping_telnum
        raise UsageError, "ec:shipping_telnumはec:shipping_listの中で使用してください" unless @shipping_address
        ERB::Util.h(@shipping_address.tel_num)
      end

      desc <<-EOS
        [概要:配送先の送料を表示します。配送先一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:shipping_listを参照してください。]
      EOS
      def shipping_cost
        raise UsageError, "ec:shipping_costはec:shipping_listの中で使用してください" unless @shipping_address
        cart = session[:cart]
        return if cart.nil?
        return ShippingCost.calc_shipping_cost(@shipping_address, cart.total_shipping_cost_point, cart.free_shipping_cost?)
      end

      desc <<-EOS
        [概要:配送先編集ボタンを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:button_to_shipping_edit />
           }}}
        ]
      EOS
      def button_to_shipping_edit(options = {}, &block)
        raise UsageError, "ec:button_to_shipping_editはec:shipping_listの中で使用してください" unless @shipping_address
        @template.form_for(:user_address, @shipping_address, :url => {:controller => 'member', :action => :change_address}, :html => {:id => "shipping_for_edit_form" }) do |f|
          @user_address = f
          begin
            @template.concat(@user_address.hidden_field(:id))
            options = {:class => "zero_button"}.update(options || {})
            submit(options) do
              @template.capture(&block)
            end
          ensure
            @user_address = nil
          end
        end
      end

      desc <<-EOS
        [概要:配送先削除ボタンを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:button_to_shipping_destoroy />
           }}}
        ]
      EOS
      def button_to_shipping_destroy(options = {}, &block)
        raise UsageError, "ec:button_to_shipping_destroyはec:shipping_listの中で使用してください" unless @shipping_address
        return unless @shipping_address.address_div_key == :ship_to
        @template.form_for(:user_address, @shipping_address, :url => {:controller => 'member', :action => :delete_address}, :html => {:id => "shipping_for_destroy_form" }) do |f|
          @user_address = f
          begin
            @template.concat(@user_address.hidden_field(:id))
            submit(options) do
              @template.capture(&block)
            end
          ensure
            @user_address = nil
          end
        end
      end

      desc <<-EOS
        [概要:配送一覧を挿入します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_form>
             <ec:include_shipping_address_list />
           </ec:order_form>
           }}}
        ]
      EOS
      def include_shipping_address_rows
        raise UsageError, "ec:shipping_address_listはec:order_formの中で使用してください" unless @order_form
        @template.render(:partial => "order/shipping_address_rows")
      end

      desc <<-EOS
        [概要:配送先入力フォームの閉じるボタンを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <div id="shipping_address_area" style="display:none">
             <ec:button_to_close_shipping_form>×閉じる</ec:button_to_close_shipping_form>
           </div>
           }}}
        ]
      EOS
      def button_to_close_shipping_form(*args, &block)
        button_name = @template.capture(&block) if block_given?
        result = @template.button_to_function(button_name,
          :onclick => "new Effect.Fade('shipping_address_area');")
        @template.concat(result) if result && block_given?
      end

      desc <<-EOS
        [概要:配送先の新規追加ボタンを出力します。]
        [使用例:
           {{{
           <ec:button_to_new_shipping :ajax => "true">
             新規配送先を入力する
           </ec:button_to_new_shipping>
           }}}
        ]
      EOS
      def button_to_new_shipping(*args, &block)
        button_name = @template.capture(&block) if block_given?

        ajax, options = extract(args, :ajax => "true")
        result = nil
        if ajax == "true"
          # Ajaxを使用する場合
          result = @template.button_to_remote(button_name,
            :url => {:action => :new_shipping_address},
            :method => :get)
        else
          # Ajaxを使用しない場合（画面遷移の場合）

        end
        @template.concat(result) if result && block_given?
      end

      desc <<-EOS
        [概要:配送先の新規登録ボタンを出力します。]
        [使用例:
           {{{
           <ec:button_to_create_shipping :ajax => "true">
             新規配送先を入力する
           </ec:button_to_create_shipping>
           }}}
        ]
      EOS
      def button_to_create_shipping(*args, &block)
        button_name = @template.capture(&block) if block_given?
        ajax, options = extract(args, :ajax => "true")
        result = nil
        if ajax == "true"
          # Ajaxを使用する場合
          options[:id] = 'button_to_create_shipping'
          options[:class] ||= ''
          options[:class] << ' ' << 'visible_when_new'
          options[:style] ||= ''
          options[:style] << ';' << 'display:none;'
          result = @template.button_to_remote(button_name, {
              :url => {:controller => :order, :action => :create_shipping_address},
              :update => "shipping_address_error_messages",
              :with => "Form.serialize($('order_form'))",
              # 出力されるJSではevalScriptsオプションが指定されているのにも関わらず、レスポンス中のscriptタグ内の
              # JSをevalしてくれないので、complete時に自前でevalを行います。
              :loading => 'disable_create_submit_button()',
              :complete => "render_create_response();"},
            options)
          js_func = button_to_remote_js_func('create', 'button_to_create_shipping')
        else
          # Ajaxを使用しない場合（画面遷移の場合）
        end
        if result && block_given?
          @template.concat(js_func)
          @template.concat(result)
        end
      end

      desc <<-EOS
        [概要:配送先の編集ボタンを出力します。]
        [引数 ajax(任意):Ajaxの使用を指定します。指定しない場合はAjaxを使用せず画面遷移して配送先を編集します。]
        [使用例:
           {{{
           <ec:button_to_edit_shipping :ajax => "true">
             編集する
           </ec:button_to_edit_shipping>
           }}}
        ]
      EOS
      def button_to_edit_shipping(*args, &block)
        raise UsageError, "ec:button_for_edit_shippingはec:shipping_listの中で使用してください" unless @shipping_address

        button_name = @template.capture(&block) if block_given?
        ajax, options = extract(args, :ajax => "false")
        result = nil
        if ajax == "true"
          # Ajaxを使用する場合
          result = @template.button_to_remote(button_name,
            :url => {:action => :edit_shipping_address, :id => @shipping_address_index.to_s},
            :method => :get)
        else
          # Ajaxを使用しない場合（画面遷移の場合）
          result = @template.button_to_url(button_name, {:controller => 'member', :action => 'change_address', :id => @shipping_address.id }, options)
        end
        @template.concat(result) if result && block_given?
      end

      desc <<-EOS
        [概要:配送先の更新ボタンを出力します。]
        [引数 ajax(任意):Ajaxの使用を指定します。指定しない場合はAjaxを使用せず画面遷移して配送先を編集します。デフォルトはtrueです。]
        [使用例:
           {{{
           <ec:button_to_update_shipping :ajax => "true">
             編集する
           </ec:button_to_update_shipping>
           }}}
        ]
      EOS
      def button_to_update_shipping(*args, &block)
        #  raise UsageError, "ec:button_for_edit_shippingはec:shipping_listの中で使用してください" unless @shipping_address

        button_name = @template.capture(&block) if block_given?
        ajax, options = extract(args, :ajax => "false")
        result = ""
        if ajax == "true"
          # Ajaxを使用する場合

          # 更新対象の配送先を送出するhiddenフィールド
          @template.concat(@template.hidden_field_tag(:edit_target_index))
          # 更新ボタン
          options[:id] = 'button_to_update_shipping'
          options[:class] ||= ''
          options[:class] << ' ' << 'visible_when_edit'
          options[:style] ||= ''
          options[:style] << ';' << 'display:none;'
          result = @template.button_to_remote(button_name, {
              :url => {:controller => :order, :action => :update_shipping_address},
              :update => "shipping_address_error_messages",
              :with => "Form.serialize($('order_form'))",
              :loading => 'disable_update_submit_button()',
              :complete => "render_update_response();"},
            options)
          js_func = button_to_remote_js_func('update', 'button_to_update_shipping')
        else
          # Ajaxを使用しない場合（画面遷移の場合）
          result = @template.button_to_url(button_name, {:controller => 'member', :action => 'change_address', :id => @shipping_address.id }, options)
        end
        if result && block_given?
          @template.concat(js_func)
          @template.concat(result)
        end
      end

      private
      def button_to_remote_js_func(action_name, button_id)
        @template.javascript_tag(<<-"SCRIPT")
            function render_#{action_name}_response() {
              enable_#{action_name}_submit_button();
              request.responseText.evalScripts();
            }
            function disable_#{action_name}_submit_button() {
              $('#{button_id}').disabled = true;
            }
            function enable_#{action_name}_submit_button() {
              $('#{button_id}').disabled = false;
            }
        SCRIPT
      end

    end
  end
end
