# -*- coding: utf-8 -*-
class Campaign < ActiveRecord::Base
  versioned

  include SafeEval

  # その時点で有効なキャンペーン
  # priorityの大きい順にソートします。
  named_scope :availables, lambda {|*args| 
    t = args.first || Time.now
    {
      :conditions => ["started_at <= ? AND finished_at >= ?", t, t],
      :order => "priority desc, id asc"
    }
  }

  # キャンペーン条件
  has_many :conditions, :class_name => 'CampaignCondition', :foreign_key => 'campaign_id'
  accepts_nested_attributes_for :conditions, :allow_destroy => true

  # キャンペーン対象商品
  has_many :targets, :class_name => 'CampaignTargetProductSku', :foreign_key => 'campaign_id'
  has_many :target_skus, :through => :targets, :source => :product_sku
  accepts_nested_attributes_for :targets, :allow_destroy => true
  attr_accessor :targets_modified

  # キャンペーン効果
  has_many :effects, :class_name => 'CampaignEffect', :foreign_key => 'campaign_id'
  accepts_nested_attributes_for :effects, :allow_destroy => true

  # キャンペーン対象日時
  attr_writer :attached_at
  def attached_at
    @attached_at || Time.now
  end

  def effect_type_key
    effect = effects.first # １キャンペーン１効果という仕様です
    effect ? effect.type_key : nil
  end

  validates_presence_of :code
  validates_presence_of :name
  validates_presence_of :started_at, :finished_at
  validates_presence_of :priority
  validates_inclusion_of :priority, :in => 1..999
  validates_numericality_of :priority

  before_validation :set_conditions_no

  def set_conditions_no
    conditions.each_with_index do |condition, index|
      condition.no = index
    end
  end

  def validate
    super
    validate_priority_duplication_in_same_period
    validate_competing_point_campaign_in_same_period
  end

  private
  # 同じ期間で同じ優先度のキャンペーンを作成できないチェック
  def validate_priority_duplication_in_same_period
    where, params = [], []
    where << "(started_at <= ? AND finished_at >= ?)"; params.concat([self.started_at, self.finished_at])
    where << "(started_at >= ? AND finished_at <= ?)"; params.concat([self.started_at, self.finished_at])
    where << "(started_at <= ? AND finished_at > ?)"; params.concat([self.started_at, self.started_at])
    where << "(started_at < ? AND finished_at >= ?)"; params.concat([self.finished_at, self.finished_at])
    where = ['(%s)' % where.join(" OR ")]
    where << "priority = ?" ; params << self.priority
    unless new_record?
      where << "id <> ?" ; params << self.id
    end
    if Campaign.count(:conditions => [where.join(" AND ")].concat(params)) > 0
      errors.add_to_base("期間が重複する場合、優先度は同じ値を設定することができません。")
    end
  end

  # 同じ期間で明細に対するポイントキャンペーンと、カートに対する割引orポイントキャンペーンを作成できないチェック
  def validate_competing_point_campaign_in_same_period
    where, params = [], []
    where << "(started_at <= ? AND finished_at >= ?)"; params.concat([self.started_at, self.finished_at])
    where << "(started_at >= ? AND finished_at <= ?)"; params.concat([self.started_at, self.finished_at])
    where << "(started_at <= ? AND finished_at > ?)"; params.concat([self.started_at, self.started_at])
    where << "(started_at < ? AND finished_at >= ?)"; params.concat([self.finished_at, self.finished_at])
    where = ['(%s)' % where.join(" OR ")]
    unless new_record?
      where << "campaigns.id <> ?" ; params << self.id
    end

    check = false
    # 自身がカートに対する割引、またはポイントキャンペーンの場合
    if targets_cart? && (effect_type_key == :discount_percentage || effect_type_key == :discount_price || effect_type_key == :point)
      where << "EXISTS (SELECT * FROM campaign_target_product_skus t WHERE t.campaign_id = campaigns.id)"
      where << "campaign_effects.type_cd = ?"
      params << CampaignEffect.type_id_by_key(:point)
      check = true
    end
    # 自身が明細に対するポイントキャンペーンの場合
    if targets_items? && (effect_type_key == :point)
      where << "NOT EXISTS (SELECT * FROM campaign_target_product_skus t WHERE t.campaign_id = campaigns.id)"
      where << "campaign_effects.type_cd IN (?)"
      params << [CampaignEffect.type_id_by_key(:discount_percentage),CampaignEffect.type_id_by_key(:discount_price),CampaignEffect.type_id_by_key(:point)]
      check = true
    end

    if check && Campaign.count(:include => :effects, :conditions => [where.join(" AND ")].concat(params)) > 0
      errors.add_to_base("対象商品へのポイント増加と、全体へのポイント増加または割引を重複する期間に設定することはできません。")
    end
  end

  public

  def mark_targets_for_destruction(modified = targets_modified)
    if ActiveRecord::ConnectionAdapters::Column.value_to_boolean(modified)
      self.targets.each do |target|
        target.mark_for_destruction
      end
    end
  end

  # 条件に適合するかどうかチェックするスクリプトを生成する
  def build_conditions(options = nil)
    options = {
      :numbering_conditions => true
    }.update(options || {})
    numbering_conditions if options[:numbering_conditions]
    conditions.map{|condition| condition.build}.join(' ')
  end

  def numbering_conditions
    conditions.each_with_index do |condition, index|
      condition.no = index
    end
  end

  # カートがキャンペーンの対象となるかどうかを判断します。
  def match?(cart)
    match_to_target?(cart) && match_to_conditions_by_cart?(cart)
  end

  # キャンペーンの対象商品SKUをカートが持っているかどうかを判断します。
  # targetsが空かtargetsに該当し、かつ条件に合致する
  def match_to_target?(cart)
    return true if targets.empty?
    contained_ids = self.target_sku_ids & cart.items.map{|(key, item)| item.product_sku_id}
    !contained_ids.empty?
  end

  # 与えられたカートでこの条件を評価します
  # targetsが指定されている場合、どれか一つにマッチすればtrueを返します
  def match_to_conditions_by_cart?(cart)
    return match_to_conditions?(cart.to_campaign_context) if targets.empty?
    cart.each do |item|
      return true if match_to_conditions?(item.to_campaign_context)
    end
    return false
  end

  # 与えられたコンテキストでこの条件を評価します
  # 余計なアクセスなどを行わないように、スレッドの$SAFE(セーフレベル)を指定して、安全に実行できるようにしています。
  # スレッドは平行処理するためではないので、このメソッドはスレッドを実行して終了を待って戻り値を返します。
  def match_to_conditions?(context = {})
    return true if conditions.empty?
    script = build_conditions.untaint # 汚染マークを外さないと下のinstance_evalでSecurityErrorが発生する
    !!safe_instance_eval(script, context, 3)
  end

  # 渡されたカートがこのキャンペーンの適用対象ならば適用します。
  def apply_if_match(cart)
    apply(cart) if match?(cart)
  end
  
  # 渡されたカートがこのキャンペーンを適用します。
  def apply(cart)
    effects.each do |effect|
      effect.apply(cart)
    end
  end

  # 対象がカートの場合にtrueを返します
  # このメソッド名の"targets"は名詞の複数形ではなく、動詞の三人称単数形です。
  def targets_cart?
    targets.empty?
  end

  # 対象がカートの明細の場合にtrueを返します。
  # このメソッド名の"targets"は名詞の複数形ではなく、動詞の三人称単数形です。
  def targets_items?
    !targets_cart?
  end

  # 効果にプレゼントを含むキャンペーンで、プレゼント品が在庫切れならtrueを返します。
  def present_but_no_stock?(cart = nil)
    effects.any?{|effect| effect.present_but_no_stock?(cart)}
  end


  #
  # = キャンペーン一覧検索入力フォーム(管理画面用) =
  #
  class SearchConditionForm < EcRider::ConditionForm
    include SelectableAttr::Base
    attr_accessor :code, :name, :started_at, :finished_at, :type_cd, :target_div, :product_name, :product_code

    selectable_attr :target_div do
      entry '01', :exist      , '対象商品あり'
      entry '02', :not_exist  , '対象商品なし'
    end

    selectable_attr :type_cd, :enum => CampaignEffect.type_enum

    def to_find_options(options = nil)
      where, params = [], []

      unless code.blank?
        where << "code like ?"
        params << "%#{code}%"
      end
      unless name.blank?
        where << "name like ?"
        params << "%#{name}%"
      end
      unless started_at.blank?
        where << "started_at >= ?"
        params << date_to_time(started_at, 0, 0, 0)
      end
      unless finished_at.blank?
        where << "finished_at <= ?"
        params << date_to_time(finished_at, 23, 59, 59)
      end
      unless type_cd.blank?
        where << "campaign_effects.type_cd = ?"
        params << type_cd
      end
      select_target_product_skus = <<-EOS
        SELECT *
        FROM campaign_target_product_skus t
        JOIN product_skus s ON t.product_sku_id = s.id AND s.deleted_at IS NULL
        JOIN products p ON s.product_id = p.id AND p.deleted_at IS NULL
        WHERE t.campaign_id = campaigns.id
      EOS
      product_conditions = nil
      unless product_name.blank?
        product_conditions = "AND (p.order_product_name_1 like ? OR s.sku_code like ?)"
        params << "%#{product_name}%"
        params << "%#{product_name}%"
      end
      if target_div_key == :exist || product_conditions
        where << "EXISTS(#{select_target_product_skus} #{product_conditions} )"
      end
      if target_div_key == :not_exist
        where << "NOT EXISTS(#{select_target_product_skus})"
      end
      
      params.unshift(where.join(' AND ')) unless where.empty?
      result = {:order => "started_at desc, campaigns.id desc"}
      result[:conditions] = params unless params.empty?
      result[:include] = :effects
      result.update(options) if options
      return result
    end

    private
    def date_to_time(value, offset_h, offset_m, offset_s)
      case value
      when Time then
        value
      when Date then
        Time.new(value.year, value.month, value.day, offset_h, offset_m, offset_s)
      else
        Time.parse('%s %02d:%02d:%02d' % [value.to_s, offset_h, offset_m, offset_s])
      end
    end
  end
end
