# -*- coding: utf-8 -*-
require 'ec_rider/config'

class CmsPage < ActiveRecord::Base
  include EcRider::EcHelper
  include EcRider::I18nSupport

  belongs_to :cms_project, :foreign_key=> "cms_project_id"
  has_many :titles, :class_name => 'CmsPageTitle', :foreign_key => 'cms_page_id', :order => 'locale asc'
  has_many :binds, :class_name => 'CmsBind', :foreign_key => 'cms_page_id', :order => 'position'
  has_one :bind_top, :class_name => 'CmsBind', :foreign_key => 'cms_page_id', :conditions => ['position = ?', CmsBind.position_id_by_key(:top)]
  has_one :bind_bottom, :class_name => 'CmsBind', :foreign_key => 'cms_page_id', :conditions => ['position = ?', CmsBind.position_id_by_key(:bottom)]
  has_one :bind_left, :class_name => 'CmsBind', :foreign_key => 'cms_page_id', :conditions => ['position = ?', CmsBind.position_id_by_key(:left)]
  has_one :bind_right, :class_name => 'CmsBind', :foreign_key => 'cms_page_id', :conditions => ['position = ?', CmsBind.position_id_by_key(:right)]
  has_one :bind_center, :class_name => 'CmsBind', :foreign_key => 'cms_page_id', :conditions => ['position = ?', CmsBind.position_id_by_key(:center)]

  # Rails2.3.2のnested_form に移行しやすくなるように名前をつけています。
  attr_accessor :titles_attributes
  attr_accessor :binds_attributes
  attr_accessor :org_id

  selectable_attr :terminal, :enum => CmsTemplate.terminal_enum

  before_validation :set_controller, :downcase_action
  def set_controller
    self.controller = 'cms' if controller.blank?
  end

  def downcase_action
    self.action.downcase! unless action.blank?
  end

  validates_presence_of :action
  validates_uniqueness_of :action, :scope => [:cms_project_id, :controller, :terminal]

  # 2009/11/14、CmsPageのactionの先頭に_(アンダースコア)を使えるようにしました。
  # 本来のCmsPageの仕様としては矛盾していますが、partial用のテンプレートを扱うために仕方なくそのようになっています。
  # 2009/11/24のリリース後、再び_を先頭に使用できないように戻して、cms_templateのdisplay_positionには、
  # 上下左右中央に加えて、"中央の一部"という新たな区分値を設けて、ページとして扱えない部分テンプレートに対してCmsPageのインスタンスが作れないようにします。
  validates_format_of :action, :with => /^[a-z\_][a-z0-9\_]*$/, :if => Proc.new {|cms_page| cms_page.controller == 'cms'}

  # validateメソッド内で記述しているエラーメッセージは、ec_adminのみで使用するメッセージです
  def validate
    self.binds_attributes.each_value do |binds_attribute|
      template_str = "#{CmsBind.position_name_by_id(binds_attribute[:position])}テンプレート"

      # テンプレート必須チェック
      if binds_attribute[:position] == CmsBind.position_id_by_key(:center) && binds_attribute[:cms_template_id].blank?
        errors.add_to_base "中央テンプレートは必須です。"
      end
      binds_attribute[:items_attributes].each_value do |items_attribute|
        position_str = "バインド#{items_attribute[:no]}"

        # バインド必須チェック
        if items_attribute[:input_type].blank?
          errors.add_to_base("#{template_str} #{position_str}の内容を入力してください。")
        end

        if items_attribute[:input_type] == CmsBindItem.input_type_id_by_key(:content)
          items_attribute[:contents_attributes].each_value do |contents_attribute|
            # 日本語必須チェック
            if terminal_key == :pc && contents_attribute[:locale] == 'ja' && contents_attribute[:descriptions].blank?
              errors.add_to_base "#{template_str} #{position_str} 日本語の内容は必須です。"
            end
            if terminal_key == :mobile && contents_attribute[:locale] == 'docomo' && contents_attribute[:descriptions].blank?
              errors.add_to_base "#{template_str} #{position_str} NTTドコモの内容は必須です。"
            end
            locale_str = "#{locale_for_display(contents_attribute[:locale])}"

            if contents_attribute[:descriptions]
              # 文字数チェック
              if  contents_attribute[:descriptions].size > 65535
                errors.add_to_base("#{template_str} #{position_str} #{locale_str} 65535バイト以内で入力してください。")
              end

              # XMLの文法チェック
              begin
                CmsXml.generate_erb(contents_attribute[:descriptions])
              rescue REXML::ParseException
                errors.add_to_base("#{template_str} #{position_str} #{locale_str} XMLの記述が不正です。")
              end

              # ecタグとしての存在チェック
              # XML的にはOKなので、ここでは閉じタグはチェックしません。
              ec_tags = contents_attribute[:descriptions].scan(/\<ec:(.*?)[\/]{0,1}\>/m)
              ec_tags.flatten.compact.each do |ec_tag|
                ec_tag_parts = ec_tag.split(/\s/, 2)
                ec_tag_name = ec_tag_parts.first
                errors.add_to_base("#{template_str} #{position_str} #{locale_str} ec:#{ec_tag} は存在しないタグです。") unless ec(self.terminal_key).respond_to?(ec_tag_name)
              end
            end
          end if items_attribute[:contents_attributes]
        end
      end if binds_attribute[:items_attributes]
    end if binds_attributes
  end

  def validate_for_setup?
    self.titles.each do |title|
      if self.terminal_key == :pc && title.locale == 'ja' && title.name.blank?
        errors.add_to_base "日本語は必須です。"
        return false
      end
      if self.terminal_key == :mobile && title.locale == 'docomo' && title.name.blank?
        errors.add_to_base "NTTドコモは必須です。"
        return false
      end
    end
    return true
  end
  
  
  MISMATCH_URLS = {'/account/policy_information' => '/policy', '/cart/cart_index' => '/cart', '/inquiry/inquiry' => '/inquiry', 
    '/member/index' => '/member','/reminder/index' => '/reminder', '/resigned_reason/resigned_reason' => '/resigned_reason', 
    '/thanks/index' => '/thanks'}
      
  def real_local_path
    local_url = cms_shop_page? ? "/#{action}" : "/#{controller}/#{action}"
    local_url = MISMATCH_URLS[local_url] if MISMATCH_URLS.has_key?(local_url)
    return local_url
  end

  # titlesとitems(とそのcontents)と一緒に削除します。
  def destroy_with_titles_and_items
    self.titles.each do|title|
      title.destroy
    end
    self.binds.each do|bind|
      bind.items.each do |item|
        item.contents.each do |content|
          content.destroy
        end if item.contents
        item.destroy
      end if bind.items
      bind.destroy
    end
    self.destroy
  end


  #
  # =一覧の検索条件=
  # 一覧には、編集対象プロジェクトと編集対象外プロジェクト両方を表示します。
  # プロジェクト編集対象のページと、プロジェクト編集対象外かつ公開中のページを表示します。
  # コントローラ名とアクション名が同じページは、優先度が高いものを取得します。
  #
  class SearchCondition < EcRider::ConditionForm
    attr_accessor :page_name, :front_url, :template_name, :status_cd, :started_at, :finished_at, :edit_target, :cms_project_id, :terminal, :disp_count

    def page_list_ids(target_project_id)
      cms_pages = CmsPage.find(:all, :include => :cms_project, :order => 'controller, action',
        :conditions => ["cms_project_id = ? or (cms_project_id <> ? and cms_projects.status_cd = ?)",
          target_project_id, target_project_id, CmsProject.status_id_by_key(:opened)])

      url_id = {}
      deleted_ids = []
      delete_page_urls = []
      cms_pages.each do |page|
        replace_status = :keep # ページを置き換えるか、何もしないかを保持します :changeが入ると置き換えます。
        page_url = "#{page.controller}/#{page.action}"
        # 識別のため、モバイルの場合は便宜的に"_mobile"を付与する
        page_url << "_mobile" if page.terminal_key == :mobile
        if url_id[page_url].nil?
          # 同じページURLを持つページが他に無ければそれを採用する
          url_id[page_url] = page
        else
          comparing_project = page.cms_project # 比較中のページのプロジェクト
          stored_project = url_id[page_url].cms_project # code_idに格納されているページのプロジェクト

          # ターゲットプロジェクトで編集されたページがある場合、そのページを最優先とする
          if comparing_project.id == target_project_id
            if page.delete_at_publish
              deleted_ids << url_id[page_url].id
            else
              url_id[page_url] = page
            end
          else
            if stored_project.id == target_project_id
              # すでにurl_idにターゲットプロジェクトで編集済みのページが格納されている場合
              # 何もしない
            else
              # ターゲットのプロジェクトで未作成のページの場合
              # プロジェクトの優先度が高い（数値が大きい）ものを優先する
              # プロジェクトの優先度が同じ場合はプロジェクトの公開日時の遅いものを優先する
              # 公開日時も同じ場合は、プロジェクトのidの大きいものを優先する
              if comparing_project.status_key == :opened
                if comparing_project.priority == stored_project.priority
                  if comparing_project.started_at == stored_project.started_at
                    if comparing_project.id > stored_project.id
                      replace_status = :change
                    end
                  elsif comparing_project.started_at > stored_project.started_at
                    replace_status = :change
                  end
                elsif comparing_project.priority > stored_project.priority
                  replace_status = :change
                end
              end
            end
          end
        end
        # replace_statusが:changeの場合現在格納されているページを
        # 優先度の高いページに置き換える
        # 削除日時が含まれている場合は削除ページに追加する
        if replace_status == :change
          if page.delete_at_publish
            deleted_ids << url_id[page_url].id
            delete_page_urls << page_url
          else
            url_id[page_url] = page
          end
        end
      end
      # 公開中の公開削除中のページは一覧に表示しないためcode_idから削除します
      delete_page_urls.each do |delete_url|
        url_id.delete(delete_url)
      end
      [url_id.values.map{|cms_page| cms_page.id }, deleted_ids]
    end

    # join文で別名をつけているため、sqlを直接書きます。
    # 表示対象のプロジェクトが、公開時に削除フラグのあるページを持つ場合は、delete_at_publish_pjにプロジェクトIDを取得します。
    def create_query(options = nil)

      base_query = <<-"EOS"
        SELECT
          page_m.id, page_m.cms_project_id, page_m.controller, page_m.action,
          concat('/', page_m.controller, '/', page_m.action) front_url, page_m.stop_public, page_m.terminal,
          pj_m.status_cd, pj_m.started_at, pj_m.finished_at,
          case when cms_page_title_pc.name is not null then cms_page_title_pc.name else cms_page_title_mb.name end page_name,
          template_t.name top_template_name, template_b.name bottom_template_name, template_l.name left_template_name,
          template_r.name right_template_name, template_c.name center_template_name
        FROM
          cms_pages page_m left outer join cms_projects pj_m on page_m.cms_project_id = pj_m.id
          left outer join cms_page_titles cms_page_title_pc on page_m.id = cms_page_title_pc.cms_page_id and cms_page_title_pc.locale = ?
          left outer join cms_page_titles cms_page_title_mb on page_m.id = cms_page_title_mb.cms_page_id and cms_page_title_mb.locale = ?
          left outer join cms_binds bind_t on page_m.id = bind_t.cms_page_id and bind_t.position = ?
          left outer join cms_templates template_t on bind_t.cms_template_id = template_t.id
          left outer join cms_binds bind_b on page_m.id = bind_b.cms_page_id and bind_b.position = ?
          left outer join cms_templates template_b on bind_b.cms_template_id = template_b.id
          left outer join cms_binds bind_l on page_m.id = bind_l.cms_page_id and bind_l.position = ?
          left outer join cms_templates template_l on bind_l.cms_template_id = template_l.id
          left outer join cms_binds bind_r on page_m.id = bind_r.cms_page_id and bind_r.position = ?
          left outer join cms_templates template_r on bind_r.cms_template_id = template_r.id
          left outer join cms_binds bind_c on page_m.id = bind_c.cms_page_id and bind_c.position = ?
          left outer join cms_templates template_c on bind_c.cms_template_id = template_c.id
        WHERE
          1 = 1
      EOS

      base_params = ['ja', 'docomo'].concat(CmsBind.position_ids)

      query = [base_query]
      delete_ids = []
      if where = Condition.block do |c|
          # 優先順位を考慮して、表示するページのIDを取得する
          page_ids, delete_ids = page_list_ids(cms_project_id)
          page_ids << '0' if page_ids.empty?
          c.and "page_m.id", "in", page_ids
          c.and do |c1|
            c1.or("cms_page_title_pc.name", "like", "%#{page_name}%")
            c1.or("cms_page_title_mb.name", "like", "%#{page_name}%")
          end unless page_name.blank?
          unless front_url.blank?
            # URLをコントローラとアクションに分ける
            url_parts = front_url.split('/')
            url_parts.delete("")
            if url_parts.length == 2
              c.and "page_m.controller", "like", "%#{url_parts[0]}%"
              c.and "page_m.action", "like", "%#{url_parts[1]}%"
            else
              c.and do |c_url|
                c_url.or("page_m.controller", "like", "%#{url_parts[0]}%")
                c_url.or("page_m.action", "like", "%#{url_parts[0]}%")
              end
            end
          end
          c.and "pj_m.status_cd", status_cd unless status_cd.blank?
          c.and "pj_m.started_at", ">=", Time.parse(started_at + " 00:00:00") unless started_at.blank?
          c.and "pj_m.finished_at", "<=", Time.parse(finished_at + " 23:59:59") unless finished_at.blank?
          c.and "page_m.cms_project_id", cms_project_id if edit_target == "1"
          c.and "page_m.terminal", "=", "#{terminal}" unless terminal.blank?
          c.and do |c2|
            c2.or("template_t.name", "like", "%#{template_name}%")
            c2.or("sql", "template_b.name like '%#{template_name}%'")
            c2.or("sql", "template_l.name like '%#{template_name}%'")
            c2.or("sql", "template_r.name like '%#{template_name}%'")
            c2.or("sql", "template_c.name like '%#{template_name}%'")
          end unless template_name.blank?
        end
        query[0] << " and #{where.shift}"
        where.each do |w|
          query.push(w)
        end
      end
      base_params.reverse_each{|base_param| query.insert(1, base_param)}
      [query, delete_ids]
    end

  end

  def current_project?(project_id)
    cms_project_id.blank? ? true : cms_project_id.to_s == project_id.to_s
  end

  def cms_shop_page?
    controller.blank? || controller == 'cms'
  end

  def deletable?
    cms_shop_page? && cms_project.status_key != :opened
  end

  #
  # CmsPageと関連するオブジェクトに値をセットして返します。
  #
  def set_values_for_create
    self.titles_attributes.each_value do |titles_attribute|
      self.titles << CmsPageTitle.new(titles_attribute)
    end

    self.binds_attributes.each_value do |binds_attribute|
      bind = CmsBind.new(binds_attribute)
      bind.items_attributes.each_value do |items_attribute|
        item = CmsBindItem.new(items_attribute)
        item.code = 'content' if item.input_type == CmsBindItem.input_type_id_by_key(:content)
        item.contents_attributes.each_value do |contents_attribute|
          content = CmsBindContent.new(contents_attribute)
          content.descriptions = '' if item.input_type == CmsBindItem.input_type_id_by_key(:code)
          item.contents << content
        end if item.contents_attributes
        bind.items << item
      end if bind.items_attributes && !bind.items_attributes.empty?
      self.binds << bind
    end
  end

  #
  # = 商品詳細ページ選択セレクトボックスのオプションタグに出力する内容を取得する =
  #
  def self.action_options(options = {:product_code => nil})
    _options = []
    self.find(:all, :conditions => "controller = 'cms'", :order => :action).each do |cms_page|
      if options[:product_code]
        _options << ["/#{cms_page.action}?product_code=#{options[:product_code]}", "/#{cms_page.action}?product_code=#{options[:product_code]}"]
      else
        _options << ["/#{cms_page.action}", "/#{cms_page.action}"]
      end
    end
    return _options.uniq
  end

  #
  # 引数で指定したロケールのテンプレート内容がある場合にtrueを返します。
  #
  def previewable?(locale)
    return false unless bind_center
    bind_center.cms_template.contents.each do |content|
      if content.locale == locale
        return !content.descriptions.nil?
      end
    end if bind_center.cms_template
    false
  end

  def deploy_to_preview(locale)
    binds.each do |bind|
      next unless bind.cms_template
      filename = bind.published_filename(locale)
      cms_publish = CmsPublish.new(:target => :preview)
      cms_publish.setup_default_template_paths(cms_project_id)
      cms_action = CmsPublish::CmsAction.new(cms_publish, filename, locale)
      bind.overwrite(cms_action)
    end
  end
  
  # centerのhtml.erbに付加する、content_for(:layout_top)内の、render :partial => "xxx" の 
  # "xxx" を求めるHashです。
  def partial_paths(locale = :ja)
    partial_paths = {}
    (CmsBind.position_keys - [:center]).each do |position_key|
      cms_bind = self.binds.detect{|bind1| bind1.position_key == position_key}
      path = nil
      if cms_bind && cms_bind.cms_template
        path = cms_bind.published_filename(locale)
      end
      unless path.blank?
        path = path.dup
        path.gsub!(/\/_/, '/')
        path.sub!(/\.html\.erb$/, '')
      end
      partial_paths[position_key] = path
    end
    partial_paths
  end
    
  #
  # プレビューに必要なファイルをec_admin/publicにコピーします。
  #
  COPY_PROHIBITED_FILES_FOR_PREVIEW = ["public/robots.txt"]
  def copy_public_files_for_preview
    # ec_admin/public/projects/1以下のファイルをコピー
    cms_ecrider_files = CmsFile.find(:all,
      :conditions => ['(cms_project_id = ?) and delete_at_publish is null and public_file_path not in (?)',
        CmsProject::BASE_PROJECT_ID, COPY_PROHIBITED_FILES_FOR_PREVIEW],
      :order => 'public_file_path')
    cms_ecrider_files.each do |cms_ecrider_file|
      dest_file = File.join(EcRider.config.ec_admin_root, cms_ecrider_file.public_file_path)
      FileUtils.mkdir_p(File.dirname(dest_file)) unless File.exist?(File.dirname(dest_file))
      FileUtils.cp(cms_ecrider_file.pj_file_path, dest_file)
    end

    # ec_front/public以下を全てコピー
    src_files = Dir.glob(File.join(EcRider.config.ec_front_root, 'public', '**', "*"))
    reg_ex_for_dir = "^\/public\/ec_rider$"
    reg_ex_for_file = "^\/public\/ec_rider\/"
    src_files.each do |src_file|
      src_file_public_name = src_file.sub(EcRider.config.ec_front_root, '')
      unless src_file =~ /^\./ && COPY_PROHIBITED_FILES_FOR_PREVIEW.include?(src_file_public_name)
        dest_file = src_file.sub(EcRider.config.ec_front_root, EcRider.config.ec_admin_root)
        if File.directory?(src_file)
          next if src_file_public_name =~ /#{reg_ex_for_dir}|#{reg_ex_for_file}/
          FileUtils.cp_r(src_file, File.dirname(dest_file)) unless File.exist?(dest_file)
        else
          next if src_file_public_name =~  /#{reg_ex_for_file}/
          FileUtils.mkdir_p(File.dirname(dest_file)) unless File.exist?(File.dirname(dest_file))
          FileUtils.cp(src_file, dest_file)
        end
      end
    end

    # 対象プロジェクトの必要なファイルをコピー
    pj_files = CmsFile.find(:all,
      :conditions => ['(cms_project_id = ?) and delete_at_publish is null and public_file_path not in (?)',
        cms_project_id, COPY_PROHIBITED_FILES_FOR_PREVIEW],
      :order => 'public_file_path')
    pj_files.each do |pj_file|
      dest_file = File.join(EcRider.config.ec_admin_root, pj_file.public_file_path)
      FileUtils.mkdir_p(File.dirname(dest_file)) unless File.exist?(File.dirname(dest_file))
      FileUtils.cp(pj_file.pj_file_path, dest_file)
    end
  end

  # ページのファイル名を返します。
  # ビューのファイル名の管理はCmsBindを中心に行います。
  def published_filename(locale = :ja)
    if (CmsTemplateContent.carrier_ids.include?(locale.to_s))
      locale_str = (locale.to_s == 'docomo') ? '_mobile' : '_mobile_' << locale.to_s
    else
      locale_str = (locale.to_s == I18n.default_locale.to_s) ? nil : '_' << locale.to_s.gsub(/\-/, '_')
    end
    "#{controller}/#{action}#{locale_str}.html.erb"
  end
  alias_method :file_name, :published_filename

  class << self
    def primary_pages(options = {})
      options = {
        :project_status_keys => [:opened],
        :project_ids => [],
       }.update(options)
      page_ids = primary_page_ids({:project_status_keys => options[:project_status_keys], :project_ids => options[:project_ids]})
      options[:conditions] = ["id in (?)", page_ids]
      options.reject! {|key, value| key == :project_status_keys}
      options.reject! {|key, value| key == :project_ids}
      find(:all, options)
    end
    

    def primary_page_ids(options = {})
      options = {
        :project_status_keys => [:opened],
        :project_ids => [],
       }.update(options)
      project_status_keys = options[:project_status_keys].flatten
      project_status_keys = [:opened] if project_status_keys.nil? || project_status_keys.empty?
      project_status_cds = project_status_keys.map{|key| "'%s'" % CmsProject.status_id_by_key(key) }
      if project_status_cds.length == 1
        status_condition = "cms_projects.status_cd = %s" % project_status_cds.first
        status_condition2 = "cms_projects2.status_cd = %s" % project_status_cds.first
      else
        status_condition = "cms_projects.status_cd in (%s)" % project_status_cds.join(", ")
        status_condition2 = "cms_projects2.status_cd in (%s)" % project_status_cds.join(", ")
      end
      
      project_ids = options[:project_ids].flatten
      project_ids = [] if project_ids.nil? || project_ids.empty?
      if project_ids.length == 1
        id_condition = " OR cms_projects.id = %s" % project_ids.first
        id_condition2 = " OR cms_projects2.id = %s" % project_ids.first
      elsif project_ids.length > 1
        id_condition = " OR cms_projects.id in (%s)" % project_ids.join(", ")
        id_condition2 = " OR cms_projects2.id in (%s)" % project_ids.join(", ")
      end
      sql = <<-"EOS"
        select
          cms_pages.id
        from cms_pages 
          inner join cms_projects on cms_projects.id = cms_pages.cms_project_id 
        where (#{status_condition}#{id_condition})
          AND cms_projects.priority =
          (
            select max(cms_projects2.priority)
            from cms_pages cms_pages2 
              inner join cms_projects cms_projects2 on cms_projects2.id = cms_pages2.cms_project_id 
            where cms_pages2.controller = cms_pages.controller
              AND cms_pages2.action = cms_pages.action
              AND(#{status_condition2}#{id_condition2})
            group by cms_pages2.controller, cms_pages2.action
          )
      EOS
      connection.select_values(sql).map(&:to_i)
    end

    

  end

end
