# -*- coding: utf-8 -*-
require 'net/http'
require 'uri'

class CmsPublish < ActiveRecord::Base
  
  # プロジェクト公開関連 
  has_many :cms_published_projects, :class_name => "CmsPublishedProject", :foreign_key => 'cms_publish_id'
    
  # プロジェクト公開関連で関連しているCMSプロジェクト 
  # ソート順はpriority降順です。
  has_many :projects, :through => :cms_published_projects, :source => :cms_project, :order => "priority desc"
  
  has_many :dead_link_checks, :class_name => "CmsDeadLinkCheck", :foreign_key => 'cms_publish_id', :order => 'created_at desc'

  selectable_attr :target do
    entry('1', :production, '本番',
      :views_reloading => {
        :urls_key => 'ECR_FRONT_RELOAD_VIEWS_URLS', 
        :basic_auth => {
          :account_key  => "ECR_FRONT_BASIC_AUTH_ACCOUNT",
          :password_key => "ECR_FRONT_BASIC_AUTH_PASSWORD"
        }
      })
    entry('2', :test, 'テスト',
      :views_reloading => {
        :urls_key => 'ECR_TEST_RELOAD_VIEWS_URLS', 
        :basic_auth => {
          :account_key  => "ECR_TEST_BASIC_AUTH_ACCOUNT",
          :password_key => "ECR_TEST_BASIC_AUTH_PASSWORD"
        }
      })
  end

  selectable_attr :status_cd do
    entry '0', :initialized, '初期化済み'
    entry '1', :waiting    , '公開待ち'
    entry '2', :processing , '公開処理中'
    entry '3', :opened     , '公開済み'
    entry '4', :failed     , '公開失敗'
  end

  validates_presence_of :designated_at # , :if =>  Proc.new {|publish| publish.target_key == :test}
  attr_accessor :use_only_view
  def validate
    # テスト環境へ公開するとき、対象日時＜現在日時の時はエラーとする
    # これは商品などのデータが削除されてページとしてちゃんと復元されない恐れがあるためです。
    # 2010/01/25 管理者サイト テスト環境作成画面上から登録するときのみチェックする制御になりました。
    if self.use_only_view
      errors.add(:designated_at, :EM990036) if self.designated_at < Time.now
    end
  end
  
  def status_name_for_test
    case status_key
      when :waiting, :processing
        '作成中'
      when :opened
        '作成成功'
      when :failed
        '作成失敗'
    end
  end
    
  # プロジェクトの公開を行います。
  def execute
    begin
      # このCmsPublishで どのデフォルトテンプレートを使用するのかを設定
      setup_default_template_paths
      
      # ファイル名とプロジェクトの優先順位からどのCmsBindでファイルを
      # 生成するのかを決めるためのマトリックスを作ります。
      setup_filename_matrix
      
      # viewのファイルに対するアクションを実行
      view_filename_matrix.each do |filepath, cms_action|
        cms_action.process
      end

      # デフォルトテンプレートのファイルを作成
      CmsTemplate.primary_default_templates(:projects => self.projects).each do |terminal_key, templates|
        templates.each do |position, template|
          template.overwrite(position, self.target_key) unless template.nil?
        end
      end
      
      # public以下のファイルに対するアクションを実行
      public_filename_matrix.each do |filepath, file_action|
        file_action.process
      end
      
      # CloudFrontにec_front/public以下のファイルをアップロード
      upload_cloud_front if self.cloud_front_upload
      
      # ec_frontに更新を通知してビューをリロードさせる
      send_reload_request
      
      # うまくいったらopenedに更新
      self.status_key = :opened
      
      # 本番へデプロイの場合は、プロジェクトのステータスを更新
      if self.target_key == :production
        self.cms_published_projects.each do |published_project|
          published_project.cms_project.up_status
          published_project.cms_project.save!
        end
      end
    rescue Exception => err
      logger.error("#{self.class}[#{self.id}].execute failure.....\n" << 
        "  because of #{err}\n  " << err.backtrace.join("\n"))
      # 失敗したらfailedに更新
      self.status_key = :failed
    end
    self.save!
  end


  # テストの場合は、選択された且つ期間中のプロジェクトに加えて、公開されているものを追加。
  def setup_published_projects
    # 自身のprojectsは必要な情報を設定してから取得させたいので、ここではselfのcloneからprojectsを取得します
    self_projects = self.clone.projects
    # すでに選択されているプロジェクトは「up」。（通常本番では空）
    # ここでは何も追加しません。
    @ignored_cms_project_ids = self_projects.map(&:id)
    begin
      # 期間中の公開済みは、テストの場合「up」、本番の場合「keep」
      calc_key = self.target_key == :test ? :up : :keep
      add_cms_published_projects_for(calc_key, :conditions =>
        ["started_at <= ? AND finished_at >= ? AND status_cd = ?",
          self.designated_at,
          self.designated_at,
          CmsProject.status_id_by_key(:opened)])
      # 期間を過ぎた公開済みは終了するので、「down」
      add_cms_published_projects_for(:down, :conditions =>
        ["finished_at < ? AND status_cd = ?", 
          self.designated_at,
          CmsProject.status_id_by_key(:opened)])
    ensure
      @ignored_cms_project_ids = nil
    end
  end
  
  def add_cms_published_projects_for(calc_type_key, find_options)
    # 選択されていたプロジェクトはここでの検索対象から外します。
    if @ignored_cms_project_ids && !@ignored_cms_project_ids.empty?
      conditions = find_options[:conditions]
      where = conditions.first
      where << "AND id NOT IN (?)"
      conditions << @ignored_cms_project_ids
    end
    CmsProject.find(:all, find_options).each do |project|
      self.cms_published_projects.create(
        :cms_project_id => project.id,
        :calc_type_cd => CmsPublishedProject.calc_type_id_by_key(calc_type_key))
    end
  end
  

  # execute内で使用される属性
  attr_accessor :default_template_paths, :view_filename_matrix, :public_filename_matrix
  
  # デフォルトテンプレートとして、どのCmsTemplateを使用するのかを調べておきます。
  def setup_default_template_paths(preview_project_id=nil)
    @default_template_paths = {:pc => {}, :mobile => {}}
    CmsTemplate.primary_default_templates({:projects => self.projects, :preferred_project_id => preview_project_id}).each do |terminal_key, templates|
      templates.each do |position_key, template|
        next unless template
        path = template.published_default_template_filename(position_key)
        path.gsub!(/\/_/, '/')
        path.sub!(/\.html\.erb$/, '')
        @default_template_paths[terminal_key][position_key] = path
      end
    end
  end

  # ファイル名とプロジェクトの優先順位からどのCmsBindでファイルを
  # 生成するのかを決めるためのマトリックスを作ります。
  #
  # 
  # id | proirity
  # =========================================================
  #  3 | [100]              |---|
  #    |                    |   |
  #  6 | [ 99]         |----|   |
  #    |               |    |   |
  #  2 | [ 14]         | |--------------------------------|
  #    |               | |  |   |                         |
  #  1 | [  3]      |--------------------------------|    |
  #    |            |  | |  |   |                    |    |
  # =========================================================
  #                 |  | |  |   |                    |    |
  #  6              |  | |  |  2010-12-30 16:07:00   |    |
  #  3              |  | | 2009-01-31 10:00:00       |    |
  #  3              | 2009-01-10 10:00:00            |    |
  #                 |    |                           |    |
  #  2              |   2009-01-20 10:00:00          |   2020-02-01 10:00:00
  #  1             2009-01-01 10:00:00              2020-01-01 10:00:00
  # 
  # というプロジェクト群があり、
  # 2009-01-31 10:00:01 時点
  #   == CmsProject#3 の公開終了日時が過ぎた直後 
  #   == CmsProject#6 の公開開始日時が過ぎた直後 
  # という場合、cms_published_projectsは、priority降順でソートされ、以下のようになるはず。
  #   cms_published_projects[0] {:cms_proejct_id => 3, :calc_type_key => :up}   # priority:100
  #   cms_published_projects[1] {:cms_proejct_id => 6, :calc_type_key => :down} # priority: 99
  #   cms_published_projects[2] {:cms_proejct_id => 2, :calc_type_key => :keep} # priority: 14
  #   cms_published_projects[3] {:cms_proejct_id => 1, :calc_type_key => :keep} # priority:  3
  #
  # 各プロジェクトが以下のようなファイル(となるCmsPage/CmsTemplate)を持っているとします。
  # CmsProject#1
  #    index.html.erb
  #    list.html.erb
  #    show.html.erb
  #    campaign1.html.erb
  #    campaign2.html.erb
  #    _default_top.html.erb
  #    _default_bottom.html.erb
  # CmsProject#2
  #    campaign1.html.erb[削除]
  #    campaign2.html.erb[削除]
  # CmsProject#6
  #    index.html.erb
  #    list.html.erb[削除] 
  #    show.html.erb[削除]
  #    campaign1.html.erb
  #    sorry.html.erb
  # CmsProject#3
  #    index.html.erb
  #    list.html.erb [削除]
  #    _default_top.html.erb
  #    discount.html.erb
  #
  # このメソッドは、各プロジェクトのファイルと優先順位のmatrixとしては以下のようなデータを作成します。
  #
  #  |--------------------------|-----------------|-----------------|--------------------|--------------------|
  #  |                          | id:3, :up       | id:6, :down     | id:2, :keep        | id:1, :keep        |
  #  |--------------------------|-----------------|-----------------|--------------------|--------------------|
  #  | index.html.erb           | :overwrite      | :next_or_delete |                    | :overwrite_if_next | => CmsPublish::FileAction
  #  | list.html.erb            | :delete         | :next           |                    | :overwrite_if_next | => CmsPublish::FileAction
  #  | show.html.erb            |                 | :next           |                    | :overwrite_if_next | => CmsPublish::FileAction#next(@project#6) ==> CmsPublish::FileAction#overwrite_if_next(@project#1, true)
  #  | campaign1.html.erb       |                 | :next_or_delete | :delete_if_next    | :overwrite_if_next | => CmsPublish::FileAction#next_or_delete(@project#6) ==> CmsPublish::FileAction#next_or_delete(@project#2, true) ==> CmsPublish::FileAction#overwrite_if_next(@project#1, true)
  #  | campaign2.html.erb       |                 |                 | :delete_if_next    | :overwrite_if_next | => CmsPublish::FileAction#next_or_delete(@project#2, false) ==> CmsPublish::FileAction#overwrite_if_next(@project#1, true)
  #  | _default_top.html.erb    | :overwrite      |                 |                    | :overwrite_if_next | => CmsPublish::FileAction#overwrite(@project#3)
  #  | _default_bottom.html.erb |                 |                 |                    | :overwrite_if_next | => CmsPublish::FileAction#overwrite_if_next(@project#6, false) => (何もしない)
  #  | sorry.html.erb           |                 | :next_or_delete |                    |                    | => CmsPublish::FileAction#next_or_delete(@project#6)
  #  | discount.html.erb        | :overwrite      |                 |                    |                    | => CmsPublish::FileAction#overwrite(@project#3)
  #  |--------------------------|-----------------|-----------------|--------------------|--------------------|
  # 
  # 具体的には、matrixは以下のキーと値の組み合わせを持つHashです。
  #  キー：ファイルパス
  #  値  : ファイル名ごとの各プロジェクトのCmsBindを複数保持するCmsPublish::FileAction
  # 
  # 
  # 上記のマトリックスの中は各CmsBindが行うべき動作です。
  #   詳しくはCmsPublish::FileActionを参照してください
  #  
  def setup_filename_matrix
    @view_filename_matrix = {}
    @public_filename_matrix = {}

    targets = self.cms_published_projects.find(:all,
      :order => "cms_projects.priority desc, cms_projects.started_at desc, cms_projects.id desc",
      :include => :cms_project

# すべてincludeすると、プロジェクト数が増えたときにfindに時間がかかりすぎて、プロセスがKillされてしまうため
# cms_projectのみインクルードして、そのほかのデータは必要な時点で取得するようにしました。
#
#      :include => {
#        :cms_project => [
#        :files, # :filesについては別で取得した方が良い？
#        {:pages => {
#            :binds => [
#              {:items => :contents},
#              {:cms_template => :contents}
#            ]
#          }}
#      ]}
    )
    targets.each do |target|
      project = target.cms_project
      project.pages.each do |page|
        page.binds.each do |bind|
          next unless bind.cms_template
          # より正確を期するなら published_filename に対応する全てのロケールを渡してそれぞれのファイル名を作るべきかもしれないが、
          # 無駄が多いので、各言語でのファイル名については、CmsPublish::FileAction内で扱うことにします。
          # ....と思っていましたが、けっこう分かり難くなってしまうので、それは後回しにします。
          if page.terminal_key == :pc
            I18n.supported_locales.each do |locale|
              locale = locale.to_s
              filename = bind.published_filename(locale)
              cms_action = (@view_filename_matrix[filename] ||= CmsPublish::CmsAction.new(self, filename, locale))
              bind.publish_calc_type_key = target.calc_type_key
              cms_action.targets << bind
            end
          elsif page.terminal_key == :mobile
            CmsTemplateContent.carrier_ids.each do |locale|
              locale = locale.to_s
              filename = bind.published_filename(locale)
              cms_action = (@view_filename_matrix[filename] ||= CmsPublish::CmsAction.new(self, filename, locale))
              bind.publish_calc_type_key = target.calc_type_key
              cms_action.targets << bind
            end
          end
        end
      end
      project.files.each do |file|
        public_filename = file.public_file_path
        file_action = (@public_filename_matrix[public_filename] ||= CmsPublish::FileAction.new(self, public_filename))
        file.publish_calc_type_key = target.calc_type_key
        file_action.targets << file
      end
    end
  end


  def upload_cloud_front
    files = ::EcRider::CloudFront.files_to_upload
    timestamp = Time.now.strftime(ENV['ECR_TIMESTAMP'] || ::EcRider::CloudFront::CLOUD_FRONT_VERSION_FORMAT)
    s3 = Object.new
    s3.extend(::EcRider::CloudFront)
    targets = files.map{|file| file[file.index("\/public\/")+8..-1]}
    s3.create_and_upload_to_s3(targets, :timestamp => timestamp)
  end

  def send_reload_request
    self.class.send_reload_request(target_key)
  end

  class << self
    def send_reload_request(target_key)
      target_entry = self.target_entry_by_key(target_key)
      views_reloading = target_entry[:views_reloading] if target_entry
      raise ArgumentError, "Unsupported ViewReloading. CmsPublish#target => #{self.target_key.inspect}(#{self.target.inspect})" unless views_reloading
      urls = load_env_by(views_reloading[:urls_key])
      basic_auth = (views_reloading[:basic_auth] || {})
      basic_auth_account = load_env_by(basic_auth[:account_key])
      basic_auth_password = load_env_by(basic_auth[:password_key])
      (urls || '').split(/\,/).each do |reloading_url|
        uri = URI.parse(reloading_url)
        request = Net::HTTP::Get.new(uri.path)
        request.basic_auth(basic_auth_account, basic_auth_password) if basic_auth_account
        logger.info("#{reloading_url} accessing....")
        begin
          Net::HTTP.start(uri.host, uri.port) do |http|
            response = http.request(request)
            logger.info("#{reloading_url} " << response.body)
          end
        # オートスケール対応のため、リロード用指示を出す先のプロセスが起動していない場合は無視するようにしました
        rescue => ex
          logger.info("#{reloading_url} " << ex.message)
        end
      end
    end

    private

    def load_env_by(key)
      key.blank? ? nil : ENV[key]
    end
  end

  
  class FileAction
    attr_reader :cms_publish, :filepath, :targets
    
    def initialize(cms_publish, filepath)
      @cms_publish = cms_publish
      @filepath = filepath
      @targets = []
      @next_called = false
    end

    METHODS_BY_CALC_TYPE_AND_DELETE_AT_PUBLISH = {
      :up   => {false => :overwrite        , true => :delete         },
      :down => {false => :next_or_delete   , true => :next           },
      :keep => {false => :overwrite_if_next, true => :delete_if_next }
    }

    def self.method_for(target)
      method_set = METHODS_BY_CALC_TYPE_AND_DELETE_AT_PUBLISH[target.publish_calc_type_key]
      raise "Unknown calc_type_key => #{target.publish_calc_type_key.inspect}" unless method_set
      method = method_set[!!target.delete_at_publish?] # 確実にfalse/trueにするために!!を使用しています。
    end

    # |-------|---------------------------------------|
    # |       |            delete_at_publish          |
    # |       |--------------------|------------------|
    # |       |      false         |       true       |
    # |-------|--------------------|------------------|
    # | :up   | :overwrite         | :delete          |
    # | :down | :next_or_delete    | :next            |
    # | :keep | :overwrite_if_next | :delete_if_next  |
    # |-------|--------------------|------------------|
    #  
    #  :overwrite        : 必ず上書き
    #  :delete           : (ファイルが存在していたら)削除
    #  :next_or_delete   : 次に優先度の高いCmsBindがあればそれに任せる、次が何もなければ削除
    #  :next             : 次に優先度の高いCmsBindがあればそれに任せる、次が何もなければ何もしない
    #  :overwrite_if_next: :next/:next_or_deleteから呼び出されれば上書き、そうでなければ何もしない
    #  :delete_if_next   : :next/:next_or_deleteから呼び出されれば削除、そうでなければ何もしない
    def process
      @target = @targets.shift
      if @target
        method = self.class.method_for(@target)
        send(method)
      else
        if @action_at_last # 最後のアクションが指定してある場合
          @target = @action_at_last[:bind]
          send(@action_at_last[:method])
        end
      end
    end

    def overwrite
      # @filepathでファイル上書き
      @target.overwrite( self ) # publish_options
    end

    def delete
      # @filepathのファイル削除
      @target.delete( self ) # publish_options
    end

    def next_or_delete
      @action_at_last = {:method => :delete, :bind => @target}
      @next_called = true
      process
    end

    def next
      @next_called = true
      process
    end

    def overwrite_if_next
      overwrite if @next_called
    end

    def delete_if_next
      delete if @next_called
    end
    
    def inspect_file
      "#{@filepath} " <<
      @targets.map do |target|
        {
          :project_id => target.cms_project_id,
          :method => self.class.method_for(target)
        }.inspect
      end.join(', ')
    end
    
    def inspect
      "#{@filepath} " <<
      @targets.map do |target|
        {
          :bind_id => target.id,
          :project_id => target.cms_page.cms_project.id,
          :priority => target.cms_page.cms_project.priority,
          :method => self.class.method_for(target)
        }.inspect
      end.join(', ')
    end
  end
  
  class CmsAction < CmsPublish::FileAction
    attr_reader :locale
    
    def initialize(cms_publish, filepath, locale = :ja)
      super(cms_publish, filepath)
      @locale = I18n.regularize(locale)
    end
  end

  class << self
    def latest_test_publish
      find(:first,
        :conditions => ["target = ? and status_cd = ?", 
          CmsPublish.target_id_by_key(:test), CmsPublish.status_id_by_key(:opened)],
        :order => 'updated_at desc')
    end

    # 本番更新用のRakeタスク rake ecr:cms:publish から実行されるメソッドです。
    # :target_key => :production/:test 
    #   デフォルトは:productionです。
    #   :testを指定するのは、テスト環境へ公開する場合です。
    def create_and_execute(options = {})
      options = {
        :target_key => :production,
        :cloud_front_upload => true
      }.update(options || {})
      target_key = options[:target_key]
      publish = nil
      transaction do
        return if processing_now?(target_key) 
        # 本番公開の対象となるプロジェクトを取得
        published_projects = CmsProject.find_all_to_be_deployed_or_deleted
        return if published_projects.empty?
        # 各プロジェクトに対する処理(:up/:keep/:down)を求めています。
        project_calc_type_keys = {}
        published_projects.each do |project|
          project_calc_type_keys[project.id] =
            project.status_key == :approved ? :up :
            project.to_be_opened? ? :keep : :down
        end
        # プロジェクトに対する処理がすべて現状維持なら何もしません。
        return if project_calc_type_keys.values.all?{|key| key == :keep}
        now = Time.now
        publish = CmsPublish.create(
          :target => CmsPublish.target_id_by_key(target_key),
          :status_cd => CmsPublish.status_id_by_key(:processing),
          :designated_at => now,
          :published_at => now, 
          :cloud_front_upload => options[:cloud_front_upload]
          )
        published_projects.each do |project|
          calc_type_key = project_calc_type_keys[project.id]
          publish.cms_published_projects.create(
            :cms_project_id => project.id, 
            :calc_type_cd => CmsPublishedProject.calc_type_id_by_key(calc_type_key)
          )
        end
      end
      publish.execute
    end
    
    # テスト環境更新用タスク
    # :target_key => :production/:test 
    #   デフォルトは:testです。
    #   :productionは、通常waiting状態を使わないcreate_and_executeメソッドで実行されます。
    def execute_if_waiting(options = {})
      options = {
        :target_key => :test 
      }.update(options || {})
      return if CmsDeadLinkCheck.exists?(["status_cd in (?)", 
        [CmsDeadLinkCheck.status_id_by_key(:waiting), CmsDeadLinkCheck.status_id_by_key(:processing)]])
      CmsPublish.exclusive_processing(options[:target_key]) do |publish|
        begin
          publish.setup_published_projects
          publish.cloud_front_upload = false
          cleanup_dir
          publish.execute
          return if publish.status_key == :failed
        rescue Exception => err
          logger.error("#{publish.class}[#{publish.id}].execute failure.....\n" << 
            "  because of #{err}\n  " << err.backtrace.join("\n"))
          # 失敗したらfailedに更新
          publish.status_key = :failed
          publish.save!
        end
      end
    end
    
    def cleanup_dir(root_dir = EcRider.config.ec_test_root)
      # public以下で削除しないファイル、ディレクトリ
      leave_directories = ['ec_rider', 'banner_images', 'tmp_inquiry_images']

      # public以下のファイルを全て削除
      src_files = Dir.glob(File.join(root_dir, 'public', "*"))
      src_files.each do |src_file|
        remove_flg = true
        leave_directories.each do |leave_dir|
          remove_flg = false if File.fnmatch(src_file, File.join(root_dir, 'public', leave_dir))
        end
        FileUtils.rm_r(src_file) if remove_flg && File.basename(src_file) !~ /^\./
      end
      # app/views以下のCMSビューを全て削除
      EcRider::CmsSetup.ecr_controllers.each do |controller|
        files = Dir.glob(File.join(root_dir, 'app', 'views', controller, "*.html.erb"))
        FileUtils.rm_r(files)
      end
      files = Dir.glob(File.join(root_dir, 'app', 'views', 'cms', "*.html.erb"))
      FileUtils.rm_r(files)
    end
    
    # publish対象となるもの(publish_candidate)がある場合、それを:processingの状態に変更して、他のCmsPublishとの
    # 排他制御に引っかからないようにします。
    # その後ブロックを実行して、正常に動けば:openedに、エラーがあれば:failedにします。
    def exclusive_processing(target_key)
      publish = nil
      transaction do
        # publish対象となるものを検索します。
        # 他のプロジェクトのステータスが何も:processingになっていない状態で、
        # :waitingのレコードがあった場合にそのレコードを返します。
        #
        # :lock => trueで FOR UPDATE を付きのSELECTになります。超重要！！！
        publish = CmsPublish.find(:first, :lock => true,
          :conditions => [
            "status_cd = ? AND target = ? AND not exists (select * from cms_publishes where status_cd = ? AND target = ?)",
            status_id_by_key(:waiting),
            target_id_by_key(target_key),
            status_id_by_key(:processing),
            target_id_by_key(target_key)
          ])
        return unless publish
        publish.status_key = :processing
        publish.save!
      end
      yield( publish )
    end

    def processing_now?(target_key = :production)
      CmsPublish.count(
        :conditions => ["status_cd = ? AND target = ?",
          status_id_by_key(:processing), target_id_by_key(target_key)
        ]) > 0
    end
  end

end
