# -*- coding: utf-8 -*-
class Country < ActiveRecord::Base

  set_primary_key :code
  has_many :shipping_costs, :class_name => "ShippingCost", :foreign_key => "area_code", :dependent => :destroy, :order => 'id'
  belongs_to :distributor

  # 配送可能区分
  selectable_attr :ship_restriction_div do
    entry '0', :able, '配送可能'
    entry '1', :disable, '配送不可'
  end

  # 対応言語区分
  selectable_attr :use_language_div do
    entry '0', :english, '英数'
    entry '1', :chinese, '英数簡'
    entry '99', :japanese, '英数日'
  end

  # 名前の並び順区分
  selectable_attr :name_order_div do
    entry '0', :last_first, '姓＋名'
    entry '1', :first_last, '名＋姓'
  end

  # 住所の並び順区分
  selectable_attr :address_order_div do
    entry '0', :address_1_2_3, '都道府県＋住所（市町村番地）＋住所（ビル名）'
    entry '1', :address_3_2_1, '住所（ビル名）＋住所（市町村番地）＋都道府県'
  end

  #
  # 国名と国コードのリストを配列で返す
  #
  def self.country_list(only_able_to_ship = nil)
    condition = ['ship_restriction_div=?', Country.ship_restriction_div_id_by_key(:able)] if only_able_to_ship
    list = []
    find(:all, :conditions => condition, :order => 'id').each do |country|
      list << [country.name, country.code]
    end
    return list
  end

  #
  # 配送可能な国名と国コードのリストを配列で返す
  #
  def self.ship_to_country_list
    country_list(true)
  end

  #
  # 店舗設定で登録した初期表示国が配送可能な場合はその設定値を、そうでない場合は先頭の国を返す
  #
  def self.default_country
    ship_restriction_div_able = Country.ship_restriction_div_id_by_key(:able)
    code = ShopSetting.get_setting(:default_address_country, I18n.locale)
    if find(:first, :conditions => {:code => code, :ship_restriction_div => ship_restriction_div_able})
      return code
    else
      country = find(:first, :select => :code, :conditions => {:ship_restriction_div => ship_restriction_div_able}, :order => 'id')
      return country.code
    end
  end

  # 配列で渡された国コードから国を取得し、配送可能のフラグを付加し、配列で返却する
  def self.set_ship_restriction_able(country_codes = [])
    countries = find(:all)
    countries.each do |country|
      if country_codes.index(country.code).nil?
        country.ship_restriction_div = Country.ship_restriction_div_id_by_key(:disable)
      else
        country.ship_restriction_div = Country.ship_restriction_div_id_by_key(:able)
      end
    end
    return countries
  end

  class SearchCondition < EcRider::ConditionForm
    include ActiveRecord::Validations
    attr_accessor :name, :name_top, :prefecture_name, :distributor_name, :use_language_div, 
      :ship_restriction_div, :prefecture_display_flg, :disp_count, :shipping_cost

    validates_format_of :name_top, :with => /^[A-Z]$/, :message => :EM990007, :if => :name_top_check?
    validates_format_of :ship_restriction_div, :with => /^[01]$/, :message => :EM990007, :if => :ship_restriction_div_check?
    validates_numericality_of :shipping_cost, :only_integer =>true, :message => :EM990003, :if => :shipping_cost_check?

    def name_top_check?
      return !name_top.blank?
    end
    def ship_restriction_div_check?
      return !ship_restriction_div.blank?
    end
    def shipping_cost_check?
      return !shipping_cost.blank?
    end
    
    #
    # = 検索条件を付与したSQL文を作成します。 =
    #
    def get_finder_sql
      boundary = ShippingCostBoundary.find(:all, :order => 'id')
      select = "SELECT t.id, t.area_type_div, t.name, t.distributor_name, t.use_language_div, t.ship_restriction_div, t.code, "
      from = <<-EOS
        FROM (
              SELECT
                countries.id,
                '1' AS area_type_div,
                countries.name,
                distributors.name AS distributor_name,
                countries.use_language_div,
                countries.ship_restriction_div ,
                countries.code
              FROM countries LEFT OUTER JOIN distributors ON countries.distributor_id = distributors.id
              UNION
              SELECT
                '' AS id,
                '2' AS area_type_div,
                prefecture_displays.name,
                '' AS distributor_name,
                '' AS use_language_div,
                '0' AS ship_restriction_div,
                prefectures.code
              FROM shipping_costs
              LEFT OUTER JOIN prefectures ON shipping_costs.area_code = prefectures.code
              LEFT OUTER JOIN prefecture_displays ON (prefectures.id = prefecture_displays.prefecture_id AND prefecture_displays.locale = 'ja')
              WHERE shipping_costs.area_type_div = '2'
        ) t
      EOS
      # 閾値マスタの更新などでIDが変わる可能性があるため動的にLEFT OUTER JOINを生成する
      # 生成例：）
      #  join_column = sc1.shipping_cost AS shipping_cost_1, 閾値マスタ登録分
      #  LEFT OUTER  = JOIN shipping_costs AS sc1 ON (1 = sc1.shipping_cost_boundary_id AND t.code = sc1.area_code)　 閾値マスタ登録分
      join_column = ""
      join = ""
      if boundary.empty?
        select = select.slice(0,select.size-2)
      else
        boundary.each_with_index do |boundary, index|
          join_column = join_column + "sc#{index}.shipping_cost AS shipping_cost_#{index}, "
          join = join + " LEFT OUTER JOIN shipping_costs AS sc#{index} ON (#{boundary.id} = sc#{index}.shipping_cost_boundary_id AND t.code = sc#{index}.area_code) "
        end
        join_column = join_column.slice(0, join_column.size-2)
      end
      sql = select + join_column + from + join
      # 検索条件
      where = Condition.block do |c|
        c.filter(:ignore_blank)

        #日本を含み都道府県名を表示する場合
        if prefecture_display_flg == "1" && (name.blank?  ||"japan"[name] || "Japan"[name]) && (name_top.blank? || (name_top =~ /^J/))
          c.and{|d|
            #県検索条件
            d.or{|e|
              #県名
              e.and("name", "like", "%#{prefecture_name}%") unless prefecture_name.blank?
              #県
              e.and("t.area_type_div", "=", "2")
              e.or("name", "=", "japan")
            }
            #国検索条件
            d.or{|e|
              #国名（部分一致）
              e.and("name", "like", "%#{name}%") unless name.blank?
              #国名の頭文字（選択肢：A～Z）
              e.and("name", "like", "#{name_top}%") unless name_top.blank?
              #都道府県名（部分一致）
              e.and("name", "like", "%#{prefecture_name}%") unless prefecture_name.blank?
              #国
              e.and("t.area_type_div", "=", "1")
            }
          }
        else
          #都道府県名を表示しない
          #国名（部分一致）
          c.and("name", "like", "%#{name}%") unless name.blank?
          #国名の頭文字（選択肢：A～Z）
          c.and("name", "like", "#{name_top}%") unless name_top.blank?
          #都道府県名（部分一致）
          c.and("name", "like", "%#{prefecture_name}%") unless prefecture_name.blank?
          #都道府県名を表示しない
          c.and("t.area_type_div", "=", "1")
        end
        #物流名（部分一致）
        c.and("distributor_name", "like", "%#{distributor_name}%") unless distributor_name.blank?
        #対応言語（選択肢：英数、英数簡、英数日）
        c.and("use_language_div", "=", "#{use_language_div}") unless use_language_div.blank?
        #配送可能区分（選択肢：配送可能、配送不可）
        c.and("ship_restriction_div", "=", "#{ship_restriction_div}") unless ship_restriction_div.blank?
        #都道府県名を表示しない
        #c.and("t.area_type_div", "=", "1") unless prefecture_display_flg == "1"
      end
      # SQL生成
      result = []
      if where.nil?
        result << sql
      else
        sql = sql + " WHERE " + where.shift
        result << sql
        result.concat(where)
      end
      return result
    end
  end

  def self.create_condition(condition)
    return nil if condition.nil?
    Condition.block do |c|
      c.add_like_partial( "name", "like", condition.name ) unless condition.name.blank?
      c.add_like_forward( "name", "like", condition.name_top ) unless condition.name_top.blank?
      c.and "shipping_cost", "=", condition.shipping_cost unless condition.shipping_cost.blank?
      c.and "ship_restriction_div", "=", condition.ship_restriction_div unless condition.ship_restriction_div.blank?
    end
  end

  #
  # = 国ごとのSQL文を作成します。 =
  #送料編集で使用します。
  def self.get_sql_for_shipping_costs_countries(boundary_ids =[], countries = [])
    select = "SELECT countries.name, countries.code, "
    join_column = ""
    from = " FROM countries LEFT OUTER JOIN distributors ON countries.distributor_id = distributors.id "
    # boundary_idsからjoinを生成
    join = ""
    boundary_ids.size.times do |index|
      join_column = join_column + "sc#{index}.shipping_cost AS shipping_cost_#{index}, "
      join = join + " LEFT OUTER JOIN shipping_costs AS sc#{index} ON (? = sc#{index}.shipping_cost_boundary_id AND countries.code = sc#{index}.area_code) "
    end

    if boundary_ids.empty?
      select = select.slice(0,select.size-2)
    else
      join_column = join_column.slice(0,join_column.size-2)
    end
    where = "where code in (?)"
    in_country = countries.join("\',\'")
    in_country.slice!(0,in_country.size-2)
    sql = select + join_column + from + join + where
    query = [sql] + boundary_ids
    return query << countries
  end
  #
  # = 送料編集で使用するSQL文を作成します。 =
  #
  def self.get_shipping_costs_sql_for_prefectures(boundary_ids =[])
    select = <<-EOS
      SELECT
        prefecture_displays.name,
        prefectures.code,
    EOS
    # boundary_idsからjoinを生成
    join = ""
    join_column = ""
    boundary_ids.size.times do |index|
      join_column = join_column + "sc#{index}.shipping_cost AS shipping_cost_#{index}, "
      join = join + " LEFT OUTER JOIN shipping_costs AS sc#{index} ON (? = sc#{index}.shipping_cost_boundary_id AND prefectures.code = sc#{index}.area_code) "
    end
    if boundary_ids.empty?
      select = select.slice(0,select.size-2)
    else
      join_column = join_column.slice(0,join_column.size-2)
    end
    from = " FROM prefectures LEFT OUTER JOIN prefecture_displays ON (prefectures.id = prefecture_displays.prefecture_id AND prefecture_displays.locale = 'ja')"
    query = [select + join_column + from + join + "order by code"] + boundary_ids
    return query
  end
end
