require 'fastercsv'
require "csv"
class CsvDownloadPattern < ActiveRecord::Base
  
  has_many :items, :class_name => "CsvDownloadPatternItem", :foreign_key => 'csv_download_pattern_id', :order => 'csv_download_pattern_items.display_order'
  has_one :user_condition, :class_name => "CsvDownloadPatternUserCondition", :foreign_key => 'csv_download_pattern_id'
  has_one :order_condition, :class_name => "CsvDownloadPatternOrderCondition", :foreign_key => 'csv_download_pattern_id'
  has_one :product_condition, :class_name => "CsvDownloadPatternProductCondition", :foreign_key => 'csv_download_pattern_id'
  has_one :accounting_condition, :class_name => "CsvDownloadPatternAccountingCondition", :foreign_key => 'csv_download_pattern_id'
  has_many :histories, :class_name => "CsvFileDownloadHistory", :foreign_key => 'csv_download_pattern_id', :order => 'csv_file_download_histories.public_file_path'
  
  attr_accessor :items_attributes

  # ダウンロード情報
  selectable_attr :download_type do
    entry "1", :user, '会員情報（会員単位）'
    entry "2", :user_address, '会員情報（住所単位）'
    entry "3", :product, '商品情報（商品単位）'
    entry "4", :product_sku, '商品情報（SKU単位）'
    entry "5", :product_display, '商品情報（言語単位）'
    entry "6", :product_price, '商品情報（売価単位）'
    entry "7", :product_component, '商品情報（構成単位）'
    entry "8", :order, '注文情報（注文単位）'
    entry "9", :order_shipping, '注文情報（配送単位）'
    entry "10", :order_item, '注文情報（商品単位）'
    entry "12", :order_item_component, '注文情報（構成単位）'
    entry "11", :accounting, '経理情報'
  end
  
  # ダウンロード情報
  selectable_attr :character_code do
    entry "1", :shift_jis, '日本語（シフトJIS）'
    entry "2", :"utf-8", 'Unicode（UTF-8）'
  end
  
  validates_presence_of :download_type, :name, :character_code, :file_name
  validates_uniqueness_of :name, :file_name
  validates_format_of :file_name, :with => /^[a-z][a-z0-9\_]*$/, :unless =>  Proc.new {|p| p.file_name.blank?}
  
  # validateメソッド内で記述しているエラーメッセージは、ec_adminのみで使用するメッセージです
  def validate
    errors.add_to_base("出力項目を選択してください。") if self.items_attributes[:item_list].empty?
    
    @fixed_values = {}
    if input = self.items_attributes[:fixed_value_list]
      input.scan(/"(.*?)"/m).flatten.compact.each do |fixed_value|
        fixed_part = fixed_value.split(/:/, 2)
        if fixed_part.length == 1 || fixed_part.first.blank?
          errors.add_to_base("固定値のフォーマットが不正です。")
          break
        else
          @fixed_values[fixed_part.first.strip] = fixed_part.last
        end
      end
    end
    
    return false unless self.errors.empty?
    
    set_items
    
    unless @fixed_values.empty?
      # 固定値の項目名が出力項目名と合っているか
      mismatch_items = @fixed_values.keys - self.items.map{|item|item.name}
      unless mismatch_items.nil? || mismatch_items.empty?
        errors.add_to_base("固定値の項目名と出力項目名が合っていません。 " << mismatch_items.join(','))
      end
    end
    
    # 検索条件
    unless @condition_form.nil? || @condition_form.valid?
      @condition_form.errors.each_full{|msg| errors.add_to_base(msg) }
    end
  end
  
  def set_items
    self.items_attributes[:item_list].split(',').each_with_index do |items_attirbute, index|
      item_part = items_attirbute.split(':')
      new_item = CsvDownloadPatternItem.new(:name => item_part.last, :display_order => index, :code => item_part.first)
      if new_item.code_key == :extra_column && !@fixed_values.keys.include?(new_item.name)
        new_item.value  = ""
      else
        new_item.value = @fixed_values[item_part.last]
      end
      self.items << new_item
    end
  end
  
  # 検索条件の入力値をバリデーション用と登録用にセットします
  def set_condition(conditions)
    case self.download_type_key
      when :user, :user_address
        # バリデーション用
        @condition_form = User::SearchConditionForm.new(conditions)
        # 登録用
        self.user_condition = CsvDownloadPatternUserCondition.new(conditions)
      when :product, :product_sku, :product_display, :product_price, :product_component
        @condition_form = Product::SearchCondition.new(conditions)
        self.product_condition = CsvDownloadPatternProductCondition.new_by_condition_form(conditions)
      when :order, :order_shipping, :order_item, :order_item_component
        @condition_form = Order::SearchInfo.new(conditions)
        self.order_condition = CsvDownloadPatternOrderCondition.new(conditions)
      when :accounting
        @condition_form = Accounting::SearchInfo.new(conditions)
        self.accounting_condition = CsvDownloadPatternAccountingCondition.new(conditions)
    end
  end
  
  # 出力項目の変更が不可の場合にtrueを返します。
  # 一括アップロード用雛形は、出力項目の変更不可かつパターンの削除不可とします。
  def uneditable?
    ["user_bulk_upload_template", "product_sku_bulk_upload_template", "product_price_bulk_upload_template", "product_display_bulk_upload_template"].include?(file_name)
  end
  
  # 登録済み出力パターンのセレクトボックス
  def name_options
    CsvDownloadPattern.find(:all, :conditions => ["download_type = ?", download_type]).
      map{|pattern| [pattern.name, pattern.id] }
  end
  
  # 出力項目選択値
  def output_selects
    if id.nil?
      self.items_attributes[:item_list].split(',').map do |item_list|
        [item_list.split(':').last, item_list]
      end
    else
      items.map do |item|
        [item.name, "#{item.code}:#{item.name}"]
      end
    end
  end
  
  # パターンの出力項目と検索条件をセット
  def set_detail_for_edit
    item_list_array = []
    fixed_value_array = []
    items.each do |item|
      item_list_array << "#{item.code}:#{item.name}"
      if (item.code_key == :extra_column && !item.value.blank?) || (item.code_key != :extra_column && item.value)
        fixed_value_array << "\"#{item.name}:#{item.value}\""
      end
      self.items_attributes = {:item_list => item_list_array.join(","), :fixed_value_list => fixed_value_array.join("")}
    end
    case download_type_key
      when :user, :user_address
        return User::SearchConditionForm.new(extract_user_condition)
      when :product, :product_sku, :product_display, :product_price, :product_component
        return Product::SearchCondition.new(extract_product_condition)
      when :order, :order_shipping, :order_item, :order_item_component
        return Order::SearchInfo.new(extract_order_condition)
      when :accounting
        return Accounting::SearchInfo.new(extract_accounting_condition)
    end
  end
  
  def extract_user_condition
    user_condition.attributes.select{|(key, value)|
      User::SearchConditionForm::ATTRIBUTE_NAMES.include?(key.to_sym)
    }
  end
  
  def extract_product_condition
    product_condition.attributes.select{|(key, value)|
      Product::SearchCondition::SEARCH_ATTRIBUTE_NAMES.include?(key.to_sym)
    }
  end
  
  def extract_order_condition
    order_condition.attributes.select{|(key, value)|
      Order::SearchInfo::ATTRIBUTE_NAMES.include?(key.to_sym) && !value.blank?
    }
  end

  def extract_accounting_condition
    accounting_condition.attributes.select{|(key, value)|
      Accounting::SearchInfo::ATTRIBUTE_NAMES.include?(key.to_sym)
    }
  end
  
  # 出力項目と検索条件をDBから削除
  def destroy_item_and_condition
    self.items.each{|item| item.destroy}
    self.items.clear
    case download_type_key
      when :user, :user_address
        self.user_condition.destroy
        self.user_condition = nil
      when :product, :product_sku, :product_display, :product_price, :product_component
        self.product_condition.destroy
        self.product_condition = nil
      when :order, :order_shipping, :order_item, :order_item_component
        self.order_condition.destroy
        self.order_condition = nil
      when :accounting
        self.accounting_condition.destroy
        self.accounting_condition = nil
    end
  end
  
  # CSV出力に必要なデータを作成します
  def generate
    FasterCSV.generate(:headers => header, :write_headers => true) do |writer|
      
      output_rows = rows
      if output_rows.empty?
        blank_item = []
        items.length.times{blank_item << nil}
        writer << blank_item
      else
        rows.each do |row|
          case download_type_key
            when :user, :product, :order, :accounting
              writer << output_item(row)
            when :user_address
              row.user_addresses.each do |user_address|
                writer << output_item(row, user_address, 'user_address')
              end
            when :product_sku
              row.skus.each do |sku|
                writer << output_item(row, sku, 'primary_sku')
              end
            when :product_display
              row.product_displays.each do |display|
                writer << output_item(row, display, 'current_display')
              end
            when :product_price
              row.skus.each do |sku|
                sku.prices.each do |price|
                  writer << output_item(row, sku, 'primary_sku', price, ['primary_sku', 'primary_price'])
                end
              end
            when :product_component
              row.skus.each do |sku|
                sku.components.each do |component|
                  writer << output_item(row, sku, 'primary_sku', component, ['primary_sku', 'primary_component'])
                end
              end
            when :order_shipping
              row.order_shippings.each do |shipping|
                  writer << output_item(row, shipping, 'order_shipping')
              end
            when :order_item
              row.order_shippings.each do |shipping|
                shipping.order_items.each do |order_item|
                  writer << output_item(row, shipping, 'order_shipping', order_item, ['order_shipping', 'order_item'])
                end
              end
            when :order_item_component
              row.order_shippings.each do |shipping|
                shipping.order_items.each do |order_item|
                  if order_item.order_item_components.first.nil?
                    # 子商品が無い場合は、自分自身を子商品としてorder_item_componentに渡す。
                    writer << output_item(row, shipping, 'order_shipping', order_item, ['order_shipping', 'order_item'], order_item, ['order_shipping', 'order_item', 'order_item_component'])
                  else
                    order_item.order_item_components.each do |component|
                      writer << output_item(row, shipping, 'order_shipping', order_item, ['order_shipping', 'order_item'], component, ['order_shipping', 'order_item', 'order_item_component'])
                    end
                  end
                end
              end
          end
        end
      end
    end
  end
  
  # 検索結果のオブジェクトから、
  # CsvDownloadPatternItem.codeの:readerで指定されたコードに該当するデータを取り出します。
  #
  # 引数 row_obj:      検索結果の1レコード分
  #      sub1_row_obj: row_objの中でループを回す場合の要素
  #      sub1_row_str: sub1_row_objを指定した場合に、:readerで指定されたコードのうち無視する(上位階層)文字列
  #      sub2_row_obj: sub1_row_objの中でループを回す場合の要素
  #      sub2_row_str: sub2_row_objを指定した場合に、:readerで指定されたコードのうち無視する(上位階層)文字列の配列
  #      sub3_row_obj: sub2_row_objの中でループを回す場合の要素
  #      sub3_row_str: sub3_row_objを指定した場合に、:readerで指定されたコードのうち無視する(上位階層)文字列の配列
  def output_item(row_obj, sub1_row_obj = nil, sub1_row_str = '', sub2_row_obj = nil, sub2_row_str = [], sub3_row_obj = nil, sub3_row_str = [])
    output_items = []
    items.each do |item|
      if item.value.nil?
        # 固定値が設定されていない場合
        code_entry = CsvDownloadPatternItem.code_entry_by_id(item.code)
        sub1_flg = false
        sub2_flg = false
        sub3_flg = false
        value = nil
        code_entry[:reader].split('.').each do |part|
          if part == sub1_row_str
            sub1_flg = true
            next 
          end
          if sub2_row_str.include?(part)
            sub2_flg = true if part == sub2_row_str.last
            next 
          end
          if sub3_row_str.include?(part)
            sub3_flg = true if part == sub3_row_str.last
            next
          end
          if value.nil? && sub1_flg && !sub2_flg && !sub3_flg
            value = sub1_row_obj.send(part)
          elsif value.nil? && sub2_flg && !sub3_flg
            value = sub2_row_obj.send(part)
          elsif value.nil? && sub3_flg
            value = sub3_row_obj.send(part)
          elsif !value.nil?
            value = value.send(part)
          else
            value = row_obj.send(part)
          end
          if value.nil?
            break
          end
        end
        
        formated_value = format_value(code_entry[:format], value)
      else
        # 固定値が設定されている場合
        formated_value = item.value
      end
      output_items << (character_key == :shift_jis ? NKF::nkf('-sx', formated_value.to_s) : formated_value.to_s)
    end
    return output_items
  end
  
  def format_value(format_type, org_value)
    return org_value if format_type.blank? || org_value.blank?
    case format_type
      when :date
        # 日付
        org_value.strftime('%Y/%m/%d')
      when :date_join
        # 日付
        org_value.strftime('%Y%m%d')
      when :date_join_short
        # 日付
        org_value.strftime('%y%m%d')
      when :date_time
        # 日時
        org_value.strftime('%Y/%m/%d %H:%M')
      when :onebyte_kana
        # 全角カタカナ→半角カタカナ
        Moji.zen_to_han(org_value)
    end
  end
  
  def header
    if character_key == :shift_jis
      items.map{|item| NKF::nkf('-s -W', item.name_with_remark)}
    else
      items.map{|item| item.name}
    end
  end

  def rows
    case download_type_key
      when :user
        find_conditions = User.create_condition(User::SearchConditionForm.new(extract_user_condition))
        User.find(:all, :include => [:user_address, :user_email, :rank], :conditions => find_conditions)
      when :user_address
        find_conditions = User.create_condition(User::SearchConditionForm.new(extract_user_condition))
        User.find(:all, :include => [:user_addresses, :user_email, :rank], :conditions => find_conditions)
      when :product, :product_display
        find_conditions = Product::SearchCondition.new(extract_product_condition)
        Product.find(:all, find_conditions.to_find_options(
          :include => [:product_displays, {:primary_sku => {:prices => :user_rank}}], :order => :product_code)
        )
      when :product_sku, :product_price
        find_conditions = Product::SearchCondition.new(extract_product_condition)
        Product.find(:all, find_conditions.to_find_options(
          :include => [:product_displays, {:skus => [{:prices => :user_rank}]}], :order => :product_code)
        )
      when :product_component
        find_conditions = Product::SearchCondition.new(extract_product_condition)
        Product.find(:all, find_conditions.to_find_options(
          :include => [:product_displays, {:skus => [{:prices => :user_rank}, :components]}], :order => :product_code)
        )
      when :order, :order_shipping, :order_item, :order_item_component
        find_conditions = Order::SearchInfo.new(extract_order_condition)
        Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => find_conditions.create_condition, :order => 'orders.id DESC')
      when :accounting
        find_conditions = Accounting::SearchInfo.new(extract_accounting_condition)
        Accounting.find(:all, find_conditions.to_find_options(:order => "accountings.created_at",:include => {:order => :order_shippings}))
    end
  end
end
