class CsvFileUpload
  include SelectableAttr::Base

  attr_accessor :upload_id, :uploader

  def initialize(csv_file_upload)
    @upload_id = csv_file_upload["upload_id"]
    case CsvFileUpload.upload_key_by_id(@upload_id)
    when :product
      @uploader = ProductCsvUploader.new(csv_file_upload)
    when :product_display
      @uploader = ProductDisplayCsvUploader.new(csv_file_upload)
    end
  end

  # アップロード情報
  selectable_attr :upload do
    #entry 1, :user, '会員情報'
    entry 2, :product, '商品情報'
    entry 3, :product_display, '商品説明'
    #entry 4, :inquiry_mail, '問い合わせメール'
  end

  class BaseCsvUploader
    attr_accessor :errors, :success_count, :error_count, :error_rows

    def initialize(csv_file_upload)
      @success_count = 0
      @error_count = 0
      @error_rows = []

      # アップロード情報
      @upload_id = csv_file_upload["upload_id"]
      # アップロードファイル
      @files = csv_file_upload["file"]
      # ファイルチューザーのラベル
      @file_field_labels = file_field_labels
    end

    # アップロード情報の日本語名を返す
    def file_field_labels
      [CsvFileUpload.upload_name_by_id(@upload_id)]
    end
    
    # バリデーション
    def validate?
      @errors = []
      validate_file_content_type
      validate_file_count
      return (@errors.empty?)
    end
    
    # CSVを読み込み配列にして返す
    def parse_csv(file_path)
      return nil unless file_path
      parsed_csv_file = []
      open(file_path).each do |l|
        # Shift_JISの場合は、UTF-8に変換する
        if NKF::SJIS == NKF.guess(l)
          parsed_csv_file << FasterCSV.parse_line(NKF::nkf('-Sw', l))
        else
          parsed_csv_file << FasterCSV.parse_line(l)
        end
      end
      return parsed_csv_file
    end
    
    protected
    # CSVファイル形式チェック
    def validate_file_content_type
      @files.values.each do |file|
        if file.class == File
          unless file.content_type == 'text/csv' || (file.content_type == 'application/vnd.ms-excel' && File::extname(file.original_filename) == '.csv')
            @errors << 'ファイル形式がCSVではありません。'
          end
        end
      end
    end

    # CSVファイル指定チェック
    def validate_file_count
      @errors << 'CSVファイルを選択してください。' unless @files.size == @file_field_labels.size
    end
    
    # CSVから取得した1行分のデータの配列、カラム名の配列、対象のカラム名を指定して、該当するデータを取得する
    def get_value(target_row, column_names, key)
      index = column_names.index(key)
      return index ? target_row[index] : nil
    end
    
    #CSVの各列とテーブル、カラムの紐付け情報を取得する
    def get_column_names(download_file_name)
      pattern = CsvDownloadPattern.find(:first, :conditions => {:file_name => download_file_name})
      result = []
      pattern.items.map do |item|
        code_reader = item.code_entry[:reader].split('.')
        column_name = code_reader.pop
        obj_name = code_reader.pop
        result << "#{obj_name}.#{column_name}"
      end
      return result
    end
    
    # マッピング情報から値を取得して返す（該当する値が見つからない場合は例外を発生）
    def get_mapping_value(map, key, value_at_blank = nil)
      if key.blank?
        return value_at_blank
      elsif map[key]
        return map[key]
      else
        raise MappingError
      end
    end
    
    # エラーの一覧をCSVファイルに出力する
    def generate_csv_file_exclude_success(file_name=nil)
      file_name = CsvFileUpload.upload_key_by_id(@upload_id).to_s unless file_name
      @csv_file_path = "#{EcRider.config.csv_upload_dir}/#{file_name}_#{Time.now.strftime('%Y%m%d%H%M%S')}.csv"
      FasterCSV.open(@csv_file_path, "w") {|writer|
        @error_rows.each do |error_row|
          writer << error_row
        end
      }
    end

    # 取り込み履歴を作成する
    def create_csv_file_upload_history(upload_id=nil)
      upload_id = @upload_id unless upload_id
      CsvFileUploadHistory.create!(
        :upload_id => upload_id,
        :input_count => @success_count + @error_count,
        :success_count => @success_count,
        :error_count => @error_count,
        :csv_file_path => @csv_file_path
      )
    end
  end

  class ProductDisplayCsvUploader < BaseCsvUploader
    def file_field_labels
      ["商品情報（言語単位）"]
    end
    
    def upload
      product_display_csv = parse_csv(@files[file_field_labels[0]])
      product_display_column_names = get_column_names('product_display_bulk_upload_template')

      # 言語ごとの商品説明を登録・更新する
      create_or_update_product_displays(product_display_csv, product_display_column_names)
      
      # アップロード結果を生成する
      generate_csv_file_exclude_success if @error_count > 0
      create_csv_file_upload_history
    end
    
    private
    # 商品説明を登録・更新する
    def create_or_update_product_displays(product_display_csv, column_names)
      product_display_csv.each_with_index do |target_row, i|
        next if i == 0 #タイトル行を読み飛ばす

        # 商品がない場合はエラー
        product = Product.find_by_product_code(get_value(target_row, column_names, '.product_code'))
        unless product
          @error_count += 1
          @error_rows << ['商品説明', "該当する商品がありません"] + target_row
          next
        end
        
        product_display = ProductDisplay.find(:first,
          :conditions => {:product_id => product.id, :locale => get_value(target_row, column_names, 'current_display.locale')})
        product_display ||= ProductDisplay.new()
        product_display.product = product
        
        column_names.each_with_index do |column_name, j|
          attribute_names = column_name.split('.')
          next if attribute_names.size < 2
          # 商品説明
          if attribute_names[0] == 'current_display'
            product_display.send("#{attribute_names[1]}=", target_row[j])
          end
        end

        # 商品説明を登録・更新する
        if product_display.valid?
          product_display.save
          @success_count += 1
        else
          @error_count += 1
          product_display.errors.each_full do |message|
            @error_rows << ['商品説明', message] + target_row
          end
        end
      end
    end
  end
  
  class ProductCsvUploader < BaseCsvUploader
    def initialize(csv_file_upload)
      super(csv_file_upload)
      
      # 削除フラグの立っている商品、商品SKU
      @deleted_product_code = []
      @deleted_sku_code = []
    end
    
    def file_field_labels
      ["商品情報（SKU単位）", "商品情報（売価単位）"]
    end
    
    def upload
      product_sku_csv = parse_csv(@files[file_field_labels[0]])
      product_price_csv = parse_csv(@files[file_field_labels[1]])

      product_sku_column_names = get_product_sku_column_names
      product_price_column_names = get_column_names('product_price_bulk_upload_template')
      
      # 商品＆商品SKUを削除する
      destroy_products(product_sku_csv, product_sku_column_names)
      
      # 商品、商品SKUを登録・更新する
      create_or_update_products(product_sku_csv, product_sku_column_names, product_price_csv, product_price_column_names)
      
      # アップロード結果を生成する
      generate_csv_file_exclude_success if @error_count > 0
      create_csv_file_upload_history
    end

    private
    # CSVから取得した1行分のデータの配列からSKUコードを取得する
    def get_sku_code(target_row, column_names)
      product_code = get_value(target_row, column_names, '.product_code')
      variation_item_1 = get_value(target_row, column_names, 'variation_item_1.code')
      variation_item_1 = "00" if variation_item_1.blank?
      variation_item_2 = get_value(target_row, column_names, 'variation_item_2.code')
      variation_item_2 = "00" if variation_item_2.blank?
      variation_item_3 = get_value(target_row, column_names, 'variation_item_3.code')
      variation_item_3 = "00" if variation_item_3.blank?
      return "#{product_code}-#{variation_item_1}-#{variation_item_2}-#{variation_item_3}"
    end
    
    # 削除フラグが立っている商品、商品SKUを削除する
    def destroy_products(product_sku_csv, column_names)
      product_sku_csv.each_with_index do |target_row, i|
        next if i == 0 #タイトル行を読み飛ばす
        
        # 商品削除
        if get_value(target_row, column_names, 'product_delete_flg') == '1'
          product_code = get_value(target_row, column_names, '.product_code')
          product = Product.find_by_product_code(product_code)
          
          # 配送前の注文に含まれている商品、商品構成の子になっている商品は削除しない
          if product && !product.error_if_unable_to_delete_product
            @error_count += 1
            product.errors.each_full do |message|
              @error_rows << ['商品', message] + target_row
            end
          else
            product.destroy if product
            @success_count += 1
          end
          @deleted_product_code << product_code
        end
        
        # 商品SKU削除
        if get_value(target_row, column_names, 'sku_delete_flg') == '1'
          sku_code = get_sku_code(target_row, column_names)
          product_sku = ProductSku.find_by_sku_code(sku_code)
          
          # 配送前の注文に含まれているSKU、商品構成の子になっているSKUは削除しない
          if product_sku && !product_sku.error_if_unable_to_delete_product_sku
            @error_count += 1
            product_sku.errors.each_full do |message|
              @error_rows << ['商品SKU', message] + target_row
            end
          else
            product_sku.destroy if product_sku
            @success_count += 1
          end
          @deleted_sku_code << sku_code
        end
      end
    end
    
    # 商品、商品SKUを登録・更新する
    def create_or_update_products(product_sku_csv, sku_column_names, product_price_csv, price_column_names)
      category_id_mapping = get_category_code_mapping
      variation_name_mapping = get_variation_name_mapping
      variation_item_code_mapping = get_variation_item_code_mapping
      user_rank_code_mapping = get_user_rank_code_mapping
      product_price_hash = product_price_cvs_to_hash(product_price_csv, price_column_names)
      
      product_sku_csv.each_with_index do |target_row, i|
        next if i == 0 #タイトル行を読み飛ばす
        
        product_code = get_value(target_row, sku_column_names, '.product_code')
        sku_code = get_sku_code(target_row, sku_column_names)
        
        # 削除フラグの立っている行は無視する
        if @deleted_product_code.include?(product_code) || @deleted_sku_code.include?(sku_code)
          product_price_hash.delete(sku_code) if product_price_hash[sku_code]
          next
        end
        
        product = Product.find_by_product_code(product_code)
        product_sku = ProductSku.find_by_sku_code(sku_code)
        product ||= Product.new(:course_div_key => :normal)
        product_sku ||= ProductSku.new()
        product_sku.product = product
        product_sku_inventory = nil
        
        error_messages = []
        sku_column_names.each_with_index do |column_name, j|
          attribute_names = column_name.split('.')
          next if attribute_names.size < 2
          # 商品
          if attribute_names[0] == ''
            product.send("#{attribute_names[1]}=", target_row[j])
            # 商品SKU
          elsif attribute_names[0] == 'primary_sku'
            product_sku.send("#{attribute_names[1]}=", target_row[j])
            # カテゴリ
          elsif attribute_names[0] =~ /^category_[1-5]$/
            begin 
              product.send("#{attribute_names[0]}_id=", get_mapping_value(category_id_mapping, target_row[j], nil))
            rescue MappingError
              error_messages << ['商品', "「#{target_row[j]}」は存在しないカテゴリコードです"]
            end
            # バリエーション
          elsif attribute_names[0] =~ /^variation_[1-3]$/
            variation_index = /^variation_([1-3])$/.match(attribute_names[0]).to_a[1]
            begin 
              variation_id = get_mapping_value(variation_name_mapping, target_row[j], 0)
              if product.new_record? || product.send("#{attribute_names[0]}_id") == variation_id
                product.send("#{attribute_names[0]}_id=", get_mapping_value(variation_name_mapping, target_row[j], 0))
              else
                error_messages << ['商品', "バリエーション#{variation_index}は変更できません"]
              end
            rescue MappingError
              error_messages << ['商品', "「#{target_row[j]}」は存在しないバリエーション名です"]
            end
            # バリエーション項目
          elsif attribute_names[0] =~ /^variation_item_[1-3]$/
            variation_index = /^variation_item_([1-3])$/.match(attribute_names[0]).to_a[1]
            variation_name = get_value(target_row, sku_column_names, "variation_#{variation_index}.name_for_admin")
            if variation_item_code_mapping[variation_name]
              begin 
                raise MappingError if variation_item_code_mapping[variation_name] && target_row[j].blank?
                product_sku.send("#{attribute_names[0]}_id=", get_mapping_value(variation_item_code_mapping[variation_name], target_row[j], nil))
              rescue MappingError
                error_messages << ['商品', "「#{variation_name}（#{target_row[j]}）」は存在しないバリエーション項目名です"]
              end
            end
            # 在庫数
          elsif attribute_names[0] == 'inventory'
            unless target_row[j] =~ /^\-?\+?[0-9]{1,5}$/
              error_messages << ['商品', "在庫数増減の値が不正です"]
            end
            stock_count_diff = target_row[j].to_i
            if stock_count_diff == 0
              # 0の場合は在庫数を変更しない
            elsif stock_count_diff > 0 || product_sku.orderable_count >= stock_count_diff.abs
              product_sku_inventory = ProductSkuInventory.new(:count => stock_count_diff, :comment => '商品CSVアップロードで変更')
              product_sku_inventory.product_sku = product_sku
              product_sku.parent_stock_count += stock_count_diff
            else
              error_messages << ['商品', "減らせる在庫数は#{product_sku.stock_count-product_sku.orderable_count}個までです"]
            end
          end
        end
        
        # 商品売価
        product_prices = []
        if product_price_hash[sku_code] && (user_rank_code_mapping.size == product_price_hash[sku_code].size)
          product_price_hash[sku_code].values.each do |price_target_row|
            product_price = ProductPrice.new()
            product_price.product_sku = product_sku
            
            price_column_names.each_with_index do |price_column_name, k|
              price_attribute_names = price_column_name.split('.')
              next if price_attribute_names.size < 2
              # 商品売価
              if price_attribute_names[0] == 'primary_price'
                product_price.send("#{price_attribute_names[1]}=", price_target_row[k])
                # 会員ランク
              elsif price_attribute_names[0] == 'user_rank'
                begin 
                  raise MappingError if price_target_row[k].blank?
                  product_price.user_rank_id = get_mapping_value(user_rank_code_mapping, price_target_row[k], nil)
                rescue MappingError
                  error_messages << ['商品売価', "「#{price_target_row[k]}」は存在しない会員ランクコードです"]
                end
              end
            end
            # バリデーションチェック
            if product_price.valid?
              product_prices << product_price
            else
              product_price.errors.each_full do |message|
                error_messages << ['商品売価', message]
              end
            end
          end
        else
          error_messages << ['商品売価', "商品売価を正しく設定してください"]
        end
        product_price_hash.delete(sku_code)
        
        # 商品、商品SKUを登録・更新する
        if product.valid? && product_sku.valid? && error_messages.empty?
          Product.transaction do
            product.save!
            
            #同じ商品のほかのSKUに代表SKUが無ければ自身が代表SKUになる
            primary_sku = ProductSku.find(:first, :conditions => {:product_id => product.id, :primary_sku_flg => true})
            product_sku.primary_sku_flg = true unless primary_sku
            product_sku.save!
            
            #自身が代表SKUの場合は同じ商品のほかのSKUの代表SKUフラグを外す
            ProductSku.update_all({:primary_sku_flg => false}, ['id <> ? AND product_id = ?', product_sku.id, product.id]) if product_sku.primary_sku_flg
            
            product_sku.prices.each do |old_product_price|
              old_product_price.destroy
            end
            product_prices.each do |new_product_price|
              new_product_price.save!
            end
            
            product_sku_inventory.save! if product_sku_inventory
          end
          @success_count += 1
        else
          @error_count += 1
          product.errors.each_full do |message|
            @error_rows << ['商品', message] + target_row
          end
          product_sku.errors.each_full do |message|
            @error_rows << ['商品SKU', message] + target_row
          end
          error_messages.each do |error_message|
            @error_rows << error_message + target_row
          end
        end
      end
      
      # 商品データが存在しない商品売価
      product_price_hash.keys.each do |sku_code|
        @error_rows << ['商品売価', "該当する商品SKU「#{sku_code}」が商品情報（SKU単位）CSVファイルに存在しません"]
      end
    end
    
    #カテゴリコードとidのマッピング情報を生成する
    def get_category_code_mapping
      result = {}
      categories = ProductCategory.find(:all, :select => 'id, code')
      categories.each do |category|
        result[category.code.to_s] = category.id
      end
      return result
    end
    
    #バリエーション名とidのマッピング情報を生成する
    def get_variation_name_mapping
      result = {}
      variations = ProductVariation.find(:all, :select => 'id, name_for_admin')
      variations.each do |variation|
        result[variation.name_for_admin] = variation.id
      end
      return result
    end
    
    #バリエーション項目コードとidのマッピング情報を生成する
    def get_variation_item_code_mapping
      result = {}
      variation_items = ProductVariationItem.find(:all, :include => :product_variation)
      variation_items.each do |variation_item|
        result[variation_item.product_variation.name_for_admin] ||= {}
        result[variation_item.product_variation.name_for_admin][variation_item.code] = variation_item.id
      end
      return result
    end
    
    #会員ランクコードとidのマッピング情報を生成する
    def get_user_rank_code_mapping
      result = {}
      user_ranks = UserRank.find(:all, :select => 'id, code')
      user_ranks.each do |user_rank|
        result[user_rank.code] = user_rank.id
      end
      return result
    end
    
    #商品/商品SKUのCSVの各列とテーブル、カラムの紐付け情報を生成する
    def get_product_sku_column_names
      pattern = CsvDownloadPattern.find(:first, :conditions => {:file_name => 'product_sku_bulk_upload_template'})
      result = []
      pattern.items.map do |item|
        model_column = nil
        case item.display_order
        when 0
          model_column = 'product_delete_flg'
        when 1
          model_column = 'sku_delete_flg'
        when 2
          model_column = 'inventory.count_diff'
        else
          code_reader = item.code_entry[:reader].split('.')
          column_name = code_reader.pop
          obj_name = code_reader.pop
          model_column = "#{obj_name}.#{column_name}"
        end
        result << model_column
      end
      return result
    end
    
    #CSVから取得した商品価格データをSKUコードをキーにしたハッシュに成形する
    def product_price_cvs_to_hash(product_price_csv, column_names)
      result = {}
      product_price_csv.each_with_index do |target_row, index|
        next if index == 0 #タイトル行を読み飛ばす
        sku_code = get_value(target_row, column_names, "primary_sku.sku_code")
        user_rank_code = get_value(target_row, column_names, "user_rank.code")
        result[sku_code] ||= {}
        result[sku_code][user_rank_code] = target_row
      end
      return result
    end
  end
  
  # マッピングエラー
  # 存在しないコードが指定された際に発生する
  class MappingError < StandardError
  end
end
