# -*- coding: utf-8 -*-
class Inquiry < ActiveRecord::Base
  include EcRider::ModelUtility

  belongs_to :user
  belongs_to :country, :foreign_key => 'country_code'
  has_many :inquiry_products, :dependent => :destroy
  has_many :inquiry_memos, :dependent => :destroy
  before_create :associate_user

  accepts_nested_attributes_for :inquiry_products, :inquiry_memos

  # 問い合わせ区分
  # 追加、変更する場合はec_front/app/views/inquiry/inquiry.rhtmlのスクリプトも修正が必要
  selectable_attr :inquiry_div do
    entry '1', :use_site, 'サイトの使い方について'
    entry '2', :order, 'ご注文について'
    entry '3', :stocks, '在庫について'
    entry '4', :payment, 'お支払いについて'
    entry '5', :deliver, '商品のお届けについて'
    entry '6', :product, '商品について'
    entry '7', :cansel, 'キャンセル・返品について'
    entry '8', :member, '会員登録について'
    entry '99',:other, 'その他のお問合せ'
  end

  selectable_attr :status_div do
    entry '0', :not_yet, '未対応'
    entry '1', :done, '対応済み'
    entry '2', :waiting, '返答待ち'
  end

  # 関連商品全削除有無フラグ
  attr_accessor :targets_modified

  #未入力バリデーション
  validates_presence_of :email
  validates_presence_of :name_last
  validates_presence_of :name_first
  validates_presence_of :tel_num
  validates_presence_of :inquiry
  validates_presence_of :status_div

  # 日本語住所で必須
  validates_presence_of :kana_name_last, :if => :is_japanese_format?
  validates_presence_of :kana_name_first, :if => :is_japanese_format?

  #文字数バリデーション
  validates_length_of :name_last,  :in => 0..30, :allow_nil => true
  validates_length_of :name_first, :in => 0..30, :allow_nil => true
  validates_length_of :inquiry, :maximum => 1000, :message => :EM990012, :allow_nil => true

  # メールアドレス
  validates_format_of :email,  :with => /^([a-zA-Z0-9])+([a-zA-Z0-9\._-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/, :message => :EM990007, :if => Proc.new { |obj| !obj.email.blank? }

  # カナ
  validates_format_of :kana_name_last,  :with => /^[ァ-ヶ・ーヽヾ]+$/, :message => :EM990006, :if => :is_japanese_format?
  validates_format_of :kana_name_first, :with => /^[ァ-ヶ・ーヽヾ]+$/, :message => :EM990006, :if => :is_japanese_format?

  # 電話番号の桁数
  validates_length_of :tel_num, :in => 12..13, :allow_nil => true, :if => :is_japanese_format?
  validates_format_of :tel_num, :with => /^\d{2,5}-\d{1,4}-\d{4}$/, :message => :EM990007, :if => :is_japanese_format?
  validates_format_of :tel_num, :with => /^[\-\+0-9]*$/, :message => :EM990007

  #（半角英数＋半角記号＋半角スペース）
  validates_format_of :name_last,  :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/, :message => :EM990028, :if => :is_english_format?
  validates_format_of :name_first, :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]+$/, :message => :EM990028, :if => :is_english_format?
  validates_format_of :inquiry, :with => /^[A-Za-z0-9,_\?\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :is_english_format?
  validates_format_of :image_url_1, :with => /^[A-Za-z0-9,_:\?\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :image_url_1?
  validates_format_of :image_url_2, :with => /^[A-Za-z0-9,_:\?\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :image_url_2?
  validates_format_of :image_url_3, :with => /^[A-Za-z0-9,_:\?\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :image_url_3?
  validates_format_of :image_url_4, :with => /^[A-Za-z0-9,_:\?\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :image_url_4?
  validates_format_of :image_url_5, :with => /^[A-Za-z0-9,_:\?\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :image_url_5?

  # (ファイルタイプ)
  validates_format_of :image_url_1, :with => /(\.jpg|\.gif|\.png)$/i, :message => :EM070005, :if => :image_url_1?
  validates_format_of :image_url_2, :with => /(\.jpg|\.gif|\.png)$/i, :message => :EM070005, :if => :image_url_2?
  validates_format_of :image_url_3, :with => /(\.jpg|\.gif|\.png)$/i, :message => :EM070005, :if => :image_url_3?
  validates_format_of :image_url_4, :with => /(\.jpg|\.gif|\.png)$/i, :message => :EM070005, :if => :image_url_4?
  validates_format_of :image_url_5, :with => /(\.jpg|\.gif|\.png)$/i, :message => :EM070005, :if => :image_url_5?

  def is_japanese_format?
    (locale == 'ja' && (country_code.blank? || country_code == 'JP'))
  end

  def is_chinese_format?
    locale == 'zh_CN' && !is_japanese_format? && EcRider.config.chinese_format_using
  end
  
  def is_english_format?
    locale != 'ja' && !is_japanese_format? && !is_chinese_format?
  end

  before_validation :set_test_flg

  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  # メールアドレスを元に、登録済みユーザと紐付ける
  def associate_user
    emails = UserEmail.find(:all, :conditions =>["email = ?", self.email])
    emails.each do |email|
      self.user_id = email.user_id
      break if email.user.member_div_key == :membership
    end
  end

  # ファイルサイズのバリデーション
  def self.file_validate(images = nil)
    images.each_key do |key|
      if key =~ /^image_url_/
        if !images[key].nil? && images[key].size? > 1000000
          errors.add(key, :message => :EM070004)
          return false
        end
      end
    end
    return true
  end

  #
  # 検索フォーム
  #
  class SearchInfo < EcRider::ConditionForm
    attr_accessor :date_from, :date_to, :inquiry_div, :status_div, :disp_count, :inquiry, :answer, :memo, :test_flg

    # 日付フォーマット
    validates_format_of :date_from, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => Proc.new { |obj| !obj.date_from.blank? }
    validates_format_of :date_to,   :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => Proc.new { |obj| !obj.date_to.blank? }

    #
    # = 検索条件を作成します。 =
    #
    def to_find_options(options = nil)
      joins = []
      where = Condition.block do |c|
        c.filter(:ignore_blank)
        # 登録日Fromの条件
        c.and("inquiries.created_at", ">=", Time.parse(date_from + " 00:00:00")) unless date_from.blank?
        # 登録日Toの条件
        c.and("inquiries.created_at", "<=", Time.parse(date_to + " 23:59:59")) unless date_to.blank?
        # 問合せ条件
        c.and(:inquiry_div, inquiry_div)
        # 対応区分せ条件
        c.and(:status_div, status_div)
        # 問合せ内容条件
        c.and("inquiries.inquiry", "like", "%#{inquiry}%" )
        # 回答内容条件
        c.and("inquiries.answer", "like", "%#{answer}%" )
        # 問合せメモ条件
        c.and("inquiry_memos.memo", "like", "%#{memo}%" )

        # テスト問合せ条件
        if test_flg == CmsPublish.target_id_by_key(:test)
          c.and("sql", "test_flg = '1'")
        elsif test_flg == CmsPublish.target_id_by_key(:production)
          c.and("sql", "test_flg IS NULL")
        end
      end
      result = {:conditions => where}
      result[:joins] = joins.uniq.join(" ")
      result.update(options) if options
      return result
    end

    def validate
      unless date_from.blank?
        _date = date_from.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:date_from, :EM990007)
        end
      end
      unless date_to.blank?
        _date = date_to.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:date_to, :EM990007)
        end
      end
    end
  end

  # 削除フラグ付与処理
  def old_product_mark_for_destruction(modified = targets_modified)
    if ActiveRecord::ConnectionAdapters::Column.value_to_boolean(modified)
      self.inquiry_products.each do |target|
        target.mark_for_destruction
      end
    end
  end

  #
  # =フルネームを連結して返す=
  #
  def fullname
    if is_japanese_format?
      return "#{self.name_last} #{self.name_first}"
    else
      return "#{self.name_first} #{self.name_last}"
    end
  end

  #
  # テストデータかどうかを返す
  #
  def test?
    test_flg ? test_flg : false
  end
end
