# -*- coding: utf-8 -*-
class Order < ActiveRecord::Base

  #
  # =SearchInfo=
  #
  # 検索条件を保持するためのクラス
  # 下記のアクセッサーを持つ
  #
  class SearchInfo < ActiveForm
    include EcRider::PluginModule::Extendable

    include SelectableAttr::Base
    
    ATTRIBUTE_NAMES = [
      :order_date_start,
      :order_date_end,
      :shipping_date_start,
      :shipping_date_end,
      :order_code,
      :order_shipping_code,
      :status_div,
      :name_first,
      :tel_num,
      :delivery_kanji_name,
      :delivery_tel_num,
      :total_amount,
      :upper_total_amount,
      :product_code,
      :qty_from,
      :qty_to,
      :email,
      :payment_type_div,
      :free_request_div,
      :send_count_div,
      :disp_count,
      :test_flg,
      :delivery_to_native_flg,
      :delivery_to_foreign_flg,
      :display_summary
    ]
    
    # ec_frontの履歴表示用検索条件
    ALL_ATTRIBUTE_NAMES = [:user_id, :successful_only_flg].concat(ATTRIBUTE_NAMES)
    
    attr_accessor *ALL_ATTRIBUTE_NAMES

    # 注文ステータス
    selectable_attr :status_div, :enum => OrderShipping.status_div_enum
    
    # 注文ステータス タブ
    selectable_attr :status_tab do
      entry nil, :all, "全注文", :show_status_column => true
      [OrderShipping::STATUS_REGULAR_TRANSITION, OrderShipping::STATUS_CANCEL_TRANSITION].each do |keys|
        keys.each do |key|
          div_entry = OrderShipping.status_div_entry_by_key(key)
          entry(div_entry.id, div_entry.key, div_entry.name,
            :name_for_front => div_entry[:name_for_front],
            :series => keys,
            :has_next => (key != keys.last), 
            :has_prev => (key != keys.first))
        end
      end
      entry '99', :error, "注文エラー", :div_keys => OrderShipping::STATUS_ERRORS, :show_status_column => true
    end
    
    # 指定された注文ステータスのみ、その前、その後
    selectable_attr :tab_inclusion do
      entry '0', :selected_one   , '選択されたもののみ'
      entry '1', :before_selected, '選択されたもの以前'
      entry '2', :after_selected , '選択されたもの以後'
    end

    # 自由記入欄入力区分
    selectable_attr :free_request_div do
      entry '0', :without_input, '入力なし'
      entry '1', :with_input, '入力あり'
    end

    # お届先相違区分
    selectable_attr :send_difference_div do
      entry '0', :same, 'お届先と登録住所が同一'
      entry '1', :diff, 'お届先と登録住所が相違'
    end

    # お届先数区分
    selectable_attr :send_count_div do
      entry '0', :single, 'お届先が１箇所'
      entry '1', :plural, 'お届先が２箇所以上'
    end

    # 開始日の入力制限(yyyy/MM/dd)
    validates_format_of :order_date_start, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :order_date_start_unempty?

    # 終了日の入力制限(yyyy/MM/dd))
    validates_format_of :order_date_end,   :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :order_date_end_unempty?

    def order_date_start_unempty?; order_date_start && !order_date_start.empty?; end
    def order_date_end_unempty?; order_date_end && !order_date_end.empty?; end

    #
    # バリデーション
    #
    def validate
      # 金額指定の矛盾チェック
      unless upper_total_amount.blank?
        if total_amount.to_i > upper_total_amount.to_i
          errors.add(:upper_total_amount, :EM200001)
        end
      end
      # 日付の存在チェック
      if order_date_start_unempty?
        _date = order_date_start.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:order_date_start, :EM990007)
        end
      end
      if order_date_end_unempty?
        _date = order_date_end.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:order_date_end, :EM990007)
        end
      end
      #購入数量の矛盾チェック
      if qty_to
        unless qty_to.empty?
          if !qty_from.empty? && (qty_from.to_i > qty_to.to_i)
            errors.add(:qty_to, :EM200001)
          end
        end
      end
    end

    #
    # = 注文情報検索の結合文作成 =
    #
    # create_conditionのためのinclude句、またはjoin句を返します
    # （select句を使用する場合はincludeではなくjoinを使う必要がある）
    #
    def self.create_join(use_inner_join=false)
      outer_join = use_inner_join ? '' : 'LEFT OUTER'
      return <<-END
          #{outer_join} JOIN order_shippings ON orders.id = order_shippings.order_id AND order_shippings.deleted_at IS NULL
          #{outer_join} JOIN order_items ON order_shippings.id = order_items.order_shipping_id AND order_items.deleted_at IS NULL
          LEFT OUTER JOIN product_skus ON order_items.product_sku_id = product_skus.id AND product_skus.deleted_at IS NULL
          LEFT OUTER JOIN users ON orders.user_id = users.id AND users.deleted_at IS NULL
      END
    end
    def self.create_include
      # :includeの順番に注意！
      # 順番を変更すると例外が発生することもあります。
      # ecr_payment_gmoで完全にこの戻り値を上書きしています。実装を変更する際には、プラグインも変更してください。
      return [{:order_shippings => [:order, {:order_items => :product_sku}]}, :user]
    end

    #
    # = 注文情報検索の条件文作成 =
    #
    def create_condition
      instance_variables.each do |var_name|
        value = instance_variable_get(var_name)
        remove_instance_variable(var_name) if value.blank?
      end

      where = Condition.block do |c|
        # 注文日時(開始)の条件
        c.and("orders.ordered_at", ">=", Time.parse(order_date_start + " 00:00:00")) if order_date_start
        # 注文日時(終了)の条件
        c.and("orders.ordered_at", "<=", Time.parse(order_date_end + " 23:59:59")) if order_date_end
        # 配送予定日(開始)の条件
        c.and("order_shippings.arrival_date", ">=", Time.parse(shipping_date_start + " 00:00:00")) if shipping_date_start
        # 配送予定日(終了)の条件
        c.and("order_shippings.arrival_date", "<=", Time.parse(shipping_date_end + " 23:59:59")) if shipping_date_end
        # 注文コードの条件
        c.and("orders.ec_order_code", "like", "%#{order_code}%") if order_code
        # 配送コードの条件
        c.and("order_shippings.delivery_code", "like", "%#{order_shipping_code}%") if order_shipping_code

        # テスト注文の条件
        if test_flg == CmsPublish.target_id_by_key(:test)
          c.and("orders.test_flg", "1")
        elsif test_flg == CmsPublish.target_id_by_key(:production)
          c.and("sql", "orders.test_flg IS NULL")
        end

        # 注文者の条件
        if name_first
          c.and do |c1|
            _name = delete_space(name_first)
            c1.or("CONCAT(users.name_last, users.name_first)", "like", "%#{_name}%")
            c1.or("CONCAT(users.name_first, users.name_last)", "like", "%#{_name}%")
            c1.or("CONCAT(users.kana_name_last, users.kana_name_first)", "like", "%#{_name}%")
            c1.or("CONCAT(users.kana_name_first, users.kana_name_last)", "like", "%#{_name}%")
          end
        end
        
        # 商品名、商品コードの条件
        if product_code
          c.and do |c7|
            c7.or("product_skus.sku_code", 'like' , "%#{product_code}%")
            c7.or("order_items.product_name", 'like' , "%#{product_code}%")
            c7.or("order_items.order_product_name_1", 'like' , "%#{product_code}%")
            c7.or("order_items.order_product_name_2", 'like' , "%#{product_code}%")
          end
        end
        # 受注金額の条件
        c.and("order_shippings.total_amount", ">=", total_amount.to_i) if total_amount
        c.and("order_shippings.total_amount", "<=", upper_total_amount.to_i) if upper_total_amount
        # 数量の条件
        c.and("order_items.qty", ">=" ,qty_from) if qty_from
        c.and("order_items.qty", "<=" ,qty_to) if qty_to
        # 電話番号の条件
        c.and("replace(users.tel_num, '-', '')", "like", "%#{tel_num.gsub("-", "")}%") if tel_num
        # 状態の条件
        c.and("order_shippings.status_div", status_div) if status_div
        
        unless status_tab.blank?
          selected_status_div_ids = self.selected_status_div_keys.map{|key| OrderShipping.status_div_id_by_key(key)}
          c.and("sql", "order_shippings.status_div in (#{selected_status_div_ids.join(',')})")
        end

        # お届け先氏名の条件
        if delivery_kanji_name
          c.and do |c2|
            _name = delete_space(delivery_kanji_name)
            c2.or("CONCAT(order_shippings.name_last, order_shippings.name_first)", "like", "%#{_name}%")
            c2.or("CONCAT(order_shippings.name_first, order_shippings.name_last)", "like", "%#{_name}%")
            c2.or("CONCAT(order_shippings.kana_name_last, order_shippings.kana_name_first)", "like", "%#{_name}%")
            c2.or("CONCAT(order_shippings.kana_name_first, order_shippings.kana_name_last)", "like", "%#{_name}%")
          end
        end
        # お届け先電話番号の条件
        c.and("replace(order_shippings.tel_num, '-', '')", "like", "%#{delivery_tel_num.gsub("-", "")}%") if delivery_tel_num
        # メールアドレスの条件
        c.and("order_shippings.email", "like", "%#{email}%") if email
        # 支払方法の条件
        c.and("order_shippings.payment_type_div", payment_type_div) if payment_type_div
        # 自由記入欄の条件
        if free_request_div
          if free_request_div == Order::SearchInfo.free_request_div_id_by_key(:without_input)
            c.and do |c3|
              c3.or("LENGTH(orders.free_request)", "=", 0)
              c3.or("sql", "LENGTH(orders.free_request) IS NULL")
            end
          else
            c.and("LENGTH(orders.free_request)", ">", 0)
          end
        end

        # 配送区分
        unless (delivery_to_native_flg == delivery_to_foreign_flg)
          # 国内配送の条件
          # ec_adminを多言語化する場合、"JP"(日本)固定ではなくなるかもしれません。
          if delivery_to_native_flg == "1"
            c.and("order_shippings.country_code", "=", "JP")
          end
          # 海外配送の条件
          if delivery_to_foreign_flg == "1"
            c.and("order_shippings.country_code", "!=", "JP")
          end
        end

        # お届先数の条件
        if send_count_div
          if send_count_div == Order::SearchInfo.send_count_div_id_by_key(:single)
            c.and("sql", "1 = (SELECT COUNT(order_id) FROM order_shippings SUB WHERE order_shippings.order_id = SUB.order_id)")
          else
            c.and("sql", "1 < (SELECT COUNT(order_id) FROM order_shippings SUB WHERE order_shippings.order_id = SUB.order_id)")
          end
        end

        # order_shippingsの論理削除の条件
        c.and do |c5|
          c5.or("sql", "order_shippings.deleted_at IS NULL")
          c5.or("sql", "order_shippings.deleted_at > '#{Time.now().strftime('%Y-%m-%d %H:%M:%S')}'")
        end
        c.and("sql", "order_shippings.id IS NOT NULL")

        # order_itemsの論理削除の条件
        c.and do |c6|
          c6.or("sql", "order_items.deleted_at IS NULL")
          c6.or("sql", "order_items.deleted_at > '#{Time.now().strftime('%Y-%m-%d %H:%M:%S')}'")
        end
        c.and("sql", "order_items.id IS NOT NULL")

        # 会員IDの条件
        c.and("orders.user_id", user_id.to_i) if user_id
        
        # 会員の注文履歴に表示されるステータス
        # (正常系 + キャンセル)のみ
        if successful_only_flg
          successful_div = OrderShipping.status_div_ids(*(
              OrderShipping::STATUS_REGULAR_TRANSITION +
                OrderShipping::STATUS_CANCEL_TRANSITION))
          c.and("sql", "order_shippings.status_div in (#{successful_div.join(',')})")
        end
      end
      return where
    end

    def selected_tab_keys
      case status_tab_key
      when :all, :error then
        [status_tab_key]
      else
        selected_status_div_keys
      end
    end

    def selected_status_div_keys
      return nil if status_tab_key == :all
      result = status_tab_entry[:div_keys]
      return result if result
      series = status_tab_entry[:series]
      idx = series.index(status_tab_key)
      case tab_inclusion_key
      when :before_selected
        series[0..idx]
      when :after_selected then
        series[idx..-1]
      else
        [status_tab_key]
      end
    end

    def order_counts_by_tab
      unless @order_counts_by_tab
        status_div_bak = self.status_div
        status_tab_bak = self.status_tab
        successful_only_flg_bak = self.successful_only_flg
        begin
          self.status_div = nil
          self.status_tab = nil
          self.successful_only_flg = nil
          
          counts = Order.count('order_shippings.id',
            :joins => SearchInfo.create_join(true),
            :conditions => self.create_condition,
            :distinct => 'order_shippings.id',
            :group => 'order_shippings.status_div')

          ActiveRecord::Base.logger.debug("counts:" << counts.inspect)

          @order_counts_by_tab = Hash.new{|hash, key| hash[key] = 0}
          counts.each do |status_div, count|
            @order_counts_by_tab[:all] += count
            key = OrderShipping.status_div_key_by_id(status_div)
            if OrderShipping::STATUS_REGULAR_TRANSITION.include?(key) ||
                OrderShipping::STATUS_CANCEL_TRANSITION.include?(key)
              @order_counts_by_tab[key] += count
            elsif OrderShipping::STATUS_ERRORS.include?(key)
              @order_counts_by_tab[:error] += count
            end
          end
          ActiveRecord::Base.logger.debug("@order_counts_by_tab:" << @order_counts_by_tab.inspect)
        ensure
          self.status_div = status_div_bak
          self.status_tab = status_tab_bak
          self.successful_only_flg = successful_only_flg_bak
        end
      end
      @order_counts_by_tab
    end


    private
    #
    # 指定の文字列のスペースを削除します。
    #
    def delete_space(str)
      # 半角スペースと全角スペースを削除
      str.gsub(' ', '').gsub('　', '')
    end
  end

end
