# -*- coding: utf-8 -*-
class OrderForm < ActiveForm
  include SelectableAttr::Base
  include ActiveRecord::Validations

  attr_accessor( :send_address_index, :normal_payment_type, :mail_payment_type, 
    :gift_flg, :send_invoice_separately_flg, :write_sender_flg, :free_request,
    :card_kind, :no, :no_1, :no_2, :no_3, :no_4, :expire_year, :expire_month, :card_name,
    :arrival_date, :arrival_hour,
    :sent_one_place, :has_credit, # 複数送付の場合に利用
    :expire, # 有効期限のエラーメッセージ用
    :include_order_id, :delivery_date, :delivery_time,
    :campaign_attached_div,
    :campaign_attached_at,   # 計算時にいつの時点のキャンペーンを適用するのかを指定可能です。
    # 指定されてなかった場合は現時刻(注文時間)が使用されます。
    :use_point_flg, :use_point, :before_total_point, :after_total_point,
    :cvs_kind, :card_security_code
  )

  # カードの種類
  selectable_attr :card_kind do
    update_with_kbn KbnCode.name_div_id_by_key(:card_kind)
    entry 'visa'  , :visa  , 'Visa'           , :as_amex => false
    entry 'master', :master, 'MasterCard'     , :as_amex => false
    entry 'amex'  , :amex  , 'AmericanExpress', :as_amex => true
    entry 'diners', :diners, 'DinersClub'     , :as_amex => true
    entry 'jcb'   , :jcb   , 'JCB'            , :as_amex => false
  end

  # TODO コンビニの種類が決済会社によって変る場合は、プラグインに移す等の検討が必要です。
  selectable_attr :cvs_kind do
    entry '00001', :lawson  , 'ローソン'
    entry '00002', :family  , 'ファミリーマート'
    entry '00003', :sunkus  , 'サンクス'
    entry '00004', :circlek , 'サークルＫ'
    entry '00005', :ministop, 'ミニストップ'
    entry '00006', :yamazaki, 'デイリーヤマザキ'
    entry '00007', :seven   , 'セブンイレブン'
  end

  # キャンペーン対象区分
  selectable_attr :campaign_attached_div, :default => 0 do
    entry '0', :now, '現在時刻'
    entry '1', :select, '選択した日時'
  end

  selectable_attr :normal_payment_type, :enum => OrderShipping.payment_type_div_enum
  selectable_attr :mail_payment_type  , :enum => OrderShipping.payment_type_div_enum

  # 必須項目
  validates_presence_of :send_address_index, :if => :sent_one_place

  #文字数バリデーション
  validates_length_of   :free_request, :maximum => 1000, :message => :EM990012, :if => :check_free_request?
  #（半角英数＋半角記号＋半角スペース）
  validates_format_of :free_request, :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :is_english_format?

  validates_presence_of :use_point, :if => :use_point?
  validates_numericality_of :use_point, :only_integer => true, :allow_nil => true, :if => :use_point?, :greater_than_or_equal_to => 0

  # カード情報バリデーションチェック
  with_options(:if => :credit_card?) do |card|
    card.validates_presence_of :card_kind, :message => :EM020001
    card.validates_presence_of :expire_year, :message => :EM990001
    card.validates_presence_of :expire_month, :message => :EM990001
    card.validates_presence_of :card_name, :message => :EM990001
    card.validates_presence_of :card_security_code, :message => :EM990001, :if => :credit_card_use_security_code?
    card.validates_numericality_of :card_security_code, :only_integer => true, :allow_nil => true
    card.validates_length_of       :card_security_code, :within => 3..4, :too_long => :EM990045, :too_short => :EM990045, :allow_nil => true
  end

  # カード番号チェック（「AMEX」でない場合）
  with_options(:if => Proc.new { |f| f.credit_card_not_amex? && f.no.nil?}) do |card|
    card.validates_format_of :no_1, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_2, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_3, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_4, :with => /\d{4}/, :message => :EM990013
  end
  # カード番号チェック（「AMEX」の場合）
  with_options(:if => Proc.new { |f| f.credit_card_amex? && f.no.nil?}) do |card|
    card.validates_format_of :no_1, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_2, :with => /\d{6}/, :message => :EM990015
    card.validates_format_of :no_3, :with => /^[0-9]{4,5}$/, :message => :EM990019
  end
  # カード番号チェックの場合(テキストフィールドの場合)
  with_options(:if => Proc.new { |f| f.credit_card? && f.no.blank? && f.no_1.nil? }) do |card|
    card.validates_presence_of :no, :message => "カード番号を入力してください。"
  end

  # 有効期限
  validates_format_of :expire_year,  :with => /^[0-9]{2}$/, :message => :EM990033, :if => :expire_year_check?
  validates_format_of :expire_month, :with => /^[0-9]{2}$/, :message => :EM990033, :if => :expire_month_check?
  # 有効期限の存在チェック
  def validate
    if normal_payment_type_key == :credit
      if !expire_year.nil? && !expire_month.nil?
        if !expire_year.empty? || !expire_month.empty?
          unless Date::exist?(expire_year.to_i, expire_month.to_i, 1)
            errors.add(:expire, :EM990007)
          end
        end
      end
    end
    if use_point?
      if (self.before_total_point.to_i < self.use_point.to_i)
        errors.add(:use_point, :EM990007)
      end
    end
  end

  # ポイントを利用するにチェックがついている場合、trueを返却します。
  # 利用しないにチェックがついている場合、またはどちらにもチェックがついていない場合、
  # falseを返却します。
  def use_point?
    if self.use_point_flg == "true"
      return true
    end
    self.use_point = 0
    return false
  end

  # コンビニ決済の場合、合計金額が下記範囲内かどうかをチェックし、範囲内ならtrueを返します。
  #   セブンイレブン以外：1円以上、300,000円未満
  #   セブンイレブン：200円以上、300,000円未満
  def total_price_valid?(total_price)
    return true unless normal_payment_type_key == :cvs
    min_price = cvs_kind_key == :seven ? 200 : 1
    return true if total_price >= min_price && total_price < 300000
    errors.add(:base, :EM020017)
    return false
  end

  # クレジットカード払いの場合はtrueを返す
  def credit_card?
    (normal_payment_type_key == :credit) || (mail_payment_type_key == :credit) || has_credit
  end
  # クレジットカード払いかつ、「AMEX」「Diners」の場合はtrueを返す
  def credit_card_amex?
    credit_card? && card_kind_entry[:as_amex]
  end
  # クレジットカード払いかつ、「AMEX」「Diners」以外の場合はtrueを返す
  def credit_card_not_amex?
    credit_card? && !card_kind_entry[:as_amex]
  end

  # クレジットカード払いかつ、セキュリティコードを利用する場合はtrueを返す
  def credit_card_use_security_code?
    credit_card? && (ShopSetting.get_setting(:security_code, I18n.locale) == "true")
  end

  # カード番号を連結します
  def card_no
    self.no.blank? ? [no_1, no_2, no_3, no_4].join : self.no
  end

  # 有効期限を「yymm」に整形します
  def expire
    [expire_year, expire_month].join
  end

  # 有効期限が入力されていればtrueを返す
  def expire_year_check?
    !expire_year.blank? && normal_payment_type_key == :credit
  end
  def expire_month_check?
    !expire_month.blank? && normal_payment_type_key == :credit
  end

  # キャンペーンの対象日時が現在日時の場合trueを返す
  def campaign_attached_now?
    campaign_attached_div.nil? || campaign_attached_div.blank? || campaign_attached_div_key == :now
  end

  # 自由記入欄が入力されていればtrueを返す
  def check_free_request?
    !free_request.blank?
  end

  # 英数チェックする場合はtrueを返す
  def is_english_format?
    I18n.locale.to_s != 'ja'
  end

  #
  # = 配送希望日を返却します =
  #
  def available_delivery_date(country_code = "JP")
    list = [[]] # 配送希望日を未選択にするブランクを設定しています
    Distributor.delivery_date_list(country_code).each do |date|
      list << [date, date]
    end
    return list
  end

  #
  # = 配送希望時間を返却します =
  #
  def available_delivery_time(country_code="JP")
    country = Country.find_by_code(country_code)
    distributor_id = country.distributor_id
    DistributorDeliveryTime.find(:all, :conditions => ["distributor_id = ?", distributor_id])
  end
end
