# -*- coding: utf-8 -*-
class OrderItem < ActiveRecord::Base
  include EcRider::ValidationExt
  versioned
  acts_as_paranoid

  attr_accessor :product_image, :delete_flg

  belongs_to :order_shipping, :class_name => 'OrderShipping', :foreign_key => 'order_shipping_id'
  belongs_to :product_sku, :with_deleted => true

  has_many :product_sku_secure_inventories
  has_many :order_item_components
  has_many :order_item_campaigns

  # コース区分
  selectable_attr :course_div do
    entry '1', :normal  , '単品'      # 都度購入
    entry '2', :periodic, '定期'      # 定期購入
    entry '3', :present , 'プレゼント' # キャンペーンによるプレゼント
  end
  
  # validates_length_ofが呼び出すことになるFixnum#sizeは、32bit版OSで動かした時は4を、64bit版OS上で動作した場合には8を返します。
  # 以下のようにvalidates_length_ofでは数値を文字列に変換した上での長さのチェックを行うことはできません。
  # validates_inclusion_ofを使用するか、validateメソッドにチェックするコードを用意してください。
  # validates_length_of   :qty, :maximum=>5, :allow_nil => false
  # validates_length_of   :subtotal_amount, :maximum=> 10, :allow_nil => false
  validates_numeric_max_length_of(:qty, :maximum => 5, :allow_nil => false)
  validates_numeric_max_length_of(:subtotal_amount, :maximum => 10, :allow_nil => false)

  validates_numericality_of :qty, :only_integer =>true, :message => :EM990003
  validates_numericality_of :subtotal_amount, :only_integer =>true, :message => :EM990003

  #
  # =order_itemを保存して在庫を引き当てる=
  # 注文情報編集画面からの注文情報更新時に呼ばれます。
  #
  def save_with_stock_control(status_div_key = nil)
    self.save!
    case status_div_key
      when :undelivery_settlement, :undelivery_unpaid
        self.secure_stocks(:before_pay)
      when :undelivery, :deliver_request
        self.secure_stocks
    end
  end

  #
  # =order_itemを削除して在庫を開放する=
  #
  def destroy_with_stock_control
    self.unsecure_stocks
    self.destroy
  end

  #
  # HashからOrderItemオブジェクトの配列を返す
  #
  def self.create_order_items(order_item_hash)
    result = []
    return result unless order_item_hash
    order_item_hash.values.each do |order_item_attr|
      if order_item_attr[:delete_flg].nil?
        result << OrderItem.new(order_item_attr)
      end
    end
    return result
  end

  # =販売期間をチェックする=
  # 販売期間外の商品が含まれる注文がある場合はOrderItem::NonperiodicSalesErrorが発生する
  def check_sales_period
    product_sku = ProductSku.find_by_id(self.product_sku_id)
    return unless product_sku || prodcut_sku.product
    # 販売期間外の場合
    target_at = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : self.order_shipping.order.ordered_at
    unless product_sku.product.check_sales_period?(target_at)
      message = "#{self.product_name}#{I18n.t("activerecord.errors.messages.EM990044")}"
      raise NonPeriodicSalesError.new(self.id, message, self.product_name)
    end
    return true
  end

  #
  # =在庫をチェックする=
  # 引き当て後の在庫数を返す
  # 在庫数を超える注文が含まれる場合はOrderItem::OutOfStockErrorが発生する
  #
  def check_stocks
    product_sku = ProductSku.find_by_id(self.product_sku_id)
    return unless product_sku

    add_quantity = self.qty

    #引き当て済みの在庫がある場合は差分を計算する
    if secure_stock = ProductSkuSecureInventory.find(:first, :conditions => {:order_item_id => self.id, :product_sku_id => self.product_sku_id})
      add_quantity -= secure_stock.count
    end

    new_stock_count = product_sku.orderable_count - add_quantity
    if new_stock_count < 0
      message = I18n.t("activerecord.errors.messages.EM200002", :product => self.product_name, :count => product_sku.orderable_count)
      raise OutOfStockError.new(self.id, message, self.product_name)
    end

    return new_stock_count
  end
  
  #
  # =在庫を仮引き当て、引き当て済みにする=
  # 引数(mode)  :paidを指定すると引き当て済みにします。指定しないと仮引き当てにします。
  #
  def secure_stocks(mode = :paid)
    raise SecureStockError, 'cannot secure_stocks for new_record' if new_record?
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg

    check_stocks
    
    unless ::EcRider.config.ec_test
      # 子商品を引き当てる
      product_sku.components.each do |component|
        create_or_update_inventory(component.component_sku, mode, component.count * self.qty)
      end

      # 自身の在庫を引き当てる
      product_sku = ProductSku.find(self.product_sku_id, :include => :components, :with_deleted => true)
      create_or_update_inventory(product_sku, mode)

      # 親商品の在庫を再計算する
      product_sku.parent_skus.each do |parent_component|
        parent_sku = parent_component.product_sku
        parent_sku.orderable_count = parent_sku.stock_count(:orderable_count)
        parent_sku.save!
      end
    end
  end
  
  #
  # =引き当て情報を元に実在庫を減らす=
  #
  def reduce_stocks
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg
    comment = "オンライン受注番号[#{self.order_shipping.order.ec_order_code}]を出荷しました"

    self.product_sku_secure_inventories.each do |secure_inventory|
      # 出庫情報作成
      inventory = ProductSkuInventory.new(:count => -secure_inventory.count,  :comment => comment, :product_sku_id => secure_inventory.product_sku_id)
      inventory.save!
      # 引き当て情報削除
      secure_inventory.destroy
      # SKUも更新する
      sku = inventory.product_sku
      sku.parent_stock_count -= secure_inventory.count
      sku.secure_stock_count -= secure_inventory.count
      sku.composable_stock_count = sku.stock_count
      sku.save!
    end
    
    # 自身（商品構成の親）の在庫を再計算する
    self_product_sku = ProductSku.find(self.product_sku_id)
    self_product_sku.composable_stock_count = self_product_sku.stock_count
    self_product_sku.save!
    
    # 自身の親商品がある場合は、親商品の在庫を再計算する
    self.product_sku.parent_skus.each do |parent_component|
      parent_sku = parent_component.product_sku
      parent_sku.composable_stock_count = parent_sku.stock_count
      parent_sku.save!
    end
  end
  
  #
  # =在庫を戻して、引き当て情報を作成する=
  #
  def restore_stocks
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg
    comment = "オンライン受注番号[#{self.order_shipping.order.ec_order_code}]を配送指示済に戻しました"
    product_sku = ProductSku.find(self.product_sku_id, :include => :components, :with_deleted => true)

    # 子SKUがある場合は子SKUの在庫を戻す
    product_sku.components.each do |component|
      count = component.count * self.qty
      # 在庫を増やす
      ProductSkuInventory.create!(:count => count, :comment => comment, :product_sku_id => component.component_sku_id)
      # 引き当て情報を作成する
      ProductSkuSecureInventory.create!(:count => count, :paid_flg => true, :product_sku_id => component.component_sku_id, :order_item_id => self.id)
      # SKUも更新する
      component_sku = component.component_sku
      component_sku.parent_stock_count += count
      component_sku.secure_stock_count += count
      component_sku.composable_stock_count = component_sku.stock_count
      component_sku.save!
    end

    # 自身の在庫を戻す（子供の在庫数を元に再計算するため、子供の在庫を先に処理しておく）
    # 在庫を増やす
    ProductSkuInventory.create!(:count => self.qty, :product_sku_id => self.product_sku_id, :comment => comment)
    # 引き当て情報を作成する
    ProductSkuSecureInventory.create!(:count => self.qty, :paid_flg => true, :product_sku_id => self.product_sku_id, :order_item_id => self.id)
    # SKUも更新する
    product_sku.parent_stock_count += self.qty
    product_sku.secure_stock_count += self.qty
    product_sku.composable_stock_count = product_sku.stock_count
    product_sku.save!

    # 親SKUの在庫を再計算する
    product_sku.parent_skus.each do |parent_component|
      parent_sku = parent_component.product_sku
      parent_sku.composable_stock_count = parent_sku.stock_count
      parent_sku.save!
    end
  end
  
  private
  # 引き当て情報を作成、または、更新する
  def create_or_update_inventory(sku, mode = :paid, count = self.qty)
    # 引き当て情報を作成、または、更新する
    secure_stock = ProductSkuSecureInventory.find(:first, :conditions => {:order_item_id => self.id, :product_sku_id => sku.id})
    org_secure_count = secure_stock ? secure_stock.count : 0
    secure_stock ||= ProductSkuSecureInventory.new(:order_item_id => self.id, :product_sku_id => sku.id, :count => count)
    secure_stock.count = count
    secure_stock.paid_flg = (mode == :paid)
    secure_stock.save!
    # SKUも更新します。
    sku.secure_stock_count += (count - org_secure_count)
    sku.orderable_count = sku.stock_count(:orderable_count)
    sku.save!
  end
  
  public
  #
  # =引き当て在庫を開放する=
  #
  def unsecure_stocks
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg
    # 引き当てている在庫を開放する
    self.product_sku_secure_inventories.each do |secure_inventory|
      # SKUも更新する
      sku = secure_inventory.product_sku
      sku.secure_stock_count -= secure_inventory.count
      sku.orderable_count = sku.stock_count(:orderable_count)
      sku.save!
      # 引き当て情報を削除する
      secure_inventory.destroy
    end

    # 自身（商品構成の親）の在庫数を再計算する
    self_product_sku = ProductSku.find(self.product_sku_id)
    self_product_sku.orderable_count = self_product_sku.stock_count(:orderable_count)
    self_product_sku.save!

    # 自身の親商品があれば、親商品の在庫数を再計算する
    self.product_sku.parent_skus.each do |parent_component|
      parent_sku = parent_component.product_sku
      parent_sku.orderable_count = parent_sku.stock_count(:orderable_count)
      parent_sku.save!
    end
  end

  # =販売期間例外=
  # 販売期間外の注文を確定しようとした際に発生する
  #
  class NonPeriodicSalesError < StandardError
    attr_reader :order_item_id, :message, :product_name
    def initialize(order_item_id, message = nil, product_name = nil)
      @order_item_id = order_item_id
      @message = message
      @product_name = product_name
    end
  end

  #
  # =在庫切れ例外=
  # 在庫数以上の注文を確定しようとした際に発生する
  #
  class OutOfStockError < StandardError
    attr_reader :order_item_id, :message, :product_name
    def initialize(order_item_id, message = nil, product_name = nil)
      @order_item_id = order_item_id
      @message = message
      @product_name = product_name
    end
  end

  #
  # =引き当て例外=
  # new_recordに対してsecure_stocksを呼び出した際に発生する
  #
  class SecureStockError < StandardError
  end

  # 対象の商品SKUを返す
  # 紐付く商品SKUが無い場合（削除されている場合）は、新規ProductSkuインスタンスを返す
  def get_sku
    return self.product_sku if self.product_sku
    return ProductSku.new()
  end

  # クラス名が OrderItemComponent の場合、自分自身を返す
  # そうでない場合、nilを返す
  def self_if_component
    return self if self.class.name == "OrderItemComponent"
    return nil
  end
end
