# -*- coding: utf-8 -*-

# このテストケースは削除しないでください。
# 本来であればmodelのテストはec_admin,ec_front共用のプラグインに記述しますが、
# ec_admin, ec_admin_mobileで挙動が違うため、
# ec_adminのプラグインが有効になっている場合のみテストを実行したく、ec_admin側に作っています。
# ec_adminにはAdminMailerがありますが, ec_admin_mobileはAdminMobileMailerとなるため
# ec_admin_mobileにはAdminMailerがないためエラーになります。

require 'digest/sha1'
require 'ec_rider/selectable_attr_ext'

class OrderShipping < ActiveRecord::Base
  include EcRider::PluginModule::Extendable

  versioned
  acts_as_paranoid
  include EcRider::ModelUtility

  has_many   :order_items, :class_name => 'OrderItem', :foreign_key => 'order_shipping_id' #:dependent => :destroy
  belongs_to :order, :class_name => 'Order', :foreign_key => 'order_id'
  belongs_to :country, :foreign_key => 'country_code'

  after_create :generate_order_code
  
  # 注文ステータス
  # 新規ステータス追加時は、ec_frontのキャンセル依頼ボタン制御、ec_adminトップページの売上集計などにも注意すること。
  selectable_attr :status_div do
    entry '14', :before_settlement    , '決済前'
    entry '12', :undelivery_settlement, '決済中', :cancel_requestable => true
    entry '11', :undelivery_unpaid    , '入金待ち'        , :cancel_requestable => true, :event_to_next => :confirm_pay # 配送準備中（入金待ち）
    entry '0' , :undelivery           , '配送準備中'      , :cancel_requestable => true, :event_to_next => :request_delivery
    entry '1' , :deliver_request      , '配送指示済'      , :event_to_next => :deliver        , :name_for_front => 'order_shippings.status_div.name_for_front.deliver_request'
    entry '2' , :delivered            , '配送済'          , :user_deletable => true, :event_to_next => :deliver_notify, :name_for_front => 'order_shippings.status_div.name_for_front.delivered'
    entry '3' , :deliver_advised      , '配送案内済'      , :user_deletable => true, :name_for_front => 'order_shippings.status_div.name_for_front.deliver_advised'
    entry '10', :cancel_request       , 'キャンセル依頼中', :event_to_next => :accept_cancel
    entry '8' , :canceled             , 'キャンセル完了', :user_deletable => true
    entry '13', :settlement_failed    , '決済NG', :user_deletable => true
    entry '15', :stock_error          , '在庫不足エラー', :user_deletable => true
    entry '16', :internal_error       , '内部エラー', :user_deletable => true
    entry '17', :external_error       , '外部エラー', :user_deletable => true
    entry '18', :returned            , '返品完了', :user_deletable => true
    entry '19', :sales_period_error   , '販売期間エラー', :user_deletable => true
  end

  # 注文ステータスの状態遷移
  state_flow(:status_div) do

    origin(:before_settlement) # 注文ステータス開始

    # 正常系
    group(:valid) do
      # recoverの順番は重要です。Exceptionを先に書くと全ての例外は:internal_errorになってしまいます。
      # recover(:'Net::HTTPHeaderSyntaxError').to(:external_error)
      # recover(:'Net::ProtocolError').to(:external_error)
      recover(:'Exception').to(:internal_error)

      # 正常系>自動キャンセル可
      group(:auto_cancelable) do
        from(:before_settlement) do # 決済前

          guard(:paid_by_cod?). # 代引き
            action(:check_sales_period).action(:secure_points).
            action(:secure_stocks, :paid).action(:send_thanks_mail).
            to(:undelivery) # => 配送準備中

          guard(:paid_by_bank_transfer?). # 銀行振込
            action(:check_sales_period).action(:secure_points).
            action(:secure_stocks, :before_pay).action(:send_thanks_mail).
            to(:undelivery_unpaid) # => 入金待ち

          guard(:paid_by_paypal?). #PayPal, paygent
            action(:check_sales_period).action(:secure_points).
            action(:secure_stocks, :before_pay).action(:update_no_membership).
            to(:undelivery_settlement) # => 決済中

          guard(:paid_by_mobile_career?). # 携帯キャリア
            action(:check_sales_period).action(:secure_points).
            action(:secure_stocks, :before_pay).action(:update_no_membership).
            to(:undelivery_settlement) # => 決済中

          guard_else. # クレジットカード、Alipay、コンビニ
            action(:check_sales_period).action(:secure_points).
            action(:secure_stocks).to(:undelivery_settlement) # => 決済中

          recover(OrderItem::NonPeriodicSalesError).
            action(:update_order_item_nonperiodic_sale_flg).
            action(:save_user_as_no_membership).to(:sales_period_error)
          recover(OrderItem::OutOfStockError).
            action(:update_order_item_out_of_stock_flg).
            action(:save_user_as_no_membership).to(:stock_error)
          recover(Exception).action(:save_user_as_no_membership).to(:internal_error)
        end

        from(:undelivery_settlement) do # 決済中
          guard(:paid_by_credit?). # クレジットカード
            action(:credit_card_payment).
            action(:secure_stocks, :paid).
            action(:send_thanks_mail).
            to(:undelivery) # => 配送準備中

          guard(:paid_by_paypal?). # PayPal
            event(:payment_settlement_for_paypal).
              action(:paypal_payment).
              action(:secure_stocks, :paid).
              action(:update_membership).
              action(:send_thanks_mail).
              to(:undelivery) # => 配送準備中

          guard(:paid_by_paypal?). # PayPal 決済エラー更新
            event(:update_status_settlement_failed_for_paypal).
              action(:save_user_as_no_membership).
              action(:unsecure_points).action(:unsecure_stocks).
              to(:settlement_failed) # => 決済エラー

          guard(:paid_by_paypal?). # PayPal 内部エラー更新
            event(:update_status_internal_error_for_paypal).
              action(:save_user_as_no_membership).
              action(:unsecure_points).action(:unsecure_stocks).
              to(:internal_error) # => 決済エラー

          guard(:paid_by_mobile_career?). # キャリア決済
            event(:payment_settlement_for_mobile_career).
              action(:secure_stocks, :paid).
              action(:update_membership).
              action(:send_thanks_mail).
              to(:undelivery) # => 配送準備中

          guard(:paid_by_mobile_career?). # キャリア決済 決済エラー更新
            event(:update_status_settlement_failed_for_mobile_career).
              action(:save_user_as_no_membership).
              action(:unsecure_points).action(:unsecure_stocks).
              to(:settlement_failed) # => 決済エラー

          guard(:paid_by_mobile_career?). # キャリア決済 内部エラー更新
            event(:update_status_internal_error_for_mobile_career).
              action(:save_user_as_no_membership).
              action(:unsecure_points).action(:unsecure_stocks).
              to(:internal_error) # => 決済エラー

          # guard(:paid_by_alipay?){ # Alipay
          #   event(:alipay_ok).action(:secure_stocks, :paid).action(:send_thanks_mail).to(:deliver_preparing)
          #   event(:alipay_ng).action(:release_stock).action(:delete_point).to(:settlement_failed)
          # }

          guard(:paid_by_cvs?). # コンビニ
            action(:cvs_payment).
            action(:secure_stocks, :before_pay).
            action(:send_thanks_mail).
            to(:undelivery_unpaid) # => 入金待ち

          recover(OrderItem::OutOfStockError).
            action(:update_order_item_out_of_stock_flg).action(:save_user_as_no_membership).
            action(:unsecure_points).action(:unsecure_stocks).to(:stock_error)
          recover(Order::SettlementError).
            action(:save_error_payment).action(:save_user_as_no_membership).
            action(:unsecure_points).action(:unsecure_stocks).to(:settlement_failed)
          recover(Exception).
            action(:save_user_as_no_membership).
            action(:unsecure_points).action(:unsecure_stocks).to(:internal_error)
        end

        from :undelivery_unpaid do # 入金待ち
          # = 入金確認 =
          event(:confirm_pay).
            action(:secure_stocks).
            action(:send_receipt_payment_mail).
            to(:undelivery) # => 配送準備中

          # = コンビニ入金結果通知受信 =
          guard(:paid_by_cvs?). # 入金完了 
            event(:receive_notification_paysuccess).
              action(:secure_stocks).
              action(:send_receipt_payment_mail).
              to(:undelivery) # => 配送準備中
          guard(:paid_by_cvs?). # 期限切れ 決済エラー更新
            event(:receive_notification_expired).
              action(:save_user_as_no_membership).
              action(:unsecure_points).action(:unsecure_stocks).
              to(:settlement_failed) # => 決済エラー
        end

        from :undelivery do # 配送準備中
          # = 配送指示 =
          event(:request_delivery).
            action(:send_delivery_request_mail).
            to(:deliver_request) # => 配送指示済

          # = 入力ミス訂正 =
          event(:back_to_undelivery_unpaid).action(:secure_stocks, :before_pay).
            to(:undelivery_unpaid) # =>入金待ち
        end

        # = キャンセル依頼 =
        event(:request_cancel).
          action(:send_cancel_request_mail).
          to(:cancel_request) # => キャンセル依頼中

        # = キャンセル =
        event(:cancel).action(:unsecure_stocks).action(:unsecure_points).
          action(:send_cancel_complete_mail).
          to(:canceled) # => キャンセル完了
      end

      from(:deliver_request) do # 配送指示済
        # = 配送 =
        event(:deliver).
          action(:reduce_stocks).
          action(:send_shipment_complete_for_shop_mail).
          action(:active_given_point).
          action(:create_order_accounting).
          to(:delivered) # => 配送済み

        # = キャンセル =
        event(:cancel).action(:unsecure_stocks).action(:unsecure_points).
          action(:send_cancel_complete_mail).
          to(:canceled) # => キャンセル完了
      end

      from(:delivered) do # 配送済
        # = 配送案内送付 =
        event(:deliver_notify).
          action(:send_shipment_complete_mail).
          to(:deliver_advised) # => 配送案内済

        # = 配送ミス訂正 =
        event(:back_to_deliver_request).action(:restore_stocks).
          action(:inactive_given_point).action(:create_cancel_accounting).
          to(:deliver_request) # => 配送指示済

        # = 返品 =
        event(:returned_request).
          action(:unsecure_points).action(:create_cancel_accounting).
          action(:send_returned_mail).
          to(:returned) # 返品完了
      end

      from(:deliver_advised) do# 配送案内済
        # = 返品 =
        event(:returned_request).
          action(:unsecure_points).action(:create_cancel_accounting).
          action(:send_returned_mail).
          to(:returned) # 返品完了
      end

      termination(:returned)# 終端

      # 正常系>キャンセル系
      state_group(:canceling) do
        from :cancel_request do # キャンセル依頼中
          # = キャンセル受付 =
          event(:accept_cancel).
            action(:unsecure_stocks).action(:unsecure_points).
            action(:send_cancel_complete_mail).
            to(:canceled) # => キャンセル完了
        end

        termination(:canceled) # 終端
      end
    end

    group(:error) do # 異常系
      state(:settlement_failed)
      state(:sales_period_error)
      state(:stock_error)
      state(:internal_error)
      state(:external_error)
      termination # 終端
    end

  end



  # 正常系の注文ステータス。
  # 注文一覧のタブとして使用されます。実際の状態遷移とは完全に合致しませんが、
  # 「配送指示以前」や「配送準備中以後」などのステータスの集合を導くもととなる配列です。
  STATUS_REGULAR_TRANSITION = [
    # :before_settlement # 決済前は含めません。
    :undelivery_settlement,
    :undelivery_unpaid,
    :undelivery,
    :deliver_request,
    :delivered,
    :deliver_advised
  ]

  # 正常キャンセル系のステータス
  STATUS_CANCEL_TRANSITION = [
    :returned,
    :cancel_request,
    :canceled
  ]

  # 異常系のステータス
  # 基本的に、正常系、正常キャンセル系と違って順序は関係ありませんが、
  # 注文一覧でこの順序を利用しています。
  STATUS_ERRORS = [
    :settlement_failed,
    :sales_period_error,
    :stock_error,
    :internal_error,
    :external_error
  ]

  # 商品・商品SKUを削除可能なステータス
  # 以下のステータス以外(=発送前)の注文に含まれている商品は削除できません。
  STATUS_DIV_KEYS_PRODUCT_DELETEABLE = [
    :delivered,
    :deliver_advised, 
    :canceled, 
    :settlement_failed,
    :sales_period_error,
    :stock_error,
    :internal_error,
    :external_error,
  ]

  # 「発送前」のステータス
  STATUS_DIV_KEYS_BEFORE_DELIVERY = self.status_div_keys - STATUS_DIV_KEYS_PRODUCT_DELETEABLE
  # 「発送前」のステータスの区分値の配列
  STATUS_DIVS_BEFORE_DELIVERY = STATUS_DIV_KEYS_BEFORE_DELIVERY.map{|key| self.status_div_id_by_key(key)}

  # この注文がどのように送付されるか/送付されたのかを示す区分
  #
  # Product#delivery_divと同じようなデータを持ちますが、
  # Product#delivery_divとは異なるようになるはずなので、分けて定義します。
  #
  selectable_attr :delivery_div do
    entry '0', :normal, '通常便'
    entry '1', :mail, 'メール便'
  end

  # 支払い方法区分
  # TODO: 要DB見直し
  # 支払に関する情報は Order に集約させるべきですので、OrderShippingにこの属性を保持しているのはおかしいです。
  # ですので、機会があるときに order_shippings.payment_type_div を orders.payment_type_div として移動することを
  # 検討してください。
  selectable_attr :payment_type_div do
    entry '1', :cod, '代金引換', :pay_in_advance => true, :default_status => :undelivery
    entry '2', :credit, 'クレジットカード', :pay_in_advance => true, :default_status => :undelivery
    entry '3', :alipay, 'アリペイ決済', :pay_in_advance => true, :default_status => :undelivery_settlement
    entry '4', :bank_transfer, '銀行振り込み', :pay_in_advance => true, :default_status => :undelivery_unpaid #先払いの銀行振り込み
    entry '5', :cvs, 'コンビニ決済', :pay_in_advance => true, :default_status => :undelivery_unpaid #先払いのコンビニ決済
    entry '6', :paypal, 'PayPal決済', :pay_in_advance => true, :default_status => :undelivery_settlement #PayPalチェックアウト決済
    entry '7', :mobile_career, '携帯キャリア決済', :pay_in_advance => true, :default_status => :undelivery_settlement
  end

   payment_type_div_keys.each do |key|
     class_eval(<<-"EOS", __FILE__, __LINE__)
       def paid_by_#{key.to_s}?
         self.payment_type_div_key == :#{key.to_s}
      end
    EOS
  end

  # ヤマト運輸の支払い区分コードを返す
  # 代引き：２、発払い：０
  def payment_type_code_yamato
    if self.payment_type_div_key == :cod
      return 2
    end
    return 0
  end

  # 配達時間
  selectable_attr :arrival_hour do
    entry '0', :none, '配達時間を指定しない'
  end

  # CSV出力区分
  selectable_attr :csv_output_div do
    entry '0', :not_yet, '未出力'
    entry '1', :done, '出力済'
  end

  # バリデーション
  validates_length_of :delivery_content, :maximum => 10000, :message => :EM990012, :allow_nil => true
  validates_numericality_of :shipping_cost, :only_integer =>true, :if => :shipping_cost?, :greater_than_or_equal_to => 0

  # 個人情報のみを空白にし論理削除する場合があるため独自にバリデーションを作成する
  def validates_for
    validates_presence_of_keys = [:name_last, :name_first, :tel_num, :zip_code,
      :address_2, :shipping_cost]
    no_error = true
    validates_presence_of_keys.each do |key|
      if (self.attributes[key.to_s].nil? || self.attributes[key.to_s].blank?)
        errors.add(key, :EM990001)
        no_error = false
      end
    end
    return no_error
  end

  #
  # = 同送の指定が可能な配送先一覧を取得する =
  #
  def self.find_includable_shipping(user_id)
    OrderShipping.find(:all,
      :select => "orders.ordered_at, order_shippings.*",
      :joins => "LEFT OUTER JOIN orders ON orders.id = order_shippings.order_id",
      :conditions => ["user_id = ? AND status_div = ?", user_id, OrderShipping.status_div_id_by_key(:undelivery)]
    )
  end

  #
  # = 注文コードを生成 =
  #
  # 11桁の注文コードを発番します。
  # 注文コードを生成し、注文コード属性に設定します。after_createでコールしてください。
  def generate_order_code
    self.order_code = self.id.to_s.rjust(11, '0')
    self.save!
  end
  
  #
  # = ステータスを更新する =
  #
  def update_status(new_status_div_key = nil)
    if new_status_div_key
      self.status_div_key = new_status_div_key.to_sym
    else
      case status_div_key
        when :before_settlement
          self.status_div_key = OrderShipping.payment_type_div_entry_by_key(payment_type_div_key)[:default_status]
        when :undelivery_settlement, :undelivery_unpaid
          self.status_div_key = :undelivery
        when :undelivery
          self.status_div_key = :deliver_request
        when :deliver_request
          self.status_div_key = :delivered
        when :delivered
          self.status_div_key = :deliver_advised
        when :cancel_request
          self.status_div_key = :canceled
        when :returned_request
          self.status_div_key = :returned
      end
    end
    self.save!
  end

  #
  # =紐付く商品情報を更新する=
  #
  # 在庫数を超える注文が含まれる場合はOrderItem::OutOfStockErrorが発生します
  #
  def update_order_items(new_order_items, stock_control = true)
    # 先に在庫の引当を取り消さないと、更新前の注文数量を考慮できずに在庫不足になる場合があります
    OrderShipping.transaction do
      if stock_control
        self.unsecure_stocks
      end
      current_order_items = self.order_items.dup
      new_order_items.each do |new_order_item|
        if stock_control
          new_order_item.save_with_stock_control(self.status_div_key)
        else
          new_order_item.save!
        end

        # OrderItemComponentインスタンスを生成する
        product_sku = ProductSku.find(new_order_item.product_sku_id)
        product_sku.components.each do |component|
          oic = OrderItemComponent.new({:order_item_id => new_order_item.id, :component_sku_id => component.component_sku_id, :qty => component.count * new_order_item.qty})
          oic.product_name = oic.product_sku.product.product_name_1
          oic.order_product_name_1 = oic.product_sku.product_name_for_admin_ja
          oic.order_product_name_2 = oic.product_sku.product.order_product_name_2
          oic.save!
        end
        
        current_order_items.each do |current_order_item|
          current_order_items.delete(current_order_item) if current_order_item.id == new_order_item.id
        end
      end
      current_order_items.each do |order_item|
        order_item.order_item_components.each do |component|
          component.destroy
        end
        order_item.destroy
      end
    end
  end

  # 決済処理に必要なパラメータを保持したアクセッサー
  # payment_arg[0] : order_form
  # payment_arg[1] : user
  attr_accessor :payment_arg

  # コンビニで決済する
  def cvs_payment
    order.cvs_payment(payment_arg[0], payment_arg[1])
  rescue Order::SettlementError => error
    @error_payment = error.error_payment
    raise error
  end

  # PayPalで決済する
  # プラグインで拡張しています
  def paypal_payment
    order.paypal_payment(payment_arg[0], payment_arg[1], payment_arg[2])
  rescue Order::SettlementError => error
    @error_payment = error.error_payment
    raise error
  end

  # クレジットカードで決済する
  def credit_card_payment
    order.credit_card_payment(payment_arg[0])
  rescue Order::SettlementError => error
    @error_payment = error.error_payment
    raise error
  end

  # 決済エラー履歴を登録します。各決済プラグインで拡張されています。
  def save_error_payment
  end

  # send_thanks_mailの実行時の引数
  # send_thanks_mailを呼び出す前には必ず設定してください。
  # # state_flowで呼び出すために実行時の引数を属性として渡しています。
  attr_accessor :send_thanks_mail_args

  # THANKSメール送信
  def send_thanks_mail
    if EcRider.config.ec_admin
      AdminMailer.send_thanks_mail(*(send_thanks_mail_args + [order.id]))
    else
      FrontSiteMailer.send_thanks_mail(*(send_thanks_mail_args + [order.id]))
    end
  end

  attr_accessor :save_user_as_no_membership_target

  # 注文時に例外が発生した場合に非会員の情報を保存します。
  # パスワード、秘密の質問等、会員独自の入力項目データも削除されます
  def save_user_as_no_membership
    @save_user_as_no_membership_target.save_as_no_membership if @save_user_as_no_membership_target
  end

  # 会員情報を非会員から会員に戻すメソッド
  # パスワードが登録されている非会員の場合、
  # 会員区分を｢会員｣に設定します。
  # 会員に戻せない場合、パスワードを削除します。
  def update_membership
    return if @save_user_as_no_membership_target.nil?
    unless @save_user_as_no_membership_target.crypted_password.nil?
      @save_user_as_no_membership_target.member_div_key = :membership
      # ↓これをしないとメールアドレス一意チェックで会員登録前の情報を参照してしまいます
      @save_user_as_no_membership_target.user_email.user = @save_user_as_no_membership_target
      if (@save_user_as_no_membership_target.valid? && @save_user_as_no_membership_target.user_email.valid?)
        @save_user_as_no_membership_target.save!
        # 会員情報を更新した情報に置き換えregist_errorをsend_thanks_mail_argsにセット
        send_thanks_mail_args[0] = @save_user_as_no_membership_target
      else
        save_user_as_no_membership
        # 会員エラー属性regist_errorをsend_thanks_mail_argsにセット
        send_thanks_mail_args[0].regist_error = true
      end
    end
  end

  # 会員情報を非会員に戻すメソッド
  # パスワード
  # 会員登録後、非会員として登録、決済確定後に本会員に設定するフローを
  # 使用する場合はこちらのメソッドを使用してください
  def update_no_membership
    return if @save_user_as_no_membership_target.nil?
    @save_user_as_no_membership_target.member_div_key = :no_membership
    @save_user_as_no_membership_target.save!
  end

  def update_order_item_out_of_stock_flg
    context = state_flow_contexts[:status_div]
    exception = context.exceptions.last
    OrderItem.update(exception.order_item_id, :out_of_stock_flg => true)
  end

  # 販売期間外エラーフラグを更新します。
  def update_order_item_nonperiodic_sale_flg
    context = state_flow_contexts[:status_div]
    exception = context.exceptions.last
    OrderItem.update(exception.order_item_id, :nonperiodic_sale_flg => true)
  end

  #
  # =販売期間をチェックする=
  # 販売期間を超える注文が含まれる場合は、OrderItem::NonperiodicSalesErrorが発生します
  # オフライン受注時には、チェックしません。
  def check_sales_period
    return true if EcRider.config.ec_admin
    current_order_items = self.order_items.dup
    current_order_items.each do |order_item|
     order_item.check_sales_period
    end
  end

  #
  # =在庫をチェックする=
  # 在庫数を超える注文が含まれる場合はOrderItem::OutOfStockErrorが発生します
  #
  def check_stocks
    current_order_items = self.order_items.dup
    current_order_items.each do |order_item|
      order_item.check_stocks
    end
  end

  #
  # =在庫を仮引き当て、引き当て済みにする=
  # 引数(mode)  :paidを指定すると引き当て済みにします。指定しないと仮引き当てにします。
  # 在庫数を超える注文が含まれる場合はOrderItem::OutOfStockErrorが発生します
  #
  def secure_stocks(mode = :paid)
    raise OrderItem::SecureStockError, 'cannot secure_stocks for new_record' if new_record?
    current_order_items = self.order_items.dup
    current_order_items.each do |order_item|
      order_item.secure_stocks(mode)
    end
  end

  #
  # =在庫の仮引き当て／引き当てを取り消す=
  #
  def unsecure_stocks
    current_order_items = self.order_items.dup
    current_order_items.each do |order_item|
      order_item.unsecure_stocks
    end
  end

  attr_accessor :given_point
  attr_accessor :used_point
  #
  # =付与ポイント、利用ポイントを仮ポイントとして発行します。=
  #
  def secure_points(given_point = self.given_point || 0, used_point = self.used_point || 0)
    return if self.order.user.member_div_key == :no_membership
    self.order.create_user_point(given_point, used_point)
  end

  #
  # =付与ポイント、利用ポイントを0に戻します。=
  #
  def unsecure_points
    self.order.delete_point
  end

  # 在庫を減らす
  def reduce_stocks
    self.order_items.each do |order_item|
      order_item.reduce_stocks
    end
  end

  # 在庫を戻す
  def restore_stocks
    self.order_items.each do |order_item|
      order_item.restore_stocks
    end
  end

  attr_accessor :shipment_content, :invoice_content

  # 入金確認メール送信
  def send_receipt_payment_mail
    mail_form = new_mail_form(:receipt_payment, order.locale)
    mail_form.from = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    mail_form.to = self.email
    if EcRider::Runtime.ec_admin?
      AdminMailer.send_free_mail(:receipt_payment, mail_form.from, mail_form.to, mail_form.subject, mail_form.body, mail_form.locale, self.order.user, self.order)
    else
      FrontSiteMailer.send_free_mail(:receipt_payment, mail_form.from, mail_form.to, mail_form.subject, mail_form.body, mail_form.locale, self.order.user, self.order)
    end
  end

  # 出荷指示メール送信
  def send_delivery_request_mail
    mail_form = new_mail_form(:shipment_instruction, "ja", {:shipment_content => shipment_content})
    mail_form.from = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    mail_form.to = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shipment_instruction_mail].split(/\s*,\s*/)
    AdminMailer.send_shipment_mail(:shipment_instruction, mail_form.from, mail_form.to, mail_form.subject, mail_form.body, mail_form.locale, self.order.user, self.order, invoice_content)
  end

  # 出荷完了メール（店舗宛て）送信
  def send_shipment_complete_for_shop_mail
    mail_form = new_mail_form(:shipment_complete_for_shop, "ja", {:shipment_complete_mail => true})
    mail_form.from = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    mail_form.to = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shipment_complete_mail].split(/\s*,\s*/)
    AdminMailer.send_free_mail(:shipment_complete_for_shop, mail_form.from, mail_form.to, mail_form.subject, mail_form.body, mail_form.locale, self.order.user, self.order)
  end

  # 出荷完了メール（顧客宛て）送信
  def send_shipment_complete_mail
    mail_form = new_mail_form(:shipment_complete, order.locale)
    mail_form.from = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    mail_form.to = self.email
    AdminMailer.send_free_mail(:shipment_complete, mail_form.from, mail_form.to, mail_form.subject, mail_form.body, mail_form.locale, self.order.user, self.order)
  end

  def new_mail_form(mail_type_div_key, locale, options = {})
    user = self.order.user
    # メールテンプレートを取得
    template = MailTemplate.find_valid_template(mail_type_div_key, locale)
    # メールテンプレートにバインドする変数を設定
    options.update({:first_name => user.name_first, :last_name => user.name_last, :user_name => user.user_address.fullname,
      :email_address => self.email, :order => order, :order_shipping => self})
    # メールテンプレートに注文情報をバインドする
    pc_mobile_div = UserEmail.check_by_emall(self.email)
    mail_form = MailForm.new
    mail_form.subject = template.get_mail_subject(pc_mobile_div, options)
    mail_form.body = template.get_mail_body(pc_mobile_div, options)
    mail_form.locale = template.locale
    return mail_form
  end

  # キャンセル依頼メール送信
  def send_cancel_request_mail
    FrontSiteMailer.send_notice_cancel_reqest_mail(self.order, self.order.user, self.email) unless EcRider.config.ec_admin
  end

  # キャンセル処理完了メール送信
  def send_cancel_complete_mail
    AdminMailer.send_cancel_complete_mail(self.order, self.email) if EcRider.config.ec_admin
  end

  # 返品完了メール送信
  def send_returned_mail
    AdminMailer.send_returned_mail(self.order, self.email) if EcRider.config.ec_admin
  end

  # 仮ポイントを本ポイントに切り替えます。
  def active_given_point
    UserPoint.active_given_point(order.id)
  end

  # 本ポイントを仮ポイントに切り替えます。
  def inactive_given_point
    UserPoint.inactive_given_point(order.id)
  end

  # 注文経理データ登録
  def create_order_accounting
    Accounting.create_for_order(self)
  end

  # キャンセル、返品経理データ登録
  def create_cancel_accounting
    Accounting.create_for_cancel(self)
  end
  #
  # = 選択可能な支払い方法のリストを返す =
  #
  def self.select_form_list
    payment_type_div_keys_to_options(EcRider.config.ecr_payment_types)
  end

  #
  # = 送付先が削除されているかどうかを判断する =
  #
  def deleted_send_address?
    return (self.name_last.blank? && self.name_first.blank?) ? true : false
  end

  #
  # = 送付先アドレスを削除する =
  #
  def delete_send_address
    self.name_last = nil
    self.name_first = nil
    self.kana_name_last = nil
    self.kana_name_first = nil
    self.tel_num = nil
    self.zip_code = nil
    self.address_1 = nil
    self.address_2 = nil
    self.address_3 = nil
    self.email = nil

    self.save!
  end

  #
  # =住所が日本語ならtrueを返す=
  #
  def is_japanese_format?
    if order
      (order.locale == 'ja' && country_code == 'JP')
    else
      (I18n.locale.to_s == 'ja' && country_code == 'JP')
    end
  end

  #
  # =住所が中国語(簡体字)ならtrueを返す=
  #
  def is_chinese_zh_cn_format?
    if order
      (order.locale == 'zh_CN' && country_code == 'CN')
    else
      (I18n.locale.to_s == 'zh_CN' && country_code == 'CN')
    end
  end

  #
  # =フルネームを連結して返す=
  #
  def fullname
    if is_japanese_format? || is_chinese_zh_cn_format?
      return "#{self.name_last} #{self.name_first}"
    else
      return "#{self.name_first} #{self.name_last}"
    end
  end

  #
  # =住所（市町村以下）を連結して返す=
  #
  def fulladdress
    if is_japanese_format? || is_chinese_zh_cn_format?
      return "#{self.address_2} #{self.address_3}"
    else
      address = ''
      address += "#{self.address_3}, " unless self.address_3.blank?
      address += "#{self.address_2}"
      return address
    end
  end

  # =住所が日本の場合は000-0000にして返す=
  def zip_code_jp
    is_japanese_format? ? "#{zip_code[0, 3].to_s}-#{zip_code[3, 6].to_s}" : zip_code
  end

  # =重量の合計を返す=
  def total_weight
    order_items.inject(0) {|result, item| result + item.product_sku.weight * item.qty}
  end

  # =割引、送料を含まない合計を返す=
  # 単価*数量の合計
  def sum_order_item_subtotal_amount
    order_items.inject(0) {|result, item| result + item.subtotal_amount}
  end

  # 
  # 注文の請求金額
  #
  def order_amount
    self.order.amount
  end

  # 前払いかどうか（決済処理上）
  def pay_in_advance?
    payment_type_div_entry[:pay_in_advance]
  end


  # 注文の領収金額を返す(元は送り先ごとでしたが、STEP1以降の仕様変更に伴ない注文の請求金額を使用して計算します)
  # 元は app/helpers/order_helper.rbにあったものです。
  def paid_order_amount
    # 前払い、または、代金引換
    pay_in_advance? ? order_amount: 0
  end

  # 注文の請求金額を返す(元は送り先ごとでしたが、STEP1以降の仕様変更に伴ない注文の請求金額を使用して計算します)
  # 元は app/helpers/order_helper.rbにあったものです。
  def due_order_amount
    # 前払い、または、代金引換
    pay_in_advance? ? 0 : order_amount
  end


  #
  # = 配送希望日を返却します =
  #
  def available_delivery_date
    list = [[]]
    Distributor.delivery_date_list(self.country_code).each do |date|
      list << [date, date]
    end
    return list
  end

  #
  # = 配送希望時間を返却します =
  #
  def available_delivery_time
    country = Country.find_by_code(self.country_code)
    distributor_id = country.distributor_id
    DistributorDeliveryTime.find(:all, :conditions => ["distributor_id = ?", distributor_id])
  end

  # 自動キャンセル可の場合はtrueを返します
  def auto_cancelable?
    flow = OrderShipping.state_flow_for(:status_div)
    flow.all_states[:auto_cancelable].include?(status_div_key)
  end

  #
  # = 注文情報編集画面でキャンセル可能なステータスの場合はtrueを返します =
  #
  def cancelable?
    cancelable_status_key = STATUS_REGULAR_TRANSITION - [:delivered, :deliver_advised]
    cancelable_status_key.include?(status_div_key)
  end

  #
  # = 返品可能な場合はtrueを返します =
  #
  def returnable?
    accounting = Accounting.find(:first, :conditions => ["order_id = ?", order_id], :order => "id desc")
    accounting && ![:cancel_cash, :cancel_receivable].include?(accounting.debt_account_div_key)
  end

  #
  # =配達希望時間の表示名を返す=
  #
  def arrival_time_display(locale = nil)
    return '' if self.arrival_hour.nil?
    display_locale = locale.nil? ? self.order.locale : locale
    DistributorDeliveryTime.get_name(self.country_code, self.arrival_hour, display_locale)
  end

  private
  #
  # =支払い方法のキーの配列から、区分値名、区分値の配列を返す=
  #
  def self.payment_type_div_keys_to_options(keys)
    OrderShipping.payment_type_div_enum.options(*keys)
  end
end
