# -*- coding: utf-8 -*-
class ProductSku < ActiveRecord::Base
  versioned

  acts_as_paranoid
  
  include EcRider::I18nSupport
  
  belongs_to :product

  has_one :current_inventory, :class_name => 'ProductSkuCurrentInventory'
  has_many :inventories, :class_name => 'ProductSkuInventory'
  has_many :secure_inventories, :class_name => 'ProductSkuSecureInventory'

  has_many :prices, :class_name => "ProductPrice", :foreign_key => 'product_sku_id',
    :dependent => :destroy
  accepts_nested_attributes_for :prices
  
  has_one :primary_price, :class_name => 'ProductPrice', :include => :user_rank, :conditions => ["user_ranks.no_membership_flg = 1"]

  has_many :components, :class_name => "ProductComponent", :foreign_key => 'product_sku_id',
    :dependent => :destroy
  accepts_nested_attributes_for :components

  has_many :favorites
  
  attr_reader :primary_component
  attr_accessor :sku_unpaid_secure_stock_count, :component_parent_sku_id

  # 在庫切れ時の表示
  selectable_attr :stockout_display_cd do
    update_with_kbn KbnCode.name_div_id_by_key(:stockout_display)
  end

  variation_item_base_names = (1..3).map{|index| "variation_item_#{index}"}

  variation_item_base_names.each do |base_name|
    belongs_to(base_name.to_sym, :class_name => 'ProductVariationItem', :foreign_key => "#{base_name}_id")

    class_eval(<<-"EOS", __FILE__, __LINE__)
      def #{base_name}_code
        obj = #{base_name}
        obj ? obj.code : nil
      end

      def #{base_name}_name_for_admin
        obj = #{base_name}
        obj ? obj.name_for_admin : nil
      end

      def #{base_name}_name(locale = I18n.locale)
        obj = #{base_name}
        obj ? obj.display_name(locale) : nil
      end
    EOS
  end

  class_eval(<<-"EOS", __FILE__, __LINE__)
    def variation_item_ids
      [ #{variation_item_base_names.map{|m| "#{m}_id"}.join(',')} ]
    end

    def variation_item_codes
      [ #{variation_item_base_names.map{|m| "#{m}_code"}.join(',')} ]
    end

    def variation_item_names_for_admin
      [ #{variation_item_base_names.map{|m| "#{m}_name_for_admin"}.join(',')} ]
    end

    def variation_item_names(locale = I18n.locale)
      [ #{variation_item_base_names.map{|m| "#{m}_name(locale)"}.join(',')} ]
    end
  EOS

  before_validation :build_sku_code_if_blank
  before_save :create_or_update_primary_sku_flg

  validates_presence_of :weight, :safety_stock_count, :shipping_cost_point, :stockout_display_cd
  validates_numericality_of :shipping_cost_point, :greater_than_or_equal_to => 0
  validates_numericality_of :weight, :greater_than_or_equal_to => 0

  def validate
    # 自身を商品構成の子に持つことはできない
    if self.components.map{|comp| comp.component_sku_id }.include?(self.id)
      errors.add_to_base("自身を商品構成の子に持つことはできません")
    end
  end

  before_destroy :error_if_unable_to_delete_product_sku

  def build_sku_code_if_blank
    product = self.product
    return unless product
    self.sku_code = "#{product.product_code}-#{self.branch_code}"
  end

  #
  # = 代表商品SKU =
  #
  def primary_sku
    ProductSku.find(:first, :conditions => ["product_id = ? AND primary_sku_flg = true ", self.product_id])
  end

  #
  # = 代表フラグの設定 =
  # 
  # 対象商品に対して最初に作成された商品SKUは代表SKUフラグを有効にする
  #
  def create_or_update_primary_sku_flg
    _primary_sku = primary_sku
    # 代表商品SKUが存在しないの場合
    if _primary_sku.nil?
      self.primary_sku_flg = true
    else
      # 代表商品SKUが存在しており、代表フラグを有効に更新する場合
      if self.primary_sku_flg == true && _primary_sku.id != self.id
        # 存在する代表商品SKUの代表フラグを無効にする
        _primary_sku.primary_sku_flg = false
        _primary_sku.save!
      end
    end
  end

  #
  # = 親商品SKUが存在するかを判定する =
  #
  def has_parent_sku?
    parent_sku_count = ProductComponent.count(:conditions => ["component_sku_id = ?", self.id])
    return parent_sku_count > 0
  end

  #
  # = 親商品SKUを返却する =
  #
  def parent_skus
    ProductComponent.find(:all, :conditions => ["component_sku_id = ?", self.id], :include => [:product_sku])
  end
  
  def name_for_admin
    names = variation_item_names_for_admin.compact
    names.empty? ? 'バリエーションなし' : names.join("-")
  end
  

  # 枝番
  def branch_code
    variation_item_codes.map{|code| code || '00'}.join('-')
  end

  # バリエーション表示を返す
  def variations_name
    result = ''
    variation_item_names.each do |item|
      unless item.blank?
        result << '-' unless result.blank?
        result << item
      end
    end
    return result
  end
  
  # 商品名 (バリエーション表示)を返す
  def product_name_1
    variation = variations_name
    variation.blank? ? self.product.product_name_1 : "#{self.product.product_name_1} (#{variation})"
  end
  
  # 管理用商品名（バリエーション表示）日本語を返す
  def product_name_for_admin_ja
    variation = name_for_admin
    return '' unless self.product
    variation.blank? ? self.product.order_product_name_1 : "#{self.product.order_product_name_1} (#{name_for_admin})"
  end
  
  # 管理用商品名（バリエーション表示）英語を返す
  def product_name_for_admin_en
    variation = nil
    i18n_tmp_block('en'){ variation = variations_name }
    variation.blank? ? self.product.order_product_name_2 : "#{self.product.order_product_name_2} (#{variation})"
  end
  
  #在庫数を返す（子商品を考慮する）
  # :stock_count  商品構成を考慮した在庫数を返す（デフォルト）
  # :orderable_count  商品構成を考慮した販売可能数を返す
  # : all  {:stock_count => 商品構成を考慮した在庫数, :orderable_count => 商品構成を考慮した販売可能数}を返す
  def stock_count(mode = :stock_count)
    # 親の在庫数
    parent_count = self.parent_stock_count
    parent_orderable_count = parent_count - self.secure_stock_count
    # 子の在庫数
    child_count = nil
    child_orderable_count = nil
    self.components.each do |component|
      component_sku = component.component_sku
      if component_sku
        composable_count = (component_sku.parent_stock_count / component.count).to_i
        composable_orderable_count = ((component_sku.parent_stock_count - component_sku.secure_stock_count) / component.count).to_i
        child_count = child_count ?  [composable_count, child_count].min : composable_count
        child_orderable_count = child_orderable_count ?  [composable_orderable_count, child_orderable_count].min : composable_orderable_count
      end
    end
    # 親子を考慮した在庫数
    count = parent_count
    orderable_count = parent_orderable_count
    if child_count
      count = child_count
      orderable_count = child_orderable_count
      if self.parent_stockable_flg
        count = [parent_count, child_count].min
        orderable_count = [parent_orderable_count, child_orderable_count].min
      end
    end

    case mode
    when :stock_count; return (count ? count : 0)
    when :orderable_count; return (orderable_count ? orderable_count : 0)
    when :all; return {:stock_count => (count ? count : 0), :orderable_count => (orderable_count ? orderable_count : 0)}
    end
  end

  # 商品構成を考慮した在庫数と販売可能数を再計算し、更新する
  def update_composable_count
    composable_count = self.stock_count(:all)
    self.composable_stock_count = composable_count[:stock_count]
    self.orderable_count = composable_count[:orderable_count]
    self.save!
  end
  
  # 自身の引き当て数を返す
  # :secure_stock_count => 引き当て数（仮引き当てを含まない）を返す（デフォルト）
  # :unpaid_secure_stock_count => 仮引き当て数を返す
  # :all => 引き当て数（仮引き当てを含む）を返す
  def get_secure_stock_count(mode=:all)
    count = nil
    if mode == :secure_stock_count
      count = ProductSkuSecureInventory.sum('count', :conditions => ['product_sku_id = ? AND paid_flg = 1', self.id])
    elsif mode == :unpaid_secure_stock_count
      count = ProductSkuSecureInventory.sum('count', :conditions => ['product_sku_id = ? AND paid_flg = 0', self.id])
    else
      count = self.secure_stock_count
    end
    return count ? count : 0
  end
  
  #標準価格（非会員用の価格）を返す
  def price
    return self.primary_price ? self.primary_price.standard_price : 0
  end

  # ユーザーランクに対する商品SKUの単価を返します
  # このSKUに対する単価が見つからなかった場合は、例外をraiseします。
  def price_for(user_rank_or_user_rank_id)
    user_rank_id = user_rank_or_user_rank_id.is_a?(UserRank) ?
      user_rank_or_user_rank_id.id : user_rank_or_user_rank_id
    product_price = prices.find(:first, :conditions => {:user_rank_id => user_rank_id})
    raise ArgumentError, "ユーザーランク[#{user_rank_id}]に対する商品SKU[#{sku_code}]の単価が見つかりませんでした"  unless product_price
    product_price.standard_price
  end

  # 商品SKUが削除可能かどうかを判定します
  def deletable_sku?
    # ありえないけど、そもそも代表SKUがない場合は削除可能にしておきます
    return true if self.primary_sku.nil?
    
    if self.id == self.primary_sku.id
      # 代表SKUの場合、他の商品SKUが登録されていなければ削除できる
      product_skus = ProductSku.find(:all, :conditions => ["product_id = ? AND deleted_at is null", self.product_id])
      # バリエーションなしのSKUデータはカウントしませんので、マイナス１しています
      return true if (product_skus.size - 1) < 1
    else
      # 代表SKUではない商品SKUの場合、商品構成が登録されていないかつ他の商品SKUの構成商品でなければ削除できる
      return true if (self.components.size == 0 && !self.has_parent_sku?)
    end
    return false
  end

  # 配送前の注文に含まれていたら削除できないようにバリデートします
  # destroy実行時に呼び出されます
  def error_if_unable_to_delete_product_sku
    result = true
    # 発送前のものがなければfalseをセット
    if included_in_order_before_delivery?
      result = false
      errors.add_to_base("配送前の注文に含まれているため商品SKUを削除できません。")
    end

    # 商品構成の子になっていなければfalseをセット
    if include_product_component?
      result = false
      errors.add_to_base("商品構成の子になっているため商品SKUを削除できません。")
    end
    result
  end
  
  # 配送前の注文に含まれていたらtrueを返します
  def included_in_order_before_delivery?
    before_delivery_count = OrderItem.count(
      :joins => "inner join order_shippings on order_shippings.id = order_items.order_shipping_id",
      :conditions => [
        "order_items.product_sku_id = ? AND order_shippings.status_div in (?)",
        self.id, OrderShipping::STATUS_DIVS_BEFORE_DELIVERY
      ])
    before_delivery_count > 0
  end

  # 商品構成の子になっていたらtrueを返します
  def include_product_component?
    product_component_count = ProductComponent.count(
      :joins => "inner join product_skus ps on ps.id = product_components.product_sku_id and ps.deleted_at is null
                 inner join products p on ps.product_id = p.id and p.deleted_at is null",
      :conditions => ["product_components.component_sku_id = ?", self.id]
    )
    product_component_count > 0
  end

end
