# -*- coding: utf-8 -*-
require 'digest/sha1'
require 'csv'
require 'ec_rider/selectable_attr_ext'
class User < ActiveRecord::Base
  include EcRider::PluginModule::Extendable

  versioned

  # 個人情報保護の観点から削除は物理削除とします。
  #  acts_as_paranoid

  #ログイン最大試行回数（連続でログイン失敗が続いて最大試行回数に達するとロック）10回
 	MAX_LOGIN_CHALLENGE_COUNT = 10
 	#ロックが解除されるまでの時間（秒）30分
 	USER_LOCK_PERIOD_SECONDS = 1800

  include EcRider::ModelUtility

  has_many    :user_addresses, :class_name => 'UserAddress', :foreign_key => 'user_id', :order => :id, :dependent => :destroy
  has_many    :inquries, :class_name => 'Inquiry', :foreign_key => 'user_id'

  has_many    :orders, :class_name => 'Order', :foreign_key => 'user_id'
  has_many    :user_points, :class_name => 'UserPoint', :foreign_key => 'user_id', :dependent => :destroy
  has_one     :user_email, :class_name => 'UserEmail', :foreign_key => 'user_id', :conditions => ['pc_mobile_div = ?', UserEmail.pc_mobile_div_id_by_key(:pc)]
  has_one     :user_email_mobile, :class_name => 'UserEmail', :foreign_key => 'user_id', :conditions => ['pc_mobile_div = ?', UserEmail.pc_mobile_div_id_by_key(:mobile)]
  has_many    :user_emails, :class_name => 'UserEmail', :foreign_key => 'user_id', :dependent => :destroy
  has_many    :favorites

  belongs_to :rank, :class_name => 'UserRank', :foreign_key => 'user_rank_id'

  has_one :resigned_reason, :class_name => 'ResignedReason', :foreign_key => 'user_id'

  # 会員自身の住所
  has_one     :user_address, :class_name => 'UserAddress', :foreign_key => 'user_id', :conditions => ['address_div=?',UserAddress.address_div_id_by_key(:user)]

  # 顧客区分
  selectable_attr :member_div do
    entry '1', :membership, 'WEB会員'
    entry '3', :offline_membership, 'オフライン専用会員'
    entry '0', :no_membership, '非会員'
    entry '2', :resigned, '退会者'
  end

  # 変更前の顧客区分（編集画面用）
  def original_member_div=(value)
    @original_member_div = value
  end
  def original_member_div
    @original_member_div || self.member_div
  end
  selectable_attr :original_member_div, :enum => User.member_div_enum

  # PCメールマガジン購読区分
  selectable_attr :pc_mail_magazine_send_div do
    entry 0, :prefer_html, 'HTML(絵や写真入のメール)を受け取る'
    entry 1, :prefer_text, 'テキストメール(文字だけのメール)を受け取る'
    entry 99, :not_prefer, '受け取らない'
  end
  # モバイルメールマガジン購読区分
  selectable_attr :mobile_mail_magazine_send_div do
    entry 0, :prefer_html, 'HTML(絵や写真入のメール)を受け取る'
    entry 1, :prefer_text, 'テキストメール(文字だけのメール)を受け取る'
    entry 99, :not_prefer, '受け取らない'
  end

  # DM送付希望区分
  selectable_attr :dm_send_div do
    entry 0, :not_prefer, '希望しない'
    entry 1, :prefer, '希望する'
  end

  # CSV出力区分
  selectable_attr :csv_output_div do
    entry 0, :not_yet, '未出力'
    entry 1, :done, '出力済'
  end

  # 性別区分
  selectable_attr :sex do
    entry '1', :male, '男性'
    entry '2', :female, '女性'
  end

  # 職業区分
  selectable_attr :job_div do
    update_with_kbn KbnCode.name_div_id_by_key(:job)
  end

  # 登録サイト区分
  selectable_attr :regist_div do
    entry '1', :pc, 'PC'
    entry '2', :mobile, 'モバイル'
  end

  # 会員種別
  selectable_attr :member_category_div do
    update_with_kbn KbnCode.name_div_id_by_key(:member_category)
    entry '000', :general, '一般会員'
  end

  # 注意顧客種別
  selectable_attr :blacklist_div do
    update_with_kbn KbnCode.name_div_id_by_key(:blacklist)
  end

  #
  # 会員住所（海外配送用）を返す
  #
  def sub_address
    self.user_addresses.each do |user_address|
      return user_address if user_address.address_div_key == :user_en
    end
    return nil
  end

  #
  # 海外配送可能かどうかを返す
  #
  def shippable_to_oversea?
    # 住所が日本語の場合
    if self.user_address.locale == 'ja' && self.user_address.country_code == 'JP'
      # 会員住所（海外配送用）が設定されている場合
      if self.sub_address
        return true
      end
      # 会員住所が英語の場合
    else
      return true
    end
    return false
  end

  #
  # テスト会員かどうかを返す
  #
  def test?
    test_flg ? test_flg : false
  end

  # ハイフンを抜いた電話番号を返す
  def tel_num_not_hyphen
    self.tel_num.gsub("-", "")
  end

  # 誕生日をYYYY/MM/DDのフォーマットで返す
  def birth_day_yyyy_mm_dd
    if self.birth_year && self.birth_month && birth_day
      date = Date::new(self.birth_year, self.birth_month, self.birth_day)
      return date.strftime('%Y/%m/%d')
    end
    return nil
  end

  # 値をセットしてUserインスタンスを生成する
  def self.new_user(user_form, address_form, mail_address_form, sub_address_form, site_type_div = :pc)
    user_address = UserAddress.new(address_form)
    user_address.address_div_key = :user
    user_address.locale = I18n.locale.to_s

    user = User.new(user_form)
    user.mirror_from_address(user_address)
    # フロントで会員登録される場合必ずWEB会員、しない場合は非会員にする
    user.member_div_key = user.password.blank? ? :no_membership : :membership

    user_email = UserEmail.new(mail_address_form)

    site_type_div == :pc ? user.user_email = user_email : user.user_email_mobile = user_email
    user.user_address = user_address
    user.user_addresses << user_address

    if sub_address = UserAddress.hash_to_sub_address(sub_address_form)
      sub_address.set_sub_address_from_address(user_address)
      user.user_addresses << sub_address
    end

    return user
  end

  # 配送先として登録された住所を返す
  def shipping_addresses
    list = []
    self.user_addresses.each do |user_address|
      list << user_address if user_address.address_div_key == :ship_to
    end
    return list
  end

  # 会員住所と配送先住所を返す（会員住所（海外配送用）は含めない）
  def user_and_shipping_addresses
    return [self.user_address] + self.shipping_addresses
  end

  attr_accessor :password,
    :password_confirmation,
    :birth_check,  #生年月日をチェックする場合True
  :birth #生年月日全体のエラーメッセージに使用
  :locale #言語別必須チェック制御に使用

  attr_accessor :remember_question_check  #秘密の質問をチェックする場合True
  attr_accessor :not_decided_address  #住所が確定していない場合True
  attr_accessor :user_rank_check_end_date # 会員ランク判定終了日
  attr_accessor :regist_error # 会員登録時登録エラー有無

  #
  # バリデーション
  #
  validates_confirmation_of  :password,                  :if => :password_check?
  validates_presence_of     :sex,                        :message => :EM990001, :if => :gender_check?
  validates_presence_of     :password,                   :message => :EM990001,  :if => :password_required?
  validates_presence_of     :password_confirmation,      :message => :EM990001,  :if => :password_required?
  validates_length_of       :password, :within => 4..40, :if => :password_check?
  validates_confirmation_of :password,              :message => :EM990008, :if => :password_required?

  ## 追加分(User_formのバリデーション項目を追加)
  validates_presence_of    :remember_question,     :message => :EM990001, :if => :remember_question_check?
  validates_presence_of    :remember_answer,       :message => :EM990001, :if => :remember_question_check?

  #文字数バリデーション
  validates_length_of      :password_confirmation, :minimum => 4, :if => :password_check?
  #パスワードの複雑さバリデーション
  validates_format_of :password, :with => /^[a-zA-Z\d]*([a-zA-Z]\d|\d[a-zA-Z])[a-zA-Z\d]*$/, :message => :EM990042, :if => :password_check?

  validates_presence_of    :birth_year,            :message => :EM990001, :if => :birth_check?
  validates_presence_of    :birth_month,           :message => :EM990001, :if => :birth_check?
  validates_presence_of    :birth_day,             :message => :EM990001, :if => :birth_check?
  validates_presence_of    :job_div,                   :message => :EM990001, :if => :job_check?
  validates_presence_of    :pc_mail_magazine_send_div,   :message => :EM990001, :if => :pc_mail_magazine_check?
  validates_presence_of    :login_id,                  :message => :EM990001, :if => :login_id_check?
  validates_format_of :login_id, :with => /^[A-Za-z0-9_]*$/, :message => :EM990028 # 半角英数とアンダースコア

  # パスワード必須チェック
  def password_required?
    if is_membership? && !crypted_password
      return password.blank?
    else
      return !password.blank?
    end
  end

  # WEB会員かどうかを返す
  def is_membership?
    self.member_div_key == :membership
  end

  # オフライン専用会員かどうかを返す
  def is_offline_membership?
    self.member_div_key == :offline_membership
  end

  # パスワードチェック可否()
  def password_check?
    return !password.blank?
  end

  # 生年月日チェックするしない
  def birth_check?
    return (birth_check || ShopSetting.get_setting(:user_form_birthday, I18n.locale.to_s) == "true")
  end

  # 職業必須チェックするしない
  def job_check?
    return ShopSetting.get_setting(:user_form_job, I18n.locale.to_s) == "true"
  end

  def pc_mail_magazine_check?
    return ShopSetting.get_setting(:user_form_pc_mail_magazine, I18n.locale.to_s) == "true"
  end

  # 性別チェックするしない
  def gender_check?
    return ShopSetting.get_setting(:user_form_gender, I18n.locale.to_s) == "true"
  end

  # 秘密の質問チェックする
  def remember_question_check?
    return remember_question_check.nil? ? password_required? : remember_question_check
  end

  # ログインIDをチェックするしない
  def login_id_check?
    ShopSetting.get_setting(:user_form_login_id, I18n.locale.to_s) == "true"
  end

  # メールアドレスを持っているかどうかを返す
  def have_mailaddress?    
    !!(self.user_email && self.user_email.email)
  end

  # 携帯メールアドレスを持っているかどうかを返す
  def have_mobile_mailaddress?
    !!(self.user_email_mobile && self.user_email_mobile.email)
  end

  # ログインIDがユニークな場合はtrueを返す
  #　会員ユーザーのみが対象。非会員、退会会員の重複は許可します。
  #　本番会員とテスト会員の重複は許可します。
  def is_unique_login_id?
    if EcRider.config.ec_test
      conditions = ['login_id = ? and member_div = ? and id <> ? and test_flg = ?', self.login_id, User.member_div_id_by_key(:membership), self.id || '', true]
    else
      conditions = ['login_id = ? and member_div = ? and id <> ? and test_flg is null', self.login_id, User.member_div_id_by_key(:membership), self.id || '']
    end
    user_count = User.count(:conditions => conditions)
    return (user_count <= 0) || !(self.is_membership?)
  end

  def validate
    unless birth_year.blank? || birth_month.blank? || birth_day.blank?
      unless Date::exist?(birth_year.to_i, birth_month.to_i, birth_day.to_i)
        errors.add(:birth, :EM990007)
      end
    end
    errors.add(:login_id, :NM990009) if login_id_check? && !is_unique_login_id?
    errors.add(:email, :EM990001) if is_membership? && invalid_email?
  end

  def invalid_email?
    user_email.nil? || user_email.email.nil? || user_email.email.blank?
  end


  before_save :encrypt_password, :set_user_rank

  after_create :generate_user_code

  before_validation :set_test_flg

  def set_user_rank
    # 2009/11/17 現在、会員ランクが有効になっていないので未設定の場合は固定で設定しておきます。
    # 会員ランクの機能が有効になった時に不要であれば、このメソッドとbefore_saveの定義を削除してください。
    self.user_rank_id = 1 if self.user_rank_id.nil?
  end
  
  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  # メールアドレスからWEB会員を取得する
  def self.find_membership_user(email)
    where = ['(users.login_id = ? OR user_emails.email = ?)', 'users.member_div = ?']
    param = [email, email, User.member_div_id_by_key(:membership)]
    where << (EcRider.config.ec_test ? 'users.test_flg = 1' : '(users.test_flg = 0 OR users.test_flg IS NULL)')
    find(:first, :include => :user_emails, :conditions => [where.join(' AND ')]+param)
  end

  # Authenticates a user by their user_email and unencrypted password.  Returns the user or nil.
  def self.authenticate(email, password)
    user = find_membership_user(email)
    user && user.authenticated?(password) ? user : nil
  end

  # = ロックされているかどうか = 
  # ロックされている場合にtrue, それ以外にfalseを返す。 
  def locked? 
    return false if login_challenge_count.nil? || locked_at.nil? 
    if login_challenge_count >= MAX_LOGIN_CHALLENGE_COUNT && 
        locked_at.since(USER_LOCK_PERIOD_SECONDS) > DateTime.now
      #ログイン試行回数とロック時刻でロック中を判断 
      return true 
    end 
    false 
  end 

  # = ログイン試行回数をインクリメントする = 
  # ログイン試行回数を増加させ、最大試行回数に達したら現在時刻を 
  # ユーザロック時刻として登録する。 
  def increment_login_challenge_count 
    return if locked? 
    if self.login_challenge_count.nil? || 
        (self.locked_at && self.locked_at.since(USER_LOCK_PERIOD_SECONDS) < DateTime.now) 
      #未初期化、およびロック期間が終了している場合 
      clear_login_challenge_count 
    end 
    self.login_challenge_count = self.login_challenge_count + 1 
    self.locked_at = DateTime.now if MAX_LOGIN_CHALLENGE_COUNT <= self.login_challenge_count 
    save! 
  end 

  # = ログイン試行回数、ロック時刻を初期化する = 
  # ログイン成功時、ロック期間が経過した際に初期化するための処理。 
  def clear_login_challenge_count 
    self.login_challenge_count = 0 
    self.locked_at = nil 
    save! 
  end

  # Encrypts some data with the salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # Encrypts the password with the user salt
  def encrypt(password)
    self.class.encrypt(password, salt)
  end

  def authenticated?(password)
    crypted_password == encrypt(password)
  end

  def remember_token?
    #remember_token_expires_at && Time.now.utc < remember_token_expires_at
    remember_token_expires_at
  end

  # These create and unset the fields required for remembering users between browser closes
  def remember_me
    self.remember_token_expires_at = 2.weeks.from_now.utc
    self.remember_token = encrypt("#{user_email.email}--#{remember_token_expires_at}")
    save(false)
  end

  def forget_me
    self.remember_token_expires_at = nil
    self.remember_token            = nil
    save(false)
  end

  # 検索条件生成
  # このメソッドは使わないでください。User::SearchConditionForm#build_conditionを使ってください。
  def self.create_condition(form, time_now = Time.now)
    form.build_condition(time_now)
  end

  #
  # 退会する
  #
  def resigned(resigned_reason)
    resigened_at = Time.now
    resigned_reason[:sex] = self.sex unless self.sex.nil?
    resigned_reason[:age] = self.age unless self.birthday.nil?
    resigned_reason[:job_div] = self.job_div unless self.job_div.nil?
    resigned_reason[:admission_at] = self.created_at
    resigned_reason[:resigened_at] = resigened_at
    self.resigned_reason = resigned_reason
    self.member_div_key = :resigned
    self.resigened_at = resigened_at
    self.save!
  end

  #
  # パスワード再設定
  #
  def self.remind(email, remember_question, remember_answer, new_passwd, password_confirmation)
    user = find_membership_user(email)
    return false unless user
    return false unless user.remember_question.eql?(remember_question)
    return false unless user.remember_answer.eql?(remember_answer)
    user.password = new_passwd
    user.password_confirmation = password_confirmation
    user.save!
    return true
  end

  #
  # = 生年月日に対応した年齢を返します。 =
  #
  def age(time_now = Time.now)
    now_date = time_now.strftime("%Y%m%d").to_i
    birth_date = birthday.strftime("%Y%m%d").to_i
    return (now_date - birth_date) / 10000
  end

  #
  # = 購入履歴の総額を返す =
  #
  # 購入額が存在しない場合は0を返します。
  def total_purchase
    OrderShipping.sum('total_amount', :include => :order,
      :conditions => ['orders.user_id = ?', self.id])
  end


  # 会員ランク判定に使用
  # = 会員ランク更新日時～会員ランク判定終了日時までの購入履歴の総額を返す =
  #
  # 購入額が存在しない場合は0を返します。
  def total_purchase_for_distance
    if self.user_rank_check_end_date.nil?
      return total_purchase
    else
      return OrderShipping.sum('total_amount', :include => :order,
        :conditions => ['orders.user_id = ? AND orders.ordered_at >= (?) AND orders.ordered_at <= (?)',
          self.id, self.user_rank_updated_at, self.user_rank_check_end_date])
    end
  end

  # 会員ランク判定に使用
  # = 会員ランク更新日時～会員ランク判定終了日時までの購入回数を返す =
  #
  # 購入額が存在しない場合は0を返します。
  def order_count_for_distance
    if self.user_rank_check_end_date.nil?
      return Order.count(:all, :conditions => ['orders.user_id = ?', self.id])
    else
      return Order.count(:all, :conditions => ['orders.user_id = ? AND orders.ordered_at >= (?) AND orders.ordered_at <= (?)',
          self.id, self.user_rank_updated_at, self.user_rank_check_end_date])
    end
  end

  # 会員登録日から本日までの日数を返却します。
  def count_day_of_member_created
    return (Time.now - self.created_at).divmod(24*60*60)[0]
  end

  # 会員の最終購入日から本日までの日数を返却します。
  def count_day_of_ordered
    status_div_ids = []
    OrderShipping::STATUS_REGULAR_TRANSITION.each do |key|
      status_div_ids << OrderShipping.status_div_id_by_key(key)
    end
    order = Order.find(:first, :include => :order_shippings,
      :conditions => ["order_shippings.status_div in (?)", status_div_ids], :order => "orders.ordered_at desc")
    result = order.nil? ? 0 : (Time.now - order.ordered_at).divmod(24*60*60)[0]
    return result
  end
  
  #
  # = 削除可能かどうかを判断します =
  #
  def deletable?
    result = true
    self.orders.each do |order|
      order.order_shippings.each do |order_shipping|
        result = false unless order_shipping.status_div_entry[:user_deletable]
      end
    end
    result
  end

  # セキュリティ対策
  # 実行した会員が作成した情報である場合true,そうでない場合falseを返却します。
  # パラメータ改ざんされ、別会員の個人情報が閲覧できないように自分の登録した情報かをチェックするメソッドです。
  def users_data?(model_name, params_id)
    case model_name
    when "order"
      order_ids = self.orders.collect {|order| order.id}
      return order_ids.index(params_id.to_i).nil?
    when "user_address"
      address_ids = self.user_addresses.collect {|user_address| user_address.id}
      return address_ids.index(params_id.to_i).nil?
    end
  end


  #
  # addressに含まれる会員名、電話番号をuserに反映する
  #
  def mirror_from_address(address)
    self.name_last = address.name_last
    self.name_first = address.name_first
    self.kana_name_last = address.kana_name_last
    self.kana_name_first = address.kana_name_first
    self.tel_num = address.tel_num
    self.fax_num = address.fax_num
  end

  #
  # 姓名を連結して返す
  #
  def fullname
    (user_address.nil? ? '' : user_address.fullname)
  end

  #
  # 姓名のヨミガナを連結して返す
  #
  def fullname_kana
    (user_address.nil? ? '' : user_address.fullname_kana)
  end

  #
  # 敬称を返す
  #
  def honorific_prefix
    honorific = {:male =>'Mr.', :female => 'Ms.'}
    return honorific[sex_key]
  end

  #
  # = パスワード変更 =
  # パスワードとパスワード確認を引数とし、バリデーションチェックを行い
  # 新しいパスワードに更新します。
  #
  def change_password(password, password_confirmation)
    self.password = password
    self.password_confirmation = password_confirmation
    self.crypted_password = nil

    if self.valid? && self.save
      return true
    end

    return false
  end

  # 誕生日をDateで返します。
  def birthday
    (birth_year && birth_month && birth_day) ? Date.new(birth_year, birth_month, birth_day) : nil
  end

  # 誕生日(月日)をStringで返します。
  def birthday_of_year
    d = birthday
    d ? d.strftime('%m/%d') : nil
  end

  # キャンペーンの条件で使用される住所の国コードを返します。
  def address_country_code
    user_address.nil? ? '' : user_address.country_code
  end

  # キャンペーンの条件で使用される住所を返します。
  def address_for_campaign_condition
    user_address.nil? ? '' : user_address.fulladdress(true, :delimeter => '')
  end

  # 住所が離島の場合にtrueを返します。
  # キャンペーンの条件で使用されます。
  def address_isolated?
    address = self.user_address
    zipcode = address ? address.zip_code : ""
    !(ShopSetting.shipping_cost_outlier_or_nil(zipcode).nil?)
  end


  def prepare_to_safe_eval
    super
    user_address.prepare_to_safe_eval if user_address
  end
  
  def save_as_no_membership(only_member_div = false)
    self.member_div_key = :no_membership
    unless only_member_div
      self.password = nil
      self.crypted_password = nil
      self.salt = nil
      self.remember_answer = nil
      self.remember_question = nil
    end
    self.save!
  end

  protected
  # before filter
  def encrypt_password
    return if password.blank?
    self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{kana_name_first}--") if new_record?
    self.crypted_password = encrypt(password)
  end

  #
  # ユーザーコードを生成 
  #
  def generate_user_code
    self.user_code = sprintf("%012d", self.id)
    self.save!
  end

end
