# -*- coding: utf-8 -*-
class User < ActiveRecord::Base

  #
  # = 管理画面の顧客検索の入力フォーム =
  #
  class SearchConditionForm < ActiveForm
    ATTRIBUTE_NAMES = [:user_code, :name, :address, :email, :sex, :tel_num, :old, :member_div,
    :blacklist_div, :regist_date_from, :regist_date_to, :disp_count, :test_flg]
    attr_accessor *ATTRIBUTE_NAMES

    def disp_count
      @disp_count.to_i == 0 ? 20 : @disp_count.to_i
    end

    # 日付のバリデーション(yyyy/MM/dd)
    self.with_options(:with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001) do |s|
      # 開始日
      s.validates_format_of :regist_date_from, :if => :regist_date_from_unempty?
      # 終了日
      s.validates_format_of :regist_date_from, :if => :regist_date_to_unempty?
    end

    # regist_date_fromに値がある場合trueをかえします。
    def regist_date_from_unempty?
      regist_date_from && !regist_date_from.empty?
    end

    # regist_date_toに値がある場合trueをかえします。
    def regist_date_to_unempty?
      regist_date_to&& !regist_date_to.empty?
    end

    #
    # 入力日付が存在する日付か否かをチェック
    #
    def validate
      if regist_date_from_unempty?
        _date = regist_date_from.split('/')
        errors.add(:regist_date_from, :EM990007) unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
      end
      if regist_date_to_unempty?
        _date = regist_date_to.split('/')
        errors.add(:regist_date_to, :EM990007) unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
      end
    end

    # = 登録日時FROMをTime型で返す。 =
    def regist_date_from_time
      self.regist_date_from.blank? ? nil : Time.parse("#{self.regist_date_from} 00:00:00")
    end

    # = 登録日時TOをTime型で返す。 =
    def regist_date_to_time
      self.regist_date_to.blank? ? nil : Time.parse("#{self.regist_date_to} 23:59:59")
    end

    # = ハイフンを抜いた電話番号 =
    def tel_num_not_hyphen
      self.tel_num ? self.tel_num.gsub("-", "") : nil
    end

    def build_condition(time_now = Time.now)
      Condition.block do |c|
        c.filter(:ignore_blank)
        # ユーザーコードの条件
        c.and "user_code", "like", "%#{self.user_code}%"
        # 名前の条件
        c.and do |c1|
          _name = delete_space(self.name)
          c1.or "CONCAT(users.name_last, users.name_first)", "like", "%#{_name}%"
          c1.or "CONCAT(users.name_first, users.name_last)", "like", "%#{_name}%"
          c1.or "CONCAT(users.kana_name_last, users.kana_name_first)", "like", "%#{_name}%"
          c1.or "CONCAT(users.kana_name_first, users.kana_name_last)", "like", "%#{_name}%"
        end if self.name && !(self.name.empty?)
        # 性別の条件
        c.and :sex, self.sex
        # 住所の条件
        c.and do |c2|
          c2.or "CONCAT(user_addresses.address_1, CONCAT(user_addresses.address_2, user_addresses.address_3))", "like", "%#{delete_space(self.address)}%"
        end
        # メールアドレスの条件
        c.and "user_emails.email", "like", "%#{self.email}%"
        # 電話番号/FAX番号の条件
        c.and  do |c3|
          c3.or "replace(users.tel_num, '-', '')", "like", "%#{self.tel_num_not_hyphen}%"
          c3.or "replace(users.fax_num, '-', '')", "like", "%#{self.tel_num_not_hyphen}%"
        end
        # 顧客区分の条件
        c.and :member_div, self.member_div
        # 住所の論理削除の条件
        c.and do |c4|
          c4.or "sql", "user_addresses.deleted_at IS NULL"
          c4.or "sql", "user_addresses.deleted_at > '#{time_now.strftime('%Y-%m-%d %H:%M:%S')}'"
        end
        # 登録日Fromの条件
        c.and "users.created_at", ">=", self.regist_date_from_time
        # 登録日Toの条件
        c.and "users.created_at", "<=", self.regist_date_to_time
        # emailの論理削除の条件
        c.and do |c5|
          c5.or "sql", "user_emails.deleted_at IS NULL"
          c5.or "sql", "user_emails.deleted_at > '#{time_now.strftime('%Y-%m-%d %H:%M:%S')}'"
        end
        # 注意顧客区分の条件
        c.and :blacklist_div, self.blacklist_div
        # テスト会員の条件
        if self.test_flg == CmsPublish.target_id_by_key(:test)
          c.and :test_flg, true
        elsif self.test_flg == CmsPublish.target_id_by_key(:production)
          c.and "sql", "test_flg IS NULL"
        end
      end
    end

    private

    # 指定の文字列のスペースを削除します。
    def delete_space(str)
      return nil unless str
      str.gsub(' ', '').gsub('　', '')
    end
    
  end

end
