# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe UserController do
  fixtures :users, :user_emails, :user_addresses, :mail_templates,
    :mail_deliveries, :kbn_codes, :kbn_code_names, :user_mobiles,
    :user_points, :user_ranks, :point_settings, :shop_settings,
    :goldberg_users, :goldberg_site_actions, :goldberg_site_action_groups,
    :goldberg_roles, :goldberg_role_actions

  # 指定したユーザ名でログインします。
  def login_as(user_name)
    user = Goldberg::User.find_by_name(user_name)
    Goldberg::User.model_stamper    # UserがIDを覚えられるようにする。
    Goldberg::User.stamper = user   # IDを覚えておく。stamper=にはオブジェクトを渡してもOK。
    Goldberg.user = user
    session[:user] = user.id
    session[:current_user] = user
  end

  # 正常なデータを保持したインスタンスを生成します。
  def setup_user_instance
    @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト",
      :kana_name_first => "ジロウ",   :name_last => "てすと",
      :name_first => "じろう", :zip_code => "0000000",  :address_1 => "愛知県",
      :address_2 => "名古屋市中区",
      :address_3 => "丸の内１－１－１ EC-Riderテストマンション1F",
      :country_code => 'JP', :locale => 'ja'}
    @user_email = {:email => "test@test.jp"}
    @user_email_mobile = {:email => "test@softbank.ne.jp"}
    @user = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
      :member_div_key => :membership, :birth_day => "23", :sex => "1",
      :birth_year => "1970", :pc_mail_magazine_send_div_key => :not_prefer,
      :password => 'test1', :password_confirmation => 'test1',
      :remember_question => 'Q', :remember_answer => 'A'}
  end

  # モデルの件数を取得します。
  def setup_model_count
    @user_count         = User.count
    @user_email_count   = UserEmail.count
    @user_address_count = UserAddress.count
  end

  before(:each) do
    login_as("admin")
    setup_user_instance
  end

  describe "new" do
    it "初期表示時" do
      get :new
      assigns[:user].sex_key.should  == :male
      assigns[:user].dm_send_div_key.should == :prefer
      assigns[:user].member_div_key.should == :membership
      assigns[:user].pc_mail_magazine_send_div_key.should == :prefer_html
      assigns[:user].mobile_mail_magazine_send_div_key.should == :prefer_text
      assigns[:user_address].locale.should == I18n.locale
      assigns[:user_address].country_code.should ==
        ShopSetting.get_setting(:default_address_country, I18n.locale)
      assigns[:user_email].email.should be_nil
      assigns[:user_email_mobile].email.should be_nil
    end
    it "バリデーションエラー時、モバイルサイトのメール情報がない場合" do
      flash[:user] = User.new(@user)
      flash[:user_address] = UserAddress.new(@user_address)
      flash[:user_email] = UserEmail.new(@user_email)
      get :new
      assigns[:user_email].email.should == "test@test.jp"
      assigns[:user_email_mobile].email.should be_nil
    end
    it "バリデーションエラー時、PCサイトのメール情報がない場合" do
      flash[:user] = User.new(@user)
      flash[:user_address] = UserAddress.new(@user_address)
      flash[:user_email_mobile] = UserEmail.new(@user_email_mobile)
      get :new
      assigns[:user_email].email.should be_nil
      assigns[:user_email_mobile].email.should == "test@softbank.ne.jp"
    end
  end

  describe "create" do
    describe "[Web会員]" do
      it "PCサイト、モバイルサイト共にEmailが入力されていない場合、入力画面に戻る。" do
        post :create, :user => @user, :user_address => @user_address,
          :user_email => {:email => ""}, :user_email_mobile => {:email => ""}
        response.should redirect_to(:action => :new)
        flash[:user].errors.length.should == 1
        flash[:user_address].errors.length.should == 0
        flash[:user_email].should be_nil
        flash[:user_email_mobile].should be_nil
      end
      it "モバイルサイトのみEmailを入力した場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => {:email => ""}, :user_email_mobile => @user_email_mobile
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 1
      end
      it "PCサイトのみEmailを入力した場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => @user_email, :user_email_mobile => {:email => ""}
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 1
      end
      it "入力漏れがない場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => @user_email, :user_email_mobile => @user_email_mobile
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 2
      end
    end

    describe "[オフライン会員]" do
      before(:each) do
        @user[:member_div_key] = :offline_membership
      end
      it "PCサイト、モバイルサイト共にEmailが入力されていない場合でも、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => {:email => ""}, :user_email_mobile => {:email => ""}
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count
      end
      it "モバイルサイトのみEmailを入力した場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => {:email => ""}, :user_email_mobile => @user_email_mobile
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 1
      end
      it "PCサイトのみEmailを入力した場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => @user_email, :user_email_mobile => {:email => ""}
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 1
      end
      it "入力漏れがない場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => @user_email, :user_email_mobile => @user_email_mobile
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 2
      end
    end

    describe "[非会員]" do
      before(:each) do
        @user[:member_div_key] = :no_membership
      end
      it "PCサイト、モバイルサイト共にEmailが入力されていない場合でも、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => {:email => ""}, :user_email_mobile => {:email => ""}
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count
      end
      it "モバイルサイトのみEmailを入力した場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => {:email => ""}, :user_email_mobile => @user_email_mobile
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 1
      end
      it "PCサイトのみEmailを入力した場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => @user_email, :user_email_mobile => {:email => ""}
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 1
      end
      it "入力漏れがない場合、ユーザ情報が登録される。" do
        setup_model_count
        post :create, :user => @user, :user_address => @user_address,
          :user_email => @user_email, :user_email_mobile => @user_email_mobile
        response.should redirect_to(:action => :index)
        User.count.should == @user_count + 1
        UserAddress.count.should == @user_address_count + 1
        UserEmail.count.should == @user_email_count + 2
      end
    end
  end

  describe "edit" do
    it "初期表示時" do
      user = User.find(1008)
      get :edit, :id => 1008, :page => '1'
      response.should render_template(:edit)
      assigns[:user].should == user
      assigns[:user_address].should == user.user_address
      assigns[:user_email].should == user.user_email
      assigns[:user_email_mobile].should == user.user_email_mobile
      assigns[:page_no].should == '1'
    end
    it "バリデーションエラー時、モバイルサイトのメール情報がない場合" do
      flash[:user] = User.new(@user)
      flash[:user_address] = UserAddress.new(@user_address)
      flash[:user_email] = UserEmail.new(@user_email)
      flash[:user_email_mobile] = nil
      get :edit
      assigns[:user_email].email.should == "test@test.jp"
      assigns[:user_email_mobile].should be_nil
    end
    it "バリデーションエラー時、PCサイトのメール情報がない場合" do
      flash[:user] = User.new(@user)
      flash[:user_address] = UserAddress.new(@user_address)
      flash[:user_email_mobile] = UserEmail.new(@user_email_mobile)
      get :edit
      assigns[:user_email].should be_nil
      assigns[:user_email_mobile].email.should == "test@softbank.ne.jp"
    end
  end

  describe "update" do
    # PCおよびモバイルサイトのメールアドレスを持った
    # ユーザの情報をDBに登録しておきます。
    before(:each) do
      @stub_user = User.find(1008)
      user_email = UserEmail.new(:email =>'user_update_test@ec-one.com',
        :pc_mobile_div_key => :pc)
      @stub_user.user_email.destroy if @stub_user.user_email
      @stub_user.user_email = user_email
      user_email.user = @stub_user
      user_email_mobile = UserEmail.new(:email =>'user_update_test@softbank.ne.jp',
        :pc_mobile_div_key => :mobile)
      @stub_user.user_email_mobile.destroy if @stub_user.user_email_mobile
      @stub_user.user_email_mobile = user_email_mobile
      user_email_mobile.user = @stub_user
      @stub_user.save!
      user_email.save!
      user_email_mobile.save!
      User.stub!(:find).and_return(@stub_user)
    end
    describe "[Web会員]" do
      before(:each) do
        @user[:member_div_key] = :membership
        @stub_user.member_div_key = :membership
      end
      describe "【異常系】両サイトのメールアドレスがDBに無いとき、" do
        before(:each) do
          @stub_user.user_email.destroy if @stub_user.user_email
          @stub_user.user_email = nil
          @stub_user.user_email_mobile.destroy if @stub_user.user_email_mobile
          @stub_user.user_email_mobile = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :edit, :page => '1', :id => '1008')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
          flash[:user].errors_on(:email).should_not be_nil
          flash[:user].errors_on(:email).should == ["を入力してください。"]
          flash[:user].errors.length.should == 1
          flash[:user_email].should be_nil
          flash[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "PCサイトのメールアドレスだけがDBにあったとき、" do
        before(:each) do
          @stub_user.user_email_mobile.destroy if @stub_user.user_email_mobile
          @stub_user.user_email_mobile = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :edit, :page => '1', :id => '1008')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "モバイルサイトのメールアドレスだけがDBにあったとき、" do
        before(:each) do
          @stub_user.user_email.destroy if @stub_user.user_email
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :edit, :page => '1', :id => '1008')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "両サイトのメールアドレスがDBにあったとき、" do
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :edit, :page => '1', :id => '1008')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
    end

    describe "[オフライン会員]" do
      before(:each) do
        @user[:member_div_key] = :offline_membership
        @stub_user.member_div_key = :offline_membership
      end
      describe "両サイトのメールアドレスがDBに無いとき、" do
        before(:each) do
          @stub_user.user_email.destroy if @stub_user.user_email
          @stub_user.user_email = nil
          @stub_user.user_email_mobile.destroy if @stub_user.user_email_mobile
          @stub_user.user_email_mobile = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "PCサイトのメールアドレスだけがDBにあったとき、" do
        before(:each) do
          @stub_user.user_email_mobile.destroy if @stub_user.user_email_mobile
          @stub_user.user_email_mobile = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "モバイルサイトのメールアドレスだけがDBにあったとき、" do
        before(:each) do
          @stub_user.user_email.destroy if @stub_user.user_email
          @stub_user.user_email = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "両サイトのメールアドレスがDBにあったとき、" do
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
    end

    describe "[非会員]" do
      before(:each) do
        @user[:member_div_key] = :no_membership
        @stub_user.member_div_key = :no_membership
      end
      describe "両サイトのメールアドレスがDBに無いとき、" do
        before(:each) do
          @stub_user.user_email.destroy if @stub_user.user_email
          @stub_user.user_email = nil
          @stub_user.user_email_mobile.destroy if @stub_user.user_email_mobile
          @stub_user.user_email_mobile = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "PCサイトのメールアドレスだけがDBにあったとき、" do
        before(:each) do
          @stub_user.user_email_mobile.destroy if @stub_user.user_email_mobile
          @stub_user.user_email_mobile = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "モバイルサイトのメールアドレスだけがDBにあったとき、" do
        before(:each) do
          @stub_user.user_email.destroy if @stub_user.user_email
          @stub_user.user_email = nil
        end
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
      describe "両サイトのメールアドレスがDBにあったとき、" do
        it "両サイトのメールアドレスが何も入力されなかった場合" do
          post :update, :user => @user, :user_address => @user_address, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].should be_nil
        end
        it "PCサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].should be_nil
        end
        it "モバイルサイトのメールアドレスのみ入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email_mobile => @user_email_mobile, :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].should be_nil
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
        it "両サイトのメールアドレスが入力された場合" do
          post :update, :user => @user, :user_address => @user_address,
            :user_email => @user_email, :user_email_mobile => @user_email_mobile,
            :id => '1008', :page => '1'
          response.should redirect_to(:action => :index, :page => '1')
          assigns[:user_email].email.should == 'test@test.jp'
          assigns[:user_email_mobile].email.should == 'test@softbank.ne.jp'
        end
      end
    end
  end
end