# -*- coding: utf-8 -*-
require 'ec_rider'
require 'uri'

module EcRider
  class ConfigurationError < StandardError
  end

  class Config

    attr_accessor :default_ec_admin_url
    attr_accessor :default_ec_front_url
    attr_accessor :default_ec_test_url

    # ブロック中でのみ設定を書き換えます
    def override(config_name, value)
      backup = send(config_name)
      send("#{config_name}=", value)
      begin
        yield if block_given?
      ensure
        send("#{config_name}=", backup)
      end
    end

    # ec_adminの実行時のパス
    attr_accessor_with_default(:ec_admin_root) do
      @ec_admin_root ||= ENV['ECR_ADMIN_ROOT'] || ENV['EC_RIDER_ADMIN_ROOT'] || (
        ENV['EC_RIDER_HOME'] ?
          File.join(ENV['EC_RIDER_HOME'], 'ec_admin') : File.join(RAILS_ROOT, '..', 'ec_admin')
      )
    end

    # ec_frontの実行時のパス
    attr_accessor_with_default(:ec_front_root) do
      @ec_front_root ||= ENV['ECR_FRONT_ROOT'] || ENV['EC_RIDER_FRONT_ROOT'] || (
        ENV['EC_RIDER_HOME'] ?
          File.join(ENV['EC_RIDER_HOME'], 'ec_front') : File.join(RAILS_ROOT, '..', 'ec_front')
      )
    end

    # ec_testの実行時のパス
    attr_accessor_with_default(:ec_test_root) do
      @ec_test_root ||= ENV['ECR_TEST_ROOT'] || ENV['EC_RIDER_TEST_ROOT'] || (
        ENV['EC_RIDER_HOME'] ?
          File.join(ENV['EC_RIDER_HOME'], 'ec_test') : File.join(RAILS_ROOT, '..', 'ec_test')
      )
    end

    # ec_adminの基本URL
    attr_accessor_with_default(:ec_admin_url) do
      (@ec_admin_url ||= ENV['ECR_ADMIN_URL'] || ENV['EC_RIDER_ADMIN_URL'] || @default_ec_admin_url) or
        raise no_config(:ec_admin_url, "set 'ECR_ADMIN_URL' environment variable")
    end

    # ec_frontの基本URL
    attr_accessor_with_default(:ec_front_url) do
      (@ec_front_url ||= ENV['ECR_FRONT_URL'] || ENV['EC_RIDER_FRONT_URL'] || @default_ec_front_url) or
        raise no_config(:ec_front_url, "set 'ECR_FRONT_URL' environment variable")
    end

    # ec_testの基本URL
    attr_accessor_with_default(:ec_test_url) do
      (@ec_test_url ||= ENV['ECR_TEST_URL'] || ENV['EC_RIDER_TEST_URL'] || @default_ec_test_url) or
        raise no_config(:ec_test_url, "set 'ECR_TEST_URL' environment variable")
    end

    # ec_adminのHTTPSでアクセスする際のURL
    attr_accessor_with_default(:ec_admin_https_url) do
      # 設定する項目はありませんが、他の設定と密接に関係するので、ここに定義しておきます。
      @https_uri ||= (
        http_uri = URI(ec_admin_url)
        URI::HTTPS.build(
          :userinfo => http_uri.userinfo,
          :host => http_uri.host,
          :port => self.ec_admin_https_port,
          :path => http_uri.path,
          :query => http_uri.fragment,
          :fragment => http_uri.fragment
        ).to_s
      )
    end

    # ec_adminのHTTPSでアクセスする際のポート
    attr_accessor_with_default(:ec_admin_https_port) do
      @ec_admin_https_port ||= (ENV['ECR_ADMIN_HTTPS_PORT'] || 8443)
    end

    # ec_testとして起動しているか
    attr_accessor_with_default(:ec_test) do
      if File.exists?(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        test_environment = YAML::load_file(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        test_environment["ec_test"]
      else
        false
      end
    end

    # ec_frontとして起動しているか
    attr_accessor_with_default(:ec_front) do
      if File.exists?(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        test_environment = YAML::load_file(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        !test_environment["ec_test"]
      else
        false
      end
    end

    # ec_adminとして起動しているか
    attr_accessor_with_default(:ec_admin) do
      !File.exists?(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
    end

    # ec_frontのBASIC認証アカウント。ステージングで起動したときのために定義しておきます。
    attr_accessor_with_default(:ec_front_basic_auth_account) do
      @ec_front_basic_account = ENV['ECR_FRONT_BASIC_AUTH_ACCOUNT']
    end

    # ec_frontのBASIC認証パスワード。ステージングで起動したときのために定義しておきます。
    attr_accessor_with_default(:ec_front_basic_auth_password) do
      @ec_front_basic_password = ENV['ECR_FRONT_BASIC_AUTH_PASSWORD']
    end

    # ec_testのBASIC認証アカウント
    attr_accessor_with_default(:ec_test_basic_auth_account) do
      @ec_test_basic_account = ENV['ECR_TEST_BASIC_AUTH_ACCOUNT']
    end

    # ec_testのBASIC認証パスワード
    attr_accessor_with_default(:ec_test_basic_auth_password) do
      @ec_test_basic_password = ENV['ECR_TEST_BASIC_AUTH_PASSWORD']
    end

    # このアプリケーションがサポートしているロケールを指定します
    attr_accessor_with_default(:ec_locale) do
      @ec_locale = (ENV['ECR_LOCALE'] ? ENV['ECR_LOCALE'].split(' ') : %w(ja ko zh_CN zh_TW en th))
    end

    # CMSのgitで管理するディレクトリ。.gitがあるディレクトリ
    attr_accessor_with_default(:cms_git_dir) do
      @cms_git_dir ||= ENV['ECR_CMS_GIT_DIR']
    end

    # 障害発生時のエラーメール送付先
    attr_accessor_with_default(:error_mail_from_address, %w(xxxxxx@xxxxxxx.com))

    # ユーザー,注文などのCSVを保存しておくディレクトリの親(になるかもしれない)ディレクトリ
    attr_accessor_with_default(:csv_dir) do
      ENV['ECR_CSV_DIR'] || @csv_dir || File.join(ec_admin_root, 'tmp', 'csv_file_download')
    end

    # CSVアップロードのエラーCSVを保存しておくディレクトリ
    attr_accessor_with_default(:csv_upload_dir) do
      ENV['ECR_CSV_UPLOAD_DIR'] || @csv_upload_dir || File.join(ec_admin_root, 'tmp', 'csv_file_upload')
    end

    # ユーザーCSVを保存しておくディレクトリ
    attr_accessor_with_default(:csv_users_dir) do
      (ENV['ECR_CSV_USERS_DIR'] || @csv_users_dir) or
        (csv_dir ? File.join(csv_dir, 'users') : nil) or
        raise no_config(:csv_users_dir, "set 'ECR_CSV_USERS_DIR' environment variable")
    end

    # 注文CSVを保存しておくディレクトリ
    attr_accessor_with_default(:csv_orders_dir) do
      (ENV['ECR_CSV_ORDERS_DIR'] || @csv_orders_dir) or
        (csv_dir ? File.join(csv_dir, 'orders') : nil) or
        raise no_config(:csv_orders_dir, "set 'ECR_CSV_ORDERS_DIR' environment variable")
    end

    # お問い合わせ画面の確認用画像を保存しておくディレクトリ
    #
    attr_accessor_with_default(:tmp_inquiry_images_dir) do
      ENV['ECR_TMP_INQUIRY_IMAGES_DIR'] || @tmp_inquiry_images_dir || File.join(RAILS_ROOT, 'public', 'tmp_inquiry_images')
    end

    # productionで動作させる環境でSSLを無効にしたい場合には、この環境変数を設定してください
    attr_accessor_with_default(:ssl_ignored) do
      @ssl_ignored ||= !!(ENV['ECR_SSL_IGNORED'] =~ /true/i)
    end

    # ec_frontのセッション保持期間（日）
    attr_accessor_with_default(:ec_front_session_cache_expire) do
      @ec_front_session_cache_expire ||= ENV['EC_FRONT_SESSION_CACHE_EXPIRE'] ? ENV['EC_FRONT_SESSION_CACHE_EXPIRE'].to_i : 40
    end

    # 結合テスト用の共通テーブル
    attr_accessor_with_default(:team_testing_enabled) do
      ActiveRecord::Base.configurations['team_testing'] && ENV['RAILS_ENV'] != 'test'
    end

    # CloudFrontのURL
    attr_accessor_with_default(:cloud_front_url) do
      @cloud_front_url ||= ENV['ECR_CLOUD_FRONT_URL']
    end

    # Amazon Simple Storage Service (S3)のACCESS_KEY_ID
    attr_accessor_with_default(:s3_access_key_id) do
      @s3_access_key_id ||= ENV['ECR_S3_ACCESS_KEY_ID']
    end

    # Amazon Simple Storage Service (S3)のURL
    attr_accessor_with_default(:s3_secret_access_key) do
      @s3_secret_access_key ||= ENV['ECR_S3_SECRET_ACCESS_KEY']
    end

    # Amazon Simple Storage Service (S3)のBucket名
    attr_accessor_with_default(:s3_bucket_name) do
      @s3_bucket_name ||= ENV['ECR_S3_BUCKET_NAME']
    end




    # 簡体字入力許可
    attr_accessor_with_default(:chinese_format_using) do
      @chinese_format_using ||= ENV['CHINESE_FORMAT_USING']
      raise no_config(:chinese_format_using, "set 'CHINESE_FORMAT_USING' environment variable")
    end

    # 決済システムID（1:GMO, 2:SPS）
    attr_accessor_with_default(:payment_system_div) do
      @payment_system_div ||= ENV['PAYMENT_SYSTEM_DIV'] if ENV['RAILS_ENV'] == 'production'
      @payment_system_div ||= '1'
    end

    # 支払方法
    attr_accessor_with_default(:ecr_payment_types) do
      default_array = ['cod', 'credit', 'bank_transfer']
      @ecr_payment_types = (ENV['ECR_PAYMENT_TYPES'] ? ENV['ECR_PAYMENT_TYPES'].split(',') : default_array)
    end

    # Passengerが有効かどうか
    attr_accessor_with_default(:enable_passenger) do
      @enable_passenger = ENV['ECR_ENABLE_PASSENGER'] || false
    end

    private

    def no_config(name, msg)
      ConfigurationError.new("no setting for '#{name.to_s}': #{msg}")
    end
  end
end
