# -*- coding: utf-8 -*-

# テストを実行する際はschema_commentsのschema_comments.ymlへの出力を抑制します。
Rake.application.send(:eval, "@tasks.delete('db:test:prepare')")
namespace :db do
  namespace :test do
    desc 'Check for pending migrations and load the test schema'
    task :prepare => 'db:abort_if_pending_migrations' do
      SchemaComments.quiet = true
      puts "SchemaComments.quiet => #{SchemaComments.quiet.inspect}"
      if defined?(ActiveRecord) && !ActiveRecord::Base.configurations.blank?
        Rake::Task[{ :sql  => "db:test:clone_structure", :ruby => "db:test:load" }[ActiveRecord::Base.schema_format]].invoke
      end
    end
  end

  desc 'Migrate the database include ECR plugins and Load schema_comments'
  task :migrate_all => :environment do
    EcRider::SchemaCommentsExt.dump_schema_comments
    Rake::Task["db:migrate"].invoke
    Rake::Task["ecr:plugins:migrate_all"].invoke
    EcRider::SchemaCommentsExt.load_schema_comments
  end

  namespace :schema_comments do
    desc 'Load schema_comments from yaml to database'
    task :load => :environment do
      EcRider::SchemaCommentsExt.load_schema_comments
    end

    desc 'Dump schema_comments from database to yaml'
    task :dump => :environment do
      EcRider::SchemaCommentsExt.dump_schema_comments
    end
  end
end


require 'fileutils'
namespace :ecr do

  def symlink_or_mkdir_p(src, dest)
    if RUBY_PLATFORM =~ /mswin|mingw/
      FileUtils.mkdir_p(dest, :verbose => true)
    else
      FileUtils.mkdir_p(src, :verbose => true) unless File.exist?(src)
      FileUtils.rm_rf(dest, :verbose => true) if File.exist?(dest)
      FileUtils.ln_s(File.expand_path(src), File.expand_path(dest), :verbose => true)
    end
  end

  desc "setup cms_pages, cms_layouts and cms_files"
  task :cms => [
    :"ecr:cms:create_git_dir",
    :"ecr:cms:symlinks",
    :"ecr:cms:deploy",
    :"ecr:cms:publish",
    :"ecr:cms:copy_fixtures",
    :"ecr:cms::initialize"]

  namespace :cms do
    desc "create git dir"
    task :create_git_dir => :environment do
      git_dir = ENV['ECR_CMS_GIT_DIR'] || File.join(RAILS_ROOT, '..', 'cms_under_git')
      unless File.exist?(git_dir)
        puts "cd #{git_dir} && git init && git add . %% git commit -a -m 'initial commit'"
      end
    end

    desc "symlinks to cms_git_dir"
    task :symlinks => [:"ecr:cms:symlinks:ec_admin", :"ecr:cms:symlinks:ec_front"]

    namespace :symlinks do
      desc "symlinks to cms_git_dir for ec_admin"
      task :ec_admin => :environment do
        # プレビューを行う際に必要なファイルは本来ec_frontと同じものである必要がありますが、
        # 本番時はcapistranoでのデプロイ時にsymlinkが張られるので、以下の処理は必要ありません。
        git_dir = ENV['ECR_CMS_GIT_DIR'] || File.join(RAILS_ROOT, '..', 'cms_under_git')
        symlink_or_mkdir_p(
          File.join(git_dir, 'public_cms'),
          File.join(EcRider.config.ec_admin_root, 'public', 'cms'))
      end

      desc "symlinks to cms_git_dir for ec_front"
      task :ec_front => :environment do
        # ec_admin側のapp/views/cmsはCMSのバージョン管理下にはなく、
        # subversionで管理されます。
        # # ただし、プレビュー実行時に常に最新を取得して更新してから使われるので、
        # # 内容は全てsvn:externalsの対象になります。
        # よって、セットアップ時にはディレクトリを作成する必要はありません。
        git_dir = ENV['ECR_CMS_GIT_DIR'] || File.join(RAILS_ROOT, '..', 'cms_under_git')
        symlink_or_mkdir_p(
          File.join(git_dir, 'app_views', 'cms'),
          File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms'))
        symlink_or_mkdir_p(
          File.join(git_dir, 'public_cms'),
          File.join(EcRider.config.ec_front_root, 'public', 'cms'))
      end
    end

    # STEP1のrakeタスクです。データ移行後に削除します。
    desc "deploy cms_pages"
    task :deploy => :environment do
      CmsPage.update_all_to_be_deployed_or_deleted_without_status do |deploy_or_undeploy, cms_page|
        puts "#{deploy_or_undeploy.to_s} #{cms_page.deploy_path}"
      end
    end

    desc "publish cms_pages and cms_files. and upload to CloudFront"
    task :publish => :environment do
      return if CmsPublish.find(:first, :conditions => ["status_cd = ?", CmsPublish.status_id_by_key(:processing)])
      begin
        processing_publish = CmsPublish.new(:status_key => :processing)
        processing_publish.save!
        # ec_frontにビューとファイルを作成
        if published_projects = CmsProject.update_all_to_be_deployed_or_deleted
          CmsPublish.create!(:published_at => Time.now, :project_ids => published_projects.map(&:id), :status_key => :opened, :target_key => :production)

          # CloudFrontにec_front/public以下のファイルをアップロード
          files = ::EcRider::CloudFront.files_to_upload
          timestamp = Time.now.strftime(ENV['ECR_TIMESTAMP'] || ::EcRider::CloudFront::CLOUD_FRONT_VERSION_FORMAT)
          s3 = Object.new
          s3.extend(::EcRider::CloudFront)
          targets = files.map{|file| file[file.index("\/public\/")+8..-1]}
          s3.create_and_upload_to_s3(targets, :timestamp => timestamp)
        end
      rescue Exception
        CmsPublish.create(:published_at => Time.now, :status_key => :failed, :target_key => :production)
      end
      processing_publish.destroy
    end

    desc "copy cms_files in test/fixtures"
    task :copy_fixtures => :environment do
      dest_dir = File.join(EcRider.config.ec_front_root, *"public/cms".split('/'))
      Dir[File.join(dest_dir, '*')].each do |file|
        FileUtils.rm_rf(file, :verbose => true)
      end
      src_dir = File.join(RAILS_ROOT, *"test/fixtures/cms_files".split('/'))
      Dir[File.join(src_dir, '*')].each do |file|
        FileUtils.cp_r(file, File.join(EcRider.config.ec_front_root, *"public/cms".split('/')), :verbose => true)
      end
    end

    desc "initialize cms data"
    task :initialize => :environment do
      if EcRider.config.ec_admin
        ecr_page = Object.new
        ecr_page.extend(::EcRider::CmsSetup)

        puts "cleanup"
        ecr_page.cleanup_project
        puts "creating ecrider cms page data"
        ecr_page.create_ecrider_cms_pages
        puts "creating base project shop cms page data"
        ecr_page.create_shop_base_cms_pages
        puts "creating base project shop cms template data"
        ecr_page.create_shop_base_cms_templates
      else
        puts "***************************************************"
        puts "* This rake task is only executable for ec_admin. *"
        puts "***************************************************"
      end
    end


    def to_arg_attrs(*args)
      return '' if args.empty?
      raise "Illegal argument #{args.inspect}" if args.length > 1
      hash = args.first
      raise "Illegal argument #{hash.inspect}. It must be a Hash" unless hash.is_a?(Hash)
      hash.inject(""){|dest, (key, value)| dest << ' %s="%s"' % [key, value] }
    end

    # app/viewsの中の、rhtml/html.erb を XMLに変換したファイルを test/fixtures/ecrider_pagesに作成します。
    # ただし、元のrhtml/html.erbのうち、ecメソッドのみを使用しているファイルだけが対象です。
    desc "generate XMLs from erb/rhtml"
    task :generate_xml => :environment do
      erb_comment_regexp = /\<\%\s*\#(.+?)[\-]{0,1}\%\>/
      erb_tag_regexp = /\<\%[^\#][\=\-]{0,1}(.+?)[\-]{0,1}\%\>/
      erb_replace_regexp = /(\<\%[\=\-]{0,1})(.+?)[\-]{0,1}\%\>/
      ec_tag_regexp = /^ec\./
      ec_tag_close_regexp = Regexp.union(/^end$/, /^\}$/)
      ec_tag_or_close_regexp = Regexp.union(ec_tag_regexp, ec_tag_close_regexp)

      logger = ActiveRecord::Base.logger

      layout_escape_bases = [
        [ '<% content_for(:layout_top) do -%>',
          '<%= render(:partial => "cms/default_top") %>',
          '<% end -%>'],
        [ '<% content_for(:layout_bottom) do -%>',
          '<%= render(:partial => "cms/default_bottom") %>',
          '<% end -%>'],
        [ '<% content_for(:layout_left) do -%>',
          '<%= render(:partial => "cms/default_left") %>',
          '<% end -%>'],
        [ '<% content_for(:layout_right) do -%>',
          '<%= render(:partial => "cms/default_right") %>',
          '<% end -%>','<%= ec.prevent_double_subbmit -%>']
      ]
      layout_escape_regexps = layout_escape_bases.map do |layout_escape_base|
        Regexp.new(
          '%s\s*$?\s*%s\s*$?\s*%s' % layout_escape_base.map{|s| Regexp.escape(s)}
          )
      end

      ec_front_root = File.join(File.dirname(__FILE__), '../../../ec_front')
      ec_front_path = File.expand_path(ec_front_root)
      Dir.glob("#{ec_front_path}/app/views/**/*.{rhtml,erb}") do |filepath|
        logger.debug("\nprocessing #{filepath}...")
        file_content = IO.read(filepath)

        layout_escape_regexps.each do |layout_escape_regexp|
          file_content.gsub!(layout_escape_regexp, '')
        end
        file_content.sub!(/^[\s$]*/, '') # 最初の空白行を削除

        erb_tags = file_content.scan(erb_tag_regexp)

        unless erb_tags.flatten.all?{|erb_tag| erb_tag.strip =~ ec_tag_or_close_regexp}
          logger.debug "NON ec tag found."
          next
        end

        tag_stack = []
        file_content.gsub!(erb_comment_regexp) do
          "<!-- #{$1.strip} -->"
        end

        file_content.gsub!(erb_replace_regexp) do
          opener = $1
          content = $2
          tag = content.strip.gsub(/\s*(do|\{)\s*$/, '').gsub(/^ec\./, 'ec:')
          if tag =~ ec_tag_close_regexp
            "</#{tag_stack.pop}>"
          else
            tag, arg_opener, arg_str = *tag.scan(/^([^\s\(\)]+)(\s+|[\(]{0,1})([^\)]*)[\)]{0,1}\s*$/).flatten
            arg_attrs = instance_eval("to_arg_attrs(#{arg_str})")
            if content =~ /\s*(do|\{)\s*$/
              tag_stack << tag
              "<#{tag}#{arg_attrs}>"
            elsif opener == '<%='
              "<#{tag}#{arg_attrs} />"
            else
              raise "Something wrong in #{filepath}"
            end
          end
        end

        # 生成された内容がXMLとして正しくなかったら警告を出して、ファイルには出力しません。
        begin
          CmsXml.generate_erb(file_content)
        rescue REXML::ParseException => err
          if filepath == '/Users/akm/ec-one.com/xec/ec_front/app/views/cms/sitemap_th.html.erb'
            puts "-" * 100
            puts file_content
          end
          puts "-" * 100
          puts "Invalid XML from #{filepath}"
          puts err.message
          next
        rescue Exception => err
          puts "-" * 100
          puts "file: #{filepath}"
          puts "error: #{err.class.name}"
          puts err.message
          raise
        end


        dest_path = filepath.gsub(/app\/views/, 'test/fixtures/ecrider_pages').gsub(/html\.erb$|rhtml$/, 'xml')
        dest_dir = File.dirname(dest_path)
        FileUtils.mkdir_p(dest_dir, :verbose => true) unless File.exist?(dest_dir)
        overwriting = false
        if File.exist?(dest_path)
          old_dest_content = IO.read(dest_path)
          next if old_dest_content == file_content
          overwriting = true
        end
        open(dest_path, "w") do |f|
          f.write(file_content)
        end
        logger.debug "#{overwriting ? 'Overwriting' : ''} Generated #{dest_path} "
        puts "#{overwriting ? 'Overwriting' : ''} Generated #{dest_path} "
      end

    end
  end

  namespace :plugins do
    # ecr_plugins.ymlの例)
    # ---
    # - ecr_plugins/ecr_standard_plugin1
    # - ecr_plugins/ecr_standard_plugin1_admin

    app_root_path = File.expand_path(File.join(File.dirname(__FILE__), '../..'))
    this_app = File.basename(app_root_path)

    svn_repo = "svn://lions/opt/svnroot/xec"
    master_plugins_path = "/master/#{this_app}/vendor/plugins"
    ecr_plugins_config_path = File.expand_path(File.join(app_root_path, 'ecr_plugins.yml'))

    desc "svn propset svn:externals on vendor/plugins by using config/ecr_plugins.yml and #{master_plugins_path} to "
    task :svn_externals do
      master_plugins = `svn ls #{svn_repo}#{master_plugins_path}`.split(/$/).
        map{|s| s.strip}.select{|s| s =~ /\/$/}
      prop_value = master_plugins.
        map{|d| name = d.sub(/\/$/, '').strip; "^#{master_plugins_path}/#{name} #{name}" }.
        join("\n")
      if File.exist?(ecr_plugins_config_path)
        ecr_plugins = YAML.load_file(ecr_plugins_config_path) || {}
        prop_value << "\n"
        prop_value << ecr_plugins.values.flatten.map{|plugin_path| "../../#{plugin_path} #{File.basename(plugin_path)}"}
      end
      cmd = "svn propset svn:externals \"#{prop_value}\" vendor/plugins"
      # puts cmd
      system(cmd)
    end
    
    desc "delete symlinks and set symlinks by using ecr_plugins.yml"
    task :symlink do
      plugin_paths = Dir['vendor/plugins/ecr_*']
      symlink_paths = plugin_paths.select{|path| File.symlink?(path)}
      # symlinkのはずだけど、symlinkじゃなかったら警告
      undeleted_paths = plugin_paths - symlink_paths
      unless undeleted_paths.empty?
        puts "WARNING: these plugins aren't symlink: #{undeleted_paths}"
      end
      # 一旦すべてのECRプラグインのsymlinkを削除
      symlink_paths.each do |plugin_path|
        FileUtils.rm(plugin_path, :verbose => true)
      end
      # で、symlinkを貼り直す
      ecr_plugins = YAML.load_file(ecr_plugins_config_path) || {}
      ecr_plugins.values.flatten.each do |plugin_path|
        FileUtils.ln_sf(
          File.join(app_root_path, plugin_path),
          File.join(app_root_path, 'vendor/plugins', File.basename(plugin_path)),
          :verbose => true
          )
      end
    end

    desc "remove all ECR plugins and copy available ECR plugins"
    task :reset => :reset_soft

    # 不要なECRプラグインを削除して、必要でかつまだコピーされていないECRプラグインをコピー
    desc "remove unavailable ECR plugins and copy available uncopied ECR plugins"
    task :reset_soft => [:delete_unavailable, :copy]
    
    # ECRプラグインを一度すべて消して、必要なものをコピー
    desc "remove all ECR plugins and copy available ECR plugins"
    task :reset_hard => [:delete_all, :copy]

    # すべてのECRプラグインをvendor/pluginsから削除
    desc "delete all ECR plugins"
    task :delete_all do
      current_plugin_paths = Dir[File.join(app_root_path, 'vendor/plugins/ecr_*')]
      current_plugin_paths.each do |plugin_path|
        FileUtils.rm_rf(plugin_path, :verbose => true)
      end
    end

    # 不要なECRプラグインを削除
    desc "delete unavailable ECR plugins"
    task :delete_unavailable do
      current_plugin_paths = Dir[File.join(app_root_path, 'vendor/plugins/ecr_*')]
      current_plugin_names = current_plugin_paths.map{|path| File.basename(path)}
      available_plugin_paths = YAML.load_file(ecr_plugins_config_path) || {}
      available_plugin_paths = available_plugin_paths.values.flatten
      available_plugin_names = available_plugin_paths.map{|path| File.basename(path)}
      (current_plugin_names - available_plugin_names).each do |plugin_name|
        FileUtils.rm_rf(File.join(app_root_path, 'vendor/plugins', plugin_name), :verbose => true)
      end
    end

    # 有効でかつコピーされていないECRプラグインをコピー
    desc "copy available ECR plugins unless exists"
    task :copy do
      available_plugin_paths = YAML.load_file(ecr_plugins_config_path) || {}
      available_plugin_paths = available_plugin_paths.values.flatten
      available_plugin_paths.each do |plugin_path|
        plugin_name = File.basename(plugin_path)
        unless File.exist?(File.join(app_root_path, 'vendor/plugins', plugin_name))
          src_path = File.join(app_root_path, plugin_path)
          dest_path = File.join(app_root_path, 'vendor/plugins', File.basename(plugin_path))
          FileUtils.cp_r(src_path, dest_path, :verbose => true)
          Dir[File.join(dest_path, '**/.svn')].each do |svn_path|
            FileUtils.rm_rf(svn_path, :verbose => true)
          end
        end
      end
    end

    desc "Create tables for current ECR plugins"
    task :migrate_all do
      current_plugin_paths = Dir[File.join(app_root_path, 'vendor/plugins/ecr_*')]
      current_plugin_names = current_plugin_paths.map{|path| File.basename(path)}
      current_plugin_names.each do |plugin_name|
        next unless Rake::Task.task_defined?("#{plugin_name}:db:migrate")
        Rake::Task["#{plugin_name}:db:migrate"].invoke
      end
    end

  end


  namespace :development do

    desc "make symlink from ec_front to ec_admin"
    task :symlink do
      ec_front_root = File.join(File.dirname(__FILE__), '../..')
      ec_front_path = File.expand_path(ec_front_root)
      ec_admin_path = File.expand_path(File.join(ec_front_path, '../ec_admin'))
      #
      cms_under_git_path = File.expand_path(File.join(ec_front_path, '../cms_under_git'))
      raise "ec_front doesn't exist!" unless File.exist?(ec_front_path)
      raise "ec_admin doesn't exist!" unless File.exist?(ec_admin_path)
      raise "cms_under_git doesn't exist!" unless File.exist?(cms_under_git_path)
      symlinks = [
        'app/helpers/ec_rider',
        'app/models',
        # 'app/views/error_mailer',
        'config/deploy',
        # 'config/initializers', # ec_adminとec_frontで別の設定をすることがあるので分けました。
        'config/locales',
        'db',
        'ecr_plugins',
        'lib/ec_rider',
        # 'public/cms',
        # 'public/themes',
        'spec/helpers/ec_rider',
        'spec/models',
        'test/fixtures',
        # 'test/functional/ec_rider',
        # 'test/unit',
        # プラグインは数が多すぎてデプロイに時間がかかりすぎるので、svn:externalsを使わないようにしたので、
        # symlinkも必要なくなりました。
        # 'vendor/plugins/actionmailserver',
        # 'vendor/plugins/active_form',
        # 'vendor/plugins/acts_as_authenticated',
        # 'vendor/plugins/acts_as_paranoid',
        # 'vendor/plugins/ar_fixtures',
        # 'vendor/plugins/attachment_fu',
        # 'vendor/plugins/deep_cloning',
        # 'vendor/plugins/dir_fixtures',
        # 'vendor/plugins/dynamic_session_exp',
        # 'vendor/plugins/exception_notification',
        'vendor/plugins/i18n_erb_backend', # これはecrider用にカスタマイズしているので、他のプラグインとは別扱いです。
        # 'vendor/plugins/jp_address',
        # 'vendor/plugins/rename_model',
        # 'vendor/plugins/rspec',
        # 'vendor/plugins/rspec-rails',
        # 'vendor/plugins/rspec_on_rails',
        # 'vendor/plugins/schema_comments',
        # 'vendor/plugins/selectable_attr',
        # 'vendor/plugins/selenium-on-rails',
        # 'vendor/plugins/ssl_requirement'
      ]

      symlinks.each do |symlink|
        path_in_front = File.join(ec_front_path, symlink)
        if FileTest.symlink?(path_in_front)
          FileUtils.rm_f(path_in_front, :verbose => true)
        end
        FileUtils.ln_sf(
          File.join(ec_admin_path, symlink),
          path_in_front, :verbose => true)
      end
    end

  end

end
