# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')


describe MemberController do
  fixtures :users, :user_addresses, :user_emails, :shop_settings,
    :orders, :order_shippings, :order_items, :mail_templates, :product_skus, :countries

  # fixturesで作成されるordersメソッドがOrderオブジェクトを返すので、
  # Orderを返すようにメソッドを上書きします。
  # クラス名とテーブル名の対応が取れるまでの一時的な措置です。
  def orders(fixture_name)
    Order.find(fixture_name.to_s.gsub(/[^\d]/, '').to_i)
  end


  include AuthenticatedTestHelper

  before(:each) do
    # session[:user] = users(:user_01002).id
    login_as(:users_00003)
  end

  describe 'POST history' do
    it 'should be success' do
      controller.send(:logged_in?).should == true
      get 'history'
      response.should be_success
      controller.instance_variable_get(:@orders).length.should == 2
    end
    
    it '決済前の注文は出さない' do
      controller.send(:logged_in?).should == true
      order_shipping = OrderShipping.find(2)
      order_shipping.status_div_key = :before_settlement
      order_shipping.save!
      get 'history'
      response.should be_success
      controller.instance_variable_get(:@orders).length.should == 1
    end
    
    it 'エラーの注文は出さない' do
      controller.send(:logged_in?).should == true
      order_shipping = OrderShipping.find(2)
      order_shipping.status_div_key = :stock_error
      order_shipping.save!
      get 'history'
      response.should be_success
      controller.instance_variable_get(:@orders).length.should == 1
    end

    it 'エラーの場合' do
      controller.send(:logged_in?).should == true
      flash[:user_order_error] = true
      get 'history'
      response.should be_success
      controller.instance_variable_get(:@user_order_error).should be_true
    end
  end

  describe 'GET history_order_detail' do
    it 'should get order history data' do
      get 'history_order_detail', :id => 2
      response.should be_success
    end

    it 'order history data is duplicate' do
      Order.stub!(:find).and_return([orders(:orders_00002), orders(:orders_00002)])
      get 'history_order_detail', :id => 2
      response.should redirect_to(:controller => 'cms', :action => 'error_404', :lang => 'ja')
    end

    it 'not match user_id' do
      order = orders(:orders_00001)
      order.class.should == Order
      order.user = users(:users_00002)
      Order.stub!(:find).and_return([order])
      request.session[:user] = users(:users_00002)
      get 'history_order_detail', :id => 2
      response.should redirect_to(:controller => 'member', :action => 'history', :lang => 'ja')
      flash[:user_order_error].should be_true
    end
  end

  describe 'POST confirm_cancel' do
    def mock_order(stubs={})
      @mock_order ||= mock_model(Order, stubs)
    end

    it "キャンセル依頼確認" do
      Order.stub!(:find).with("37").and_return(mock_order)
      get :confirm_cancel, :id => "37"
      assigns[:order].should equal(mock_order)
    end
  end

  describe 'POST request_cancel' do
    it 'キャンセル依頼' do
      request.stub!(:xml_http_request?).and_return(true)
      request.session[:user] = Order.find_by_id(2).user
      post 'request_cancel', :id => 2
      response.should redirect_to(:controller => 'member', :action => 'history', :lang => 'ja')
      Order.find(2).order_shippings[0].status_div_key.should == :cancel_request
      ActionMailer::Base.deliveries.size.should == 1
      ActionMailer::Base.deliveries[0].subject.should == 'cancel pc!'
    end
    
    it 'zh_CN' do
      request.stub!(:xml_http_request?).and_return(true)
      I18n.stub!(:locale).and_return("zh_CN")
      session[:user] = Order.find_by_id(2).user

      post 'request_cancel', :id => 2
      response.should redirect_to(:controller => 'member', :action => 'history', :lang => 'ja')
      Order.find(2).order_shippings[0].status_div_key.should == :cancel_request
      ActionMailer::Base.deliveries.size.should == 1
      ActionMailer::Base.deliveries[0].subject.should == 'cancel pc! sample  衷心感谢您对我们的支持参加'
    end

    it "#211 When I canceled the order for plural articles for one order." do
      pending "STEP2では複数配送先に対応しません。"
      request.stub!(:xml_http_request?).and_return(true)
      # ひとつの注文に対して複数の商品の注文をキャンセルした場合
      session[:user] = users(:users_00006).id
      post 'request_cancel', :id => 5
      controller.instance_variable_get(:@orders)[0].order_shippings.size.should == 2
      controller.instance_variable_get(:@orders)[0].order_shippings[0].status_div_key.should == :cancel_request
      controller.instance_variable_get(:@orders)[0].order_shippings[1].status_div_key.should == :cancel_request
      ActionMailer::Base.deliveries.size.should == 1
      ActionMailer::Base.deliveries[0].subject.should == 'cancel pc!'
    end

    it "別ユーザの注文を参照しようとしたらエラーにする" do
      request.stub!(:xml_http_request?).and_return(true)
      session[:user] = users(:users_00006).id
      post 'request_cancel', :id => 100
      response.should redirect_to(:controller => 'member', :action => 'history', :lang => 'ja')
    end
  end

  describe 'GET show_addresses' do
    it 'not have data in flash' do
      get 'show_addresses'
      response.should be_success
      controller.instance_variable_get(:@user_address).name_last.should == nil
      controller.instance_variable_get(:@shipping_addresses).size.should == 1
    end

    it 'has data in flash' do
      flash[:address] = UserAddress.new(:name_last => '鈴木')
      session[:change_address] = UserAddress.new(:name_last => '佐藤')
      get 'show_addresses'
      response.should be_success
      controller.instance_variable_get(:@user_address).name_last.should == "鈴木"
      controller.instance_variable_get(:@shipping_addresses).size.should == 1
    end
  end

  describe 'POST confirm_address' do
    before do
      session[:change_address] = UserAddress.find(1011)
    end

    it 'should be success' do
      post('confirm_address', :user_address => {
          :name_last => '買物', :name_first => 'あとむ',
          :kana_name_last => 'カイモノ', :kana_name_first => 'アトム',
          :tel_num => '000-1111-2222', :zip_code => '0001111',
          :address_1 => '愛知県', :address_2 => '名古屋市', :address_3 => '1-1', :country_code => 'JP'
        }
      )
      response.should be_success
      controller.instance_variable_get(:@user_address).name_last.should == "買物"
    end

    it 'raise validation error' do
      post('confirm_address', :user_address => {
          :name_last => '買物', :name_first => '',
          :kana_name_last => 'カイモノ', :kana_name_first => 'アトム',
          :tel_num => '000-1111-2222', :zip_code => '0001111',
          :address_1 => '愛知県', :address_2 => '名古屋市', :address_3 => '1-1', :country_code => 'JP'
        }
      )
      response.should redirect_to(:controller => 'member', :action => 'show_addresses', :lang => 'ja')
      flash[:error][0].should == [["name_first", "を入力してください。"]]
    end
  end

  describe 'GET change_address' do
    it 'not have data in flash' do
      get 'change_address', :id => 1002
      response.should be_success
      controller.instance_variable_get(:@user_address).name_last.should == "山田"
      controller.instance_variable_get(:@index).should == nil
      session[:change_address].name_last.should == "山田"
    end

    it 'has data in flash' do
      flash[:user_address] = UserAddress.new(:name_last => '鈴木')
      flash[:index] = 1
      get 'change_address', :id => 1002
      response.should be_success
      controller.instance_variable_get(:@user_address).name_last.should == "鈴木"
      session[:change_address].should == nil
    end

    it '別ユーザの配送先にアクセスしようとしたら配送先一覧に遷移する' do
      session[:user] = users(:users_00006).id
      get 'change_address', :id => 1002
      response.should redirect_to(:controller => 'member', :action => 'show_addresses', :lang => 'ja')
    end
  end

  describe 'POST confirm_modify_change' do
    before do
      session[:change_address] = UserAddress.find(1011)
    end

    it 'should be success' do
      post('confirm_modify_change', :user_address => {
          :name_last => '石腕', :name_first => 'あとむ',
          :kana_name_last => 'カイモノ', :kana_name_first => 'アトム',
          :tel_num => '000-1111-2222', :zip_code => '0001111',
          :address_1 => '愛知県', :address_2 => '名古屋市', :address_3 => '1-1', :country_code => 'JP'
        },
        :index => 0
      )
      response.should be_success
      controller.instance_variable_get(:@user_address).name_last.should == "石腕"
    end

    it 'raise validation error' do
      post('confirm_modify_change', :user_address => {
          :name_last => '石腕', :name_first => '',
          :kana_name_last => 'カイモノ', :kana_name_first => 'アトム',
          :tel_num => '000-1111-2222', :zip_code => '0001111',
          :address_1 => '愛知県', :address_2 => '名古屋市', :address_3 => '1-1', :country_code => 'JP'
        },
        :index => 0
      )
      response.should redirect_to(:controller => 'member', :action => 'change_address', :id => 1011, :lang => 'ja')
      flash[:error][0].should == [["name_first", "を入力してください。"]]
    end
  end

  describe 'DELETE delete_address' do
    it 'should be success' do
      user_address = {
        :name_last => '石腕', :name_first => '',
        :kana_name_last => 'カイモノ', :kana_name_first => 'アトム',
        :tel_num => '000-1111-2222', :zip_code => '0001111',
        :address_1 => '愛知県', :address_2 => '名古屋市', :address_3 => '1-1', :country_code => 'JP'
      }
      user_address[:id] = 1002
      delete 'delete_address', :user_address => user_address
      response.should redirect_to(:controller => 'member', :action => 'show_addresses', :lang => 'ja')
    end
    it 'エラーの場合show_addressに遷移' do
      delete 'delete_address', :user_address => {:id => 10002}
      response.should redirect_to(:controller => 'member', :action => 'show_addresses', :lang => 'ja')
      flash[:address_error].should be_true
    end
  end

  describe 'print receipt' do
    it 'should be print receipt' do
      get 'print_receipt', :id => 2
      response.should be_success
    end
    it 'エラーの場合historyに遷移' do
      get :print_receipt, :id => 4
      response.should redirect_to(:controller => 'member', :action => 'history', :lang => 'ja')
      flash[:user_order_error].should be_true
    end
  end

  describe "Get 'index'" do
    it "#205  should be success" do
      get(:index)
      response.should be_success
    end
  end

end
