# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe EcRider::CloudFrontHelper do # , :type => :helper do
  include EcRider::CloudFrontHelper

  class AttrMatcher
    def initialize(attr, expected)
      @expected = '%s="%s"' % [attr, expected]
    end
    def matches?(target)
      @target = target
      @target.to_s.include?(@expected)
    end
    def failure_message
      "expected #{@target.inspect} has #{@expected.inspect}"
    end
    def negative_failure_message
      "expected #{@target.inspect} doesn't have #{@expected.inspect}"
    end
  end

  module SrcMatchers
    class HasSrc < AttrMatcher
      def initialize(expected)
        super('src', expected)
      end
    end

    def has_src(expected)
      HasSrc.new(expected)
    end
  end

  module HrefMatchers
    class HasHref < AttrMatcher
      def initialize(expected)
        super('href', expected)
      end
    end

    def has_href(expected)
      HasHref.new(expected)
    end
  end

  before(:each) do
    EcRider.config.stub!(:cloud_front_url).and_return("http://0123456789.cloudfront.net")
    @public_dir_for_cloud_front = File.expand_path(File.join(EcRider.config.ec_front_root, 'public'))
  end

  def cp_in_front_public(src, dest)
    FileUtils.mkdir_p(File.join(@public_dir_for_cloud_front, File.dirname(dest)), :verbose => false)
    FileUtils.cp_r(
      File.join(@public_dir_for_cloud_front, src),
      File.join(@public_dir_for_cloud_front, dest), :verbose => false)
  end

  def cp_public_files(filepath, alt_name = nil)
    FileUtils.mkdir_p(File.join(@public_dir_for_cloud_front, File.dirname(filepath)), :verbose => false)
    FileUtils.mkdir_p(File.join(@public_dir_for_cloud_front, File.dirname(alt_name)), :verbose => false) if alt_name
    FileUtils.cp_r(
      File.join(EcRider.config.ec_front_root, 'public', filepath),
      File.join(@public_dir_for_cloud_front, alt_name ? alt_name : filepath),
      :verbose => false)
  end

  describe "image asserts by " do
    describe "image_path_for_cloud_front" do
      it "should include CloudFront url" do
        DirFixtures["test/fixtures/ec_front/publics"].process('proj1_only') do
          File.expand_path(File.join(EcRider.config.ec_front_root, 'public'))

          image_path('/themes/ec_ver2/images/icon_title.gif').should =~ %r{/themes/ec_ver2/images/icon_title.gif(?:\?\d+)?}
          image_path('themes/ec_ver2/images/icon_title.gif').should == '/images/themes/ec_ver2/images/icon_title.gif'
          image_path('/logo.gif').should == '/logo.gif'

          pending # 画像ファイル名の後ろに「?日時情報」が付加されたものが取得される
  #        image_path('logo.gif').should == '/images/logo.gif'

          # cp_public_files('themes/ec_ver2/images/icon_title.gif')
          cp_in_front_public('themes/ec_ver2/images/icon_title.gif', 'logo.gif')
          image_path_for_cloud_front('/themes/ec_ver2/images/icon_title.gif').should == '/themes/ec_ver2/images/icon_title.gif'
          image_path_for_cloud_front('/logo.gif').should == '/logo.gif'

          cp_in_front_public('themes/ec_ver2/images/icon_title.gif', 'themes/ec_ver2/images/icon_title-20090309121529.gif')
          cp_in_front_public('themes/ec_ver2/images/icon_title.gif', 'logo-20090309121529.gif')
          image_path_for_cloud_front('logo.gif').should == '/images/logo.gif'
          image_path_for_cloud_front('/logo.gif').should == 'http://0123456789.cloudfront.net/logo-20090309121529.gif'
          image_path_for_cloud_front('themes/ec_ver2/images/icon_title.gif').should == '/images/themes/ec_ver2/images/icon_title.gif'
          image_path_for_cloud_front('/themes/ec_ver2/images/icon_title.gif').should == 'http://0123456789.cloudfront.net/themes/ec_ver2/images/icon_title-20090309121529.gif'

          cp_in_front_public('themes/ec_ver2/images/icon_title.gif', 'images/themes/ec_ver2/images/icon_title-20090309121529.gif')
          cp_in_front_public('themes/ec_ver2/images/icon_title.gif', 'images/logo-20090309121529.gif')
          image_path_for_cloud_front('logo.gif').should == 'http://0123456789.cloudfront.net/images/logo-20090309121529.gif'
          image_path_for_cloud_front('/logo.gif').should == 'http://0123456789.cloudfront.net/logo-20090309121529.gif'
          image_path_for_cloud_front('themes/ec_ver2/images/icon_title.gif').should == 'http://0123456789.cloudfront.net/images/themes/ec_ver2/images/icon_title-20090309121529.gif'
          image_path_for_cloud_front('/themes/ec_ver2/images/icon_title.gif').should == 'http://0123456789.cloudfront.net/themes/ec_ver2/images/icon_title-20090309121529.gif'
        end
      end
    end

    describe "image_tag_for_cloud_front" do
      it "should include CloudFront url" do
        DirFixtures["test/fixtures/ec_front/publics"].process('proj1_only') do
          # cp_public_files('themes/ec_ver2/images/icon_title.gif')
          image_tag_for_cloud_front('/themes/ec_ver2/images/icon_title.gif').should =~ %r{<img alt="Icon_title" src="(?:http://0123456789.cloudfront.net)?/themes/ec_ver2/images/icon_title(\-\d+?)?.gif(?:\?\d+)?" />}

          cp_in_front_public('themes/ec_ver2/images/icon_title.gif', 'themes/ec_ver2/images/icon_title-20090309121529.gif')

          image_tag_for_cloud_front('/themes/ec_ver2/images/icon_title.gif').should == '<img alt="Icon_title" src="http://0123456789.cloudfront.net/themes/ec_ver2/images/icon_title-20090309121529.gif" />'
          image_tag_for_cloud_front('themes/ec_ver2/images/icon_title.gif').should == '<img alt="Icon_title" src="/images/themes/ec_ver2/images/icon_title.gif" />'
        end
      end
    end
  end


  describe "javascript assets by" do

    describe "javascript_tag_for_cloud_front" do
      it "should include CloudFront url" do
        DirFixtures["test/fixtures/ec_front/publics"].process('proj1_only') do
          javascript_path('/prototype.js').should == '/prototype.js'
          javascript_path('prototype.js').should =~ %r{/javascripts/prototype.js(?:\?\d+)?}

          cp_in_front_public('javascripts/prototype.js', 'prototype.js')
          javascript_path_for_cloud_front('prototype.js').should == '/javascripts/prototype.js'
          javascript_path_for_cloud_front('/prototype.js').should == '/prototype.js'
          javascript_path_for_cloud_front('javascripts/prototype.js').should == '/javascripts/javascripts/prototype.js'
          javascript_path_for_cloud_front('/javascripts/prototype.js').should == '/javascripts/prototype.js'

          cp_in_front_public('javascripts/prototype.js', 'prototype-20090309121529.js')
          javascript_path_for_cloud_front('prototype.js').should == '/javascripts/prototype.js'
          javascript_path_for_cloud_front('/prototype.js').should == 'http://0123456789.cloudfront.net/prototype-20090309121529.js'
          javascript_path_for_cloud_front('javascripts/prototype.js').should == '/javascripts/javascripts/prototype.js'
          javascript_path_for_cloud_front('/javascripts/prototype.js').should == '/javascripts/prototype.js'

          cp_in_front_public('javascripts/prototype.js', 'javascripts/prototype-20090309121529.js')
          javascript_path_for_cloud_front('prototype.js').should == 'http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js'
          javascript_path_for_cloud_front('/prototype.js').should == 'http://0123456789.cloudfront.net/prototype-20090309121529.js'
          javascript_path_for_cloud_front('javascripts/prototype.js').should == '/javascripts/javascripts/prototype.js'
          javascript_path_for_cloud_front('/javascripts/prototype.js').should == 'http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js'
        end
      end
    end

    describe "javascript_path_for_cloud_front" do

      include SrcMatchers

      it "should include CloudFront url" do
        DirFixtures["test/fixtures/ec_front/publics"].process('proj1_only') do
  #        puts javascript_include_tag_for_cloud_front('/prototype.js')

          javascript_include_tag_for_cloud_front('/prototype.js').should has_src('/prototype.js')
          javascript_include_tag_for_cloud_front('prototype.js').should =~ %r{src="/javascripts/prototype.js(?:\?\d+)?"}

          cp_in_front_public('javascripts/prototype.js', 'prototype.js')
          javascript_include_tag_for_cloud_front('prototype.js').should =~ %r{src="/javascripts/prototype.js(?:\?\d+)?"}
          javascript_include_tag_for_cloud_front('/prototype.js').should has_src('/prototype.js')
          javascript_include_tag_for_cloud_front('javascripts/prototype.js').should has_src('/javascripts/javascripts/prototype.js')
          javascript_include_tag_for_cloud_front('/javascripts/prototype.js').should =~ %r{src="/javascripts/prototype.js(?:\?\d+)?"}

          cp_in_front_public('javascripts/prototype.js', 'prototype-20090309121529.js')
          javascript_include_tag_for_cloud_front('prototype.js').should =~ %r{src="/javascripts/prototype.js(?:\?\d+)?"}
          javascript_include_tag_for_cloud_front('/prototype.js').should has_src('http://0123456789.cloudfront.net/prototype-20090309121529.js')
          javascript_include_tag_for_cloud_front('javascripts/prototype.js').should has_src('/javascripts/javascripts/prototype.js')
          javascript_include_tag_for_cloud_front('/javascripts/prototype.js').should =~ %r{src="/javascripts/prototype.js(?:\?\d+)?"}

          cp_in_front_public('javascripts/prototype.js', 'javascripts/prototype-20090309121529.js')
          javascript_include_tag_for_cloud_front('prototype.js').should has_src('http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js')
          javascript_include_tag_for_cloud_front('/prototype.js').should has_src('http://0123456789.cloudfront.net/prototype-20090309121529.js')
          javascript_include_tag_for_cloud_front('javascripts/prototype.js').should has_src('/javascripts/javascripts/prototype.js')
          javascript_include_tag_for_cloud_front('/javascripts/prototype.js').should has_src('http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js')
        end
      end

      it "should render scripts tags for :defaults" do
        DirFixtures["test/fixtures/ec_front/publics"].process('proj1_only') do
          script_tags = javascript_include_tag_for_cloud_front(:defaults)
          ActionView::Helpers::AssetTagHelper::JAVASCRIPT_DEFAULT_SOURCES.each do |src|
            script_tags.should =~ %r{/javascripts/#{src}.js(?:\?\d+)?}
          end

          ActionView::Helpers::AssetTagHelper::JAVASCRIPT_DEFAULT_SOURCES.each do |src|
            cp_in_front_public("javascripts/#{src}.js", "javascripts/#{src}-20090309121529.js")
          end
          script_tags = javascript_include_tag_for_cloud_front(:defaults)
          ActionView::Helpers::AssetTagHelper::JAVASCRIPT_DEFAULT_SOURCES.each do |src|
            script_tags.should has_src("http://0123456789.cloudfront.net/javascripts/#{src}-20090309121529.js")
          end
        end
      end
    end
  end


  describe "stylesheet assets by" do
    describe "stylesheet_path_for_cloud_front" do
      it "should include CloudFront url" do
        DirFixtures["test/fixtures/ec_front/publics"].process('proj1_only') do
          stylesheet_path('/themes/ec_ver2/style.css').should =~ %r{/themes/ec_ver2/style.css(?:\?\d+)?}
          stylesheet_path('themes/ec_ver2/style.css').should == '/stylesheets/themes/ec_ver2/style.css'

          # cp_public_files('themes/ec_ver2/style.css')

          # stylesheet_path_for_cloud_front('/themes/ec_ver2/style.css').should == '/themes/ec_ver2/style.css'
          stylesheet_path_for_cloud_front('/themes/ec_ver2/style.css').should =~ %r{(?:http://0123456789.cloudfront.net)?/themes/ec_ver2/style(?:\-\d+?)?.css}

          cp_in_front_public('themes/ec_ver2/style.css', 'themes/ec_ver2/style-20090309121529.css')
          stylesheet_path_for_cloud_front('themes/ec_ver2/style.css').should == '/stylesheets/themes/ec_ver2/style.css'
          stylesheet_path_for_cloud_front('/themes/ec_ver2/style.css').should == 'http://0123456789.cloudfront.net/themes/ec_ver2/style-20090309121529.css'

          cp_in_front_public('themes/ec_ver2/style.css', 'stylesheets/themes/ec_ver2/style-20090309121529.css')
          stylesheet_path_for_cloud_front('themes/ec_ver2/style.css').should == 'http://0123456789.cloudfront.net/stylesheets/themes/ec_ver2/style-20090309121529.css'
          stylesheet_path_for_cloud_front('/themes/ec_ver2/style.css').should == 'http://0123456789.cloudfront.net/themes/ec_ver2/style-20090309121529.css'
        end
      end
    end

    describe "stylesheet_link_tag_for_cloud_front" do
      include HrefMatchers

      it "should include CloudFront url" do
        DirFixtures["test/fixtures/ec_front/publics"].process('proj1_only')  do
          # cp_public_files('themes/ec_ver2/style.css')
          stylesheet_link_tag_for_cloud_front('/themes/ec_ver2/style.css').should =~ %r{href\=\"(?:http://0123456789.cloudfront.net)?/themes/ec_ver2/style(\-\d+?)?.css(?:\?\d+)?\"}

          cp_in_front_public('themes/ec_ver2/style.css', 'themes/ec_ver2/style-20090309121529.css')

          stylesheet_link_tag_for_cloud_front('/themes/ec_ver2/style.css').should has_href('http://0123456789.cloudfront.net/themes/ec_ver2/style-20090309121529.css')
          stylesheet_link_tag_for_cloud_front('themes/ec_ver2/style.css').should has_href('/stylesheets/themes/ec_ver2/style.css')
        end
      end

    end
  end
end
