# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CampaignCondition do
  fixtures :kbn_codes, :kbn_code_names, :user_ranks, :countries

  before(:each) do
    @valid_attributes = {
      :campaign_id => 1,
      :no => 1,
      :type_cd => "ty",
      :operator_cd => "op",
      :value => "value for value"
    }
  end

  it "should create a new instance given valid attributes" do
    CampaignCondition.create!(@valid_attributes)
  end

  describe "each type_entry" do
    CampaignCondition.type_entries.each do |entry|
      describe "#{entry.id}: #{entry.key.inspect}(#{entry.name})" do
        it "should have :func" do
          entry[:func].should_not be_nil
        end
      end
    end
  end


  describe "build_script" do
    # 不正な条件
    describe "invalid attributes" do
      it "raise_error" do
        cond = CampaignCondition.new
        lambda{ cond.build_script }.should raise_error(ActiveRecord::RecordInvalid)
        cond.errors[:type_cd].should_not be_empty
        cond.errors[:operator_cd].should_not be_empty

        cond.type_key = :user_birthday
        lambda{ cond.build_script }.should raise_error(ActiveRecord::RecordInvalid)
        cond.errors[:type_cd].should be_nil
        cond.errors[:operator_cd].should_not be_empty

        cond.type_cd = nil
        cond.operator_key = :eql
        lambda{ cond.build_script }.should raise_error(ActiveRecord::RecordInvalid)
        cond.errors[:type_cd].should_not be_empty
        cond.errors[:operator_cd].should be_nil
      end

    end
  end

  describe "build_script and match?" do

    # 性別
    describe "for user_gender" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_gender, :operator_key => :eql, :value => User.sex_id_by_key(:female))
        cond.build_script.should == "user.sex == '2'"
        cond.match?(:user => User.new(:sex_key => :male)).should == false
        cond.match?(:user => User.new(:sex_key => :female)).should == true
      end

      it "by not" do
        cond = CampaignCondition.new(:type_key => :user_gender, :operator_key => :not, :value => User.sex_id_by_key(:male))
        cond.build_script.should == "user.sex != '1'"
        cond.match?(:user => User.new(:sex_key => :male)).should == false
        cond.match?(:user => User.new(:sex_key => :female)).should == true
      end
    end

    # 年齢
    describe "for user_age" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_age, :operator_key => :eql, :value => '36')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.age(campaign_attached_at) == CampaignCondition.parse_age('36')"
        timenow = Time.parse("2009/07/04 00:00:00")
        cond.match?(:user => User.new(:birth_year => 1973, :birth_month => 7, :birth_day => 3), :campaign_attached_at => timenow).should == true
        cond.match?(:user => User.new(:birth_year => 1973, :birth_month => 7, :birth_day => 4), :campaign_attached_at => timenow).should == true
        cond.match?(:user => User.new(:birth_year => 1973, :birth_month => 7, :birth_day => 5), :campaign_attached_at => timenow).should == false

      end

      it "by gte" do
        cond = CampaignCondition.new(:type_key => :user_age, :operator_key => :gte, :value => '65')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.age(campaign_attached_at) >= CampaignCondition.parse_age('65')"
        timenow = Time.parse("2010/03/31 00:00:00")
        cond.match?(:user => User.new(:birth_year => 1945, :birth_month => 3, :birth_day => 30), :campaign_attached_at => timenow).should == true
        cond.match?(:user => User.new(:birth_year => 1945, :birth_month => 3, :birth_day => 31), :campaign_attached_at => timenow).should == true
        cond.match?(:user => User.new(:birth_year => 1945, :birth_month => 4, :birth_day => 1), :campaign_attached_at => timenow).should == false
      end
    end

    # 誕生日(年月日)
    describe "for user_birthday" do
      it "by gt" do
        cond = CampaignCondition.new(:type_key => :user_birthday, :operator_key => :gt, :value => '1990/4/1')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.birthday > CampaignCondition.parse_date('1990/4/1')"
        cond.match?(:user => User.new(:birth_year => 1990, :birth_month => 3, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1990, :birth_month => 4, :birth_day => 1)).should == false
        cond.match?(:user => User.new(:birth_year => 1990, :birth_month => 4, :birth_day => 2)).should == true
      end
      
      it "by lte" do
        cond = CampaignCondition.new(:type_key => :user_birthday, :operator_key => :lte, :value => '1950/01/01')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.birthday <= CampaignCondition.parse_date('1950/01/01')"
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 12, :birth_day => 31)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 2)).should == false
      end
      
      it "by eql with today" do
        cond = CampaignCondition.new(:type_key => :user_birthday, :operator_key => :eql, :value => 'today')
        cond.campaign = Campaign.new()
        cond.campaign.stub!(:attached_at).and_return(Time.parse('2010/01/01 00:00:00'))
        cond.build_script.should == "user.birthday == CampaignCondition.parse_date('2010/01/01')"
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 1)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 2)).should == false
        cond.match?(:user => User.new(:birth_year => 2009, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 2010, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 2010, :birth_month => 1, :birth_day => 2)).should == false
      end
    end

    # 誕生日(月日)
    describe "for user_birthday_of_year" do
      it "by eql with today_in_year" do
        cond = CampaignCondition.new(:type_key => :user_birthday_of_year, :operator_key => :eql, :value => '3/4')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.birthday_of_year == CampaignCondition.parse_date_of_year('3/4')"
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 3, :birth_day => 3)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 3, :birth_day => 4)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 3, :birth_day => 5)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 3, :birth_day => 3)).should == false
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 3, :birth_day => 4)).should == true
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 3, :birth_day => 5)).should == false
      end

      it "by eql with today_in_year" do
        cond = CampaignCondition.new(:type_key => :user_birthday_of_year, :operator_key => :eql, :value => 'today')
        cond.campaign = Campaign.new()
        Time.stub!(:now).and_return(Time.parse('2010/01/01 00:00:00'))
        cond.build_script.should == "user.birthday_of_year == CampaignCondition.parse_date_of_year('01/01')"
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 2)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 1, :birth_day => 2)).should == false
      end

    end

    # 誕生日(日)
    describe "for user_birth_day" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_birth_day, :operator_key => :eql, :value => '1')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.birth_day.to_i == CampaignCondition.parse_day_of_month('1')"
        cond.match?(:user => User.new(:birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_day => 2)).should == false
      end

      it "by eql with today" do
        cond = CampaignCondition.new(:type_key => :user_birth_day, :operator_key => :eql, :value => 'today')
        cond.campaign = Campaign.new()
        cond.campaign.stub!(:attached_at).and_return(Date.parse('2010/4/2'))
        cond.build_script.should == "user.birth_day.to_i == CampaignCondition.parse_day_of_month('02')"
        cond.match?(:user => User.new(:birth_day => 1)).should == false
        cond.match?(:user => User.new(:birth_day => 2)).should == true
        cond.match?(:user => User.new(:birth_day => 3)).should == false
      end
    end

    # 誕生日(月)
    describe "for user_birth_month" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_birth_month, :operator_key => :eql, :value => '4')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.birth_month.to_i == CampaignCondition.parse_month('4')"
        cond.match?(:user => User.new(:birth_month => 3)).should == false
        cond.match?(:user => User.new(:birth_month => 4)).should == true
        cond.match?(:user => User.new(:birth_month => 5)).should == false
      end

      it "by eql with today" do
        cond = CampaignCondition.new(:type_key => :user_birth_month, :operator_key => :eql, :value => 'today')
        cond.campaign = Campaign.new()
        Time.stub!(:now).and_return(Time.parse('2010/4/2 00:00:00'))
        cond.build_script.should == "user.birth_month.to_i == CampaignCondition.parse_month('04')"
        cond.match?(:user => User.new(:birth_month => 3)).should == false
        cond.match?(:user => User.new(:birth_month => 4)).should == true
        cond.match?(:user => User.new(:birth_month => 5)).should == false
      end
    end

    # 誕生日(年)
    describe "for user_birth_year" do
      it "by gte" do
        cond = CampaignCondition.new(:type_key => :user_birth_year, :operator_key => :gte, :value => '1970')
        cond.campaign = Campaign.new()
        cond.build_script.should == "user.birth_year.to_i >= CampaignCondition.parse_year('1970')"
        cond.match?(:user => User.new(:birth_year => 1969)).should == false
        cond.match?(:user => User.new(:birth_year => 1970)).should == true
        cond.match?(:user => User.new(:birth_year => 1971)).should == true
      end

      it "by lte with today" do
        cond = CampaignCondition.new(:type_key => :user_birth_year, :operator_key => :lte, :value => 'today')
        cond.campaign = Campaign.new()
        Time.stub!(:now).and_return(Time.parse('2010/4/2 00:00:00'))
        cond.build_script.should == "user.birth_year.to_i <= CampaignCondition.parse_year('2010')"
        cond.match?(:user => User.new(:birth_year => 2009)).should == true
        cond.match?(:user => User.new(:birth_year => 2010)).should == true
        cond.match?(:user => User.new(:birth_year => 2011)).should == false
      end
    end

    # 国
    describe "for user_country" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_country, :operator_key => :eql, :value => 'JP')
        cond.build_script.should == "user.address_country_code == 'JP'"
        user = User.new
        user.user_address = UserAddress.new
        user.user_address.country_code = 'GB'; cond.match?(:user => user).should == false
        user.user_address.country_code = 'US'; cond.match?(:user => user).should == false
        user.user_address.country_code = 'JP'; cond.match?(:user => user).should == true
      end
    end

    # 住所
    describe "for user_address" do
      it "by like" do
        cond = CampaignCondition.new(:type_key => :user_address, :operator_key => :like, :value => '愛知県名古屋市')
        cond.build_script.should == "user.address_for_campaign_condition =~ Regexp.new(Regexp.escape('愛知県名古屋市'))"
        user = User.new
        user.user_address = UserAddress.new(:locale => 'ja', :country_code => 'JP')
        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = nil
        user.user_address.address_3 = nil
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市'
        user.user_address.address_3 = nil
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '豊田市豊田１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '岐阜県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false
      end
    end

    # 離島
    describe "for :user_address_isolated" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_address_isolated, :operator_key => :eql, :value => '1')
        cond.build_script.should == "user.address_isolated? == ('1'.to_i==(1))"
        user = User.new
        user.user_address = UserAddress.new(:locale => 'ja')
        ShopSetting.stub!(:shipping_cost_outlier_or_nil).with("0000000").and_return(350)
        ShopSetting.stub!(:shipping_cost_outlier_or_nil).with("1111111").and_return(nil)
        user.user_address.zip_code = '0000000'; cond.match?(:user => user).should == true
        user.user_address.zip_code = '1111111'; cond.match?(:user => user).should == false
      end
    end

    describe "会員種別" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :member_category_div, :operator_key => :eql, :value => '001')
        cond.build_script.should == "user.member_category_div == '001'"

        cond.match?(:user => User.new(:member_category_div => '111')).should == false
        cond.match?(:user => User.new(:member_category_div => '001')).should == true
      end
    end

    describe "会員ランク" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_rank, :operator_key => :eql, :value => '1')
        cond.build_script.should == "user.user_rank_id == '1'.to_i"

        cond.match?(:user => User.new(:user_rank_id => 2)).should == false
        cond.match?(:user => User.new(:user_rank_id => 1)).should == true
      end
    end

    describe "注文小計" do
      it "by gte" do
        cond = CampaignCondition.new(:type_key => :cart_product_subtotal_price, :operator_key => :gte, :value => '5000')
        cond.build_script.should == "cart_or_item.subtotal_amount.to_i >= '5000'.to_i"

        cart_f = Cart.new()
        cart_f.stub!(:subtotal_amount).and_return(4999)
        cond.match?(:cart_or_item => cart_f).should == false

        cart_t = Cart.new()
        cart_t.stub!(:subtotal_amount).and_return(5000)
        cond.match?(:cart_or_item => cart_t).should == true
      end
    end

    describe "明細小計" do
      it "by lte" do
        cond = CampaignCondition.new(:type_key => :cart_product_subtotal_price, :operator_key => :lte, :value => '10000')
        cond.build_script.should == "cart_or_item.subtotal_amount.to_i <= '10000'.to_i"

        item_f = Cart::Item.new()
        item_f.stub!(:subtotal_amount).and_return(10001)
        cond.match?(:cart_or_item => item_f).should == false

        item_t = Cart::Item.new()
        item_t.stub!(:subtotal_amount).and_return(10000)
        cond.match?(:cart_or_item => item_t).should == true
      end
    end

    describe "注文合計" do
      it "by gte" do
        cond = CampaignCondition.new(:type_key => :cart_product_total_price, :operator_key => :gte, :value => '5000')
        cond.build_script.should == "cart_or_item.total_amount.to_i >= '5000'.to_i"

        cart_f = Cart.new()
        cart_f.stub!(:total_amount).and_return(4999)
        cond.match?(:cart_or_item => cart_f).should == false

        cart_t = Cart.new()
        cart_t.stub!(:total_amount).and_return(5000)
        cond.match?(:cart_or_item => cart_t).should == true
      end
    end

    describe "明細合計" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :cart_product_total_price, :operator_key => :eql, :value => '8000')
        cond.build_script.should == "cart_or_item.total_amount.to_i == '8000'.to_i"

        item_f1 = Cart::Item.new()
        item_f1.stub!(:total_amount).and_return(800)
        cond.match?(:cart_or_item => item_f1).should == false

        item_t = Cart::Item.new()
        item_t.stub!(:total_amount).and_return(8000)
        cond.match?(:cart_or_item => item_t).should == true

        item_f2 = Cart::Item.new()
        item_f2.stub!(:total_amount).and_return(80000)
        cond.match?(:cart_or_item => item_f2).should == false
      end
    end
  end


  describe "build with brace and connector" do
    it "shouldn't have connector if first condition" do
      cond = CampaignCondition.new
      cond.no = 0
      cond.should_receive(:build_script).and_return("script")
      cond.build.should == "script"
    end

    it "should have connector after second condition" do
      cond = CampaignCondition.new
      cond.no = 1
      cond.should_receive(:build_script).and_return("script")
      cond.build.should == "&& script"
    end

    it "should have connector after second condition" do
      cond = CampaignCondition.new
      cond.no = 2
      cond.should_receive(:build_script).and_return("script")
      cond.build.should == "&& script"
    end
  end


  describe "parsing date" do
    describe "parse_date" do
      it "with valid" do
        CampaignCondition.parse_date("2010/4/1").should == Date.parse("2010/4/1")
      end

      it "with invalid" do
        lambda{ CampaignCondition.parse_date("2010/4/31") }.should raise_error(ArgumentError)
      end
    end

    describe "parse_date_of_year" do
      it "with valid" do
        CampaignCondition.parse_date_of_year("4/1").should == "04/01"
        CampaignCondition.parse_date_of_year("04/1").should == "04/01"
        CampaignCondition.parse_date_of_year("04/01").should == "04/01"
        CampaignCondition.parse_date_of_year("2009/04/01").should == "04/01"
      end

      it "with invalid" do
        lambda{ CampaignCondition.parse_date_of_year("13/31") }.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_date_of_year("1/32") }.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_date_of_year("2009/02/31") }.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_date_of_year("2010/4/31") }.should raise_error(ArgumentError)
      end
    end

    describe "parse_day_of_month" do
      it "with valid" do
        CampaignCondition.parse_day_of_month("1").should == 1
        CampaignCondition.parse_day_of_month("31").should == 31
        CampaignCondition.parse_day_of_month("32").should == 32
        CampaignCondition.parse_day_of_month('100').should == 100
      end
    end

    describe "parse_month" do
      it "with valid" do
        CampaignCondition.parse_month("1").should == 1
        CampaignCondition.parse_month("12").should == 12
        CampaignCondition.parse_month("13").should == 13
        CampaignCondition.parse_month('100').should == 100
      end
    end

    describe "parse_year" do
      it "with valid" do
        CampaignCondition.parse_year("1950").should == 1950
        CampaignCondition.parse_year("1975").should == 1975
        CampaignCondition.parse_year("2000").should == 2000
        CampaignCondition.parse_year('2300').should == 2300
      end
    end

    describe "parse_day_of_week" do
      it "with valid" do
        CampaignCondition.parse_day_of_week('月').should == 1
        CampaignCondition.parse_day_of_week('火').should == 2
        CampaignCondition.parse_day_of_week('水').should == 3
        CampaignCondition.parse_day_of_week('木').should == 4
        CampaignCondition.parse_day_of_week('金').should == 5
        CampaignCondition.parse_day_of_week('土').should == 6
        CampaignCondition.parse_day_of_week('日').should == 7
        
        CampaignCondition.parse_day_of_week('月曜日').should == 1
        CampaignCondition.parse_day_of_week('火曜日').should == 2
        CampaignCondition.parse_day_of_week('水曜日').should == 3
        CampaignCondition.parse_day_of_week('木曜日').should == 4
        CampaignCondition.parse_day_of_week('金曜日').should == 5
        CampaignCondition.parse_day_of_week('土曜日').should == 6
        CampaignCondition.parse_day_of_week('日曜日').should == 7

        CampaignCondition.parse_day_of_week('Mon').should == 1
        CampaignCondition.parse_day_of_week('Tue').should == 2
        CampaignCondition.parse_day_of_week('Wed').should == 3
        CampaignCondition.parse_day_of_week('Thu').should == 4
        CampaignCondition.parse_day_of_week('Fri').should == 5
        CampaignCondition.parse_day_of_week('Sat').should == 6
        CampaignCondition.parse_day_of_week('Sun').should == 7

        CampaignCondition.parse_day_of_week('Monday').should == 1
        CampaignCondition.parse_day_of_week('Tuesday').should == 2
        CampaignCondition.parse_day_of_week('Wednesday').should == 3
        CampaignCondition.parse_day_of_week('Thursday').should == 4
        CampaignCondition.parse_day_of_week('Friday').should == 5
        CampaignCondition.parse_day_of_week('Saturday').should == 6
        CampaignCondition.parse_day_of_week('Sunday').should == 7
      end

      it "with invalid" do
        CampaignCondition.parse_day_of_week(nil).should == nil
        CampaignCondition.parse_day_of_week('').should == nil
        CampaignCondition.parse_day_of_week('UNKNOWN').should == nil
      end
    end

    describe "parse_time_of_day" do
      it "with valid" do
        CampaignCondition.parse_time_of_day('1:3').should == "01:03"
        CampaignCondition.parse_time_of_day('1:30').should == "01:30"
        CampaignCondition.parse_time_of_day('01:30').should == "01:30"
        CampaignCondition.parse_time_of_day('20:30').should == "20:30"
        CampaignCondition.parse_time_of_day('20:30:40').should == "20:30"

        CampaignCondition.parse_time_of_day('2007/12/31 20:30').should == "20:30"
        CampaignCondition.parse_time_of_day('12/31 1:0').should == "01:00"
      end

      it "with invalid" do
        lambda{ CampaignCondition.parse_time_of_day('12/20')}.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_time_of_day('25:06')}.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_time_of_day('23:66')}.should raise_error(ArgumentError)
      end
    end

  end


  describe "parse_age" do
    it "with valid" do
      CampaignCondition.parse_age('14').should == 14
      CampaignCondition.parse_age('18歳').should == 18
      CampaignCondition.parse_age('29才').should == 29
      CampaignCondition.parse_age('39').should == 39
    end
    
    it "with invalid" do
      lambda{ CampaignCondition.parse_age(nil) }.should raise_error(ArgumentError)
      lambda{ CampaignCondition.parse_age('') }.should raise_error(ArgumentError)
      lambda{ CampaignCondition.parse_age('ABC') }.should raise_error(ArgumentError)
    end
  end


  describe "create" do
    it "valid" do
      cond = CampaignCondition.new(:type_key => :user_gender, :operator_key => :eql, :value => 'male')
      cond.campaign_id = 1 # これはモデルが自動で設定できません 
      cond.no = 0 # これはCampaignが自動で設定してくれます
      cond.save
      cond.errors.full_messages.should == []
    end
  end

  describe "value_for_type" do
    it "gender" do
      cam = CampaignCondition.new(:type_key => :user_gender, :value => "1")
      cam.value_for_type.should == "男性"
    end
    it "country" do
      cam = CampaignCondition.new(:type_key => :user_country, :value => "JP")
      cam.value_for_type.should == "Japan"
    end
    it "boolean" do
      cam = CampaignCondition.new(:type_key => :user_address_isolated, :value => "0")
      cam.value_for_type.should == "いいえ"
    end
    it "user_rank" do
      cam = CampaignCondition.new(:type_key => :user_rank, :value => "2")
      cam.value_for_type.should == "ゴールド会員"
    end
    it "member_category_div" do
      cam = CampaignCondition.new(:type_key => :member_category_div, :value => "001")
      cam.value_for_type.should == "ParentsChina会員"
    end
    it "other" do
      cam = CampaignCondition.new(:type_cd => :user_address, :value => "001")
      cam.value_for_type.should == "001"
    end
  end
end
