# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CmsPublish do
  describe "project_ids" do
    it "create時に一緒に保存します" do
      publish = CmsPublish.new(:project_ids => [1, 3, 6], :designated_at => Time.now)
      publish.save.should == true
      publish = CmsPublish.find(publish.id)
      publish.cms_published_projects.count.should == 3
      publish.cms_published_projects.map(&:cms_project_id).sort.should == [1, 3, 6]
    end
  end
  
  # 
  # ==== CmsProjectの概要 ====
  # 
  # id | proirity
  #  3 | [  3]                                     |---|
  #  6 | [ 14]         |---|                       |   |
  #  2 | [ 20]         |   |                    |------------------------------|
  #  1 | [100]      |-------------------------------------------------------|  |
  #                 |  |   |                    |  |   |                    |  |
  # ================================================================================
  #                 |  |   |                    |  |   |                    |  |
  #  6              |  |   |                    |  |  2010-12-30 16:07:00   |  |
  #  6              |  |   |                    | 2010-07-30 16:07:00       |  |
  #  3              |  |  2009-01-31 10:00:00   |                           |  |
  #  3              | 2009-01-10 10:00:00       |                           |  |
  #  2              |                          2009-02-02 10:00:00          | 2020-02-02 10:00:00
  #  1             2009-01-01 10:00:00                                     2020-01-01 10:00:00
  #

  describe "rake ecr:cms:publish => CmsPublish.create_and_execute" do
    fixtures :cms_projects

    before(:each) do
      CmsPublish.delete_all
      CmsPublishedProject.delete_all
      CmsProject.update_all("status_cd = '#{CmsProject.status_id_by_key(:approved)}'")
      CmsBind.delete_all("id >= 42")
      CmsPublish.stub!(:upload_cloud_front)
    end

    describe "processing_now?" do
      it "should return false if there is no processing CmsPublish" do
        CmsPublish.processing_now?(:production).should == false
      end

      it "should return true if there is a processing CmsPublish" do
        CmsPublish.create!(
          :status_cd => CmsPublish.status_id_by_key(:processing),
          :target => CmsPublish.target_id_by_key(:production), 
          :designated_at => 1.minute.ago)
        CmsPublish.processing_now?(:production).should == true
      end
    end

    describe "2009/01/01の本番公開タスク" do
      it "タスク1: 一番最初の公開" do
        t = Time.parse("2009/01/01 10:00:01")
        Time.stub!(:now).and_return(t)
        #
        publish = mock_model(CmsPublish)
        publish.should_receive(:execute)
        # publish.should_receive(:send_reload_request)
        CmsPublish.should_receive(:create).with(:published_at => t, :designated_at => t, 
          :cloud_front_upload=>true, 
          :status_cd => CmsPublish.status_id_by_key(:processing), 
          :target => CmsPublish.target_id_by_key(:production)).and_return(publish)
        publish.should_receive(:cms_published_projects).exactly(2).and_return(mock_model(CmsPublishedProject))
        publish.cms_published_projects.should_receive(:create)
        CmsPublish.create_and_execute # この実行中に後述のタスク2が実行されます。
      end

      it "タスク2: 一番最初の本番公開タスク実行中にcronからもう一回実行した" do
        t = Time.parse("2009/01/01 10:01:00")
        Time.stub!(:now).and_return(t)
        
        # 他のタスクの処理が実行中であることを表すCmsPublishのデータを作成
        publish = CmsPublish.create!(
          :target => CmsPublish.target_id_by_key(:production),
          :status_cd => CmsPublish.status_id_by_key(:processing),
          :designated_at => 1.minute.ago)
        CmsProject.find(1).status_key.should == :approved
        #
        CmsPublish.should_not_receive(:create!)

        CmsPublish.logger.debug("@" * 100)
        lambda{
          CmsPublish.create_and_execute
        }.should_not change(CmsPublish, :count)
      end

      it "タスク3: 一番最初の本番公開タスクは無事終了その後cronなどで起動された" do
        t = Time.parse("2009/01/01 10:02:00")
        Time.stub!(:now).and_return(t)
        CmsProject.stub!(:find_all_to_be_deployed_or_deleted).and_return([])
        
        # 他のタスクの処理が実行中であることを表すCmsPublishのデータを作成
        publish = CmsPublish.create!(
          :target => CmsPublish.target_id_by_key(:production),
          :status_cd => CmsPublish.status_id_by_key(:opened),
          :designated_at => 2.minutes.ago)
        project1 = CmsProject.find(1)
        project1.status_key = :opened
        project1.save!
        CmsPublish.should_not_receive(:create!)
        #
        lambda{
          CmsPublish.create_and_execute
        }.should_not change(CmsPublish, :count)
      end
    end
  end

  describe "rake ecr:cms:publish_waiting => CmsPublish.execute_if_waiting" do
    fixtures :cms_projects

    before(:each) do
      CmsPublish.delete_all
      CmsPublishedProject.delete_all
      CmsProject.update_all("status_cd = '#{CmsProject.status_id_by_key(:approved)}'")
    end

    describe "2008/12/31のテスト公開タスク" do
      it "タスク1: 一番最初の公開" do
        t = Time.parse("2008/12/31 10:00:01")
        Time.stub!(:now).and_return(t)
      end
    end
  end

  describe "cms_published_projects" do
    fixtures :cms_projects, :cms_published_projects

    it "priority順を無視して追加しても、取得すればpriority順でソートされています" do
      cms_publish = CmsPublish.create!(
        :target => CmsPublish.target_id_by_key(:test),
        :status_cd => CmsPublish.status_id_by_key(:waiting),
        :designated_at => 1.minute.from_now)
      # 2009-02-01に、CmsProject#6をテストします。
      cms_publish.cms_published_projects.create(:cms_project_id => 6,
        :calc_type_cd => CmsPublishedProject.calc_type_id_by_key(:up))
      cms_publish.cms_published_projects.create(:cms_project_id => 1,
        :calc_type_cd => CmsPublishedProject.calc_type_id_by_key(:keep))
      cms_publish.cms_published_projects.create(:cms_project_id => 3,
        :calc_type_cd => CmsPublishedProject.calc_type_id_by_key(:down))
      cms_publish = CmsPublish.find(cms_publish.id)
      # cms_publish.reload
      cms_publish.projects.map(&:id).should == [3, 6, 1]
      cms_publish.cms_published_projects.map(&:cms_project_id).sort.should == [3, 6, 1].sort
    end

    it "setup_published_projects" do
      CmsProject.update(1, :status_key => :opened)
      cms_publish = CmsPublish.create!(
        :target => CmsPublish.target_id_by_key(:test),
        :status_cd => CmsPublish.status_id_by_key(:waiting),
        :designated_at => 1.minute.from_now)
      cms_publish.setup_published_projects
      cms_publish.projects.size.should == 1
    end
  end
  
  describe "create_and_execute" do
    if EcRider.config.ec_admin
      fixtures :cms_bind_contents, :cms_bind_items, :cms_binds, :cms_page_titles, :cms_pages,
        :cms_templates, :cms_template_contents, :cms_files, :cms_projects, :cms_published_projects, :cms_publishes, :cms_default_templates

      before(:each) do
        tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_cms_projects"))
        Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
        FileUtils.mkdir_p(File.join(tmp_dir, 'admin', 'public'))
        FileUtils.mkdir_p(File.join(tmp_dir, 'front', 'public'))
        EcRider.config.stub!(:ec_admin_root).and_return(File.join(tmp_dir, 'admin'))
        EcRider.config.stub!(:ec_front_root).and_return(File.join(tmp_dir, 'front'))
        CmsFile.find(:all).each{|cms_file| cms_file.destroy}
        CmsFile.create(:public_file_path => 'public/')

        cms_project = CmsProject.find(cms_projects(:cms_projects_00001).id)
        cms_project.status_key = :approved
        cms_project.save!
        cms_project = CmsProject.find(cms_projects(:cms_projects_00003).id)
        cms_project.status_key = :approved
        cms_project.save!
        cms_project = CmsProject.find(cms_projects(:cms_projects_00004).id)
        cms_project.status_key = :approved
        cms_project.save!

        CmsFile.create(:cms_project_id => cms_projects(:cms_projects_00001).id, :public_file_path => 'public/zzz/abc.png')
        FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'zzz'))
        FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'zzz', 'abc.png'))
        CmsFile.create(:cms_project_id => cms_projects(:cms_projects_00003).id, :public_file_path => 'public/zzz/abc.png', :delete_at_publish => true)
        FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '3', 'public', 'zzz'))
        FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '3', 'public', 'zzz', 'abc.png'))
        CmsFile.create(:cms_project_id => cms_projects(:cms_projects_00004).id, :public_file_path => 'public/zzz/abc.png')
        FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '6', 'public', 'zzz'))
        FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '6', 'public', 'zzz', 'abc.png'))
        CmsFile.create(:cms_project_id => cms_projects(:cms_projects_00001).id, :public_file_path => 'public/pj1.css')
        FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'pj1.css'))
        CmsFile.create(:cms_project_id => cms_projects(:cms_projects_00003).id, :public_file_path => 'public/pj3.css')
        FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '3', 'public', 'pj3.css'))
        CmsFile.create(:cms_project_id => cms_projects(:cms_projects_00004).id, :public_file_path => 'public/pj4.css')
        FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '6', 'public', 'pj4.css'))

        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj2.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj4.css')).should be_false

        CmsPublish.delete_all
        CmsPublishedProject.delete_all
        CmsBind.delete_all("cms_page_id not in (14,16)")
        CmsPublish.stub!(:upload_cloud_front)
        CmsPage.destroy(15)
        cms_page = CmsPage.find(14).clone(:include => [:titles, {:binds => {:items => :contents} }])
        cms_page.cms_project_id = 3
        cms_page.delete_at_publish = true
        cms_page.save!
        # 携帯ページ
        cms_page_m1 = CmsPage.find(14).clone(:include => [:titles, {:binds => {:items => :contents} }])
        cms_page_m1.cms_project_id = 1
        cms_page_m1.delete_at_publish = nil
        cms_page_m1.terminal_key = :mobile
        cms_page_m1.save!
        template_m1 = cms_page.bind_center.cms_template.dup
        template_m1.terminal_key = :mobile
        template_m1.contents << CmsTemplateContent.new(:locale => 'docomo', :descriptions => 'ドコモ')
        template_m1.save!
        cms_page_m3 = CmsPage.find(14).clone(:include => [:titles, {:binds => {:items => :contents} }])
        cms_page_m3.cms_project_id = 3
        cms_page_m3.delete_at_publish = true
        cms_page_m3.terminal_key = :mobile
        cms_page_m3.save!
        template_m3 = cms_page.bind_center.cms_template.dup
        template_m3.terminal_key = :mobile
        template_m3.contents << CmsTemplateContent.new(:locale => 'docomo', :descriptions => 'ドコモ')
        template_m3.save!
      end

      it "ページとファイルをデプロイ" do
        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                         100         true
        # project4                          (開始)---(終了)         20
        #            ▲
        # project1公開開始
        Time.stub!(:now).and_return(Time.local(2009, 1, 1, 11, 00, 00))
        # テストではec_frontは起動していない前提なので、外部接続する箇所はMockで代用します。
        CmsPublish.should_receive(:send_reload_request).exactly(6)

        CmsPublish.create_and_execute
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj4.css')).should be_false
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :approved
        CmsProject.find(cms_projects(:cms_projects_00004).id).status_key.should == :approved

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                         100         true
        # project4                          (開始)---(終了)         20
        #                   ▲
        # project3公開開始
        Time.stub!(:now).and_return(Time.local(2009, 1, 10, 11, 00, 00))
        CmsPublish.create_and_execute
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj4.css')).should be_false
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00004).id).status_key.should == :approved

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                         100         true
        # project4                          (開始)---(終了)         20
        #                             ▲
        # project3公開終了
        Time.stub!(:now).and_return(Time.local(2009, 1, 31, 11, 00, 00))
        CmsPublish.create_and_execute
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj4.css')).should be_false
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :closed
        CmsProject.find(cms_projects(:cms_projects_00004).id).status_key.should == :approved

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                         100         true
        # project4                          (開始)---(終了)         20
        #                                     ▲
        # project4公開開始
        Time.stub!(:now).and_return(Time.local(2010, 7, 30, 17, 00, 00))
        CmsPublish.create_and_execute
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj4.css')).should be_true
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :closed
        CmsProject.find(cms_projects(:cms_projects_00004).id).status_key.should == :opened

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                         100         true
        # project4                          (開始)---(終了)         20
        #                                              ▲
        # project4公開終了
        Time.stub!(:now).and_return(Time.local(2010, 12, 30, 17, 00, 00))
        CmsPublish.create_and_execute
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj4.css')).should be_false
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :closed
        CmsProject.find(cms_projects(:cms_projects_00004).id).status_key.should == :closed

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                         100         true
        # project4                          (開始)---(終了)         20
        #                                                  ▲
        # project1公開終了
        Time.stub!(:now).and_return(Time.local(2020, 1, 1, 11, 00, 00))
        CmsPublish.create_and_execute
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj4.css')).should be_false
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :closed
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :closed
        CmsProject.find(cms_projects(:cms_projects_00004).id).status_key.should == :closed
      end

      it "優先度が同じ場合は公開開始日時が後のものを優先" do
        pj1 = CmsProject.find(cms_projects(:cms_projects_00001).id)
        pj3 = CmsProject.find(cms_projects(:cms_projects_00003).id)
        pj3.priority = pj1.priority
        pj3.save!

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                           3         true
        #            ▲
        # project1公開開始
        Time.stub!(:now).and_return(Time.local(2009, 1, 1, 11, 00, 00))
        # テストではec_frontは起動していない前提なので、外部接続する箇所はMockで代用します。
        CmsPublish.should_receive(:send_reload_request).exactly(2)

        CmsPublish.create_and_execute
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_false
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :approved

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3        (開始)----(終了)                           3         true
        #                   ▲
        # project3公開開始
        Time.stub!(:now).and_return(Time.local(2009, 1, 10, 11, 00, 00))
        CmsPublish.create_and_execute
        publish = CmsPublish.find(:first, :order => 'id desc')
        publish.status_key.should == :opened
        publish.projects.map{|p|p.id}.sort.should == [1, 3]
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_true
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :opened
      end

      it "優先度も公開開始日時も同じ場合は、プロジェクトのidの大きいものを優先" do
        pj1 = CmsProject.find(cms_projects(:cms_projects_00001).id)
        pj3 = CmsProject.find(cms_projects(:cms_projects_00003).id)
        pj3.priority = pj1.priority
        pj3.started_at = pj1.started_at
        pj3.save!
        pj1.id.should < pj3.id

        #                      公開期間                          優先順位  公開時削除
        # project1  (開始)-------------------------------(終了)      3
        # project3  (開始)----(終了)                                 3         true
        #             ▲
        # project1とproject3公開開始
        Time.stub!(:now).and_return(Time.local(2009, 1, 10, 11, 00, 00))
        # テストではec_frontは起動していない前提なので、外部接続する箇所はMockで代用します。
        CmsPublish.should_receive(:send_reload_request).exactly(1)

        CmsPublish.create_and_execute
        publish = CmsPublish.find(:first, :order => 'id desc')
        publish.status_key.should == :opened
        publish.projects.map{|p|p.id}.sort.should == [1, 3]
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'app', 'views', 'cms', 'index_mobile.html.erb')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should be_false
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj1.css')).should be_true
        File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'pj3.css')).should be_true
        CmsProject.find(cms_projects(:cms_projects_00001).id).status_key.should == :opened
        CmsProject.find(cms_projects(:cms_projects_00003).id).status_key.should == :opened
      end
    end
  end

  describe "send_reload_request" do
    it "ec_frontは大抵複数プロセス起動しているので、複数のリクエストを送らなければなりません" do
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_RELOAD_VIEWS_URLS').once.
        and_return("http://localhost:3001/common/update_views,http://localhost:3002/common/update_views,http://localhost:3003/common/update_views")
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_BASIC_AUTH_ACCOUNT').once.and_return(nil)
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_BASIC_AUTH_PASSWORD').once.and_return(nil)
      #
      mock_res = mock(:response, :body => 'OK!')
      mock_http1 = mock(:http1)
      Net::HTTP.should_receive(:start).with('localhost', 3001).once.and_yield(mock_http1)
      mock_http1.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      mock_http2 = mock(:http1)
      Net::HTTP.should_receive(:start).with('localhost', 3002).once.and_yield(mock_http2)
      mock_http2.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      mock_http3 = mock(:http3)
      Net::HTTP.should_receive(:start).with('localhost', 3003).once.and_yield(mock_http3)
      mock_http3.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      #
      CmsPublish.send_reload_request(:production)
    end
    
    it "ec_frontは大抵複数プロセス起動しているので、複数のリクエストを送らなければなりません(BASIC認証付き)" do
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_RELOAD_VIEWS_URLS').once.
        and_return("http://localhost:3001/common/update_views,http://localhost:3002/common/update_views,http://localhost:3003/common/update_views")
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_BASIC_AUTH_ACCOUNT').once.and_return("previeaccount")
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_BASIC_AUTH_PASSWORD').once.and_return("previepassword")
      #
      mock_res = mock(:response, :body => 'OK!')
      mock_http1 = mock(:http1)
      Net::HTTP.should_receive(:start).with('localhost', 3001).once.and_yield(mock_http1)
      mock_http1.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      mock_http2 = mock(:http1)
      Net::HTTP.should_receive(:start).with('localhost', 3002).once.and_yield(mock_http2)
      mock_http2.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      mock_http3 = mock(:http3)
      Net::HTTP.should_receive(:start).with('localhost', 3003).once.and_yield(mock_http3)
      mock_http3.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      #
      CmsPublish.send_reload_request(:production)
    end
    
    it "ec_frontの指定されたプロセスが起動していない場合は、起動しているプロセスだけ更新するW" do
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_RELOAD_VIEWS_URLS').once.
        and_return("http://localhost:3001/common/update_views,http://localhost:3002/common/update_views,http://localhost:3003/common/update_views")
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_BASIC_AUTH_ACCOUNT').once.and_return("previeaccount")
      CmsPublish.should_receive(:load_env_by).with('ECR_FRONT_BASIC_AUTH_PASSWORD').once.and_return("previepassword")
      #
      mock_res = mock(:response, :body => 'OK!')
      mock_http1 = mock(:http1)
      Net::HTTP.should_receive(:start).with('localhost', 3001).once.and_yield(mock_http1)
      mock_http1.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      Net::HTTP.should_receive(:start).with('localhost', 3002).once.and_raise(SocketError)
      mock_http3 = mock(:http3)
      Net::HTTP.should_receive(:start).with('localhost', 3003).once.and_yield(mock_http3)
      mock_http3.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      #
      CmsPublish.send_reload_request(:production)
    end

    it "ec_testは大抵1プロセス起動していて、BASIC認証付き" do
      # ENV.should_receive(:[]).
      CmsPublish.should_receive(:load_env_by).
        with('ECR_TEST_RELOAD_VIEWS_URLS').once.
        and_return("http://localhost:3100/common/update_views")
      # ENV.should_receive(:[]).
      CmsPublish.should_receive(:load_env_by).
        with('ECR_TEST_BASIC_AUTH_ACCOUNT').once.and_return("previeaccount")
      # ENV.should_receive(:[]).
      CmsPublish.should_receive(:load_env_by).
        with('ECR_TEST_BASIC_AUTH_PASSWORD').once.and_return("previepassword")
      #
      mock_res = mock(:response, :body => 'OK!')
      mock_http1 = mock(:http1)
      Net::HTTP.should_receive(:start).with('localhost', 3100).once.and_yield(mock_http1)
      mock_http1.should_receive(:request).with(an_instance_of(Net::HTTP::Get)).and_return(mock_res)
      #
      CmsPublish.send_reload_request(:test)
    end
    
  end

end
