# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'


describe CmsXml do

  def invoke_generator(template)
    CmsXml.generate_erb(template)
  end


  it "erbに変換" do
    actual = invoke_generator(<<-"EOS")
      お名前：<ec:user_name/>
      メールアドレス：<ec:user_email/>
    EOS
    actual.should == <<-"EOS"
      お名前：<%= ec.user_name -%>
      メールアドレス：<%= ec.user_email -%>
    EOS
  end

  it "ecタグと他のタグの混在" do
    actual = invoke_generator(<<-"EOS")
      お名前：<ec:user_name/>
      <span name="foo" id="bar">baz</span>
    EOS
    actual.should == <<-"EOS"
      お名前：<%= ec.user_name -%>
      <span name="foo" id="bar">baz</span>
    EOS
  end

  it "ecタグに属性を指定" do
    actual = invoke_generator(<<-"EOS")
      今日は<ec:today format="%Y/%m/%d"/>です
    EOS
    actual.should == <<-"EOS"
      今日は<%= ec.today(:format=>"%Y/%m/%d") -%>です
    EOS
  end

  it "ecタグに属性と一緒にHTMLタグとしての属性も指定" do
    actual = invoke_generator(<<-"EOS")
      <ec:product_image no="2" border="0" alt="画像2"/>
    EOS
    actual.should == <<-"EOS"
      <%= ec.product_image(:alt=>"画像2", :border=>"0", :no=>"2") -%>
    EOS
  end

  it "ecタグで繰り返し(each)" do
    actual = invoke_generator(<<-"EOS")
      <ec:each values="1,2,3,4,5" key="index">
        <ec:print key="index"/>
      </ec:each>
    EOS
    actual.should == <<-"EOS"
      <% ec.each(:key=>"index", :values=>"1,2,3,4,5") do -%>
        <%= ec.print(:key=>"index") -%>
      <% end -%>
    EOS
  end

  it "ecタグで条件分岐(if)" do
    actual = invoke_generator(<<-"EOS")
      <ec:each values="1,2,3,4,5" key="index">
        <ec:if key="index" value="3">
          <ec:print key="index"/>
        </ec:if>
      </ec:each>
    EOS
    actual.should == <<-"EOS"
      <% ec.each(:key=>"index", :values=>"1,2,3,4,5") do -%>
        <% ec.if(:key=>"index", :value=>"3") do -%>
          <%= ec.print(:key=>"index") -%>
        <% end -%>
      <% end -%>
    EOS
  end

  it "should deal empty tag" do
    actual = invoke_generator(<<-"EOS")
      <img src="/path/to/image.png" alt="some image"/>
    EOS
    actual.should == <<-"EOS"
      <img src="/path/to/image.png" alt="some image"/>
    EOS
  end

  it "should deal ec tags made in step2" do
    actual = invoke_generator(<<-"EOS")
      <ec:user_login_form>
        <ec:user_login_form_error_messages/>
        <ec:user_login_form_email/>
        <ec:user_login_form_password/>
        <ec:user_login_form_remember_me/>
        <ec:button>戻る</ec:button>
        <ec:submit>ログインする</ec:submit>
      </ec:user_login_form>
    EOS
    actual.should == <<-"EOS"
      <% ec.user_login_form do -%>
        <%= ec.user_login_form_error_messages -%>
        <%= ec.user_login_form_email -%>
        <%= ec.user_login_form_password -%>
        <%= ec.user_login_form_remember_me -%>
        <% ec.button do -%>戻る<% end -%>
        <% ec.submit do -%>ログインする<% end -%>
      <% end -%>
    EOS
  end

  it "should deal ec tags made in step2 with yield" do
    actual = invoke_generator(<<-"EOS")
      <ec:yield/>
      <ec:yield args="css" />
      <ec:yield args="head,javascript" />
    EOS
    actual.should == <<-"EOS"
      <%= yield -%>
      <%= yield(:css) -%>
      <%= yield(:head, :javascript) -%>
    EOS
  end

  it "should deal ec tags made in step2 with comment" do
    actual = invoke_generator(<<-"EOS")
      <!-- XMLのコメントをERBでのコメントに -->
      <ec:user_login_form>
        <ec:user_login_form_error_messages/>
        <ec:user_login_form_email/>
        <ec:user_login_form_password/>
        <ec:user_login_form_remember_me/>
        <ec:button>戻る</ec:button>
        <ec:submit>ログインする</ec:submit>
      </ec:user_login_form>
    EOS
    actual.should == <<-"EOS"
      <%# XMLのコメントをERBでのコメントに %>
      <% ec.user_login_form do -%>
        <%= ec.user_login_form_error_messages -%>
        <%= ec.user_login_form_email -%>
        <%= ec.user_login_form_password -%>
        <%= ec.user_login_form_remember_me -%>
        <% ec.button do -%>戻る<% end -%>
        <% ec.submit do -%>ログインする<% end -%>
      <% end -%>
    EOS
  end

  describe "generate_erb" do
    it "BUG #627 can't generate html.erb correctly because ec tag can't be convert to ERB tag." do
      source_xml = <<-"EOS"
        <div id="bottom_area">
          <table class="transmission">
            <tr>
              <td class="go_back"><ec:button_to_back>←同意しない</ec:button_to_back></td>
              <td class="go_next"><ec:button href="/account/signup">同意して次へ進む→</ec:button></td>
            </tr>
          </table>
        </div>
      EOS
      expected_erb = <<-"EOS"
        <div id="bottom_area">
          <table class="transmission">
            <tr>
              <td class="go_back"><% ec.button_to_back do -%>←同意しない<% end -%></td>
              <td class="go_next"><% ec.button(:href=>"/account/signup") do -%>同意して次へ進む→<% end -%></td>
            </tr>
          </table>
        </div>
      EOS
      CmsXml.generate_erb(source_xml).should == expected_erb.to_s
    end

    it "BUG #627 confirm REXML::Document can parse correctly" do
      # 修正前ですが、このテストはちゃんとパスします。
      source_xml = <<-"EOS"
        <div id="bottom_area">
          <table class="transmission">
            <tr>
              <td class="go_back"><ec:button_to_back>←同意しない</ec:button_to_back></td>
              <td class="go_next"><ec:button href="/account/signup">同意して次へ進む→</ec:button></td>
            </tr>
          </table>
        </div>
      EOS
      xml = "<ec xmlns:ec=\"http://dummy\">#{source_xml}</ec>"
      doc = REXML::Document.new(xml)
      doc.children.size.should == 1
      ec = doc.children[0]
      ec.name.should == 'ec'
      ec.class.should == REXML::Element
      ec.children.size.should == 3
      bottom_area = ec.children[1]
      bottom_area.name.should == 'div'
      bottom_area.class.should == REXML::Element
      bottom_area.children.size.should == 3
      table = bottom_area.children[1]
      table.name.should == 'table'
      table.class.should == REXML::Element
      table.children.size.should == 3
      tr = table.children[1]
      tr.name.should == 'tr'
      tr.class.should == REXML::Element
      tr.children.size.should == 5
      td1 = tr.children[1]
      td1.name.should == 'td'
      td1.class.should == REXML::Element
      td1.children.size.should == 1
      button_to_back = td1.children[0]
      button_to_back.prefix.should == 'ec'
      button_to_back.class.should == REXML::Element
      button_to_back.name.should == 'button_to_back'
      td2 = tr.children[3]
      td2.name.should == 'td'
      td2.class.should == REXML::Element
      td2.children.size.should == 1
      button = td2.children[0]
      button.prefix.should == 'ec'
      button.class.should == REXML::Element
      button.name.should == 'button'
    end

    html_tags = %w(a abbr acronym address applet area b base basefont bdo bgsound big blink blockquote body br button) +
      %w(caption center cite code col colgroup comment dd del dfn dir div dl dt em embed fieldset font form frame frameset) +
      %w(h1 h2 h3 h4 h5 h6 head hr html i iframe ilayer img input ins isindex kbd keygen label layer legend li link listing) +
      %w(map marquee menu meta multicol nextid nobr noembed noframes nolayer noscript object ol optgroup option p) +
      %w(param plaintext pre q rb rp rt ruby s samp script select server small spacer span strike strong style sub sup) +
      %w(table tbody td textarea tfoot th thead title tr tt ul var wbr xmp)

    self_closed_tags = %w(input img br hr)

    it "BUG #631 generated span should not be closed" do
      CmsXml.generate_erb("<span></span>").should == "<span></span>" # 修正前は "<span/>" になってしまいました。
    end

    (html_tags - self_closed_tags).each do |tag|
      it "BUG #631 generated #{tag} tags should not be closed" do
        CmsXml.generate_erb("<#{tag}></#{tag}>").should == "<#{tag}></#{tag}>"
        CmsXml.generate_erb("<#{tag}/>").should == "<#{tag}></#{tag}>"
      end
    end

    self_closed_tags.each do |tag|
      it "BUG #631 generated #{tag} tags should be closed" do
        CmsXml.generate_erb("<#{tag}></#{tag}>").should == "<#{tag}/>"
        CmsXml.generate_erb("<#{tag}/>").should == "<#{tag}/>"
      end
    end
  end


 describe "ec_tags" do
   it "erbに変換" do
     st1 = <<-"EOS"
       お名前：<ec:user_name/>
       メールアドレス：<ec:user_email/>
     EOS
     assert_equal(<<-"EOS", CmsXml.generate_erb(st1))
       お名前：<%= ec.user_name -%>
       メールアドレス：<%= ec.user_email -%>
     EOS
   end

   it "ecタグと他のタグの混在" do
     st1 = <<-"EOS"
       お名前：<ec:user_name/>
       <span name="foo" id="bar">baz</span>
     EOS
     assert_equal(<<-"EOS", CmsXml.generate_erb(st1))
       お名前：<%= ec.user_name -%>
       <span name="foo" id="bar">baz</span>
     EOS
   end

   it "ecタグに属性を指定" do
     st1 = <<-"EOS"
       今日は<ec:today format="%Y/%m/%d"/>です
     EOS
     assert_equal(<<-"EOS", CmsXml.generate_erb(st1))
       今日は<%= ec.today(:format=>"%Y/%m/%d") -%>です
     EOS
   end

   it "ecタグに属性と一緒にHTMLタグとしての属性も指定" do
     st1 = <<-"EOS"
       <ec:product_image no="2" border="0" alt="画像2"/>
     EOS
     assert_equal(<<-"EOS", CmsXml.generate_erb(st1))
       <%= ec.product_image(:alt=>"画像2", :border=>"0", :no=>"2") -%>
     EOS
   end

   it "ecタグで繰り返し(each)" do
     st1 = <<-"EOS"
       <ec:each values="1,2,3,4,5" key="index">
         <ec:print key="index"/>
       </ec:each>
     EOS
     assert_equal(<<-"EOS", CmsXml.generate_erb(st1))
       <% ec.each(:key=>"index", :values=>"1,2,3,4,5") do -%>
         <%= ec.print(:key=>"index") -%>
       <% end -%>
     EOS
   end

   it "ecタグで条件分岐(if)" do
     st1 = <<-"EOS"
       <ec:each values="1,2,3,4,5" key="index">
         <ec:if key="index" value="3">
           <ec:print key="index"/>
         </ec:if>
       </ec:each>
     EOS
     assert_equal(<<-"EOS", CmsXml.generate_erb(st1))
       <% ec.each(:key=>"index", :values=>"1,2,3,4,5") do -%>
         <% ec.if(:key=>"index", :value=>"3") do -%>
           <%= ec.print(:key=>"index") -%>
         <% end -%>
       <% end -%>
     EOS
   end
 end



end
