require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CsvFileUpload do
  fixtures :csv_download_pattern_items, :csv_download_patterns, :product_variations, :product_prices, 
    :product_variation_items, :user_ranks, :product_categories

  describe "インスタンスの生成" do
    it "upload_id = 2の場合、ProductCsvUploaderオブジェクトが生成される" do
      csv_file_upload = CsvFileUpload.new({"upload_id" => "2"})
      csv_file_upload.uploader.class.should == CsvFileUpload::ProductCsvUploader
    end
    it "upload_id = 3の場合、ProductDisplayCsvUploaderオブジェクトが生成される" do
      csv_file_upload = CsvFileUpload.new({"upload_id" => "3"})
      csv_file_upload.uploader.class.should == CsvFileUpload::ProductDisplayCsvUploader
    end
  end

  describe "BaseCsvUploader" do
    before do
      @uploader = CsvFileUpload::BaseCsvUploader.new(          
        "upload_id" => '1',
        "file" => {:user => mock_model(File, {:original_filename => "test.xls", :content_type => "text/csv"})}
      )
    end
    
    it "インスタンス変数が正しく初期化される" do
      uploader = CsvFileUpload::BaseCsvUploader.new(
        "upload_id" => '1',
        "file" => ["#{EcRider.config.ec_admin_root}/test/fixtures/csv_file_upload/user.csv"]
      )
      uploader.success_count.should == 0
      uploader.error_count.should == 0
      uploader.error_rows.should == []
      uploader.file_field_labels.should == [nil]
    end

    it "アップロード情報の日本語名を返す" do
      uploader = CsvFileUpload::BaseCsvUploader.new("upload_id" => 2)
      uploader.file_field_labels.should == ['商品情報']
    end
    
    describe "バリデーションエラーにならない" do
      it "CSVファイルを選択した場合" do
        uploader = CsvFileUpload::BaseCsvUploader.new(
          "upload_id" => '1',
          "file" => {:user => mock_model(File, {:original_filename => 'test.csv', :content_type => "text/csv"})}
        )
        uploader.validate?.should be_true
      end

      it  "エクセル形式のCSVファイルを選択した場合" do
        uploader = CsvFileUpload::BaseCsvUploader.new(
          "upload_id" => '1',
          "file" => {:user => mock_model(File, {:original_filename => "test.csv", :content_type => "application/vnd.ms-excel"})}
        )
        uploader.validate?.should be_true
      end
    end

    describe "バリデーションエラーになる" do
      it "CSVファイルが未選択の場合" do
        uploader = CsvFileUpload::BaseCsvUploader.new(
          "upload_id" => '1',
          "file" => {}
        )
        uploader.validate?.should be_false
        uploader.errors.should == ["CSVファイルを選択してください。"]
      end

      it "テキストファイルを選択した場合" do
        uploader = CsvFileUpload::BaseCsvUploader.new(
          "upload_id" => '1',
          "file" => {:user => mock_model(File, {:original_filename => "test.txt", :content_type => "text/plain"})}
        )
        uploader.validate?.should be_false
        uploader.errors.should == ["ファイル形式がCSVではありません。"]
      end

      it "エクセルファイルを選択した場合" do
        uploader = CsvFileUpload::BaseCsvUploader.new(
          "upload_id" => '1',
          "file" => {:user => mock_model(File, {:original_filename => "test.xls", :content_type => "application/vnd.ms-excel"})}
        )
        uploader.validate?.should be_false
        uploader.errors.should == ["ファイル形式がCSVではありません。"]
      end
    end
    
    describe "parse_csv" do
      it "CSVを配列に読み込む" do
        parsed_csv = @uploader.parse_csv("#{EcRider.config.ec_admin_root}/test/fixtures/csv_file_upload/product_price_bulk_upload_template.csv")
        parsed_csv.is_a?(Array).should be_true
        parsed_csv.size.should == 168
        parsed_csv_line = parsed_csv[0]
        parsed_csv_line.is_a?(Array).should be_true
        parsed_csv_line.size.should == 3
      end
    end
    
    describe "get_value" do
      it "値を取得する" do
        @uploader.send("get_value", ['a01', 'a02', 'a03'], ['aa', 'bb', 'cc'], 'cc').should == 'a03'
      end
      it "存在しないキー" do
        @uploader.send("get_value", ['a01', 'a02', 'a03'], ['aa', 'bb', 'cc'], 'dd').should be_nil
      end
    end
    
    describe "get_column_names" do
      it "カラム名を取得する" do
        column_names = @uploader.send("get_column_names", 'product_price_bulk_upload_template')
        column_names.is_a?(Array).should be_true
        column_names.size.should == 3
        column_names[0].should == "primary_sku.sku_code"
        column_names[1].should == "user_rank.code"
        column_names[2].should == "primary_price.standard_price"
      end
    end
    
    describe "get_mapping_value" do
      it "マッピング情報を取得する" do
        map = {"key1" => 'value01',  "key2" => 'value02'}
        @uploader.send("get_mapping_value", map, "key2").should == 'value02'
      end
      it "キーが空文字だった場合" do
        map = {"key1" => 'value01',  "key2" => 'value02'}
        @uploader.send("get_mapping_value", map, '', 'blank!').should == 'blank!'
      end
      it "キーが見つからなかった場合" do
        map = {"key1" => 'value01',  "key2" => 'value02'}
        lambda{ @uploader.send("get_mapping_value", map, 'key3') }.should raise_error(CsvFileUpload::MappingError)
      end
    end
  end
  
  describe "ProductDisplayCsvUploader" do
    fixtures :products, :product_displays, :csv_file_upload_histories
    
    it "file_field_labels" do
      uploader = CsvFileUpload::ProductDisplayCsvUploader.new(          
        "upload_id" => '1',
        "file" => {:user => mock_model(File, {:original_filename => "test.xls", :content_type => "text/csv"})}
      )
      uploader.file_field_labels.should == ["商品情報（言語単位）"]
    end
    
    it "upload" do
      uploader = CsvFileUpload::ProductDisplayCsvUploader.new(          
        "upload_id" => '3',
        "file" => {
          "商品情報（言語単位）" => "#{EcRider.config.ec_admin_root}/test/fixtures/csv_file_upload/product_display_bulk_upload_template.csv"
        }
      )
      uploader.upload
      ProductDisplay.count().should == 70
      ProductDisplay.count(:conditions => {:product_id => 100040, :locale => 'zh_CN'}).should == 1
      CsvFileUploadHistory.count().should == 1
      upload_history = CsvFileUploadHistory.find(:first)
      upload_history.upload_id.should == 3
      upload_history.input_count.should == 58
      upload_history.success_count.should == 57
      upload_history.error_count.should == 1
      File.exist?(upload_history.csv_file_path).should be_true
    end
  end

  describe "ProductCsvUploader" do
    fixtures :products, :product_skus, :product_prices, :product_sku_inventories, 
      :product_sku_secure_inventories, :product_displays, :csv_file_upload_histories, :orders, :order_shippings, :order_items

    it "file_field_labels" do
      uploader = CsvFileUpload::ProductCsvUploader.new(          
        "upload_id" => '1',
        "file" => {:user => mock_model(File, {:original_filename => "test.xls", :content_type => "text/csv"})}
      )
      uploader.file_field_labels.should == ["商品情報（SKU単位）", "商品情報（売価単位）"]
    end

    it "upload" do
      uploader = CsvFileUpload::ProductCsvUploader.new(          
        "upload_id" => '2',
        "file" => {
          "商品情報（SKU単位）" => "#{EcRider.config.ec_admin_root}/test/fixtures/csv_file_upload/product_sku_bulk_upload_template.csv",
          "商品情報（売価単位）" => "#{EcRider.config.ec_admin_root}/test/fixtures/csv_file_upload/product_price_bulk_upload_template.csv"
        }
      )
      uploader.upload
      Product.count().should == 49
      Product.count(:conditions => {:product_code => '9000000000003'}).should == 1 #新規追加商品
      Product.count(:conditions => {:product_code => '0000000009050'}).should == 1 #商品削除(配送前の注文に含まれている商品削除しない)
      Product.count(:conditions => {:product_code => '0000000009051'}).should == 1 #SKU削除(配送前の注文に含まれている商品削除しない)
      ProductSku.count().should == 59
      ProductSku.count(:conditions => {:sku_code => '9000000000003-L1-BL-00'}).should == 1 #新規追加商品
      ProductSku.count(:conditions => {:sku_code => '0000000009050-00-00-00'}).should == 1 #商品削除(配送前の注文に含まれている商品削除しない)
      ProductSku.count(:conditions => {:sku_code => '0000000009051-00-00-00'}).should == 1 #SKU削除(配送前の注文に含まれている商品削除しない)

      CsvFileUploadHistory.count().should == 1
      upload_history = CsvFileUploadHistory.find(:first)
      upload_history.upload_id.should == 2
      upload_history.input_count.should == 55
      upload_history.success_count.should == 38
      upload_history.error_count.should == 17
      File.exist?(upload_history.csv_file_path).should be_true
    end

    it "upload" do
      OrderShipping.update_all("status_div = '3'")
      uploader = CsvFileUpload::ProductCsvUploader.new(          
        "upload_id" => '2',
        "file" => {
          "商品情報（SKU単位）" => "#{EcRider.config.ec_admin_root}/test/fixtures/csv_file_upload/product_sku_bulk_upload_template.csv",
          "商品情報（売価単位）" => "#{EcRider.config.ec_admin_root}/test/fixtures/csv_file_upload/product_price_bulk_upload_template.csv"
        }
      )
      uploader.upload
      Product.count().should == 48
      Product.count(:conditions => {:product_code => '9000000000003'}).should == 1 #新規追加商品
      Product.count(:conditions => {:product_code => '0000000009050'}).should == 0 #商品削除
      Product.count(:conditions => {:product_code => '0000000009051'}).should == 1 #SKU削除
      ProductSku.count().should == 57
      ProductSku.count(:conditions => {:sku_code => '9000000000003-L1-BL-00'}).should == 1 #新規追加商品
      ProductSku.count(:conditions => {:sku_code => '0000000009050-00-00-00'}).should == 0 #商品削除
      ProductSku.count(:conditions => {:sku_code => '0000000009051-00-00-00'}).should == 0 #SKU削除

      CsvFileUploadHistory.count().should == 1
      upload_history = CsvFileUploadHistory.find(:first)
      upload_history.upload_id.should == 2
      upload_history.input_count.should == 55
      upload_history.success_count.should == 40
      upload_history.error_count.should == 15
      File.exist?(upload_history.csv_file_path).should be_true
    end

  end
end
