# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe Inquiry do
  fixtures :inquiries, :inquiry_memos, :users, :user_emails, :user_addresses

  describe "A division value check" do
    it "inquiry_div means kind of request from users." do
      Inquiry.inquiry_div_options.should == [
          ['サイトの使い方について', '1'],
          ['ご注文について', '2'],
          ['在庫について', '3'],
          ['お支払いについて', '4'],
          ['商品のお届けについて', '5'],
          ['商品について', '6'],
          ['キャンセル・返品について', '7'],
          ['会員登録について', '8'],
          ['その他のお問合せ', '99']
      ]
    end

    it "status_div means status of inquiries." do
      Inquiry.status_div_options.should == [
        ["未対応", '0'],
        ["対応済み", '1'],
        ["返答待ち", '2']
#        ["ＣＦ依頼中", '3'],
#        ["相談室依頼中", '4']
      ]
      Inquiry.status_div_id_by_key(:not_yet).should == '0'
      Inquiry.status_div_id_by_key(:done).should == '1'
      Inquiry.status_div_id_by_key(:waiting).should == '2'
#      Inquiry.status_div_id_by_key(:with_cf).should == '3'
#      Inquiry.status_div_id_by_key(:with_meeting).should == '4'
    end
  end

  describe "test inquiry validation" do
    before(:each) do
      @params_en = {:email => 'test@ecrider.com', :name_last => 'kaimono', :name_first => 'taro', :tel_num => '000-000-0000', :locale => 'en'}
      @params_ja = {:email => 'test@ecrider.com', :name_last => '買物', :name_first => '太郎',
        :kana_name_last => 'カイモノ', :kana_name_first => 'タロウ', :tel_num => '000-000-0000', :locale => 'ja'}
    end

    it "test inquiry validation by english" do
      inquiry = Inquiry.new(@params_en)
      inquiry.inquiry = 'this is the question. this is the question?'
      inquiry.should be_valid
    end

    it "test inquiry validation by english wish japanese figure" do
      inquiry = Inquiry.new(@params_en)
      inquiry.inquiry = '質問です。'
      inquiry.should_not be_valid
    end

    it "test inquiry validation by japanese" do
      inquiry = Inquiry.new(@params_ja)
      inquiry.inquiry = 'this is the question. this is the question?'
      inquiry.should be_valid
    end

    it "test inquiry validation by japanese with japanese figure" do
      inquiry = Inquiry.new(@params_ja)
      inquiry.inquiry = '質問です。'
      inquiry.should be_valid
    end
  end

  describe "A number of the figures check" do
    before(:each) do
      @inquiry = ""
      @param = {:email => "email_address_01002@ec-one.com", :name_last => "意思湾", :name_first => "太郎",
        :kana_name_last => "カイモノ", :kana_name_first => "タロウ", :tel_num => "1234-56-7890", :address_2 => "住所",
        :country_code => 'JP', :locale => 'ja'}
    end
    it "An error does not occur by a figure number of inquiry check" do
      1000.times {@inquiry += "A"}
      @param[:inquiry_div] = '5'
      @param[:inquiry] = @inquiry
      inquiry = Inquiry.new(@param)
      # 桁数チェック結果
      inquiry.should be_valid
    end
    it "An error occurs by a figure number of inquiry check" do
      1001.times {@inquiry += "A"}
      @param[:inquiry_div] = '4'
      @param[:inquiry] = @inquiry
      inquiry = Inquiry.new(@param)
      # 桁数チェック結果
      inquiry.errors_on(:inquiry).should == ["は1000文字以内で入力してください。"]
    end
  end

  describe "ImageUrl test validates_of" do
    before(:each) do
      @inquiry = ""
      @param = {:inquiry => 'test', :email => 'test@ecrider.com', :name_last => 'kaimono', :name_first => 'taro', :tel_num => '000-000-0000', :locale => 'en'}
    end
    it "test inquiry validation by file type and name" do
       inquiry = Inquiry.new(@param)
       inquiry.image_url_1 = 'test.jpg'
       inquiry.should be_valid
    end
    
    it "test inquiry validation by file_type with no file type" do
       inquiry = Inquiry.new(@param)
       inquiry.image_url_1 = 'test.css'
       inquiry.should_not be_valid
    end

    it "test inquiry validation by file name" do
       inquiry = Inquiry.new(@param)
       inquiry.image_url_1 = 'テスト.jpg'
       inquiry.should_not be_valid
    end
  end

  describe " SearchInfo test validates_of" do
    it "test validate date_to" do
      condition = Inquiry::SearchInfo.new(:date_to => "2007/5/4")
      condition.valid?.should == true
    end
    it "test validate date_to with zero padding date" do
      condition = Inquiry::SearchInfo.new(:date_to => "2007/05/04")
      condition.valid?.should == true
    end
    it "test validate date_to with wrong date format" do
      condition = Inquiry::SearchInfo.new(:date_to => "2007/")
      condition.valid?.should == false
    end
    it "test validate date_from" do
      condition = Inquiry::SearchInfo.new(:date_from => "2007/5/4")
      condition.valid?.should == true
    end
    it "test validate date_from with zero padding date" do
      condition = Inquiry::SearchInfo.new(:date_from => "2007/05/04")
      condition.valid?.should == true
    end
    it "test validate date_from with wrong date format" do
      condition = Inquiry::SearchInfo.new(:date_from => "2007/")
      condition.valid?.should == false
    end
  end

  it "test create_condition" do
    condition = Inquiry::SearchInfo.new(:date_from => '2009/01/01', :date_to => '2009/01/02', :inquiry_div => '0', :status_div => '0', :memo => 'test')
    option = {:page => 1, :per_page => 10, :include => [:inquiry_memos], :order => "inquiries.created_at DESC"}
    c = condition.to_find_options(option)
    c[:page].should == 1
    c[:per_page].should == 10
    c[:include].should == [:inquiry_memos]
    c[:order].should == "inquiries.created_at DESC"
    c[:conditions].should == ["inquiries.created_at >= ? AND inquiries.created_at <= ? AND inquiry_div = ? AND status_div = ? AND inquiry_memos.memo like ?",
                 Time.parse('2009/01/01 00:00:00'), Time.parse('2009/01/02 23:59:59'), "0", "0", "%test%"]
  end

  it "associate user" do
    i1 = Inquiry.new(:email => 'email_address_01008@test.com')
    i1.user_id.should be_nil
    i1.associate_user
    i1.user_id.should == 1008
  end

  it "associate user with unknown email_address" do
    i1 = Inquiry.new(:email => 'unknown_address@test.com')
    i1.user_id.should be_nil
    i1.associate_user
    i1.user_id.should be_nil
  end

  it "associate_user select membershipping user" do
    org_user = User.find(1008)
    org_user.update_attribute(:member_div_key, :no_membership)
    new_user = User.new(User.find(1008).attributes)
    new_user.member_div_key = :membership
    new_email = UserEmail.new(org_user.user_email.attributes)
    new_user.user_email = new_email
    new_email.user = new_user
    new_user.save!
    new_email.save!

    new_user.member_div_key.should == :membership
    new_user.reload
    new_user.user_email.email.should == 'email_address_01008@test.com'

    i1 = Inquiry.new(:email => 'email_address_01008@test.com')
    i1.user_id.should be_nil
    i1.associate_user
    i1.user_id.should == new_user.id
  end

  it "test fullname english" do
    i1 = Inquiry.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP', :locale => 'en')
    i1.fullname.should == 'name2 name1'
  end
  it "test fullname japanese" do
    i1 = Inquiry.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP', :locale => 'ja')
    i1.fullname.should == 'name1 name2'
  end

  describe "create_condition" do
    it "本番問合せのみ" do
      Inquiry.update(inquiries(:inquiries_00001).id, :test_flg => true)
      c = Inquiry::SearchInfo.new(:test_flg => CmsPublish.target_id_by_key(:production))
      i = Inquiry.find(:all, c.to_find_options(:include => [:inquiry_memos], :order => "inquiries.created_at DESC"))
      i.length.should == 2
    end

    it "テスト問合せのみ" do
      Inquiry.update(inquiries(:inquiries_00001).id, :test_flg => true)
      c = Inquiry::SearchInfo.new(:test_flg => CmsPublish.target_id_by_key(:test))
      i = Inquiry.find(:all, c.to_find_options(:include => [:inquiry_memos], :order => "inquiries.created_at DESC"))
      i.length.should == 1
    end

    it "本番問合せとテスト問合せ" do
      Inquiry.update(inquiries(:inquiries_00001).id, :test_flg => true)
      c = Inquiry::SearchInfo.new()
      i = Inquiry.find(:all, c.to_find_options(:include => [:inquiry_memos], :order => "inquiries.created_at DESC"))
      i.length.should == 3
    end
  end

  describe "test?" do
    it "テスト問合せ" do
      Inquiry.new(:test_flg => true).test?.should be_true
    end

    it "本番問合せ" do
      Inquiry.new().test?.should be_false
      Inquiry.new(:test_flg => "").test?.should be_false
      Inquiry.new(:test_flg => nil).test?.should be_false
      Inquiry.new(:test_flg => false).test?.should be_false
    end
  end

  describe 'set_test_flg' do
    it "ec_adminの場合　本番データ" do
      i1 = Inquiry.new
      i1.set_test_flg
      i1.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      i1 = Inquiry.new(:test_flg => true)
      i1.set_test_flg
      i1.test_flg.should be_true
    end

    it "ec_frontの場合" do
      i1 = Inquiry.new
      i1.set_test_flg
      i1.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      i1 = Inquiry.new
      i1.set_test_flg
      i1.test_flg.should be_true
    end
  end
  
  describe "中国用バリデーションテスト" do
    before do
      @param = {:email => "email_address_01002@ec-one.com", :name_last => "意思湾", :name_first => "太郎",
        :kana_name_last => "カイモノ", :kana_name_first => "タロウ", :tel_num => "1234-56-7890",
        :inquiry_div => 5, :inquiry => "漢字漢字漢字漢字漢字漢字"
        }
    end
    describe "簡体字入力OKの場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(true)
      end
      it "日本語の場合、バリデーションなし" do
        @param[:country_code] = 'JP'
        @param[:locale] = 'ja'
        inquiry = Inquiry.new(@param)
        inquiry.valid?.should == true
        inquiry.errors_on(:name_first).should == []
        inquiry.errors_on(:name_last).should == []
        inquiry.errors_on(:inquiry).should == []
        inquiry.errors.empty?.should be_true
      end
      it "中国語(簡体字)の場合、バリデーションなし" do
        @param[:country_code] = 'CN'
        @param[:locale] = 'zh_CN'
        inquiry = Inquiry.new(@param)
        inquiry.valid?.should == true
        inquiry.errors_on(:name_first).should == []
        inquiry.errors_on(:name_last).should == []
        inquiry.errors_on(:inquiry).should == []
        inquiry.errors.empty?.should be_true
      end
      it "英語の場合、バリデーションあり" do
        @param[:country_code] = 'CN'
        @param[:locale] = 'en'
        inquiry = Inquiry.new(@param)
        inquiry.valid?.should == false
        inquiry.errors_on(:name_first).should_not == []
        inquiry.errors_on(:name_last).should_not == []
        inquiry.errors_on(:inquiry).should_not == []
        inquiry.errors.empty?.should be_false
      end
    end
    describe "簡体字入力NGの場合" do
      before do
          EcRider.config.stub!(:chinese_format_using).and_return(false)
      end
      it "日本語の場合、バリデーションなし" do
        @param[:country_code] = 'JP'
        @param[:locale] = 'ja'
        inquiry = Inquiry.new(@param)
        inquiry.valid?.should == true
        inquiry.errors_on(:name_first).should == []
        inquiry.errors_on(:name_last).should == []
        inquiry.errors_on(:inquiry).should == []
        inquiry.errors.empty?.should be_true
      end
      it "中国語(簡体字)の場合、バリデーションあり" do
        @param[:country_code] = 'CN'
        @param[:locale] = 'zh_CN'
        inquiry = Inquiry.new(@param)
        inquiry.valid?.should == false
        inquiry.errors_on(:name_first).should_not == []
        inquiry.errors_on(:name_last).should_not == []
        inquiry.errors_on(:inquiry).should_not == []
        inquiry.errors.empty?.should be_false
      end
      it "英語の場合、バリデーションあり" do
        @param[:country_code] = 'CN'
        @param[:locale] = 'en'
        inquiry = Inquiry.new(@param)
        inquiry.valid?.should == false
        inquiry.errors_on(:name_first).should_not == []
        inquiry.errors_on(:name_last).should_not == []
        inquiry.errors_on(:inquiry).should_not == []
        inquiry.errors.empty?.should be_false
      end
    end
  end
end
