# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe MailTemplate do
  fixtures :mail_templates, :orders, :order_shippings, :order_items, :products, :product_skus, :countries, :distributor_delivery_times, :distributor_delivery_time_displays

  it "test_save_and_change_use_div" do
    t1 = MailTemplate.find(:first, :conditions => {:use_div => MailTemplate.use_div_id_by_key(:true)})
    attributes = t1.attributes.dup
    attributes.delete(:id)

    t2 = MailTemplate.new(attributes)
    t2.use_div_key = :false
    t2.name = 'new template'
    t2.save

    t2.use_div_key = :false
    t2.save_and_change_use_div

    t1 = MailTemplate.find(t1.id)
    assert_equal(t1.use_div_key, :true)
    assert_equal(t2.use_div_key, :false)

    t2.use_div_key = :true
    t2.save_and_change_use_div

    t1 = MailTemplate.find(t1.id)
    assert_equal(t1.use_div_key, :false)
    assert_equal(t2.use_div_key, :true)

    t3 = MailTemplate.new(attributes)
    t3.locale = 'en'
    t3.name = 'new_template_en'
    t3.use_div_key = :true
    t3.save_and_change_use_div

    assert_equal(t1.use_div_key, :false)
    assert_equal(t2.use_div_key, :true)
    assert_equal(t3.use_div_key, :true)
  end

  it "test_find_valid_template" do
    t1 = MailTemplate.find(:first, :conditions => {:use_div => MailTemplate.use_div_id_by_key(:true)})
    attributes = t1.attributes.dup
    attributes.delete(:id)

    t2 = MailTemplate.new(attributes)
    t2.use_div_key = :false
    t2.name = 'new template'
    t2.save

    t3 = MailTemplate.find_valid_template(t1.mail_type_div_key)
    assert_equal(t3.id, t1.id)

    t2.use_div_key = :true
    t2.save_and_change_use_div

    t3 = MailTemplate.find_valid_template(t1.mail_type_div_key)
    assert_equal(t3.id, t2.id)

    t4 = MailTemplate.find(:first, :conditions => {:mail_type_div => t1.mail_type_div, :locale => 'en'})
    t3 = MailTemplate.find_valid_template(t1.mail_type_div_key, 'en')
    assert_equal(t3.id, t4.id)
  end

  it "test create_condition" do
    condition = MailTemplate.new()
    a1 = MailTemplate.find(:all, :conditions => MailTemplate.create_condition(condition))
    a1.size.should == 59

    condition = MailTemplate.new(:name => 'メール', :mail_type_div_key => :reply, :locale => 'ja')
    a2 = MailTemplate.find(:all, :conditions => MailTemplate.create_condition(condition))
    a2.size.should == 1
  end

  describe "validate check" do
    it "test validate" do
      m = MailTemplate.find(:first)
      m.stub!(:for_mobile?).and_return(true)
      m.validate
      m.errors.full_messages.should == []
      m.errors.length.should == 0
    end
    it "test validate with SyntaxError" do
      m = MailTemplate.find(:first)
      m.pc_text = '<%= h( %>'
      m.validate
      m.errors.length.should == 1
      m.errors.on('pc_text').should == 'の構文が不正です。'
    end
    it "test validate with NoMethodError" do
      m = MailTemplate.find(:first)
      m.pc_text = '<%= Product.xxxxxxxxx %>'
      m.validate
      m.errors.length.should == 1
      m.errors.on('pc_text').should == 'の構文が不正です。'
    end
    it "test validate with ArgumentError" do
      m = MailTemplate.find(:first)
      m.pc_text = '<% Array.new(-1) %>'
      m.validate
      m.errors.length.should == 1
      m.errors.on('pc_text').should == 'の想定されていない変数が含まれています。'
    end
    it "test validate with other error" do
      m = MailTemplate.find(:first)
      m.pc_text = '<%= username[-1] %>'
      m.validate
      m.errors.length.should == 1
      m.errors.on('pc_text').should == 'が構文に違反しています。'
    end
    it "pc_text validates_length valid" do
      m = MailTemplate.find(:first)
      m.pc_text = 'あ'*20000
      m.valid?.should == true
      m.save.should == true
    end
    it "pc_text validates_length invalid" do
      m = MailTemplate.find(:first)
      m.pc_text = 'あ'*30000
      m.valid?.should == false
      m.save.should == false
    end
  end

  describe "preview check" do
    it "preview subject for pc" do
      m = MailTemplate.new(:pc_subject_template => 'test<%=user_name%>')
      m.preview_mail_subject(:pc).should == 'test買物太郎'
    end
    it "preview subject for mobile" do
      m = MailTemplate.new(:mobile_subject_template => '<%=user_name%>mobile')
      m.preview_mail_subject(:mobile).should == '買物太郎mobile'
    end
    it "preview text for pc" do
      m = MailTemplate.new(:pc_text => 'test<%=user_name%>')
      m.preview_mail_body(:pc, :text).should == 'test買物太郎'
    end
    it "preview text for mobile" do
      m = MailTemplate.new(:mobile_text => '<%=user_name%>mobile')
      m.preview_mail_body(:mobile, :text).should == '買物太郎mobile'
    end
    it "preview html for pc" do
      m = MailTemplate.new(:pc_html => '<p>test<%=user_name%></p>')
      m.preview_mail_body(:pc, :html).should == '<p>test買物太郎</p>'
    end
    it "preview html for mobile" do
      m = MailTemplate.new(:mobile_html => '<p><%=user_name%>mobile</p>')
      m.preview_mail_body(:mobile, :html).should == '<p>買物太郎mobile</p>'
    end
  end

  describe "get_mail_subject check" do
    it "subject blank" do
      mail_template = MailTemplate.new
      mail_template.get_mail_subject('1').should == ' '
    end
    it "type by pc" do
      mail_template = MailTemplate.new(:pc_subject_template => 'pc')
      mail_template.get_mail_subject('1').should == "pc"
    end
    it "type by mobile" do
      mail_template = MailTemplate.new(:mobile_subject_template => 'mobile')
      mail_template.get_mail_subject('2').should == "mobile"
    end
  end

  describe "get_mail_body check" do
    it "subject blank" do
      mail_template = MailTemplate.new
      mail_template.get_mail_body('1').should == ' '
    end
    it "type by pc" do
      mail_template = MailTemplate.new(:pc_html => 'pc', :content_type_div_key => :text_html, :pc_text => 'pc_text')
      mail_template.get_mail_body('1').should == "pc"
    end
    it "type by mobile" do
      mail_template = MailTemplate.new(:mobile_html => 'mobile', :content_type_div_key => :text_html, :mobile_text => 'mobile_text')
      mail_template.get_mail_body('2').should == "mobile"
    end
  end

end
