# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductVariationItem do
  before(:each) do
    @valid_attributes = {
      :product_variation_id => 1,
      :display_order => 1,
      :code => "value for code"
      # product_variation_itemsにはname_for_adminカラムはありませんが、
      # ProductVariationItem#name_for_admin によって、product_variation_item_displaysの
      # nameを取得できます。
      # :name_for_admin => "value for name_for_admin"
    }
    @product_variation_item = ProductVariationItem.new(@valid_attributes)
  end

  describe "display" do
    it "必須チェック" do
      @product_variation_item.display_order = ''
      @product_variation_item.should_not be_valid
    end
  end

  describe "code" do
    it "必須チェック" do
      @product_variation_item.code = ''
      @product_variation_item.should_not be_valid
    end

    it "文字数チェック" do
      @product_variation_item.code = '333'
      @product_variation_item.should_not be_valid
    end

    it "「00」は登録できない" do
      @product_variation_item.code = '00'
      @product_variation_item.should_not be_valid
    end

    it "「01」は登録できる" do
      @product_variation_item.code = '01'
      @product_variation_item.should be_valid
    end

    it "「0」は登録できない" do
      @product_variation_item.code = '0'
      @product_variation_item.should_not be_valid
    end

    it "英数以外は登録できない" do
      @product_variation_item.code = 'あい'
      @product_variation_item.should_not be_valid
    end

    it "英数「A1」は登録できる" do
      @product_variation_item.code = 'A1'
      @product_variation_item.should be_valid
    end
  end

  describe "name_for_admin" do
    CODE_TO_SIZE = [
      {:code => "S4", :name => "XXS"},
      {:code => "S3", :name => "XS"},
      {:code => "S2", :name => "SS"},
      {:code => "S1", :name => "S"},
      {:code => "M1", :name => "M"},
      {:code => "L1", :name => "L"},
      {:code => "L2", :name => "LL"},
      {:code => "L3", :name => "XL"},
      {:code => "L4", :name => "XXL"}
    ]

    before(:each) do
      ProductVariationItem.delete_all
    end

    it "with saved ProductVariationItem ja" do
      variation = ProductVariation.create(:name_for_admin => "サイズA")
      CODE_TO_SIZE.each_with_index do |hash, index|
        item = variation.items.create(:display_order => index + 1, :code => hash[:code])
        item.save_displays(:ja => hash[:name], :en => hash[:name])
      end
      CODE_TO_SIZE.each_with_index do |hash, index|
        item = variation.items.find_by_code(hash[:code])
        item.name_for_admin.should == hash[:name]
      end
    end
    
    it "without saved ProductVariationItem ja" do
      variation = ProductVariation.create(:name_for_admin => "サイズB")
      CODE_TO_SIZE.each_with_index do |hash, index|
        item = variation.items.create(:display_order => index + 1, :code => hash[:code])
        item.save_displays(:en => hash[:name]) # 日本語名は登録しない
      end
      CODE_TO_SIZE.each_with_index do |hash, index|
        item = variation.items.find_by_code(hash[:code])
        item.name_for_admin.should == nil
      end
    end
    
    it "with unsaved ProductVariationItem ja" do
      variation = ProductVariation.new(:name_for_admin => "サイズC")
      items = {}
      CODE_TO_SIZE.each_with_index do |hash, index|
        item = variation.items.build(:display_order => index + 1, :code => hash[:code])
        item.build_displays(:ja => hash[:name], :en => hash[:name])
        items[hash[:code]] = item
      end
      CODE_TO_SIZE.each_with_index do |hash, index|
        item = items[hash[:code]]
        item.name_for_admin.should == hash[:name]
      end
    end
  end
  
  describe "find_by_variation_id_and_code" do
    fixtures :product_variation_items
    it "バリエーション項目を取得" do
      ProductVariationItem.find_by_variation_id_and_code(1, 'L1').id.should == 3
    end
    it "引数不正" do
      ProductVariationItem.find_by_variation_id_and_code(1, nil).should be_nil
    end
    it "該当なし" do
      ProductVariationItem.find_by_variation_id_and_code(1, 'AA').should be_nil
    end
  end
  
  describe "options_by_ids" do
    fixtures :product_variation_items, :product_variation_item_displays
    it "オプション用配列を取得" do
      I18n.stub!(:locale).and_return('ja')
      items = ProductVariationItem.options_by_ids([6,4])
      items.size.should == 2
      items[0].should == ["赤", "RD"]
      items[1].should == ["黄", "YW"]
    end
    
    it "オプション用配列を取得(en)" do
      I18n.stub!(:locale).and_return('en')
      items = ProductVariationItem.options_by_ids([6,4])
      items.size.should == 2
      items[0].should == ["RD", "RD"]
      items[1].should == ["YW", "YW"]
    end
  end

end
