require 'fileutils'
require 'tempfile'
require 'nkf'
module Rails
  module Generator
    module Commands
      class Create < Base
        def mv(relative_source, relative_destination, file_options = {}, &block)
          source = destination_path(relative_source)
          destination = destination_path(relative_destination)
          unless File.exists?(source)
            if file_options[:source_required]
              raise ArgumentError, "source not found: #{source}"
            else
              return
            end
          end
          if File.exists?(destination)
            choice = case (file_options[:collision] || options[:collision]).to_sym #|| :ask
              when :ask   then force_file_collision?(relative_destination, source, destination, file_options, &block)
              when :force then :force
              when :skip  then :skip
              else raise "Invalid collision option: #{options[:collision].inspect}"
            end

            # Take action based on our choice.  Bail out if we chose to
            # skip the file; otherwise, log our transgression and continue.
            case choice
              when :force then logger.force(relative_destination)
              when :skip  then return(logger.skip(relative_destination))
              else raise "Invalid collision choice: #{choice}.inspect"
            end
          end
          
          if options[:svn]
            svn_command = "svn mv #{source} #{destination} --force"
            if options[:pretend]
              puts svn_command
            else
              system svn_command
            end
          else
            FileUtils.mv(source, destination, :verbose => true, :noop => file_options[:pretend])
          end
        end
        
        RUBY_WORD_TOKENS = Regexp.escape("\b\s\:\;\.\,\+\-\*\/\?\<\>\(\)\[\]\{\}\'\"\@\|\!")
        
        def replace(file, replacements, file_options = {}, &block)
          return unless File.exist?(file)
          new_content = File.read(file)
          replacements.each do |search, replace|
            new_content.gsub!(/(\A|[#{RUBY_WORD_TOKENS}])#{search}([#{RUBY_WORD_TOKENS}]|\Z|$)/){"#{$1}#{replace}#{$2}"}
          end

          diff = diff(file, new_content, file_options)
          return if diff.nil? or diff.to_s.strip.blank?

          show_diff(file, diff) if file_options[:diff]
          
          choice = case (file_options[:collision] || options[:collision]).to_sym #|| :ask
            when :ask   then force_replace?(file, diff, file_options, &block)
            when :force then :force
            when :skip  then :skip
            else raise "Invalid collision option: #{options[:collision].inspect}"
          end
          case choice
            when :force then logger.force(file)
            when :skip  then return(logger.skip(file))
            else raise "Invalid collision choice: #{choice}.inspect"
          end

          unless file_options[:pretend]
            File.open(file, "w"){|io|io.puts new_content}
          end
        end
        
        def show_diff(file, diff)
          puts 
          title = "== update about #{file} "
          puts title << "=" * (80 - title.length)
          puts encode(diff)
          puts "-" * 80
          puts 
        end
        
        def encode(str)
          options[:encoding].blank? ?
            str : NKF.nkf("-W --#{options[:encoding]}", str)
        end
        
        def decode(str)
          options[:encoding].blank? ?
            str : NKF.nkf("--#{options[:encoding]}-input --utf8", str)
        end
        
        DIFF_COMMAND = (RUBY_PLATFORM =~ /mswin|mingw/) ? "diff -u -1 -p -w" : "diff"
        
        def diff(file_path, new_content, options)
          unless options[:using_diff]
            old_content = File.read(file_path)
            return (new_content == old_content) ? nil : new_content
          end
          tmp_file_path = file_path.gsub(/\//, '__')
          if (options[:encoding] || 'utf8') == 'utf8'
            create_temp_file("#{tmp_file_path}.new", new_content) do |new_file|
              return `#{DIFF_COMMAND} #{file_path} #{new_file.path}`
            end
          else
            create_temp_file("#{tmp_file_path}.old", encode(File.read(file_path))) do |old_file|
              create_temp_file("#{tmp_file_path}.new", encode(new_content)) do |new_file|
                return decode(`#{DIFF_COMMAND} #{old_file.path} #{new_file.path}`)
              end
            end
          end
        rescue
          $stdout.puts "diff failed: #{$!}"
          $stdout.puts $!
          $stdout.puts $!.backtrace.join("\n  ")
          # new_content
          raise
        end
        
        def create_temp_file(file_path, content)
          Tempfile.open(file_path, 'tmp') do |file|
            file.write(content)
            file.flush
            yield(file)
          end
        end

        def force_replace?(file, diff, file_options = {}, &block)
          $stdout.print "overwrite #{file}? [Ynaqdsh] "
          case $stdin.gets
            when /h/i
              $stdout.puts "keep diff hidden"
              file_options[:diff] = false
              puts "retrying"
              raise 'retry diff'
            when /s/i
              $stdout.puts "keep diff shown"
              show_diff(file, diff)
              file_options[:diff] = true
              puts "retrying"
              raise 'retry diff'
            when /d/
              show_diff(file, diff)
              puts "retrying"
              raise 'retry diff'
            when /a/i
              $stdout.puts "forcing #{spec.name}"
              options[:collision] = :force
            when /q/i
              $stdout.puts "aborting #{spec.name}"
              raise SystemExit
            when /n/i then :skip
            else :force
          end
        rescue
          retry
        end
      end
    end
  end
end
